package com.tiffintom;

import static com.tiffintom.models.AppDatabase.MIGRATION_1_2;

import android.app.Activity;
import android.content.Context;
import android.content.Intent;
import android.database.Cursor;
import android.graphics.Bitmap;
import android.graphics.BitmapFactory;
import android.graphics.Canvas;
import android.graphics.Matrix;
import android.graphics.Paint;
import android.media.ExifInterface;
import android.net.ConnectivityManager;
import android.net.NetworkInfo;
import android.net.Uri;
import android.os.Environment;
import android.os.Handler;
import android.provider.MediaStore;
import android.provider.Settings;
import android.util.Log;

import androidx.localbroadcastmanager.content.LocalBroadcastManager;
import androidx.multidex.MultiDexApplication;
import androidx.room.Room;

import com.androidnetworking.AndroidNetworking;
import com.androidnetworking.error.ANError;
import com.androidnetworking.interfaces.JSONObjectRequestListener;
import com.balsikandar.crashreporter.CrashReporter;
import com.chuckerteam.chucker.api.ChuckerInterceptor;
import com.google.firebase.messaging.FirebaseMessaging;
import com.google.gson.Gson;
import com.tiffintom.activity.NoInternetActivity;
import com.tiffintom.activity.SplashActivity;
import com.tiffintom.common.Validators;
import com.tiffintom.interfaces.ProfileChangeListener;
import com.tiffintom.models.AppDatabase;
import com.tiffintom.models.BecomePartner;
import com.tiffintom.models.CartItem;
import com.tiffintom.models.Category;
import com.tiffintom.models.DineinModels.Addon;
import com.tiffintom.models.DineinModels.DineinMenus;
import com.tiffintom.models.DineinModels.DineinPaymentMethods;
import com.tiffintom.models.DineinModels.DineinSiteSettings;
import com.tiffintom.models.PaymentMethod;
import com.tiffintom.models.RequestResponse;
import com.tiffintom.models.Restaurant;
import com.tiffintom.models.SearchResult;
import com.tiffintom.network.ApiEndPoints;
import com.tiffintom.network.NetworkInterceptor;
import com.tiffintom.notification.NotificationService;
import com.tiffintom.storageutils.MyPreferences;
import com.tiffintom.utils.Constants;
import com.tiffintom.utils.LogUtils;

import org.json.JSONException;
import org.json.JSONObject;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.net.HttpURLConnection;
import java.net.MalformedURLException;
import java.net.URL;
import java.util.ArrayList;
import java.util.List;
import java.util.Timer;
import java.util.TimerTask;
import java.util.concurrent.TimeUnit;

import okhttp3.OkHttpClient;

/**
 * Created by Amrish on 18-08-2020.
 */
public class MyApp extends MultiDexApplication {
    public static String banne_path;
    public static String menu_image_path;
    private static MyApp ourInstance;
    private MyPreferences myPreferences;
    private String currencySymbol;
    private AppDatabase appDatabase;
    public static BecomePartner becomePartner;
    public static CartItem cartItem;
    public static List<String> favouriteList = new ArrayList<>();
    public int chatCount;
    public static boolean isDineInOpen;
    public static SearchResult searchResult;
    public boolean isInternetAvailable = true;
    public static String RESTAURANT_ID;
    public ArrayList<DineinSiteSettings> dineinSiteSettings = new ArrayList<>();
    public ArrayList<DineinPaymentMethods> dineinPaymentMethods = new ArrayList<>();
    public String banquetDay, banquetOrderTypes;
    public String current_day;
    public boolean showBanquetPorducts = false;

    public String getCurrencySymbol() {
        return (getMyPreferences().getSiteSettings() == null) ? "" : getMyPreferences().getSiteSettings().site_currency;
    }

    public static MyApp getInstance() {
        return ourInstance;
    }

    public MyPreferences getMyPreferences() {
        return myPreferences;
    }

    public AppDatabase getAppDatabase() {
        return appDatabase;
    }

    public static Restaurant openedRestaurant;
    public static Category openedRestaurantMenuCategories;
    private ProfileChangeListener profileChangeListener;

    public void setProfileChangeListener(ProfileChangeListener profileChangeListener) {
        this.profileChangeListener = profileChangeListener;
    }


    @Override
    public void onCreate() {
        super.onCreate();
        ourInstance = this;
        CrashReporter.initialize(this);
        myPreferences = new MyPreferences(this);
        appDatabase = Room.databaseBuilder(getApplicationContext(),
                AppDatabase.class, "tiffin-tom-database").addMigrations(MIGRATION_1_2).build();
        new Thread(this::initializeApi).start();
        myPreferences.setDeviceId(Settings.Secure.getString(getContentResolver(), Settings.Secure.ANDROID_ID));
        manageOrderRestaurant();
        initService();
        setAutoFetch();
        getFirebaseToken();
    }

    private void manageOrderRestaurant() {
        new Thread(() -> {
            int res_id = appDatabase.cartDao().getRestaurantId();
            if (res_id == 0) {
                myPreferences.deleteOrderRestaurant();
            } else {
                Restaurant orderRestaurant = myPreferences.getOrderRestaurant();
                if (orderRestaurant == null) {
                    appDatabase.cartDao().nukeTable();
                } else if (orderRestaurant.id != res_id) {
                    appDatabase.cartDao().nukeTable();
                    myPreferences.deleteOrderRestaurant();
                }
            }
        }).start();
    }

    private void initializeApi() {
        OkHttpClient okHttpClient = new OkHttpClient().newBuilder()
                .connectTimeout(1, TimeUnit.MINUTES)
                .readTimeout(1, TimeUnit.MINUTES)
                .writeTimeout(1, TimeUnit.MINUTES)
                .addInterceptor(new NetworkInterceptor()).addInterceptor(new ChuckerInterceptor(getApplicationContext()))
                .build();
        AndroidNetworking.initialize(getApplicationContext(), okHttpClient);

        new Timer().schedule(new CheckConnection(ourInstance), 0, 40000);
    }

    public String checkErrorMessage(JSONObject jsonObject) {
        RequestResponse requestResponse = new Gson().fromJson(jsonObject.toString(), RequestResponse.class);
        if (requestResponse.result != null) {
            if (requestResponse.result.success != 1) {
                if (requestResponse.result.message != null)
                    return requestResponse.result.message;
                else return "No results found";
            } else {
                return null;
            }
        } else {
            return "No results found";
        }
    }

    public void logout(Activity activity) {
        getMyPreferences().logout();
        new Thread(() -> {
            getAppDatabase().cartDao().nukeTable();
        }).start();
        startActivity(new Intent(activity, SplashActivity.class).addFlags(Intent.FLAG_ACTIVITY_NEW_TASK | Intent.FLAG_ACTIVITY_CLEAR_TASK | Intent.FLAG_ACTIVITY_CLEAR_TOP));
    }

    public String getStripePublishKey() {
        if (getMyPreferences().getSiteSettings().stripe_mode.equalsIgnoreCase("test")) {
            return getMyPreferences().getSiteSettings().stripe_publishkeyTest;
        } else {
            return getMyPreferences().getSiteSettings().stripe_publishkey;
        }
    }

    public int paymentMethodLogo(PaymentMethod paymentMethod) {
        int logo = -1;
        switch (paymentMethod.type.toLowerCase()) {
            case "cod":
                logo = R.drawable.icon_cod;
                break;
            case "paypal":
                logo = R.drawable.ic_card_paypal;
                break;
            case "credit":
            case "wallet":
                logo = R.drawable.ic_tiffin;
                break;
            case "stripe":
                if (!Validators.isNullOrEmpty(paymentMethod.card_brand) && paymentMethod.card_brand.toLowerCase().equalsIgnoreCase("mastercard"))
                    logo = R.drawable.ic_card_master;
                else if (!Validators.isNullOrEmpty(paymentMethod.card_brand) && paymentMethod.card_brand.toLowerCase().equalsIgnoreCase("american express"))
                    logo = R.drawable.ic_card_american;
                else if (!Validators.isNullOrEmpty(paymentMethod.card_brand) && paymentMethod.card_brand.toLowerCase().equalsIgnoreCase("visa"))
                    logo = R.drawable.ic_card_visa;
                else logo = R.drawable.ic_outline_credit_card_24;
                break;
            default:
                logo = R.drawable.ic_tiffin;
                break;
        }
        return logo;
    }

    public void profileChanged() {
        if (profileChangeListener != null) profileChangeListener.onChange();
    }

    public static class df {
        public static String format(float value) {
            return String.format("%.2f", value);
        }
    }

    private void initService() {
        new NotificationService(getApplicationContext());
    }

    public boolean isLoggedIn() {
        return getMyPreferences().getLoggedInUser() != null;
    }

    public void noInternet(Activity activity) {
        if (activity != null)
            startActivity(new Intent(activity, NoInternetActivity.class).setFlags(Intent.FLAG_ACTIVITY_CLEAR_TASK | Intent.FLAG_ACTIVITY_NEW_TASK | Intent.FLAG_ACTIVITY_CLEAR_TOP));
    }

    private void fetchCounts() {
        AndroidNetworking.get(ApiEndPoints.NEW_CHATALLCOUNT)
                .addQueryParameter("customer_id", String.valueOf(getMyPreferences().getLoggedInUser().id))
                .build().getAsJSONObject(new JSONObjectRequestListener() {
            @Override
            public void onResponse(JSONObject response) {
                try {
                    chatCount = Integer.parseInt(response.getString("total_count"));
                } catch (JSONException exception) {
                    exception.printStackTrace();
                }
                sendChatCountBroadCast(chatCount);

            }

            @Override
            public void onError(ANError anError) {

            }
        });
    }

    Handler handler = new Handler();
    Runnable runnable = new Runnable() {
        @Override
        public void run() {
            if (getMyPreferences().getLoggedInUser() != null)
                fetchCounts();
            handler.postDelayed(runnable, 5 * 1000);
        }
    };

    private void setAutoFetch() {
        handler.postDelayed(runnable, 5 * 1000);
    }

    private void sendChatCountBroadCast(int count) {
        Intent pushNotification = new Intent(Constants.CHAT_COUNT_UPDATE);
        pushNotification.putExtra("count", count);
        LocalBroadcastManager.getInstance(getApplicationContext()).sendBroadcast(pushNotification);
    }

    public File saveBitmapToFile(File file) {
        try {

            // BitmapFactory options to downsize the image_url
            BitmapFactory.Options o = new BitmapFactory.Options();
            o.inJustDecodeBounds = true;
            o.inSampleSize = 6;
            // factor of downsizing the image_url

            FileInputStream inputStream = new FileInputStream(file);
            //Bitmap selectedBitmap = null;
            BitmapFactory.decodeStream(inputStream, null, o);
            inputStream.close();

            // The new size we want to scale to
            final int REQUIRED_SIZE = 75;

            // Find the correct scale value. It should be the power of 2.
            int scale = 1;
            while (o.outWidth / scale / 2 >= REQUIRED_SIZE &&
                    o.outHeight / scale / 2 >= REQUIRED_SIZE) {
                scale *= 2;
            }

            BitmapFactory.Options o2 = new BitmapFactory.Options();
            o2.inSampleSize = scale;
            inputStream = new FileInputStream(file);

            Bitmap selectedBitmap = BitmapFactory.decodeStream(inputStream, null, o2);
            inputStream.close();

            // here i override the original image_url file
            file.createNewFile();
            FileOutputStream outputStream = new FileOutputStream(file);

            selectedBitmap.compress(Bitmap.CompressFormat.JPEG, 80, outputStream);

            return file;
        } catch (Exception e) {
            return null;
        }
    }

    public String compressImage(Context context, String imageUri) {

        String filePath = getRealPathFromURI(context, imageUri);
        Bitmap scaledBitmap = null;

        BitmapFactory.Options options = new BitmapFactory.Options();

//      by setting this field as true, the actual bitmap pixels are not loaded in the memory. Just the bounds are loaded. If
//      you try the use the bitmap here, you will get null.
        options.inJustDecodeBounds = true;
        Bitmap bmp = BitmapFactory.decodeFile(filePath, options);

        int actualHeight = options.outHeight;
        int actualWidth = options.outWidth;

//      max Height and width values of the compressed image_url is taken as 816x612

        float maxHeight = 816.0f;
        float maxWidth = 612.0f;
        float imgRatio = actualWidth / actualHeight;
        float maxRatio = maxWidth / maxHeight;

//      width and height values are set maintaining the aspect ratio of the image_url

        if (actualHeight > maxHeight || actualWidth > maxWidth) {
            if (imgRatio < maxRatio) {
                imgRatio = maxHeight / actualHeight;
                actualWidth = (int) (imgRatio * actualWidth);
                actualHeight = (int) maxHeight;
            } else if (imgRatio > maxRatio) {
                imgRatio = maxWidth / actualWidth;
                actualHeight = (int) (imgRatio * actualHeight);
                actualWidth = (int) maxWidth;
            } else {
                actualHeight = (int) maxHeight;
                actualWidth = (int) maxWidth;

            }
        }

//      setting inSampleSize value allows to load a scaled down version of the original image_url

        options.inSampleSize = calculateInSampleSize(options, actualWidth, actualHeight);

//      inJustDecodeBounds set to false to load the actual bitmap
        options.inJustDecodeBounds = false;

//      this options allow android to claim the bitmap memory if it runs low on memory
        options.inPurgeable = true;
        options.inInputShareable = true;
        options.inTempStorage = new byte[16 * 1024];

        try {
//          load the bitmap from its path
            bmp = BitmapFactory.decodeFile(filePath, options);
        } catch (OutOfMemoryError exception) {
            exception.printStackTrace();

        }
        try {
            scaledBitmap = Bitmap.createBitmap(actualWidth, actualHeight, Bitmap.Config.ARGB_8888);
        } catch (OutOfMemoryError exception) {
            exception.printStackTrace();
        }

        float ratioX = actualWidth / (float) options.outWidth;
        float ratioY = actualHeight / (float) options.outHeight;
        float middleX = actualWidth / 2.0f;
        float middleY = actualHeight / 2.0f;

        Matrix scaleMatrix = new Matrix();
        scaleMatrix.setScale(ratioX, ratioY, middleX, middleY);

        Canvas canvas = new Canvas(scaledBitmap);
        canvas.setMatrix(scaleMatrix);
        canvas.drawBitmap(bmp, middleX - bmp.getWidth() / 2, middleY - bmp.getHeight() / 2, new Paint(Paint.FILTER_BITMAP_FLAG));

//      check the rotation of the image_url and display it properly
        ExifInterface exif;
        try {
            exif = new ExifInterface(filePath);

            int orientation = exif.getAttributeInt(
                    ExifInterface.TAG_ORIENTATION, 0);
            Log.d("EXIF", "Exif: " + orientation);
            Matrix matrix = new Matrix();
            if (orientation == 6) {
                matrix.postRotate(90);
                Log.d("EXIF", "Exif: " + orientation);
            } else if (orientation == 3) {
                matrix.postRotate(180);
                Log.d("EXIF", "Exif: " + orientation);
            } else if (orientation == 8) {
                matrix.postRotate(270);
                Log.d("EXIF", "Exif: " + orientation);
            }
            scaledBitmap = Bitmap.createBitmap(scaledBitmap, 0, 0,
                    scaledBitmap.getWidth(), scaledBitmap.getHeight(), matrix,
                    true);
        } catch (IOException e) {
            e.printStackTrace();
        }

        FileOutputStream out = null;
        String filename = getFilename();
        try {
            out = new FileOutputStream(filename);

//          write the compressed bitmap at the destination specified by filename.
            scaledBitmap.compress(Bitmap.CompressFormat.JPEG, 80, out);

        } catch (FileNotFoundException e) {
            e.printStackTrace();
        }

        return filename;

    }

    public String getFilename() {
        File file = new File(Environment.getExternalStorageDirectory().getPath(), "TiffinTom/ProfilePictures");
        if (!file.exists()) {
            file.mkdirs();
        }
        String uriSting = (file.getAbsolutePath() + "/" + System.currentTimeMillis() + ".jpg");
        return uriSting;

    }

    private String getRealPathFromURI(Context context, String contentURI) {
        Uri contentUri = Uri.parse(contentURI);
        Cursor cursor = context.getContentResolver().query(contentUri, null, null, null, null);
        if (cursor == null) {
            return contentUri.getPath();
        } else {
            cursor.moveToFirst();
            int index = cursor.getColumnIndex(MediaStore.Images.ImageColumns.DATA);
            return cursor.getString(index);
        }
    }

    public int calculateInSampleSize(BitmapFactory.Options options, int reqWidth, int reqHeight) {
        final int height = options.outHeight;
        final int width = options.outWidth;
        int inSampleSize = 1;

        if (height > reqHeight || width > reqWidth) {
            final int heightRatio = Math.round((float) height / (float) reqHeight);
            final int widthRatio = Math.round((float) width / (float) reqWidth);
            inSampleSize = heightRatio < widthRatio ? heightRatio : widthRatio;
        }
        final float totalPixels = width * height;
        final float totalReqPixelsCap = reqWidth * reqHeight * 2;
        while (totalPixels / (inSampleSize * inSampleSize) > totalReqPixelsCap) {
            inSampleSize++;
        }

        return inSampleSize;
    }

    public boolean isConnected(Context context) {
        return isInternetAvailable;
    }

    class CheckConnection extends TimerTask {
        private Context context;

        public CheckConnection(Context context) {
            this.context = context;
        }

        public void run() {
            isInternetAvailable = hasInternetConnected(context);
        }

        private boolean hasInternetConnected(Context context) {
            if (hasNetworkAvailable(context)) {
                try {
                    HttpURLConnection connection = (HttpURLConnection) new URL(ApiEndPoints.NEW_BASE_URL_RMS()).openConnection();
                    connection.setRequestProperty("User-Agent", "Test");
                    connection.setRequestProperty("Connection", "close");
                    connection.setConnectTimeout(1500);
                    connection.connect();

                    boolean internetAvailable = (connection.getResponseCode() == 200);

                    return internetAvailable;
                } catch (MalformedURLException e) {
                    e.printStackTrace();
                } catch (IOException e) {
                    e.printStackTrace();
                }
            }
            return false;
        }
    }

    private boolean hasNetworkAvailable(Context context) {
        ConnectivityManager cm = (ConnectivityManager) context.getSystemService(Context.CONNECTIVITY_SERVICE);
//
        NetworkInfo activeNetwork = cm.getActiveNetworkInfo();
        return activeNetwork != null && activeNetwork.isConnectedOrConnecting();
    }
    private void getFirebaseToken() {
        FirebaseMessaging.getInstance().getToken().addOnCompleteListener(task -> {
            if (!task.isSuccessful()) {
                return;
            }
            String token = task.getResult();
            myPreferences.setUserFCMToken(token);
        });
    }

    public DineinSiteSettings findSetting(String key) {
        DineinSiteSettings settings = new DineinSiteSettings();
        for (DineinSiteSettings siteSettings : dineinSiteSettings) {
            if (siteSettings.key.equalsIgnoreCase(key)) {
                settings = siteSettings;
            }
        }
        return settings;
//        DineinSiteSettings siteSetting = new DineinSiteSettings();
//        siteSetting.key = key;
//        int index = dineinSiteSettings.indexOf(siteSetting);
//        if (index == -1) return null;
//        if (dineinSiteSettings.size() == 0 || dineinSiteSettings.size() - 1 < index) return null;
//        return dineinSiteSettings.get(index);
    }

    public DineinPaymentMethods findPaymentEnabled(String name) {
        DineinPaymentMethods payments = new DineinPaymentMethods();
        for (DineinPaymentMethods paymentMethods : dineinPaymentMethods) {
            if (paymentMethods.name.equalsIgnoreCase(name)) {
                payments = paymentMethods;
            }
        }
        return payments;
//        DineinPaymentMethods paymentMethod = new DineinPaymentMethods();
//        paymentMethod.name = name;
//        int index = dineinPaymentMethods.indexOf(paymentMethod);
//        if (index == -1) return null;
//        if (dineinPaymentMethods.size() == 0 || dineinPaymentMethods.size() - 1 < index)
//            return null;
//        return dineinPaymentMethods.get(index);
    }

    public static float calculatedAddonPrice(String orderTypeId, Addon addon) {
        if (orderTypeId == null) return addon.price;
        if (orderTypeId.equalsIgnoreCase("3")) return addon.delivery_price;
        if (orderTypeId.equalsIgnoreCase("5")) return addon.takeaway_price;
        if (orderTypeId.equalsIgnoreCase("2")) return addon.takeaway_price;
        return addon.price;
    }

    public static float calculatedProductPrice(String orderTypeId, DineinMenus product) {
        MyApp myApp = MyApp.getInstance();
        if (!Validators.isNullOrEmpty(myApp.banquetDay) && !Validators.isNullOrEmpty(myApp.banquetOrderTypes) && product.is_banquet) {

            if (orderTypeId == null) return product.price;
            if (orderTypeId.equalsIgnoreCase("3")) {
                if (myApp.banquetOrderTypes.contains("delivery"))
                    return product.waiting_price;
                else return product.delivery_price;
            }
            if (orderTypeId.equalsIgnoreCase("5")) {
                if (myApp.banquetOrderTypes.contains("waiting"))
                    return product.waiting_price;
                else
                    return product.takeaway_price;
            }
            if (orderTypeId.equalsIgnoreCase("2")) {
                if (myApp.banquetOrderTypes.contains("collection"))
                    return product.waiting_price;
                else
                    return product.takeaway_price;
            }
            if (orderTypeId.equalsIgnoreCase("1")) {
                if (myApp.banquetOrderTypes.contains("dinein"))
                    return product.waiting_price;
                else
                    return product.price;
            }
            return product.price;
        } else {
            if (orderTypeId == null) return product.price;
            if (orderTypeId.equalsIgnoreCase("3")) return product.delivery_price;
            if (orderTypeId.equalsIgnoreCase("5")) return product.takeaway_price;
            if (orderTypeId.equalsIgnoreCase("2")) return product.takeaway_price;
            return product.price;
        }
    }

}

