package com.tiffintom.activity;

import android.content.BroadcastReceiver;
import android.content.Context;
import android.content.Intent;
import android.content.IntentFilter;
import android.graphics.Color;
import android.graphics.drawable.ColorDrawable;
import android.os.Bundle;
import android.view.LayoutInflater;
import android.view.View;
import android.widget.ImageView;
import android.widget.LinearLayout;
import android.widget.ProgressBar;
import android.widget.TextView;

import androidx.annotation.Nullable;
import androidx.appcompat.app.AlertDialog;
import androidx.appcompat.widget.Toolbar;
import androidx.cardview.widget.CardView;
import androidx.localbroadcastmanager.content.LocalBroadcastManager;
import androidx.recyclerview.widget.LinearLayoutManager;
import androidx.recyclerview.widget.RecyclerView;

import com.airbnb.lottie.LottieAnimationView;
import com.androidnetworking.AndroidNetworking;
import com.androidnetworking.error.ANError;
import com.androidnetworking.interfaces.JSONArrayRequestListener;
import com.androidnetworking.interfaces.JSONObjectRequestListener;
import com.google.gson.Gson;
import com.google.gson.reflect.TypeToken;
import com.tiffintom.R;
import com.tiffintom.adapters.AddressBookAdapter;
import com.tiffintom.base.BaseActivity;
import com.tiffintom.common.CommonFunctions;
import com.tiffintom.common.Validators;
import com.tiffintom.fragment.DeliverableChangeAddressDialog;
import com.tiffintom.models.Address;
import com.tiffintom.models.ApiError;
import com.tiffintom.models.Postcode;
import com.tiffintom.models.User;
import com.tiffintom.network.ApiEndPoints;
import com.tiffintom.network.ApiUtils;
import com.tiffintom.utils.Constants;
import com.tiffintom.utils.LogUtils;
import com.tiffintom.utils.ToastUtils;

import org.json.JSONArray;
import org.json.JSONException;
import org.json.JSONObject;

import java.lang.reflect.Type;
import java.util.ArrayList;
import java.util.List;

import static com.tiffintom.utils.Constants.TOAST_UPDATE;

/**
 * Created by Amrish on 25-08-2020.
 */
public class AddressesActivity extends BaseActivity {
    private RecyclerView rvAddress;
    private ProgressBar pbLoading;
    private ArrayList<Address> addresses = new ArrayList<>();
    private AddressBookAdapter addressAdapter;
    private TextView tvScreenName;
    private Toolbar toolbar;
    private ImageView ivBackArrow;
    private LinearLayout llBackArrow;
    private boolean withCard = true;
    private TextView tv_CurrentLocation, tvAddAddress, tvSavedLocations;
    private User loggedInUser = myApp.getMyPreferences().getLoggedInUser();
    private LottieAnimationView loadingView;
    private CardView llToastLayout;
    private TextView tvToastMessage;
    private BroadcastReceiver toastReceiver;

    @Override
    protected void onCreate(@Nullable Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_addresses);
        initViews();
        updateUserName();
        updateCurrentLocation();
        if (myApp.getMyPreferences().getLoggedInUser() != null) {
            fetchAddress();
        } else {
            startActivity(new Intent(this, AuthActivity.class));
            finish();
            ToastUtils.makeToast(this, "Please login first");
        }
        tvScreenName.setText("Address Book");
        tvScreenName.setVisibility(View.VISIBLE);
        setListener();
    }

    private void updateCurrentLocation() {
        Postcode currentPostcode = myApp.getMyPreferences().getCurrentPostcode();
        tv_CurrentLocation.setText(currentPostcode.street + ", " + currentPostcode.post_code);
    }
    private void setListener() {
        llBackArrow.setOnClickListener(view -> onBackPressed());
        tvAddAddress.setOnClickListener(view -> {
            DeliverableChangeAddressDialog dialogFragment = DeliverableChangeAddressDialog.getInstance(true);
            dialogFragment.show(getSupportFragmentManager(), "dialog");
            dialogFragment.setDialogDismissListener(o -> {
                fetchAddress();
            });
        });
        toastReceiver = new BroadcastReceiver() {
            @Override
            public void onReceive(Context context, Intent intent) {
                runOnUiThread(() -> {
                    if (intent != null && intent.getExtras() != null) {
                        String message = intent.getStringExtra("message");
                        if (!Validators.isNullOrEmpty(message)) {
                            llToastLayout.setVisibility(View.VISIBLE);
                            tvToastMessage.setText(message);
                        } else {
                            llToastLayout.setVisibility(View.GONE);
                        }
                    } else {
                        llToastLayout.setVisibility(View.GONE);
                    }
                    llToastLayout.postDelayed(() -> {
                        llToastLayout.setVisibility(View.GONE);
                    }, 2500);
                });
            }
        };
    }

    @Override
    protected void initViews() {
        super.initViews();
        rvAddress = findViewById(R.id.rvAddress);
        pbLoading = findViewById(R.id.pbLoading);
        toolbar = findViewById(R.id.toolbar);
        tvScreenName = findViewById(R.id.tvScreenTitle);
        tvUserName = findViewById(R.id.tvUserName);
        ivBackArrow = findViewById(R.id.ivBackArrow);
        llBackArrow = findViewById(R.id.llBackArrow);
        tv_CurrentLocation = findViewById(R.id.tvCurrentLocation);
        tvAddAddress = findViewById(R.id.tvAddAddress);
        loadingView = findViewById(R.id.lodingView);
        tvSavedLocations = findViewById(R.id.tvSavedLocations);
        llToastLayout = findViewById(R.id.llToastMessage);
        tvToastMessage = findViewById(R.id.tvToastMessage);

        if (getSupportActionBar() == null) {
            setSupportActionBar(toolbar);
        }
        addressAdapter = new AddressBookAdapter(this, addresses, (position, data) -> {
        }, (position, data) -> {
//           AddAddressBottomSheetDialogFragment addAddressBottomSheetDialogFragment = AddAddressBottomSheetDialogFragment.getInstance((Address) data);
//            addAddressBottomSheetDialogFragment.show(getSupportFragmentManager(), "Edit address");
//            addAddressBottomSheetDialogFragment.setDialogDismissListener(o -> fetchAddress());
            DeliverableChangeAddressDialog dialogFragment = DeliverableChangeAddressDialog.getInstance(true, (Address) data);
            dialogFragment.show(getSupportFragmentManager(), "dialog");
            dialogFragment.setDialogDismissListener(o -> {
                fetchAddress();
            });

        }, (position, data) -> {
            deleteConfirmation((Address) data);
        });
        rvAddress.setLayoutManager(new LinearLayoutManager(this, RecyclerView.VERTICAL, false));
        rvAddress.setAdapter(addressAdapter);
    }

    private void fetchAddress() {
        runOnUiThread(() -> {
            loadingView.setVisibility(View.VISIBLE);
        });
        ApiUtils.getCheckoutAddresses(myApp.getMyPreferences().getLoggedInUser().id, 0, 1)
                .getAsJSONArray(new JSONArrayRequestListener() {
                    @Override
                    public void onResponse(JSONArray response) {
                        Type type = new TypeToken<List<Address>>() {
                        }.getType();
                        addresses.clear();
                        addresses.addAll(new Gson().fromJson(response.toString(), type));
                        addressAdapter.notifyDataSetChanged();
                        tvSavedLocations.setVisibility(View.VISIBLE);

                        runOnUiThread(() -> loadingView.setVisibility(View.GONE));
                    }

                    @Override
                    public void onError(ANError anError) {
                        anError.printStackTrace();
                        if (anError.getErrorCode() == 400) {
                            addresses.clear();
                            addressAdapter.notifyDataSetChanged();
                            myApp.getMyPreferences().saveCurrentAddress(null);
                            ApiError apiError = anError.getErrorAsObject(ApiError.class);
                            ToastUtils.makeSnackToast(getApplicationContext(), apiError.getMessage());
                        }
                        runOnUiThread(() -> loadingView.setVisibility(View.GONE));
                        if (!CommonFunctions.isConnected(AddressesActivity.this))
                            myApp.noInternet(AddressesActivity.this);
                    }
                });
    }

    private void deleteConfirmation(Address address) {
        AlertDialog alertDialog = new AlertDialog.Builder(this).create();
        alertDialog.getWindow().setBackgroundDrawable(new ColorDrawable(Color.TRANSPARENT));
        LayoutInflater inflater = (LayoutInflater) this.getSystemService(Context.LAYOUT_INFLATER_SERVICE);
        View dialogView = inflater.inflate(R.layout.dialog_remove_card, null);

        TextView tvMessage = dialogView.findViewById(R.id.tvMessage);
        tvMessage.setText("Are you sure you want to remove this address?");
        TextView tvCancel = dialogView.findViewById(R.id.tvCancel);
        TextView tvConfirm = dialogView.findViewById(R.id.tvConfirm);
        tvConfirm.setOnClickListener(view -> {
            deleteAddress(address);
            alertDialog.dismiss();
        });
        tvCancel.setOnClickListener(view -> {
            alertDialog.dismiss();
        });
        alertDialog.setView(dialogView);
        alertDialog.setCancelable(false);
        alertDialog.show();
    }

    private void deleteAddress(Address address) {
        runOnUiThread(() -> progressDialog.show());
        AndroidNetworking.delete(ApiEndPoints.NEW_ADDRESSBOOK + "/" + address.id).build()
                .getAsJSONObject(new JSONObjectRequestListener() {
                    @Override
                    public void onResponse(JSONObject response) {
                        ToastUtils.makeSnackToast(AddressesActivity.this, "Address deleted successfully");
                        fetchAddress();


                        runOnUiThread(() -> progressDialog.dismiss());
                    }

                    @Override
                    public void onError(ANError anError) {
                        runOnUiThread(() -> progressDialog.dismiss());

                        anError.printStackTrace();

                        if (!CommonFunctions.isConnected(AddressesActivity.this))
                            myApp.noInternet(AddressesActivity.this);
                    }
                });
    }

    @Override
    protected void onResume() {
        super.onResume();
        LocalBroadcastManager.getInstance(this).registerReceiver(toastReceiver, new IntentFilter(TOAST_UPDATE));
    }

    @Override
    protected void onStop() {
        LocalBroadcastManager.getInstance(this).unregisterReceiver(toastReceiver);
        super.onStop();
    }

    @Override
    protected void onActivityResult(int requestCode, int resultCode, @Nullable Intent data) {
        super.onActivityResult(requestCode, resultCode, data);
        if (requestCode == Constants.CODE_MESSAGE && resultCode == RESULT_OK) {
            LocalBroadcastManager.getInstance(this).registerReceiver(toastReceiver, new IntentFilter(TOAST_UPDATE));
            if (data != null) {
                setResult(RESULT_OK, data);
                String message = data.getStringExtra("message");
                if (!Validators.isNullOrEmpty(message)) {
                    ToastUtils.makeSnackToast(this, message);
                }

            }
        }
    }
}
