package com.tiffintom.activity;

import android.content.Context;
import android.content.Intent;
import android.graphics.Color;
import android.graphics.drawable.ColorDrawable;
import android.os.Bundle;
import android.util.Log;
import android.view.LayoutInflater;
import android.view.View;
import android.widget.Button;
import android.widget.CheckBox;
import android.widget.EditText;
import android.widget.ImageView;
import android.widget.LinearLayout;
import android.widget.RelativeLayout;
import android.widget.TextView;

import androidx.annotation.Nullable;
import androidx.appcompat.app.AlertDialog;

import com.androidnetworking.AndroidNetworking;
import com.androidnetworking.error.ANError;
import com.androidnetworking.interfaces.JSONObjectRequestListener;
import com.androidnetworking.interfaces.ParsedRequestListener;
import com.facebook.CallbackManager;
import com.facebook.FacebookCallback;
import com.facebook.FacebookException;
import com.facebook.GraphRequest;
import com.facebook.login.LoginManager;
import com.facebook.login.LoginResult;
import com.facebook.login.widget.LoginButton;
import com.google.android.gms.auth.api.signin.GoogleSignIn;
import com.google.android.gms.auth.api.signin.GoogleSignInAccount;
import com.google.android.gms.auth.api.signin.GoogleSignInClient;
import com.google.android.gms.auth.api.signin.GoogleSignInOptions;
import com.google.android.gms.common.api.ApiException;
import com.google.android.gms.tasks.Task;
import com.google.android.material.textfield.TextInputEditText;
import com.google.android.material.textfield.TextInputLayout;
import com.google.gson.Gson;
import com.tiffintom.MyApp;
import com.tiffintom.R;
import com.tiffintom.base.BaseActivity;
import com.tiffintom.common.CommonFunctions;
import com.tiffintom.common.Validators;
import com.tiffintom.fragment.ChangePasswordFragment;
import com.tiffintom.fragment.PhoneOrEmailVerificationBottomsheet;
import com.tiffintom.models.CartItem;
import com.tiffintom.models.FavouriteRestaurant;
import com.tiffintom.models.HistoryCart;
import com.tiffintom.models.Restaurant;
import com.tiffintom.models.User;
import com.tiffintom.network.ApiEndPoints;
import com.tiffintom.network.ApiUtils;
import com.tiffintom.utils.LogUtils;
import com.tiffintom.utils.ToastUtils;

import org.json.JSONException;
import org.json.JSONObject;

import java.util.Arrays;
import java.util.List;

import static com.tiffintom.network.ApiEndPoints.NEW_FORGET_PASSWORD;
import static com.tiffintom.network.ApiEndPoints.SOCIAL_LOGIN;
import static com.tiffintom.utils.Constants.CODE_GOOGLE_LOGIN;

/**
 * Created by Amrish on 18-08-2020.
 */
public class AuthActivity extends BaseActivity {
    private TextView tvSkip, tvForgotPass, tvSelectedCountry;
    private TextInputEditText etEmail, etPassword;
    private TextInputLayout tilEmail, tilPassword;
    private Button btnSignIn;
    private RelativeLayout signupButton;
    private CheckBox cbRemember;
    private LinearLayout llCountry;
    private ImageView ivMap;
    // Facebook
    private RelativeLayout fbButton;
    private LoginButton btnFacebook;
    List<String> fbPermissions = Arrays.asList("email", "public_profile");
    private CallbackManager fbCallbackManager;

    //Google
    GoogleSignInClient mGoogleSignInClient;
    RelativeLayout gplusButton;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_auth);
        initViews();
        setListeners();
        updateViews();
    }

    @Override
    protected void initViews() {
        super.initViews();
        tvSkip = findViewById(R.id.tvSkip);
        etEmail = findViewById(R.id.etEmail);
        etPassword = findViewById(R.id.etPassword);
        btnSignIn = findViewById(R.id.btnSignin);
        tilEmail = findViewById(R.id.tilEmail);
        tilPassword = findViewById(R.id.tilPassword);
        cbRemember = findViewById(R.id.cvRemember);
        signupButton = findViewById(R.id.signupButton);
        tvForgotPass = findViewById(R.id.tvForgotPass);

        fbButton = findViewById(R.id.fbButton);
        btnFacebook = findViewById(R.id.btnFacebook);
        btnFacebook.setPermissions(fbPermissions);
        fbCallbackManager = CallbackManager.Factory.create();
        gplusButton = findViewById(R.id.gplusButton);
        llCountry = findViewById(R.id.llCountry);
        tvSelectedCountry = findViewById(R.id.tvCountry);
        ivMap = findViewById(R.id.ivMap);

        if (myApp.getMyPreferences().getSiteSettings() != null && myApp.getMyPreferences().getSiteSettings().google_login.equalsIgnoreCase("y")) {
            gplusButton.setVisibility(View.VISIBLE);
            initGoogleSignIn();
        } else {
            gplusButton.setVisibility(View.GONE);
        }
        if (myApp.getMyPreferences().getSiteSettings() != null && myApp.getMyPreferences().getSiteSettings().facebook_login.equalsIgnoreCase("y")) {
            fbButton.setVisibility(View.VISIBLE);
        } else {
            fbButton.setVisibility(View.GONE);
        }

    }

    private void initGoogleSignIn() {
        // profile. ID and basic profile are included in DEFAULT_SIGN_IN.
        GoogleSignInOptions gso = new GoogleSignInOptions.Builder(GoogleSignInOptions.DEFAULT_SIGN_IN)
                .requestEmail()
                .build();
        mGoogleSignInClient = GoogleSignIn.getClient(this, gso);
    }

    private void updateViews() {
        if (myApp.getMyPreferences().getRememberDetails() != null) {
            User rememberedUser = myApp.getMyPreferences().getRememberDetails();
            etEmail.setText(rememberedUser.username);
            etPassword.setText(rememberedUser.password);
            cbRemember.setChecked(true);
        }
        if (myApp.getMyPreferences().getSelectedCountry() != null) {
            tvSelectedCountry.setText(myApp.getMyPreferences().getSelectedCountry());
            if (tvSelectedCountry.getText().toString().equalsIgnoreCase("IN")) {
                ivMap.setImageResource(R.drawable.country_india_filled);
            } else
                ivMap.setImageResource(R.drawable.country_uk_filled);
        }
    }

    private void setListeners() {
        tvSkip.setOnClickListener(view -> {
            myApp.getMyPreferences().login_skipped(true);
            checkPostCodeAndOpenMainActivity();
        });
        tvForgotPass.setOnClickListener(view -> {
            openForgotPasswordDialog();
        });
        btnSignIn.setOnClickListener(view -> {
            if (isValid()) {
                CommonFunctions.hideKeyboard(this, btnSignIn);
                new Thread(this::doLogin).start();
            }

        });
        signupButton.setOnClickListener(view -> {
            startActivity(new Intent(AuthActivity.this, SignupActivity.class));
        });
        gplusButton.setOnClickListener(view -> {
            Intent signInIntent = mGoogleSignInClient.getSignInIntent();
            startActivityForResult(signInIntent, CODE_GOOGLE_LOGIN);
        });
        fbButton.setOnClickListener(view -> {
            btnFacebook.performClick();

        });
        btnFacebook.registerCallback(fbCallbackManager, new FacebookCallback<LoginResult>() {
            @Override
            public void onSuccess(LoginResult loginResult) {

                GraphRequest graphRequest = GraphRequest.newMeRequest(loginResult.getAccessToken(), (object, response) -> {
                    LogUtils.e(":::FB RESPONSE:::");
                    LogUtils.e(new Gson().toJson(object));
                    try {
                        String fbEmail = object.getString("email");
                        String fbFirstName = object.getString("first_name");
                        String fbLastName = object.getString("last_name");
                        String fbId = object.getString("id");
                        performSocialLogin(fbFirstName, fbLastName, fbEmail, fbId);
                    } catch (JSONException e) {
                        e.printStackTrace();
                        LogUtils.e("Please allow required permissions to continue using app.");
                    }
                    LoginManager.getInstance().logOut();

                });
                Bundle parameters = new Bundle();
                parameters.putString("fields", "first_name,last_name,email");
                graphRequest.setParameters(parameters);
                graphRequest.executeAsync();
            }

            @Override
            public void onCancel() {

            }

            @Override
            public void onError(FacebookException error) {

            }
        });

        llCountry.setOnClickListener(view -> startActivity(new Intent(AuthActivity.this, TransportActivity.class).putExtra("destination", "select_country").putExtra("hideToolbar", true)));
    }

    private void openForgotPasswordDialog() {
        AlertDialog alertDialog = new AlertDialog.Builder(AuthActivity.this).create();
        alertDialog.getWindow().setBackgroundDrawable(new ColorDrawable(Color.TRANSPARENT));
        LayoutInflater inflater = (LayoutInflater) getSystemService(Context.LAYOUT_INFLATER_SERVICE);
        View dialogView = inflater.inflate(R.layout.dialog_forgot_password, null);

        TextView tvSubmit = dialogView.findViewById(R.id.tvSubmit);
        TextView tvCancel = dialogView.findViewById(R.id.tvCancel);
        EditText etDialogEmail = dialogView.findViewById(R.id.etDialogEmail);

        tvSubmit.setOnClickListener(view -> {
            etDialogEmail.setError(null);
            if (Validators.isEmail(etDialogEmail.getText().toString())) {
                alertDialog.dismiss();
                forgotPassword(etDialogEmail.getText().toString());
            } else {
                etDialogEmail.requestFocus();
                etDialogEmail.setError("Please enter valid email address");
            }
        });

        tvCancel.setOnClickListener(view -> alertDialog.dismiss());
        alertDialog.setView(dialogView);
        alertDialog.setCancelable(false);
        alertDialog.show();

    }

    private void forgotPassword(String s) {
        runOnUiThread(() -> progressDialog.show());
        AndroidNetworking.post(NEW_FORGET_PASSWORD)
                .addBodyParameter("username", s)
                .build()
                .getAsJSONObject(new JSONObjectRequestListener() {
                    @Override
                    public void onResponse(JSONObject response) {
                        runOnUiThread(() -> progressDialog.dismiss());
                        ToastUtils.makeToast(getApplicationContext(), "Link send successfully ");
                    }

                    @Override
                    public void onError(ANError anError) {
                        runOnUiThread(() -> progressDialog.dismiss());
                        anError.printStackTrace();
                        if (!CommonFunctions.isConnected(AuthActivity.this))
                            myApp.noInternet(AuthActivity.this);
                    }
                });
    }

    private void performSocialLogin(String firstName, String lastName, String email, String socialId) {
        runOnUiThread(() -> progressDialog.show());
        AndroidNetworking.post(ApiEndPoints.NEW_CUSTOMER_SIGNUP)
                .addBodyParameter("username", email)
                .addBodyParameter("password", etPassword.getText().toString())
                .addBodyParameter("first_name", firstName)
                .addBodyParameter("last_name", lastName)
                .addBodyParameter("phone_number", "")
                .addBodyParameter("referred_by", "")
                .addBodyParameter("social_id", socialId)
                .addBodyParameter("device_id", !Validators.isNullOrEmpty(myApp.getMyPreferences().getDeviceId()) ? myApp.getMyPreferences().getDeviceId() : "")
                .build()
                .getAsObject(User.class, new ParsedRequestListener<User>() {
                    @Override
                    public void onResponse(User response) {
                        runOnUiThread(() -> progressDialog.dismiss());
                        LogUtils.e(new Gson().toJson(response));
                        myApp.getMyPreferences().saveLoggedInUser(response);
                        if (Validators.isNullOrEmpty(myApp.getMyPreferences().getLoggedInUser().first_name) || Validators.isNullOrEmpty(myApp.getMyPreferences().getLoggedInUser().last_name)) {
                            startActivity(new Intent(AuthActivity.this, TransportActivity.class).putExtra("destination", "profile_edit").putExtra("name", "Profile").putExtra("hideBackArrow", true).putExtra("fromAuth", true));
                        }else if (myApp.getMyPreferences().getSiteSettings().signup_verify.equalsIgnoreCase("1") && myApp.getMyPreferences().getLoggedInUser().phone_verify.equalsIgnoreCase("false"))
                        {
                            startActivity(new Intent(AuthActivity.this, TransportActivity.class).putExtra("destination", "phoneverify").putExtra("hideBackArrow", true).putExtra("type","phone").putExtra("customer_phone",myApp.getMyPreferences().getLoggedInUser().phone_number).putExtra("customer_email",myApp.getMyPreferences().getLoggedInUser().username).putExtra("customer_id",myApp.getMyPreferences().getLoggedInUser().id).putExtra("hideToolbar",true));

                        }else
                        {
                            fetchProfile();
                        }
                    }

                    @Override
                    public void onError(ANError anError) {
                        anError.printStackTrace();
                        LogUtils.e("SIGNUP ERROR:", anError.getErrorBody());
                        ToastUtils.makeToast(AuthActivity.this, "Something went wrong!!");
                        runOnUiThread(() -> progressDialog.dismiss());

                        if (!CommonFunctions.isConnected(AuthActivity.this)) {
                            myApp.noInternet(AuthActivity.this);
                        }
                    }
                });

    }

    private boolean isValid() {
        tilEmail.setError(null);
        tilPassword.setError(null);
        if (Validators.isNullOrEmpty(etEmail.getText().toString())) {
            tilEmail.setError("Please enter email");
            etEmail.requestFocus();
            return false;
        }
        if (!Validators.isEmail(etEmail.getText().toString())) {
            tilEmail.setError("Please enter valid email");
            etEmail.requestFocus();
            return false;
        }
        if (Validators.isNullOrEmpty(etPassword.getText().toString())) {
            tilPassword.setError("Please enter password");
            etPassword.requestFocus();
            return false;
        }
        return true;
    }

    private void doLogin() {
        runOnUiThread(() -> progressDialog.show());

        User user = new User();
        user.username = etEmail.getText().toString();
        user.password = etPassword.getText().toString();
        user.device_name = "ANDROID";
        user.device_id = myApp.getMyPreferences().getDeviceId();

        ApiUtils.getLogin(user)
                .getAsJSONObject(new JSONObjectRequestListener() {
                    @Override
                    public void onResponse(JSONObject response) {
                        runOnUiThread(() -> progressDialog.dismiss());
                        User user1 = new Gson().fromJson(response.toString(), User.class);
                        ToastUtils.makeToast(AuthActivity.this, "Welcome Back");
                        if (cbRemember.isChecked()) {
                            myApp.getMyPreferences().saveRememberDetails(user);
                        } else {
                            myApp.getMyPreferences().removeRememberDetails();
                        }
                        if (Validators.isNullOrEmpty(user1.first_name) && Validators.isNullOrEmpty(user1.last_name)) {
                            startActivity(new Intent(getApplicationContext(), TransportActivity.class).putExtra("destination", "profile_edit").putExtra("name", "Profile"));

                        } else if (myApp.getMyPreferences().getSiteSettings().signup_verify.equalsIgnoreCase("1") && user1.phone_verify.equalsIgnoreCase("false"))
                        {
                            startActivity(new Intent(AuthActivity.this, TransportActivity.class).putExtra("destination", "phoneverify").putExtra("hideBackArrow", true).putExtra("type","phone").putExtra("customer_phone",user1.phone_number).putExtra("customer_email",user1.username).putExtra("customer_id",user1.id).putExtra("hideToolbar",true));

                        }else
                        {
                            fetchProfile();
                        }
                    }


                    @Override
                    public void onError(ANError anError) {
                        anError.printStackTrace();

                        runOnUiThread(() -> progressDialog.dismiss());
                        if (anError.getErrorCode() == 400 || anError.getErrorCode() == 401) {
                            try {
                                JSONObject jsonObject = new JSONObject(anError.getErrorBody().toString());
                                ToastUtils.makeToast(getApplicationContext(), jsonObject.getString("message"));
                            } catch (JSONException exception) {
                                exception.printStackTrace();
                            }
                        } else{
                            if (!CommonFunctions.isConnected(AuthActivity.this))
                                myApp.noInternet(AuthActivity.this);
                        }


                    }
                });
    }

    private void fetchProfile() {
        runOnUiThread(() -> progressDialog.show());
        AndroidNetworking.get(ApiEndPoints.NEW_ACCOUNT)
                .build()
                .getAsJSONObject(new JSONObjectRequestListener() {
                    @Override
                    public void onResponse(JSONObject response) {
                        User user1 = new Gson().fromJson(response.toString(), User.class);
                        myApp.getMyPreferences().saveLoggedInUser(user1);
                        checkPostCodeAndOpenMainActivity();
                        runOnUiThread(() -> progressDialog.dismiss());
                    }

                    @Override
                    public void onError(ANError anError) {
                        anError.printStackTrace();
                        LogUtils.e("SIGNUP PROFILE FETCH ERROR: ", anError.getErrorBody());
                        runOnUiThread(() -> progressDialog.dismiss());

                        if (!CommonFunctions.isConnected(AuthActivity.this))
                            myApp.noInternet(AuthActivity.this);
                    }
                });
    }

    private void checkPostCodeAndOpenMainActivity() {
        if (myApp.getMyPreferences().getCurrentPostcode() == null) {
            Bundle bundle = new Bundle();
            bundle.putString("destination", "postcode");
            bundle.putBoolean("hideToolbar", true);
            bundle.putBoolean("finish", true);
            startActivity(new Intent(AuthActivity.this, TransportActivity.class).putExtras(bundle).setFlags(Intent.FLAG_ACTIVITY_CLEAR_TASK | Intent.FLAG_ACTIVITY_NEW_TASK | Intent.FLAG_ACTIVITY_CLEAR_TOP));
        } else {
            startActivity(new Intent(AuthActivity.this, MainActivity.class).setFlags(Intent.FLAG_ACTIVITY_CLEAR_TASK | Intent.FLAG_ACTIVITY_NEW_TASK | Intent.FLAG_ACTIVITY_CLEAR_TOP));
            finish();
        }
    }

    private void handleSignInResult(Task<GoogleSignInAccount> completedTask) {
        try {
            GoogleSignInAccount account = completedTask.getResult(ApiException.class);
            performSocialLogin(account.getDisplayName(), "", account.getEmail(), account.getId());
            mGoogleSignInClient.signOut();
        } catch (ApiException e) {
            mGoogleSignInClient.signOut();
            e.printStackTrace();
            LogUtils.e("" + e.getStatusCode());
        }

    }

    @Override
    protected void onActivityResult(int requestCode, int resultCode, @Nullable Intent data) {
        super.onActivityResult(requestCode, resultCode, data);
        if (data != null) {
            if (requestCode == CODE_GOOGLE_LOGIN) {
                Task<GoogleSignInAccount> task = GoogleSignIn.getSignedInAccountFromIntent(data);

                handleSignInResult(task);
            } else {
                fbCallbackManager.onActivityResult(requestCode, resultCode, data);
            }
        }
    }

    @Override
    protected void onDestroy() {
        super.onDestroy();
    }
}
