package com.tiffintom.activity;

import android.content.Intent;
import android.net.Uri;
import android.os.Bundle;

import androidx.annotation.Nullable;

import com.tiffintom.R;
import com.tiffintom.base.BaseActivity;
import com.tiffintom.common.Validators;

import java.util.List;

/**
 * Created by Amrish on 09-10-2020.
 */
public class DeepLinkProcessActivity extends BaseActivity {
    String url;
    Uri data;

    @Override
    protected void onCreate(@Nullable Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_deeplink_process);
        url = getIntent().getStringExtra("url");
        if (!Validators.isNullOrEmpty(url)) {
            data = Uri.parse(url);
        } else {
            data = getIntent().getData();
        }
        handleNavigation();
    }

    private void handleNavigation() {
        String host = data.getHost();
        List<String> pathSegments = data.getPathSegments();
        if (host.trim().equalsIgnoreCase("profile")) {
            if (myApp.isLoggedIn()) {
                if (pathSegments.size() > 0) {
                    String mainPath = pathSegments.get(0);
                    String subPath = null;
                    if (pathSegments.size() > 1) {
                        subPath = pathSegments.get(1);//booking_id
                    }
                    if (mainPath.equalsIgnoreCase("orders") || mainPath.equalsIgnoreCase("orderhistory")) {
                        if (!Validators.isNullOrEmpty(subPath)) {
                            Intent intent = new Intent(this, TransportActivity.class);
                            intent.putExtra("destination", "orderView");
                            intent.putExtra("hideToolbar", true);
                            intent.putExtra("order_id", subPath);
                            startActivity(intent);
                        } else {
                            startActivity(new Intent(this, TransportActivity.class).putExtra("destination", "orders").putExtra("name", "Order History").putExtra("hideToolbar", true));
                        }
                        finish();
                    }

                    if (mainPath.equalsIgnoreCase("favourites")) {
                        startActivity(new Intent(this, TransportActivity.class).putExtra("destination", "favourites").putExtra("name", "Your Favourites"));
                        finish();
                    }
                    if (mainPath.equalsIgnoreCase("rewards")) {
                        startActivity(new Intent(this, TransportActivity.class).putExtra("destination", "rewards").putExtra("name", "Rewards"));
                        finish();
                    }
                    if (mainPath.equalsIgnoreCase("tiffintom-credit") || mainPath.equalsIgnoreCase("credits") || mainPath.equalsIgnoreCase("tiffintom-credits")) {
                        startActivity(new Intent(this, TransportActivity.class).putExtra("destination", "credits").putExtra("name", "Tiffintom Credit"));
                        finish();
                    }
                    if (mainPath.equalsIgnoreCase("wallet")) {
                        startActivity(new Intent(this, TransportActivity.class).putExtra("destination", "wallet").putExtra("name", "Wallet"));
                        finish();
                    }
                    if (mainPath.equalsIgnoreCase("edit")) {
                        startActivity(new Intent(this, TransportActivity.class).putExtra("destination", "profile_edit").putExtra("name", "Profile"));
                        finish();
                    }
                    if (mainPath.equalsIgnoreCase("addresses") || mainPath.equalsIgnoreCase("address-book")) {
                        startActivity(new Intent(this, AddressesActivity.class));
                        finish();
                    }

                    if (mainPath.equalsIgnoreCase("dinein-orders") || mainPath.equalsIgnoreCase("dineinorders")) {
                        if (!Validators.isNullOrEmpty(subPath)) {
                            Intent intent = new Intent(this, TransportActivity.class);
                            intent.putExtra("destination", "orderView");
                            intent.putExtra("hideToolbar", true);
                            intent.putExtra("order_id", subPath);
                            startActivity(intent);
                        } else {
                            startActivity(new Intent(this, TransportActivity.class).putExtra("destination", "dinein_history").putExtra("name", "Dinein History").putExtra("hideToolbar", true));
                        }
                        finish();
                    }

                    if (mainPath.equalsIgnoreCase("reservations") || mainPath.equalsIgnoreCase("reservation-history")) {
                        startActivity(new Intent(this, ReservationHistoryActivity.class));
                        finish();
                    }

                    if (mainPath.equalsIgnoreCase("restaurants")) {
                        if (!Validators.isNullOrEmpty(subPath)) {
                            Intent intent = new Intent(this, RestaurantDetailsActivity.class);
                            intent.putExtra("resId", subPath);
                            startActivity(intent);
                        } else {
                            startActivity(new Intent(this, MainActivity.class));
                        }
                        finish();
                    }
                } else {
                    startActivity(new Intent(this, MainActivity.class).putExtra("destination", "profile"));
                    finish();
                }
            } else {
                startActivity(new Intent(this, AuthActivity.class));
                finish();
            }
        }
        if (host.equalsIgnoreCase("setting") || host.equalsIgnoreCase("settings")) {
            if (pathSegments.size() > 0) {
                String mainPath = pathSegments.get(0);
                String subPath = null;
                if (pathSegments.size() > 1) {
                    subPath = pathSegments.get(1);//sub topic
                }
                if (mainPath.equalsIgnoreCase("feedback")) {
                    if (myApp.isLoggedIn())
                        startActivity(new Intent(this, TransportActivity.class).putExtra("destination", "feedback").putExtra("name", "Feedback"));
                    else startActivity(new Intent(this, AuthActivity.class));
                }
                if (mainPath.equalsIgnoreCase("become-partner")) {
                    startActivity(new Intent(this, TransportActivity.class).putExtra("destination", "becomepartner").putExtra("name", "Become a Partner"));
                }
                if (mainPath.equalsIgnoreCase("notifications") || mainPath.equalsIgnoreCase("notification")) {
                    if (myApp.isLoggedIn())
                        startActivity(new Intent(this, TransportActivity.class).putExtra("destination", "notification").putExtra("name", "Notification"));
                    else startActivity(new Intent(this, AuthActivity.class));
                }
                if (mainPath.equalsIgnoreCase("contact-us") || mainPath.equalsIgnoreCase("contactus")) {
                    startActivity(new Intent(this, TransportActivity.class).putExtra("destination", "tiffintom_chat_window").putExtra("name", "Chat").putExtra("hideToolbar", true));
                }
                finish();
            } else {
                startActivity(new Intent(this, MainActivity.class).putExtra("destination", "settings"));
                finish();
            }
        }

        if (host.equalsIgnoreCase("home")) {
            if (pathSegments.size() > 0) {
                String mainPath = pathSegments.get(0);
                String subPath = null;
                if (pathSegments.size() > 1) {
                    subPath = pathSegments.get(1);//booking_id
                }
                if (mainPath.equalsIgnoreCase("restaurants")) {
                    if (!Validators.isNullOrEmpty(subPath)) {
                        Intent intent = new Intent(this, RestaurantDetailsActivity.class);
                        intent.putExtra("resId", subPath);
                        startActivity(intent);
                    } else {
                        startActivity(new Intent(this, MainActivity.class));
                    }
                    finish();
                }
            } else {
                startActivity(new Intent(this, MainActivity.class));
                finish();
            }
        }

    }
}
