package com.tiffintom.activity;

import android.animation.Animator;
import android.annotation.SuppressLint;
import android.content.Intent;
import android.os.Bundle;
import android.view.KeyEvent;
import android.view.View;
import android.view.inputmethod.EditorInfo;
import android.widget.Button;
import android.widget.EditText;
import android.widget.ImageView;
import android.widget.LinearLayout;
import android.widget.TextView;
import android.widget.Toast;

import com.airbnb.lottie.LottieAnimationView;
import com.androidnetworking.AndroidNetworking;
import com.androidnetworking.error.ANError;
import com.androidnetworking.interfaces.JSONArrayRequestListener;
import com.androidnetworking.interfaces.JSONObjectRequestListener;
import com.budiyev.android.codescanner.CodeScanner;
import com.budiyev.android.codescanner.CodeScannerView;
import com.google.gson.Gson;
import com.google.gson.JsonSyntaxException;
import com.google.gson.reflect.TypeToken;
import com.tiffintom.MyApp;
import com.tiffintom.R;
import com.tiffintom.base.BaseActivity;
import com.tiffintom.common.CommonFunctions;
import com.tiffintom.common.Validators;
import com.tiffintom.fragment.TableSelectDialogFragment;
import com.tiffintom.models.BusinessRestaurant;
import com.tiffintom.models.DineinModels.DineinPaymentMethods;
import com.tiffintom.models.DineinModels.DineinSiteSettings;
import com.tiffintom.models.DineinModels.TablesStatuses;
import com.tiffintom.models.DineinModels.UsersList;
import com.tiffintom.models.Tables;
import com.tiffintom.network.ApiEndPoints;
import com.tiffintom.utils.LogUtils;
import com.tiffintom.utils.ToastUtils;

import org.json.JSONArray;
import org.json.JSONObject;

import java.lang.reflect.Type;
import java.util.ArrayList;
import java.util.List;

public class DineInScanActivity extends BaseActivity {

    private CodeScanner codeScanner;
    private CodeScannerView scannerView;
    private EditText etCode;
    private Button btnSubmitCode;
    private ImageView ivBack;
    String codeResult, tableNumber, restQrNumber;
    private LinearLayout llBeforeScan, llAfterScan, llTableNumber;
    private EditText etTableNumber;
    private TextView tvGuestCount, tvRestaurant, tvDineInInfo, tvTableNumber;
    private ImageView ivIncrease, ivDecrease;
    private int guestCount = 1;
    private Button btnConfirm;
    private BusinessRestaurant restaurant;
    private LinearLayout llLoading;
    private LottieAnimationView animationView;

    // Dine In Declarations
    ArrayList<DineinSiteSettings> siteSettings = new ArrayList<>();
    ArrayList<DineinPaymentMethods> paymentMethods = new ArrayList<>();
    ArrayList<TablesStatuses> tablesStatuses = new ArrayList<>();
    ArrayList<UsersList> userMinis = new ArrayList<>();
    private Tables selectedTable;
    private String business_id;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_dinein);
        try {
            initViews();
            setListeners();

            new Thread(() -> {
                myApp.getAppDatabase().dineinCartItemDao().nukeTable();
                myApp.getAppDatabase().orderItemDao().nukeTable();
            }).start();
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    @Override
    protected void initViews() {
        super.initViews();
        try {
            scannerView = findViewById(R.id.scanner_view);
            etCode = findViewById(R.id.etCode);
            btnSubmitCode = findViewById(R.id.btnCodeSubmit);
            ivBack = findViewById(R.id.ivBackArrow);

            etTableNumber = findViewById(R.id.etTableNumber);
            llBeforeScan = findViewById(R.id.llBeforeScan);
            llAfterScan = findViewById(R.id.llAfterScan);
            llTableNumber = findViewById(R.id.llTableNumber);
            tvDineInInfo = findViewById(R.id.tvDineInInfo);
            tvGuestCount = findViewById(R.id.tvGuestCount);
            tvRestaurant = findViewById(R.id.tvRestaurant);
            tvTableNumber = findViewById(R.id.tvTableNumber);
            ivDecrease = findViewById(R.id.ivDecrease);
            ivIncrease = findViewById(R.id.ivIncrease);
            btnConfirm = findViewById(R.id.btnConfirm);

            codeScanner = new CodeScanner(this, scannerView);

            llLoading = findViewById(R.id.llLoading);
            animationView = findViewById(R.id.animation_view);
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    @SuppressLint("WrongThread")
    private void setListeners() {
        try {
            codeScanner.setDecodeCallback(result -> {
                try {
                    codeResult = result.getText();
                    if (result.getText().contains(",")) {
                        String[] animalsArray = result.getText().split(",");
                        if (animalsArray.length > 1) {
                            tableNumber = animalsArray[1];
                        }
                        restQrNumber = animalsArray[0];
                    } else {
                        restQrNumber = codeResult;
                    }
                    codeScanner.releaseResources();
                    fetchDineInMenuDetails(restQrNumber);
                } catch (Exception e) {
                    e.printStackTrace();
                }
            });

            btnSubmitCode.setOnClickListener(view -> {
                try {
                    codeScanner.releaseResources();
                    if (!etCode.getText().toString().isEmpty()) {
                        if (etCode.getText().toString().contains(",")) {
                            String[] animalsArray = etCode.getText().toString().split(",");
                            if (animalsArray.length > 1) {
                                tableNumber = animalsArray[1];
                            }
                            restQrNumber = animalsArray[0];
                        } else {
                            restQrNumber = etCode.getText().toString();
                        }
                        fetchDineInMenuDetails(restQrNumber);
                    } else {
                        Toast.makeText(this, "Please enter restaurant code", Toast.LENGTH_SHORT).show();
                    }
                } catch (Exception e) {
                    e.printStackTrace();
                }
            });
            ivBack.setOnClickListener(view -> onBackPressed());
            ivIncrease.setOnClickListener(view -> {
                guestCount++;
                tvGuestCount.setText(String.valueOf(guestCount));
            });
            ivDecrease.setOnClickListener(view -> {
                if (guestCount > 1) {
                    guestCount--;
                    tvGuestCount.setText(String.valueOf(guestCount));
                }
            });
            btnConfirm.setOnClickListener(view -> {
                try {
                    new Thread(() -> {
                        myApp.getAppDatabase().dineinCartItemDao().nukeTable();
                        myApp.getAppDatabase().orderItemDao().nukeTable();
                    }).start();
                    if (etTableNumber.getText().toString().isEmpty()) {
                        ToastUtils.makeToast(this, "Please enter table number");
                    } else {
                        //startActivity(new Intent(this, RestaurantDetailsActivity.class).putExtra("is_dinein", true).putExtra("qr_code", restQrNumber).putExtra("resId", restaurant.id).putExtra("res_name", restaurant.restaurant_name).putExtra("guests", guestCount).putExtra("table_number", etTableNumber.getText().toString()));
                        //finish();
                        startActivity(new Intent(this, TransportActivity.class).putExtra("hideToolbar", true).putExtra("destination", "dinein_menu").putExtra("is_dinein", true).putExtra("qr_code", restQrNumber).putExtra("resId", restaurant.id).putExtra("res_name", restaurant.restaurant_name).putExtra("guests", guestCount).putExtra("table_id", selectedTable.id).putExtra("table_number", etTableNumber.getText().toString()));
                        finish();
                    }
                } catch (Exception e) {
                    e.printStackTrace();
                }
            });
            animationView.addAnimatorListener(new Animator.AnimatorListener() {
                @Override
                public void onAnimationStart(Animator animator) {

                }

                @Override
                public void onAnimationEnd(Animator animator) {
                    llLoading.setVisibility(View.GONE);
                    codeScanner.startPreview();
                }

                @Override
                public void onAnimationCancel(Animator animator) {

                }

                @Override
                public void onAnimationRepeat(Animator animator) {

                }
            });
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    private void fetchDineInMenuDetails(String qrCode) {
        try {
            String businessId = "";
            if (myApp.getMyPreferences().getCurrentDineInRestaurantDetail() != null&&myApp.getMyPreferences().getCurrentDineInRestaurantDetail().business!=null) {
                businessId = myApp.getMyPreferences().getCurrentDineInRestaurantDetail().business.id;
            }
            runOnUiThread(() -> progressDialog.show());
            AndroidNetworking.get(ApiEndPoints.NEW_DINE_IN_Restaurants_DETAILS)
                    .addQueryParameter("customer_id", String.valueOf(myApp.getMyPreferences().getLoggedInUser().id))
                    .addQueryParameter("qr_code", qrCode)
                    .addQueryParameter("table_id", (!Validators.isNullOrEmpty(tableNumber) ? tableNumber : ""))
                    .addHeaders("business-id", businessId)
                    .build()
                    .getAsJSONObject(new JSONObjectRequestListener() {
                        @Override
                        public void onResponse(JSONObject response) {
                            try {
                                runOnUiThread(() -> progressDialog.dismiss());
                                restaurant = new Gson().fromJson(response.toString(), BusinessRestaurant.class);
                                myApp.getMyPreferences().saveCurrentDineinRestaurant(restaurant);
                                business_id = restaurant.business.id;
                            /*if (!String.valueOf(restaurant.id).equalsIgnoreCase(MyApp.RES)) {
                                ToastUtils.makeToast(DineinScanActivity.this, "It seems the QR code you are scanning is not for " + getString(R.string.app_name) + ". Please try again with correct QR code.");
                            } else {*/
                                restQrNumber = qrCode;
                                if (restaurant.table != null) {
                                    if (!restaurant.table.table_status.status.equalsIgnoreCase("Vacant")) {
                                        ToastUtils.makeToast(DineInScanActivity.this, "Sorry!! It seems selected table is not vacant");
                                    } else {
                                        updateUi();
                                    }
                                } else {
                                    fetchSiteSettings();
                                    if (selectedTable == null) {
                                        openTablesDialog();
                                    }

                                }
                            } catch (JsonSyntaxException e) {
                                e.printStackTrace();
                            }
                            // }
                        }

                        @Override
                        public void onError(ANError anError) {
                            try {
                                anError.printStackTrace();
                                runOnUiThread(() -> progressDialog.dismiss());

                                if (!CommonFunctions.isConnected(DineInScanActivity.this))
                                    myApp.noInternet(DineInScanActivity.this);
                            } catch (Exception e) {
                                e.printStackTrace();
                            }
                        }
                    });
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    private void updateUi() {
        try {
            if (restaurant != null) {
                llAfterScan.setVisibility(View.VISIBLE);
                llBeforeScan.setVisibility(View.GONE);
                tvRestaurant.setText(restaurant.restaurant_name);
                tvDineInInfo.setText("Would you like to dine in at " + restaurant.restaurant_name);
                if (!Validators.isNullOrEmpty(tableNumber)) {
                    llTableNumber.setVisibility(View.VISIBLE);
                    etTableNumber.setText(tableNumber);
                    etTableNumber.setVisibility(View.GONE);
                    tvTableNumber.setText("Table no." + tableNumber);
                } else {
                    llTableNumber.setVisibility(View.GONE);
                    etTableNumber.setVisibility(View.VISIBLE);
                }
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    // dine in code start
    private void openTablesDialog() {
        try {
            TableSelectDialogFragment tableSelectDialogFragment = TableSelectDialogFragment.getInstance();
            tableSelectDialogFragment.show(getSupportFragmentManager(), "table_list");
            tableSelectDialogFragment.setCancelable(false);
            tableSelectDialogFragment.setDialogDismissListener(o -> {
                try {
                    //table data after selection
                    selectedTable = (Tables) o;
                    tableNumber = selectedTable.number;
                    updateUi();
                } catch (Exception e) {
                    e.printStackTrace();
                }
            });
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    private void fetchSiteSettings() {
        try {
            AndroidNetworking.get(ApiEndPoints.dinein_site_settings)
                    .addQueryParameter("nopaginate", "1")
                    .addHeaders("business-id", business_id)
                    .build()
                    .getAsJSONArray(new JSONArrayRequestListener() {
                        @Override
                        public void onResponse(JSONArray response) {
                            try {
                                runOnUiThread(() -> progressDialog.dismiss());
                                Type type = new TypeToken<List<DineinSiteSettings>>() {
                                }.getType();
                                siteSettings.addAll(new Gson().fromJson(response.toString(), type));
                                myApp.getMyPreferences().saveDineinSiteSettings(siteSettings);
                                myApp.dineinSiteSettings.clear();
                                myApp.dineinSiteSettings.addAll(siteSettings);
                                fetchPayments();
                                LogUtils.e("DINE IN SITE SETTINGS");
                            } catch (JsonSyntaxException e) {
                                e.printStackTrace();
                            }
                        }

                        @Override
                        public void onError(ANError anError) {
                            try {
                                runOnUiThread(() -> progressDialog.dismiss());
                                anError.printStackTrace();
                                LogUtils.e("DINEIN SITE SETTING ERROR::" + anError.getErrorBody());
                                if (!CommonFunctions.isConnected(DineInScanActivity.this)) {
                                    myApp.noInternet(DineInScanActivity.this);
                                }
                            } catch (Exception e) {
                                e.printStackTrace();
                            }
                        }
                    });
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    private void fetchPayments() {
        try {
            AndroidNetworking.get(ApiEndPoints.dinein_payment_methods)
                    .addQueryParameter("nopaginate", "1")
                    .addHeaders("business-id", business_id)
                    .build()
                    .getAsJSONArray(new JSONArrayRequestListener() {
                        @Override
                        public void onResponse(JSONArray response) {
                            try {
                                Type type = new TypeToken<List<DineinPaymentMethods>>() {
                                }.getType();
                                paymentMethods.addAll(new Gson().fromJson(response.toString(), type));
                                myApp.getMyPreferences().saveDineinPaymentMethods(paymentMethods);
                                myApp.dineinPaymentMethods.clear();
                                myApp.dineinPaymentMethods.addAll(paymentMethods);
                                LogUtils.e("DINE IN PAYMENTS");
                                fetchTableStatuses();
                            } catch (JsonSyntaxException e) {
                                e.printStackTrace();
                            }
                        }

                        @Override
                        public void onError(ANError anError) {
                            try {
                                runOnUiThread(() -> progressDialog.dismiss());
                                anError.printStackTrace();
                                LogUtils.e("DINEIN PAYMENT METHOD ERROR::" + anError.getErrorBody());
                                if (!CommonFunctions.isConnected(DineInScanActivity.this)) {
                                    myApp.noInternet(DineInScanActivity.this);
                                }
                            } catch (Exception e) {
                                e.printStackTrace();
                            }
                        }
                    });
        } catch (Exception e) {
            e.printStackTrace();
        }

    }

    private void fetchTableStatuses() {
        try {
            AndroidNetworking.get(ApiEndPoints.dinein_table_statuses)
                    .addQueryParameter("nopaginate", "1")
                    .addHeaders("business-id", business_id)
                    .build()
                    .getAsJSONArray(new JSONArrayRequestListener() {
                        @Override
                        public void onResponse(JSONArray response) {
                            try {
                                Type type = new TypeToken<List<TablesStatuses>>() {
                                }.getType();
                                tablesStatuses.addAll(new Gson().fromJson(response.toString(), type));
                                myApp.getMyPreferences().saveDineinTablesStatus(tablesStatuses);
                                LogUtils.e("DINE IN TABLES");
                                fetchUserMini();
                            } catch (JsonSyntaxException e) {
                                e.printStackTrace();
                            }
                        }

                        @Override
                        public void onError(ANError anError) {
                            try {
                                anError.printStackTrace();
                                LogUtils.e("DINEIN TABLES ERROR::" + anError.getErrorBody());
                                if (!CommonFunctions.isConnected(DineInScanActivity.this)) {
                                    myApp.noInternet(DineInScanActivity.this);
                                }
                            } catch (Exception e) {
                                e.printStackTrace();
                            }
                        }
                    });
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    private void fetchUserMini() {
        try {
            AndroidNetworking.get(ApiEndPoints.dinein_user_mini)
                    .addQueryParameter("nopaginate", "1")
                    .addHeaders("business-id", business_id)
                    .build()
                    .getAsJSONArray(new JSONArrayRequestListener() {
                        @Override
                        public void onResponse(JSONArray response) {
                            try {
                                Type type = new TypeToken<List<UsersList>>() {
                                }.getType();
                                userMinis.addAll(new Gson().fromJson(response.toString(), type));
                                myApp.getMyPreferences().saveDineinUser(userMinis.get(0));
                                LogUtils.e("USER MINI");
                            } catch (JsonSyntaxException e) {
                                e.printStackTrace();
                            }
                        }

                        @Override
                        public void onError(ANError anError) {
                            try {
                                anError.printStackTrace();
                                LogUtils.e("DINEIN USERS ERROR::" + anError.getErrorBody());
                                if (!CommonFunctions.isConnected(DineInScanActivity.this)) {
                                    myApp.noInternet(DineInScanActivity.this);
                                }
                            } catch (Exception e) {
                                e.printStackTrace();
                            }
                        }
                    });
        } catch (Exception e) {
            e.printStackTrace();
        }
    }
    // dine code ends

    @Override
    protected void onResume() {
        super.onResume();
        try {
            codeScanner.startPreview();
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    @Override
    protected void onPause() {
        try {
            codeScanner.releaseResources();
        } catch (Exception e) {
            e.printStackTrace();
        }
        super.onPause();
    }
}