package com.tiffintom.activity;

import android.app.ActivityManager;
import android.content.BroadcastReceiver;
import android.content.Context;
import android.content.Intent;
import android.content.IntentFilter;
import android.os.Bundle;
import android.util.Log;
import android.view.LayoutInflater;
import android.view.MenuItem;
import android.view.View;
import android.widget.TextView;

import androidx.annotation.Nullable;
import androidx.cardview.widget.CardView;
import androidx.fragment.app.Fragment;
import androidx.fragment.app.FragmentManager;
import androidx.fragment.app.FragmentTransaction;
import androidx.localbroadcastmanager.content.LocalBroadcastManager;

import com.androidnetworking.AndroidNetworking;
import com.androidnetworking.common.ANRequest;
import com.androidnetworking.error.ANError;
import com.androidnetworking.interfaces.JSONObjectRequestListener;
import com.google.android.material.bottomnavigation.BottomNavigationItemView;
import com.google.android.material.bottomnavigation.BottomNavigationMenuView;
import com.google.android.material.bottomnavigation.BottomNavigationView;
import com.google.gson.Gson;
import com.tiffintom.R;
import com.tiffintom.base.BaseActivity;
import com.tiffintom.common.Validators;
import com.tiffintom.fragment.CartCheckoutFragment;
import com.tiffintom.fragment.ExploreFragment;
import com.tiffintom.fragment.HomeFragment;
import com.tiffintom.fragment.ProfileFragment;
import com.tiffintom.fragment.SettingsFragment;
import com.tiffintom.fragment.TrackOrderFragment;
import com.tiffintom.models.CartSummary;
import com.tiffintom.models.DineinModels.DineinSiteSettings;
import com.tiffintom.models.User;
import com.tiffintom.network.ApiEndPoints;
import com.tiffintom.notification.NotificationService;
import com.tiffintom.utils.Constants;
import com.tiffintom.utils.LogUtils;
import com.tiffintom.utils.ToastUtils;

import static com.tiffintom.utils.Constants.CART_UPDATE;
import static com.tiffintom.utils.Constants.CHAT_COUNT_UPDATE;
import static com.tiffintom.utils.Constants.CODE_REFRESH;
import static com.tiffintom.utils.Constants.TOAST_UPDATE;

import org.json.JSONObject;

import java.util.HashMap;

/**
 * Created by Amrish on 18-08-2020.
 */
public class MainActivity extends BaseActivity {
    private BottomNavigationView bottomNavigationView;
    public static TextView tvCartItemsCountBadge;
    public TextView tvSettingNotification_Badge;
    public static int totalMessageCount = 0;
    private String destination;
    private String orderId;
    private BroadcastReceiver broadcastReceiver, chatCountBroadCastReceiver, toastReceiver;
    CardView llToastLayout;
    TextView tvToastMessage;
    private User loggedInUser = myApp.getMyPreferences().getLoggedInUser();
    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_main);
        destination = getIntent().getStringExtra("destination");
        orderId = getIntent().getStringExtra("order_id");

        initViews();
        setListeners();
        if (!Validators.isNullOrEmpty(destination) && destination.equalsIgnoreCase("profile")) {
            bottomNavigationView.setSelectedItemId(R.id.btmProfile);
        } else if (!Validators.isNullOrEmpty(destination) && destination.equalsIgnoreCase("settings")) {
            bottomNavigationView.setSelectedItemId(R.id.btmSettings);
        } else if (!Validators.isNullOrEmpty(destination) && destination.equalsIgnoreCase("cart")) {
            bottomNavigationView.setSelectedItemId(R.id.btmCart);
        } else if (!Validators.isNullOrEmpty(destination) && destination.equalsIgnoreCase("explore")) {
            bottomNavigationView.setSelectedItemId(R.id.btmExplore);
        } else if (!Validators.isNullOrEmpty(destination) && destination.equalsIgnoreCase("trackOrder")) {
            bottomNavigationView.setSelectedItemId(R.id.btmProfile);
            startActivity(new Intent(MainActivity.this, TransportActivity.class).putExtra("destination", "trackOrder").putExtra("order_id", orderId).putExtra("hideToolbar", true).putExtra("from", "cart").putExtra("OrderPlace", true));
        }  else if (!Validators.isNullOrEmpty(destination) && destination.equalsIgnoreCase("orderView")) {
            bottomNavigationView.setSelectedItemId(R.id.btmProfile);
            startActivity(new Intent(getApplicationContext(), TransportActivity.class).putExtra("destination", "orderView").putExtra("order_id", orderId).putExtra("is_dinein", false).putExtra("hideToolbar", true).putExtra("OrderPlace", true));
        }  else if (!Validators.isNullOrEmpty(destination) && destination.equalsIgnoreCase("reservation_history")) {
            bottomNavigationView.setSelectedItemId(R.id.btmProfile);
            startActivity(new Intent(this, ReservationHistoryActivity.class));
        } else if (!Validators.isNullOrEmpty(destination) && destination.equalsIgnoreCase("restaurant_chat")) {
            bottomNavigationView.setSelectedItemId(R.id.btmSettings);
            startActivity(new Intent(MainActivity.this, TransportActivity.class).putExtra("destination", "restaurant_chat").putExtra("hideToolbar", true));//Check perameter from api
        } else if (!Validators.isNullOrEmpty(destination) && destination.equalsIgnoreCase("tiffintom_chat_window")) {
            bottomNavigationView.setSelectedItemId(R.id.btmSettings);
            startActivity(new Intent(MainActivity.this, TransportActivity.class).putExtra("destination", "tiffintom_chat_window").putExtra("hideToolbar", true));//Check perameter from api
        }  else {
            bottomNavigationView.setSelectedItemId(R.id.btmHome);
        }
//        initService();
        updateChatCount(myApp.chatCount);
        try {
            DineinSiteSettings banquetDay = myApp.findSetting("banquet_day");
            DineinSiteSettings banquetOrderType = myApp.findSetting("banquet_type");
            if (banquetDay != null && !Validators.isNullOrEmpty(banquetDay.value) && banquetDay.value.contains(myApp.current_day.toLowerCase())) {
                myApp.banquetDay = myApp.current_day;
                if (banquetOrderType != null && !Validators.isNullOrEmpty(banquetOrderType.value)) {
                    myApp.banquetOrderTypes = banquetOrderType.value;
                }
                myApp.showBanquetPorducts = true;
            } else {
                myApp.banquetDay = null;
                myApp.showBanquetPorducts = false;
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
    }


    @Override
    protected void initViews() {
        super.initViews();
        bottomNavigationView = findViewById(R.id.bottmNavigationView);
        bottomNavigationView.getMenu().findItem(R.id.btmHome).setChecked(true);
        bottomNavigationView.setItemIconTintList(null);
        llToastLayout = findViewById(R.id.llToastMessage);
        tvToastMessage = findViewById(R.id.tvToastMessage);

        BottomNavigationMenuView bottomNavigationMenuView =
                (BottomNavigationMenuView) bottomNavigationView.getChildAt(0);

        View settingView = bottomNavigationMenuView.getChildAt(3);
        BottomNavigationItemView settingItemView = (BottomNavigationItemView) settingView;
        View settingBadge = LayoutInflater.from(this)
                .inflate(R.layout.bottomnavigation_notification_badge, bottomNavigationMenuView, false);
        tvSettingNotification_Badge = settingBadge.findViewById(R.id.tvNotification_Badge);
        tvSettingNotification_Badge.setVisibility(View.GONE);
        settingItemView.addView(settingBadge);

        View cartView = bottomNavigationMenuView.getChildAt(4);
        BottomNavigationItemView cartItemView = (BottomNavigationItemView) cartView;
        View badge = LayoutInflater.from(this)
                .inflate(R.layout.bottomnavigation_notification_badge, bottomNavigationMenuView, false);
        tvCartItemsCountBadge = badge.findViewById(R.id.tvNotification_Badge);
        tvCartItemsCountBadge.setVisibility(View.GONE);
        cartItemView.addView(badge);
        if (loggedInUser!=null)  updateFcmToken();

    }


    private void setListeners() {
        bottomNavigationView.setOnNavigationItemSelectedListener(this::manageFragments);
        broadcastReceiver = new BroadcastReceiver() {
            @Override
            public void onReceive(Context context, Intent intent) {
                boolean updateForCart = intent.getBooleanExtra("update_cart_count", false);
                if (updateForCart) updateCartCount();
            }
        };
        chatCountBroadCastReceiver = new BroadcastReceiver() {
            @Override
            public void onReceive(Context context, Intent intent) {
                int count = intent.getIntExtra("count", 0);
                updateChatCount(count);
            }
        };
        toastReceiver = new BroadcastReceiver() {
            @Override
            public void onReceive(Context context, Intent intent) {
                runOnUiThread(() -> {
                    if (intent != null && intent.getExtras() != null) {
                        String message = intent.getStringExtra("message");
                        if (!Validators.isNullOrEmpty(message)) {
                            llToastLayout.setVisibility(View.VISIBLE);
                            tvToastMessage.setText(message);
                        } else {
                            llToastLayout.setVisibility(View.GONE);
                        }
                    } else {
                        llToastLayout.setVisibility(View.GONE);
                    }
                    llToastLayout.postDelayed(() -> {
                        llToastLayout.setVisibility(View.GONE);
                    }, 2500);
                });
            }
        };
    }

    private void updateChatCount(int count) {
        runOnUiThread(() -> {
            tvSettingNotification_Badge.setText(String.valueOf(count));
            if (count > 0) {
                tvSettingNotification_Badge.setVisibility(View.VISIBLE);
            } else {
                tvSettingNotification_Badge.setVisibility(View.GONE);
            }
        });
    }

    private void updateCartCount() {
        new Thread(() -> {
            CartSummary cartSummary = myApp.getAppDatabase().cartDao().getCartSummary();
            runOnUiThread(() -> {
                if (cartSummary != null && cartSummary.items > 0) {
                    tvCartItemsCountBadge.setVisibility(View.VISIBLE);
                    tvCartItemsCountBadge.setText(String.valueOf(cartSummary.items));
                } else {
                    tvCartItemsCountBadge.setVisibility(View.GONE);
                    tvCartItemsCountBadge.setText("0");
                }
            });
        }).start();
    }

    private boolean manageFragments(MenuItem item) {
        if (item.getItemId() == R.id.btmHome) {
            changeFragment(HomeFragment.getInstance());
            return true;
        }
        if (item.getItemId() == R.id.btmExplore) {
            changeFragment(ExploreFragment.getInstance());
            return true;
        }
        if (item.getItemId() == R.id.btmProfile) {
            if (myApp.getMyPreferences().getLoggedInUser() == null) {
                openLoginScreen();
                return false;
            } else {
                changeFragment(ProfileFragment.getInstance());
                return true;
            }
        }
        if (item.getItemId() == R.id.btmCart) {
            Fragment openedFragment = getSupportFragmentManager().findFragmentById(R.id.flHome);
            //TODO
            // && !(openedFragment instanceof DineInCartCheckoutFragment)
            if (!(openedFragment instanceof CartCheckoutFragment)) {
                if (myApp.getMyPreferences().getLoggedInUser() == null) {
                    openLoginScreen();
                    return false;
                } else {
                    changeFragment(CartCheckoutFragment.getInstance());
                    return true;
                }
            }
        }
        if (item.getItemId() == R.id.btmSettings) {
            changeFragment(SettingsFragment.getInstance());
            return true;
        }
        return false;
    }

    private void openLoginScreen() {
        startActivityForResult(new Intent(this, AuthActivity.class), CODE_REFRESH);
    }

    private void changeFragment(Fragment fragment) {
        FragmentManager manager = getSupportFragmentManager();
        FragmentTransaction transaction = manager.beginTransaction();
        transaction.replace(R.id.flHome, fragment);
        transaction.commit();
    }

    @Override
    public void onBackPressed() {
        Fragment openedFragment = getSupportFragmentManager().findFragmentById(R.id.flHome);
       if (openedFragment == null || openedFragment instanceof HomeFragment) {
                finish();
//            super.onBackPressed();
        } else {
            bottomNavigationView.getMenu().findItem(R.id.btmHome).setChecked(true);
            changeFragment(HomeFragment.getInstance());
        }
    }

    Intent mServiceIntent;

    private void initService() {
        NotificationService mSensorService = new NotificationService(this);
        mServiceIntent = new Intent(this, mSensorService.getClass());
        if (!isMyServiceRunning(mSensorService.getClass())) {
            startService(mServiceIntent);
        }
    }

    private boolean isMyServiceRunning(Class<?> serviceClass) {
        ActivityManager manager = (ActivityManager) getSystemService(Context.ACTIVITY_SERVICE);
        for (ActivityManager.RunningServiceInfo service : manager.getRunningServices(Integer.MAX_VALUE)) {
            if (serviceClass.getName().equals(service.service.getClassName())) {
                Log.i("isMyServiceRunning?", true + "");
                return true;
            }
        }
        Log.i("isMyServiceRunning?", false + "");
        return false;
    }

    @Override
    protected void onResume() {
        updateCartCount();
        LocalBroadcastManager.getInstance(this).registerReceiver(broadcastReceiver, new IntentFilter(CART_UPDATE));
        LocalBroadcastManager.getInstance(this).registerReceiver(chatCountBroadCastReceiver, new IntentFilter(CHAT_COUNT_UPDATE));
        LocalBroadcastManager.getInstance(this).registerReceiver(toastReceiver, new IntentFilter(TOAST_UPDATE));
        super.onResume();
    }

    @Override
    public void onStop() {
        LocalBroadcastManager.getInstance(this).unregisterReceiver(broadcastReceiver);
        LocalBroadcastManager.getInstance(this).unregisterReceiver(toastReceiver);
        LocalBroadcastManager.getInstance(this).unregisterReceiver(chatCountBroadCastReceiver);
        super.onStop();
    }

    @Override
    protected void onDestroy() {
        super.onDestroy();
    }

    @Override
    protected void onActivityResult(int requestCode, int resultCode, @Nullable Intent data) {
        super.onActivityResult(requestCode, resultCode, data);
        if (requestCode == Constants.CODE_MESSAGE && resultCode == RESULT_OK) {
            LocalBroadcastManager.getInstance(this).registerReceiver(toastReceiver, new IntentFilter(TOAST_UPDATE));
            if (data != null) {
                setResult(RESULT_OK, data);
                String message = data.getStringExtra("message");
                if (!Validators.isNullOrEmpty(message)) {
                    ToastUtils.makeSnackToast(this, message);
                }

            }
        }
    }
    private HashMap<String, String> gernrateParams() {
        HashMap<String, String> params = new HashMap<>();
        params.put("id", String.valueOf(loggedInUser.id));
        params.put("first_name", loggedInUser.first_name);
        params.put("last_name", loggedInUser.last_name);
        params.put("phone_number", loggedInUser.phone_number);
        params.put("wallet_amount", String.valueOf(loggedInUser.wallet_amount));
        params.put("device_type", "Android");
        params.put("device_id", myApp.getMyPreferences().getUserFCMToken());
        params.put("order_place", loggedInUser.order_place);
        params.put("order_accept", loggedInUser.order_accept);
        params.put("order_reject", loggedInUser.order_reject);
        params.put("order_delivered", loggedInUser.order_delivered);
        params.put("book_table", loggedInUser.book_table);
        params.put("status", loggedInUser.status);
        params.put("important_update", loggedInUser.important_update);
        params.put("email_verify", loggedInUser.email_verify);
        params.put("phone_verify", loggedInUser.phone_verify);
        return params;
    }

    private void updateFcmToken() {
        ANRequest.MultiPartBuilder androidNetworking = AndroidNetworking.upload(ApiEndPoints.UPDATE_ACCOUNT + loggedInUser.id)
                .addMultipartParameter(gernrateParams());
        androidNetworking.addMultipartFile("image", null);
        androidNetworking.addMultipartFile("cover_image", null);

        androidNetworking
                .build()
                .getAsJSONObject(new JSONObjectRequestListener() {
                    @Override
                    public void onResponse(JSONObject response) {
//                        LogUtils.e("updatetoken responce::" + new Gson().toJson(response));
                    }

                    @Override
                    public void onError(ANError anError) {
                        anError.printStackTrace();

                    }
                });

    }
}
