package com.tiffintom.activity;

import android.content.Intent;
import android.os.Bundle;
import android.util.TypedValue;
import android.view.LayoutInflater;
import android.view.View;
import android.widget.ImageView;
import android.widget.LinearLayout;
import android.widget.RelativeLayout;
import android.widget.TextView;

import androidx.annotation.Nullable;
import androidx.appcompat.widget.Toolbar;
import androidx.fragment.app.FragmentPagerAdapter;
import androidx.viewpager.widget.ViewPager;

import com.google.android.material.appbar.AppBarLayout;
import com.google.android.material.tabs.TabLayout;
import com.tiffintom.R;
import com.tiffintom.adapters.ViewpagerAdapter;
import com.tiffintom.base.BaseActivity;
import com.tiffintom.fragment.ReservationsPreviousFragment;
import com.tiffintom.fragment.ReservationsUpcomingFragment;

/**
 * Created by Amrish on 22-08-2020.
 */
public class ReservationHistoryActivity extends BaseActivity {
    private AppBarLayout appBarLayout;
    private Toolbar toolbar;
    private TabLayout tabLayout;
    private ViewPager vpReservations;
    private ImageView ivBack;
    private LinearLayout llBackArrow;
    private ViewpagerAdapter viewpagerAdapter;
    private TextView tvScreenTitle;

    @Override
    protected void onCreate(@Nullable Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_reservation_history);

        initViews();
        updateUserName();
        setListeners();
        initTabs();
        tvScreenTitle.setText("Reservation History");
        RelativeLayout.LayoutParams params = new RelativeLayout.LayoutParams(
                RelativeLayout.LayoutParams.WRAP_CONTENT,
                RelativeLayout.LayoutParams.WRAP_CONTENT
        );
        params.setMargins(0, 0, (int) TypedValue.applyDimension(TypedValue.COMPLEX_UNIT_DIP, 4, getResources().getDisplayMetrics()), 0);
        tvScreenTitle.setVisibility(View.VISIBLE);
    }

    @Override
    protected void initViews() {
        super.initViews();
        tvUserName = findViewById(R.id.tvUserName);
        appBarLayout = findViewById(R.id.appbarLayout);
        toolbar = findViewById(R.id.toolbar);
        tabLayout = findViewById(R.id.tabLayout);
        vpReservations = findViewById(R.id.vpReservations);
        ivBack = findViewById(R.id.ivBackArrow);
        llBackArrow = findViewById(R.id.llBackArrow);
        tvScreenTitle = findViewById(R.id.tvScreenTitle);
        if (getSupportActionBar() == null) {
            setSupportActionBar(toolbar);
        }
    }

    private void setListeners() {
        llBackArrow.setOnClickListener(view -> {
            onBackPressed();
        });
    }

    private void initTabs() {
        viewpagerAdapter = new ViewpagerAdapter(getSupportFragmentManager(), FragmentPagerAdapter.BEHAVIOR_RESUME_ONLY_CURRENT_FRAGMENT);
        viewpagerAdapter.addFragment(ReservationsUpcomingFragment.getInstance("upcoming"), "Upcoming");
        viewpagerAdapter.addFragment(ReservationsPreviousFragment.getInstance("previous"), "Previous");
        vpReservations.setAdapter(viewpagerAdapter);
        tabLayout.setupWithViewPager(vpReservations);
        setTabIcons();
    }

    public String[] tabTitles = new String[]{"Upcoming", "Previous"};
    public int[] tabIcons = new int[]{R.drawable.tab_icon_upcoming, R.drawable.tab_icon_previous};

    private void setTabIcons() {

        for (int i = 0; i < tabLayout.getTabCount(); i++) {
            TabLayout.Tab tab = tabLayout.getTabAt(i);
            View v = LayoutInflater.from(this).inflate(R.layout.item_tab_with_icon, null);
            TextView tv = v.findViewById(R.id.tvTabTitle);
            ImageView icon = v.findViewById(R.id.ivTabLogo);
            tv.setText(tabTitles[i]);
            icon.setImageResource(tabIcons[i]);
            tab.setCustomView(v);
            LinearLayout layout = ((LinearLayout) ((LinearLayout) tabLayout.getChildAt(0)).getChildAt(i));
            LinearLayout.LayoutParams layoutParams = (LinearLayout.LayoutParams) layout.getLayoutParams();
            layoutParams.weight = 1f; // e.g. 0.5f
            layoutParams.width = LinearLayout.LayoutParams.WRAP_CONTENT;
            layout.setLayoutParams(layoutParams);
        }
    }

    @Override
    public void onBackPressed() {
        if (!isTaskRoot()) {
            super.onBackPressed();
        } else {
            finish();
            Intent i = new Intent(this, MainActivity.class);
            startActivity(i);
        }
    }
}
