package com.tiffintom.activity;

import android.content.BroadcastReceiver;
import android.content.Context;
import android.content.Intent;
import android.content.IntentFilter;
import android.content.res.Resources;
import android.os.Bundle;
import android.view.LayoutInflater;
import android.view.View;
import android.widget.ImageView;
import android.widget.LinearLayout;
import android.widget.TextView;

import androidx.annotation.Nullable;
import androidx.appcompat.widget.Toolbar;
import androidx.cardview.widget.CardView;
import androidx.core.content.res.ResourcesCompat;
import androidx.fragment.app.FragmentPagerAdapter;
import androidx.localbroadcastmanager.content.LocalBroadcastManager;
import androidx.viewpager.widget.ViewPager;

import com.airbnb.lottie.LottieAnimationView;
import com.androidnetworking.AndroidNetworking;
import com.androidnetworking.common.ANRequest;
import com.androidnetworking.error.ANError;
import com.androidnetworking.interfaces.JSONArrayRequestListener;
import com.androidnetworking.interfaces.JSONObjectRequestListener;
import com.google.android.material.appbar.AppBarLayout;
import com.google.android.material.tabs.TabLayout;
import com.google.gson.Gson;
import com.google.gson.JsonSyntaxException;
import com.google.gson.reflect.TypeToken;
import com.tiffintom.BuildConfig;
import com.tiffintom.MyApp;
import com.tiffintom.R;
import com.tiffintom.adapters.ViewpagerAdapter;
import com.tiffintom.base.BaseActivity;
import com.tiffintom.common.CommonFunctions;
import com.tiffintom.common.Validators;
import com.tiffintom.fragment.RestaurantInfoFragment;
import com.tiffintom.fragment.RestaurantMenuFragment;
import com.tiffintom.fragment.RestaurantOffersFragment;
import com.tiffintom.fragment.RestaurantReviewsFragment;
import com.tiffintom.models.Category;
import com.tiffintom.models.DayOffer;
import com.tiffintom.models.DayPriceOffer;
import com.tiffintom.models.DayProductOffer;
import com.tiffintom.models.Menu;
import com.tiffintom.models.MenuItem;
import com.tiffintom.models.MenuItemNew;
import com.tiffintom.models.MultiplePriceOffer;
import com.tiffintom.models.Offer;
import com.tiffintom.models.Postcode;
import com.tiffintom.models.PriceOffer;
import com.tiffintom.models.ProductOffer;
import com.tiffintom.models.Restaurant;
import com.tiffintom.models.Review;
import com.tiffintom.models.TimeSlots;
import com.tiffintom.models.User;
import com.tiffintom.models.Voucher;
import com.tiffintom.network.ApiEndPoints;
import com.tiffintom.network.ApiUtils;
import com.tiffintom.utils.Constants;
import com.tiffintom.utils.LogUtils;
import com.tiffintom.utils.ToastUtils;

import org.json.JSONArray;
import org.json.JSONException;
import org.json.JSONObject;

import static com.tiffintom.utils.Constants.CODE_NO_INTERNET;
import static com.tiffintom.utils.Constants.CODE_REFRESH;
import static com.tiffintom.utils.Constants.TOAST_UPDATE;

import java.lang.reflect.Type;
import java.util.ArrayList;
import java.util.List;

/**
 * Created by Amrish on 22-08-2020.
 */
public class RestaurantDetailsActivity extends BaseActivity {
    private AppBarLayout appBarLayout;
    private Postcode currentPostcode;
    private static Toolbar toolbar;
    public static TabLayout tabLayout;
    private ViewPager vpRestaurant;
    private ImageView ivBack;
    private LinearLayout llBackArrow;
    private ViewpagerAdapter viewpagerAdapter;
    public static Restaurant openedRestaurant;
    private int res_id;
    private LinearLayout llLoading;
    public static boolean isDineIn;
    public static String qrCode, tableNumber;
    public static int guest_count;
    public static TextView tvScreenTitle, tvUserNameDetails;
    public static ImageView ivSearch, ivTiffin, ivShare;
    public int[] tabIcons = new int[]{
            R.drawable.tab_icon_menu, R.drawable.tab_icon_info, R.drawable.tab_icon_reviews, R.drawable.tab_icon_offers
    };
    public String[] tabTitles = new String[]{"Menu", "Information", "Reviews", "Offers"};
    private LottieAnimationView loadingAnimation;
    private TextView tvMessage, tvToastMessage;
    private String restaurantName;
    private String shareMessage;
    public static AppBarLayout id_appbar;
    private CardView llToastMessage;
    private BroadcastReceiver toastReceiver;

    private User loggedInUser = myApp.getMyPreferences().getLoggedInUser();
    @Override
    protected void onCreate(@Nullable Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_restaurant_details);
        try {
            if (getIntent() != null) {

                openedRestaurant = new Gson().fromJson(getIntent().getStringExtra("restaurant"), Restaurant.class);
                isDineIn = getIntent().getBooleanExtra("is_dinein", false);
                res_id = getIntent().getIntExtra("resId", 0);
                restaurantName = getIntent().getStringExtra("res_name");
                if (isDineIn) {
                    qrCode = getIntent().getStringExtra("qr_code");
                    tableNumber = getIntent().getStringExtra("table_number");
                    guest_count = getIntent().getIntExtra("guests", 1);
                } else {
                    if (openedRestaurant == null && res_id == 0)
                    {
                        openedRestaurant = MyApp.openedRestaurant;
                    }
                }
            }
            checkAndEmptyCart();
            initViews();
            updateUserName();
            setListeners();
            if (openedRestaurant != null)
                initTabs();
            else fetchRestaurantDetails();
        } catch (JsonSyntaxException e) {
            e.printStackTrace();
        }
    }

    private void checkAndEmptyCart() {
        try {
            new Thread(() -> {
                if (isDineIn) {
                    int dbRestId = myApp.getAppDatabase().dineinCartItemDao().getRestaurantId();
                    if (dbRestId != res_id) {
                        myApp.getAppDatabase().dineinCartItemDao().nukeTable();
                    }
                }
            }).start();
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    @Override
    protected void initViews() {
        super.initViews();
        try {
            currentPostcode = myApp.getMyPreferences().getCurrentPostcode();
            llToastMessage = findViewById(R.id.llToastMessage);
            tvToastMessage = findViewById(R.id.tvToastMessage);
            tvUserName = findViewById(R.id.tvUserName);
            id_appbar = findViewById(R.id.id_appbar);
            appBarLayout = findViewById(R.id.appbarLayout);
            toolbar = findViewById(R.id.toolbar);
            tabLayout = findViewById(R.id.tabLayout);
            vpRestaurant = findViewById(R.id.vpRestaurant);
            ivBack = findViewById(R.id.ivBackArrow);
            llBackArrow = findViewById(R.id.llBackArrow);
            tvScreenTitle = findViewById(R.id.tvScreenTitle);
            tvUserNameDetails = tvUserName;
            ivTiffin = findViewById(R.id.ivTiffin);
            ivSearch = findViewById(R.id.ivSearch);
            ivShare = findViewById(R.id.ivShare);
            tvMessage = findViewById(R.id.tvMessage);
            loadingAnimation = findViewById(R.id.animation_view);

            llLoading = findViewById(R.id.llLoading);

            if (getSupportActionBar() == null) {
                setSupportActionBar(toolbar);
            }
            tvScreenTitle.setTypeface(ResourcesCompat.getFont(this, R.font.avenir_heavy));
        } catch (Resources.NotFoundException e) {
            e.printStackTrace();
        }
    }

    public static void hideDefaultUI() {
        ivTiffin.setVisibility(View.GONE);
        tvUserNameDetails.setVisibility(View.GONE);
        tvScreenTitle.setVisibility(View.VISIBLE);
        ivSearch.setVisibility(View.VISIBLE);
        ivShare.setVisibility(View.VISIBLE);
//        id_appbar.setExpanded(false, true);
//        tabLayout.setVisibility(View.GONE);
    }

    public static void showDefaultUI() {
        ivTiffin.setVisibility(View.VISIBLE);
        if (MyApp.getInstance().getMyPreferences().getLoggedInUser() != null) {
            tvUserNameDetails.setVisibility(View.VISIBLE);
        } else {
            tvUserNameDetails.setVisibility(View.GONE);
        }
        tvScreenTitle.setVisibility(View.GONE);
        ivSearch.setVisibility(View.GONE);
        ivShare.setVisibility(View.GONE);
//        id_appbar.setExpanded(true, true);
//        tabLayout.setVisibility(View.VISIBLE);
    }

    private void setListeners() {
        llBackArrow.setOnClickListener(view -> {
            onBackPressed();
        });
        ivSearch.setOnClickListener(view -> {
            startActivityForResult(new Intent(this, TransportActivity.class).putExtra("destination", "search_menu").putExtra("name", openedRestaurant.restaurant_name).putExtra("hideToolbar", true), CODE_REFRESH);
        });

        ivShare.setOnClickListener(view -> {
            shareMessage = "Checkout " + restaurantName + " restaurant on TiffinTom." +
                    "iOS - https://apps.apple.com/in/app/id1437199436\n" +
                    "Android - https://play.google.com/store/apps/details?id=" + BuildConfig.APPLICATION_ID;

            Intent share = new Intent(Intent.ACTION_SEND);
            share.setType("text/plain");
            share.putExtra(Intent.EXTRA_TEXT, shareMessage);
            LogUtils.e("message::", shareMessage);
            startActivity(Intent.createChooser(share, "Share using"));
        });
        toastReceiver = new BroadcastReceiver() {
            @Override
            public void onReceive(Context context, Intent intent) {
                try {
                    runOnUiThread(() -> {
                        if (intent != null && intent.getExtras() != null) {
                            String message = intent.getStringExtra("message");
                            if (!Validators.isNullOrEmpty(message)) {
                                llToastMessage.setVisibility(View.VISIBLE);
                                tvToastMessage.setText(message);
                            } else {
                                llToastMessage.setVisibility(View.GONE);
                            }
                        } else {
                            llToastMessage.setVisibility(View.GONE);
                        }
                        llToastMessage.postDelayed(() -> {
                            llToastMessage.setVisibility(View.GONE);
                        }, 2500);
                    });
                } catch (Exception e) {
                    e.printStackTrace();
                }
            }
        };
    }

    private void initTabs() {
        viewpagerAdapter = new ViewpagerAdapter(getSupportFragmentManager(), FragmentPagerAdapter.BEHAVIOR_RESUME_ONLY_CURRENT_FRAGMENT);
        viewpagerAdapter.addFragment(RestaurantMenuFragment.getInstance(), "Menu");
        viewpagerAdapter.addFragment(RestaurantInfoFragment.getInstance(), "Information");
        viewpagerAdapter.addFragment(RestaurantReviewsFragment.getInstance(), "Reviews");
        viewpagerAdapter.addFragment(RestaurantOffersFragment.getInstance(), "Offers");
        vpRestaurant.setAdapter(viewpagerAdapter);
        tvScreenTitle.setText(openedRestaurant.restaurant_name);
        tabLayout.setupWithViewPager(vpRestaurant);
        setTabIcons();
    }

    private void setTabIcons() {
//        tabLayout.getTabAt(0).setIcon(tabIcons[0]);
//        tabLayout.getTabAt(1).setIcon(tabIcons[1]);
//        tabLayout.getTabAt(2).setIcon(tabIcons[2]);
//        tabLayout.getTabAt(3).setIcon(tabIcons[3]);

        for (int i = 0; i < tabLayout.getTabCount(); i++) {
            TabLayout.Tab tab = tabLayout.getTabAt(i);
            View v = LayoutInflater.from(this).inflate(R.layout.item_tab_with_icon, null);
            TextView tv = v.findViewById(R.id.tvTabTitle);
            ImageView icon = v.findViewById(R.id.ivTabLogo);
            tv.setText(tabTitles[i]);
            icon.setImageResource(tabIcons[i]);
            tab.setCustomView(v);
            LinearLayout layout = ((LinearLayout) ((LinearLayout) tabLayout.getChildAt(0)).getChildAt(i));
            LinearLayout.LayoutParams layoutParams = (LinearLayout.LayoutParams) layout.getLayoutParams();
            layoutParams.weight = 1f; // e.g. 0.5f
            layoutParams.width = LinearLayout.LayoutParams.WRAP_CONTENT;
            layout.setLayoutParams(layoutParams);
        }
    }

    private void fetchRestaurantDetails() {
        try {
            runOnUiThread(() -> {
                llLoading.setVisibility(View.VISIBLE);
                tabLayout.setVisibility(View.GONE);
                if (!Validators.isNullOrEmpty(restaurantName))
                    tvMessage.setText("Loading menu for " + restaurantName);
                else tvMessage.setText("Loading menu for restaurant");
            });
            ANRequest anRequest;
            if (isDineIn) {
                String businessId = "";
                if (myApp.getMyPreferences().getCurrentDineInRestaurantDetail() != null&&myApp.getMyPreferences().getCurrentDineInRestaurantDetail().business!=null) {
                    businessId = myApp.getMyPreferences().getCurrentDineInRestaurantDetail().business.id;
                }
                anRequest = ApiUtils.getDineInRestaurantFromBusiness(qrCode, businessId);
            } else {
              anRequest = ApiUtils.getRestaurantDetails(String.valueOf(res_id),  (MyApp.getInstance().getMyPreferences().getLoggedInUser() == null)  ? "": String.valueOf(loggedInUser.id),myApp.getMyPreferences().getCurrentPostcode().latitude,myApp.getMyPreferences().getCurrentPostcode().longitude);

            }
            anRequest
                    .getAsJSONObject(new JSONObjectRequestListener() {
                        @Override
                        public void onResponse(JSONObject response1) {
                            ApiUtils.getMenuDetails("1",String.valueOf(res_id),"1")
                                    .getAsJSONArray(new JSONArrayRequestListener() {
                                        @Override
                                        public void onResponse(JSONArray response) {
                                            openedRestaurant = new Gson().fromJson(String.valueOf(response1), Restaurant.class);

                                            Type type = new TypeToken<List<MenuItemNew>>() {
                                            }.getType();
                                            openedRestaurant.new_restaurant_menu=new ArrayList<>();
                                            ArrayList<MenuItemNew> menusDetails = new ArrayList<>(new Gson().fromJson(response.toString(), type));
                                            openedRestaurant.new_restaurant_menu.addAll(menusDetails);
                                            fetchAllOffers();
                                            fetchTimeSlots();
                                            fetchReviews();
                                            MyApp.openedRestaurant = openedRestaurant;
                                            initTabs();
                                            runOnUiThread(() -> {
                                                llLoading.setVisibility(View.GONE);
                                                tabLayout.setVisibility(View.VISIBLE);
                                            });
                                        }

                                        @Override
                                        public void onError(ANError anError) {
                                            ToastUtils.makeToast(getApplicationContext(),anError.getErrorBody());
                                        }
                                    });
                        }

                        @Override
                        public void onError(ANError anError) {
                            anError.printStackTrace();
                            LogUtils.e("RES DETAIL ACTIVITY " + anError.getErrorBody());
                            runOnUiThread(() -> {
                                llLoading.setVisibility(View.GONE);
                                tabLayout.setVisibility(View.GONE);
                            });
                            if (!CommonFunctions.isConnected(RestaurantDetailsActivity.this))
                                startActivityForResult(new Intent(RestaurantDetailsActivity.this, NoInternetActivity.class), Constants.CODE_NO_INTERNET);
                        }
                    });
        } catch (Exception e) {
            e.printStackTrace();
        }
    }
    private  void fetchAllOffers()
    {
        try {
            AndroidNetworking.get(ApiEndPoints.NEW_OFFERSALL)
                    .addQueryParameter("restaurant_id", String.valueOf(res_id))
                    .build()
                    .getAsJSONObject(new JSONObjectRequestListener() {
                        @Override
                        public void onResponse(JSONObject response) {
                            Type type = new TypeToken<List<Voucher>>() {}.getType();
                            Type MultiplePriceOffertype = new TypeToken<List<MultiplePriceOffer>>() {}.getType();
                            Type DayOffertype = new TypeToken<List<DayOffer>>() {}.getType();
                            Type DayProductOffertype = new TypeToken<List<DayProductOffer>>() {}.getType();
                            Type DayPriceOffertype = new TypeToken<List<DayPriceOffer>>() {}.getType();
                            Type Offerstype = new TypeToken<List<Offer>>() {}.getType();
                            Type ProductOfferstype = new TypeToken<List<ProductOffer>>() {}.getType();
                            Type PriceOfferstype = new TypeToken<List<PriceOffer>>() {}.getType();
                            try {
                                ArrayList<Voucher> vouchers = new ArrayList<>(new Gson().fromJson(response.getJSONArray("restaurant_vouchers").toString(), type));
                                ArrayList<MultiplePriceOffer>   multiplePriceOffer = new ArrayList<>(new Gson().fromJson(response.getJSONArray("multiple_price_offers").toString(), MultiplePriceOffertype));
                                ArrayList<DayOffer>dayOffer = new ArrayList<>(new Gson().fromJson(response.getJSONArray("day_offers").toString(), DayOffertype));
                                ArrayList<DayProductOffer>dayProductOffer = new ArrayList<>(new Gson().fromJson(response.getJSONArray("day_product_offers").toString(), DayProductOffertype));
                                ArrayList<DayPriceOffer>dayPriceOffer = new ArrayList<>(new Gson().fromJson(response.getJSONArray("day_price_offers").toString(), DayPriceOffertype));
                                ArrayList<Offer>Offers = new ArrayList<>(new Gson().fromJson(response.getJSONArray("offers").toString(), Offerstype));
                                ArrayList<ProductOffer>ProductOffers = new ArrayList<>(new Gson().fromJson(response.getJSONArray("product_offers").toString(), ProductOfferstype));
                                ArrayList<PriceOffer>PriceOffers = new ArrayList<>(new Gson().fromJson(response.getJSONArray("price_offers").toString(), PriceOfferstype));

                                openedRestaurant.voucherLists.addAll(vouchers);
                                openedRestaurant.multiple_price_offers.addAll(multiplePriceOffer);
                                openedRestaurant.day_offers.addAll(dayOffer);
                                openedRestaurant.day_product_offers.addAll(dayProductOffer);
                                openedRestaurant.day_price_offers.addAll(dayPriceOffer);
                                openedRestaurant.offers.addAll(Offers);
                                openedRestaurant.product_offers.addAll(ProductOffers);
                                openedRestaurant.price_offers.addAll(PriceOffers);

                            } catch (JSONException exception) {
                                exception.printStackTrace();
                            }


                        }

                        @Override
                        public void onError(ANError anError) {
                            anError.printStackTrace();
                        }
                    });
        } catch (Exception e) {
            e.printStackTrace();
        }
    }
    private void fetchTimeSlots() {
        try {
            ApiUtils.getRestaurantTimeSlots(String.valueOf(res_id), CommonFunctions.getToday())
                    .getAsJSONObject(new JSONObjectRequestListener() {
                        @Override
                        public void onResponse(JSONObject response) {
                            try {
                                JSONObject JsonObject=new JSONObject(response.toString());
                                openedRestaurant.delivery_time_slot = new Gson().fromJson(JsonObject.getJSONObject("delivery").toString(), TimeSlots.class);
                                openedRestaurant.pickup_time_slot = new Gson().fromJson(JsonObject.getJSONObject("pickup").toString(), TimeSlots.class);


    //                                                      myApp.getMyPreferences().saveCurrentRestaurant(restaurant);
                            } catch (JSONException e) {
                                e.printStackTrace();
                            }
                        }

                        @Override
                        public void onError(ANError anError) {
                            anError.printStackTrace();
                        }
                    });
        } catch (Exception e) {
            e.printStackTrace();
        }
    }
    private void fetchReviews() {
        try {
            openedRestaurant.reviews.clear();
            AndroidNetworking.get(ApiEndPoints.RESTAURANTS_REVIEWS)
                    .addQueryParameter("nopaginate", "1")
                    .addQueryParameter("restaurant_id", String.valueOf(res_id))
                    .build().getAsJSONArray(new JSONArrayRequestListener() {
                        @Override
                        public void onResponse(JSONArray response) {

                            Type type = new TypeToken<List<Review>>() {
                            }.getType();
                            ArrayList<Review> reviews = new ArrayList<>(new Gson().fromJson(response.toString(), type));
                            try {
                                openedRestaurant.reviews.addAll(reviews);

                            } catch (Exception e) {
                                e.printStackTrace();
                            }


                        }

                        @Override
                        public void onError(ANError anError) {
    //                        ToastUtils.makeToast(getApplicationContext(),anError.getErrorBody().toString());
                            anError.getErrorBody();
                        }
                    });
        } catch (Exception e) {
            e.printStackTrace();
        }
    }
    @Override
    protected void onActivityResult(int requestCode, int resultCode, @Nullable Intent data) {
        super.onActivityResult(requestCode, resultCode, data);
        if (requestCode == CODE_NO_INTERNET) {
            fetchRestaurantDetails();
        }
    }

    @Override
    public void onBackPressed() {
        if (!isTaskRoot()) {
            super.onBackPressed();
        } else {
            finish();
            Intent i = new Intent(this, MainActivity.class);
            startActivity(i);
        }
    }

    @Override
    protected void onResume() {
        LocalBroadcastManager.getInstance(this).registerReceiver(toastReceiver, new IntentFilter(TOAST_UPDATE));
        super.onResume();
    }

    @Override
    protected void onStop() {
        LocalBroadcastManager.getInstance(this).unregisterReceiver(toastReceiver);
        super.onStop();
    }
}
