package com.tiffintom.activity;

import android.content.Intent;
import android.os.Bundle;
import android.text.Editable;
import android.text.TextWatcher;
import android.view.KeyEvent;
import android.widget.Button;
import android.widget.EditText;
import android.widget.ImageView;

import androidx.annotation.Nullable;
import androidx.appcompat.widget.Toolbar;
import androidx.recyclerview.widget.LinearLayoutManager;
import androidx.recyclerview.widget.RecyclerView;
import androidx.swiperefreshlayout.widget.SwipeRefreshLayout;

import com.androidnetworking.AndroidNetworking;
import com.androidnetworking.error.ANError;
import com.androidnetworking.interfaces.JSONArrayRequestListener;
import com.androidnetworking.interfaces.JSONObjectRequestListener;
import com.google.gson.Gson;
import com.google.gson.reflect.TypeToken;
import com.tiffintom.R;
import com.tiffintom.adapters.ChatRestaurantAdapter;
import com.tiffintom.base.BaseActivity;
import com.tiffintom.common.CommonFunctions;
import com.tiffintom.fragment.RecentOrdersBottomsheetFragment;
import com.tiffintom.models.RestaurantChatMessage;
import com.tiffintom.models.User;
import com.tiffintom.network.ApiEndPoints;
import com.tiffintom.network.ApiUtils;
import com.tiffintom.utils.LogUtils;

import org.json.JSONArray;
import org.json.JSONException;
import org.json.JSONObject;

import java.lang.reflect.Type;
import java.util.ArrayList;
import java.util.List;

/**
 * Created by Amrish on 18-09-2020.
 */
public class SelectRestaurantActivity extends BaseActivity {
    private RecyclerView rvRestaurants;
    private ChatRestaurantAdapter restaurantAdapter;
    private ArrayList<RestaurantChatMessage> chatMessages = new ArrayList<>();
    private User loggedInUser = myApp.getMyPreferences().getLoggedInUser();
    private ImageView ivBackArrow;
    private EditText etSearch;
    private Button btnSelectOrder;
    private SwipeRefreshLayout swipeRefreshLayout;
    private ArrayList<RestaurantChatMessage> searchedMessages = new ArrayList<>();
    private Toolbar toolbar;

    @Override
    protected void onCreate(@Nullable Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_select_restaurant);
        initViews();
        updateUserName();
        setListeners();
    }

    @Override
    protected void initViews() {
        super.initViews();
        toolbar = findViewById(R.id.toolbar);
        setSupportActionBar(toolbar);
        btnSelectOrder = findViewById(R.id.btnSelectOrder);
        tvUserName = findViewById(R.id.tvUserName);
        ivBackArrow = findViewById(R.id.ivBackArrow);
        etSearch = findViewById(R.id.etSearch);
        swipeRefreshLayout = findViewById(R.id.swipeRefreshLayout);
        rvRestaurants = findViewById(R.id.rvChatRestaurant);
        restaurantAdapter = new ChatRestaurantAdapter(searchedMessages, (position, data) -> {
            if (data instanceof RestaurantChatMessage) {
                startActivity(new Intent(this, TransportActivity.class).putExtra("destination", "restaurant_chat").putExtra("name", "Chat").putExtra("order_id", String.valueOf(((RestaurantChatMessage) data).order_id)).putExtra("restaurant_id", String.valueOf(((RestaurantChatMessage) data).restaurant_id)).putExtra("image_url", String.valueOf(((RestaurantChatMessage) data).restaurant.image_url)).putExtra("hideToolbar", true));
            }
        });
        rvRestaurants.setAdapter(restaurantAdapter);
        rvRestaurants.setLayoutManager(new LinearLayoutManager(this, RecyclerView.VERTICAL, false));

    }

    private void setListeners() {
        ivBackArrow.setOnClickListener(view -> onBackPressed());
        btnSelectOrder.setOnClickListener(view -> {
            RecentOrdersBottomsheetFragment recentOrdersBottomsheetFragment = RecentOrdersBottomsheetFragment.getInstance();
            recentOrdersBottomsheetFragment.show(getSupportFragmentManager(), "show_recent_orders");
        });
        swipeRefreshLayout.setOnRefreshListener(this::fetchRestaurants);
        etSearch.setOnKeyListener((textView, i, keyEvent) -> {
            if (keyEvent.getKeyCode() == KeyEvent.KEYCODE_BACK) {
                onBackPressed();
            } else if ((keyEvent.getKeyCode() == KeyEvent.KEYCODE_ENTER)) {
                searchedMessages.clear();
                searchedMessages.addAll(searchOrder());
                restaurantAdapter.notifyDataSetChanged();
                CommonFunctions.hideKeyboard(this, etSearch);
            }
            return false;
        });
        etSearch.addTextChangedListener(new TextWatcher() {
            @Override
            public void beforeTextChanged(CharSequence charSequence, int i, int i1, int i2) {

            }

            @Override
            public void onTextChanged(CharSequence charSequence, int i, int i1, int i2) {

            }

            @Override
            public void afterTextChanged(Editable editable) {
                if (editable.toString().length() == 0) {
                    searchedMessages.clear();
                    searchedMessages.addAll(searchOrder());
                    restaurantAdapter.notifyDataSetChanged();
                }
            }
        });
    }

    private void fetchRestaurants() {
        swipeRefreshLayout.setRefreshing(true);
        AndroidNetworking.get(ApiEndPoints.NEW_ORDERHELP).addQueryParameter("restaurant_group", "1").addQueryParameter("customer_id", String.valueOf(loggedInUser.id)).addQueryParameter("nopaginate", "1").build()
                .getAsJSONArray(new JSONArrayRequestListener() {
                    @Override
                    public void onResponse(JSONArray response) {
                        swipeRefreshLayout.setRefreshing(false);
                        Type type = new TypeToken<List<RestaurantChatMessage>>() {
                        }.getType();
                        chatMessages.clear();
                        chatMessages.addAll(new Gson().fromJson(response.toString(), type));
                        searchedMessages.clear();
                        searchedMessages.addAll(chatMessages);

                        restaurantAdapter.notifyDataSetChanged();

                    }

                    @Override
                    public void onError(ANError anError) {
                        anError.printStackTrace();
                        swipeRefreshLayout.setRefreshing(false);
                        if (!CommonFunctions.isConnected(SelectRestaurantActivity.this))
                            myApp.noInternet(SelectRestaurantActivity.this);
                    }
                });
    }

    private ArrayList<RestaurantChatMessage> searchOrder() {
        if (etSearch.getText().toString().isEmpty()) {
            return chatMessages;
        }
        ArrayList<RestaurantChatMessage> searchedRest = new ArrayList<>();
        for (RestaurantChatMessage r : chatMessages) {
            if (r.restaurant.restaurant_name != null && r.restaurant.restaurant_name.toLowerCase().contains(etSearch.getText().toString().toLowerCase())) {
                searchedRest.add(r);
            }
        }
        return searchedRest;

    }

    @Override
    protected void onResume() {
        fetchRestaurants();
        super.onResume();
    }
}
