package com.tiffintom.activity;

import android.content.Intent;
import android.os.Bundle;
import android.text.Editable;
import android.text.TextWatcher;
import android.view.View;
import android.widget.Button;
import android.widget.ImageView;
import android.widget.LinearLayout;
import android.widget.TextView;

import androidx.appcompat.widget.Toolbar;

import com.androidnetworking.AndroidNetworking;
import com.androidnetworking.error.ANError;
import com.androidnetworking.interfaces.JSONObjectRequestListener;
import com.androidnetworking.interfaces.ParsedRequestListener;
import com.google.android.material.textfield.TextInputEditText;
import com.google.android.material.textfield.TextInputLayout;
import com.google.gson.Gson;
import com.tiffintom.R;
import com.tiffintom.base.BaseActivity;
import com.tiffintom.common.CommonFunctions;
import com.tiffintom.common.Validators;
import com.tiffintom.fragment.PhoneOrEmailVerificationBottomsheet;
import com.tiffintom.models.User;
import com.tiffintom.network.ApiEndPoints;
import com.tiffintom.network.ApiUtils;
import com.tiffintom.utils.LogUtils;
import com.tiffintom.utils.ToastUtils;

import org.json.JSONObject;

/**
 * Created by Amrish on 18-08-2020.
 */
public class SignupActivity extends BaseActivity {
    private TextInputEditText etEmail, etPassword, etConfirmPassword, etFirstName, etLastName, etReferralCode, etPhone;
    private TextInputLayout tilEmail, tilPassword, tilConfirmPassword, tilFirstName, tilLastName, tilReferralCode, tilPhone;
    private Button btnSignup;
    private Toolbar toolbar;
    private TextView tvScreenTitle, tvUserName;
    private ImageView ivBackArrow;
    private LinearLayout llBackArrow;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_signup);
        initViews();
        setListeners();
        updateViews();
    }

    @Override
    protected void initViews() {
        super.initViews();
        progressDialog = CommonFunctions.customProgressDialog(this, "Loading...");
        toolbar = findViewById(R.id.toolbar);
        tvScreenTitle = findViewById(R.id.tvScreenTitle);
        tvUserName = findViewById(R.id.tvUserName);
        ivBackArrow = findViewById(R.id.ivBackArrow);
        llBackArrow = findViewById(R.id.llBackArrow);

        etFirstName = findViewById(R.id.etFirstName);
        etLastName = findViewById(R.id.etLastName);
        etEmail = findViewById(R.id.etEmail);
        etPassword = findViewById(R.id.etPassword);
        etConfirmPassword = findViewById(R.id.etConfirmPassword);
        etReferralCode = findViewById(R.id.etReferral);
        etPhone = findViewById(R.id.etMobile);

        btnSignup = findViewById(R.id.btnSignup);

        tilEmail = findViewById(R.id.tilEmail);
        tilPassword = findViewById(R.id.tilPassword);
        tilPhone = findViewById(R.id.tilMobile);
        tilConfirmPassword = findViewById(R.id.tilConfirmPassword);
        tilReferralCode = findViewById(R.id.tilReferral);
        tilFirstName = findViewById(R.id.tilFirstName);
        tilLastName = findViewById(R.id.tilLastName);
    }

    private void updateViews() {
        if (getSupportActionBar() == null) {
            setSupportActionBar(toolbar);
        }
        tvUserName.setVisibility(View.GONE);
        tvScreenTitle.setText("Signup with Email");
        tvScreenTitle.setVisibility(View.VISIBLE);
    }

    private void setListeners() {

        etFirstName.addTextChangedListener(new TextWatcher() {
            @Override
            public void beforeTextChanged(CharSequence charSequence, int i, int i1, int i2) {

            }

            @Override
            public void onTextChanged(CharSequence charSequence, int i, int i1, int i2) {
                btnSignup.setEnabled(isFilledData());
            }

            @Override
            public void afterTextChanged(Editable editable) {

            }
        });
        etLastName.addTextChangedListener(new TextWatcher() {
            @Override
            public void beforeTextChanged(CharSequence charSequence, int i, int i1, int i2) {

            }

            @Override
            public void onTextChanged(CharSequence charSequence, int i, int i1, int i2) {
                btnSignup.setEnabled(isFilledData());
            }

            @Override
            public void afterTextChanged(Editable editable) {

            }
        });
        etEmail.addTextChangedListener(new TextWatcher() {
            @Override
            public void beforeTextChanged(CharSequence charSequence, int i, int i1, int i2) {

            }

            @Override
            public void onTextChanged(CharSequence charSequence, int i, int i1, int i2) {
                btnSignup.setEnabled(isFilledData());
            }

            @Override
            public void afterTextChanged(Editable editable) {

            }
        });
        etPhone.addTextChangedListener(new TextWatcher() {
            @Override
            public void beforeTextChanged(CharSequence charSequence, int i, int i1, int i2) {

            }

            @Override
            public void onTextChanged(CharSequence charSequence, int i, int i1, int i2) {
                btnSignup.setEnabled(isFilledData());
            }

            @Override
            public void afterTextChanged(Editable editable) {

            }
        });

        etPassword.addTextChangedListener(new TextWatcher() {
            @Override
            public void beforeTextChanged(CharSequence charSequence, int i, int i1, int i2) {

            }

            @Override
            public void onTextChanged(CharSequence charSequence, int i, int i1, int i2) {
                btnSignup.setEnabled(isFilledData());
            }

            @Override
            public void afterTextChanged(Editable editable) {

            }
        });
        etConfirmPassword.addTextChangedListener(new TextWatcher() {
            @Override
            public void beforeTextChanged(CharSequence charSequence, int i, int i1, int i2) {

            }

            @Override
            public void onTextChanged(CharSequence charSequence, int i, int i1, int i2) {
                btnSignup.setEnabled(isFilledData());
            }

            @Override
            public void afterTextChanged(Editable editable) {

            }
        });

        btnSignup.setOnClickListener(view -> {
            if (isValid()) {
                new Thread(this::doSignUp).start();
            }

        });
        llBackArrow.setOnClickListener(view -> onBackPressed());
    }

    private boolean isFilledData() {
        if (!Validators.isNullOrEmpty(etFirstName.getText().toString()) && (!Validators.isNullOrEmpty(etLastName.getText().toString())) && (!Validators.isNullOrEmpty(etEmail.getText().toString())) && (!Validators.isNullOrEmpty(etPhone.getText().toString())) && (!Validators.isNullOrEmpty(etPassword.getText().toString())) && (!Validators.isNullOrEmpty(etConfirmPassword.getText().toString()))) {
            return true;
        } else
            return false;
    }

    private boolean isValid() {
        tilFirstName.setError(null);
        tilLastName.setError(null);
        tilPhone.setError(null);
        tilEmail.setError(null);
        tilPassword.setError(null);
        tilConfirmPassword.setError(null);
        if (Validators.isNullOrEmpty(etFirstName.getText().toString())) {
            tilFirstName.setError("Please enter first name");
            etFirstName.requestFocus();
            return false;
        }
        if (Validators.isNullOrEmpty(etLastName.getText().toString())) {
            tilLastName.setError("Please enter last name");
            etLastName.requestFocus();
            return false;
        }
        if (Validators.isNullOrEmpty(etEmail.getText().toString())) {
            tilEmail.setError("Please enter email");
            etEmail.requestFocus();
            return false;
        }
        if (!Validators.isEmail(etEmail.getText().toString())) {
            tilEmail.setError("Please enter valid email");
            etEmail.requestFocus();
            return false;
        }

        if (Validators.isNullOrEmpty(etPhone.getText().toString() )) {
            tilPhone.setError("Please enter mobile number");
            etPhone.requestFocus();
            return false;
        }
        if (!Validators.isValidMobile(etPhone.getText().toString())) {
            tilPhone.setError("Invalid mobile number");
            etPhone.requestFocus();
            return false;
        }
        if (Validators.isNullOrEmpty(etPassword.getText().toString())) {
            tilPassword.setError("Please enter password");
            etPassword.requestFocus();
            return false;
        }
        if (!etPassword.getText().toString().equals(etConfirmPassword.getText().toString())) {
            tilConfirmPassword.setError("Password does not match");
            etConfirmPassword.requestFocus();
            return false;
        }
        return true;
    }
    private void doSignUp() {
        AndroidNetworking.post(ApiEndPoints.NEW_CUSTOMER_SIGNUP)
                .addBodyParameter("username", etEmail.getText().toString())
                .addBodyParameter("password", etPassword.getText().toString())
                .addBodyParameter("first_name", etFirstName.getText().toString())
                .addBodyParameter("last_name", etLastName.getText().toString())
                .addBodyParameter("phone_number", etPhone.getText().toString())
                .addBodyParameter("referred_by", !Validators.isNullOrEmpty(etReferralCode.getText().toString()) ? etReferralCode.getText().toString() : "")
                .addBodyParameter("device_id", !Validators.isNullOrEmpty(myApp.getMyPreferences().getDeviceId()) ? myApp.getMyPreferences().getDeviceId() : "")
                .build()
                .getAsObject(User.class, new ParsedRequestListener<User>() {
                    @Override
                    public void onResponse(User response) {
                        runOnUiThread(() -> progressDialog.dismiss());
                        LogUtils.e(response.username+response.id);
                        myApp.getMyPreferences().saveLoggedInUser(response);

                        if (Validators.isNullOrEmpty(myApp.getMyPreferences().getLoggedInUser().first_name) || Validators.isNullOrEmpty(myApp.getMyPreferences().getLoggedInUser().last_name)) {
                            startActivity(new Intent(SignupActivity.this, TransportActivity.class).putExtra("destination", "profile_edit").putExtra("name", "Profile").putExtra("hideBackArrow", true).putExtra("fromAuth", true));
                        }else if (myApp.getMyPreferences().getSiteSettings().signup_verify.equalsIgnoreCase("1"))
                        {
                            startActivity(new Intent(SignupActivity.this, TransportActivity.class).putExtra("destination", "phoneverify").putExtra("hideBackArrow", true).putExtra("type","phone").putExtra("customer_phone",myApp.getMyPreferences().getLoggedInUser().phone_number).putExtra("customer_email",myApp.getMyPreferences().getLoggedInUser().username).putExtra("customer_id",myApp.getMyPreferences().getLoggedInUser().id).putExtra("hideToolbar",true));

                        }else
                        {
                            fetchProfile();
                        }

                    }

                    @Override
                    public void onError(ANError anError) {
                        anError.printStackTrace();
                        LogUtils.e("SIGNUP ERROR:", anError.getErrorBody());
                        ToastUtils.makeToast(SignupActivity.this, "Something went wrong!!");
                        runOnUiThread(() -> progressDialog.dismiss());

                        if (!CommonFunctions.isConnected(SignupActivity.this)) {
                            myApp.noInternet(SignupActivity.this);
                        }
                    }
                });
    }


    private void fetchProfile() {
        runOnUiThread(() -> progressDialog.show());
        AndroidNetworking.get(ApiEndPoints.NEW_ACCOUNT)
                .build()
                .getAsObject(User.class, new ParsedRequestListener<User>() {
                    @Override
                    public void onResponse(User response) {
                        runOnUiThread(() -> progressDialog.dismiss());
                        myApp.getMyPreferences().saveLoggedInUser(response);
                            if (!myApp.getMyPreferences().isLoginSkipped() && myApp.getMyPreferences().getCurrentPostcode() == null) {
                                Bundle bundle = new Bundle();
                                bundle.putString("destination", "postcode");
                                bundle.putBoolean("hideToolbar", true);
                                bundle.putBoolean("finish", true);
                                startActivity(new Intent(SignupActivity.this, TransportActivity.class).putExtras(bundle).setFlags(Intent.FLAG_ACTIVITY_CLEAR_TASK | Intent.FLAG_ACTIVITY_NEW_TASK | Intent.FLAG_ACTIVITY_CLEAR_TOP));
                            } else {
                                startActivity(new Intent(SignupActivity.this, MainActivity.class).setFlags(Intent.FLAG_ACTIVITY_CLEAR_TASK | Intent.FLAG_ACTIVITY_NEW_TASK | Intent.FLAG_ACTIVITY_CLEAR_TOP));
                                finish();
                            }

                    }

                    @Override
                    public void onError(ANError anError) {
                        myApp.logout(SignupActivity.this);
                        anError.printStackTrace();
                        LogUtils.e("SIGNUP PROFILE FETCH ERROR: ", anError.getErrorBody());
                        runOnUiThread(() -> progressDialog.dismiss());

                        if (!CommonFunctions.isConnected(SignupActivity.this)) {
                            myApp.noInternet(SignupActivity.this);
                        }
                    }
                });
    }
}
