package com.tiffintom.activity;

import android.content.Intent;
import android.graphics.drawable.Drawable;
import android.os.Bundle;
import android.os.Handler;
import android.os.Looper;
import android.widget.ImageView;
import android.widget.Toast;

import androidx.annotation.Nullable;

import com.androidnetworking.AndroidNetworking;
import com.androidnetworking.error.ANError;
import com.androidnetworking.interfaces.JSONObjectRequestListener;
import com.bumptech.glide.Glide;
import com.bumptech.glide.load.DataSource;
import com.bumptech.glide.load.engine.GlideException;
import com.bumptech.glide.load.resource.gif.GifDrawable;
import com.bumptech.glide.request.RequestListener;
import com.bumptech.glide.request.target.Target;
import com.google.gson.Gson;
import com.tiffintom.BuildConfig;
import com.tiffintom.R;
import com.tiffintom.base.BaseActivity;
import com.tiffintom.common.CommonFunctions;
import com.tiffintom.common.Validators;
import com.tiffintom.models.MainSiteSettings;
import com.tiffintom.models.Referral;
import com.tiffintom.models.RewardPoint;
import com.tiffintom.models.Rewards;
import com.tiffintom.models.SiteSettings;
import com.tiffintom.models.User;
import com.tiffintom.network.ApiEndPoints;
import com.tiffintom.network.ApiUtils;
import com.tiffintom.utils.LogUtils;
import com.tiffintom.utils.ToastUtils;

import org.json.JSONException;
import org.json.JSONObject;

import java.util.Objects;

import static com.tiffintom.utils.Constants.CODE_NO_INTERNET;

/**
 * Created by Amrish on 18-08-2020.
 */
public class SplashActivity extends BaseActivity {
    ImageView ivSplash;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_splash);
        initViews();
        if (Validators.isNullOrEmpty(myApp.getMyPreferences().getSelectedCountry())) {
            Bundle bundle = new Bundle();
            bundle.putString("destination", "select_country");
            bundle.putBoolean("hideToolbar", true);
            openActivity(TransportActivity.class, bundle);
        } else {
            fetchSiteSettings();
        }
    }


    private void fetchProfile() {
        AndroidNetworking.get(ApiEndPoints.NEW_ACCOUNT)
                .build()
                .getAsJSONObject(new JSONObjectRequestListener() {
                    @Override
                    public void onResponse(JSONObject response) {
                        User user1 = new Gson().fromJson(response.toString(), User.class);
                        myApp.getMyPreferences().saveLoggedInUser(user1);
                        checkPostCodeAndOpenMainActivity();
                    }

                    @Override
                    public void onError(ANError anError) {
                        anError.printStackTrace();
                        if (anError.getErrorCode() == 401) {
                            myApp.logout(SplashActivity.this);
                        }
//                        openActivity(AuthActivity.class);
                        if (!CommonFunctions.isConnected(SplashActivity.this)) {
                            startActivityForResult(new Intent(SplashActivity.this, NoInternetActivity.class), CODE_NO_INTERNET);
                        }
                    }
                });
    }

    private void fetchSiteSettings() {
        LogUtils.e(ApiEndPoints.NEW_BASE_URL_RMS());
        AndroidNetworking.get(ApiEndPoints.NEW_SITE_SETTINGS)
                .build()
                .getAsJSONObject(new JSONObjectRequestListener() {
                    @Override
                    public void onResponse(JSONObject response) {
                        boolean updateAvailable = false;
                        SiteSettings sitesettings = new Gson().fromJson(response.toString(), SiteSettings.class);
                        myApp.getMyPreferences().saveSiteSettings(sitesettings);
                        fetchReferrals();
                        fetchRewards();
                        try {
                            if (Float.parseFloat(sitesettings.android_version) > Float.parseFloat(BuildConfig.VERSION_NAME)) {
                                updateAvailable = true;
                                showUpdateVersion();
                            }
                        } catch (NumberFormatException e) {
                            e.printStackTrace();
                        }
                        if (!updateAvailable)
                            handleNavigation();
                    }

                    @Override
                    public void onError(ANError anError) {
                        anError.printStackTrace();
                        if (myApp.getMyPreferences().getSiteSettings() != null) {
                            handleNavigation();

                        } else {
                            if (!CommonFunctions.isConnected(SplashActivity.this)) {
                                startActivityForResult(new Intent(SplashActivity.this, NoInternetActivity.class), CODE_NO_INTERNET);
                            } else ToastUtils.makeToast(SplashActivity.this, "Unknown error");
                        }
                    }
                });
    }

    private void fetchRewards() {
        AndroidNetworking.get(ApiEndPoints.NEW_REWARDS)
                .build()
                .getAsJSONObject(new JSONObjectRequestListener() {

                    @Override
                    public void onResponse(JSONObject response) {
                        RewardPoint rewardPoint = new Gson().fromJson(response.toString(), RewardPoint.class);
                        myApp.getMyPreferences().saveRewardPoint(rewardPoint);
                    }

                    @Override
                    public void onError(ANError anError) {
                    }
                });
    }

    private void fetchReferrals() {
        AndroidNetworking.get(ApiEndPoints.NEW_Referrals)
                .build()
                .getAsJSONObject(new JSONObjectRequestListener() {
                    @Override
                    public void onResponse(JSONObject response) {
                        Referral referrals = new Gson().fromJson(response.toString(), Referral.class);
                        myApp.getMyPreferences().saveReferral(referrals);
                    }

                    @Override
                    public void onError(ANError anError) {
                    }
                });
    }

    private void handleNavigation() {
        if (myApp.getMyPreferences().getLoggedInUser() == null) {
            if (myApp.getMyPreferences().isLoginSkipped()) {
                checkPostCodeAndOpenMainActivity();
            } else {
                openActivity(AuthActivity.class);
            }
        } else {
            fetchProfile();
        }
    }

    private void openActivity(Class activityClass) {
        new Handler(Objects.requireNonNull(Looper.myLooper()))
                .postDelayed(new Runnable() {
                    @Override
                    public void run() {
                        startActivity(new Intent(SplashActivity.this, activityClass).setFlags(Intent.FLAG_ACTIVITY_CLEAR_TASK | Intent.FLAG_ACTIVITY_NEW_TASK | Intent.FLAG_ACTIVITY_CLEAR_TOP));
                    }
                }, 1500);

    }

    private void openActivity(Class activityClass, Bundle bundle) {
        new Handler(Objects.requireNonNull(Looper.myLooper()))
                .postDelayed(new Runnable() {
                    @Override
                    public void run() {
                        startActivity(new Intent(SplashActivity.this, activityClass).putExtras(bundle).setFlags(Intent.FLAG_ACTIVITY_CLEAR_TASK | Intent.FLAG_ACTIVITY_NEW_TASK | Intent.FLAG_ACTIVITY_CLEAR_TOP));
                    }
                }, 1500);

    }

    private void showUpdateVersion() {
        startActivity(new Intent(this, TransportActivity.class).putExtra("destination", "app_update").putExtra("hideToolbar", true).putExtra("finish", true));
        finish();
    }

    private void checkPostCodeAndOpenMainActivity() {
        if (myApp.getMyPreferences().getCurrentPostcode() == null) {
            Bundle bundle = new Bundle();
            bundle.putString("destination", "postcode");
            bundle.putBoolean("hideToolbar", true);
            bundle.putBoolean("finish", true);
            startActivity(new Intent(SplashActivity.this, TransportActivity.class).putExtras(bundle).setFlags(Intent.FLAG_ACTIVITY_CLEAR_TASK | Intent.FLAG_ACTIVITY_NEW_TASK | Intent.FLAG_ACTIVITY_CLEAR_TOP));
        } else {
            openActivity(MainActivity.class);
        }
    }

    @Override
    protected void initViews() {
        ivSplash = findViewById(R.id.ivSplash);
        Glide.with(this).load(R.drawable.splash_animation).listener(new RequestListener<Drawable>() {
            @Override
            public boolean onLoadFailed(@Nullable GlideException e, Object model, Target<Drawable> target, boolean isFirstResource) {
                return false;
            }

            @Override
            public boolean onResourceReady(Drawable resource, Object model, Target<Drawable> target, DataSource dataSource, boolean isFirstResource) {
                if (resource instanceof GifDrawable) {
                    ((GifDrawable) resource).setLoopCount(1);
                }
                return false;
            }
        }).into(ivSplash);
        super.initViews();
    }

    @Override
    protected void onActivityResult(int requestCode, int resultCode, @Nullable Intent data) {
        super.onActivityResult(requestCode, resultCode, data);
        if (requestCode == CODE_NO_INTERNET) {
            fetchSiteSettings();
        }
    }
}
