package com.tiffintom.activity;

import static com.tiffintom.utils.Constants.CART_UPDATE;
import static com.tiffintom.utils.Constants.CHAT_COUNT_UPDATE;
import static com.tiffintom.utils.Constants.CODE_REFRESH;
import static com.tiffintom.utils.Constants.TOAST_UPDATE;

import android.content.BroadcastReceiver;
import android.content.Context;
import android.content.Intent;
import android.content.IntentFilter;
import android.graphics.Rect;
import android.os.Bundle;
import android.view.LayoutInflater;
import android.view.MenuItem;
import android.view.View;
import android.view.ViewTreeObserver;
import android.widget.ImageView;
import android.widget.LinearLayout;
import android.widget.RelativeLayout;
import android.widget.TextView;

import androidx.annotation.Nullable;
import androidx.cardview.widget.CardView;
import androidx.fragment.app.Fragment;
import androidx.fragment.app.FragmentManager;
import androidx.fragment.app.FragmentTransaction;
import androidx.localbroadcastmanager.content.LocalBroadcastManager;

import com.google.android.material.bottomnavigation.BottomNavigationItemView;
import com.google.android.material.bottomnavigation.BottomNavigationMenuView;
import com.google.android.material.bottomnavigation.BottomNavigationView;
import com.tiffintom.R;
import com.tiffintom.base.BaseActivity;
import com.tiffintom.common.Validators;
import com.tiffintom.fragment.AboutFragment;
import com.tiffintom.fragment.CartCheckoutFragment;
import com.tiffintom.fragment.CreditsFragment;
import com.tiffintom.fragment.DineInHomeFragment;
import com.tiffintom.fragment.DineinCartCheckoutFragment;
import com.tiffintom.fragment.DineinHistoryTabsFragment;
import com.tiffintom.fragment.EnterPostcodeFragment;
import com.tiffintom.fragment.FavouriteRestaurantsFragment;
import com.tiffintom.fragment.FeedbackFragment;
import com.tiffintom.fragment.FragmentUpdateApp;
import com.tiffintom.fragment.HelpFAQFragment;
import com.tiffintom.fragment.HelpMainFragment;
import com.tiffintom.fragment.HomeFragment;
import com.tiffintom.fragment.NotificationFragment;
import com.tiffintom.fragment.OrderHistoryTabsFragment;
import com.tiffintom.fragment.OrderReceiptFragment;
import com.tiffintom.fragment.PaymentMethodsFragment;
import com.tiffintom.fragment.PhoneOrEmailVerificationFragment;
import com.tiffintom.fragment.ProfileEditFragment;
import com.tiffintom.fragment.ReferralHistoryFragment;
import com.tiffintom.fragment.RestaurantChatWindowFragment;
import com.tiffintom.fragment.RewardsFragment;
import com.tiffintom.fragment.SearchMenuFragment;
import com.tiffintom.fragment.SelectCountryFragment;
import com.tiffintom.fragment.TiffintomChatWindowFragment;
import com.tiffintom.fragment.TrackOrderFragment;
import com.tiffintom.fragment.WalletFragment;
import com.tiffintom.fragment.WebviewFragment;
import com.tiffintom.fragment.becompartner.BecomePartnerFragment;
import com.tiffintom.fragment.becompartner.CompanyDetailFragment;
import com.tiffintom.fragment.becompartner.MenuLogoFragment;
import com.tiffintom.fragment.becompartner.PartnerDetailFragment;
import com.tiffintom.fragment.becompartner.RecevingOrderFragment;
import com.tiffintom.fragment.becompartner.RestaurantAgreementFragment;
import com.tiffintom.fragment.becompartner.SummeryFragment;
import com.tiffintom.fragment.becompartner.ThankYouFragment;
import com.tiffintom.models.CartSummary;
import com.tiffintom.utils.Constants;
import com.tiffintom.utils.ToastUtils;

/**
 * Created by Amrish on 26-08-2020.
 */
public class TransportActivity extends BaseActivity {
    private ImageView ivBack;
    private TextView tvScreenTitle;
    private String destination, screenName;
    private boolean hideToolbar, finish, hideBackArrow;
    private LinearLayout llBackArrow;
    private BottomNavigationView bottomNavigationView;
    private boolean hideBottombar,shouldGoBack = true;;
    private RelativeLayout rlToolbar;
    private BroadcastReceiver broadcastReceiver, chatCountBroadCastReceiver;
    public TextView tvSettingNotification_Badge;
    public TextView tvCartItemsCountBadge;
    public static boolean isOrderPlaced = false;

    @Override
    protected void onCreate(@Nullable Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_transport);
        if (getIntent() != null) {
            destination = getIntent().getStringExtra("destination");
            screenName = getIntent().getStringExtra("name");
            hideToolbar = getIntent().getBooleanExtra("hideToolbar", false);
            hideBackArrow = getIntent().getBooleanExtra("hideBackArrow", false);
            finish = getIntent().getBooleanExtra("finish", false);
        }
        initViews();
        toastListener();
        updateUserName();
        bottomNavigationView.setOnNavigationItemSelectedListener(null);
        if (!Validators.isNullOrEmpty(screenName)) {
            tvScreenTitle.setText(screenName);
            tvScreenTitle.setVisibility(View.VISIBLE);
        }
        if (hideToolbar) {
            rlToolbar.setVisibility(View.GONE);
        }
        handleNavigation();
        setListeners();
        updateChatCount(myApp.chatCount);
    }

    @Override
    protected void initViews() {
        super.initViews();
        tvUserName = findViewById(R.id.tvUserName);
        rlToolbar = findViewById(R.id.rlToolbar);
        ivBack = findViewById(R.id.ivBackArrow);
        llBackArrow = findViewById(R.id.llBackArrow);
        tvScreenTitle = findViewById(R.id.tvScreenTitle);
        llToastLayout = findViewById(R.id.llToastMessage);
        tvToastMessage = findViewById(R.id.tvToastMessage);

        bottomNavigationView = findViewById(R.id.bottomNavigationView);
        bottomNavigationView.setItemIconTintList(null);

        BottomNavigationMenuView bottomNavigationMenuView = (BottomNavigationMenuView) bottomNavigationView.getChildAt(0);
        View cartView = bottomNavigationMenuView.getChildAt(4);
        BottomNavigationItemView cartItemView = (BottomNavigationItemView) cartView;
        View badge = LayoutInflater.from(this)
                .inflate(R.layout.bottomnavigation_notification_badge, bottomNavigationMenuView, false);
        tvCartItemsCountBadge = badge.findViewById(R.id.tvNotification_Badge);
        tvCartItemsCountBadge.setVisibility(View.GONE);
        cartItemView.addView(badge);


        View settingView = bottomNavigationMenuView.getChildAt(3);
        BottomNavigationItemView settingItemView = (BottomNavigationItemView) settingView;
        View settingBadge = LayoutInflater.from(this)
                .inflate(R.layout.bottomnavigation_notification_badge, bottomNavigationMenuView, false);
        tvSettingNotification_Badge = settingBadge.findViewById(R.id.tvNotification_Badge);
        tvSettingNotification_Badge.setVisibility(View.GONE);
        settingItemView.addView(settingBadge);
    }

    private void handleNavigation() {
        if (destination.equalsIgnoreCase("dinein_menu")) {
            bottomNavigationView.setVisibility(View.GONE);
            hideBottombar = true;
            changeFragment(DineInHomeFragment.getInstance());
        }
        if (destination.equalsIgnoreCase("profile_edit")) {
            if (!hideBackArrow) {
                changeFragment(ProfileEditFragment.getInstance());
                bottomNavigationView.getMenu().findItem(R.id.btmProfile).setChecked(true);
            } else {
                bottomNavigationView.setVisibility(View.GONE);
                hideBottombar = true;
                changeFragment(ProfileEditFragment.getInstance());
                llBackArrow.setVisibility(View.GONE);
            }

        }

        if (destination.equalsIgnoreCase("phone_verify_fragment")) {
            bottomNavigationView.setVisibility(View.GONE);
            hideBottombar = true;
            llBackArrow.setVisibility(View.GONE);
            changeFragment(PhoneOrEmailVerificationFragment.getInstance());
        }

        if (destination.equalsIgnoreCase("wallet")) {
            bottomNavigationView.getMenu().findItem(R.id.btmProfile).setChecked(true);
            changeFragment(WalletFragment.getInstance());
        }
        if (destination.equalsIgnoreCase("rewards")) {
            bottomNavigationView.getMenu().findItem(R.id.btmProfile).setChecked(true);
            changeFragment(RewardsFragment.getInstance());
        }
        if (destination.equalsIgnoreCase("referral_history")) {
            bottomNavigationView.getMenu().findItem(R.id.btmSettings).setChecked(true);
            changeFragment(ReferralHistoryFragment.getInstance());
        }
        if (destination.equalsIgnoreCase("postcode")) {
            bottomNavigationView.setVisibility(View.GONE);
            hideBottombar = true;
            changeFragment(EnterPostcodeFragment.getInstance());
        }
        if (destination.equalsIgnoreCase("select_country")) {
            bottomNavigationView.setVisibility(View.GONE);
            hideBottombar = true;
            shouldGoBack = false;
            changeFragment(SelectCountryFragment.getInstance());
        }
        if (destination.equalsIgnoreCase("credits")) {
            bottomNavigationView.getMenu().findItem(R.id.btmProfile).setChecked(true);
            changeFragment(CreditsFragment.getInstance());
        }
        if (destination.equalsIgnoreCase("favourites")) {
            bottomNavigationView.getMenu().findItem(R.id.btmProfile).setChecked(true);
            changeFragment(FavouriteRestaurantsFragment.getInstance());
        }
        if (destination.equalsIgnoreCase("cart")) {
            bottomNavigationView.getMenu().findItem(R.id.btmCart).setChecked(true);
            changeFragment(CartCheckoutFragment.getInstance());
        }
        if (destination.equalsIgnoreCase("orders")) {
            bottomNavigationView.getMenu().findItem(R.id.btmProfile).setChecked(true);
            changeFragment(OrderHistoryTabsFragment.getInstance());
        }
        if (destination.equalsIgnoreCase("orderView")) {
            bottomNavigationView.getMenu().findItem(R.id.btmProfile).setChecked(true);
            changeFragment(OrderReceiptFragment.getInstance());
        }
        // TODO
       /* if (destination.equalsIgnoreCase("dineinOrderView")) {
            bottomNavigationView.getMenu().findItem(R.id.btmProfile).setChecked(true);
            changeFragment(DineinOrderReceiptFragment.getInstance());
        }*/

        if (destination.equalsIgnoreCase("trackOrder")) {
            bottomNavigationView.getMenu().findItem(R.id.btmProfile).setChecked(true);
            changeFragment(TrackOrderFragment.getInstance());
        }
        if (destination.equalsIgnoreCase("help_main")) {
            bottomNavigationView.getMenu().findItem(R.id.btmSettings).setChecked(true);
            changeFragment(HelpMainFragment.getInstance());
        }
        if (destination.equalsIgnoreCase("restaurants")) {
            bottomNavigationView.getMenu().findItem(R.id.btmExplore).setChecked(true);
            changeFragment(HomeFragment.getInstance());
        }
        if (destination.equalsIgnoreCase("faq")) {
            bottomNavigationView.getMenu().findItem(R.id.btmSettings).setChecked(true);
            changeFragment(HelpFAQFragment.getInstance());
        }
        if (destination.equalsIgnoreCase("search_menu")) {
            bottomNavigationView.setVisibility(View.GONE);
            hideBottombar = true;
            changeFragment(SearchMenuFragment.getInstance(screenName));
        }
        if (destination.equalsIgnoreCase("feedback")) {
            bottomNavigationView.getMenu().findItem(R.id.btmSettings).setChecked(true);
            changeFragment(FeedbackFragment.getInstance());
        }
        if (destination.equalsIgnoreCase("about")) {
            bottomNavigationView.getMenu().findItem(R.id.btmSettings).setChecked(true);
            changeFragment(AboutFragment.getInstance());
        }
        if (destination.equalsIgnoreCase("webview")) {
            bottomNavigationView.setVisibility(View.GONE);
            hideBottombar = true;
            changeFragment(WebviewFragment.getInstance());
        }
        if (destination.equalsIgnoreCase("becomepartner")) {
            bottomNavigationView.getMenu().findItem(R.id.btmSettings).setChecked(true);
            changeFragment(BecomePartnerFragment.getInstance());
        }
        if (destination.equalsIgnoreCase("partnerdetail")) {
            bottomNavigationView.getMenu().findItem(R.id.btmSettings).setChecked(true);
            changeFragment(PartnerDetailFragment.getInstance());
        }
        if (destination.equalsIgnoreCase("tiffintom_chat_window")) {
            bottomNavigationView.setVisibility(View.GONE);
            hideBottombar = true;
            bottomNavigationView.getMenu().findItem(R.id.btmSettings).setChecked(true);
            changeFragment(TiffintomChatWindowFragment.getInstance());
        }
        if (destination.equalsIgnoreCase("restaurant_chat")) {
            bottomNavigationView.setVisibility(View.GONE);
            hideBottombar = true;
            bottomNavigationView.getMenu().findItem(R.id.btmSettings).setChecked(true);
            changeFragment(RestaurantChatWindowFragment.getInstance());
        }
        if (destination.equalsIgnoreCase("receivingorder")) {
            bottomNavigationView.getMenu().findItem(R.id.btmSettings).setChecked(true);
            changeFragment(RecevingOrderFragment.getInstance());
        }
        if (destination.equalsIgnoreCase("companydetail")) {
            bottomNavigationView.getMenu().findItem(R.id.btmSettings).setChecked(true);
            changeFragment(CompanyDetailFragment.getInstance());
        }
        if (destination.equalsIgnoreCase("menu&logo")) {
            bottomNavigationView.getMenu().findItem(R.id.btmSettings).setChecked(true);
            changeFragment(MenuLogoFragment.getInstance());
        }
        if (destination.equalsIgnoreCase("summery")) {
            bottomNavigationView.getMenu().findItem(R.id.btmSettings).setChecked(true);
            changeFragment(SummeryFragment.getInstance());
        }
        if (destination.equalsIgnoreCase("restaurant_agreement")) {
            bottomNavigationView.getMenu().findItem(R.id.btmSettings).setChecked(true);
            changeFragment(RestaurantAgreementFragment.getInstance());
        }
        if (destination.equalsIgnoreCase("thankyou")) {
            bottomNavigationView.getMenu().findItem(R.id.btmSettings).setChecked(true);
            changeFragment(ThankYouFragment.getInstance());
        }
        if (destination.equalsIgnoreCase("notification")) {
            bottomNavigationView.getMenu().findItem(R.id.btmSettings).setChecked(true);
            changeFragment(NotificationFragment.getInstance());
        }
        // TODO
        if (destination.equalsIgnoreCase("dinein_checkout")) {
            bottomNavigationView.setVisibility(View.GONE);
            hideBottombar = true;
            bottomNavigationView.getMenu().findItem(R.id.btmCart).setChecked(true);
            changeFragment(DineinCartCheckoutFragment.getInstance());
        }
        // TODO
        if (destination.equalsIgnoreCase("dinein_history")) {
            bottomNavigationView.getMenu().findItem(R.id.btmProfile).setChecked(true);
            changeFragment(DineinHistoryTabsFragment.getInstance());
        }
        if (destination.equalsIgnoreCase("app_update")) {
            bottomNavigationView.setVisibility(View.GONE);
            hideBottombar = true;
            changeFragment(FragmentUpdateApp.getInstance());
        }
        if (destination.equalsIgnoreCase("payment_method")) {
            bottomNavigationView.getMenu().findItem(R.id.btmCart).setChecked(true);
            changeFragment(PaymentMethodsFragment.getInstance());
        }

    }

    private void setListeners() {
        bottomNavigationView.setOnNavigationItemSelectedListener(this::manageFragments);
        llBackArrow.setOnClickListener(view -> {
            onBackPressed();
        });
        broadcastReceiver = new BroadcastReceiver() {
            @Override
            public void onReceive(Context context, Intent intent) {
                boolean updateForCart = intent.getBooleanExtra("update_cart_count", false);
                if (updateForCart) updateCartCount();
            }
        };
        chatCountBroadCastReceiver = new BroadcastReceiver() {
            @Override
            public void onReceive(Context context, Intent intent) {
                int count = intent.getIntExtra("count", 0);
                updateChatCount(count);
            }
        };
        toastReceiver = new BroadcastReceiver() {
            @Override
            public void onReceive(Context context, Intent intent) {
                runOnUiThread(() -> {
                    if (intent != null && intent.getExtras() != null) {
                        String message = intent.getStringExtra("message");
                        if (!Validators.isNullOrEmpty(message)) {
                            llToastLayout.setVisibility(View.VISIBLE);
                            tvToastMessage.setText(message);
                        } else {
                            llToastLayout.setVisibility(View.GONE);
                        }
                    } else {
                        llToastLayout.setVisibility(View.GONE);
                    }
                    llToastLayout.postDelayed(() -> {
                        llToastLayout.setVisibility(View.GONE);
                    }, 2500);
                });
            }
        };
    }

    private void updateChatCount(int count) {
        runOnUiThread(() -> {
            tvSettingNotification_Badge.setText(String.valueOf(count));
            if (count > 0) {
                tvSettingNotification_Badge.setVisibility(View.VISIBLE);
            } else {
                tvSettingNotification_Badge.setVisibility(View.GONE);
            }
        });
    }

    private void updateCartCount() {
        new Thread(() -> {
            CartSummary cartSummary = myApp.getAppDatabase().cartDao().getCartSummary();
            runOnUiThread(() -> {
                if (cartSummary != null && cartSummary.items > 0) {
                    tvCartItemsCountBadge.setVisibility(View.VISIBLE);
                    tvCartItemsCountBadge.setText(String.valueOf(cartSummary.items));
                } else {
                    tvCartItemsCountBadge.setVisibility(View.GONE);
                    tvCartItemsCountBadge.setText("0");
                }
            });
        }).start();
    }

    private boolean manageFragments(MenuItem item) {
        if (item.getItemId() == R.id.btmHome) {
            startActivity(new Intent(this, MainActivity.class).addFlags(Intent.FLAG_ACTIVITY_NEW_TASK | Intent.FLAG_ACTIVITY_CLEAR_TASK | Intent.FLAG_ACTIVITY_CLEAR_TOP));

            return true;
        }
        if (item.getItemId() == R.id.btmExplore) {
            startActivity(new Intent(this, MainActivity.class).putExtra("destination", "explore").addFlags(Intent.FLAG_ACTIVITY_NEW_TASK | Intent.FLAG_ACTIVITY_CLEAR_TASK | Intent.FLAG_ACTIVITY_CLEAR_TOP));

            return true;
        }
        if (item.getItemId() == R.id.btmProfile) {
            if (myApp.getMyPreferences().getLoggedInUser() == null) {
                openLoginScreen();
                return false;
            } else {
                startActivity(new Intent(this, MainActivity.class).putExtra("destination", "profile").addFlags(Intent.FLAG_ACTIVITY_NEW_TASK | Intent.FLAG_ACTIVITY_CLEAR_TASK | Intent.FLAG_ACTIVITY_CLEAR_TOP));

                return true;
            }
        }
        if (item.getItemId() == R.id.btmCart) {
            Fragment openedFragment = getSupportFragmentManager().findFragmentById(R.id.flContainer);
            // TODO && !(openedFragment instanceof DineinCartCheckoutFragment) add after dine in code
            if (!(openedFragment instanceof CartCheckoutFragment)) {
                if (myApp.getMyPreferences().getLoggedInUser() == null) {
                    openLoginScreen();
                    return false;
                } else {
                    startActivity(new Intent(this, MainActivity.class).putExtra("destination", "cart").addFlags(Intent.FLAG_ACTIVITY_NEW_TASK | Intent.FLAG_ACTIVITY_CLEAR_TASK | Intent.FLAG_ACTIVITY_CLEAR_TOP));
                    return true;
                }
            }
        }
        if (item.getItemId() == R.id.btmSettings) {
            if (myApp.getMyPreferences().getLoggedInUser() == null) {
                openLoginScreen();
                return false;
            } else {
                startActivity(new Intent(this, MainActivity.class).putExtra("destination", "settings").addFlags(Intent.FLAG_ACTIVITY_NEW_TASK | Intent.FLAG_ACTIVITY_CLEAR_TASK | Intent.FLAG_ACTIVITY_CLEAR_TOP));

                return true;
            }
        }
        return false;
    }

    private void openLoginScreen() {
        startActivityForResult(new Intent(this, AuthActivity.class), CODE_REFRESH);
    }

    private void changeFragment(Fragment fragment) {
        if (getIntent() != null) {
            fragment.setArguments(getIntent().getExtras());
        }
        FragmentManager manager = getSupportFragmentManager();
        FragmentTransaction transaction = manager.beginTransaction();
        transaction.replace(R.id.flContainer, fragment);
        transaction.commit();
    }


    boolean mKeyboardVisible;
    private final ViewTreeObserver.OnGlobalLayoutListener mLayoutKeyboardVisibilityListener =
            () -> {
                final Rect rectangle = new Rect();
                final View contentView = this.findViewById(android.R.id.content).getRootView();
                contentView.getWindowVisibleDisplayFrame(rectangle);
                int screenHeight = contentView.getRootView().getHeight();

                // r.bottom is the position above soft keypad or device button.
                // If keypad is shown, the rectangle.bottom is smaller than that before.
                int keypadHeight = screenHeight - rectangle.bottom;
                // 0.15 ratio is perhaps enough to determine keypad height.
                boolean isKeyboardNowVisible = keypadHeight > screenHeight * 0.15;

                if (mKeyboardVisible != isKeyboardNowVisible) {
                    if (isKeyboardNowVisible) {
                        onKeyboardShown();
                    } else {
                        onKeyboardHidden();
                    }
                }

                mKeyboardVisible = isKeyboardNowVisible;
            };

    private void onKeyboardShown() {
        bottomNavigationView.setVisibility(View.GONE);
    }

    private void onKeyboardHidden() {
        if (!hideBottombar)
            bottomNavigationView.setVisibility(View.VISIBLE);
    }

    @Override
    public void onBackPressed() {
        Fragment f = getSupportFragmentManager().findFragmentById(R.id.flContainer);
        if (isOrderPlaced) {
            //   super.onBackPressed();
            getSupportFragmentManager().popBackStack(null, FragmentManager.POP_BACK_STACK_INCLUSIVE);
            finish();
            startActivity(new Intent(this, MainActivity.class).setFlags(Intent.FLAG_ACTIVITY_CLEAR_TASK | Intent.FLAG_ACTIVITY_NEW_TASK | Intent.FLAG_ACTIVITY_CLEAR_TOP));
        } else {
            if (finish) {
                super.onBackPressed();
            } else {
                if (!isTaskRoot()) {
                    if (f instanceof DineInHomeFragment) {
                        new Thread(() -> {
                            myApp.getAppDatabase().dineinCartItemDao().nukeTable();
                            myApp.getAppDatabase().orderItemDao().nukeTable();
                        }).start();
                    }
                    super.onBackPressed();
                } else {
                    finish();
                    Intent i = new Intent(this, MainActivity.class);
                    startActivity(i);
                }
            }
        }
    }

    @Override
    protected void onResume() {
        super.onResume();
        updateCartCount();
        LocalBroadcastManager.getInstance(this).registerReceiver(broadcastReceiver, new IntentFilter(CART_UPDATE));
        LocalBroadcastManager.getInstance(this).registerReceiver(chatCountBroadCastReceiver, new IntentFilter(CHAT_COUNT_UPDATE));
        LocalBroadcastManager.getInstance(this).registerReceiver(toastReceiver, new IntentFilter(TOAST_UPDATE));
        findViewById(android.R.id.content).getViewTreeObserver()
                .addOnGlobalLayoutListener(mLayoutKeyboardVisibilityListener);
    }

    @Override
    protected void onStop() {
        LocalBroadcastManager.getInstance(this).unregisterReceiver(broadcastReceiver);
        LocalBroadcastManager.getInstance(this).unregisterReceiver(chatCountBroadCastReceiver);
        LocalBroadcastManager.getInstance(this).unregisterReceiver(toastReceiver);
        super.onStop();
    }

    @Override
    protected void onActivityResult(int requestCode, int resultCode, @Nullable Intent data) {
        super.onActivityResult(requestCode, resultCode, data);
        if (requestCode == Constants.CODE_MESSAGE && resultCode == RESULT_OK) {
            LocalBroadcastManager.getInstance(this).registerReceiver(toastReceiver, new IntentFilter(TOAST_UPDATE));
            if (data != null) {
                setResult(RESULT_OK, data);
                String message = data.getStringExtra("message");
                if (!Validators.isNullOrEmpty(message)) {
                    ToastUtils.makeSnackToast(this, message);
                }

            }
        }
    }
}
