package com.tiffintom.adapters;

import android.app.Activity;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.ImageView;
import android.widget.TextView;

import androidx.annotation.NonNull;
import androidx.recyclerview.widget.RecyclerView;

import com.tiffintom.MyApp;
import com.tiffintom.R;
import com.tiffintom.interfaces.RecyclerViewItemClickListener;
import com.tiffintom.models.Header;
import com.tiffintom.models.MainAddOn;
import com.tiffintom.models.SubAddOn;

import java.util.ArrayList;

/**
 * Created by Amrish on 10-09-2020.
 */
public class AddonsAdapter extends RecyclerView.Adapter<RecyclerView.ViewHolder> {
    private Activity activity;
    private ArrayList<Object> objects;
    private RecyclerViewItemClickListener recyclerViewItemClickListener;


    public AddonsAdapter(Activity activity, ArrayList<Object> objects, RecyclerViewItemClickListener recyclerViewItemClickListener) {
        this.activity = activity;
        this.objects = objects;
        this.recyclerViewItemClickListener = recyclerViewItemClickListener;
    }

    @Override
    public int getItemViewType(int position) {
        if (objects.get(position) instanceof MainAddOn) {
            return R.layout.menu_addon_item;
        } else if (objects.get(position) instanceof SubAddOn) {
            return R.layout.menu_subaddon_item;
        } else {
            return R.layout.item_addon_header;
        }
    }

    @NonNull
    @Override
    public RecyclerView.ViewHolder onCreateViewHolder(@NonNull ViewGroup parent, int viewType) {
        View view = LayoutInflater.from(activity).inflate(viewType, parent, false);
        if (viewType == R.layout.menu_subaddon_item) {
            return new SubAddOnViewHolder(view);
        } else if (viewType == R.layout.menu_addon_item) {
            return new MenuItemAddOnViewHolder(view);
        } else {
            return new AddonHeaderViewHolder(view);
        }
    }

    @Override
    public void onBindViewHolder(@NonNull RecyclerView.ViewHolder viewHolder, int position) {
        if (getItemViewType(position) == R.layout.menu_subaddon_item) {
            SubAddOn subAddOn = (SubAddOn) objects.get(position);
            SubAddOnViewHolder holder = (SubAddOnViewHolder) viewHolder;
//            holder.tvTitle.setText(subAddOn.subaddons_name);
            StringBuilder addOnTitle = new StringBuilder();
            addOnTitle.append(subAddOn.subaddons_name);
            if (subAddOn.subaddons_price > 0) {
                addOnTitle.append(" (").append(MyApp.getInstance().getCurrencySymbol()).append(subAddOn.subaddons_price).append(")");
            } else {
                addOnTitle.append(" (Free)");
            }
            holder.tvTitle.setText(addOnTitle.toString());

            if (subAddOn.selected) {
                holder.ivSelect.setImageResource(R.drawable.ic_baseline_done_24);
            } else {
                holder.ivSelect.setImageResource(R.drawable.dot_grey_circle);
            }
            holder.view.setOnClickListener((view) -> {
                subAddOn.selected = !subAddOn.selected;
                notifyItemChanged(position);
                if (recyclerViewItemClickListener != null)
                    recyclerViewItemClickListener.onItemClick(position, subAddOn);
            });
        }
        if (getItemViewType(position) == R.layout.menu_addon_item) {
            MainAddOn mainAddOn = (MainAddOn) objects.get(position);
            MenuItemAddOnViewHolder holder = (MenuItemAddOnViewHolder) viewHolder;
            holder.tvTitle.setText(mainAddOn.mainaddons_name);
            if (mainAddOn.selected) {
                holder.ivSelect.setImageResource(R.drawable.ic_baseline_done_24);
            } else {
                holder.ivSelect.setImageResource(R.drawable.dot_grey_circle);
            }
            holder.view.setOnClickListener((view) -> {
                mainAddOn.selected = !mainAddOn.selected;
                notifyItemChanged(position);
                if (recyclerViewItemClickListener != null)
                    recyclerViewItemClickListener.onItemClick(position, mainAddOn);
            });
            if (mainAddOn.mainaddons_mini_count > 0) {
                holder.tvTitle.setText(mainAddOn.mainaddons_name + " (required)");
            } else {
                holder.tvTitle.setText(mainAddOn.mainaddons_name + " (optional)");
            }
        }
        if (getItemViewType(position) == R.layout.item_addon_header) {
            Header header = (Header) objects.get(position);
            AddonHeaderViewHolder holder = (AddonHeaderViewHolder) viewHolder;
            holder.tvTitle.setText(header.title);
        }

    }

    @Override
    public int getItemCount() {
        return objects.size();
    }

    protected static class MenuItemAddOnViewHolder extends RecyclerView.ViewHolder {
        private TextView tvTitle;
        private ImageView ivSelect;
        private View view;

        public MenuItemAddOnViewHolder(@NonNull View itemView) {
            super(itemView);
            tvTitle = itemView.findViewById(R.id.tvTitle);
            ivSelect = itemView.findViewById(R.id.ivSelect);
            view = itemView;
        }
    }

    protected static class SubAddOnViewHolder extends RecyclerView.ViewHolder {
        private TextView tvTitle;
        private ImageView ivSelect;
        private View view;

        public SubAddOnViewHolder(@NonNull View itemView) {
            super(itemView);
            tvTitle = itemView.findViewById(R.id.tvTitle);
            ivSelect = itemView.findViewById(R.id.ivSelect);
            view = itemView;
        }
    }

    private static class AddonHeaderViewHolder extends RecyclerView.ViewHolder {
        private TextView tvTitle;

        public AddonHeaderViewHolder(@NonNull View itemView) {
            super(itemView);
            tvTitle = itemView.findViewById(R.id.tvTitle);
        }
    }
}
