package com.tiffintom.adapters;

import android.app.Activity;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.TextView;

import androidx.annotation.NonNull;
import androidx.recyclerview.widget.RecyclerView;

import com.tiffintom.R;
import com.tiffintom.common.CommonFunctions;
import com.tiffintom.interfaces.RecyclerViewItemClickListener;
import com.tiffintom.models.Address;

import java.util.ArrayList;

/**
 * Created by Amrish on 20-08-2020.
 */
public class AddressAdapter extends RecyclerView.Adapter<AddressAdapter.AddressViewHolder> {
    Activity activity;
    ArrayList<Address> addresses;
    RecyclerViewItemClickListener recyclerViewItemClickListener;

    public AddressAdapter(Activity activity, ArrayList<Address> addresses, RecyclerViewItemClickListener recyclerViewItemClickListener) {
        this.activity = activity;
        this.addresses = addresses;
        this.recyclerViewItemClickListener = recyclerViewItemClickListener;
    }

    @NonNull
    @Override
    public AddressViewHolder onCreateViewHolder(@NonNull ViewGroup parent, int viewType) {
        return new AddressViewHolder(LayoutInflater.from(activity).inflate(R.layout.address_item, parent, false));
    }

    @Override
    public void onBindViewHolder(@NonNull AddressViewHolder holder, int position) {
        Address address = addresses.get(position);
        holder.tvTitle.setText(address.title);
        holder.tvSubtitle.setText(CommonFunctions.StringAppender(address.flat_no, ", ", address.address));
        holder.tvPostcode.setText(address.zipcode);
    }

    @Override
    public int getItemCount() {
        return addresses.size();
    }

    public class AddressViewHolder extends RecyclerView.ViewHolder {
        private TextView tvTitle, tvSubtitle, tvPostcode;

        public AddressViewHolder(@NonNull View itemView) {
            super(itemView);
            tvTitle = itemView.findViewById(R.id.tvTitle);
            tvSubtitle = itemView.findViewById(R.id.tvSubtitle);
            tvPostcode = itemView.findViewById(R.id.tvPostcode);
            itemView.setOnClickListener(view -> {
                if (recyclerViewItemClickListener != null) {
                    recyclerViewItemClickListener.onItemClick(getLayoutPosition(), addresses.get(getLayoutPosition()));
                }
            });
        }
    }
}
