package com.tiffintom.adapters;

import android.app.Activity;
import android.os.Bundle;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.CheckBox;
import android.widget.LinearLayout;
import android.widget.TextView;

import androidx.annotation.NonNull;
import androidx.cardview.widget.CardView;
import androidx.recyclerview.widget.RecyclerView;

import com.chauthai.swipereveallayout.SwipeRevealLayout;
import com.chauthai.swipereveallayout.ViewBinderHelper;
import com.tiffintom.MyApp;
import com.tiffintom.R;
import com.tiffintom.common.CommonFunctions;
import com.tiffintom.interfaces.RecyclerViewItemClickListener;
import com.tiffintom.models.Address;

import java.util.ArrayList;

/**
 * Created by Amrish on 20-08-2020.
 */
public class AddressBookAdapter extends RecyclerView.Adapter<RecyclerView.ViewHolder> {
    Activity activity;
    ArrayList<Address> addresses;
    RecyclerViewItemClickListener recyclerViewItemClickListener, editItemClickListener, deleteItemClickListener;
    boolean mini_view;
    private final ViewBinderHelper viewBinderHelper = new ViewBinderHelper();
    public AddressBookAdapter(Activity activity, ArrayList<Address> addresses, RecyclerViewItemClickListener recyclerViewItemClickListener, RecyclerViewItemClickListener editItemClickListener, RecyclerViewItemClickListener deleteItemClickListener) {
        this.activity = activity;
        this.addresses = addresses;
        this.recyclerViewItemClickListener = recyclerViewItemClickListener;
        this.editItemClickListener = editItemClickListener;
        this.deleteItemClickListener = deleteItemClickListener;
    }

    public AddressBookAdapter(Activity activity, ArrayList<Address> addresses, boolean mini_view, RecyclerViewItemClickListener recyclerViewItemClickListener) {
        this.activity = activity;
        this.addresses = addresses;
        this.mini_view = mini_view;
        this.recyclerViewItemClickListener = recyclerViewItemClickListener;
    }

    @NonNull
    @Override
    public RecyclerView.ViewHolder onCreateViewHolder(@NonNull ViewGroup parent, int viewType) {
        if (mini_view) {
            return new MiniAddressViewHolder(LayoutInflater.from(activity).inflate(R.layout.dialog_addressbook_items, parent, false));
        } else
            return new AddressViewHolder(LayoutInflater.from(activity).inflate(R.layout.address_book_item, parent, false));
    }

    @Override
    public void onBindViewHolder(@NonNull RecyclerView.ViewHolder holder, int position) {
        Address address = addresses.get(position);
        if (mini_view) {
            MiniAddressViewHolder miniview = (MiniAddressViewHolder) holder;
            miniview.tvTitle.setText(address.title);
            miniview.tvSubtitle.setText(CommonFunctions.StringAppender(address.flat_no, ", ", address.address, ", ", address.zipcode));
            Address defaultAddress = MyApp.getInstance().getMyPreferences().getCurrentAddress();
            if (defaultAddress != null && defaultAddress.id == address.id) {
                miniview.cbSelect.setChecked(true);
            } else {
                miniview.cbSelect.setChecked(false);
            }
            miniview.cvAddress.setOnClickListener(view -> {
                MyApp.getInstance().getMyPreferences().saveCurrentAddress(address);
                miniview.cbSelect.setChecked(true);
                notifyDataSetChanged();
                if (recyclerViewItemClickListener != null) {
                    recyclerViewItemClickListener.onItemClick(position, address);
                }
            });

        } else {
            AddressViewHolder viewHolder = (AddressViewHolder) holder;
            viewBinderHelper.bind(viewHolder.swipeRevealLayout, String.valueOf(address.id));
            viewHolder.tvTitle.setText(address.title);
            viewHolder.tvSubtitle.setText(CommonFunctions.StringAppender(address.flat_no, ", ", address.address, ", ", address.zipcode));
            Address defaultAddress = MyApp.getInstance().getMyPreferences().getCurrentAddress();
            if (defaultAddress != null && defaultAddress.id == address.id) {
                viewHolder.cbSelect.setChecked(true);
            } else {
                viewHolder.cbSelect.setChecked(false);
            }
            viewHolder.cvAddress.setOnClickListener(view -> {
                MyApp.getInstance().getMyPreferences().saveCurrentAddress(address);
                notifyDataSetChanged();
            });
            viewHolder.llEdit.setOnClickListener(view -> {
                viewHolder.swipeRevealLayout.close(true);
                if (editItemClickListener != null)
                    editItemClickListener.onItemClick(position, address);
            });
            viewHolder.llDelete.setOnClickListener(view -> {
                viewHolder.swipeRevealLayout.close(true);
                if (deleteItemClickListener != null)
                    deleteItemClickListener.onItemClick(position, address);
            });
        }
    }

    @Override
    public int getItemCount() {
        return addresses.size();
    }

    public class AddressViewHolder extends RecyclerView.ViewHolder {
        private TextView tvTitle, tvSubtitle;
        private LinearLayout cvAddress;
        private CheckBox cbSelect;
        private LinearLayout llEdit, llDelete;
        private SwipeRevealLayout swipeRevealLayout;

        public AddressViewHolder(@NonNull View itemView) {
            super(itemView);
            tvTitle = itemView.findViewById(R.id.tvTitle);
            tvSubtitle = itemView.findViewById(R.id.tvSubtitle);
            cvAddress = itemView.findViewById(R.id.cvAddress);
            cbSelect = itemView.findViewById(R.id.cbSelect);
            llEdit = itemView.findViewById(R.id.llEdit);
            llDelete = itemView.findViewById(R.id.llDelete);
            swipeRevealLayout = itemView.findViewById(R.id.swipeRevealLayout);
        }
    }

    public class MiniAddressViewHolder extends RecyclerView.ViewHolder {
        private TextView tvTitle, tvSubtitle;
        private CardView cvAddress;
        private CheckBox cbSelect;

        public MiniAddressViewHolder(@NonNull View itemView) {
            super(itemView);
            tvTitle = itemView.findViewById(R.id.tvTitle);
            tvSubtitle = itemView.findViewById(R.id.tvSubtitle);
            cvAddress = itemView.findViewById(R.id.cvAddress);
            cbSelect = itemView.findViewById(R.id.cbSelect);
        }
    }

    public void saveStates(Bundle outState) {
        viewBinderHelper.saveStates(outState);
    }

    public void restoreStates(Bundle inState) {
        viewBinderHelper.restoreStates(inState);
    }
}
