package com.tiffintom.adapters;

import android.content.Context;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.ImageView;
import android.widget.TextView;

import androidx.annotation.NonNull;
import androidx.core.content.ContextCompat;
import androidx.recyclerview.widget.RecyclerView;

import com.tiffintom.R;
import com.tiffintom.interfaces.RecyclerViewItemClickListener;
import com.tiffintom.models.AllergyItem;

import java.util.ArrayList;

/**
 * Created by Amrish on 19-08-2020.
 */
public class AllergiesAdapter extends RecyclerView.Adapter<AllergiesAdapter.AllergyViewHolder> {
    private ArrayList<AllergyItem> allergies = new ArrayList<>();
    private RecyclerViewItemClickListener moreClickListener, lessClickListener;

    public AllergiesAdapter(ArrayList<AllergyItem> allergies, RecyclerViewItemClickListener recyclerViewItemClickListener, RecyclerViewItemClickListener lessClickListener) {
        this.allergies = allergies;
        this.moreClickListener = recyclerViewItemClickListener;
        this.lessClickListener = lessClickListener;
    }

    @NonNull
    @Override
    public AllergiesAdapter.AllergyViewHolder onCreateViewHolder(@NonNull ViewGroup parent, int viewType) {
        return new AllergyViewHolder(LayoutInflater.from(parent.getContext()).inflate(R.layout.allery_item_with_text, parent, false));
    }

    @Override
    public void onBindViewHolder(@NonNull AllergiesAdapter.AllergyViewHolder holder, int position) {
        AllergyItem allergyItem = allergies.get(position);
        holder.tvTitle.setText(allergyItem.title);
        holder.tvTitle.setSelected(true);
        Context activity = holder.itemView.getContext();
        if (allergyItem.icon > 0) {
            holder.ivAllergy.setImageResource(allergyItem.icon);
            holder.tvTitle.setTextColor(ContextCompat.getColor(activity, R.color.black));
        } else {
            if (allergyItem.icon == -1) {
                holder.tvTitle.setTextColor(ContextCompat.getColor(activity, R.color.dark_grey_color));
            } else {
                holder.tvTitle.setTextColor(ContextCompat.getColor(activity, R.color.black));
            }
        }
        holder.view.setOnClickListener(view -> {
            if (allergyItem.icon == -1) {
                if (allergyItem.title.equalsIgnoreCase("more")) {
                    moreClickListener.onItemClick(position, allergyItem);
                } else {
                    lessClickListener.onItemClick(position, allergyItem);
                }
            }
        });


    }

    @Override
    public int getItemCount() {
        return allergies.size();
    }

    protected static class AllergyViewHolder extends RecyclerView.ViewHolder {
        private TextView tvTitle;
        private ImageView ivAllergy;
        private View view;

        public AllergyViewHolder(@NonNull View itemView) {
            super(itemView);
            view = itemView;
            tvTitle = itemView.findViewById(R.id.tvTitle);
            ivAllergy = itemView.findViewById(R.id.ivAllergy);

        }
    }
}
