package com.tiffintom.adapters;

import android.app.Activity;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.ImageView;
import android.widget.TextView;

import androidx.annotation.NonNull;
import androidx.cardview.widget.CardView;
import androidx.recyclerview.widget.RecyclerView;

import com.bumptech.glide.Glide;
import com.tiffintom.MyApp;
import com.tiffintom.R;
import com.tiffintom.common.CommonFunctions;
import com.tiffintom.interfaces.RecyclerViewItemClickListener;
import com.tiffintom.models.Banner;

import java.util.ArrayList;

/**
 * Created by Amrish on 19-08-2020.
 */
public class BannerAdapter extends RecyclerView.Adapter<BannerAdapter.BannerViewHolder> {
    private Activity activity;
    private ArrayList<Banner> banners = new ArrayList<>();
    private String selectedBanner = null;
    private String bannerLink = null;
    private RecyclerViewItemClickListener bannerClickListener;

    public BannerAdapter(Activity activity, ArrayList<Banner> banners, String selectedBanner, RecyclerViewItemClickListener bannerClickListener) {
        this.activity = activity;
        this.banners = banners;
        this.selectedBanner = selectedBanner;
        this.bannerClickListener = bannerClickListener;
    }

    @NonNull
    @Override
    public BannerAdapter.BannerViewHolder onCreateViewHolder(@NonNull ViewGroup parent, int viewType) {
        return new BannerViewHolder(LayoutInflater.from(activity).inflate(R.layout.home_banner_item, parent, false));
    }

    @Override
    public void onBindViewHolder(@NonNull BannerAdapter.BannerViewHolder holder, int position) {
        Banner banner = banners.get(position);
        holder.tvTitle.setText(banner.title);
        holder.tvDescription.setText(banner.title);
        Glide.with(activity)
                .load(banner.image_url)
                .placeholder(R.drawable.banner_placeholder)
                .error(R.drawable.banner_placeholder)
                .into(holder.ivBanner);

    }

    @Override
    public int getItemCount() {
        return banners.size();
    }

    protected class BannerViewHolder extends RecyclerView.ViewHolder {
        private TextView tvTitle, tvDescription;
        private CardView cvBanner;
        private ImageView ivBanner;

        public BannerViewHolder(@NonNull View itemView) {
            super(itemView);
            tvTitle = itemView.findViewById(R.id.tvTitle);
            tvDescription = itemView.findViewById(R.id.tvDescription);
            ivBanner = itemView.findViewById(R.id.ivBanner);
            cvBanner = itemView.findViewById(R.id.cvBanner);

            cvBanner.setOnClickListener(view -> {
                if (bannerClickListener != null) {
                    bannerClickListener.onItemClick(getLayoutPosition(), banners.get(getLayoutPosition()));
                }
            });
        }
    }
}
