package com.tiffintom.adapters;

import android.app.Activity;
import android.content.Context;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.ImageView;
import android.widget.TextView;
import android.widget.Toast;

import androidx.annotation.NonNull;
import androidx.cardview.widget.CardView;
import androidx.viewpager.widget.PagerAdapter;
import androidx.viewpager.widget.ViewPager;

import com.bumptech.glide.Glide;
import com.tiffintom.MyApp;
import com.tiffintom.R;
import com.tiffintom.common.CommonFunctions;
import com.tiffintom.interfaces.RecyclerViewItemClickListener;
import com.tiffintom.models.Banner;

import java.util.ArrayList;

public class BannerViewPagerAdapter extends PagerAdapter {
    private Activity activity;
    private ArrayList<Banner> banners = new ArrayList<>();
    private String selectedBanner = null;
    private String bannerLink = null;
    private RecyclerViewItemClickListener bannerClickListener;
    private LayoutInflater layoutInflater;

    public BannerViewPagerAdapter(Activity activity, ArrayList<Banner> banners, String selectedBanner, RecyclerViewItemClickListener bannerClickListener) {
        this.activity = activity;
        this.banners = banners;
        this.selectedBanner = selectedBanner;
        this.bannerClickListener = bannerClickListener;
    }

    @Override
    public int getCount() {
        return banners.size();
    }

    @Override
    public boolean isViewFromObject(@NonNull View view, @NonNull Object object) {
        return view == object;
    }

    @NonNull
    @Override
    public Object instantiateItem(@NonNull ViewGroup container, int position) {
        layoutInflater = (LayoutInflater) activity.getSystemService(Context.LAYOUT_INFLATER_SERVICE);
        View view = layoutInflater.inflate(R.layout.home_banner_item, null);

        Banner banner = banners.get(position);

        TextView tvTitle = view.findViewById(R.id.tvTitle);
        TextView tvDescription = view.findViewById(R.id.tvDescription);
        CardView cvBanner = view.findViewById(R.id.cvBanner);
        ImageView ivBanner = view.findViewById(R.id.ivBanner);

        tvTitle.setText(banner.banner_image);
        tvDescription.setText(banner.banner_image);
         Glide.with(activity)
                .load(banner.image_url)
                .placeholder(R.drawable.banner_placeholder)
                .error(R.drawable.banner_placeholder)
                .into(ivBanner);

        cvBanner.setOnClickListener(view1 -> {
            if (bannerClickListener != null) {
                bannerClickListener.onItemClick(position, banners.get(position));
            }
        });
        container.addView(view);
        return view;
    }

    @Override
    public void destroyItem(@NonNull ViewGroup container, int position, @NonNull Object object) {
        ViewPager vp = (ViewPager) container;
        View view = (View) object;
        vp.removeView(view);
    }
}
