package com.tiffintom.adapters;

import android.app.Activity;
import android.graphics.Paint;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.ImageView;
import android.widget.LinearLayout;
import android.widget.TextView;

import androidx.annotation.NonNull;
import androidx.recyclerview.widget.RecyclerView;

import com.tiffintom.MyApp;
import com.tiffintom.R;
import com.tiffintom.common.Validators;
import com.tiffintom.interfaces.RecyclerViewItemClickListener;
import com.tiffintom.models.CartItem;
import com.tiffintom.models.DineinCartItem;
import com.tiffintom.models.Menu;
import com.tiffintom.models.MenuItem;
import com.tiffintom.models.Variant;

import java.util.ArrayList;

/**
 * Created by Amrish on 27-08-2020.
 */
public class CartBasketItemsAdapter extends RecyclerView.Adapter<RecyclerView.ViewHolder> {
    private Activity activity;
    private ArrayList<Object> cartItems;
    private RecyclerViewItemClickListener plusListener, minusListener;
    private boolean isDinein = false;

    public CartBasketItemsAdapter(Activity activity, ArrayList<Object> cartItems, RecyclerViewItemClickListener plusListener, RecyclerViewItemClickListener minusListener) {
        this.activity = activity;
        this.cartItems = cartItems;
        this.plusListener = plusListener;
        this.minusListener = minusListener;
    }

    public CartBasketItemsAdapter(Activity activity, ArrayList<Object> cartItems, boolean isDinein, RecyclerViewItemClickListener plusListener, RecyclerViewItemClickListener minusListener) {
        this.activity = activity;
        this.cartItems = cartItems;
        this.plusListener = plusListener;
        this.minusListener = minusListener;
        this.isDinein = isDinein;
    }

    @Override
    public int getItemViewType(int position) {
            if (cartItems.get(position) instanceof CartItem) {
                return R.layout.cart_basket_item;
            } else if (cartItems.get(position) instanceof DineinCartItem) {
                return R.layout.dinein_cart_basket_item;
            } else if (cartItems.get(position) instanceof Menu) {
                return R.layout.cart_suggestion_item;
            } else {
                return R.layout.cart_suggestion_header;
            }
    }

    @NonNull
    @Override
    public RecyclerView.ViewHolder onCreateViewHolder(@NonNull ViewGroup parent, int viewType) {
        View view = LayoutInflater.from(activity).inflate(viewType, parent, false);
        if (viewType == R.layout.cart_basket_item) {
            return new CartBasketItemViewHolder(view);
        } else if (viewType == R.layout.dinein_cart_basket_item) {
            return new CartBasketItemViewHolder(view);
        } else if (viewType == R.layout.cart_suggestion_item) {
            return new CartSuggestionItemViewHolder(view);
        } else {
            return new CartSuggestionHeaderViewHolder(view);
        }

    }

    @Override
    public long getItemId(int position) {
        if (cartItems.get(position) instanceof CartItem) {
            return ((CartItem) cartItems.get(position)).cartItemId;
        } else if (cartItems.get(position) instanceof DineinCartItem) {
            return ((DineinCartItem) cartItems.get(position)).dineInCartItemId;
        } else if (cartItems.get(position) instanceof Menu) {
            return ((Menu) cartItems.get(position)).id;
        } else {
            return System.currentTimeMillis();
        }
    }

    @Override
    public void onBindViewHolder(@NonNull RecyclerView.ViewHolder viewHolder, int position) {
        if (getItemViewType(position) == R.layout.cart_basket_item) {
            CartBasketItemViewHolder holder = (CartBasketItemViewHolder) viewHolder;
            CartItem cartItem = (CartItem) cartItems.get(position);

            if (!Validators.isNullOrEmpty(cartItem.menu_size)) {
                holder.tvName.setText(String.format("%s (%s)", cartItem.menu_name, cartItem.menu_size));
            } else {
                holder.tvName.setText(cartItem.menu_name);
            }
            if (!Validators.isNullOrEmpty(cartItem.Addon_name)) {
                holder.tvSubAddon.setVisibility(View.VISIBLE);
            } else {
                holder.tvSubAddon.setVisibility(View.GONE);
            }
            holder.tvSubAddon.setText(cartItem.Addon_name);
            String currency = MyApp.getInstance().getCurrencySymbol();
            if (cartItem.Total == 0) {
                holder.tvPrice.setText("Free");
            } else {
                holder.tvPrice.setText(currency + MyApp.df.format(cartItem.Total));
            }
            holder.tvCount.setText(String.valueOf(cartItem.quantity));
            if (cartItem.Addon_name.equalsIgnoreCase("offer")) {
                holder.ivPlus.setVisibility(View.GONE);
                holder.ivMinus.setVisibility(View.GONE);
                LinearLayout.LayoutParams params = new LinearLayout.LayoutParams(
                        LinearLayout.LayoutParams.WRAP_CONTENT,
                        LinearLayout.LayoutParams.WRAP_CONTENT
                );
                params.setMargins(16, 0, 16, 0);
                holder.tvCount.setLayoutParams(params);
            } else {
                holder.ivPlus.setVisibility(View.VISIBLE);
                holder.ivMinus.setVisibility(View.VISIBLE);
                LinearLayout.LayoutParams params = new LinearLayout.LayoutParams(
                        LinearLayout.LayoutParams.WRAP_CONTENT,
                        LinearLayout.LayoutParams.WRAP_CONTENT
                );
                params.setMargins(0, 0, 0, 0);
                holder.tvCount.setLayoutParams(params);
            }
            holder.ivPlus.setOnClickListener(view -> {
                plusListener.onItemClick(position, cartItem);

            });
            holder.ivMinus.setOnClickListener(view -> {
                minusListener.onItemClick(position, cartItem);

            });
        }
        if (getItemViewType(position) == R.layout.dinein_cart_basket_item) {
            CartBasketItemViewHolder holder = (CartBasketItemViewHolder) viewHolder;
            DineinCartItem cartItem = (DineinCartItem) cartItems.get(position);
            if (!Validators.isNullOrEmpty(cartItem.menu_size)) {
                holder.tvName.setText(String.format("%s (%s)", cartItem.menu_name, cartItem.menu_size));
            } else {
                holder.tvName.setText(cartItem.menu_name);
            }
            if (!Validators.isNullOrEmpty(cartItem.Addon_name)) {
                holder.tvSubAddon.setVisibility(View.VISIBLE);
            } else {
                holder.tvSubAddon.setVisibility(View.GONE);
            }
            holder.tvSubAddon.setText(cartItem.Addon_name);
            String currency = MyApp.getInstance().getCurrencySymbol();
            if (cartItem.Total == 0) {
                holder.tvPrice.setText("Free");
            } else {
                holder.tvPrice.setText(currency + MyApp.df.format(cartItem.Total));
            }
            holder.tvCount.setText(String.valueOf(cartItem.quantity));
            if (cartItem.Addon_name.equalsIgnoreCase("offer")) {
                holder.ivPlus.setVisibility(View.GONE);
                holder.ivMinus.setVisibility(View.GONE);
                LinearLayout.LayoutParams params = new LinearLayout.LayoutParams(
                        LinearLayout.LayoutParams.WRAP_CONTENT,
                        LinearLayout.LayoutParams.WRAP_CONTENT
                );
                params.setMargins(16, 0, 16, 0);
                holder.tvCount.setLayoutParams(params);
            } else {
                holder.ivPlus.setVisibility(View.VISIBLE);
                holder.ivMinus.setVisibility(View.VISIBLE);
                LinearLayout.LayoutParams params = new LinearLayout.LayoutParams(
                        LinearLayout.LayoutParams.WRAP_CONTENT,
                        LinearLayout.LayoutParams.WRAP_CONTENT
                );
                params.setMargins(0, 0, 0, 0);
                holder.tvCount.setLayoutParams(params);
            }
            holder.ivPlus.setOnClickListener(view -> {
                plusListener.onItemClick(position, cartItem);

            });
            holder.ivMinus.setOnClickListener(view -> {
                minusListener.onItemClick(position, cartItem);
            });
        }
        if (getItemViewType(position) == R.layout.cart_suggestion_item) {
            CartSuggestionItemViewHolder holder = (CartSuggestionItemViewHolder) viewHolder;
            Menu menuItem = (Menu) cartItems.get(position);


            holder.tvName.setText(menuItem.menu_name);
            String currency = MyApp.getInstance().getCurrencySymbol();
            if (isDinein) {
                if (menuItem.dinein_menu_details != null && menuItem.dinein_menu_details.size() > 0) {
                    Variant variant = menuItem.dinein_menu_details.get(0);
                    if (menuItem.getProductPercentage() > 0) {
                        float discountedPrice = variant.orginal_price - (variant.orginal_price * menuItem.getProductPercentage() / 100);
                        holder.tvPrice.setText(currency + MyApp.df.format(discountedPrice));
                        holder.tvStrikedPrice.setText(currency + MyApp.df.format(variant.orginal_price));
                        holder.tvStrikedPrice.setVisibility(View.VISIBLE);
                    } else {
                        holder.tvPrice.setText(currency + MyApp.df.format(variant.orginal_price));
                        holder.tvStrikedPrice.setVisibility(View.GONE);
                    }
                } else {
                    holder.tvPrice.setText("");
                    holder.tvStrikedPrice.setVisibility(View.GONE);
                }
            } else {
                if (menuItem.variants != null && menuItem.variants.size() > 0) {
                    Variant variant = menuItem.variants.get(0);
                    if (menuItem.getProductPercentage() > 0) {
                        float discountedPrice = variant.orginal_price - (variant.orginal_price * menuItem.getProductPercentage() / 100);
                        holder.tvPrice.setText(currency + MyApp.df.format(discountedPrice));
                        holder.tvStrikedPrice.setText(currency + MyApp.df.format(variant.orginal_price));
                        holder.tvStrikedPrice.setVisibility(View.VISIBLE);
                    } else {
                        holder.tvStrikedPrice.setVisibility(View.GONE);
                        holder.tvPrice.setText(currency + MyApp.df.format(variant.orginal_price));
                    }
                } else {
                    holder.tvPrice.setText("");
                    holder.tvStrikedPrice.setVisibility(View.GONE);
                }
            }
            holder.ivPlus.setOnClickListener(view -> {
                if (plusListener != null) {
                    plusListener.onItemClick(position, cartItems.get(position));
                }
            });
        }
       /* if (getItemViewType(position) == R.layout.cart_suggestion_header) {
c
        }*/
    }

    @Override
    public int getItemCount() {

        return cartItems.size();
    }

    protected static class CartBasketItemViewHolder extends RecyclerView.ViewHolder {

        private TextView tvCount, tvName, tvSubAddon, tvPrice;
        private ImageView ivPlus, ivMinus;

        public CartBasketItemViewHolder(@NonNull View itemView) {
            super(itemView);
            tvCount = itemView.findViewById(R.id.tvQuantity);
            tvName = itemView.findViewById(R.id.tvName);
            tvPrice = itemView.findViewById(R.id.tvPrice);
            tvSubAddon = itemView.findViewById(R.id.tvSubAddon);
            ivMinus = itemView.findViewById(R.id.ivMinus);
            ivPlus = itemView.findViewById(R.id.ivPlus);
        }
    }

    protected static class CartSuggestionItemViewHolder extends RecyclerView.ViewHolder {
        private TextView tvName, tvPrice, tvStrikedPrice;
        private ImageView ivPlus;

        public CartSuggestionItemViewHolder(View view) {
            super(view);
            tvName = itemView.findViewById(R.id.tvName);
            tvPrice = itemView.findViewById(R.id.tvPrice);
            tvStrikedPrice = itemView.findViewById(R.id.tvStrikedPrice);
            ivPlus = itemView.findViewById(R.id.ivPlus);

            tvStrikedPrice.setPaintFlags(Paint.STRIKE_THRU_TEXT_FLAG);
        }
    }

    protected static class CartSuggestionHeaderViewHolder extends RecyclerView.ViewHolder {
        public CartSuggestionHeaderViewHolder(View view) {
            super(view);
        }
    }
}
