package com.tiffintom.adapters;

import android.content.Context;
import android.media.AudioManager;
import android.media.MediaPlayer;
import android.os.Handler;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.ImageView;
import android.widget.LinearLayout;
import android.widget.ProgressBar;
import android.widget.TextView;

import androidx.annotation.NonNull;
import androidx.appcompat.widget.AppCompatSeekBar;
import androidx.cardview.widget.CardView;
import androidx.recyclerview.widget.DividerItemDecoration;
import androidx.recyclerview.widget.LinearLayoutManager;
import androidx.recyclerview.widget.RecyclerView;

import com.bumptech.glide.Glide;
import com.mikhaellopez.circularimageview.CircularImageView;
import com.tiffintom.MyApp;
import com.tiffintom.R;
import com.tiffintom.common.CommonFunctions;
import com.tiffintom.common.Validators;
import com.tiffintom.interfaces.RecyclerViewItemClickListener;
import com.tiffintom.models.ChatMainQuestion;
import com.tiffintom.models.ChatMiniOrderSnippet;
import com.tiffintom.models.Header;
import com.tiffintom.models.Message;
import com.tiffintom.models.User;

import java.io.IOException;
import java.util.ArrayList;

/**
 * Created by Amrish on 12-09-2020.
 */
public class ChatAdapter extends RecyclerView.Adapter<RecyclerView.ViewHolder> {
    private ArrayList<Object> objects;
    private RecyclerViewItemClickListener recyclerViewItemClickListener;
    private User loggedInUser = MyApp.getInstance().getMyPreferences().getLoggedInUser();
    private String restaurantLogo;
    private boolean tiffintomChat = false;

    public ChatAdapter(ArrayList<Object> objects, RecyclerViewItemClickListener recyclerViewItemClickListener, String restaurantLogo) {
        this.objects = objects;
        this.recyclerViewItemClickListener = recyclerViewItemClickListener;
        this.restaurantLogo = restaurantLogo;
    }

    public ChatAdapter(ArrayList<Object> objects, RecyclerViewItemClickListener recyclerViewItemClickListener) {
        this.objects = objects;
        this.tiffintomChat = true;
        this.recyclerViewItemClickListener = recyclerViewItemClickListener;
    }

    @NonNull
    @Override
    public RecyclerView.ViewHolder onCreateViewHolder(@NonNull ViewGroup parent, int viewType) {
        View view = LayoutInflater.from(parent.getContext()).inflate(viewType, parent, false);
        if (viewType == R.layout.item_default_response) {
            return new MessageViewHolder(view);
        } else if (viewType == R.layout.item_chat_response_options) {
            return new DefaultOptionsViewHolder(view);
        } else if (viewType == R.layout.item_chat_orders_snippet) {
            return new ChatOrdersSnippetViewHolder(view);
        } else if (viewType == R.layout.item_chat_message_nonowner || viewType == R.layout.item_chat_message_owner) {
            return new MessageViewHolder(view);
        } else if (viewType == R.layout.item_chat_image_message_owner || viewType == R.layout.item_chat_image_message_nonowner) {
            return new ChatImageMessageViewHolder(view);
        } else if (viewType == R.layout.item_chat_audio_message_owner || viewType == R.layout.item_chat_audio_message_nonowner) {
            return new ChatAudioViewHolder(view);
        }
        return null;
    }

    @Override
    public int getItemViewType(int position) {
        if (objects.get(position) instanceof Message) {
            Message m = (Message) objects.get(position);
            if (m.is_attach == 1) {
                if (m.admin == 0) {
                    return R.layout.item_chat_image_message_owner;
                } else
                    return R.layout.item_chat_image_message_nonowner;
            } else if (m.is_attach == 2) {
                if (m.admin == 0) {
                    return R.layout.item_chat_audio_message_owner;
                } else
                    return R.layout.item_chat_audio_message_nonowner;
            } else {
                if (m.admin == 0)
                    return R.layout.item_chat_message_owner;
                else return R.layout.item_chat_message_nonowner;
            }
        } else if (objects.get(position) instanceof Header) {
            return R.layout.item_default_response;
        } else if (objects.get(position) instanceof ChatMainQuestion) {
            return R.layout.item_chat_response_options;
        } else if (objects.get(position) instanceof ChatMiniOrderSnippet) {
            return R.layout.item_chat_orders_snippet;
        }
        return super.getItemViewType(position);
    }

    @Override
    public void onBindViewHolder(@NonNull RecyclerView.ViewHolder viewHolder, int position) {
        if (getItemViewType(position) == R.layout.item_default_response) {
            MessageViewHolder holder = (MessageViewHolder) viewHolder;
            Header message = (Header) objects.get(position);
            holder.tvMessage.setText(message.title);
            holder.ivSenderImage.setVisibility(View.VISIBLE);
            if (!Validators.isNullOrEmpty(restaurantLogo)) {
                Glide.with(holder.itemView.getContext())
                        .load(restaurantLogo)
                        .placeholder(R.drawable.ic_tiffin_square)
                        .error(R.drawable.ic_tiffin_square)
                        .into(holder.ivSenderImage);
            } else {
                holder.ivSenderImage.setImageResource(R.drawable.ic_tiffin_square);
            }
        }

        if (getItemViewType(position) == R.layout.item_chat_message_nonowner || getItemViewType(position) == R.layout.item_chat_message_owner) {
            MessageViewHolder holder = (MessageViewHolder) viewHolder;
            Message message = (Message) objects.get(position);
            holder.tvMessage.setText(message.message);
            String logo = "";
            int placeHolder = 0;
            if (message.admin == 1) {
                if (!Validators.isNullOrEmpty(restaurantLogo))
                    logo = restaurantLogo;
                placeHolder = R.drawable.ic_tiffin_square;
            } else if (message.admin == 0) {
                if (!Validators.isNullOrEmpty(loggedInUser.image_url))
                    logo = loggedInUser.image_url;
                placeHolder = R.drawable.user_profile_image_placeholder;
            }

            if (!Validators.isNullOrEmpty(logo)) {
                Glide.with(holder.itemView.getContext())
                        .load(logo)
                        .placeholder(placeHolder)
                        .error(placeHolder)
                        .into(holder.ivSenderImage);
            } else {
                holder.ivSenderImage.setImageResource(placeHolder);
            }

            if (!Validators.isNullOrEmpty(message.created)) {
                holder.tvMessageTime.setText(CommonFunctions.formatTimestamp(message.created.replace(":", ""), "EEE, dd MMM 'at' hh:mm a"));
            } else {
                holder.tvMessageTime.setText("");
            }
        }


        if (getItemViewType(position) == R.layout.item_chat_response_options) {
            Context context = viewHolder.itemView.getContext();
            DefaultOptionsViewHolder holder = (DefaultOptionsViewHolder) viewHolder;
            ChatMainQuestion message = (ChatMainQuestion) objects.get(position);
            ArrayList<Object> objects;
            if (!message.forOrder) {
                holder.tvTitle.setText(message.questions.get(0).question);
                objects = new ArrayList<>((message.questions.subList(1, message.questions.size())));
            } else {
                holder.tvTitle.setText(message.question);
                objects = new ArrayList<>((message.questions));
            }
            ChatDefaultQuestionsAdapter chatDefaultQuestionsAdapter = new ChatDefaultQuestionsAdapter(objects, (position1, data) -> {
                if (recyclerViewItemClickListener != null) {
                    recyclerViewItemClickListener.onItemClick(position, data);
                }
            });
            holder.rvOptions.setLayoutManager(new LinearLayoutManager(context, RecyclerView.VERTICAL, false));
            holder.rvOptions.addItemDecoration(new DividerItemDecoration(context, DividerItemDecoration.VERTICAL));
            holder.rvOptions.setAdapter(chatDefaultQuestionsAdapter);

        }
        if (getItemViewType(position) == R.layout.item_chat_orders_snippet) {
            Context context = viewHolder.itemView.getContext();
            ChatMiniOrderSnippet chatMiniOrderSnippet = (ChatMiniOrderSnippet) objects.get(position);
            ChatOrdersSnippetViewHolder holder = (ChatOrdersSnippetViewHolder) viewHolder;
            holder.tvTitle.setText(chatMiniOrderSnippet.title);
            holder.rvOrders.setAdapter(new ChatOrdersAdapter(chatMiniOrderSnippet.orders, (position12, data) -> {
                if (recyclerViewItemClickListener != null) {
                    recyclerViewItemClickListener.onItemClick(position, data);
                }
            }));
            holder.rvOrders.setLayoutManager(new LinearLayoutManager(context, RecyclerView.VERTICAL, false));
            holder.rvOrders.addItemDecoration(new DividerItemDecoration(context, DividerItemDecoration.VERTICAL));
        }
        if (getItemViewType(position) == R.layout.item_chat_image_message_owner || getItemViewType(position) == R.layout.item_chat_image_message_nonowner) {
            Context context = viewHolder.itemView.getContext();
            Message message = (Message) objects.get(position);
            ChatImageMessageViewHolder holder = (ChatImageMessageViewHolder) viewHolder;
            if (message.is_attach == 1) {
                holder.ivMessage.setVisibility(View.VISIBLE);
                Glide.with(context).load(message.image_url).placeholder(R.drawable.banner_placeholder).error(R.drawable.banner_placeholder).into(holder.ivMessage);
            } else {
                holder.ivMessage.setVisibility(View.GONE);
            }

            String logo = "";
            int placeHolder = 0;
            if (message.admin == 1) {
                if (!Validators.isNullOrEmpty(restaurantLogo))
                    logo = restaurantLogo;
                placeHolder = R.drawable.ic_tiffin_square;
            } else if (message.admin == 0) {
                if (!Validators.isNullOrEmpty(loggedInUser.image_url))
                    logo = loggedInUser.image_url;
                placeHolder = R.drawable.user_profile_image_placeholder;
            }

            if (!Validators.isNullOrEmpty(logo)) {
                Glide.with(holder.itemView.getContext())
                        .load(logo)
                        .placeholder(placeHolder)
                        .error(placeHolder)
                        .into(holder.ivSenderImage);
            } else {
                holder.ivSenderImage.setImageResource(placeHolder);
            }
            if (!Validators.isNullOrEmpty(message.created)) {
                holder.tvMessageTime.setText(CommonFunctions.formatTimestamp(message.created.replace(":", ""), "EEE, dd MMM 'at' hh:mm a"));
            } else {
                holder.tvMessageTime.setText("");
            }
        }
        if (getItemViewType(position) == R.layout.item_chat_audio_message_owner || getItemViewType(position) == R.layout.item_chat_audio_message_nonowner) {
            Context context = viewHolder.itemView.getContext();
            Message message = (Message) objects.get(position);
            ChatAudioViewHolder holder = (ChatAudioViewHolder) viewHolder;

            Handler mSeekbarUpdateHandler = new Handler();
            Runnable mUpdateSeekbar = new Runnable() {
                @Override
                public void run() {
                    try {
                        if (holder.mPlayer != null && holder.mPlayer.isPlaying()) {
                            holder.seekBar.setProgress(holder.mPlayer.getCurrentPosition());
                            mSeekbarUpdateHandler.postDelayed(this, 50);
                        } else {
                            mSeekbarUpdateHandler.removeCallbacks(this);
                        }
                    } catch (IllegalStateException e) {
                        e.printStackTrace();
                    }
                }
            };

            holder.ivPlay.setOnClickListener(view -> {
                holder.mPlayer = new MediaPlayer();
                // Set the media player audio stream type
                holder.mPlayer.setAudioStreamType(AudioManager.STREAM_MUSIC);
                try {
                    holder.pbLoading.setVisibility(View.VISIBLE);
                    holder.ivPlay.setVisibility(View.GONE);
                    holder.ivStop.setVisibility(View.GONE);
                    holder.mPlayer.setDataSource(message.image_url);
                    holder.mPlayer.prepare();
                    holder.mPlayer.start();

                } catch (IOException e) {
                    e.printStackTrace();
                }
                holder.mPlayer.setOnCompletionListener(mediaPlayer -> {
                    mSeekbarUpdateHandler.removeCallbacks(mUpdateSeekbar);
                    holder.mPlayer.stop();
                    holder.mPlayer.release();
                    holder.ivPlay.setVisibility(View.VISIBLE);
                    holder.seekBar.setProgress(0);
                    holder.ivStop.setVisibility(View.GONE);
                });
                holder.mPlayer.setOnPreparedListener(mediaPlayer -> {
                    holder.seekBar.setMax(mediaPlayer.getDuration());
                    mSeekbarUpdateHandler.postDelayed(mUpdateSeekbar, 500);
                    holder.pbLoading.setVisibility(View.GONE);
                    holder.ivPlay.setVisibility(View.GONE);
                    holder.ivStop.setVisibility(View.VISIBLE);
                });

            });
            holder.seekBar.setOnTouchListener((view, motionEvent) -> true);
            holder.ivStop.setOnClickListener(view1 -> {
                if (holder.mPlayer != null) {
                    holder.mPlayer.stop();
                    holder.mPlayer.release();
                }
                holder.ivPlay.setVisibility(View.VISIBLE);
                holder.ivStop.setVisibility(View.GONE);
            });


            String logo = "";
            int placeHolder = 0;
            if (message.admin == 1) {
                if (!Validators.isNullOrEmpty(restaurantLogo))
                    logo = restaurantLogo;
                placeHolder = R.drawable.ic_tiffin_square;
            } else if (message.admin == 0) {
                if (!Validators.isNullOrEmpty(loggedInUser.image_url))
                    logo = loggedInUser.image_url;
                placeHolder = R.drawable.user_profile_image_placeholder;
            }

            if (!Validators.isNullOrEmpty(logo)) {
                Glide.with(holder.itemView.getContext())
                        .load(logo)
                        .placeholder(placeHolder)
                        .error(placeHolder)
                        .into(holder.ivSenderImage);
            } else {
                holder.ivSenderImage.setImageResource(placeHolder);
            }
            if (!Validators.isNullOrEmpty(message.created)) {
                holder.tvMessageTime.setText(CommonFunctions.formatTimestampZulu(message.created, "EEE, dd MMM 'at' hh:mm a"));
            } else {
                holder.tvMessageTime.setText("");
            }
        }
    }

    @Override
    public int getItemCount() {
        return objects.size();
    }

    protected static class DefaultOptionsViewHolder extends RecyclerView.ViewHolder {
        private TextView tvTitle;
        private RecyclerView rvOptions;

        public DefaultOptionsViewHolder(@NonNull View itemView) {
            super(itemView);
            tvTitle = itemView.findViewById(R.id.tvTitle);
            rvOptions = itemView.findViewById(R.id.rvQuestions);
        }
    }

    protected static class ChatOrdersSnippetViewHolder extends RecyclerView.ViewHolder {
        private TextView tvTitle;
        private RecyclerView rvOrders;

        public ChatOrdersSnippetViewHolder(@NonNull View itemView) {
            super(itemView);
            tvTitle = itemView.findViewById(R.id.tvTitle);
            rvOrders = itemView.findViewById(R.id.rvOrders);
        }
    }


    protected static class MessageViewHolder extends RecyclerView.ViewHolder {
        private TextView tvMessage, tvMessageTime;
        private CardView cvMessage;
        private LinearLayout llMessage;
        private CircularImageView ivSenderImage;

        public MessageViewHolder(@NonNull View itemView) {
            super(itemView);
            tvMessageTime = itemView.findViewById(R.id.tvMessageTime);
            tvMessage = itemView.findViewById(R.id.tvMessage);
            cvMessage = itemView.findViewById(R.id.cvMessage);
            llMessage = itemView.findViewById(R.id.llMessage);
            ivSenderImage = itemView.findViewById(R.id.ivSenderImage);
        }
    }


    protected static class ChatImageMessageViewHolder extends RecyclerView.ViewHolder {
        private ImageView ivMessage;
        private CircularImageView ivSenderImage;
        private TextView tvMessageTime;


        public ChatImageMessageViewHolder(@NonNull View itemView) {
            super(itemView);
            ivMessage = itemView.findViewById(R.id.ivMessage);
            ivSenderImage = itemView.findViewById(R.id.ivSenderImage);
            tvMessageTime = itemView.findViewById(R.id.tvMessageTime);
        }
    }

    protected static class ChatAudioViewHolder extends RecyclerView.ViewHolder {
        private ImageView ivPlay, ivStop;
        private AppCompatSeekBar seekBar;
        MediaPlayer mPlayer;
        int totalDuration = 0;
        private ProgressBar pbLoading;
        private CircularImageView ivSenderImage;
        private TextView tvMessageTime;

        public ChatAudioViewHolder(@NonNull View itemView) {
            super(itemView);
            ivPlay = itemView.findViewById(R.id.ivPlay);
            ivStop = itemView.findViewById(R.id.ivStop);
            seekBar = itemView.findViewById(R.id.seekbar);
            pbLoading = itemView.findViewById(R.id.pbLoading);
            ivSenderImage = itemView.findViewById(R.id.ivSenderImage);
            tvMessageTime = itemView.findViewById(R.id.tvMessageTime);
        }
    }
}
