package com.tiffintom.adapters;

import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.TextView;

import androidx.annotation.NonNull;
import androidx.recyclerview.widget.RecyclerView;

import com.tiffintom.R;
import com.tiffintom.common.CommonFunctions;
import com.tiffintom.interfaces.RecyclerViewItemClickListener;
import com.tiffintom.models.ChatOrder;

import java.util.ArrayList;

/**
 * Created by Amrish on 17-09-2020.
 */
public class ChatOrdersAdapter extends RecyclerView.Adapter<ChatOrdersAdapter.ChatOrderViewHolder> {
    private ArrayList<ChatOrder> chatOrders;
    private RecyclerViewItemClickListener recyclerViewItemClickListener;

    public ChatOrdersAdapter(ArrayList<ChatOrder> chatOrders, RecyclerViewItemClickListener recyclerViewItemClickListener) {
        this.chatOrders = chatOrders;
        this.recyclerViewItemClickListener = recyclerViewItemClickListener;
    }

    @NonNull
    @Override
    public ChatOrderViewHolder onCreateViewHolder(@NonNull ViewGroup parent, int viewType) {
        return new ChatOrderViewHolder(LayoutInflater.from(parent.getContext()).inflate(R.layout.item_chat_order, parent, false));
    }

    @Override
    public void onBindViewHolder(@NonNull ChatOrderViewHolder holder, int position) {
        ChatOrder chatOrder = chatOrders.get(position);
        holder.tvTitle.setText("Order from " + chatOrder.restaurant.restaurant_name);
        holder.tvOrderStatus.setText("Your order is " + chatOrder.status);
        holder.tvOrderDate.setText("Placed on " + CommonFunctions.formatDate(chatOrder.delivery_date, "EEE dd, MMM 'at'")+chatOrder.delivery_time);
        holder.itemView.setOnClickListener(view -> {
            if (recyclerViewItemClickListener != null)
                recyclerViewItemClickListener.onItemClick(position, chatOrder);
        });
    }

    @Override
    public int getItemCount() {
        return chatOrders.size();
    }

    protected static class ChatOrderViewHolder extends RecyclerView.ViewHolder {

        private TextView tvTitle, tvOrderStatus, tvOrderDate;

        public ChatOrderViewHolder(@NonNull View itemView) {
            super(itemView);
            tvTitle = itemView.findViewById(R.id.tvTitle);
            tvOrderDate = itemView.findViewById(R.id.tvOrderDate);
            tvOrderStatus = itemView.findViewById(R.id.tvOrderStatus);
        }
    }
}
