package com.tiffintom.adapters;

import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.RelativeLayout;
import android.widget.TextView;

import androidx.annotation.NonNull;
import androidx.recyclerview.widget.RecyclerView;

import com.bumptech.glide.Glide;
import com.mikhaellopez.circularimageview.CircularImageView;
import com.tiffintom.R;
import com.tiffintom.common.CommonFunctions;
import com.tiffintom.common.Validators;
import com.tiffintom.interfaces.RecyclerViewItemClickListener;
import com.tiffintom.models.Restaurant;
import com.tiffintom.models.RestaurantChatMessage;

import java.util.ArrayList;

/**
 * Created by Amrish on 18-09-2020.
 */
public class ChatRestaurantAdapter extends RecyclerView.Adapter<ChatRestaurantAdapter.RestaurantViewHolder> {


    private ArrayList<RestaurantChatMessage> restaurantChatMessages;
    private RecyclerViewItemClickListener recyclerViewItemClickListener;

    public ChatRestaurantAdapter(ArrayList<RestaurantChatMessage> restaurantChatMessages, RecyclerViewItemClickListener recyclerViewItemClickListener) {
        this.restaurantChatMessages = restaurantChatMessages;
        this.recyclerViewItemClickListener = recyclerViewItemClickListener;
    }

    @NonNull
    @Override
    public ChatRestaurantAdapter.RestaurantViewHolder onCreateViewHolder(@NonNull ViewGroup parent, int viewType) {
        return new RestaurantViewHolder(LayoutInflater.from(parent.getContext()).inflate(R.layout.item_chat_restaurant, parent, false));
    }

    @Override
    public void onBindViewHolder(@NonNull RestaurantViewHolder holder, int position) {
        RestaurantChatMessage chatMessage = restaurantChatMessages.get(position);

        holder.tvRestaurant.setText(chatMessage.restaurant.restaurant_name);
        holder.tvChatDate.setText(CommonFunctions.formatTimestampZulu(chatMessage.created, "EEE dd, MMM"));
        if (chatMessage.unread_count_restaurant > 0) {
            holder.rlChatCount.setVisibility(View.VISIBLE);
        } else {
            holder.rlChatCount.setVisibility(View.GONE);
        }
        holder.tvChatCount.setText(String.valueOf(chatMessage.unread_count_restaurant));
        if (!Validators.isNullOrEmpty(chatMessage.restaurant.image_url)) {
            Glide.with(holder.itemView.getContext())
                    .load(chatMessage.restaurant.image_url)
                    .placeholder(R.drawable.restaurant_logo_placeholder)
                    .error(R.drawable.restaurant_logo_placeholder)
                    .into(holder.ivRestaurant);
        } else {
            holder.ivRestaurant.setImageResource(R.drawable.restaurant_logo_placeholder);
        }
        holder.itemView.setOnClickListener(view -> {
            if (recyclerViewItemClickListener != null)
                recyclerViewItemClickListener.onItemClick(position, chatMessage);
        });
    }

    @Override
    public int getItemCount() {
        return restaurantChatMessages.size();
    }

    protected static class RestaurantViewHolder extends RecyclerView.ViewHolder {
        private TextView tvRestaurant, tvChatCount, tvChatDate;
        private CircularImageView ivRestaurant;
        private RelativeLayout rlChatCount;

        public RestaurantViewHolder(@NonNull View itemView) {
            super(itemView);
            tvChatCount = itemView.findViewById(R.id.tvUnreadCount);
            tvChatDate = itemView.findViewById(R.id.tvChatDate);
            tvRestaurant = itemView.findViewById(R.id.tvRestaurant);
            rlChatCount = itemView.findViewById(R.id.rlUnreadCount);
            ivRestaurant = itemView.findViewById(R.id.ivRestaurantLogo);
        }
    }
}
