package com.tiffintom.adapters;

import android.content.Context;
import android.graphics.Color;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.CheckBox;
import android.widget.ImageView;
import android.widget.LinearLayout;
import android.widget.TextView;

import androidx.annotation.NonNull;
import androidx.recyclerview.widget.RecyclerView;

import com.tiffintom.MyApp;
import com.tiffintom.R;
import com.tiffintom.common.CommonFunctions;
import com.tiffintom.interfaces.RecyclerViewItemClickListener;
import com.tiffintom.models.Address;
import com.tiffintom.models.Header;
import com.tiffintom.utils.ToastUtils;

import java.util.ArrayList;

/**
 * Created by Amrish on 04-09-2020.
 */
public class CheckoutEligibleAddressAdapter extends RecyclerView.Adapter<RecyclerView.ViewHolder> {
    ArrayList<Object> feed;
    RecyclerViewItemClickListener recyclerViewItemClickListener;
    boolean mini_view;

    public CheckoutEligibleAddressAdapter(ArrayList<Object> feed, RecyclerViewItemClickListener recyclerViewItemClickListener) {
        this.feed = feed;
        this.recyclerViewItemClickListener = recyclerViewItemClickListener;
    }

    public CheckoutEligibleAddressAdapter(ArrayList<Object> feed, boolean mini_view, RecyclerViewItemClickListener recyclerViewItemClickListener) {
        this.feed = feed;
        this.recyclerViewItemClickListener = recyclerViewItemClickListener;
        this.mini_view = mini_view;
    }

    @NonNull
    @Override
    public RecyclerView.ViewHolder onCreateViewHolder(@NonNull ViewGroup parent, int viewType) {
        View view = LayoutInflater.from(parent.getContext()).inflate(viewType, parent, false);
        if (viewType == R.layout.address_item_elibility) {
            return new AddressViewHolder(view);
        } else if (viewType == R.layout.dialog_addressbook_items) {
            return new MiniAddressViewHolder(view);
        } else {
            return new HeaderViewHolder(view);
        }
    }

    @Override
    public int getItemViewType(int position) {
        if (feed.get(position) instanceof Address) {
            if (mini_view) return R.layout.dialog_addressbook_items;
            else
                return R.layout.address_item_elibility;
        } else {
            return R.layout.cart_suggestion_header;
        }
    }

    @Override
    public void onBindViewHolder(@NonNull RecyclerView.ViewHolder viewHolder, int position) {
        if (getItemViewType(position) == R.layout.address_item_elibility) {
            Context activity = viewHolder.itemView.getContext();
            AddressViewHolder holder = (AddressViewHolder) viewHolder;
            Address address = (Address) feed.get(position);
            holder.tvTitle.setText(address.title);
            holder.tvSubtitle.setText(CommonFunctions.StringAppender(address.flat_no, ", ", address.address));
            holder.tvPostcode.setText(address.zipcode);
            holder.cvAddress.setOnClickListener(view -> {
                if (address.delivery_status.equalsIgnoreCase("delivery_available")) {
                    if (recyclerViewItemClickListener != null) {
                        MyApp.getInstance().getMyPreferences().saveCurrentAddress(address);
                        recyclerViewItemClickListener.onItemClick(position, address);
                    }
                } else {
                    ToastUtils.makeToast(activity, "Could not deliver to this address");
                }
            });
            Address defaultAddress = MyApp.getInstance().getMyPreferences().getCurrentAddress();
            if (defaultAddress != null && defaultAddress.id == address.id) {
                holder.ivSelect.setImageResource(R.drawable.ic_tiffin);
            } else {
                holder.ivSelect.setImageResource(R.drawable.dot_grey_circle);
            }
            if (address.delivery_status.equalsIgnoreCase("delivery_available")) {
                holder.cvAddress.setEnabled(true);
                holder.cvAddress.setAlpha(1);
            } else {
                holder.cvAddress.setEnabled(false);
                holder.cvAddress.setAlpha(0.2f);
            }
        }
        if (getItemViewType(position) == R.layout.dialog_addressbook_items) {
            Address address = (Address) feed.get(position);
            MiniAddressViewHolder miniview = (MiniAddressViewHolder) viewHolder;
            miniview.tvTitle.setText(address.title);
            miniview.tvSubtitle.setText(CommonFunctions.StringAppender(address.flat_no, ", ", address.address, ", ", address.zipcode));
            Address defaultAddress = MyApp.getInstance().getMyPreferences().getCurrentAddress();
            miniview.cbSelect.setChecked(defaultAddress != null && defaultAddress.id == address.id && address.delivery_status.equalsIgnoreCase("delivery_available"));
            miniview.cvAddress.setOnClickListener(view -> {
                if (address.delivery_status.equalsIgnoreCase("delivery_available")) {
                    if (recyclerViewItemClickListener != null) {
                        MyApp.getInstance().getMyPreferences().saveCurrentAddress(address);
                        miniview.cbSelect.setChecked(true);
                        notifyDataSetChanged();
                        recyclerViewItemClickListener.onItemClick(position, address);
                    }
                } else {
                    ToastUtils.makeToast(miniview.itemView.getContext(), "Could not deliver to this address");
                }

            });

            if (address.delivery_status.equalsIgnoreCase("delivery_available")) {
                miniview.cvAddress.setEnabled(true);
                miniview.cvAddress.setAlpha(1);
                miniview.cbSelect.setVisibility(View.VISIBLE);
                miniview.ivNonDeliverable.setVisibility(View.GONE);
            } else {
                miniview.cvAddress.setEnabled(false);
                miniview.cvAddress.setAlpha(0.6f);
                miniview.cbSelect.setVisibility(View.GONE);
                miniview.ivNonDeliverable.setVisibility(View.VISIBLE);
            }
        }
        if (getItemViewType(position) == R.layout.cart_suggestion_header) {
            Header header = (Header) feed.get(position);
            HeaderViewHolder holder = (HeaderViewHolder) viewHolder;
            holder.tvTitle.setTextColor(Color.parseColor("#FC0301"));
            holder.tvTitle.setText("Not deliverable locations");
            holder.tvTitle.setPadding(20, 0, 16, 0);
        }
    }

    @Override
    public int getItemCount() {
        return feed.size();
    }

    protected class HeaderViewHolder extends RecyclerView.ViewHolder {
        TextView tvTitle;

        protected HeaderViewHolder(@NonNull View itemView) {
            super(itemView);
            tvTitle = itemView.findViewById(R.id.tvTitle);
        }
    }

    protected class AddressViewHolder extends RecyclerView.ViewHolder {
        private TextView tvTitle, tvSubtitle, tvPostcode;
        private LinearLayout cvAddress;
        private ImageView ivSelect;

        protected AddressViewHolder(@NonNull View itemView) {
            super(itemView);
            tvTitle = itemView.findViewById(R.id.tvTitle);
            tvSubtitle = itemView.findViewById(R.id.tvSubtitle);
            tvPostcode = itemView.findViewById(R.id.tvPostcode);
            cvAddress = itemView.findViewById(R.id.cvAddress);
            ivSelect = itemView.findViewById(R.id.ivSelect);
        }
    }

    protected class MiniAddressViewHolder extends RecyclerView.ViewHolder {
        private TextView tvTitle, tvSubtitle;
        private LinearLayout cvAddress;
        private CheckBox cbSelect;
        private ImageView ivNonDeliverable;

        public MiniAddressViewHolder(@NonNull View itemView) {
            super(itemView);
            ivNonDeliverable = itemView.findViewById(R.id.ivNonDeliverable);
            tvTitle = itemView.findViewById(R.id.tvTitle);
            tvSubtitle = itemView.findViewById(R.id.tvSubtitle);
            cvAddress = itemView.findViewById(R.id.cvAddress);
            cbSelect = itemView.findViewById(R.id.cbSelect);
        }
    }
}
