package com.tiffintom.adapters;

import android.content.Context;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.TextView;

import androidx.annotation.NonNull;
import androidx.cardview.widget.CardView;
import androidx.core.content.ContextCompat;
import androidx.recyclerview.widget.RecyclerView;

import com.tiffintom.R;
import com.tiffintom.fragment.CuisineBottomSheetFragment;
import com.tiffintom.interfaces.RecyclerViewItemClickListener;
import com.tiffintom.models.Cuisine;
import com.tiffintom.utils.ToastUtils;

import java.util.ArrayList;

public class CuisionitemAdapter extends RecyclerView.Adapter<CuisionitemAdapter.CuisionViewHolder> {

    private ArrayList<Cuisine> cuisineArrayList;
    private Context activity;
    private RecyclerViewItemClickListener recyclerViewItemClickListener;

    public CuisionitemAdapter(Context activity, ArrayList<Cuisine> cuisineArrayList, RecyclerViewItemClickListener recyclerViewItemClickListener) {
        this.activity = activity;
        this.cuisineArrayList = cuisineArrayList;
        this.recyclerViewItemClickListener = recyclerViewItemClickListener;
    }

    @NonNull
    @Override
    public CuisionViewHolder onCreateViewHolder(@NonNull ViewGroup parent, int viewType) {
        return new CuisionViewHolder(LayoutInflater.from(parent.getContext()).inflate(R.layout.cuisine_item, parent, false));
    }

    @Override
    public void onBindViewHolder(@NonNull CuisionViewHolder holder, int position) {

        Cuisine cuisine = cuisineArrayList.get(position);
        if (cuisine.isSelected) {
            holder.tvCuision.setTextColor(ContextCompat.getColor(activity, R.color.white));
            holder.cvCuision.setCardBackgroundColor(ContextCompat.getColor(activity, R.color.black));
        } else {
            holder.tvCuision.setTextColor(ContextCompat.getColor(activity, R.color.black));
            holder.cvCuision.setCardBackgroundColor(ContextCompat.getColor(activity, R.color.white));
        }

        holder.tvCuision.setText(cuisine.cuisine_name);
        holder.cvCuision.setOnClickListener(view -> {
            if (!cuisine.isSelected) {
                if (CuisineBottomSheetFragment.selectedCuisineCount >= 5) {
                    ToastUtils.makeToast(activity, "Maximum of 5 cuisines selected");
                } else {
                    CuisineBottomSheetFragment.selectedCuisineCount++;
                    cuisine.isSelected = true;
                    notifyDataSetChanged();
                    if (recyclerViewItemClickListener != null) {
                        recyclerViewItemClickListener.onItemClick(position, cuisine);
                    }
                }
            } else {
                CuisineBottomSheetFragment.selectedCuisineCount--;
                cuisine.isSelected = false;
                notifyDataSetChanged();
                if (recyclerViewItemClickListener != null) {
                    recyclerViewItemClickListener.onItemClick(position, cuisine);
                }
            }
        });
    }

    @Override
    public int getItemCount() {
        return cuisineArrayList.size();
    }

    public class CuisionViewHolder extends RecyclerView.ViewHolder {
        private TextView tvCuision;
        private CardView cvCuision;

        public CuisionViewHolder(@NonNull View itemView) {
            super(itemView);
            tvCuision = itemView.findViewById(R.id.tvCuisine);
            cvCuision = itemView.findViewById(R.id.cvCuisine);
        }
    }
}
