package com.tiffintom.adapters;

import android.content.Context;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.ImageView;
import android.widget.RelativeLayout;
import android.widget.TextView;

import androidx.annotation.NonNull;
import androidx.recyclerview.widget.RecyclerView;

import com.tiffintom.MyApp;
import com.tiffintom.R;
import com.tiffintom.interfaces.RecyclerViewItemClickListener;
import com.tiffintom.models.DineinModels.ProductIngredients;

import java.util.ArrayList;

public class DineInIngredientsAdapter extends RecyclerView.Adapter<DineInIngredientsAdapter.ViewHolder> {
    private ArrayList<ProductIngredients> productIngredients;
    private RecyclerViewItemClickListener withItemClickListener;
    private RecyclerViewItemClickListener withoutItemClickListener;

    public DineInIngredientsAdapter(ArrayList<ProductIngredients> productIngredients, RecyclerViewItemClickListener withItemClickListener, RecyclerViewItemClickListener withoutItemClickListener) {
        this.productIngredients = productIngredients;
        this.withItemClickListener = withItemClickListener;
        this.withoutItemClickListener = withoutItemClickListener;
    }

    @NonNull
    @Override
    public ViewHolder onCreateViewHolder(@NonNull ViewGroup parent, int viewType) {
        return new ViewHolder(LayoutInflater.from(parent.getContext()).inflate(R.layout.item_ingredients, parent, false));
    }

    @Override
    public void onBindViewHolder(@NonNull ViewHolder holder, int position) {
        ProductIngredients productIngredient = productIngredients.get(position);
        holder.tvIngredientName.setText(productIngredient.ingredient_name);
        holder.tvIngredientPrice.setText(MyApp.getInstance().getCurrencySymbol() + productIngredient.price);
        holder.tvIngredientQty.setText(String.valueOf(productIngredient.quantity));
        Context context = holder.itemView.getContext();
        if (productIngredient.quantity > 0) {
            if (productIngredient.isAlreadyRemoved) {
                holder.tvRemoved.setVisibility(View.VISIBLE);
                holder.tvRemove.setVisibility(View.INVISIBLE);
                holder.rlPlusModified.setVisibility(View.VISIBLE);
                holder.rlMinusModify.setVisibility(View.INVISIBLE);
                holder.tvIngredientQty.setVisibility(View.GONE);
            } else {
                holder.tvRemoved.setVisibility(View.GONE);
                if (productIngredient.without) {
                    holder.tvRemove.setVisibility(View.VISIBLE);
                } else {
                    holder.tvRemove.setVisibility(View.INVISIBLE);
                }
                if (productIngredient.with) {
                    holder.rlPlusModified.setVisibility(View.VISIBLE);
                } else {
                    holder.rlPlusModified.setVisibility(View.INVISIBLE);
                }
                holder.rlMinusModify.setVisibility(View.VISIBLE);
                holder.tvIngredientQty.setVisibility(View.VISIBLE);
            }
        } else {
            holder.tvRemoved.setVisibility(View.GONE);
            holder.tvIngredientQty.setVisibility(View.VISIBLE);
            if (productIngredient.with) {
                holder.rlPlusModified.setVisibility(View.VISIBLE);
            } else {
                holder.rlPlusModified.setVisibility(View.INVISIBLE);
            }
            if (productIngredient.without) {
                holder.rlMinusModify.setVisibility(View.VISIBLE);
                holder.tvRemove.setVisibility(View.VISIBLE);
            } else {
                holder.rlMinusModify.setVisibility(View.INVISIBLE);
                holder.tvRemove.setVisibility(View.INVISIBLE);
            }
        }

//        if (!holder.ivMinus.isEnabled()) {
//            holder.rlMinusModify.getBackground().setTint(context.getResources().getColor(R.color.light_grey));
//        } else {
//            holder.rlMinusModify.getBackground().setTint(context.getResources().getColor(R.color.red));
//        }

        holder.rlPlusModified.setOnClickListener(v -> {
            if (productIngredient.isAlreadyRemoved) {
                productIngredient.quantity = 0;
                productIngredient.isAlreadyRemoved = false;
            } else {
                productIngredient.quantity++;
            }
            if (withoutItemClickListener != null) {
                withoutItemClickListener.onItemClick(position, productIngredient);
            }
            notifyDataSetChanged();
        });

        holder.rlMinusModify.setOnClickListener(v -> {
            if (productIngredient.quantity > 0 && !productIngredient.isAlreadyRemoved) {
                productIngredient.quantity--;
                productIngredient.isAlreadyRemoved = false;
            } else {
                // Without
                productIngredient.quantity = 1;
                productIngredient.isAlreadyRemoved = true;

            }
            if (withoutItemClickListener != null) {
                withoutItemClickListener.onItemClick(position, productIngredient);
            }
            notifyDataSetChanged();
        });

        holder.tvRemove.setOnClickListener(view -> {
            productIngredient.quantity = 1;
            holder.tvRemove.setVisibility(View.INVISIBLE);
            productIngredient.isAlreadyRemoved = true;
            notifyDataSetChanged();

            if (withoutItemClickListener != null) {
                withoutItemClickListener.onItemClick(position, productIngredient);
            }
        });

    }

    @Override
    public int getItemCount() {
        return productIngredients.size();
    }

    public class ViewHolder extends RecyclerView.ViewHolder {
        TextView tvIngredientName, tvIngredientPrice, tvRemove, tvRemoved, tvIngredientQty;
        ImageView ivMinus, ivPlus;
        RelativeLayout rlMinusModify, rlPlusModified;

        public ViewHolder(@NonNull View itemView) {
            super(itemView);

            tvRemove = itemView.findViewById(R.id.tvRemove);
            tvRemoved = itemView.findViewById(R.id.tvRemoved);
            tvIngredientName = itemView.findViewById(R.id.tvIngredientName);
            tvIngredientPrice = itemView.findViewById(R.id.tvIngredientPrice);
            ivMinus = itemView.findViewById(R.id.ivMinusModified);
            ivPlus = itemView.findViewById(R.id.ivPlusModified);
            rlMinusModify = itemView.findViewById(R.id.rlMinusModified);
            rlPlusModified = itemView.findViewById(R.id.rlPlusModified);
            tvIngredientQty = itemView.findViewById(R.id.tvIngredientQty);
        }
    }
}
