package com.tiffintom.adapters;

import android.content.Context;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.TextView;

import androidx.annotation.NonNull;
import androidx.cardview.widget.CardView;
import androidx.core.content.ContextCompat;
import androidx.recyclerview.widget.RecyclerView;

import com.tiffintom.MyApp;
import com.tiffintom.R;
import com.tiffintom.interfaces.RecyclerViewItemClickListener;
import com.tiffintom.models.DineinModels.Addon;

import java.util.ArrayList;

public class DineInSubAddonAdapter extends RecyclerView.Adapter<DineInSubAddonAdapter.ViewHolder> {

    private final ArrayList<Addon> subAddons;
    private final RecyclerViewItemClickListener itemClickListener;

    public DineInSubAddonAdapter(ArrayList<Addon> subAddons, RecyclerViewItemClickListener itemClickListener) {
        this.subAddons = subAddons;
        this.itemClickListener = itemClickListener;
    }

    @NonNull
    @Override
    public ViewHolder onCreateViewHolder(@NonNull ViewGroup parent, int viewType) {
        return new ViewHolder(LayoutInflater.from(parent.getContext()).inflate(R.layout.item_subaddon, parent, false));
    }

    @Override
    public void onBindViewHolder(@NonNull ViewHolder holder, int position) {
        Addon addon = subAddons.get(position);
        Context context = holder.itemView.getContext();
        holder.tvSubAddon.setText(addon.name);
        holder.tvPrice.setText(MyApp.getInstance().getCurrencySymbol() + MyApp.df.format(addon.price));

        if (addon.selected) {
            holder.cvSubAddon.setCardBackgroundColor(ContextCompat.getColor(context, R.color.colorPrimary));
            holder.tvSubAddon.setTextColor(ContextCompat.getColor(context, R.color.white));
            holder.tvPrice.setTextColor(ContextCompat.getColor(context, R.color.white));
        } else {
            holder.cvSubAddon.setCardBackgroundColor(ContextCompat.getColor(holder.itemView.getContext(), R.color.light_grey));
            holder.tvSubAddon.setTextColor(ContextCompat.getColor(context, R.color.black));
            holder.tvPrice.setTextColor(ContextCompat.getColor(context, R.color.black));
        }

        holder.cvSubAddon.setOnClickListener(v -> {
            if (itemClickListener != null) {
                itemClickListener.onItemClick(position, addon);
            }
        });
    }

    @Override
    public int getItemCount() {
        return subAddons.size();
    }

    public class ViewHolder extends RecyclerView.ViewHolder {
        private TextView tvSubAddon, tvPrice;
        private CardView cvSubAddon;

        public ViewHolder(@NonNull View itemView) {
            super(itemView);
            tvSubAddon = itemView.findViewById(R.id.tvSubAddon);
            tvPrice = itemView.findViewById(R.id.tvPrice);
            cvSubAddon = itemView.findViewById(R.id.cvSubAddon);
        }
    }
}
