package com.tiffintom.adapters;

import android.app.Activity;
import android.content.Context;
import android.graphics.PorterDuff;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.ImageView;
import android.widget.TextView;

import androidx.annotation.NonNull;
import androidx.cardview.widget.CardView;
import androidx.core.content.ContextCompat;
import androidx.recyclerview.widget.GridLayoutManager;
import androidx.recyclerview.widget.RecyclerView;

import com.tiffintom.MyApp;
import com.tiffintom.R;
import com.tiffintom.interfaces.RecyclerViewItemClickListener;
import com.tiffintom.models.DineinModels.Addon;
import com.tiffintom.models.DineinModels.OrderItemAddon;
import com.tiffintom.models.DineinModels.ProductAddons;
import com.tiffintom.utils.LogUtils;
import com.tiffintom.utils.ToastUtils;

import java.util.ArrayList;

/**
 * Created by Aradhna on 28-12-2021.
 */
public class DineinAddonsAdapter extends RecyclerView.Adapter<DineinAddonsAdapter.ViewHolder> {
    private ArrayList<ProductAddons> productAddons;
    private RecyclerViewItemClickListener recyclerviewItemClickListener, subAddonClickListener;
    private RecyclerView.RecycledViewPool viewPool;
    public int expandedPosition = -1;
    private String orderTypeId;
    private Activity activity;

    public DineinAddonsAdapter(Activity activity, ArrayList<ProductAddons> productAddons, String orderTypeId, RecyclerViewItemClickListener recyclerviewItemClickListener, RecyclerViewItemClickListener subAddonClickListener) {
        this.activity = activity;
        this.productAddons = productAddons;
        this.orderTypeId = orderTypeId;
        this.recyclerviewItemClickListener = recyclerviewItemClickListener;
        this.subAddonClickListener = subAddonClickListener;
        viewPool = new RecyclerView.RecycledViewPool();
    }

    @NonNull
    @Override
    public ViewHolder onCreateViewHolder(@NonNull ViewGroup parent, int viewType) {
        return new ViewHolder(LayoutInflater.from(parent.getContext()).inflate(R.layout.item_product_addon, parent, false));
    }

    @Override
    public void onBindViewHolder(@NonNull ViewHolder holder, int position) {
        ProductAddons productAddon = productAddons.get(position);
        Addon addon = productAddon.addon;
        Context context = holder.itemView.getContext();
        if (expandedPosition == position) {
            holder.rvSubAddons.setVisibility(View.VISIBLE);
            holder.ivCollapseImage.setRotation(270);
            holder.tvAddonName.setTextColor(ContextCompat.getColor(context, R.color.colorPrimary));
            holder.ivCollapseImage.setColorFilter(ContextCompat.getColor(context, R.color.colorPrimary), PorterDuff.Mode.SRC_IN);
            holder.tvMaxQty.setTextColor(ContextCompat.getColor(context, R.color.colorPrimary));
        } else {
            holder.rvSubAddons.setVisibility(View.GONE);
            holder.ivCollapseImage.setRotation(90);
            holder.tvAddonName.setTextColor(ContextCompat.getColor(context, R.color.black));
            holder.ivCollapseImage.setColorFilter(ContextCompat.getColor(context, R.color.black), PorterDuff.Mode.SRC_IN);
            holder.tvMaxQty.setTextColor(ContextCompat.getColor(context, R.color.black));
        }
        productAddon.category = "header";
        if (addon != null) {
            if (productAddon.min_quantity > 0) {
                holder.tvAddonName.setText(addon.name + " (Required)");
                holder.tvMaxQty.setText("Max. " + productAddon.max_quantity);
                holder.tvMaxQty.setVisibility(View.VISIBLE);
            } else {
                holder.tvAddonName.setText(addon.name + " (Optional)");
                holder.tvMaxQty.setVisibility(View.INVISIBLE);
            }
            holder.subAddons.clear();
            if (addon.children != null && addon.children.size() > 0) {
                holder.subAddons.addAll(addon.children);
            }
            holder.subAddonAdapter.notifyDataSetChanged();
            holder.rvSubAddons.setRecycledViewPool(viewPool);
        } else {
            LogUtils.e("Addon null");
        }

        holder.tvAddonName.setOnClickListener(view -> {
            if (expandedPosition == position) {
                expandedPosition = -1;
            } else {
                expandedPosition = position;
            }
            notifyDataSetChanged();
        });
        holder.ivCollapseImage.setOnClickListener(view -> {
            if (expandedPosition == position) {
                expandedPosition = -1;
            } else {
                expandedPosition = position;
            }
            notifyDataSetChanged();
        });
        holder.subAddonListener = (subAddonPosition, data) -> {
            Addon subAddon = (Addon) data;
            subAddon.selected = !subAddon.selected;
            if (subAddon.selected) {
                if (productAddon.selectedQuantity >= productAddon.max_quantity) {
                    subAddon.selected = false;
                    ToastUtils.makeLongToast(activity, "You can select maximum " + productAddon.max_quantity + " addons");
                    holder.subAddonAdapter.notifyDataSetChanged();
                    return;
                }
            }

            if (!subAddon.selected) {
                productAddon.selectedQuantity = productAddon.selectedQuantity - 1;
                productAddons.get(position).selectedQuantity = productAddon.selectedQuantity;

                OrderItemAddon orderItemAddon = new OrderItemAddon();
                orderItemAddon.addon_id = subAddon.id;
                productAddon.selectedOrderItemAddon.remove(orderItemAddon);
            } else {
                productAddon.selectedQuantity = productAddon.selectedQuantity + 1;
                productAddons.get(position).selectedQuantity = productAddon.selectedQuantity;
                subAddon.selected_price = MyApp.calculatedAddonPrice(orderTypeId, subAddon);
                OrderItemAddon orderItemAddon = new OrderItemAddon();
                orderItemAddon.addon_id = subAddon.id;
                orderItemAddon.addon_name = subAddon.name;
                orderItemAddon.price = subAddon.selected_price;
                orderItemAddon.quantity = 1;
                orderItemAddon.total = orderItemAddon.price * orderItemAddon.quantity;
                orderItemAddon.updater_id = MyApp.getInstance().getMyPreferences().getDineinUser().id;
                productAddon.selectedOrderItemAddon.add(orderItemAddon);
                if (productAddon.selectedQuantity == productAddon.max_quantity && (position + 1) < getItemCount()) {
                    expandedPosition = expandedPosition + 1;
                }
            }
            notifyDataSetChanged();
            holder.subAddonAdapter.notifyDataSetChanged();
        };
    }

    @Override
    public int getItemCount() {
        return productAddons.size();
    }

    public class ViewHolder extends RecyclerView.ViewHolder {
        private TextView tvAddonName, tvMaxQty;
        private ImageView ivCollapseImage;
        private RecyclerView rvSubAddons;
        private DineInSubAddonAdapter subAddonAdapter;
        private CardView cvMainAddon;
        private ArrayList<Addon> subAddons = new ArrayList<>();//Maybe this subaddon will differ
        private RecyclerViewItemClickListener subAddonListener;

        public ViewHolder(@NonNull View itemView) {
            super(itemView);
            cvMainAddon = itemView.findViewById(R.id.cvMainAddon);
            rvSubAddons = itemView.findViewById(R.id.rvSubAddons);
            tvAddonName = itemView.findViewById(R.id.tvAddonName);
            ivCollapseImage = itemView.findViewById(R.id.ivCollapseIcon);
            tvMaxQty = itemView.findViewById(R.id.tvMax);

            subAddonAdapter = new DineInSubAddonAdapter(subAddons, (position, data) -> {
                if (subAddonListener != null) subAddonListener.onItemClick(position, data);
            });
            rvSubAddons.setAdapter(subAddonAdapter);
            rvSubAddons.setLayoutManager(new GridLayoutManager(itemView.getContext(), 4));
        }
    }
}
