package com.tiffintom.adapters;

import android.app.Activity;
import android.content.Context;
import android.content.Intent;
import android.graphics.Paint;
import android.view.LayoutInflater;
import android.view.MotionEvent;
import android.view.View;
import android.view.ViewGroup;
import android.widget.ImageView;
import android.widget.TextView;

import androidx.annotation.NonNull;
import androidx.recyclerview.widget.LinearLayoutManager;
import androidx.recyclerview.widget.RecyclerView;

import com.iarcuschin.simpleratingbar.SimpleRatingBar;
import com.tiffintom.MyApp;
import com.tiffintom.R;
import com.tiffintom.activity.TransportActivity;
import com.tiffintom.common.CommonFunctions;
import com.tiffintom.common.Validators;
import com.tiffintom.interfaces.RecyclerViewItemClickListener;
import com.tiffintom.models.DineinModels.CreatedOrder;
import com.tiffintom.models.DineinModels.OrderItem;
import com.tiffintom.models.DineinModels.OrderItemAddon;
import com.tiffintom.models.DineinModels.OrderItemIngredient;
import com.tiffintom.models.HistoryCart;

import java.text.DecimalFormat;
import java.util.ArrayList;
import java.util.Date;

/**
 * Created by Amrish on 07-09-2020.
 */
public class DineinOrderHistoryAdapter extends RecyclerView.Adapter<RecyclerView.ViewHolder> {
    private Activity activity;
    private ArrayList<Object> feed;
    private RecyclerViewItemClickListener recyclerViewItemClickListenerReceipt, recyclerViewItemClickListenerReOrder, recyclerViewItemClickListenerAddReview;

    public DineinOrderHistoryAdapter(Activity activity, ArrayList<Object> feed, RecyclerViewItemClickListener viewReceiptClickListener, RecyclerViewItemClickListener reOrderClickListener, RecyclerViewItemClickListener recyclerViewItemClickListenerAddReview) {
        this.feed = feed;
        this.activity = activity;
        this.recyclerViewItemClickListenerReceipt = viewReceiptClickListener;
        this.recyclerViewItemClickListenerReOrder = reOrderClickListener;
        this.recyclerViewItemClickListenerAddReview = recyclerViewItemClickListenerAddReview;
    }

    @Override
    public int getItemViewType(int position) {
        if (feed.get(position) instanceof CreatedOrder) {
            return R.layout.dinein_order_history_item;
        }
        return super.getItemViewType(position);
    }

    @NonNull
    @Override
    public RecyclerView.ViewHolder onCreateViewHolder(@NonNull ViewGroup parent, int viewType) {
        View view = LayoutInflater.from(parent.getContext()).inflate(viewType, parent, false);
        return new OrderViewHolder(view);
    }

    @Override
    public void onBindViewHolder(@NonNull RecyclerView.ViewHolder viewHolder, int position) {

        try {
            Context context = viewHolder.itemView.getContext();
            if (getItemViewType(position) == R.layout.dinein_order_history_item) {
                OrderViewHolder holder = (OrderViewHolder) viewHolder;
                CreatedOrder orderHistory = (CreatedOrder) feed.get(position);
    //            if (orderHistory.restaurant != null)
    //                holder.tvRestaurant.setText(orderHistory.restaurant.restaurant_name);
                holder.tvOrderId.setPaintFlags(Paint.UNDERLINE_TEXT_FLAG);
                holder.tvOrderId.setText("Order #" + orderHistory.id);
                DecimalFormat df = new DecimalFormat("#.##");
                holder.tvTotal.setText("Total: " + MyApp.getInstance().getCurrencySymbol() + MyApp.df.format(orderHistory.total));//+ orderHistory.charity_amount));
                holder.tvOrderDate.setText(CommonFunctions.formatTimestampZulu(orderHistory.created_at, "dd MMM yyyy 'at' HH:mm a"));
                // holder.ratingBar.setRating(orderHistory.rating);
                StringBuilder stringBuilder = new StringBuilder();
                if (!Validators.isNullOrEmpty(orderHistory.table_number))
                    stringBuilder.append("Table No ").append(orderHistory.table_number);

                if (!Validators.isNullOrEmpty(orderHistory.table_number) &&!orderHistory.no_guest.equalsIgnoreCase( "0"))
                    stringBuilder.append(", ");

                if (!orderHistory.no_guest.equalsIgnoreCase( "0"))
                    stringBuilder.append("Total Guests " + orderHistory.no_guest);

                if (Validators.isNullOrEmpty(stringBuilder.toString())) {
                    holder.tvTableNumberAndGuests.setVisibility(View.GONE);
                } else {
                    holder.tvTableNumberAndGuests.setVisibility(View.VISIBLE);
                    holder.tvTableNumberAndGuests.setText(stringBuilder);
                }


                holder.cartHistory.clear();
                if (orderHistory.order_items != null) {
                    float total_amount = 0;

                    for (OrderItem item : orderHistory.order_items) {
                        StringBuilder selectedProductAddons = new StringBuilder();

                        HistoryCart cart = new HistoryCart();
                        cart.menu_name = item.product_name;
                        cart.quantity = item.quantity;

                        if (item.order_item_addons.size() > 0) {
                            selectedProductAddons.append("(");
                            for (OrderItemAddon orderItemAddon : item.order_item_addons) {
                                if (selectedProductAddons.length() > 0) {
                                    selectedProductAddons.append("(");
                                }
                                selectedProductAddons.append(orderItemAddon.addon_name).append(": ").append(MyApp.getInstance().getCurrencySymbol()).append(MyApp.df.format(orderItemAddon.price));
                                if (selectedProductAddons.length() > 0) {
                                    selectedProductAddons.append(")");
                                }
                                total_amount = total_amount + orderItemAddon.total;
                            }
                            selectedProductAddons.append(")");
                        }

                        if (item.order_item_ingredients.size() > 0) {
                            if (item.order_item_addons.size() > 0) {
                                selectedProductAddons.append(",");
                            }
                            selectedProductAddons.append("(");
                            for (OrderItemIngredient orderItemIngredient : item.order_item_ingredients) {
                                if (selectedProductAddons.length() > 0) {
                                    selectedProductAddons.append("(");
                                }
                                if (orderItemIngredient.with) {
                                    selectedProductAddons.append("Extra ");
                                    selectedProductAddons.append(orderItemIngredient.quantity).append(" ");
                                }
                                if (orderItemIngredient.without) {
                                    selectedProductAddons.append("No ");
                                }
                                selectedProductAddons.append(orderItemIngredient.ingredient_name).append(": ").append(MyApp.getInstance().getCurrencySymbol()).append(MyApp.df.format(orderItemIngredient.total));
                                if (selectedProductAddons.length() > 0) {
                                    selectedProductAddons.append(")");
                                }
                                total_amount = total_amount + orderItemIngredient.total;
                            }
                            selectedProductAddons.append(")");
                        }
                        if (!(Validators.isNullOrEmpty(item.special_instruction)) && (item.instruction_price > 0)) {
                            if (selectedProductAddons.length() > 0) {
                                selectedProductAddons.append(",(");
                            } else {
                                selectedProductAddons.append("(");
                            }
                            selectedProductAddons.append(item.special_instruction).append(": ").append(MyApp.getInstance().getCurrencySymbol()).append(MyApp.df.format(item.instruction_price));
                            if (selectedProductAddons.length() > 0) {
                                selectedProductAddons.append(")");
                            }
                        }
                        cart.subaddons_name = selectedProductAddons.toString();
                        item.addon_name = selectedProductAddons.toString();
                        cart.total_price = item.total;
                        holder.cartHistory.add(cart);
                    }

                    orderHistory.addon_total = total_amount;
                }
                //   holder.cartHistory.addAll(orderHistory.order_items);
                holder.adapter.notifyDataSetChanged();

                holder.rvCartHistory.addOnItemTouchListener(new RecyclerView.OnItemTouchListener() {
                    @Override
                    public boolean onInterceptTouchEvent(@NonNull RecyclerView rv, @NonNull MotionEvent e) {
                        return false;
                    }

                    @Override
                    public void onTouchEvent(@NonNull RecyclerView rv, @NonNull MotionEvent e) {

                    }

                    @Override
                    public void onRequestDisallowInterceptTouchEvent(boolean disallowIntercept) {

                    }
                });

    //            if (orderHistory.promotionList != null && orderHistory.promotionList.size() > 0 && !Validators.isNullOrEmpty(orderHistory.promotionList.get(0).promo_image)) {
    //                String bannerImage = orderHistory.promotionList.get(0).image_url;
    //                Glide.with(context)
    //                        .load(bannerImage)
    //                        .placeholder(R.drawable.restaurant_banner_placeholder)
    //                        .error(R.drawable.restaurant_banner_placeholder)
    //                        .into(holder.ivRestaurant);
    //            } else {
    //                Glide.with(context).load(R.drawable.restaurant_banner_placeholder).into(holder.ivRestaurant);
    //            }

                holder.tvReceipt.setOnClickListener(view -> {
                    Intent intent = new Intent(context, TransportActivity.class);
                    intent.putExtra("destination", "dineinOrderView");
                    intent.putExtra("hideToolbar", true);
                    intent.putExtra("is_dinein", true);
                    intent.putExtra("order_id", String.valueOf(orderHistory.id));
                    context.startActivity(intent);
                });

                holder.tvTrackOrder.setOnClickListener(view -> {
                    Intent intent = new Intent(context, TransportActivity.class);
                    intent.putExtra("destination", "trackOrder");
                    intent.putExtra("hideToolbar", true);
                    intent.putExtra("order_id", String.valueOf(orderHistory.id));
                    context.startActivity(intent);
                });


                holder.tvAddreview.setOnClickListener(view -> {
                    if (recyclerViewItemClickListenerAddReview != null) {
                        recyclerViewItemClickListenerAddReview.onItemClick(position, orderHistory);
                    }
                });
                holder.tvViewOrder.setOnClickListener(view -> {
                    if (recyclerViewItemClickListenerReOrder != null) {
                        recyclerViewItemClickListenerReOrder.onItemClick(position, orderHistory);
                    }
                });

                if (orderHistory.status.equalsIgnoreCase("completed")) {
                    holder.tvReceipt.setVisibility(View.VISIBLE);
                    holder.tvViewOrder.setVisibility(View.GONE);
                } else {
                    holder.tvReceipt.setVisibility(View.GONE);
                    holder.tvViewOrder.setVisibility(View.VISIBLE);
                }
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
    }


    @Override
    public int getItemCount() {
        return feed.size();
    }

    protected class OrderViewHolder extends RecyclerView.ViewHolder {
        private TextView tvOrderDate, tvOrderId, tvRestaurant, tvTableNumberAndGuests, tvTotal, tvAddreview;
        private ImageView ivRestaurant;
        private SimpleRatingBar ratingBar, reviewRating;
        private View vReservation;
        private TextView tvViewOrder, tvReceipt, tvTrackOrder;
        private RecyclerView rvCartHistory;
        private OrderHistoryCartAdapter adapter;
        private ArrayList<HistoryCart> cartHistory = new ArrayList<>();

        public OrderViewHolder(@NonNull View itemView) {
            super(itemView);
            tvRestaurant = itemView.findViewById(R.id.tvRestaurant);
            tvOrderId = itemView.findViewById(R.id.tvOrderId);
            ivRestaurant = itemView.findViewById(R.id.ivRestaurantBanner);
            tvViewOrder = itemView.findViewById(R.id.tvReOrder);
            tvReceipt = itemView.findViewById(R.id.tvViewReceipt);
            tvTrackOrder = itemView.findViewById(R.id.tvTrackOrder);
            tvOrderDate = itemView.findViewById(R.id.tvOrderDate);
            tvTableNumberAndGuests = itemView.findViewById(R.id.tvTableNumberAndGuests);
            tvTotal = itemView.findViewById(R.id.tvTotal);
            ratingBar = itemView.findViewById(R.id.rating);
            rvCartHistory = itemView.findViewById(R.id.rvCartItems);
            tvAddreview = itemView.findViewById(R.id.tvAddReview);
            reviewRating = itemView.findViewById(R.id.userRating);

            adapter = new OrderHistoryCartAdapter(cartHistory);
            LinearLayoutManager linearLayoutManager = new LinearLayoutManager(itemView.getContext(), RecyclerView.VERTICAL, false);
            rvCartHistory.setLayoutManager(linearLayoutManager);
            rvCartHistory.setRecycledViewPool(new RecyclerView.RecycledViewPool());
            rvCartHistory.setHasFixedSize(true);
            adapter.setHasStableIds(true);

            rvCartHistory.setNestedScrollingEnabled(false);
            rvCartHistory.setAdapter(adapter);
        }
    }

    public static int getPercentageLeft(Date start, Date end) {
        long now = System.currentTimeMillis();
        long s = start.getTime();
        long e = end.getTime();
        if (s >= e || now >= e) {
            return 0;
        }
        if (now <= s) {
            return 100;
        }
        return (int) ((e - now) * 100 / (e - s));
    }
}
