package com.tiffintom.adapters;

import android.content.Context;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.ImageView;
import android.widget.TextView;

import androidx.annotation.NonNull;
import androidx.cardview.widget.CardView;
import androidx.recyclerview.widget.RecyclerView;

import com.bumptech.glide.Glide;
import com.tiffintom.R;
import com.tiffintom.common.Validators;
import com.tiffintom.interfaces.RecyclerViewItemClickListener;
import com.tiffintom.models.Cuisine;

import java.util.ArrayList;

/**
 * Created by Amrish on 19-08-2020.
 */
public class ExploreCuisinesAdapter extends RecyclerView.Adapter<ExploreCuisinesAdapter.CategoryViewHolder> {
    private ArrayList<Cuisine> cuisines = new ArrayList<>();
    private RecyclerViewItemClickListener onCategoryClick;

    public ExploreCuisinesAdapter(ArrayList<Cuisine> cuisines, RecyclerViewItemClickListener onCategoryClick) {
        this.cuisines = cuisines;
        this.onCategoryClick = onCategoryClick;
    }

    @NonNull
    @Override
    public ExploreCuisinesAdapter.CategoryViewHolder onCreateViewHolder(@NonNull ViewGroup parent, int viewType) {
        return new CategoryViewHolder(LayoutInflater.from(parent.getContext()).inflate(R.layout.explore_category_item, parent, false));
    }

    @Override
    public void onBindViewHolder(@NonNull ExploreCuisinesAdapter.CategoryViewHolder holder, int position) {
        Context context = holder.itemView.getContext();
        Cuisine cuisine = cuisines.get(position);
        holder.tvCategory.setText(cuisine.cuisine_name);
        if (!Validators.isNullOrEmpty(cuisine.image_url)) {
            Glide.with(context).load(cuisine.image_url).centerCrop().placeholder(R.drawable.restaurant_banner_placeholder).error(R.drawable.restaurant_banner_placeholder).into(holder.ivCategory);
        } else {
            Glide.with(context).load(R.drawable.restaurant_banner_placeholder).into(holder.ivCategory);
        }
        holder.cvCategory.setOnClickListener(view -> {
            if (onCategoryClick != null) onCategoryClick.onItemClick(position, cuisine);
        });
    }

    @Override
    public int getItemCount() {
        return cuisines.size();
    }

    protected static class CategoryViewHolder extends RecyclerView.ViewHolder {
        private TextView tvCategory;
        private ImageView ivCategory;
        private CardView cvCategory;

        public CategoryViewHolder(@NonNull View itemView) {
            super(itemView);
            tvCategory = itemView.findViewById(R.id.tvCategory);
            ivCategory = itemView.findViewById(R.id.ivCategory);
            cvCategory = itemView.findViewById(R.id.cvCuisine);
        }
    }
}
