package com.tiffintom.adapters;

import android.content.Context;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.ImageView;
import android.widget.TextView;

import androidx.annotation.NonNull;
import androidx.cardview.widget.CardView;
import androidx.recyclerview.widget.RecyclerView;

import com.bumptech.glide.Glide;
import com.iarcuschin.simpleratingbar.SimpleRatingBar;
import com.tiffintom.R;
import com.tiffintom.common.Validators;
import com.tiffintom.interfaces.RecyclerViewItemClickListener;
import com.tiffintom.models.Restaurant;

import java.util.ArrayList;

/**
 * Created by Amrish on 19-08-2020.
 */
public class ExploreNearByRestaurantAdapter extends RecyclerView.Adapter<ExploreNearByRestaurantAdapter.RestaurantViewHolder> {
    private ArrayList<Restaurant> restaurants;
    private RecyclerViewItemClickListener restaurantClickListener;

    public ExploreNearByRestaurantAdapter(ArrayList<Restaurant> restaurants, RecyclerViewItemClickListener restaurantClickListener) {
        this.restaurants = restaurants;
        this.restaurantClickListener = restaurantClickListener;
    }

    @NonNull
    @Override
    public ExploreNearByRestaurantAdapter.RestaurantViewHolder onCreateViewHolder(@NonNull ViewGroup parent, int viewType) {
        return new RestaurantViewHolder(LayoutInflater.from(parent.getContext()).inflate(R.layout.explore_nearby_item, parent, false));
    }

    @Override
    public void onBindViewHolder(@NonNull ExploreNearByRestaurantAdapter.RestaurantViewHolder holder, int position) {
        Restaurant restaurant = restaurants.get(position);
        Context context = holder.itemView.getContext();
        holder.tvRestaurant.setText(restaurant.restaurant_name);
        holder.ratingBar.setRating(restaurant.average_rating);
        if (restaurant.promotions != null && restaurant.promotions.size() > 0 && !Validators.isNullOrEmpty(restaurant.promotions.get(0).promo_image)) {
            String bannerImage = restaurant.promotions.get(0).image_url;
            Glide.with(context)
                    .load(bannerImage)
                    .placeholder(R.drawable.restaurant_banner_placeholder)
                    .error(R.drawable.restaurant_banner_placeholder)
                    .into(holder.ivRestaurant);
        } else {
            Glide.with(context).load(R.drawable.restaurant_banner_placeholder).into(holder.ivRestaurant);
        }

    }

    @Override
    public int getItemCount() {
        return restaurants.size();
    }

    protected class RestaurantViewHolder extends RecyclerView.ViewHolder {
        private TextView tvRestaurant;
        private ImageView ivRestaurant;
        private SimpleRatingBar ratingBar;

        public RestaurantViewHolder(@NonNull View itemView) {
            super(itemView);
            tvRestaurant = itemView.findViewById(R.id.tvRestaurant);
            ivRestaurant = itemView.findViewById(R.id.ivRestaurant);
            ratingBar = itemView.findViewById(R.id.rating);
            CardView cvRestaurant = itemView.findViewById(R.id.cvRestaurant);
            cvRestaurant.setOnClickListener(view -> {
                if (restaurantClickListener != null) {
                    restaurantClickListener.onItemClick(getLayoutPosition(), restaurants.get(getLayoutPosition()));
                }
            });
        }
    }
}
