package com.tiffintom.adapters;

import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.RelativeLayout;
import android.widget.TextView;

import androidx.annotation.NonNull;
import androidx.recyclerview.widget.RecyclerView;

import com.tiffintom.R;
import com.tiffintom.interfaces.RecyclerViewItemClickListener;
import com.tiffintom.models.ChatCount;
import com.tiffintom.models.FAQ;
import com.tiffintom.models.FAQCategory;
import com.tiffintom.models.Header;

import java.util.ArrayList;

/**
 * Created by Amrish on 10-09-2020.
 */
public class FaqAdapter extends RecyclerView.Adapter<RecyclerView.ViewHolder> {
    private ArrayList<Object> objects;
    private RecyclerViewItemClickListener recyclerViewItemClickListener;
    private int FAQ = 1, CATEGORY = 0, STATIC_OPTIONS = 2;
    private ChatCount chatCount;

    public FaqAdapter(ArrayList<Object> objects, RecyclerViewItemClickListener recyclerViewItemClickListener) {
        this.objects = objects;
        this.recyclerViewItemClickListener = recyclerViewItemClickListener;
    }


    @Override
    public int getItemViewType(int position) {
        if (objects.get(position) instanceof FAQ) {
            return FAQ;
        } else if (objects.get(position) instanceof FAQCategory) {
            return CATEGORY;
        } else {
            return STATIC_OPTIONS;
        }
    }

    @NonNull
    @Override
    public RecyclerView.ViewHolder onCreateViewHolder(@NonNull ViewGroup parent, int viewType) {
        View view = LayoutInflater.from(parent.getContext()).inflate(R.layout.item_faq, parent, false);
        if (viewType == FAQ) {
            return new FaqViewHolder(view);
        } else if (viewType == CATEGORY) {
            return new FaqCategoryViewHolder(view);
        } else {
            return new StaticOptionsViewHolder(view);
        }
    }

    @Override
    public void onBindViewHolder(@NonNull RecyclerView.ViewHolder viewHolder, int position) {
        if (getItemViewType(position) == FAQ) {
            FaqViewHolder holder = (FaqViewHolder) viewHolder;
            com.tiffintom.models.FAQ faq = (com.tiffintom.models.FAQ) objects.get(position);
            holder.tvTitle.setText(faq.question);
            holder.itemView.setOnClickListener(view -> {
                if (recyclerViewItemClickListener != null)
                    recyclerViewItemClickListener.onItemClick(position, objects.get(position));
            });
        }
        if (getItemViewType(position) == CATEGORY) {
            FaqCategoryViewHolder holder = (FaqCategoryViewHolder) viewHolder;
            FAQCategory category = (FAQCategory) objects.get(position);
            holder.tvTitle.setText(category.name);
            holder.itemView.setOnClickListener(view -> {
                if (recyclerViewItemClickListener != null)
                    recyclerViewItemClickListener.onItemClick(position, objects.get(position));
            });
        }
        if (getItemViewType(position) == STATIC_OPTIONS) {
            StaticOptionsViewHolder holder = (StaticOptionsViewHolder) viewHolder;
            Header category = (Header) objects.get(position);
            holder.tvTitle.setText(category.title);

            if (category.count > 0) {
                holder.rlUnreadChatCount.setVisibility(View.VISIBLE);
                holder.tvUnreadChatCount.setText(String.valueOf(category.count));
            } else {
                holder.rlUnreadChatCount.setVisibility(View.GONE);

            }

            holder.itemView.setOnClickListener(view -> {
                if (recyclerViewItemClickListener != null)
                    recyclerViewItemClickListener.onItemClick(position, objects.get(position));
            });
        }


    }

    @Override
    public int getItemCount() {
        return objects.size();
    }

    protected static class FaqViewHolder extends RecyclerView.ViewHolder {
        private TextView tvTitle;

        public FaqViewHolder(@NonNull View itemView) {
            super(itemView);
            tvTitle = itemView.findViewById(R.id.tvTitle);
        }
    }

    protected static class FaqCategoryViewHolder extends RecyclerView.ViewHolder {
        private TextView tvTitle;

        public FaqCategoryViewHolder(@NonNull View itemView) {
            super(itemView);
            tvTitle = itemView.findViewById(R.id.tvTitle);
        }
    }

    private class StaticOptionsViewHolder extends RecyclerView.ViewHolder {
        private TextView tvTitle, tvUnreadChatCount;
        private RelativeLayout rlUnreadChatCount;

        public StaticOptionsViewHolder(View view) {
            super(view);
            tvTitle = view.findViewById(R.id.tvTitle);
            rlUnreadChatCount = view.findViewById(R.id.rlUnreadCount);
            tvUnreadChatCount = view.findViewById(R.id.tvUnreadCount);
        }
    }
}
