package com.tiffintom.adapters;

import android.app.Activity;
import android.content.Intent;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.ImageView;
import android.widget.LinearLayout;
import android.widget.TextView;

import androidx.annotation.NonNull;
import androidx.cardview.widget.CardView;
import androidx.recyclerview.widget.RecyclerView;

import com.airbnb.lottie.LottieAnimationView;
import com.bumptech.glide.Glide;
import com.iarcuschin.simpleratingbar.SimpleRatingBar;
import com.tiffintom.MyApp;
import com.tiffintom.R;
import com.tiffintom.activity.AuthActivity;
import com.tiffintom.common.CommonFunctions;
import com.tiffintom.common.Validators;
import com.tiffintom.interfaces.RecyclerViewItemClickListener;
import com.tiffintom.models.FavouriteRestaurant;

import java.util.ArrayList;

/**
 * Created by Amrish on 19-08-2020.
 */
public class FavouriteRestaurantAdapter extends RecyclerView.Adapter<FavouriteRestaurantAdapter.RestaurantViewHolder> {
    private Activity activity;
    private ArrayList<FavouriteRestaurant> restaurants = new ArrayList<>();
    private RecyclerViewItemClickListener restaurantClickListener, favouriteClickListener;

    public FavouriteRestaurantAdapter(Activity activity, ArrayList<FavouriteRestaurant> restaurants, RecyclerViewItemClickListener favouriteClickListener, RecyclerViewItemClickListener restaurantClickListener) {
        this.activity = activity;
        this.restaurants = restaurants;
        this.favouriteClickListener = favouriteClickListener;
        this.restaurantClickListener = restaurantClickListener;
    }

    @NonNull
    @Override
    public FavouriteRestaurantAdapter.RestaurantViewHolder onCreateViewHolder(@NonNull ViewGroup parent, int viewType) {
        return new RestaurantViewHolder(LayoutInflater.from(activity).inflate(R.layout.favourite_restaurant_item, parent, false));
    }

    @Override
    public void onBindViewHolder(@NonNull FavouriteRestaurantAdapter.RestaurantViewHolder holder, int position) {
        FavouriteRestaurant favouriteRestaurant = restaurants.get(position);
        FavouriteRestaurant.RestaurantMiniDetails restaurant = favouriteRestaurant.restaurant;

        holder.tvRestaurant.setText(restaurant.restaurant_name);
        if (favouriteRestaurant.restaurant.deliveryCharge > 0)
            holder.tvFreeDelivery.setText(CommonFunctions.StringAppender(MyApp.getInstance().getCurrencySymbol(), MyApp.df.format(favouriteRestaurant.restaurant.deliveryCharge), " del fee"));
        else holder.tvFreeDelivery.setText("Free del");
        holder.tvOrderCount.setText(String.format("You have ordered %d times before", favouriteRestaurant.order_count));
        holder.tvMinOrder.setText(CommonFunctions.StringAppender(restaurant.currency, String.valueOf(restaurant.minimum_order), " min cost"));

        if (!Validators.isNullOrEmpty(String.valueOf(favouriteRestaurant.restaurant.distance))) {
            holder.tvDistance.setText(CommonFunctions.StringAppender(String.valueOf(favouriteRestaurant.restaurant.distance), " miles"));
        } else {
            holder.llDistance.setVisibility(View.GONE);
        }

        Glide.with(activity)
                .load(favouriteRestaurant.restaurant.image_url)
                .placeholder(R.drawable.restaurant_logo_placeholder)
                .error(R.drawable.restaurant_logo_placeholder)
                .into(holder.ivRestaurantLogo);
        holder.ratingBar.setRating(favouriteRestaurant.finalReview);

        if (favouriteRestaurant.restaurant.promotions != null && favouriteRestaurant.restaurant.promotions.size() > 0 && !Validators.isNullOrEmpty(favouriteRestaurant.restaurant.promotions.get(0).promo_image)) {
            String bannerImage = favouriteRestaurant.restaurant.promotions.get(0).image_url;
            Glide.with(activity)
                    .load(bannerImage)
                    .placeholder(R.drawable.restaurant_banner_placeholder)
                    .error(R.drawable.restaurant_banner_placeholder)
                    .into(holder.ivRestaurantBanner);
        } else {
            Glide.with(activity).load(R.drawable.restaurant_banner_placeholder).into(holder.ivRestaurantBanner);
        }
        holder.ivFavouriteImage.setImageResource(R.drawable.ic_baseline_favorite_24);
        holder.ivFavouriteImage.setOnClickListener(view -> {
            if (MyApp.getInstance().getMyPreferences().getLoggedInUser() == null) {
                activity.startActivity(new Intent(activity, AuthActivity.class));
            } else {
                holder.ivFavourite.setProgress(0);
                holder.ivFavourite.cancelAnimation();
                holder.ivFavouriteImage.setImageResource(R.drawable.ic_baseline_favorite_border_24);
                if (favouriteClickListener != null) {
                    favouriteClickListener.onItemClick(position, favouriteRestaurant);
                }
            }
        });
        if (restaurant.close.equals("1")) {
            holder.tvClosed.setVisibility(View.VISIBLE);
            holder.tvRestaurant.setAlpha(0.8f);
            holder.ratingBar.setAlpha(0.8f);
            holder.tvOrderCount.setAlpha(0.8f);
        } else {
            holder.tvClosed.setVisibility(View.GONE);
            holder.tvRestaurant.setAlpha(1);
            holder.ratingBar.setAlpha(1);
            holder.tvOrderCount.setAlpha(1);
        }
    }

    @Override
    public int getItemCount() {
        return restaurants.size();
    }

    protected class RestaurantViewHolder extends RecyclerView.ViewHolder {
        private TextView tvRestaurant, tvClosed, tvMinOrder, tvDistance, tvFreeDelivery, tvOrderCount;
        private SimpleRatingBar ratingBar;
        private CardView cvRestaurant;
        private ImageView ivRestaurantBanner, ivRestaurantLogo, ivFavouriteImage;
        private LottieAnimationView ivFavourite;
        private LinearLayout llMinOrder, llDistance, llFreeDelivery;

        public RestaurantViewHolder(@NonNull View itemView) {
            super(itemView);

            tvRestaurant = itemView.findViewById(R.id.tvRestaurant);
            tvOrderCount = itemView.findViewById(R.id.tvOrderCount);
            tvClosed = itemView.findViewById(R.id.tvClosed);
            tvMinOrder = itemView.findViewById(R.id.tvMinOrder);
            tvDistance = itemView.findViewById(R.id.tvDistance);
            tvFreeDelivery = itemView.findViewById(R.id.tvDeliveryFee);
            ratingBar = itemView.findViewById(R.id.rating);

            ivFavourite = itemView.findViewById(R.id.ivFavourite);
            ivFavouriteImage = itemView.findViewById(R.id.ivFavouriteImage);

            ivRestaurantBanner = itemView.findViewById(R.id.ivRestaurantBanner);
            ivRestaurantLogo = itemView.findViewById(R.id.ivRestaurantLogo);

            cvRestaurant = itemView.findViewById(R.id.cvRestaurant);

            llDistance = itemView.findViewById(R.id.llDistance);
            llFreeDelivery = itemView.findViewById(R.id.llDeliveryFee);
            llMinOrder = itemView.findViewById(R.id.llMinOrder);

            cvRestaurant.setOnClickListener(view -> {
                if (restaurantClickListener != null) {
                    restaurantClickListener.onItemClick(getLayoutPosition(), restaurants.get(getLayoutPosition()));
                }
            });
        }
    }
}
