package com.tiffintom.adapters;

import android.app.Activity;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.CheckBox;
import android.widget.ImageView;
import android.widget.TextView;

import androidx.annotation.NonNull;
import androidx.recyclerview.widget.RecyclerView;

import com.tiffintom.R;
import com.tiffintom.interfaces.RecyclerViewItemClickListener;
import com.tiffintom.models.Variant;

import java.util.ArrayList;

/**
 * Created by Amrish on 19-08-2020.
 */
public class MenuVariantAdapter extends RecyclerView.Adapter<MenuVariantAdapter.MenuItemAddOnViewHolder> {
    private Activity activity;
    private ArrayList<Variant> categories = new ArrayList<>();
    public static int selectedVariant = 0;
    private RecyclerViewItemClickListener recyclerViewItemClickListener;
    private int lastCheckedPosition = -1, checkedPosition = -1;
    private ImageView checkedImage;


    public MenuVariantAdapter(Activity activity, ArrayList<Variant> categories, RecyclerViewItemClickListener recyclerViewItemClickListener) {
        this.activity = activity;
        this.categories = categories;
        this.recyclerViewItemClickListener = recyclerViewItemClickListener;
    }

    @NonNull
    @Override
    public MenuItemAddOnViewHolder onCreateViewHolder(@NonNull ViewGroup parent, int viewType) {
        return new MenuItemAddOnViewHolder(LayoutInflater.from(activity).inflate(R.layout.menu_variant_item, parent, false));
    }

    @Override
    public void onBindViewHolder(@NonNull MenuItemAddOnViewHolder holder, int position) {
        Variant variant = categories.get(position);
        holder.tvTitle.setText(variant.sub_name);
        if (variant.id == (selectedVariant)) {
            holder.cbSelect.setChecked(true);
            holder.ivSelect.setImageResource(R.drawable.ic_baseline_done_24);
            lastCheckedPosition = position;
            checkedImage = holder.ivSelect;
        } else {
            holder.ivSelect.setImageResource(R.drawable.dot_grey_circle);
            holder.cbSelect.setChecked(false);
        }

        holder.view.setOnClickListener((view) -> {
            selectedVariant = variant.menu_id;
            if (checkedImage != null) {
                checkedImage.setImageResource(R.drawable.dot_grey_circle);
            }
            holder.ivSelect.setImageResource(R.drawable.ic_baseline_done_24);
            checkedImage = holder.ivSelect;
            lastCheckedPosition = position;
            if (recyclerViewItemClickListener != null)
                recyclerViewItemClickListener.onItemClick(position, variant);
        });

    }

    @Override
    public int getItemCount() {
        return categories.size();
    }

    protected class MenuItemAddOnViewHolder extends RecyclerView.ViewHolder {
        private TextView tvTitle;
        private ImageView ivSelect;
        private View view;
        private CheckBox cbSelect;

        public MenuItemAddOnViewHolder(@NonNull View itemView) {
            super(itemView);
            tvTitle = itemView.findViewById(R.id.tvTitle);
            ivSelect = itemView.findViewById(R.id.ivSelect);
            cbSelect = itemView.findViewById(R.id.cbSelect);
            view = itemView;
        }
    }
}
