package com.tiffintom.adapters;

import android.content.Context;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.CheckBox;
import android.widget.ImageView;
import android.widget.LinearLayout;
import android.widget.TextView;

import androidx.annotation.NonNull;
import androidx.recyclerview.widget.RecyclerView;

import com.tiffintom.R;
import com.tiffintom.interfaces.RecyclerViewItemClickListener;
import com.tiffintom.models.OfferProduct;
import com.tiffintom.utils.ToastUtils;

import java.util.ArrayList;

/**
 * Created by Amrish on 23-12-2020.
 */
public class MultipleProductOfferAdapter extends RecyclerView.Adapter<MultipleProductOfferAdapter.ProductViewHolder> {
    private final ArrayList<OfferProduct> offerProducts;
    private final RecyclerViewItemClickListener itemChangeClickListener;
    public int maxQty, checkedQty;

    public MultipleProductOfferAdapter(ArrayList<OfferProduct> offerProducts, RecyclerViewItemClickListener itemChangeClickListener) {
        this.offerProducts = offerProducts;
        this.itemChangeClickListener = itemChangeClickListener;
    }

    @NonNull
    @Override
    public ProductViewHolder onCreateViewHolder(@NonNull ViewGroup parent, int viewType) {
        return new ProductViewHolder(LayoutInflater.from(parent.getContext()).inflate(R.layout.item_multiple_product_offer, parent, false));
    }

    @Override
    public void onBindViewHolder(@NonNull ProductViewHolder holder, int position) {
        Context context = holder.itemView.getContext();
        OfferProduct offerProduct = offerProducts.get(position);
        holder.tvName.setText(offerProduct.name);
        holder.tvQty.setText(offerProduct.quantity + "");
//        holder.cbCheck.setChecked(offerProduct.checked);
        holder.cbCheck.setChecked(offerProduct.quantity > 0);
        if (offerProduct.quantity == 0) {
            holder.ivMinus.setEnabled(false);
            holder.ivMinus.setAlpha(0.8f);
        } else {
            holder.ivMinus.setEnabled(true);
            holder.ivMinus.setAlpha(1f);
        }

        holder.llCheck.setOnClickListener(view -> {
            if (!holder.cbCheck.isChecked()) {
                holder.ivPlus.performClick();
            } else {
                holder.cbCheck.setChecked(false);
                checkedQty = checkedQty - offerProduct.quantity;
                offerProduct.quantity = 0;
                notifyDataSetChanged();
                if (itemChangeClickListener != null)
                    itemChangeClickListener.onItemClick(position, offerProduct);
            }
        });
        holder.ivPlus.setOnClickListener(view -> {
            checkedQty++;
            offerProduct.quantity++;
            offerProduct.checked = true;
            if (maxQty < checkedQty) {
                checkedQty--;
                offerProduct.quantity--;
                ToastUtils.makeToast(context, "You can select only " + maxQty + " products");
            }
            offerProduct.checked = offerProduct.quantity != 0;
            notifyDataSetChanged();
            if (itemChangeClickListener != null)
                itemChangeClickListener.onItemClick(position, offerProduct);
        });
        holder.ivMinus.setOnClickListener(view -> {
            if (offerProduct.quantity > 0) {
                offerProduct.quantity--;
                checkedQty--;
                notifyDataSetChanged();
                if (itemChangeClickListener != null)
                    itemChangeClickListener.onItemClick(position, offerProduct);
                offerProduct.checked = offerProduct.quantity != 0;
            }
        });

    }

    @Override
    public int getItemCount() {
        return offerProducts.size();
    }

    protected static class ProductViewHolder extends RecyclerView.ViewHolder {
        CheckBox cbCheck;
        TextView tvName, tvQty;
        ImageView ivPlus, ivMinus;
        LinearLayout llCheck;
        public ProductViewHolder(@NonNull View itemView) {
            super(itemView);
            llCheck = itemView.findViewById(R.id.llCheckBox);
            cbCheck = itemView.findViewById(R.id.cbCheck);
            tvName = itemView.findViewById(R.id.tvName);
            tvQty = itemView.findViewById(R.id.tvQuantity);
            ivMinus = itemView.findViewById(R.id.ivMinus);
            ivPlus = itemView.findViewById(R.id.ivPlus);
        }
    }

}
