package com.tiffintom.adapters;

import android.app.Activity;
import android.view.Gravity;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.LinearLayout;
import android.widget.TextView;

import androidx.annotation.NonNull;
import androidx.cardview.widget.CardView;
import androidx.core.content.ContextCompat;
import androidx.recyclerview.widget.RecyclerView;

import com.tiffintom.R;
import com.tiffintom.models.OfferSimplify;

import java.util.ArrayList;

/**
 * Created by Amrish on 15-09-2020.
 */
public class OffersAdapter extends RecyclerView.Adapter<RecyclerView.ViewHolder> {
    private ArrayList<OfferSimplify> objects;
    private boolean restaurantOffers;
    private Activity activity;

    public OffersAdapter(Activity activity, ArrayList<OfferSimplify> objects) {
        this.objects = objects;
        this.activity = activity;
    }

    public OffersAdapter(Activity activity, ArrayList<OfferSimplify> objects, boolean restaurantOffers) {
        this.objects = objects;
        this.restaurantOffers = restaurantOffers;
        this.activity = activity;
    }

    @NonNull
    @Override
    public RecyclerView.ViewHolder onCreateViewHolder(@NonNull ViewGroup parent, int viewType) {
        if (restaurantOffers) {
            return new RestaurantOfferViewHolder(LayoutInflater.from(activity).inflate(R.layout.item_restaurant_offer, parent, false));
        } else
            return new HomeOfferViewHolder(LayoutInflater.from(activity).inflate(R.layout.item_restaurant_offer_home, parent, false));
    }

    @Override
    public void onBindViewHolder(@NonNull RecyclerView.ViewHolder holder, int position) {
        if (restaurantOffers) {
            RestaurantOfferViewHolder restaurantHolder = (RestaurantOfferViewHolder) holder;
            OfferSimplify simplify = objects.get(position);
            restaurantHolder.tvTitle.setText(simplify.title);
            restaurantHolder.tvSubTitle.setText(simplify.subtitle);

            int colors[] = {R.color.left_card, R.color.right_card, R.color.center_card};

            restaurantHolder.cvOffer.setCardBackgroundColor(ContextCompat.getColor(restaurantHolder.itemView.getContext(), colors[position % colors.length]));
            LinearLayout.LayoutParams params = new LinearLayout.LayoutParams(ViewGroup.LayoutParams.WRAP_CONTENT, ViewGroup.LayoutParams.MATCH_PARENT);
            if ((position + 1) % 3 == 0) {
                params.weight = 1.0f;
                params.gravity = Gravity.CENTER;
                restaurantHolder.tvTitle.setLayoutParams(params);
                restaurantHolder.tvSubTitle.setLayoutParams(params);

                restaurantHolder.tvTitle.setTextAlignment(View.TEXT_ALIGNMENT_CENTER);
                restaurantHolder.tvSubTitle.setTextAlignment(View.TEXT_ALIGNMENT_CENTER);

            } else {

                if (position <= 4) {

                    if (position == 0) {
                        params.weight = 1.0f;
                        params.gravity = Gravity.LEFT;
                        restaurantHolder.tvTitle.setLayoutParams(params);
                        restaurantHolder.tvSubTitle.setLayoutParams(params);

                        restaurantHolder.tvTitle.setTextAlignment(View.TEXT_ALIGNMENT_TEXT_START);
                        restaurantHolder.tvSubTitle.setTextAlignment(View.TEXT_ALIGNMENT_TEXT_START);

                    } else if (position == 1) {
                        params.weight = 1.0f;
                        params.gravity = Gravity.END;
                        restaurantHolder.tvTitle.setLayoutParams(params);
                        restaurantHolder.tvSubTitle.setLayoutParams(params);

                        restaurantHolder.tvTitle.setTextAlignment(View.TEXT_ALIGNMENT_TEXT_END);
                        restaurantHolder.tvSubTitle.setTextAlignment(View.TEXT_ALIGNMENT_TEXT_END);

                    } else if ((position + 1) % 2 != 0) {
                        params.weight = 1.0f;
                        params.gravity = Gravity.END;
                        restaurantHolder.tvTitle.setLayoutParams(params);
                        restaurantHolder.tvSubTitle.setLayoutParams(params);

                        restaurantHolder.tvTitle.setTextAlignment(View.TEXT_ALIGNMENT_TEXT_END);
                        restaurantHolder.tvSubTitle.setTextAlignment(View.TEXT_ALIGNMENT_TEXT_END);

                    } else {
                        params.weight = 1.0f;
                        params.gravity = Gravity.END;
                        restaurantHolder.tvTitle.setLayoutParams(params);
                        restaurantHolder.tvSubTitle.setLayoutParams(params);

                        restaurantHolder.tvTitle.setTextAlignment(View.TEXT_ALIGNMENT_TEXT_START);
                        restaurantHolder.tvSubTitle.setTextAlignment(View.TEXT_ALIGNMENT_TEXT_START);
                    }
                } else {
                    if ((position) % 2 != 0) {
                        params.weight = 1.0f;
                        params.gravity = Gravity.END;
                        restaurantHolder.tvTitle.setLayoutParams(params);
                        restaurantHolder.tvSubTitle.setLayoutParams(params);

                        restaurantHolder.tvTitle.setTextAlignment(View.TEXT_ALIGNMENT_TEXT_END);
                        restaurantHolder.tvSubTitle.setTextAlignment(View.TEXT_ALIGNMENT_TEXT_END);

                    } else {
                        params.weight = 1.0f;
                        params.gravity = Gravity.END;
                        restaurantHolder.tvTitle.setLayoutParams(params);
                        restaurantHolder.tvSubTitle.setLayoutParams(params);

                        restaurantHolder.tvTitle.setTextAlignment(View.TEXT_ALIGNMENT_TEXT_START);
                        restaurantHolder.tvSubTitle.setTextAlignment(View.TEXT_ALIGNMENT_TEXT_START);
                    }
                }
            }

        } else {
            HomeOfferViewHolder homeHolder = (HomeOfferViewHolder) holder;
            OfferSimplify simplify = objects.get(position);
            homeHolder.tvTitle.setText(simplify.title);
            homeHolder.tvSubTitle.setText(simplify.subtitle);
        }
    }


    @Override
    public int getItemCount() {
        return objects.size();
    }

    protected static class HomeOfferViewHolder extends RecyclerView.ViewHolder {
        private TextView tvTitle, tvSubTitle;

        public HomeOfferViewHolder(@NonNull View itemView) {
            super(itemView);
            tvTitle = itemView.findViewById(R.id.tvTitle);
            tvSubTitle = itemView.findViewById(R.id.tvSubtitle);


        }
    }

    protected static class RestaurantOfferViewHolder extends RecyclerView.ViewHolder {
        private TextView tvTitle, tvSubTitle;
        private CardView cvOffer;

        public RestaurantOfferViewHolder(@NonNull View itemView) {
            super(itemView);
            tvTitle = itemView.findViewById(R.id.tvTitle);
            tvSubTitle = itemView.findViewById(R.id.tvSubtitle);
            cvOffer = itemView.findViewById(R.id.cvOffer);
        }
    }

}
