package com.tiffintom.adapters;

import android.app.Activity;
import android.content.Context;
import android.content.Intent;
import android.os.CountDownTimer;
import android.view.LayoutInflater;
import android.view.MotionEvent;
import android.view.View;
import android.view.ViewGroup;
import android.widget.FrameLayout;
import android.widget.ImageView;
import android.widget.LinearLayout;
import android.widget.ProgressBar;
import android.widget.TextView;

import androidx.annotation.NonNull;
import androidx.core.content.ContextCompat;
import androidx.recyclerview.widget.LinearLayoutManager;
import androidx.recyclerview.widget.RecyclerView;

import com.bumptech.glide.Glide;
import com.iarcuschin.simpleratingbar.SimpleRatingBar;
import com.tiffintom.MyApp;
import com.tiffintom.R;
import com.tiffintom.activity.TransportActivity;
import com.tiffintom.common.CommonFunctions;
import com.tiffintom.common.Validators;
import com.tiffintom.interfaces.RecyclerViewItemClickListener;
import com.tiffintom.models.HistoryCart;
import com.tiffintom.models.OrderHistory;
import com.tiffintom.models.Review;
import com.tiffintom.utils.LogUtils;

import java.text.DecimalFormat;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.Locale;

import static java.lang.Math.abs;

/**
 * Created by Amrish on 07-09-2020.
 */
public class OrderHistoryAdapter extends RecyclerView.Adapter<RecyclerView.ViewHolder> {
    private Activity activity;
    private ArrayList<Object> feed;
    private RecyclerViewItemClickListener recyclerViewItemClickListenerReceipt, recyclerViewItemClickListenerReOrder, recyclerViewItemClickListenerAddReview;
    private RecyclerView.RecycledViewPool viewPool;
    public OrderHistoryAdapter(Activity activity, ArrayList<Object> feed, RecyclerViewItemClickListener viewReceiptClickListener, RecyclerViewItemClickListener reOrderClickListener, RecyclerViewItemClickListener recyclerViewItemClickListenerAddReview) {
        this.feed = feed;
        this.activity = activity;
        this.recyclerViewItemClickListenerReceipt = viewReceiptClickListener;
        this.recyclerViewItemClickListenerReOrder = reOrderClickListener;
        this.recyclerViewItemClickListenerAddReview = recyclerViewItemClickListenerAddReview;
        viewPool = new RecyclerView.RecycledViewPool();
    }

    @Override
    public int getItemViewType(int position) {
        if (feed.get(position) instanceof OrderHistory) {
            return R.layout.order_history_item;
        }
        return super.getItemViewType(position);
    }

    @NonNull
    @Override
    public RecyclerView.ViewHolder onCreateViewHolder(@NonNull ViewGroup parent, int viewType) {
        View view = LayoutInflater.from(parent.getContext()).inflate(viewType, parent, false);
        return new OrderViewHolder(view);
    }

    @Override
    public void onBindViewHolder(@NonNull RecyclerView.ViewHolder viewHolder, int position) {

        Context context = viewHolder.itemView.getContext();
        if (getItemViewType(position) == R.layout.order_history_item) {
            OrderViewHolder holder = (OrderViewHolder) viewHolder;
            OrderHistory orderHistory = (OrderHistory) feed.get(position);
            holder.tvRestaurant.setText(orderHistory.restaurant.restaurant_name);
            holder.tvOrderId.setText(orderHistory.order_number);
            DecimalFormat df = new DecimalFormat("#.##");
            holder.tvTotal.setText("Total: " + MyApp.getInstance().getCurrencySymbol() + MyApp.df.format(orderHistory.order_grand_total));
            holder.tvOrderDate.setText(CommonFunctions.formatTimestampZulu(orderHistory.created, "dd MMM yyyy 'at' hh:mm a"));
            holder.rbRestaurant.setRating(orderHistory.restaurant.average_rating);
            holder.tvOrderStatus.setText("Order " + orderHistory.status);

            holder.cartHistory.clear();
            holder.cartHistory.addAll(orderHistory.cart_view);
            holder.adapter.notifyDataSetChanged();

            holder.rvCartHistory.addOnItemTouchListener(new RecyclerView.OnItemTouchListener() {
                @Override
                public boolean onInterceptTouchEvent(@NonNull RecyclerView rv, @NonNull MotionEvent e) {
                    return false;
                }

                @Override
                public void onTouchEvent(@NonNull RecyclerView rv, @NonNull MotionEvent e) {

                }

                @Override
                public void onRequestDisallowInterceptTouchEvent(boolean disallowIntercept) {

                }
            });

            if (orderHistory.restaurant.promotions != null && orderHistory.restaurant.promotions .size() > 0 && !Validators.isNullOrEmpty(orderHistory.restaurant.promotions .get(0).promo_image)) {
                String bannerImage = orderHistory.restaurant.promotions .get(0).image_url;
                Glide.with(context)
                        .load(bannerImage)
                        .placeholder(R.drawable.restaurant_banner_placeholder)
                        .error(R.drawable.restaurant_banner_placeholder)
                        .into(holder.ivRestaurant);
            } else {
                Glide.with(context).load(R.drawable.restaurant_banner_placeholder).into(holder.ivRestaurant);
            }

            holder.tvReceipt.setOnClickListener(view -> {
                Intent intent = new Intent(context, TransportActivity.class);
                intent.putExtra("destination", "orderView");
                intent.putExtra("hideToolbar", true);
                intent.putExtra("order_id", String.valueOf(orderHistory.id));
                intent.putExtra("restaurant_name", orderHistory.restaurant.restaurant_name);
                context.startActivity(intent);
            });

            holder.tvTrackOrder.setOnClickListener(view -> {
                Intent intent = new Intent(context, TransportActivity.class);
                intent.putExtra("destination", "trackOrder");
                intent.putExtra("hideToolbar", true);
                intent.putExtra("order_id", String.valueOf(orderHistory.id));
                context.startActivity(intent);
            });

            if (orderHistory.status.equalsIgnoreCase("accepted") || orderHistory.status.equalsIgnoreCase("pending") || orderHistory.status.equalsIgnoreCase("waiting") || orderHistory.status.equalsIgnoreCase("driver accepted") || orderHistory.status.equalsIgnoreCase("collected")) {
                holder.tvTrackOrder.setVisibility(View.VISIBLE);
                holder.tvReOrder.setVisibility(View.GONE);
                holder.tvReceipt.setVisibility(View.GONE);
            } else if (orderHistory.status.equalsIgnoreCase("delivered") || orderHistory.status.equalsIgnoreCase("failed")) {
                holder.tvTrackOrder.setVisibility(View.GONE);
                boolean showReOrder = true;
                for (HistoryCart cart : orderHistory.cart_view) {

                    if (cart.restaurant_menu != null && (CommonFunctions.formatTimestampAndGetMs(cart.restaurant_menu.modified.replace(":", "")) > CommonFunctions.formatTimestampAndGetMs(orderHistory.created.replace(":", "")))) {
                        showReOrder = false;
                        break;
                    }
                }
                if (showReOrder) {
                    holder.tvReOrder.setVisibility(View.VISIBLE);
                } else {
                    holder.tvReOrder.setVisibility(View.GONE);
                }
            }
            if (holder.timer != null) holder.timer.cancel();

            if (orderHistory.status.equalsIgnoreCase("delivered") || orderHistory.status.equalsIgnoreCase("failed")) {
                holder.llTimer.setVisibility(View.GONE);
            } else {
                holder.llTimer.setVisibility(View.VISIBLE);
                if (!Validators.isNullOrEmpty(orderHistory.preparation)) {
                    SimpleDateFormat prepFormat = new SimpleDateFormat("hh:mm aa");
                    long diff = 1000;
                    Date prepTime, currentTime, createdTime;
                    try {
                        prepTime = prepFormat.parse(orderHistory.preparation);
                        currentTime = prepFormat.parse(new SimpleDateFormat("hh:mm aa", Locale.getDefault()).format(new Date()));
                        createdTime = prepFormat.parse(CommonFunctions.formatTimestampZulu(orderHistory.created, "hh:mm aa"));
                        diff = prepTime.getTime() - currentTime.getTime();
                        holder.timer = new CountDownTimer(diff, 5000) {
                            @Override
                            public void onTick(long diffLong) {
                                String message = "Est. delivery time " + orderHistory.preparation;
                                double percent = 0;
                                long diff = diffLong;
                                double div = (double) diff * 100 / (prepTime.getTime() - createdTime.getTime());
                                percent = 100 - (div);
                                long diffMinutes = diff / (60 * 1000) % 60;
                                long diffHours = diff / (60 * 60 * 1000) % 24;
                                LogUtils.e("ID::", orderHistory.order_number, " Percentage: ", String.valueOf(percent));
                                LogUtils.e("Diff: " + diff, "=>>" + diffHours + ":" + diffMinutes);
                                if (diffHours > 1) {
                                    message = "Est. delivery time " + df.format(diffHours) + " hours";
                                } else if (diffHours == 1) {
                                    message = "Est. delivery time " + df.format(diffHours) + " hour and " + df.format(diffMinutes) + " minutes";
                                } else if (diffMinutes > 1) {
                                    message = "Est. delivery time " + df.format(diffMinutes) + " minutes";
                                } else {
                                    message = "Est. delivery time " + orderHistory.preparation;
                                }
                                holder.tvOrderTimer.setText(message);
                                holder.pbTime.setProgress(abs((int) percent));
                            }

                            @Override
                            public void onFinish() {
                                holder.pbTime.setProgress(100);
                                holder.tvOrderTimer.setText("Est. delivery time 0 minutes");
                            }
                        };
                        holder.timer.start();
                    } catch (ParseException e) {
                        e.printStackTrace();
                        holder.pbTime.setProgress(100);
                        holder.tvOrderTimer.setText("Est. delivery time 0 minutes");
                        if (holder.timer != null)
                            holder.timer.cancel();
                    }


                    // 1000*10*60 every 10 minut
                    holder.ivStatusIcon.setImageResource(R.drawable.icon_delivery);
                    holder.pbTime.setProgressDrawable(ContextCompat.getDrawable(context, R.drawable.brand_color_progressbar));
                } else {
                    holder.pbTime.setVisibility(View.VISIBLE);
                    holder.ivStatusIcon.setImageResource(R.drawable.icon_min_order_amount);
                    holder.pbTime.setProgress(100);
                    holder.tvOrderTimer.setText("Waiting for restaurant to accept order");
                    holder.pbTime.setProgressDrawable(ContextCompat.getDrawable(context, R.drawable.golden_color_progressbar_order));
                }
            }


            holder.tvAddreview.setOnClickListener(view -> {
                if (recyclerViewItemClickListenerAddReview != null) {
                    recyclerViewItemClickListenerAddReview.onItemClick(position, orderHistory);
                }
            });
            holder.tvReOrder.setOnClickListener(view -> {
                if (recyclerViewItemClickListenerReOrder != null) {
                    recyclerViewItemClickListenerReOrder.onItemClick(position, orderHistory);
                }
            });

            if (orderHistory.status.equalsIgnoreCase("delivered")) {
                if (orderHistory.reviews != null ) {

                    Review review = orderHistory.reviews;
                    if (review.customer_id == MyApp.getInstance().getMyPreferences().getLoggedInUser().id) {
                        holder.rbUser.setRating((orderHistory.reviews.rating));
                        holder.tvAddreview.setVisibility(View.GONE);
                    } else {
                        holder.tvAddreview.setVisibility(View.VISIBLE);
                        holder.rbUser.setVisibility(View.GONE);
                    }
                    holder.rbUser.setVisibility(View.VISIBLE);
                    holder.tvAddreview.setVisibility(View.GONE);
                } else {
                    holder.rbUser.setVisibility(View.GONE);
                    holder.tvAddreview.setVisibility(View.VISIBLE);

                }

            } else {
                holder.tvAddreview.setVisibility(View.GONE);
                holder.rbUser.setVisibility(View.GONE);
            }


        }
    }

    @Override
    public int getItemCount() {
        return feed.size();
    }

    protected class OrderViewHolder extends RecyclerView.ViewHolder {
        private TextView tvOrderDate, tvOrderId, tvRestaurant, tvOrderStatus, tvTotal, tvOrderTimer, tvAddreview;
        private ImageView ivRestaurant, ivStatusIcon;
        private SimpleRatingBar rbRestaurant, rbUser;
        private View vReservation;
        private ProgressBar pbTime;
        private TextView tvReOrder, tvReceipt, tvTrackOrder;
        private RecyclerView rvCartHistory;
        private FrameLayout flTimer;
        private LinearLayout llTimer;
        private OrderHistoryCartAdapter adapter;
        private ArrayList<HistoryCart> cartHistory = new ArrayList<>();
        CountDownTimer timer;

        public OrderViewHolder(@NonNull View itemView) {
            super(itemView);
            tvRestaurant = itemView.findViewById(R.id.tvRestaurant);
            tvOrderId = itemView.findViewById(R.id.tvOrderId);
            ivRestaurant = itemView.findViewById(R.id.ivRestaurantBanner);
            ivStatusIcon = itemView.findViewById(R.id.ivStatusIcon);
            tvReOrder = itemView.findViewById(R.id.tvReOrder);
            tvReceipt = itemView.findViewById(R.id.tvViewReceipt);
            tvTrackOrder = itemView.findViewById(R.id.tvTrackOrder);
            tvOrderDate = itemView.findViewById(R.id.tvOrderDate);
            tvOrderStatus = itemView.findViewById(R.id.tvOrderStatus);
            tvRestaurant = itemView.findViewById(R.id.tvRestaurant);
            tvTotal = itemView.findViewById(R.id.tvTotal);
            rvCartHistory = itemView.findViewById(R.id.rvCartItems);
            pbTime = itemView.findViewById(R.id.pbTime);
            tvOrderTimer = itemView.findViewById(R.id.tvOrderTimer);
            flTimer = itemView.findViewById(R.id.flTimer);
            llTimer = itemView.findViewById(R.id.llTimer);
            tvAddreview = itemView.findViewById(R.id.tvAddReview);

            rbRestaurant = itemView.findViewById(R.id.restrauarntRating);
            rbUser = itemView.findViewById(R.id.userRating);

            adapter = new OrderHistoryCartAdapter(cartHistory);
            LinearLayoutManager linearLayoutManager = new LinearLayoutManager(itemView.getContext(), RecyclerView.VERTICAL, false);
            rvCartHistory.setLayoutManager(linearLayoutManager);
            rvCartHistory.setRecycledViewPool(viewPool);
            rvCartHistory.setHasFixedSize(true);
            adapter.setHasStableIds(true);

            rvCartHistory.setNestedScrollingEnabled(false);
            rvCartHistory.setAdapter(adapter);
        }
    }

    public static int getPercentageLeft(Date start, Date end) {
        long now = System.currentTimeMillis();
        long s = start.getTime();
        long e = end.getTime();
        if (s >= e || now >= e) {
            return 0;
        }
        if (now <= s) {
            return 100;
        }
        return (int) ((e - now) * 100 / (e - s));
    }
}
