package com.tiffintom.adapters;

import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.TextView;

import androidx.annotation.NonNull;
import androidx.recyclerview.widget.RecyclerView;

import com.tiffintom.MyApp;
import com.tiffintom.R;
import com.tiffintom.common.Validators;
import com.tiffintom.models.HistoryCart;

import java.text.DecimalFormat;
import java.util.ArrayList;

/**
 * Created by Amrish on 07-09-2020.
 */
public class OrderHistoryCartAdapter extends RecyclerView.Adapter<OrderHistoryCartAdapter.CartItemViewHolder> {

    private ArrayList<HistoryCart> carts;
    private boolean showPrice = false;

    public OrderHistoryCartAdapter(ArrayList<HistoryCart> carts) {
        this.carts = carts;
    }

    public OrderHistoryCartAdapter(ArrayList<HistoryCart> carts, boolean showPrice) {
        this.carts = carts;
        this.showPrice = showPrice;
    }

    private DecimalFormat df = new DecimalFormat("#.##");

    @NonNull
    @Override
    public CartItemViewHolder onCreateViewHolder(@NonNull ViewGroup parent, int viewType) {
        return new CartItemViewHolder(LayoutInflater.from(parent.getContext()).inflate(R.layout.order_history_item_cart_item, parent, false));
    }

    @Override
    public void onBindViewHolder(@NonNull CartItemViewHolder holder, int position) {

        HistoryCart cart = carts.get(position);
        holder.tvName.setText(cart.menu_name);
        holder.tvCount.setText(String.valueOf(cart.quantity));
        holder.tvSubAddon.setVisibility(View.GONE);
        if (!Validators.isNullOrEmpty(cart.subaddons_name)) {
            String subAddonName = cart.subaddons_name.replaceAll("(^(\\s*?\\,+)+\\s?)|(^\\s+)|(\\s+$)|((\\s*?\\,+)+\\s?$)", "");
            holder.tvSubAddon.setText(subAddonName);
            holder.tvSubAddon.setText(subAddonName);
            holder.tvSubAddon.setVisibility(View.VISIBLE);
        }
        if (showPrice) {
            holder.tvPrice.setVisibility(View.VISIBLE);
        } else {
            holder.tvPrice.setVisibility(View.GONE);
        }
        holder.tvPrice.setText(String.format("%s%s", MyApp.getInstance().getCurrencySymbol(), MyApp.df.format(cart.total_price)));
    }

    @Override
    public int getItemCount() {
        return carts.size();
    }

    @Override
    public long getItemId(int position) {
        return carts.get(position).id;
    }

    protected class CartItemViewHolder extends RecyclerView.ViewHolder {

        private TextView tvCount, tvName, tvPrice, tvSubAddon;

        public CartItemViewHolder(@NonNull View itemView) {
            super(itemView);
            tvCount = itemView.findViewById(R.id.tvCount);
            tvName = itemView.findViewById(R.id.tvName);
            tvPrice = itemView.findViewById(R.id.tvPrice);
            tvSubAddon = itemView.findViewById(R.id.tvSubAddon);
        }
    }
}
