package com.tiffintom.adapters;

import android.app.Activity;
import android.util.Log;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.ImageView;
import android.widget.RadioButton;
import android.widget.TextView;

import androidx.annotation.NonNull;
import androidx.recyclerview.widget.RecyclerView;

import com.google.gson.Gson;
import com.tiffintom.MyApp;
import com.tiffintom.R;
import com.tiffintom.common.Validators;
import com.tiffintom.interfaces.RecyclerViewItemClickListener;
import com.tiffintom.models.PaymentMethod;
import com.tiffintom.utils.LogUtils;

import java.util.ArrayList;

/**
 * Created by Amrish on 19-08-2020.
 */
public class PaymentMethodsAdapter extends RecyclerView.Adapter<PaymentMethodsAdapter.PaymentMethodViewHolder> {
    private Activity activity;
    private ArrayList<PaymentMethod> paymentMethods = new ArrayList<>();
    public PaymentMethod selectedPaymentMethod;
    private RecyclerViewItemClickListener recyclerViewItemClickListener;
    private int lastCheckedPosition = -1, checkedPosition = -1;
    private RadioButton checkedRadio;
    private boolean showDefault = false;


    public PaymentMethodsAdapter(Activity activity, ArrayList<PaymentMethod> paymentMethods, boolean showDefault, RecyclerViewItemClickListener recyclerViewItemClickListener) {
        this.activity = activity;
        this.paymentMethods = paymentMethods;
        this.showDefault = showDefault;
        this.recyclerViewItemClickListener = recyclerViewItemClickListener;
    }

    @NonNull
    @Override
    public PaymentMethodsAdapter.PaymentMethodViewHolder onCreateViewHolder(@NonNull ViewGroup parent, int viewType) {
        return new PaymentMethodViewHolder(LayoutInflater.from(activity).inflate(R.layout.payment_method_item, parent, false));
    }

    @Override
    public void onBindViewHolder(@NonNull PaymentMethodViewHolder holder, int position) {
        PaymentMethod paymentMethod = paymentMethods.get(position);
        if (paymentMethod.type.equalsIgnoreCase("add")) {
            holder.tvSelect.setVisibility(View.GONE);
            holder.ivSelect.setImageResource(R.drawable.ic_baseline_navigate_next_24_black);
            holder.ivMethod.setVisibility(View.GONE);
        } else {
            holder.ivMethod.setVisibility(View.VISIBLE);
            holder.tvSelect.setVisibility(View.VISIBLE);
            holder.ivSelect.setImageResource(R.drawable.dot_grey_circle);
            if (showDefault) {
                holder.tvSelect.setVisibility(View.VISIBLE);
            } else {
                holder.tvSelect.setVisibility(View.GONE);
            }

            if (selectedPaymentMethod != null) {
                if (!Validators.isNullOrEmpty(selectedPaymentMethod.type) && selectedPaymentMethod.type.equalsIgnoreCase(paymentMethod.type)) {
                    if (paymentMethod.type.equalsIgnoreCase("stripe")) {
                        if (paymentMethod.stripe_token_id.equalsIgnoreCase(selectedPaymentMethod.stripe_token_id)) {
                            holder.ivSelect.setImageResource(R.drawable.ic_baseline_done_24);
                            holder.tvSelect.setVisibility(View.GONE);
                        }
                    } else if (paymentMethod.title.equalsIgnoreCase(selectedPaymentMethod.title)) {
                        holder.ivSelect.setImageResource(R.drawable.ic_baseline_done_24);
                        holder.tvSelect.setVisibility(View.GONE);
                    }
                } else {
                    holder.ivSelect.setImageResource(R.drawable.dot_grey_circle);
                }
            } else {
                PaymentMethod defaultPaymentMethod = MyApp.getInstance().getMyPreferences().getDefaultPaymentMethod();
                if (defaultPaymentMethod != null) {
                    if (defaultPaymentMethod.type.equalsIgnoreCase(paymentMethod.type)) {
                        if (paymentMethod.type.equalsIgnoreCase("stripe")) {
                            if (paymentMethod.stripe_token_id.equalsIgnoreCase(defaultPaymentMethod.stripe_token_id)) {
                                holder.ivSelect.setImageResource(R.drawable.ic_baseline_done_24);
                                holder.tvSelect.setVisibility(View.GONE);
                            }
                        } else if (paymentMethod.title.equalsIgnoreCase(defaultPaymentMethod.title)) {
                            holder.ivSelect.setImageResource(R.drawable.ic_baseline_done_24);
                            holder.tvSelect.setVisibility(View.GONE);
                        }
                    }
                }else
                {
                    holder.ivSelect.setImageResource(R.drawable.dot_grey_circle);
                }
            }
        }
        holder.tvTitle.setText(paymentMethod.title);
        holder.tvSubtitle.setText(paymentMethod.subtitle);
        holder.ivMethod.setImageResource(paymentMethod.method_logo);
        if (Validators.isNullOrEmpty(paymentMethod.subtitle)) {
            holder.tvSubtitle.setVisibility(View.GONE);
        } else {
            holder.tvSubtitle.setVisibility(View.VISIBLE);
        }

        holder.tvSelect.setOnClickListener(view -> {
            MyApp.getInstance().getMyPreferences().saveDefaultPaymentMethod(paymentMethod);
            notifyDataSetChanged();
        });

    }

    @Override
    public int getItemCount() {
        return paymentMethods.size();
    }

    protected class PaymentMethodViewHolder extends RecyclerView.ViewHolder {
        private RadioButton rbTitle;
        private TextView tvTitle, tvSubtitle, tvSelect;
        private ImageView ivSelect, ivMethod;

        public PaymentMethodViewHolder(@NonNull View itemView) {
            super(itemView);
            tvSelect = itemView.findViewById(R.id.tvSelect);
            tvTitle = itemView.findViewById(R.id.tvTitle);
            tvSubtitle = itemView.findViewById(R.id.tvSubtitle);
            ivSelect = itemView.findViewById(R.id.ivSelect);
            ivMethod = itemView.findViewById(R.id.ivMethod);
            itemView.setOnClickListener(view -> {
                if (recyclerViewItemClickListener != null) {
                    recyclerViewItemClickListener.onItemClick(getLayoutPosition(), paymentMethods.get(getLayoutPosition()));

                }
            });
        }
    }
}
