package com.tiffintom.adapters;

import android.app.Activity;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.ImageView;
import android.widget.LinearLayout;
import android.widget.TextView;

import androidx.annotation.NonNull;
import androidx.recyclerview.widget.RecyclerView;

import com.tiffintom.R;
import com.tiffintom.interfaces.RecyclerViewItemClickListener;
import com.tiffintom.models.ProfileMenu;

import java.util.ArrayList;

/**
 * Created by Amrish on 19-08-2020.
 */
public class ProfileMenuAdapter extends RecyclerView.Adapter<ProfileMenuAdapter.ProfileMenuViewHolder> {
    private Activity activity;
    private ArrayList<ProfileMenu> menuOptions = new ArrayList<>();
    private RecyclerViewItemClickListener recyclerViewItemClickListener;

    public ProfileMenuAdapter(Activity activity, ArrayList<ProfileMenu> menuOptions, RecyclerViewItemClickListener recyclerViewItemClickListener) {
        this.activity = activity;
        this.menuOptions = menuOptions;
        this.recyclerViewItemClickListener = recyclerViewItemClickListener;
    }

    @NonNull
    @Override
    public ProfileMenuAdapter.ProfileMenuViewHolder onCreateViewHolder(@NonNull ViewGroup parent, int viewType) {
        return new ProfileMenuViewHolder(LayoutInflater.from(activity).inflate(R.layout.profile_menu_item, parent, false));
    }

    @Override
    public void onBindViewHolder(@NonNull ProfileMenuAdapter.ProfileMenuViewHolder holder, int position) {
        ProfileMenu profileMenu = menuOptions.get(position);
        holder.tvTitle.setText(profileMenu.title);
        holder.ivIcon.setImageResource(profileMenu.icon);


    }

    @Override
    public int getItemCount() {
        return menuOptions.size();
    }

    protected class ProfileMenuViewHolder extends RecyclerView.ViewHolder {
        private TextView tvTitle, tvDescription;
        private LinearLayout llProfileMenu;
        private ImageView ivIcon;

        public ProfileMenuViewHolder(@NonNull View itemView) {
            super(itemView);
            tvTitle = itemView.findViewById(R.id.tvProfileMenuTitle);
            ivIcon = itemView.findViewById(R.id.ivProfileMenuIcon);
            llProfileMenu = itemView.findViewById(R.id.cvProfileMenu);
            llProfileMenu.setOnClickListener(view -> {
                if (recyclerViewItemClickListener != null) {
                    recyclerViewItemClickListener.onItemClick(getLayoutPosition(), menuOptions.get(getLayoutPosition()));
                }
            });
        }
    }
}
