package com.tiffintom.adapters;

import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.TextView;

import androidx.annotation.NonNull;
import androidx.recyclerview.widget.RecyclerView;

import com.tiffintom.MyApp;
import com.tiffintom.R;
import com.tiffintom.common.CommonFunctions;
import com.tiffintom.common.Validators;
import com.tiffintom.interfaces.RecyclerViewItemClickListener;
import com.tiffintom.models.OrderHistory;

import java.util.ArrayList;

/**
 * Created by Amrish on 18-09-2020.
 */
public class RecentOrdersAdapter extends RecyclerView.Adapter<RecentOrdersAdapter.RestaurantViewHolder> {


    private ArrayList<OrderHistory> restaurantChatMessages;
    private RecyclerViewItemClickListener recyclerViewItemClickListener;

    public RecentOrdersAdapter(ArrayList<OrderHistory> restaurantChatMessages, RecyclerViewItemClickListener recyclerViewItemClickListener) {
        this.restaurantChatMessages = restaurantChatMessages;
        this.recyclerViewItemClickListener = recyclerViewItemClickListener;
    }

    @NonNull
    @Override
    public RecentOrdersAdapter.RestaurantViewHolder onCreateViewHolder(@NonNull ViewGroup parent, int viewType) {
        return new RestaurantViewHolder(LayoutInflater.from(parent.getContext()).inflate(R.layout.item_recent_orders, parent, false));
    }

    @Override
    public void onBindViewHolder(@NonNull RestaurantViewHolder holder, int position) {
        OrderHistory order = restaurantChatMessages.get(position);
        holder.tvRestaurant.setText(order.restaurant.restaurant_name);
        holder.tvOrderId.setText("Order #" + order.id);

        if (!Validators.isNullOrEmpty(order.payment_method)) {

            if (order.payment_method.equalsIgnoreCase("stripe"))
                holder.tvPaymentMode.setText("Credit/Debit Card");

            else if (order.payment_method.equalsIgnoreCase("paypal"))
                holder.tvPaymentMode.setText("PayPal");

            else if (order.payment_method.equalsIgnoreCase("credit") || order.payment_method.equalsIgnoreCase("wallet"))
                holder.tvPaymentMode.setText("TiffinTom credit");

            else
                holder.tvPaymentMode.setText("Cash on delivery");
        }

        holder.tvOrderTotal.setText("Total: " + MyApp.getInstance().getCurrencySymbol() + order.order_grand_total);
        holder.tvOrderDate.setText(CommonFunctions.formatTimestamp(order.modified.replace(":", ""), "EEE dd, MMM"));

        holder.itemView.setOnClickListener(view -> {
            if (recyclerViewItemClickListener != null)
                recyclerViewItemClickListener.onItemClick(position, order);
        });
    }

    @Override
    public int getItemCount() {
        return restaurantChatMessages.size();
    }

    protected static class RestaurantViewHolder extends RecyclerView.ViewHolder {
        private TextView tvRestaurant, tvOrderId, tvOrderDate, tvOrderTotal, tvPaymentMode;

        public RestaurantViewHolder(@NonNull View itemView) {
            super(itemView);
            tvOrderId = itemView.findViewById(R.id.tvOrderNumber);
            tvOrderTotal = itemView.findViewById(R.id.tvOrderTotal);
            tvOrderDate = itemView.findViewById(R.id.tvOrderDate);
            tvRestaurant = itemView.findViewById(R.id.tvRestaurant);
            tvPaymentMode = itemView.findViewById(R.id.tvPaymentType);
        }
    }
}
