package com.tiffintom.adapters;

import android.app.Activity;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.Button;
import android.widget.ImageView;
import android.widget.TextView;

import androidx.annotation.NonNull;
import androidx.recyclerview.widget.RecyclerView;

import com.bumptech.glide.Glide;
import com.tiffintom.R;
import com.tiffintom.common.CommonFunctions;
import com.tiffintom.common.Validators;
import com.tiffintom.interfaces.RecyclerViewItemClickListener;
import com.tiffintom.models.Reservation;

import java.util.ArrayList;

/**
 * Created by Amrish on 19-08-2020.
 */
public class ReservationsAdapter extends RecyclerView.Adapter<ReservationsAdapter.ReservationViewHolder> {
    private Activity activity;
    private ArrayList<Reservation> reservations = new ArrayList<>();
    private RecyclerViewItemClickListener rebookClickListener, cancelClickListener;

    public ReservationsAdapter(Activity activity, ArrayList<Reservation> reservations, RecyclerViewItemClickListener rebookClickListener, RecyclerViewItemClickListener cancelClickListener) {
        this.activity = activity;
        this.reservations = reservations;
        this.rebookClickListener = rebookClickListener;
        this.cancelClickListener = cancelClickListener;
    }

    @NonNull
    @Override
    public ReservationsAdapter.ReservationViewHolder onCreateViewHolder(@NonNull ViewGroup parent, int viewType) {
        return new ReservationViewHolder(LayoutInflater.from(activity).inflate(R.layout.reservation_item, parent, false));
    }

    @Override
    public void onBindViewHolder(@NonNull ReservationsAdapter.ReservationViewHolder holder, int position) {
        Reservation reservation = reservations.get(position);
        holder.tvReservationId.setText("Reservation " + reservation.booking_id);
        holder.tvRestaurant.setText(reservation.restaurant.restaurant_name);
        if (reservation.status.equalsIgnoreCase("cancel")) {
            holder.tvReservationStatus.setText("Reservation Cancelled");
        } else
            holder.tvReservationStatus.setText("Reservation " + reservation.status);
        holder.tvReservationName.setText("Name: " + reservation.customer_name);
        holder.tvReservationEmail.setText("E-mail: " + reservation.booking_email);
        holder.tvReservationPhone.setText("Phone number: " + reservation.booking_phone);
        holder.tvReservationGuest.setText("Guest count: " + reservation.guest_count);
        holder.tvReservationDateTime.setText("Date and time: " + CommonFunctions.formatDate(reservation.booking_date, "yyyy-MM-dd", "dd MMM yyyy") + " at " + reservation.booking_time);
        holder.tvReservationDate.setText(CommonFunctions.formatDate(reservation.booking_date, "dd-MM-yyyy", "dd MMM yyyy") + " at " + reservation.booking_time);

        if (reservation.status.equalsIgnoreCase("pending")) {
            holder.btnCancel.setVisibility(View.VISIBLE);
            holder.btnRebook.setVisibility(View.GONE);
        } else {
            holder.btnCancel.setVisibility(View.GONE);
            holder.btnRebook.setVisibility(View.VISIBLE);
        }

        holder.btnCancel.setOnClickListener(view -> {
            if (cancelClickListener != null) {
                cancelClickListener.onItemClick(position, reservation);
            }
        });
        holder.btnRebook.setOnClickListener(view -> {
            if (rebookClickListener != null) {
                rebookClickListener.onItemClick(position, reservation);
            }
        });
        if (reservation.restaurant.promotions != null && reservation.restaurant.promotions.size() > 0 && !Validators.isNullOrEmpty(reservation.restaurant.promotions.get(0).promo_image)) {
            String bannerImage = reservation.restaurant.promotions.get(0).image_url;
            Glide.with(activity)
                    .load(bannerImage)
                    .placeholder(R.drawable.restaurant_banner_placeholder)
                    .error(R.drawable.restaurant_banner_placeholder)
                    .into(holder.ivRestaurant);
        } else {
            Glide.with(activity).load(R.drawable.restaurant_banner_placeholder).into(holder.ivRestaurant);
        }
        if (!Validators.isNullOrEmpty(reservation.cancel_reason)) {
            holder.tvReservationCancelReason.setVisibility(View.VISIBLE);
            holder.tvReservationCancelReason.setText("Cancellation Reason: " + reservation.cancel_reason);
        } else {
            holder.tvReservationCancelReason.setVisibility(View.GONE);
        }
    }

    @Override
    public int getItemCount() {
        return reservations.size();
    }

    protected class ReservationViewHolder extends RecyclerView.ViewHolder {
        private TextView tvReservationDate, tvReservationId, tvRestaurant, tvReservationDateTime, tvReservationName, tvReservationEmail, tvReservationGuest, tvReservationPhone, tvReservationStatus, tvReservationCancelReason;
        private ImageView ivRestaurant;
        private View vReservation;
        private Button btnRebook, btnCancel;

        public ReservationViewHolder(@NonNull View itemView) {
            super(itemView);
            tvReservationDate = itemView.findViewById(R.id.tvReservationDate);
            tvReservationDateTime = itemView.findViewById(R.id.tvReservationDateTime);
            tvReservationName = itemView.findViewById(R.id.tvReservationName);
            tvReservationEmail = itemView.findViewById(R.id.tvReservationEmail);
            tvReservationPhone = itemView.findViewById(R.id.tvReservationPhone);
            tvReservationGuest = itemView.findViewById(R.id.tvReservationGuest);
            tvReservationStatus = itemView.findViewById(R.id.tvReservationStatus);
            tvReservationCancelReason = itemView.findViewById(R.id.tvReservationCancelReason);
            tvReservationId = itemView.findViewById(R.id.tvReservationId);
            tvRestaurant = itemView.findViewById(R.id.tvRestaurant);
            ivRestaurant = itemView.findViewById(R.id.ivRestaurantBanner);
            btnRebook = itemView.findViewById(R.id.btnRebook);
            btnCancel = itemView.findViewById(R.id.btnCancel);
        }
    }
}
