package com.tiffintom.adapters;

import android.content.Context;
import android.content.Intent;
import android.util.TypedValue;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.ImageView;
import android.widget.LinearLayout;
import android.widget.TextView;

import androidx.annotation.NonNull;
import androidx.cardview.widget.CardView;
import androidx.recyclerview.widget.RecyclerView;

import com.airbnb.lottie.LottieAnimationView;
import com.bumptech.glide.Glide;
import com.iarcuschin.simpleratingbar.SimpleRatingBar;
import com.tiffintom.MyApp;
import com.tiffintom.R;
import com.tiffintom.activity.AuthActivity;
import com.tiffintom.common.CommonFunctions;
import com.tiffintom.common.Validators;
import com.tiffintom.interfaces.RecyclerViewItemClickListener;
import com.tiffintom.models.Restaurant;

import java.util.ArrayList;

import static com.tiffintom.common.CommonFunctions.df;

/**
 * Created by Amrish on 19-08-2020.
 */
public class RestaurantAdapter extends RecyclerView.Adapter<RestaurantAdapter.RestaurantViewHolder> {
    private Context activity;
    private ArrayList<Restaurant> restaurants = new ArrayList<>();
    private RecyclerViewItemClickListener restaurantClickListener, favouriteClickListener;

    public RestaurantAdapter(ArrayList<Restaurant> restaurants, RecyclerViewItemClickListener favouriteClickListener, RecyclerViewItemClickListener restaurantClickListener) {
        this.restaurants = restaurants;
        this.favouriteClickListener = favouriteClickListener;
        this.restaurantClickListener = restaurantClickListener;
    }

    @NonNull
    @Override
    public RestaurantAdapter.RestaurantViewHolder onCreateViewHolder(@NonNull ViewGroup parent, int viewType) {
        activity = parent.getContext();
        return new RestaurantViewHolder(LayoutInflater.from(activity).inflate(R.layout.home_restaurant_item, parent, false));
    }

    @Override
    public void onBindViewHolder(@NonNull RestaurantAdapter.RestaurantViewHolder holder, int position) {
        activity = holder.itemView.getContext();
        Restaurant restaurant = restaurants.get(position);
        holder.tvRestaurant.setText(restaurant.restaurant_name);
        if (restaurant.deliveryCharges > 0)
        {
              holder.tvDeliveryFee.setText(CommonFunctions.StringAppender(MyApp.getInstance().getCurrencySymbol(), df.format(restaurant.deliveryCharges), " Del Fee"));

        } else {
            holder.tvDeliveryFee.setText("Free Del");
        }
        if (restaurant.delivery_charge != null && restaurant.delivery_charge.equalsIgnoreCase("Yes")) {
            holder.llFreeDelivery.setVisibility(View.VISIBLE);
        } else {
            holder.llFreeDelivery.setVisibility(View.INVISIBLE);
        }
        holder.tvMinOrder.setText(CommonFunctions.StringAppender(MyApp.getInstance().getCurrencySymbol(), df.format(restaurant.minimum_order), " min cost"));
        holder.tvDistance.setText(CommonFunctions.StringAppender(String.valueOf(restaurant.distance), " miles"));
        if (restaurant.online_order != null && restaurant.online_order.equalsIgnoreCase("no")) {
            holder.tvOnlineOrders.setVisibility(View.VISIBLE);
            holder.tvClosed.setVisibility(View.GONE);
            holder.tvPreorder.setVisibility(View.GONE);
        } else {
            holder.tvOnlineOrders.setVisibility(View.GONE);
            if (restaurant.currentStatus.equalsIgnoreCase("closed")) {
                holder.tvClosed.setVisibility(View.VISIBLE);
                holder.tvPreorder.setVisibility(View.GONE);
                holder.tvRestaurant.setTextSize(TypedValue.COMPLEX_UNIT_SP, 20);
            } else if (restaurant.currentStatus.equalsIgnoreCase("open")) {
                holder.tvClosed.setVisibility(View.GONE);
                holder.tvPreorder.setVisibility(View.GONE);
                holder.tvRestaurant.setTextSize(TypedValue.COMPLEX_UNIT_SP, 24);
            } else if (restaurant.currentStatus.equalsIgnoreCase("preorder") || restaurant.currentStatus.equalsIgnoreCase("pre-order")) {
                holder.tvClosed.setVisibility(View.VISIBLE);
                holder.tvPreorder.setVisibility(View.VISIBLE);
                holder.tvRestaurant.setTextSize(TypedValue.COMPLEX_UNIT_SP, 20);
            } else {
                holder.tvClosed.setVisibility(View.GONE);
                holder.tvPreorder.setVisibility(View.GONE);
            }
        }
        Glide.with(activity)
                .load(restaurant.image_url)
                .placeholder(R.drawable.restaurant_logo_placeholder)
                .error(R.drawable.restaurant_logo_placeholder)
                .into(holder.ivRestaurantLogo);
        holder.ratingBar.setRating(restaurant.average_rating);

        if (restaurant.promotions != null && restaurant.promotions.size() > 0 && !Validators.isNullOrEmpty(restaurant.promotions.get(0).promo_image)) {
            String bannerImage = restaurant.promotions.get(0).image_url;
            Glide.with(activity)
                    .load(bannerImage)
                    .placeholder(R.drawable.restaurant_banner_placeholder)
                    .error(R.drawable.restaurant_banner_placeholder)
                    .dontAnimate()
                    .into(holder.ivRestaurantBanner);
        } else {
            Glide.with(activity).load(R.drawable.restaurant_banner_placeholder).into(holder.ivRestaurantBanner);
        }
        if (restaurant.offer_texts != null && restaurant.offer_texts.size() > 0) {

            if (restaurant.currentStatus.equalsIgnoreCase("open")) {
                StringBuilder offerText = new StringBuilder();
                for (int i = 0; i < restaurant.offer_texts.size(); i++) {
                    offerText.append(restaurant.offer_texts.get(i).offer_name);
                    if (i != (restaurant.offer_texts.size() - 1)) {
                        offerText.append(", ");
                    }
                }
                holder.tvDiscount.setSelected(true);
                holder.tvDiscount.setText(offerText.toString());
                holder.tvDiscount.setVisibility(View.VISIBLE);
            } else
                holder.tvDiscount.setVisibility(View.INVISIBLE);
        } else {
            holder.tvDiscount.setSelected(false);
            holder.tvDiscount.setVisibility(View.INVISIBLE);
        }
        if (restaurant.is_favourite==1) {
            holder.ivFavouriteImage.setImageResource(R.drawable.ic_baseline_favorite_24);
        } else {
            holder.ivFavouriteImage.setImageResource(R.drawable.icon_favourite);
        }
        holder.ivFavouriteImage.setOnClickListener(view -> {
            if (MyApp.getInstance().getMyPreferences().getLoggedInUser() == null) {
                activity.startActivity(new Intent(activity, AuthActivity.class));
            } else {
                restaurant.is_favourite = !(restaurant.is_favourite==1) ? 1:0;
                if (restaurant.is_favourite==1) {
                    holder.ivFavourite.playAnimation();
                    holder.ivFavouriteImage.setImageResource(R.drawable.ic_baseline_favorite_24);
                } else
                {
                    holder.ivFavourite.setProgress(0);
                    holder.ivFavourite.cancelAnimation();
                    holder.ivFavouriteImage.setImageResource(R.drawable.icon_favourite);
                }
                if (favouriteClickListener != null) {
                    favouriteClickListener.onItemClick(position, restaurant);
                }
            }
        });
    }

    @Override
    public int getItemCount() {
        return restaurants.size();
    }

    protected class RestaurantViewHolder extends RecyclerView.ViewHolder {
        private TextView tvRestaurant, tvDiscount, tvClosed, tvPreorder, tvMinOrder, tvDistance, tvDeliveryFee, tvOnlineOrders;
        private SimpleRatingBar ratingBar;
        private CardView cvRestaurant;
        private ImageView ivRestaurantBanner, ivRestaurantLogo, ivFavouriteImage;
        private LottieAnimationView ivFavourite;
        private LinearLayout llMinOrder, llDistance, llFreeDelivery;

        public RestaurantViewHolder(@NonNull View itemView) {
            super(itemView);

            tvRestaurant = itemView.findViewById(R.id.tvRestaurant);
            tvClosed = itemView.findViewById(R.id.tvClosed);
            tvOnlineOrders = itemView.findViewById(R.id.tvOnlineOrders);
            tvPreorder = itemView.findViewById(R.id.tvPreOrder);
            tvDiscount = itemView.findViewById(R.id.tvDiscount);
            tvMinOrder = itemView.findViewById(R.id.tvMinOrder);
            tvDistance = itemView.findViewById(R.id.tvDistance);
            tvDeliveryFee = itemView.findViewById(R.id.tvDeliveryFee);
            ratingBar = itemView.findViewById(R.id.rating);

            ivFavourite = itemView.findViewById(R.id.ivFavourite);
            ivFavouriteImage = itemView.findViewById(R.id.ivFavouriteImage);

            ivRestaurantBanner = itemView.findViewById(R.id.ivRestaurantBanner);
            ivRestaurantLogo = itemView.findViewById(R.id.ivRestaurantLogo);

            cvRestaurant = itemView.findViewById(R.id.cvRestaurant);

            llDistance = itemView.findViewById(R.id.llDistance);
            llFreeDelivery = itemView.findViewById(R.id.llDeliveryFee);
            llMinOrder = itemView.findViewById(R.id.llMinOrder);

            cvRestaurant.setOnClickListener(view -> {
                if (restaurantClickListener != null) {
                    restaurantClickListener.onItemClick(getLayoutPosition(), restaurants.get(getLayoutPosition()));
                }
            });
        }
    }
}
