package com.tiffintom.adapters;

import static com.tiffintom.MyApp.isDineInOpen;
import static com.tiffintom.activity.RestaurantDetailsActivity.isDineIn;

import android.app.Activity;
import android.graphics.Paint;
import android.os.SystemClock;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.FrameLayout;
import android.widget.ImageView;
import android.widget.RelativeLayout;
import android.widget.TextView;

import androidx.annotation.NonNull;
import androidx.core.content.ContextCompat;
import androidx.recyclerview.widget.GridLayoutManager;
import androidx.recyclerview.widget.RecyclerView;

import com.bumptech.glide.Glide;
import com.makeramen.roundedimageview.RoundedImageView;
import com.tiffintom.MyApp;
import com.tiffintom.R;
import com.tiffintom.common.Validators;
import com.tiffintom.interfaces.RecyclerViewItemClickListener;
import com.tiffintom.models.Allergy;
import com.tiffintom.models.AllergyCaution;
import com.tiffintom.models.Category;
import com.tiffintom.models.DineinModels.DineinMenus;
import com.tiffintom.models.Header;
import com.tiffintom.models.Menu;
import com.tiffintom.models.Variant;

import java.util.ArrayList;

/**
 * Created by Amrish on 19-08-2020.
 */
public class RestaurantDineInMenuAdapter extends RecyclerView.Adapter<RecyclerView.ViewHolder> {
    private Activity activity;
    private ArrayList<Object> menuItems = new ArrayList<>();
    private RecyclerViewItemClickListener recyclerViewItemClickListener;
    public boolean showMenuImage;
    private String currency = MyApp.getInstance().getCurrencySymbol();
    final long MIN_CLICK_INTERVAL = 1000; //in millis
    long lastClickTime = 0;
    private MyApp myApp = MyApp.getInstance();

    public RestaurantDineInMenuAdapter(Activity activity, ArrayList<Object> restaurants, boolean showMenuImage, RecyclerViewItemClickListener recyclerViewItemClickListener) {
        this.activity = activity;
        this.menuItems = restaurants;
        this.recyclerViewItemClickListener = recyclerViewItemClickListener;
        this.showMenuImage = showMenuImage;
    }

    @Override
    public int getItemViewType(int position) {
        if (menuItems.get(position) instanceof Header) {
            return R.layout.restaurant_menu_header_main;
        } else if (menuItems.get(position) instanceof Category) {
            return R.layout.restaurant_menu_header_category;
        } else if (menuItems.get(position) instanceof Allergy) {
            return R.layout.restaurant_allergy_recyclerview;
        } else if (menuItems.get(position) instanceof AllergyCaution) {
            return R.layout.allergy_caution_item;
        } else {
            return R.layout.restaurant_menu_item;
        }
    }

    @NonNull
    @Override
    public RecyclerView.ViewHolder onCreateViewHolder(@NonNull ViewGroup parent, int viewType) {
        View view = LayoutInflater.from(parent.getContext()).inflate(viewType, parent, false);
        if (viewType == R.layout.restaurant_menu_header_category) {
            return new RestaurantMenuHeaderViewHolder(view);
        } else if (viewType == R.layout.restaurant_allergy_recyclerview) {
            return new AllergiesViewHolder(view);
        } else if (viewType == R.layout.allergy_caution_item) {
            return new AllergyCautionViewHolder(view);
        } else if (viewType == R.layout.restaurant_menu_header_main) {
            return new RestaurantMenuHeaderViewHolder(view);
        } else {
            return new RestaurantMenuViewHolder(view);
        }
    }

    @Override
    public void onBindViewHolder(@NonNull RecyclerView.ViewHolder holder, int position) {
        if (getItemViewType(position) == R.layout.restaurant_menu_item) {
            RestaurantMenuViewHolder menuHolder = (RestaurantMenuViewHolder) holder;
            if (menuItems.get(position) instanceof Menu) {
                Menu menuItem = (Menu) menuItems.get(position);
                menuHolder.tvMenuItemTitle.setText(menuItem.menu_name);
                menuHolder.tvMenuItemDescription.setText(menuItem.menu_description);
                if (showMenuImage && !Validators.isNullOrEmpty(menuItem.image_url)) {

                    menuHolder.ivMenuItem.setVisibility(View.VISIBLE);
                    Glide.with(activity).load(menuItem.image_url)
                            .placeholder(R.drawable.menu_item_placeholder)
                            .error(R.drawable.menu_item_placeholder)
                            .into(menuHolder.ivMenuItem);
                } else {
                    menuHolder.ivMenuItem.setVisibility(View.GONE);
                }
                if (myApp.getMyPreferences().getCurrentDineInRestaurantDetail().online_order.equalsIgnoreCase("no")) {
                    menuHolder.ivAddToCart.setVisibility(View.GONE);
                } else {
                    menuHolder.ivAddToCart.setVisibility(View.VISIBLE);
                }
                menuHolder.tvMenuItemCartCount.setVisibility(View.GONE);
                new Thread(() -> {
                    int counts;
                    if (isDineInOpen) {
                        counts = MyApp.getInstance().getAppDatabase().dineinCartItemDao().getMenuItemCartCounts(menuItem.id);
                    } else {
                        counts = MyApp.getInstance().getAppDatabase().cartDao().getMenuItemCartCounts(menuItem.id);
                    }
                    int finalCounts = counts;
                    activity.runOnUiThread(() -> {
                        if (counts > 0) {
                            menuHolder.tvMenuItemCartCount.setVisibility(View.VISIBLE);
                            menuHolder.tvMenuItemCartCount.setText(String.valueOf(counts));
                        } else {
                            menuHolder.tvMenuItemCartCount.setVisibility(View.GONE);
                        }
                    });
                }).start();

                if (isDineIn) {
                    if (menuItem.variants != null && menuItem.variants.size() > 0) {
                        Variant variant = menuItem.variants.get(0);
                        if (menuItem.getProductPercentage() > 0) {
                            float discountedPrice = variant.orginal_price - (variant.orginal_price * menuItem.getProductPercentage() / 100);
                            menuHolder.tvMenuItemPrice.setText(currency + MyApp.df.format(discountedPrice));
                            menuHolder.tvStrikedPrice.setText(currency + MyApp.df.format(variant.orginal_price));
                            menuHolder.tvStrikedPrice.setVisibility(View.VISIBLE);
                        } else {
                            menuHolder.tvMenuItemPrice.setText(currency + MyApp.df.format(variant.orginal_price));
                            menuHolder.tvStrikedPrice.setVisibility(View.GONE);
                        }
                    } else {
                        menuHolder.tvMenuItemPrice.setText("");
                        menuHolder.tvStrikedPrice.setVisibility(View.GONE);
                    }
                } else {
                    if (menuItem != null) {
                        if (menuItem.variants != null && menuItem.variants.size() > 0) {
                            Variant variant = menuItem.variants.get(0);
                            if (menuItem.getProductPercentage() > 0) {
                                float discountedPrice = variant.orginal_price - (variant.orginal_price * menuItem.getProductPercentage() / 100);
                                menuHolder.tvMenuItemPrice.setText(currency + MyApp.df.format(discountedPrice));
                                menuHolder.tvStrikedPrice.setText(currency + MyApp.df.format(variant.orginal_price));
                                menuHolder.tvStrikedPrice.setVisibility(View.VISIBLE);
                            } else {
                                menuHolder.tvStrikedPrice.setVisibility(View.GONE);
                                menuHolder.tvMenuItemPrice.setText(currency + MyApp.df.format(variant.orginal_price));
                            }
                        } else {
                            menuHolder.tvMenuItemPrice.setText("");
                            menuHolder.tvStrikedPrice.setVisibility(View.GONE);
                        }

                    } else {
                        menuHolder.tvMenuItemPrice.setText("");
                        menuHolder.tvStrikedPrice.setVisibility(View.GONE);
                    }
                }

                menuHolder.rlMenuItem.setOnClickListener(view -> {
                    long currentTime = SystemClock.elapsedRealtime();
                    if (currentTime - lastClickTime > MIN_CLICK_INTERVAL) {
                        lastClickTime = currentTime;
                        if (recyclerViewItemClickListener != null) {
                            menuItem.showImage = this.showMenuImage;
                            recyclerViewItemClickListener.onItemClick(position, menuItem);
                        }
                    }
                });
                menuHolder.ivVegan.setVisibility(View.GONE);
                menuHolder.ivVegetarian.setVisibility(View.GONE);
                menuHolder.ivDairy.setVisibility(View.GONE);
                menuHolder.ivFish.setVisibility(View.GONE);
                menuHolder.ivHalal.setVisibility(View.GONE);
                menuHolder.ivNuts.setVisibility(View.GONE);
                menuHolder.ivGluten.setVisibility(View.GONE);
                menuHolder.ivEggs.setVisibility(View.GONE);
                menuHolder.ivCrab.setVisibility(View.GONE);
                menuHolder.ivShell.setVisibility(View.GONE);
                menuHolder.ivSulphites.setVisibility(View.GONE);
                menuHolder.ivSo2.setVisibility(View.GONE);
                menuHolder.ivMustard.setVisibility(View.GONE);
                menuHolder.ivKoshar.setVisibility(View.GONE);


                if (isDineInOpen) {
                    if (!Validators.isNullOrEmpty(menuItem.vegan)) {
                        if (menuItem.vegan.equalsIgnoreCase("1"))
                            menuHolder.ivVegan.setVisibility(View.VISIBLE);
                    } else {
                        menuHolder.ivVegan.setVisibility(View.GONE);
                    }
                    if (!Validators.isNullOrEmpty(menuItem.vegetarian)) {
                        if (menuItem.vegetarian.equalsIgnoreCase("1"))
                            menuHolder.ivVegetarian.setVisibility(View.VISIBLE);
                    } else {
                        menuHolder.ivVegetarian.setVisibility(View.GONE);
                    }
                    if (!Validators.isNullOrEmpty(menuItem.milk)) {
                        if (menuItem.milk.equalsIgnoreCase("1"))
                            menuHolder.ivDairy.setVisibility(View.VISIBLE);
                    } else {
                        menuHolder.ivDairy.setVisibility(View.GONE);
                    }
                    if (!Validators.isNullOrEmpty(menuItem.fish)) {
                        if (menuItem.fish.equalsIgnoreCase("1"))
                            menuHolder.ivFish.setVisibility(View.VISIBLE);
                    } else {
                        menuHolder.ivFish.setVisibility(View.GONE);
                    }
                    if (!Validators.isNullOrEmpty(menuItem.halal)) {
                        if (menuItem.halal.equalsIgnoreCase("1"))
                            menuHolder.ivHalal.setVisibility(View.VISIBLE);
                    } else {
                        menuHolder.ivHalal.setVisibility(View.GONE);
                    }
                    if (!Validators.isNullOrEmpty(menuItem.contains_nuts)) {
                        if (menuItem.contains_nuts.equalsIgnoreCase("1"))
                            menuHolder.ivNuts.setVisibility(View.VISIBLE);
                    } else {
                        menuHolder.ivNuts.setVisibility(View.GONE);
                    }
                    if (!Validators.isNullOrEmpty(menuItem.gluten_free)) {
                        if (menuItem.gluten_free.equalsIgnoreCase("1"))
                            menuHolder.ivGluten.setVisibility(View.VISIBLE);
                    } else {
                        menuHolder.ivGluten.setVisibility(View.GONE);
                    }
                    if (!Validators.isNullOrEmpty(menuItem.eggs)) {
                        if (menuItem.eggs.equalsIgnoreCase("1"))
                            menuHolder.ivEggs.setVisibility(View.VISIBLE);
                    } else {
                        menuHolder.ivEggs.setVisibility(View.GONE);
                    }
                    if (!Validators.isNullOrEmpty(menuItem.crustaceans)) {
                        if (menuItem.crustaceans.equalsIgnoreCase("1"))
                            menuHolder.ivCrab.setVisibility(View.VISIBLE);
                    } else {
                        menuHolder.ivCrab.setVisibility(View.GONE);
                    }
                    if (!Validators.isNullOrEmpty(menuItem.molluscs)) {
                        if (menuItem.molluscs.equalsIgnoreCase("1"))
                            menuHolder.ivShell.setVisibility(View.VISIBLE);
                    } else {
                        menuHolder.ivShell.setVisibility(View.GONE);
                    }
                    if (!Validators.isNullOrEmpty(menuItem.sulphites)) {
                        if (menuItem.sulphites.equalsIgnoreCase("1"))
                            menuHolder.ivSulphites.setVisibility(View.VISIBLE);
                    } else {
                        menuHolder.ivSulphites.setVisibility(View.GONE);
                    }
                    if (!Validators.isNullOrEmpty(menuItem.sulphur_dioxide)) {
                        if (menuItem.sulphur_dioxide.equalsIgnoreCase("1"))
                            menuHolder.ivSo2.setVisibility(View.VISIBLE);
                    } else {
                        menuHolder.ivSo2.setVisibility(View.GONE);
                    }
                    if (!Validators.isNullOrEmpty(menuItem.mustard)) {
                        if (menuItem.mustard.equalsIgnoreCase("1"))
                            menuHolder.ivMustard.setVisibility(View.VISIBLE);
                    } else {
                        menuHolder.ivMustard.setVisibility(View.GONE);
                    }
                    if (!Validators.isNullOrEmpty(menuItem.koshar)) {
                        if (menuItem.koshar.equalsIgnoreCase("1"))
                            menuHolder.ivKoshar.setVisibility(View.VISIBLE);
                    } else {
                        menuHolder.ivKoshar.setVisibility(View.GONE);
                    }
                } else {
                    if (!Validators.isNullOrEmpty(menuItem.vegan)) {
                        if (menuItem.vegan.equalsIgnoreCase("true"))
                            menuHolder.ivVegan.setVisibility(View.VISIBLE);
                    } else {
                        menuHolder.ivVegan.setVisibility(View.GONE);
                    }
                    if (!Validators.isNullOrEmpty(menuItem.vegetarian)) {
                        if (menuItem.vegetarian.equalsIgnoreCase("true"))
                            menuHolder.ivVegetarian.setVisibility(View.VISIBLE);
                    } else {
                        menuHolder.ivVegetarian.setVisibility(View.GONE);
                    }
                    if (!Validators.isNullOrEmpty(menuItem.milk)) {
                        if (menuItem.milk.equalsIgnoreCase("true"))
                            menuHolder.ivDairy.setVisibility(View.VISIBLE);
                    } else {
                        menuHolder.ivDairy.setVisibility(View.GONE);
                    }
                    if (!Validators.isNullOrEmpty(menuItem.fish)) {
                        if (menuItem.fish.equalsIgnoreCase("true"))
                            menuHolder.ivFish.setVisibility(View.VISIBLE);
                    } else {
                        menuHolder.ivFish.setVisibility(View.GONE);
                    }
                    if (!Validators.isNullOrEmpty(menuItem.halal)) {
                        if (menuItem.halal.equalsIgnoreCase("true"))
                            menuHolder.ivHalal.setVisibility(View.VISIBLE);
                    } else {
                        menuHolder.ivHalal.setVisibility(View.GONE);
                    }
                    if (!Validators.isNullOrEmpty(menuItem.contains_nuts)) {
                        if (menuItem.contains_nuts.equalsIgnoreCase("true"))
                            menuHolder.ivNuts.setVisibility(View.VISIBLE);
                    } else {
                        menuHolder.ivNuts.setVisibility(View.GONE);
                    }
                    if (!Validators.isNullOrEmpty(menuItem.gluten_free)) {
                        if (menuItem.gluten_free.equalsIgnoreCase("true"))
                            menuHolder.ivGluten.setVisibility(View.VISIBLE);
                    } else {
                        menuHolder.ivGluten.setVisibility(View.GONE);
                    }
                    if (!Validators.isNullOrEmpty(menuItem.eggs)) {
                        if (menuItem.eggs.equalsIgnoreCase("true"))
                            menuHolder.ivEggs.setVisibility(View.VISIBLE);
                    } else {
                        menuHolder.ivEggs.setVisibility(View.GONE);
                    }
                    if (!Validators.isNullOrEmpty(menuItem.crustaceans)) {
                        if (menuItem.crustaceans.equalsIgnoreCase("true"))
                            menuHolder.ivCrab.setVisibility(View.VISIBLE);
                    } else {
                        menuHolder.ivCrab.setVisibility(View.GONE);
                    }
                    if (!Validators.isNullOrEmpty(menuItem.molluscs)) {
                        if (menuItem.molluscs.equalsIgnoreCase("true"))
                            menuHolder.ivShell.setVisibility(View.VISIBLE);
                    } else {
                        menuHolder.ivShell.setVisibility(View.GONE);
                    }
                    if (!Validators.isNullOrEmpty(menuItem.sulphites)) {
                        if (menuItem.sulphites.equalsIgnoreCase("true"))
                            menuHolder.ivSulphites.setVisibility(View.VISIBLE);
                    } else {
                        menuHolder.ivSulphites.setVisibility(View.GONE);
                    }
                    if (!Validators.isNullOrEmpty(menuItem.sulphur_dioxide)) {
                        if (menuItem.sulphur_dioxide.equalsIgnoreCase("true"))
                            menuHolder.ivSo2.setVisibility(View.VISIBLE);
                    } else {
                        menuHolder.ivSo2.setVisibility(View.GONE);
                    }
                    if (!Validators.isNullOrEmpty(menuItem.mustard)) {
                        if (menuItem.mustard.equalsIgnoreCase("true"))
                            menuHolder.ivMustard.setVisibility(View.VISIBLE);
                    } else {
                        menuHolder.ivMustard.setVisibility(View.GONE);
                    }
                    if (!Validators.isNullOrEmpty(menuItem.koshar)) {
                        if (menuItem.koshar.equalsIgnoreCase("true"))
                            menuHolder.ivKoshar.setVisibility(View.VISIBLE);
                    } else {
                        menuHolder.ivKoshar.setVisibility(View.GONE);
                    }
                }
            } else if (menuItems.get(position) instanceof DineinMenus) {
                DineinMenus menuItem = (DineinMenus) menuItems.get(position);
                menuHolder.tvMenuItemTitle.setText(menuItem.name);
                menuHolder.tvMenuItemDescription.setText(menuItem.description);
                menuHolder.tvMenuItemPrice.setText(currency + MyApp.df.format(menuItem.price));

                if (showMenuImage && !Validators.isNullOrEmpty(menuItem.image_url)) {

                    menuHolder.ivMenuItem.setVisibility(View.VISIBLE);
                    Glide.with(activity).load(menuItem.image_url)
                            .placeholder(R.drawable.menu_item_placeholder)
                            .error(R.drawable.menu_item_placeholder)
                            .into(menuHolder.ivMenuItem);
                } else {
                    menuHolder.ivMenuItem.setVisibility(View.GONE);
                }
                if (myApp.getMyPreferences().getCurrentDineInRestaurantDetail().online_order.equalsIgnoreCase("no")) {
                    menuHolder.ivAddToCart.setVisibility(View.GONE);
                } else {
                    menuHolder.ivAddToCart.setVisibility(View.VISIBLE);
                }
                menuHolder.tvMenuItemCartCount.setVisibility(View.GONE);
                new Thread(() -> {
                    int counts;
                    counts = MyApp.getInstance().getAppDatabase().dineinCartItemDao().getMenuItemCartCounts(menuItem.id);

                    int finalCounts = counts;
                    activity.runOnUiThread(() -> {
                        if (counts > 0) {
                            menuHolder.tvMenuItemCartCount.setVisibility(View.VISIBLE);
                            menuHolder.tvMenuItemCartCount.setText(String.valueOf(counts));
                        } else {
                            menuHolder.tvMenuItemCartCount.setVisibility(View.GONE);
                        }
                    });
                }).start();

                menuHolder.rlMenuItem.setOnClickListener(view -> {
                    long currentTime = SystemClock.elapsedRealtime();
                    if (currentTime - lastClickTime > MIN_CLICK_INTERVAL) {
                        lastClickTime = currentTime;
                        if (recyclerViewItemClickListener != null) {
                            menuItem.showImage = this.showMenuImage;
                            recyclerViewItemClickListener.onItemClick(position, menuItem);
                        }
                    }
                });
            }
        }
        if (getItemViewType(position) == R.layout.restaurant_menu_header_main) {
            Header header = (Header) menuItems.get(position);
            RestaurantMenuHeaderViewHolder menuHeaderViewHolder = (RestaurantMenuHeaderViewHolder) holder;
            menuHeaderViewHolder.tvTitle.setText(header.title);
            ((RestaurantMenuHeaderViewHolder) holder).ivSearch.setOnClickListener(view -> {
                header.addonId = 1;
                if (recyclerViewItemClickListener != null)
                    recyclerViewItemClickListener.onItemClick(position, header);
            });
        }
        if (

                getItemViewType(position) == R.layout.restaurant_menu_header_category) {
            Category category = (Category) menuItems.get(position);
            RestaurantMenuHeaderViewHolder menuHeaderViewHolder = (RestaurantMenuHeaderViewHolder) holder;
            menuHeaderViewHolder.tvTitle.setText(category.category_name);
        }
        if (

                getItemViewType(position) == R.layout.restaurant_allergy_recyclerview) {
            Allergy allergy = (Allergy) menuItems.get(position);
            AllergiesViewHolder viewHolder = (AllergiesViewHolder) holder;
            int spanCount = 7;
            if (allergy.allergies == null || allergy.allergies.size() == 0) {
                spanCount = 1;
            } else {
                spanCount = Math.min(allergy.allergies.size(), 7);
            }
            viewHolder.rvAllergies.setLayoutManager(new GridLayoutManager(viewHolder.itemView.getContext(), 7));
            viewHolder.rvAllergies.setAdapter(new AllergiesAdapter(allergy.allergies, (position1, data) -> {
                recyclerViewItemClickListener.onItemClick(position, "more");
            }, (position12, data) -> {
                recyclerViewItemClickListener.onItemClick(position, "less");
            }));
        }
        if (

                getItemViewType(position) == R.layout.allergy_caution_item) {
            AllergyCaution caution = (AllergyCaution) menuItems.get(position);
            AllergyCautionViewHolder cautionViewHolder = (AllergyCautionViewHolder) holder;
            cautionViewHolder.tvText.setText(caution.text);
            if (caution.icon == 0) {
                cautionViewHolder.ivInfo.setVisibility(View.GONE);
                cautionViewHolder.tvText.setTextColor(ContextCompat.getColor(activity, R.color.black));
            } else {
                cautionViewHolder.tvText.setTextColor(ContextCompat.getColor(activity, R.color.dark_grey_color));
            }
        }

    }

    @Override
    public int getItemCount() {
        return menuItems.size();
    }

    protected static class RestaurantMenuViewHolder extends RecyclerView.ViewHolder {
        private TextView tvMenuItemTitle, tvMenuItemPrice, tvMenuItemDescription, tvMenuItemCartCount, tvStrikedPrice;
        private ImageView ivAddToCart;
        private RoundedImageView ivMenuItem;
        private FrameLayout rlMenuItem;
        private ImageView ivVegan, ivHalal, ivVegetarian, ivNuts, ivGluten, ivDairy, ivFish, ivEggs, ivCrab, ivSo2, ivSulphites, ivShell, ivMustard, ivKoshar;


        public RestaurantMenuViewHolder(@NonNull View itemView) {
            super(itemView);
            tvMenuItemCartCount = itemView.findViewById(R.id.tvMenuItemCartCount);
            tvMenuItemDescription = itemView.findViewById(R.id.tvMenuItemDescription);
            tvMenuItemPrice = itemView.findViewById(R.id.tvMenuPrice);
            tvStrikedPrice = itemView.findViewById(R.id.tvStrikedPrice);
            tvMenuItemTitle = itemView.findViewById(R.id.tvMenuTitle);
            ivAddToCart = itemView.findViewById(R.id.ivAddToCart);
            ivMenuItem = itemView.findViewById(R.id.ivMenuItem);
            rlMenuItem = itemView.findViewById(R.id.cvMenuItem);

            ivVegan = itemView.findViewById(R.id.ivVegan);
            ivHalal = itemView.findViewById(R.id.ivHalal);
            ivVegetarian = itemView.findViewById(R.id.ivVegetarian);
            ivNuts = itemView.findViewById(R.id.ivNuts);
            ivGluten = itemView.findViewById(R.id.ivGluten);
            ivDairy = itemView.findViewById(R.id.ivDairy);
            ivFish = itemView.findViewById(R.id.ivFish);
            ivEggs = itemView.findViewById(R.id.ivEggs);
            ivCrab = itemView.findViewById(R.id.ivCrab);
            ivSo2 = itemView.findViewById(R.id.ivSo2);
            ivSulphites = itemView.findViewById(R.id.ivSulphites);
            ivShell = itemView.findViewById(R.id.ivShell);
            ivMustard = itemView.findViewById(R.id.ivMustard);
            ivKoshar = itemView.findViewById(R.id.ivKoshar);

            tvStrikedPrice.setPaintFlags(Paint.STRIKE_THRU_TEXT_FLAG);
        }
    }

    protected static class RestaurantMenuHeaderViewHolder extends RecyclerView.ViewHolder {
        private TextView tvTitle;
        private ImageView ivSearch;

        public RestaurantMenuHeaderViewHolder(View view) {
            super(view);
            tvTitle = view.findViewById(R.id.tvTitle);
            ivSearch = view.findViewById(R.id.ivSearch);
        }
    }

    protected static class AllergiesViewHolder extends RecyclerView.ViewHolder {
        private RecyclerView rvAllergies;

        public AllergiesViewHolder(@NonNull View itemView) {
            super(itemView);
            rvAllergies = itemView.findViewById(R.id.rvAllergies);
            rvAllergies.setNestedScrollingEnabled(false);
        }
    }

    protected static class AllergyCautionViewHolder extends RecyclerView.ViewHolder {
        private TextView tvText;
        private ImageView ivInfo;

        public AllergyCautionViewHolder(@NonNull View itemView) {
            super(itemView);
            tvText = itemView.findViewById(R.id.tvText);
            ivInfo = itemView.findViewById(R.id.ivInfo);
        }
    }
}
