package com.tiffintom.adapters;

import android.app.Activity;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.LinearLayout;
import android.widget.TextView;

import androidx.annotation.NonNull;
import androidx.recyclerview.widget.RecyclerView;

import com.iarcuschin.simpleratingbar.SimpleRatingBar;
import com.tiffintom.R;
import com.tiffintom.common.CommonFunctions;
import com.tiffintom.common.Validators;
import com.tiffintom.interfaces.RecyclerViewItemClickListener;
import com.tiffintom.models.Review;
import com.tiffintom.utils.LogUtils;

import java.util.ArrayList;

/**
 * Created by Amrish on 19-08-2020.
 */
public class RestaurantReviewsAdapter extends RecyclerView.Adapter<RestaurantReviewsAdapter.RestaurantViewHolder> {
    private Activity activity;
    private ArrayList<Review> reviews = new ArrayList<>();
    private RecyclerViewItemClickListener recyclerViewItemClickListener;

    public RestaurantReviewsAdapter(Activity activity, ArrayList<Review> reviews, RecyclerViewItemClickListener recyclerViewItemClickListener) {
        this.activity = activity;
        this.reviews = reviews;
        this.recyclerViewItemClickListener = recyclerViewItemClickListener;
    }

    @NonNull
    @Override
    public RestaurantReviewsAdapter.RestaurantViewHolder onCreateViewHolder(@NonNull ViewGroup parent, int viewType) {
        return new RestaurantViewHolder(LayoutInflater.from(activity).inflate(R.layout.restaurant_review_item, parent, false));
    }

    @Override
    public void onBindViewHolder(@NonNull RestaurantReviewsAdapter.RestaurantViewHolder holder, int position) {
        try {
            Review review = reviews.get(position);
            holder.tvReview.setText(review.message);
            if (!Validators.isNullOrEmpty(review.responce)) {
                holder.llRestaurantReply.setVisibility(View.VISIBLE);
                holder.tvRestaurantReply.setText(review.responce);
            } else {
                holder.llRestaurantReply.setVisibility(View.GONE);
            }
            //LogUtils.e("USER::", review.user == null ? " NULL" : review.user.first_name);
            if (review.first_name != null) {
                holder.tvRatingUserName.setVisibility(View.VISIBLE);
                holder.tvRatingUserName.setText(review.first_name);
            } else {
                holder.tvRatingUserName.setVisibility(View.GONE);
            }
            holder.userRating.setRating(review.rating);
            holder.tvDate.setText(CommonFunctions.formatTimestampZulu(review.created, "dd/MM/yyyy"));
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    @Override
    public int getItemCount() {
        return reviews.size();
    }

    protected class RestaurantViewHolder extends RecyclerView.ViewHolder {
        private TextView tvReview, tvRatingUserName, tvRestaurantReply, tvDate;
        private SimpleRatingBar userRating;
        private LinearLayout llRestaurantReply;

        public RestaurantViewHolder(@NonNull View itemView) {
            super(itemView);
            tvDate = itemView.findViewById(R.id.tvReviewDate);
            tvRestaurantReply = itemView.findViewById(R.id.tvRestaurantReply);
            tvReview = itemView.findViewById(R.id.tvReview);
            tvRatingUserName = itemView.findViewById(R.id.tvRatingUserName);
            llRestaurantReply = itemView.findViewById(R.id.llRestaurantReply);
            userRating = itemView.findViewById(R.id.userRating);
        }
    }
}
