package com.tiffintom.adapters;

import android.app.Activity;
import android.content.Context;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.TextView;

import androidx.annotation.NonNull;
import androidx.viewpager.widget.PagerAdapter;
import androidx.viewpager.widget.ViewPager;

import com.tiffintom.R;
import com.tiffintom.interfaces.RecyclerViewItemClickListener;
import com.tiffintom.models.SavedCard;

import java.util.ArrayList;

public class SaveCardsUIViewPagerAdapter extends PagerAdapter {

    private Activity activity;
    private ArrayList<SavedCard> savedCards = new ArrayList<>();
    private int selectedSavedCard = 0;
    private RecyclerViewItemClickListener recyclerViewItemClickListener;
    private int lastCheckedPosition = -1, checkedPosition = -1;
    private LayoutInflater layoutInflater;


    public SaveCardsUIViewPagerAdapter(Activity activity, ArrayList<SavedCard> savedCards, int selectedSavedCard, RecyclerViewItemClickListener recyclerViewItemClickListener) {
        this.activity = activity;
        this.savedCards = savedCards;
        this.selectedSavedCard = selectedSavedCard;
        this.recyclerViewItemClickListener = recyclerViewItemClickListener;

    }

    @Override
    public int getCount() {
        return savedCards.size();
    }

    @Override
    public int getItemPosition(@NonNull Object object) {
        return POSITION_NONE;
    }

    @Override
    public boolean isViewFromObject(@NonNull View view, @NonNull Object object) {
        return view == object;
    }

    @NonNull
    @Override
    public Object instantiateItem(@NonNull ViewGroup container, int position) {
        layoutInflater = (LayoutInflater) activity.getSystemService(Context.LAYOUT_INFLATER_SERVICE);
        View view = layoutInflater.inflate(R.layout.item_saved_card_ui, null);

        SavedCard savedCard = savedCards.get(position);
        TextView tvCardNumber = view.findViewById(R.id.tvCardNumber);
        TextView tvCardHolder = view.findViewById(R.id.tvCardHolder);
        TextView tvCardExpiry = view.findViewById(R.id.tvCardExpiry);
        TextView tvRemove = view.findViewById(R.id.tvRemove);

        tvCardNumber.setText("**** **** **** " + savedCard.card_number);
        tvCardHolder.setText(savedCard.customer_name);
        tvCardExpiry.setText(String.format("%02d", savedCard.exp_month) + "/" + savedCard.exp_year);

        tvRemove.setOnClickListener(view1 -> {
            if (recyclerViewItemClickListener != null)
                recyclerViewItemClickListener.onItemClick(position, savedCard);
        });
        container.addView(view);
        return view;
    }

    @Override
    public void destroyItem(@NonNull ViewGroup container, int position, @NonNull Object object) {
        ViewPager vp = (ViewPager) container;
        View view = (View) object;
        vp.removeView(view);
    }
}
