package com.tiffintom.adapters;

import android.app.Activity;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.ImageView;
import android.widget.LinearLayout;
import android.widget.RadioButton;
import android.widget.TextView;

import androidx.annotation.NonNull;
import androidx.recyclerview.widget.RecyclerView;

import com.chauthai.swipereveallayout.SwipeRevealLayout;
import com.chauthai.swipereveallayout.ViewBinderHelper;
import com.tiffintom.MyApp;
import com.tiffintom.R;
import com.tiffintom.interfaces.RecyclerViewItemClickListener;
import com.tiffintom.models.PaymentMethod;
import com.tiffintom.models.SavedCard;

import java.util.ArrayList;

/**
 * Created by Amrish on 19-08-2020.
 */
public class SavedCardsAdapter extends RecyclerView.Adapter<SavedCardsAdapter.SavedCardViewHolder> {
    private Activity activity;
    private ArrayList<SavedCard> savedCards = new ArrayList<>();
    private int selectedSavedCard = 0;
    private RecyclerViewItemClickListener recyclerViewItemClickListener, removeCardListener;
    private int lastCheckedPosition = -1, checkedPosition = -1;
    private RadioButton checkedRadio;
    private final ViewBinderHelper viewBinderHelper = new ViewBinderHelper();

    public SavedCardsAdapter(Activity activity, ArrayList<SavedCard> savedCards, int selectedSavedCard, RecyclerViewItemClickListener recyclerViewItemClickListener, RecyclerViewItemClickListener removeCardListener) {
        this.activity = activity;
        this.savedCards = savedCards;
        this.selectedSavedCard = selectedSavedCard;
        this.recyclerViewItemClickListener = recyclerViewItemClickListener;
        this.removeCardListener = removeCardListener;
    }

    @NonNull
    @Override
    public SavedCardsAdapter.SavedCardViewHolder onCreateViewHolder(@NonNull ViewGroup parent, int viewType) {
        return new SavedCardViewHolder(LayoutInflater.from(activity).inflate(R.layout.item_saved_card, parent, false));
    }

    @Override
    public void onBindViewHolder(@NonNull SavedCardsAdapter.SavedCardViewHolder holder, int position) {
        SavedCard savedCard = savedCards.get(position);
        PaymentMethod paymentMethod = new PaymentMethod();
        paymentMethod.type = "stripe";
        paymentMethod.card_brand = savedCard.card_brand;
        holder.ivMethod.setImageResource(MyApp.getInstance().paymentMethodLogo(paymentMethod));
        holder.tvTitle.setText("**** " + savedCard.card_number);
        PaymentMethod defaultPaymentMethod = MyApp.getInstance().getMyPreferences().getDefaultPaymentMethod();
        if (defaultPaymentMethod != null) {
            if (defaultPaymentMethod.type.equalsIgnoreCase("stripe") && defaultPaymentMethod.stripe_token_id.equalsIgnoreCase(savedCard.stripe_token_id)) {
                holder.ivSelect.setImageResource(R.drawable.ic_baseline_done_24);
                holder.tvDefault.setVisibility(View.VISIBLE);
            } else {
                holder.tvDefault.setVisibility(View.GONE);
                holder.ivSelect.setImageResource(R.drawable.dot_grey_checkbox);
            }
        } else {
            holder.tvDefault.setVisibility(View.GONE);
            holder.ivSelect.setImageResource(R.drawable.dot_grey_checkbox);
        }

        holder.llDelete.setOnClickListener(view -> {
            if (removeCardListener != null) {
                removeCardListener.onItemClick(position, savedCard);
            }
            holder.swipeRevealLayout.close(true);
        });
    }

    @Override
    public int getItemCount() {
        return savedCards.size();
    }

    protected class SavedCardViewHolder extends RecyclerView.ViewHolder {
        private TextView tvTitle, tvDefault;
        private ImageView ivSelect, ivMethod;
        private SwipeRevealLayout swipeRevealLayout;
        private LinearLayout llDelete, llCard;

        public SavedCardViewHolder(@NonNull View itemView) {
            super(itemView);
            tvTitle = itemView.findViewById(R.id.tvTitle);
            ivSelect = itemView.findViewById(R.id.ivSelect);
            ivMethod = itemView.findViewById(R.id.ivMethod);
            llDelete = itemView.findViewById(R.id.llDelete);
            llCard = itemView.findViewById(R.id.llCard);
            tvDefault = itemView.findViewById(R.id.tvDefault);
            swipeRevealLayout = itemView.findViewById(R.id.swipeRevealLayout);
            llCard.setOnClickListener(view -> {
                SavedCard card = savedCards.get(getLayoutPosition());
                PaymentMethod paymentMethod = new PaymentMethod();
                paymentMethod.type = "stripe";
                paymentMethod.title = "**** " + card.card_number;
                paymentMethod.card_number = card.card_number;
                paymentMethod.card_id = card.id;
                paymentMethod.stripe_customer_id = card.stripe_customer_id;
                paymentMethod.service_type = card.service_type;
                paymentMethod.stripe_token_id = card.stripe_token_id;
                paymentMethod.card_brand = card.card_brand;
                paymentMethod.exp_month = card.exp_month;
                paymentMethod.exp_year = card.exp_year;
                paymentMethod.method_logo = R.drawable.icon_visa;
                MyApp.getInstance().getMyPreferences().saveDefaultPaymentMethod(paymentMethod);
                notifyDataSetChanged();
                if (recyclerViewItemClickListener != null) {
                    recyclerViewItemClickListener.onItemClick(getLayoutPosition(), savedCards);
                }
            });
        }
    }
}
