package com.tiffintom.adapters;

import android.app.Activity;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.ImageView;
import android.widget.RadioButton;
import android.widget.TextView;

import androidx.annotation.NonNull;
import androidx.recyclerview.widget.RecyclerView;

import com.tiffintom.R;
import com.tiffintom.interfaces.RecyclerViewItemClickListener;
import com.tiffintom.models.SavedCard;

import java.util.ArrayList;

/**
 * Created by Amrish on 19-08-2020.
 */
public class SavedCardsUIAdapter extends RecyclerView.Adapter<SavedCardsUIAdapter.SavedCardViewHolder> {
    private Activity activity;
    private ArrayList<SavedCard> savedCards = new ArrayList<>();
    private int selectedSavedCard = 0;
    private RecyclerViewItemClickListener recyclerViewItemClickListener;
    private int lastCheckedPosition = -1, checkedPosition = -1;
    private RadioButton checkedRadio;
    private RecyclerView recyclerView;


    public SavedCardsUIAdapter(Activity activity, ArrayList<SavedCard> savedCards, RecyclerView recyclerView, int selectedSavedCard, RecyclerViewItemClickListener recyclerViewItemClickListener) {
        this.activity = activity;
        this.savedCards = savedCards;
        this.selectedSavedCard = selectedSavedCard;
        this.recyclerView = recyclerView;
        this.recyclerViewItemClickListener = recyclerViewItemClickListener;
    }

    @NonNull
    @Override
    public SavedCardsUIAdapter.SavedCardViewHolder onCreateViewHolder(@NonNull ViewGroup parent, int viewType) {
        View view = LayoutInflater.from(activity).inflate(R.layout.item_saved_card_ui, parent, false);
        int width = recyclerView.getWidth();
        ViewGroup.LayoutParams params = view.getLayoutParams();
        params.width = (int) (width * 0.8);
        view.setLayoutParams(params);
        return new SavedCardViewHolder(view);
    }

    @Override
    public void onBindViewHolder(@NonNull SavedCardsUIAdapter.SavedCardViewHolder holder, int position) {
        SavedCard savedCard = savedCards.get(position);

        holder.tvCardNumber.setText("**** " + savedCard.card_number);
        holder.tvCardHolder.setText(savedCard.customer_name);
        holder.tvCardExpiry.setText(String.format("%02d", savedCard.exp_month) + "/" + savedCard.exp_year);

        holder.tvRemove.setOnClickListener(view -> {
            if (recyclerViewItemClickListener != null)
                recyclerViewItemClickListener.onItemClick(position, savedCard);
        });

    }

    @Override
    public int getItemCount() {
        return savedCards.size();
    }

    protected class SavedCardViewHolder extends RecyclerView.ViewHolder {
        private TextView tvCardNumber, tvCardExpiry, tvCardHolder, tvRemove;
        private ImageView ivSelect;

        public SavedCardViewHolder(@NonNull View itemView) {
            super(itemView);
            tvCardNumber = itemView.findViewById(R.id.tvCardNumber);
            ivSelect = itemView.findViewById(R.id.ivSelect);
            tvCardExpiry = itemView.findViewById(R.id.tvCardExpiry);
            tvCardHolder = itemView.findViewById(R.id.tvCardHolder);
            tvRemove = itemView.findViewById(R.id.tvRemove);
        }
    }
}
