package com.tiffintom.adapters;

import android.app.Activity;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.ImageView;
import android.widget.RelativeLayout;
import android.widget.TextView;

import androidx.annotation.NonNull;
import androidx.recyclerview.widget.RecyclerView;

import com.tiffintom.R;
import com.tiffintom.interfaces.RecyclerViewItemClickListener;

import java.util.ArrayList;

public class SelectCountryAdapter extends RecyclerView.Adapter<SelectCountryAdapter.ViewHolder> {
    private Activity activity;
    private ArrayList<String> countries;
    private RecyclerViewItemClickListener itemClickListener;
    private String selectedCountry;

    public SelectCountryAdapter(Activity activity, ArrayList<String> countries, String selectedCountry, RecyclerViewItemClickListener itemClickListener) {
        this.activity = activity;
        this.countries = countries;
        this.selectedCountry = selectedCountry;
        this.itemClickListener = itemClickListener;
    }

    @NonNull
    @Override
    public SelectCountryAdapter.ViewHolder onCreateViewHolder(@NonNull ViewGroup parent, int viewType) {
        return new ViewHolder(LayoutInflater.from(activity).inflate(R.layout.country_item, parent, false));
    }

    @Override
    public void onBindViewHolder(@NonNull SelectCountryAdapter.ViewHolder holder, int position) {

        holder.tvCountryName.setText(countries.get(position));


        if (selectedCountry != null && countries.get(position).equalsIgnoreCase(selectedCountry)) {
            holder.ivSelected.setVisibility(View.VISIBLE);
        } else {
            holder.ivSelected.setVisibility(View.INVISIBLE);
        }

        holder.rlCountries.setOnClickListener(view -> {
            if (itemClickListener != null)
                itemClickListener.onItemClick(position, countries.get(position));
        });

    }

    @Override
    public int getItemCount() {
        return countries.size();
    }

    public class ViewHolder extends RecyclerView.ViewHolder {
        private TextView tvCountryName;
        private ImageView ivSelected;
        private RelativeLayout rlCountries;

        public ViewHolder(@NonNull View itemView) {
            super(itemView);
            tvCountryName = itemView.findViewById(R.id.tvCountryName);
            ivSelected = itemView.findViewById(R.id.ivSelectedCountry);
            rlCountries = itemView.findViewById(R.id.rlCountries);
        }
    }
}
