package com.tiffintom.adapters;

import android.app.Activity;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.ImageView;
import android.widget.RelativeLayout;
import android.widget.TextView;

import androidx.annotation.NonNull;
import androidx.recyclerview.widget.RecyclerView;

import com.tiffintom.R;
import com.tiffintom.interfaces.RecyclerViewItemClickListener;
import com.tiffintom.models.TimeSlot;

import java.util.ArrayList;


public class ShowTimeAdapter extends RecyclerView.Adapter<ShowTimeAdapter.ViewHolder> {
    private Activity activity;
    private ArrayList<TimeSlot> deliveryList = new ArrayList<>();
    private RecyclerViewItemClickListener recyclerViewItemClickListener;
    private TimeSlot selectedTimeSlot;

    public ShowTimeAdapter(Activity activity, TimeSlot selectedTimeSlot, ArrayList<TimeSlot> deliveryList, RecyclerViewItemClickListener recyclerViewItemClickListener) {
        this.activity = activity;
        this.deliveryList = deliveryList;
        this.recyclerViewItemClickListener = recyclerViewItemClickListener;
        this.selectedTimeSlot = selectedTimeSlot;
    }

    @NonNull
    @Override
    public ShowTimeAdapter.ViewHolder onCreateViewHolder(@NonNull ViewGroup parent, int viewType) {
        return new ViewHolder(LayoutInflater.from(activity).inflate(R.layout.show_time_item, parent, false));
    }

    @Override
    public void onBindViewHolder(@NonNull ShowTimeAdapter.ViewHolder holder, int position) {

        if (selectedTimeSlot != null && deliveryList.get(position).name.equalsIgnoreCase(selectedTimeSlot.name)) {
            holder.rlTimeSlots.performClick();
            holder.ivSelected.setVisibility(View.VISIBLE);
        } else {
            holder.ivSelected.setVisibility(View.INVISIBLE);
        }
        holder.textView.setText(deliveryList.get(position).name);
        holder.rlTimeSlots.setOnClickListener(view -> {
            if (recyclerViewItemClickListener != null)
                recyclerViewItemClickListener.onItemClick(position, deliveryList.get(position));
        });
    }

    @Override
    public int getItemCount() {
        return deliveryList.size();
    }

    public class ViewHolder extends RecyclerView.ViewHolder {
        private TextView textView;
        private RelativeLayout rlTimeSlots;
        private ImageView ivSelected;

        public ViewHolder(@NonNull View itemView) {
            super(itemView);
            textView = itemView.findViewById(R.id.tvTime);
            rlTimeSlots = itemView.findViewById(R.id.rlTime);
            ivSelected = itemView.findViewById(R.id.ivSelectedTime);
        }
    }
}
