package com.tiffintom.adapters;

import android.content.Intent;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.ImageView;
import android.widget.TextView;

import androidx.annotation.NonNull;
import androidx.recyclerview.widget.RecyclerView;

import com.bumptech.glide.Glide;
import com.google.android.material.floatingactionbutton.FloatingActionButton;
import com.tiffintom.MyApp;
import com.tiffintom.R;
import com.tiffintom.activity.RestaurantDetailsActivity;
import com.tiffintom.common.Validators;
import com.tiffintom.interfaces.RecyclerViewItemClickListener;
import com.tiffintom.models.OrderHistory;
import com.tiffintom.models.RestaurantMiniSnippet;
import com.tiffintom.models.User;

import java.util.ArrayList;

/**
 * Created by Amrish on 30-09-2020.
 */
public class SnippetAdapter extends RecyclerView.Adapter<RecyclerView.ViewHolder> {
    private int RESTAURANT_CART_ITEM = 1, LAST_ORDER = 2;
    private ArrayList<Object> objects;
    private RecyclerViewItemClickListener recyclerViewItemClickListener;
    private User loggedInUser;

    public SnippetAdapter(ArrayList<Object> objects, RecyclerViewItemClickListener recyclerViewItemClickListener) {
        this.objects = objects;
        this.recyclerViewItemClickListener = recyclerViewItemClickListener;
        loggedInUser = MyApp.getInstance().getMyPreferences().getLoggedInUser();
    }

    public SnippetAdapter(ArrayList<Object> objects) {
        this.objects = objects;
    }

    @Override
    public int getItemViewType(int position) {
        if (objects.get(position) instanceof OrderHistory) {
            return LAST_ORDER;
        } else {
            return RESTAURANT_CART_ITEM;
        }
    }

    @NonNull
    @Override
    public RecyclerView.ViewHolder onCreateViewHolder(@NonNull ViewGroup parent, int viewType) {
        View view = LayoutInflater.from(parent.getContext()).inflate(R.layout.layout_bottom_miniview, parent, false);
        if (viewType == LAST_ORDER) {
            return new LastOrderViewHolder(LayoutInflater.from(parent.getContext()).inflate(R.layout.layout_bottom_last_order, parent, false));
        } else {
            return new CartRestaurantViewHolder(LayoutInflater.from(parent.getContext()).inflate(R.layout.layout_bottom_miniview, parent, false));
        }
    }

    @Override
    public void onBindViewHolder(@NonNull RecyclerView.ViewHolder holder, int position) {
        if (getItemViewType(position) == RESTAURANT_CART_ITEM) {
            CartRestaurantViewHolder viewHolder = (CartRestaurantViewHolder) holder;
            RestaurantMiniSnippet miniSnippet = (RestaurantMiniSnippet) objects.get(position);
            viewHolder.tvItems.setText(miniSnippet.items + " Items");
            viewHolder.tvRestaurant.setText(miniSnippet.restaurantName);
            if (!Validators.isNullOrEmpty(miniSnippet.imageUrl)) {
                viewHolder.ivRestaurant.setVisibility(View.VISIBLE);

                Glide.with(viewHolder.itemView.getContext())
                        .load(miniSnippet.imageUrl)
                        .placeholder(R.drawable.restaurant_logo_placeholder)
                        .error(R.drawable.restaurant_logo_placeholder)
                        .into(viewHolder.ivRestaurant);
            } else {
                viewHolder.ivRestaurant.setVisibility(View.GONE);
            }
            viewHolder.tvView.setOnClickListener(view -> {
                viewHolder.itemView.getContext().startActivity(new Intent(viewHolder.itemView.getContext(), RestaurantDetailsActivity.class).putExtra("resId", miniSnippet.res_id).putExtra("res_name", miniSnippet.restaurantName));
            });
            viewHolder.fabClose.setOnClickListener(view -> {
                if (recyclerViewItemClickListener != null) {
                    recyclerViewItemClickListener.onItemClick(position, miniSnippet);
                }

            });

        }
        if (getItemViewType(position) == LAST_ORDER) {
            LastOrderViewHolder viewHolder = (LastOrderViewHolder) holder;
            OrderHistory orderDetail = (OrderHistory) objects.get(position);
            if (!Validators.isNullOrEmpty(orderDetail.order_type)) {
                if (orderDetail.order_type.equalsIgnoreCase("delivery"))
                    viewHolder.tvOrderStatus.setText("Delivery Order " + orderDetail.status);
                else if (orderDetail.order_type.equalsIgnoreCase("pickup"))
                    viewHolder.tvOrderStatus.setText("Pickup Order " + orderDetail.status);
                else viewHolder.tvOrderStatus.setText("Pickup Order " + orderDetail.status);
            } else {
                viewHolder.tvOrderStatus.setText("Dine in Order " + orderDetail.status);
            }
            viewHolder.tvRestaurant.setText(orderDetail.restaurant.restaurant_name);
            if (!Validators.isNullOrEmpty(orderDetail.preparation)) {
                viewHolder.tvTime.setText("Est. arrival - " + orderDetail.preparation);
                viewHolder.tvTime.setVisibility(View.VISIBLE);
            } else {
                viewHolder.tvTime.setVisibility(View.GONE);
            }
            if (orderDetail.status.equalsIgnoreCase("delivered") || orderDetail.status.equalsIgnoreCase("failed")) {
                viewHolder.tvTime.setVisibility(View.GONE);
            } else {
                viewHolder.tvTime.setVisibility(View.VISIBLE);
            }
            viewHolder.tvView.setText("View");
            Glide.with(viewHolder.itemView.getContext())
                    .load(orderDetail.logoUrl)
                    .placeholder(R.drawable.restaurant_logo_placeholder)
                    .error(R.drawable.restaurant_logo_placeholder)
                    .into(viewHolder.ivRestaurant);
            viewHolder.tvView.setOnClickListener(view -> {
                if (recyclerViewItemClickListener != null) {
                    recyclerViewItemClickListener.onItemClick(position, orderDetail);
                }
            });

        }
    }

    @Override
    public int getItemCount() {
        return objects.size();
    }

    private static class LastOrderViewHolder extends RecyclerView.ViewHolder {
        private TextView tvOrderStatus;
        private ImageView ivRestaurant;
        private TextView tvRestaurant, tvTime, tvView;

        public LastOrderViewHolder(@NonNull View itemView) {
            super(itemView);
            tvOrderStatus = itemView.findViewById(R.id.tvOrderStatus);
            tvRestaurant = itemView.findViewById(R.id.tvRestaurant);
            tvView = itemView.findViewById(R.id.tvView);
            tvTime = itemView.findViewById(R.id.tvTime);
            ivRestaurant = itemView.findViewById(R.id.ivRestaurantLogo);
        }
    }

    protected static class CartRestaurantViewHolder extends RecyclerView.ViewHolder {

        private ImageView ivRestaurant;
        private TextView tvRestaurant, tvItems, tvView;
        private FloatingActionButton fabClose;

        public CartRestaurantViewHolder(@NonNull View itemView) {
            super(itemView);
            ivRestaurant = itemView.findViewById(R.id.ivRestaurantLogo);
            tvRestaurant = itemView.findViewById(R.id.tvRestaurant);
            tvItems = itemView.findViewById(R.id.tvItems);
            fabClose = itemView.findViewById(R.id.fabClose);
            tvView = itemView.findViewById(R.id.tvView);

        }
    }

}
