package com.tiffintom.adapters;

import android.app.Activity;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.TextView;

import androidx.annotation.NonNull;
import androidx.recyclerview.widget.RecyclerView;

import com.tiffintom.MyApp;
import com.tiffintom.R;
import com.tiffintom.common.CommonFunctions;
import com.tiffintom.common.Validators;
import com.tiffintom.interfaces.RecyclerViewItemClickListener;
import com.tiffintom.models.WalletHistory;

import java.util.ArrayList;

/**
 * Created by Amrish on 19-08-2020.
 */
public class TransactionHistoryAdapter extends RecyclerView.Adapter<TransactionHistoryAdapter.BannerViewHolder> {
    private Activity activity;
    private ArrayList<WalletHistory> transactions = new ArrayList<>();
    private RecyclerViewItemClickListener bannerClickListener;

    public TransactionHistoryAdapter(Activity activity, ArrayList<WalletHistory> transactions, RecyclerViewItemClickListener bannerClickListener) {
        this.activity = activity;
        this.transactions = transactions;
        this.bannerClickListener = bannerClickListener;
    }

    @NonNull
    @Override
    public TransactionHistoryAdapter.BannerViewHolder onCreateViewHolder(@NonNull ViewGroup parent, int viewType) {
        return new BannerViewHolder(LayoutInflater.from(activity).inflate(R.layout.wallet_transaction_history_item, parent, false));
    }

    @Override
    public void onBindViewHolder(@NonNull TransactionHistoryAdapter.BannerViewHolder holder, int position) {
        WalletHistory walletHistory = transactions.get(position);
        holder.tvDate.setText(CommonFunctions.formatDate(walletHistory.created.replace(":", ""), "dd MMM yyyy"));
        if (Validators.isNullOrEmpty(walletHistory.store_name)) {
            holder.tvTitle.setText("Credit added");
        } else {
            holder.tvTitle.setText(walletHistory.store_name);
        }
        if (!Validators.isNullOrEmpty(walletHistory.transaction_details)) {
            holder.tvSubtitle.setText(walletHistory.transaction_details.replace("ORD", "Order #"));
            holder.tvSubtitle.setVisibility(View.VISIBLE);
        } else {
            holder.tvSubtitle.setVisibility(View.GONE);
        }
        holder.tvAmount.setText((walletHistory.transaction_type.equalsIgnoreCase("debited") ? "-" : "") + MyApp.getInstance().getCurrencySymbol() + MyApp.df.format(walletHistory.amount));
        if (Validators.isNullOrEmpty(walletHistory.balance)) {
            holder.tvBalanceLabel.setVisibility(View.GONE);
            holder.tvBalance.setVisibility(View.GONE);
        } else {
            holder.tvBalanceLabel.setVisibility(View.VISIBLE);
            holder.tvBalance.setVisibility(View.VISIBLE);
            holder.tvBalance.setText(MyApp.getInstance().getCurrencySymbol() + walletHistory.balance);
        }
    }

    @Override
    public int getItemCount() {
        return transactions.size();
    }

    protected class BannerViewHolder extends RecyclerView.ViewHolder {
        private TextView tvTitle, tvSubtitle, tvDate, tvAmount, tvBalance, tvBalanceLabel;

        public BannerViewHolder(@NonNull View itemView) {
            super(itemView);
            tvTitle = itemView.findViewById(R.id.tvTitle);
            tvAmount = itemView.findViewById(R.id.tvAmount);
            tvBalance = itemView.findViewById(R.id.tvBalance);
            tvBalanceLabel = itemView.findViewById(R.id.tvAfterTransactionLabel);
            tvSubtitle = itemView.findViewById(R.id.tvSubtitle);
            tvDate = itemView.findViewById(R.id.tvDate);

        }
    }
}
