package com.tiffintom.base;

import android.content.BroadcastReceiver;
import android.content.Context;
import android.content.Intent;
import android.view.View;
import android.widget.TextView;
import android.widget.Toast;

import androidx.appcompat.app.AlertDialog;
import androidx.appcompat.app.AppCompatActivity;
import androidx.cardview.widget.CardView;

import com.google.gson.Gson;
import com.tiffintom.MyApp;
import com.tiffintom.activity.TransportActivity;
import com.tiffintom.common.CommonFunctions;
import com.tiffintom.common.Validators;
import com.tiffintom.models.RequestResponse;
import com.tiffintom.utils.ToastUtils;

import org.json.JSONObject;

/**
 * Created by admin on 14-03-2017.
 */

public class BaseActivity extends AppCompatActivity {

    protected MyApp myApp = MyApp.getInstance();
    protected AlertDialog progressDialog;
    protected TextView tvUserName;
    protected CardView llToastLayout;
    protected TextView tvToastMessage;
    protected BroadcastReceiver toastReceiver;

    protected void initViews() {
        progressDialog = CommonFunctions.customProgressDialog(this, "Loading...");
    }

    @Override
    public boolean onSupportNavigateUp() {
        onBackPressed();
        return super.onSupportNavigateUp();
    }

    protected String checkErrorMessage(JSONObject jsonObject) {
        RequestResponse requestResponse = new Gson().fromJson(jsonObject.toString(), RequestResponse.class);
        if (requestResponse.result != null) {
            if (requestResponse.result.success != 1) {
                if (requestResponse.result.message != null)
                    return requestResponse.result.message;
                else return "No results found";
            } else {
                return "No data found";
            }
        } else {
            return "No results found";
        }
    }

    protected void updateUserName() {
        if (tvUserName != null) {
            if (myApp.getMyPreferences().getLoggedInUser() != null) {
                tvUserName.setText("Hi, " + myApp.getMyPreferences().getLoggedInUser().first_name);
                tvUserName.setOnClickListener(view -> {
                    startActivity(new Intent(this, TransportActivity.class).putExtra("destination", "profile_edit").putExtra("name", "Profile"));
                });
                myApp.setProfileChangeListener(this::updateUserName);
            } else {
                tvUserName.setText("Hi, User");
                tvUserName.setVisibility(View.GONE);
            }
        }
    }


    protected void toastListener() {
        toastReceiver = new BroadcastReceiver() {
            @Override
            public void onReceive(Context context, Intent intent) {
                runOnUiThread(() -> {
                    if (llToastLayout != null && tvToastMessage != null) {
                        if (intent != null && intent.getExtras() != null) {
                            String message = intent.getStringExtra("message");
                            if (!Validators.isNullOrEmpty(message)) {
                                llToastLayout.setVisibility(View.VISIBLE);
                                tvToastMessage.setText(message);
                            } else {
                                llToastLayout.setVisibility(View.GONE);
                            }
                        } else {
                            llToastLayout.setVisibility(View.GONE);
                        }
                        llToastLayout.postDelayed(() -> {
                            llToastLayout.setVisibility(View.GONE);
                        }, 2500);
                    }
                });
            }
        };
    }
}
