package com.tiffintom.base;

import android.content.Intent;
import android.os.Bundle;
import android.view.View;
import android.widget.TextView;

import androidx.annotation.NonNull;
import androidx.annotation.Nullable;
import androidx.appcompat.app.AlertDialog;
import androidx.fragment.app.Fragment;

import com.google.gson.Gson;
import com.tiffintom.MyApp;
import com.tiffintom.activity.TransportActivity;
import com.tiffintom.common.CommonFunctions;
import com.tiffintom.models.RequestResponse;
import com.tiffintom.utils.LogUtils;
import com.tiffintom.utils.ToastUtils;

import org.json.JSONObject;

/**
 * Created by admin on 22-01-2018.
 */

public class BaseFragment extends Fragment {

    protected MyApp myApp = MyApp.getInstance();
    protected AlertDialog progressDialog;
    protected TextView tvUserName;

    @Override
    public void onViewCreated(@NonNull View view, @Nullable Bundle savedInstanceState) {
        super.onViewCreated(view, savedInstanceState);
        initViews(view);
        updateUserName();
    }

    protected void initViews(View view) {
        progressDialog = CommonFunctions.customProgressDialog(getActivity(), "Loading...");
    }

    protected void updateUserName() {
        if (tvUserName != null) {
            if (myApp.getMyPreferences().getLoggedInUser() != null) {
                tvUserName.setText("Hi, " + myApp.getMyPreferences().getLoggedInUser().first_name);
                tvUserName.setOnClickListener(view -> {
                    startActivity(new Intent(getActivity(), TransportActivity.class).putExtra("destination", "profile_edit").putExtra("name", "Profile"));
                });
                myApp.setProfileChangeListener(this::updateUserName);
            } else {
                tvUserName.setText("Hi, User");
                tvUserName.setVisibility(View.GONE);
            }
        }
    }

    protected String checkErrorMessage(JSONObject jsonObject) {
        RequestResponse requestResponse = new Gson().fromJson(jsonObject.toString(), RequestResponse.class);
        if (requestResponse.result != null) {
            if (requestResponse.result.success != 1) {
                if (requestResponse.result.message != null)
                    return requestResponse.result.message;
                else return "No results found";
            } else {
                return null;
            }
        } else {
            return "No results found";
        }
    }
}
