package com.tiffintom.common;

import android.app.Activity;
import android.content.Context;
import android.content.Intent;
import android.location.Address;
import android.location.Geocoder;
import android.net.ConnectivityManager;
import android.net.NetworkInfo;
import android.util.Log;
import android.view.LayoutInflater;
import android.view.View;
import android.view.inputmethod.InputMethodManager;
import android.widget.TextView;

import androidx.appcompat.app.AlertDialog;
import androidx.localbroadcastmanager.content.LocalBroadcastManager;

import com.androidnetworking.AndroidNetworking;
import com.androidnetworking.error.ANError;
import com.androidnetworking.interfaces.JSONArrayRequestListener;
import com.androidnetworking.interfaces.JSONObjectRequestListener;
import com.google.gson.Gson;
import com.tiffintom.MyApp;
import com.tiffintom.R;
import com.tiffintom.models.User;
import com.tiffintom.network.ApiEndPoints;
import com.tiffintom.network.ApiUtils;
import com.tiffintom.utils.Constants;
import com.tiffintom.utils.LogUtils;

import org.json.JSONArray;
import org.json.JSONObject;

import java.io.IOException;
import java.security.MessageDigest;
import java.security.NoSuchAlgorithmException;
import java.sql.ResultSet;
import java.text.DateFormat;
import java.text.DecimalFormat;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.Calendar;
import java.util.Date;
import java.util.List;
import java.util.Locale;

public class CommonFunctions {
    public static DecimalFormat df = new DecimalFormat("#.##");

    public static AlertDialog customProgressDialog(Context context, String title) {
        AlertDialog alertDialog;
        AlertDialog.Builder builder = new AlertDialog.Builder(context);
        LayoutInflater inflater = (LayoutInflater) context.getSystemService(Context.LAYOUT_INFLATER_SERVICE);
        View dialogView = inflater.inflate(R.layout.progress_dialog_layout, null);

        TextView tvMessage = dialogView.findViewById(R.id.tvMessage);
        if (!Validators.isNullOrEmpty(title)) {
            tvMessage.setText(title);
        }

        builder.setView(dialogView);
        builder.setCancelable(false);
        alertDialog = builder.create();
        alertDialog.getWindow().getDecorView().setBackgroundResource(android.R.color.transparent);
        alertDialog.getWindow().setDimAmount(0.0f);
        return alertDialog;
    }

    public static String StringAppender(String... s) {
        StringBuilder builder = new StringBuilder();
        for (String string : s) {
            builder.append(string);
        }
        return builder.toString();
    }

    public static JSONArray convertToJSONArray(ResultSet resultSet)
            throws Exception {
        JSONArray jsonArray = new JSONArray();
        while (resultSet.next()) {
            int total_columns = resultSet.getMetaData().getColumnCount();
            JSONObject obj = new JSONObject();
            for (int i = 0; i < total_columns; i++) {
                obj.put(resultSet.getMetaData().getColumnLabel(i + 1).toLowerCase(), resultSet.getObject(i + 1));
            }
            jsonArray.put(obj);
        }
        return jsonArray;
    }

    public static String md5(String s) {
        try {
            MessageDigest digest = MessageDigest.getInstance("MD5");
            digest.update(s.getBytes());
            byte[] messageDigest = digest.digest();
            StringBuffer hexString = new StringBuffer();
            for (byte b : messageDigest) {
                hexString.append(Integer.toHexString(b & 255));
            }
            return hexString.toString();
        } catch (NoSuchAlgorithmException e) {
            return "";
        }
    }

    public static void hideKeyboard(Activity activity, View view) {
        InputMethodManager imm = (InputMethodManager) activity.getSystemService(Activity.INPUT_METHOD_SERVICE);
        imm.hideSoftInputFromWindow(view.getWindowToken(), 0);
    }

    public static Date convertStringDateToDate(String date, String inputFormat) {
        DateFormat format = new SimpleDateFormat(inputFormat, Locale.getDefault());
        Date formattedDate = new Date();
        try {
            formattedDate = format.parse(date);
        } catch (ParseException e) {
            e.printStackTrace();
        }
        return formattedDate;
    }

    public static String getFormattedDate(final String date) {

        DateFormat inputFormat = new SimpleDateFormat("yyyy-MM-dd", Locale.getDefault());
        DateFormat outputFormat = new SimpleDateFormat("dd MMM yyyy", Locale.getDefault());

        String formattedDate = null;

        try {
            formattedDate = outputFormat.format(inputFormat.parse(date));
        } catch (ParseException e) {
            e.printStackTrace();
        }

        return formattedDate;
    }

    public static String getDesiredFormattedDate(final String date, final String desireFormat) {

        DateFormat inputFormat = new SimpleDateFormat("yyyy-MM-dd hh:mm:ss", Locale.getDefault());
        DateFormat outputFormat = new SimpleDateFormat(desireFormat, Locale.getDefault());

        String formattedDate = null;

        try {
            formattedDate = outputFormat.format(inputFormat.parse(date));
        } catch (ParseException e) {
            e.printStackTrace();
        }

        return formattedDate;
    }

    public static String formatDate(final String date, final String desireFormat) {

        DateFormat inputFormat = new SimpleDateFormat("yyyy-MM-dd", Locale.getDefault());
        DateFormat outputFormat = new SimpleDateFormat(desireFormat, Locale.getDefault());

        String formattedDate = null;

        try {
            formattedDate = outputFormat.format(inputFormat.parse(date));
        } catch (ParseException e) {
            e.printStackTrace();
        }

        return formattedDate;
    }

    public static String formatDate(final String date, final String currentFormat, final String desireFormat) {

        DateFormat inputFormat = new SimpleDateFormat(currentFormat, Locale.getDefault());
        DateFormat outputFormat = new SimpleDateFormat(desireFormat, Locale.getDefault());

        String formattedDate = null;

        try {
            formattedDate = outputFormat.format(inputFormat.parse(date));
        } catch (ParseException e) {
            e.printStackTrace();
        }

        return formattedDate;
    }

    public static String formatTimestamp(final String timestamp, final String desireFormat) {

        DateFormat inputFormat = new SimpleDateFormat("yyyy-MM-dd'T'hhmmssZ", Locale.getDefault());
        DateFormat outputFormat = new SimpleDateFormat(desireFormat, Locale.getDefault());

        String formattedDate = null;

        try {
            formattedDate = outputFormat.format(inputFormat.parse(timestamp));
        } catch (ParseException e) {
            e.printStackTrace();
        }

        return formattedDate;
    }

    public static long formatTimestampAndGetMs(final String timestamp) {

        DateFormat inputFormat = new SimpleDateFormat("yyyy-MM-dd'T'hhmmssZ", Locale.getDefault());

        Date formattedDate = null;
        try {
            formattedDate = inputFormat.parse(timestamp);
        } catch (ParseException e) {
            e.printStackTrace();
        }
        if (formattedDate == null) return 0;
        else return formattedDate.getTime();
    }

    public static String formatMili(final long timestamp, final String desireFormat) {

        DateFormat outputFormat = new SimpleDateFormat(desireFormat, Locale.getDefault());

        String formattedDate = null;

        formattedDate = outputFormat.format(timestamp);

        return formattedDate;
    }

    public static int getProfilePercentage(User user) {
        int progress = 0;

        if (!Validators.isNullOrEmpty(user.first_name)) {
            progress = progress + 20;
        }
        if (!Validators.isNullOrEmpty(user.last_name)) {
            progress = progress + 20;
        }
        if (!Validators.isNullOrEmpty(user.phone_number)) {
            progress = progress + 20;
        }
        if (!Validators.isNullOrEmpty(user.username)) {
            progress = progress + 20;
        }
        if (!Validators.isNullOrEmpty(user.image_url)) {
            progress = progress + 20;
        }
        /*Comment by dipawali parmar 17-11*/
        /*  if (!Validators.isNullOrEmpty(user.customerDetails.first_name)) {
            progress = progress + 20;
        }
        if (!Validators.isNullOrEmpty(user.customerDetails.last_name)) {
            progress = progress + 20;
        }
        if (!Validators.isNullOrEmpty(user.customerDetails.phone_number)) {
            progress = progress + 20;
        }
        if (!Validators.isNullOrEmpty(user.customerDetails.username)) {
            progress = progress + 20;
        }
        if (!Validators.isNullOrEmpty(user.customerDetails.image_url)) {
            progress = progress + 20;
        }*/
        return progress;
    }

    public static Address getAddressByLocation(Context ctx, double latitude, double longitude) {
        StringBuilder result = new StringBuilder();
        try {
            Geocoder geocoder = new Geocoder(ctx, Locale.getDefault());
            List<Address>
                    addresses = geocoder.getFromLocation(latitude, longitude, 1);
            if (addresses != null && addresses.size() > 0) {
                return addresses.get(0);
//                String locality = address.getLocality();
//                String city = address.getCountryName();
//                String region_code = address.getCountryCode();
//                String zipcode = address.getPostalCode();
//                double lat = address.getLatitude();
//                double lon = address.getLongitude();
//                result.append(locality + " ");
//                result.append(city + " " + region_code + " ");
//                result.append(zipcode);
//                result=new StringBuilder(zipcode);
            }
        } catch (IOException e) {
            Log.e("tag", e.getMessage());
        }
        return null;
    }

    public static void updateCartOnline(int customerId, String details) {
        ApiUtils.updateCartOnline(customerId, details)
                .getAsJSONArray(new JSONArrayRequestListener() {
                    @Override
                    public void onResponse(JSONArray response) {
                        LogUtils.e("Update Cart Online: ", new Gson().toJson(response));
                    }

                    @Override
                    public void onError(ANError anError) {
                        anError.printStackTrace();
                        LogUtils.e("Update cart online error: "+anError.getErrorDetail()+details);
                    }
                });
    }

    public static void  deleteAllCart() {
        AndroidNetworking.delete(ApiEndPoints.NEW_DELETEALLCART)
                .build().getAsJSONObject(new JSONObjectRequestListener() {
                    @Override
                    public void onResponse(JSONObject response) {
                        LogUtils.e("Delete Cart Online: ", new Gson().toJson(response));
                    }

                    @Override
                    public void onError(ANError anError) {
                        LogUtils.e("Delete cart online error: "+anError.getErrorDetail());
                    }
                });
    }

    public static String getTodayDay() {
        Calendar calendar = Calendar.getInstance();
        return calendar.getDisplayName(Calendar.DAY_OF_WEEK, Calendar.LONG, Locale.getDefault());
    }

    public static void notifyCartCount(Context context) {
        Intent cartBroadcast = new Intent(Constants.CART_UPDATE);
        cartBroadcast.putExtra("update_cart_count", true);
        LocalBroadcastManager.getInstance(context).sendBroadcast(cartBroadcast);
    }

    public static void notifyOnNetChange(Context context) {
        Intent cartBroadcast = new Intent(Constants.NET_UPDATE);
        cartBroadcast.putExtra("net_update", true);
        LocalBroadcastManager.getInstance(context).sendBroadcast(cartBroadcast);
    }

    public static boolean isConnected(Context context) {
        ConnectivityManager cm = (ConnectivityManager) context.getSystemService(Context.CONNECTIVITY_SERVICE);
        NetworkInfo activeNetwork = cm.getActiveNetworkInfo();
//        return MyApp.getInstance().isConnected(context) &&  activeNetwork != null && activeNetwork.isConnected();
        return activeNetwork != null && activeNetwork.isConnected();
    }

    public static Intent setMessageResult(String message) {
        Intent intent = new Intent();
        intent.putExtra("message", message);
        return intent;
    }
    public static String getToday() {
        SimpleDateFormat formatter = new SimpleDateFormat("yyyy-MM-dd");
        String str = formatter.format(new Date());
        return str;
    }

    public static String formatTimestampZulu(final String timestamp, final String desireFormat) {

        DateFormat inputFormat = new SimpleDateFormat("yyyy-MM-dd'T'hh:mm:ss.SSSSSS'Z'", Locale.getDefault());
        DateFormat outputFormat = new SimpleDateFormat(desireFormat, Locale.getDefault());

        String formattedDate = null;

        try {
            formattedDate = outputFormat.format(inputFormat.parse(timestamp));
        } catch (ParseException e) {
            e.printStackTrace();
        }

        return formattedDate;
    }

    public static void deleteCartOnline(String customer_id) {
        ApiUtils.deleteCartOnline()
                .getAsJSONObject(new JSONObjectRequestListener() {
                    @Override
                    public void onResponse(JSONObject response) {
                        LogUtils.e("Delete Cart Online: ", new Gson().toJson(response));
                    }

                    @Override
                    public void onError(ANError anError) {
                        LogUtils.e("Delete cart online error: " + anError.getErrorBody());
                    }
                });
    }

    public static String getCurrentDate() {
        SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss", Locale.getDefault());
        String str = sdf.format(new Date());
        return str;
    }

}
