package com.tiffintom.fragment;

import android.app.Dialog;
import android.content.DialogInterface;
import android.os.Build;
import android.os.Bundle;
import android.text.Editable;
import android.text.TextWatcher;
import android.view.KeyEvent;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.Button;
import android.widget.EditText;
import android.widget.FrameLayout;
import android.widget.RadioButton;
import android.widget.RadioGroup;
import android.widget.TextView;

import androidx.annotation.NonNull;
import androidx.annotation.Nullable;
import androidx.appcompat.app.AlertDialog;

import com.androidnetworking.AndroidNetworking;
import com.androidnetworking.common.ANRequest;
import com.androidnetworking.error.ANError;
import com.androidnetworking.interfaces.JSONObjectRequestListener;
import com.androidnetworking.interfaces.ParsedRequestListener;
import com.google.android.material.bottomsheet.BottomSheetBehavior;
import com.google.android.material.bottomsheet.BottomSheetDialog;
import com.google.android.material.bottomsheet.BottomSheetDialogFragment;
import com.google.gson.Gson;
import com.tiffintom.MyApp;
import com.tiffintom.R;
import com.tiffintom.common.CommonFunctions;
import com.tiffintom.common.Validators;
import com.tiffintom.interfaces.DialogDismissListener;
import com.tiffintom.models.Address;
import com.tiffintom.models.ApiError;
import com.tiffintom.models.Postcode;
import com.tiffintom.models.User;
import com.tiffintom.network.ApiEndPoints;
import com.tiffintom.network.ApiUtils;
import com.tiffintom.utils.LogUtils;
import com.tiffintom.utils.ToastUtils;

import org.json.JSONException;
import org.json.JSONObject;

import java.util.ArrayList;

/**
 * Created by Amrish on 20-08-2020.
 */
public class AddAddressBottomSheetDialogFragment extends BottomSheetDialogFragment {
    private MyApp myApp = MyApp.getInstance();
    private AlertDialog progressDialog;
    private ArrayList<Address> addresses = new ArrayList<>();
    private DialogDismissListener dialogDismissListener;
    private EditText etPostcode, etHouseNumber;
    private TextView tvAddress, tvTitle;
    private RadioGroup rgAddressTitle;
    private RadioButton rbHome, rbWork, rbOther;
    private Button btnSubmit, btnSearch;
    private Postcode searchedPostcode;
    private User loggedInUser = myApp.getMyPreferences().getLoggedInUser();
    private Address savedAddress;

    public static AddAddressBottomSheetDialogFragment getInstance() {
        return new AddAddressBottomSheetDialogFragment();
    }

    public static AddAddressBottomSheetDialogFragment getInstance(Postcode postcode) {
        AddAddressBottomSheetDialogFragment fragment = new AddAddressBottomSheetDialogFragment();
        Bundle bundle = new Bundle();
        bundle.putString("postcode", new Gson().toJson(postcode));
        fragment.setArguments(bundle);
        return fragment;
    }

    public static AddAddressBottomSheetDialogFragment getInstance(Address savedAddress) {
        AddAddressBottomSheetDialogFragment fragment = new AddAddressBottomSheetDialogFragment();
        Bundle bundle = new Bundle();
        bundle.putString("savedAddress", new Gson().toJson(savedAddress));
        fragment.setArguments(bundle);
        return fragment;
    }

    public void setDialogDismissListener(DialogDismissListener dialogDismissListener) {
        this.dialogDismissListener = dialogDismissListener;
    }

    @Override
    public int getTheme() {
        if (Build.VERSION.SDK_INT > 21) {
            return R.style.BottomSheetDialogTheme;
        } else {
            return super.getTheme();
        }
    }

    @NonNull
    @Override
    public Dialog onCreateDialog(Bundle savedInstanceState) {

        BottomSheetDialog dialog = (BottomSheetDialog) super.onCreateDialog(savedInstanceState);

        dialog.setOnShowListener((DialogInterface.OnShowListener) dialog1 -> {
            BottomSheetDialog d = (BottomSheetDialog) dialog1;
            FrameLayout bottomSheet = (FrameLayout) d.findViewById(com.google.android.material.R.id.design_bottom_sheet);
            if (bottomSheet != null)
                BottomSheetBehavior.from(bottomSheet).setState(BottomSheetBehavior.STATE_EXPANDED);
        });

        return dialog;
    }

    @Nullable
    @Override
    public View onCreateView(@NonNull LayoutInflater inflater, @Nullable ViewGroup container, @Nullable Bundle savedInstanceState) {
        return inflater.inflate(R.layout.fragment_add_address, container, false);
    }

    @Override
    public void onViewCreated(@NonNull View view, @Nullable Bundle savedInstanceState) {
        super.onViewCreated(view, savedInstanceState);
        if (getArguments() != null) {
            String stringValue = getArguments().getString("postcode");
            String savedAddressValue = getArguments().getString("savedAddress");
            if (stringValue != null) {
                searchedPostcode = new Gson().fromJson(stringValue, Postcode.class);
            }
            if (savedAddressValue != null)
                savedAddress = new Gson().fromJson(savedAddressValue, Address.class);
        }
        initViews(view);
        updateUi();
        setListeners();
    }


    private void initViews(View view) {
        progressDialog = CommonFunctions.customProgressDialog(getActivity(), "Loading...");
        tvTitle = view.findViewById(R.id.tvTitle);
        tvAddress = view.findViewById(R.id.etAddress);
        etHouseNumber = view.findViewById(R.id.etFlatNo);
        etPostcode = view.findViewById(R.id.etPostcode);
        rgAddressTitle = view.findViewById(R.id.rgAddressTitle);
        rbHome = view.findViewById(R.id.rbHome);
        rbWork = view.findViewById(R.id.rbWork);
        rbOther = view.findViewById(R.id.rbOther);
        btnSubmit = view.findViewById(R.id.btnConfirm);
        btnSearch = view.findViewById(R.id.btnSearch);

    }

    private void setListeners() {
        btnSubmit.setOnClickListener(view -> {
            if (isValid()) {
                new Thread(this::saveAddress).start();
            }
        });
        btnSearch.setOnClickListener(view -> {
            etPostcode.setError(null);
            if (!Validators.isNullOrEmpty(etPostcode.getText().toString())) {
                String postcode = etPostcode.getText().toString();
                new Thread(() -> searchPostcode(postcode)).start();
            } else {
                etPostcode.setError("Please enter postcode");
            }
        });
        etPostcode.setOnEditorActionListener((textView, i, keyEvent) -> {
            if (keyEvent != null && keyEvent.getKeyCode() == KeyEvent.KEYCODE_ENTER) {
                if (getActivity() != null) {
                    etPostcode.setError(null);
                    if (!Validators.isNullOrEmpty(etPostcode.getText().toString())) {
                        String postcode = etPostcode.getText().toString();
                        new Thread(() -> searchPostcode(postcode)).start();
                    } else {
                        etPostcode.setError("Please enter postcode");
                    }
                }
            }
            return false;
        });
        etPostcode.addTextChangedListener(new TextWatcher() {
            @Override
            public void beforeTextChanged(CharSequence charSequence, int i, int i1, int i2) {

            }

            @Override
            public void onTextChanged(CharSequence charSequence, int i, int i1, int i2) {

            }

            @Override
            public void afterTextChanged(Editable editable) {
                if (searchedPostcode != null && !searchedPostcode.post_code.equalsIgnoreCase(editable.toString())) {
                    searchedPostcode = null;
                }
            }
        });
    }

    private void saveAddress() {
        if (getActivity() != null) getActivity().runOnUiThread(() -> progressDialog.show());
        Address toSaveAddress = new Address();
        toSaveAddress.action = "MyAccount";
        toSaveAddress.page = "AddressBookAdd";
        if (savedAddress != null) {
            toSaveAddress.page = "AddressBook";
            toSaveAddress.addressAction = "AddressBookEdit";
            toSaveAddress.addressBookId = String.valueOf(savedAddress.id);
            toSaveAddress.zipcode = savedAddress.zipcode;
            toSaveAddress.latitude = savedAddress.latitude;
            toSaveAddress.longitude = savedAddress.longitude;
        } else {
            toSaveAddress.zipcode = searchedPostcode.post_code;
            toSaveAddress.latitude = searchedPostcode.latitude;
            toSaveAddress.longitude = searchedPostcode.longitude;
        }
        toSaveAddress.address = tvAddress.getText().toString();
        toSaveAddress.customer_id = String.valueOf(loggedInUser.id);
        toSaveAddress.flat_no = etHouseNumber.getText().toString();

        if (rgAddressTitle.getCheckedRadioButtonId() == R.id.rbHome) {
            toSaveAddress.title = "Home";
        }
        if (rgAddressTitle.getCheckedRadioButtonId() == R.id.rbWork) {
            toSaveAddress.title = "Work";
        }
        if (rgAddressTitle.getCheckedRadioButtonId() == R.id.rbOther) {
            toSaveAddress.title = "Other";
        }
        ANRequest anRequest;
        if (savedAddress == null) {
            anRequest = ApiUtils.saveAddress(toSaveAddress.address, toSaveAddress.customer_id, toSaveAddress.flat_no, toSaveAddress.title, toSaveAddress.zipcode, toSaveAddress.latitude, toSaveAddress.longitude, null);
        } else {
            anRequest = ApiUtils.updateAddress(toSaveAddress.addressBookId, toSaveAddress.address, toSaveAddress.customer_id, toSaveAddress.flat_no, toSaveAddress.title, toSaveAddress.zipcode, toSaveAddress.latitude, toSaveAddress.longitude);
        }
        anRequest
                .getAsJSONObject(new JSONObjectRequestListener() {
                    @Override
                    public void onResponse(JSONObject response) {
                        String errorMessage = myApp.checkErrorMessage(response);
                        if (errorMessage == null) {
                            Address address = null;
                            try {
                                ToastUtils.makeToast(getActivity(), response.getJSONObject("result").getString("message"));
                                address = new Gson().fromJson(response.getJSONObject("result").getJSONObject("Address").toString(), Address.class);
                                myApp.getMyPreferences().saveCurrentAddress(address);
                            } catch (JSONException e) {
                                e.printStackTrace();
//                                if (savedAddress != null) {
//                                    ToastUtils.makeToast(getActivity(), "Address updated successfully");
//                                } else {
//                                    ToastUtils.makeToast(getActivity(), "Address added successfully");
//                                }
                                address = toSaveAddress;
                                myApp.getMyPreferences().saveCurrentAddress(toSaveAddress);
                            }
                            if (getActivity() != null)
                                CommonFunctions.hideKeyboard(getActivity(), etHouseNumber);
                            if (dialogDismissListener != null)
                                dialogDismissListener.onDialogDismiss(address);
                            dismiss();
                        } else {
                            LogUtils.e("ERROR MESSAGE IN SAVE ADDRESS::", new Gson().toJson(response));
                            ToastUtils.makeToast(getActivity(), errorMessage);
                        }
                        if (getActivity() != null)
                            getActivity().runOnUiThread(() -> progressDialog.dismiss());
                    }

                    @Override
                    public void onError(ANError anError) {
                        anError.printStackTrace();
                        LogUtils.e("::SAVE ADDRESS ANERROR::");
                        if (getActivity() != null)
                            getActivity().runOnUiThread(() -> progressDialog.hide());
                    }
                });
    }

    private boolean isValid() {
        etHouseNumber.setError(null);
        etPostcode.setError(null);
        tvAddress.setError(null);
        if (savedAddress == null && searchedPostcode == null) {
            ToastUtils.makeToast(getActivity(), "Please enter valid postcode or search postcode again");
            return false;
        }
        if (Validators.isNullOrEmpty(etHouseNumber.getText().toString())) {
            etHouseNumber.setError("Please enter house number/name");
            etHouseNumber.requestFocus();
            return false;
        }

        return true;
    }

    private void searchPostcode(String postcode) {
        if (getActivity() != null) getActivity().runOnUiThread(progressDialog::show);
        AndroidNetworking.get(ApiEndPoints.RMS_POSTCODE + postcode)
                .build()
                .getAsObject(Postcode.class, new ParsedRequestListener<Postcode>() {
                    @Override
                    public void onResponse(Postcode response) {
                        searchedPostcode = response;
                        if (getActivity() != null)
                            CommonFunctions.hideKeyboard(getActivity(), etPostcode);
                        if (getActivity() != null)
                            getActivity().runOnUiThread(() -> {
                                progressDialog.dismiss();
                                updateUi();
                            });
                        myApp.getMyPreferences().saveCurrentPostcode(response);
                    }

                    @Override
                    public void onError(ANError anError) {
                        if (getActivity() != null)
                            getActivity().runOnUiThread(() -> progressDialog.dismiss());
                        if (anError.getErrorCode() == 400) {
                            ApiError apiError = anError.getErrorAsObject(ApiError.class);
                            ToastUtils.makeSnackToast(getActivity(), apiError.getMessage());
                        }
                        if (!CommonFunctions.isConnected(getActivity())) {
                            myApp.noInternet(getActivity());
                        }
                    }
                });
    }

    private void updateUi() {
        if (searchedPostcode != null) {
            StringBuilder builder = new StringBuilder();
            builder.append(searchedPostcode.street);
            if (!Validators.isNullOrEmpty(searchedPostcode.dependent_locality)) {
                builder.append(", ");
                builder.append(searchedPostcode.dependent_locality);
            }
            if (!Validators.isNullOrEmpty(searchedPostcode.post_town)) {
                builder.append(", ");
                builder.append(searchedPostcode.post_town);
            }
            tvAddress.setText(builder.toString());
            etPostcode.setText(searchedPostcode.post_code);
            etHouseNumber.requestFocus();
        }
        if (savedAddress != null) {
            etPostcode.setText(savedAddress.zipcode);
            tvAddress.setText(savedAddress.address);
            etHouseNumber.setText(savedAddress.flat_no);
            btnSearch.setVisibility(View.GONE);
            etPostcode.setFocusable(false);
            if (savedAddress.title.equalsIgnoreCase("home")) {
                rbHome.setChecked(true);
            } else if (savedAddress.title.equalsIgnoreCase("work")) {
                rbWork.setChecked(true);
            } else {
                rbOther.setChecked(true);
            }
            tvTitle.setText("Edit address");
        }
    }


    @Override
    public void onDismiss(@NonNull DialogInterface dialog) {
        if (progressDialog != null) {
            progressDialog.dismiss();
        }
        super.onDismiss(dialog);
    }
}
