package com.tiffintom.fragment;

import android.app.Activity;
import android.app.Dialog;
import android.content.DialogInterface;
import android.content.Intent;
import android.os.Build;
import android.os.Bundle;
import android.text.Editable;
import android.text.TextWatcher;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.Button;
import android.widget.EditText;
import android.widget.FrameLayout;
import android.widget.ImageView;
import android.widget.RelativeLayout;
import android.widget.TextView;

import androidx.annotation.NonNull;
import androidx.annotation.Nullable;
import androidx.appcompat.app.AlertDialog;

import com.androidnetworking.AndroidNetworking;
import com.androidnetworking.error.ANError;
import com.androidnetworking.interfaces.JSONObjectRequestListener;
import com.google.android.material.bottomsheet.BottomSheetBehavior;
import com.google.android.material.bottomsheet.BottomSheetDialog;
import com.google.android.material.bottomsheet.BottomSheetDialogFragment;
import com.google.gson.Gson;
import com.stripe.android.ApiResultCallback;
import com.stripe.android.Stripe;
import com.stripe.android.model.Card;
import com.stripe.android.model.PaymentMethod;
import com.stripe.android.model.PaymentMethodCreateParams;
import com.stripe.android.view.CardMultilineWidget;
import com.tiffintom.MyApp;
import com.tiffintom.R;
import com.tiffintom.common.CommonFunctions;
import com.tiffintom.common.Validators;
import com.tiffintom.common.ccwatchers.CreditCardEditText;
import com.tiffintom.common.ccwatchers.OtherCardTextWatcher;
import com.tiffintom.interfaces.DialogDismissListener;
import com.tiffintom.models.AddCard;
import com.tiffintom.models.Address;
import com.tiffintom.models.ApiError;
import com.tiffintom.models.Postcode;
import com.tiffintom.models.Restaurant;
import com.tiffintom.models.User;
import com.tiffintom.network.ApiEndPoints;
import com.tiffintom.network.ApiUtils;
import com.tiffintom.utils.LogUtils;
import com.tiffintom.utils.ToastUtils;

import org.json.JSONException;
import org.json.JSONObject;

import java.util.Calendar;

import cards.pay.paycardsrecognizer.sdk.ScanCardIntent;

import static com.tiffintom.utils.Constants.CODE_REFRESH;

/**
 * Created by Amrish on 26-08-2020.
 */
public class AddCardFragment extends BottomSheetDialogFragment {
    private MyApp myApp = MyApp.getInstance();
    private AlertDialog progressDialog;
    private EditText etCardName, etCvv, etPostcode;
    private CreditCardEditText etCardNumber;
    private EditText etExpYear;
    private Button btnSave;
    private DialogDismissListener dialogDismissListener;
    private User loggedInUser;
    private Stripe stripe;
    private CardMultilineWidget card_multiline_widget;
    private TextView tvCardNumber, tvCardExpiry, tvCardHolder, tvCountryName;
    private ImageView ivScan;
    private RelativeLayout rlCountry;
    private Restaurant restaurant;
    private String selectedCountry;
    private boolean onlycards;



    public static AddCardFragment getInstance(boolean onlycards, Restaurant restaurant) {
        AddCardFragment fragment = new AddCardFragment();
        Bundle bundle = new Bundle();
        bundle.putBoolean("onlycards", onlycards);
        bundle.putString("restaurant", new Gson().toJson(restaurant));
        fragment.setArguments(bundle);
        return fragment;
    }

    @Override
    public int getTheme() {
        if (Build.VERSION.SDK_INT > 21) {
            return R.style.BottomSheetDialogTheme;
        } else {
            return super.getTheme();
        }
    }

    @NonNull
    @Override
    public Dialog onCreateDialog(Bundle savedInstanceState) {

        BottomSheetDialog dialog = (BottomSheetDialog) super.onCreateDialog(savedInstanceState);

        dialog.setOnShowListener((DialogInterface.OnShowListener) dialog1 -> {
            BottomSheetDialog d = (BottomSheetDialog) dialog1;
            FrameLayout bottomSheet = (FrameLayout) d.findViewById(com.google.android.material.R.id.design_bottom_sheet);
            if (bottomSheet != null) {
                BottomSheetBehavior.from(bottomSheet).setState(BottomSheetBehavior.STATE_EXPANDED);
                BottomSheetBehavior.from(bottomSheet).setSkipCollapsed(true);
            }
        });

        return dialog;
    }

    @Nullable
    @Override
    public View onCreateView(@NonNull LayoutInflater inflater, @Nullable ViewGroup container, @Nullable Bundle savedInstanceState) {
        return inflater.inflate(R.layout.fragment_add_card, container, false);

    }

    @Override
    public void onViewCreated(@NonNull View view, @Nullable Bundle savedInstanceState) {
        super.onViewCreated(view, savedInstanceState);
        if (getArguments() != null) {
            onlycards = getArguments().getBoolean("onlycards");
            restaurant = new Gson().fromJson(getArguments().getString("restaurant"), Restaurant.class);

        }
        initViews(view);
        setListener();
    }

    public void setDialogDismissListener(DialogDismissListener dialogDismissListener) {
        this.dialogDismissListener = dialogDismissListener;
    }

    private void initViews(View view) {
        if (onlycards) {
            stripe = new Stripe(getActivity(), myApp.getStripePublishKey());
        } else {
            if (restaurant != null && restaurant.business != null) {
                if (restaurant.business.connect_service) {
                    stripe = new Stripe(getActivity(), restaurant.business.connect_stripe_public_key);
                } else {
                    stripe = new Stripe(getActivity(), myApp.getStripePublishKey());
                }
            }else
            {
                stripe = new Stripe(getActivity(), myApp.getStripePublishKey());
            }
        }

//        stripe = new Stripe(getActivity(), myApp.getStripePublishKey());
        card_multiline_widget = view.findViewById(R.id.card_multiline_widget);
        loggedInUser = myApp.getMyPreferences().getLoggedInUser();
        progressDialog = CommonFunctions.customProgressDialog(getActivity(), "Loading...");
        etCardName = view.findViewById(R.id.etCardHolder);
        etCardNumber = view.findViewById(R.id.etCardNumber);
        etExpYear = view.findViewById(R.id.etExpDate);
        etCvv = view.findViewById(R.id.etCvv);
        btnSave = view.findViewById(R.id.btnSave);
        tvCardNumber = view.findViewById(R.id.tvCardNumber);
        tvCardExpiry = view.findViewById(R.id.tvCardExpiry);
        tvCardHolder = view.findViewById(R.id.tvCardHolder);
        ivScan = view.findViewById(R.id.ivScan);
        etPostcode = view.findViewById(R.id.etPostcode);
        rlCountry = view.findViewById(R.id.rlCountry);
        tvCountryName = view.findViewById(R.id.tvCountryName);
    }

    private void setListener() {
        btnSave.setOnClickListener(view -> {
            if (isValid()) createStripeCall();
        });
        etExpYear.addTextChangedListener(new TextWatcher() {
            int previousLength = 0;
            String previousText = "";

            @Override
            public void beforeTextChanged(CharSequence charSequence, int i, int i1, int i2) {
                previousLength = etExpYear.getText().toString().length();
                previousText = charSequence.toString();
            }

            @Override
            public void onTextChanged(CharSequence charSequence, int start, int before, int i2) {
                String current = charSequence.toString();
                if (charSequence.length() == 1) {
                    int n = Integer.parseInt(current);
                    if (n > 1) {
                        etExpYear.setText("0" + current + "/");
                        etExpYear.setSelection(current.length() + 2);
                    } else {
                        if (current.length() == 2 && start == 1) {
                            etExpYear.setText(current + "/");
                            etExpYear.setSelection(current.length() + 1);
                        } else if (current.length() == 2 && before == 1) {
                            current = current.substring(0, 1);
                            etExpYear.setText(current);
                            etExpYear.setSelection(current.length());
                        }
                    }
                } else {
                    if (charSequence.length() == 2) {
                        int n = Integer.parseInt(current);
                        if (n > 12) {
                            etExpYear.setText(previousText);
                            etExpYear.setSelection(etExpYear.getText().toString().length());
                            return;
                        }
                    }
                    if (current.length() == 2 && start == 1) {
                        etExpYear.setText(current + "/");
                        etExpYear.setSelection(current.length() + 1);
                    } else if (current.length() == 2 && before == 1) {
                        current = current.substring(0, 1);
                        etExpYear.setText(current);
                        etExpYear.setSelection(current.length());
                    }
                }
                if (etExpYear.getText().toString().length() == 5) {
                    etCvv.requestFocus();
                }
                tvCardExpiry.setText(etExpYear.getText().toString());
            }

            @Override
            public void afterTextChanged(Editable editable) {

            }
        });
        etCardNumber.addTextChangedListener(new OtherCardTextWatcher(etCardNumber, tvCardNumber));
        etCardName.addTextChangedListener(new TextWatcher() {
            @Override
            public void beforeTextChanged(CharSequence charSequence, int i, int i1, int i2) {

            }

            @Override
            public void onTextChanged(CharSequence charSequence, int i, int i1, int i2) {
                tvCardHolder.setText(etCardName.getText().toString());
            }

            @Override
            public void afterTextChanged(Editable editable) {

            }
        });
        ivScan.setOnClickListener(view -> {
            Intent intent = new ScanCardIntent.Builder(getActivity()).build();
            startActivityForResult(intent, CODE_REFRESH);
        });

        rlCountry.setOnClickListener(view -> {
            SelectCountryBottomSheetFragment countryBottomSheetFragment = SelectCountryBottomSheetFragment.getInstance(selectedCountry);
            countryBottomSheetFragment.show(getChildFragmentManager(), "country");
            countryBottomSheetFragment.setDialogDismissListener(o -> {
                tvCountryName.setText(o.toString());
                selectedCountry = o.toString();
            });
        });
    }

    private boolean isValid() {
        if (Validators.isNullOrEmpty(etCardNumber.getText().toString())) {
            ToastUtils.makeToast(getActivity(), "Please enter card number");
            return false;
        }
        if (Validators.isNullOrEmpty(etCardName.getText().toString())) {
            etCardName.setError("Please enter card holder name");
            etCardName.requestFocus();
            return false;
        }
        if (Validators.isNullOrEmpty(etExpYear.getText().toString())) {
            ToastUtils.makeToast(getActivity(), "Please enter card expiry");
            return false;
        }
        String exp = etExpYear.getText().toString();
        String[] data = exp.split("/");
        if (!isNumberChar(data[0])) {
            ToastUtils.makeToast(getActivity(), "Invalid expiry month");
            return false;
        }
        if (data.length == 1) {
            ToastUtils.makeToast(getActivity(), "Please enter expiry year");
            return false;
        }
        if (data.length > 1 && !isNumberChar(data[1])) {
            ToastUtils.makeToast(getActivity(), "Invalid expiry year");
            return false;
        }
        if (Validators.isNullOrEmpty(etCvv.getText().toString())) {
            ToastUtils.makeToast(getActivity(), "Please enter card cvv");
            return false;
        }
        Card.Builder builder = new Card.Builder(etCardNumber.getText().toString().replace(" ", ""), Integer.parseInt(data[0]), Integer.parseInt(data[1]), etCvv.getText().toString());
        Card cardToSave = builder.build();

        if (cardToSave != null) {
            if (!cardToSave.validateNumber()) {
                ToastUtils.makeToast(getActivity(), "Invalid Card Number");
                return false;
            }

            if (!cardToSave.validateExpiryDate()) {
                ToastUtils.makeToast(getActivity(), "Invalid Card Expiry");
                return false;
            }
            if (!cardToSave.validateCVC()) {
                ToastUtils.makeToast(getActivity(), "Invalid Card CVV");
                return false;
            }
        } else {
            ToastUtils.makeToast(getActivity(), "Please enter card details");
            return false;
        }


        return true;
    }

    private boolean isNumberChar(String string) {
        return string.matches(".*\\d.*");
    }

    String paymentMethodId;

    Card.Builder cardBuilder;
    Card cardToSave;

    private void createStripeCall() {
        if (getActivity() != null) {
            getActivity().runOnUiThread(() -> progressDialog.show());
        }
        String[] data = etExpYear.getText().toString().split("/");
        cardBuilder = new Card.Builder(etCardNumber.getText().toString().replace(" ", ""), Integer.parseInt(data[0]), Integer.parseInt(data[1]), etCvv.getText().toString());
        cardToSave = cardBuilder.build();
        PaymentMethodCreateParams params = cardToSave.toPaymentMethodsParams();
        if (params == null) {
            ToastUtils.makeToast(getActivity(), "Invalid Details");
            return;
        }
        // Configure the SDK with your Stripe publishable key so that it can make requests to the Stripe API
        stripe.createPaymentMethod(params, new ApiResultCallback<PaymentMethod>() {
            @Override
            public void onSuccess(@NonNull PaymentMethod result) {
                if (getActivity() != null) {
                    getActivity().runOnUiThread(() -> progressDialog.dismiss());
                }
                paymentMethodId = result.id;
                LogUtils.e(paymentMethodId);
                saveCard();
                // Send paymentMethodId to your server for the next steps
            }

            @Override
            public void onError(@NonNull Exception e) {
                // Display the error to the user
                e.printStackTrace();
                LogUtils.e(e.getMessage());
                ToastUtils.makeToast(getActivity(), e.getMessage());
                if (getActivity() != null) {
                    getActivity().runOnUiThread(() -> progressDialog.dismiss());
                }
            }
        });
    }

    private void saveCard() {
        if (getActivity() != null) {
            getActivity().runOnUiThread(() -> progressDialog.show());
        }
        AddCard addCard = new AddCard();
        addCard.customer_id = String.valueOf(loggedInUser.id);
        addCard.stripe_token_id = paymentMethodId;
        addCard.card_brand = cardToSave.getBrand().getDisplayName();
        addCard.stripe_customer_id = cardToSave.getCustomerId();
        addCard.address_zip = cardToSave.getAddressZip();
        addCard.exp_month = String.valueOf(cardToSave.getExpMonth());
        addCard.exp_year = String.valueOf(cardToSave.getExpYear());
        addCard.card_number = cardToSave.getLast4();
        addCard.customer_name = etCardName.getText().toString();
        addCard.country = tvCountryName.getText().toString();
        addCard.card_type = "";
        addCard.client_ip = "";
        if (onlycards) {
            addCard.service_type = "normal";
        } else {
            if (restaurant != null && restaurant.business != null) {
                if (restaurant.business.connect_service) {
                    addCard.service_type = "connect";
                } else {
                    addCard.service_type = "normal";
                }
            }


        }
        AndroidNetworking.post(ApiEndPoints.NEW_CARDS).addBodyParameter(addCard).build()
                .getAsJSONObject(new JSONObjectRequestListener() {
                    @Override
                    public void onResponse(JSONObject response) {
                        String message = "Card successfully added";
                        LogUtils.e(response.toString());
                        ToastUtils.makeToast(getActivity(), message);
                        if (dialogDismissListener != null) {
                            dialogDismissListener.onDialogDismiss(message);
                        }

                        dismiss();

                        if (getActivity() != null) {
                            getActivity().runOnUiThread(() -> progressDialog.dismiss());
                        }
                    }

                    @Override
                    public void onError(ANError anError) {
                        anError.printStackTrace();
                       if (anError.getErrorCode() == 400) {
                            try {
                                JSONObject jsonObject = new JSONObject(anError.getErrorBody().toString());
                                ToastUtils.makeToast(getContext(), jsonObject.getString("message"));
                            } catch (JSONException exception) {
                                exception.printStackTrace();
                            }
                        }
                        if (getActivity() != null) {
                            getActivity().runOnUiThread(() -> progressDialog.dismiss());
                        }

                        if (!CommonFunctions.isConnected(getActivity()))
                            myApp.noInternet(getActivity());
                    }
                });
    }

    final Calendar today = Calendar.getInstance();
    int selectedM = -1, selectedY = -1;

    @Override
    public void onActivityResult(int requestCode, int resultCode, @Nullable Intent data) {
        super.onActivityResult(requestCode, resultCode, data);

        if (requestCode == CODE_REFRESH) {
            if (resultCode == Activity.RESULT_OK) {
                cards.pay.paycardsrecognizer.sdk.Card card = data.getParcelableExtra(ScanCardIntent.RESULT_PAYCARDS_CARD);
                if (card != null) {

                    if (!Validators.isNullOrEmpty(card.getCardNumber())) {
                        etCardName.setText(null);
                        tvCardNumber.setText("**** **** **** ****");
                        etCardNumber.addTextChangedListener(new OtherCardTextWatcher(etCardNumber, tvCardNumber));

                        String cardnumber = card.getCardNumber();
                        etCardNumber.setText(cardnumber);
                    }
                    if (!Validators.isNullOrEmpty(card.getCardHolderName()))
                        etCardName.setText(card.getCardHolderName());
                    if (!Validators.isNullOrEmpty(card.getExpirationDate())) {
                        etExpYear.setText(card.getExpirationDate());
                    }
                }

            } else if (resultCode == Activity.RESULT_CANCELED) {
                LogUtils.e("Scan canceled");
            } else {
                LogUtils.e("Scan failed");
            }
        }
    }
}
