package com.tiffintom.fragment;

import static com.facebook.FacebookSdk.getApplicationContext;

import android.os.Build;
import android.os.Bundle;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.Button;
import android.widget.EditText;
import android.widget.TextView;

import androidx.annotation.NonNull;
import androidx.annotation.Nullable;
import androidx.appcompat.app.AlertDialog;

import com.androidnetworking.AndroidNetworking;
import com.androidnetworking.error.ANError;
import com.androidnetworking.interfaces.JSONObjectRequestListener;
import com.google.android.material.bottomsheet.BottomSheetDialogFragment;
import com.tiffintom.MyApp;
import com.tiffintom.R;
import com.tiffintom.activity.SplashActivity;
import com.tiffintom.common.CommonFunctions;
import com.tiffintom.common.Validators;
import com.tiffintom.network.ApiEndPoints;
import com.tiffintom.network.ApiUtils;
import com.tiffintom.utils.ToastUtils;

import org.json.JSONException;
import org.json.JSONObject;

import java.util.HashMap;

/**
 * Created by Amrish on 09-09-2020.
 */
public class ChangePasswordFragment extends BottomSheetDialogFragment {

    public static ChangePasswordFragment getInstance(String userId, String otp) {
        ChangePasswordFragment fragment = new ChangePasswordFragment();
        Bundle bundle = new Bundle();
        bundle.putString("user_id", userId);
        bundle.putString("otp", otp);
        fragment.setArguments(bundle);
        return fragment;
    }

    public static ChangePasswordFragment getInstance(String userId, boolean isChangePass) {
        ChangePasswordFragment fragment = new ChangePasswordFragment();
        Bundle bundle = new Bundle();
        bundle.putString("user_id", userId);
        bundle.putBoolean("isChangePass", isChangePass);
        fragment.setArguments(bundle);
        return fragment;
    }

    private TextView tvOldPassLabel;
    private EditText etOldPassword, etNewPassword, etConfirmNewPassword;
    private Button btnUpdate;
    private String userId, otp;
    private MyApp myApp = MyApp.getInstance();
    private AlertDialog progressDialog;
    private boolean isChangePass = false;

    @Override
    public int getTheme() {
        if (Build.VERSION.SDK_INT > 21) {
            return R.style.BottomSheetDialogTheme;
        } else {
            return super.getTheme();
        }
    }

    @Nullable
    @Override
    public View onCreateView(@NonNull LayoutInflater inflater, @Nullable ViewGroup container, @Nullable Bundle savedInstanceState) {
        if (getArguments() != null) {
            userId = getArguments().getString("user_id");
            otp = getArguments().getString("otp");
            isChangePass = getArguments().getBoolean("isChangePass");
        }
        return inflater.inflate(R.layout.fragment_change_password, container, false);
    }

    @Override
    public void onViewCreated(@NonNull View view, @Nullable Bundle savedInstanceState) {
        super.onViewCreated(view, savedInstanceState);
        initViews(view);
        setListener();
    }

    private void initViews(View view) {
        progressDialog = CommonFunctions.customProgressDialog(getContext(), "Loading...");
        etConfirmNewPassword = view.findViewById(R.id.etConfirmNewPassword);
        etOldPassword = view.findViewById(R.id.etOldPassword);
        etNewPassword = view.findViewById(R.id.etNewPassword);
        btnUpdate = view.findViewById(R.id.btnUpdate);
        tvOldPassLabel = view.findViewById(R.id.tvOldPassLabel);

        if (isChangePass) {
            tvOldPassLabel.setText("Old password");
            etOldPassword.setHint("Old password");
        } else {
            tvOldPassLabel.setText("OTP");
            etOldPassword.setHint("OTP");
        }
    }

    private void setListener() {
        btnUpdate.setOnClickListener(view -> {
            if (isValid()) {
                changePassword();
            }
        });
    }

    private boolean isValid() {
        etOldPassword.setError(null);
        etNewPassword.setError(null);
        etConfirmNewPassword.setError(null);
        if (Validators.isNullOrEmpty(etOldPassword.getText().toString())) {
            if (isChangePass)
                etOldPassword.setError("Please enter current password");
            else etOldPassword.setError("Please enter otp");
            etOldPassword.requestFocus();
            return false;
        }
        if (Validators.isNullOrEmpty(etNewPassword.getText().toString())) {
            etNewPassword.setError("Please enter new password");
            etNewPassword.requestFocus();
            return false;
        }
        if (!etConfirmNewPassword.getText().toString().equalsIgnoreCase(etNewPassword.getText().toString())) {
            etConfirmNewPassword.setError("Confirm password does not match");
            etConfirmNewPassword.requestFocus();
            return false;
        }
        return true;
    }

    private void changePassword() {
        if (getActivity() != null) getActivity().runOnUiThread(() -> progressDialog.show());
        AndroidNetworking.post(ApiEndPoints.UPDATE_ACCOUNT+userId+"/change-password")
                .addBodyParameter("current_password", etOldPassword.getText().toString())
                .addBodyParameter("password", etNewPassword.getText().toString())
                .build()
                .getAsJSONObject(new JSONObjectRequestListener() {
                    @Override
                    public void onResponse(JSONObject response) {
                        if (getActivity() != null)
                            getActivity().runOnUiThread(() -> progressDialog.dismiss());
                            ToastUtils.makeToast(getActivity(),"Password successfully changed");
                            dismiss();
                    }

                    @Override
                    public void onError(ANError anError) {
                        if (getActivity() != null)
                            getActivity().runOnUiThread(() -> progressDialog.dismiss());
                        anError.printStackTrace();
                        if (anError.getErrorCode() == 400) {
                            try {
                                JSONObject jsonObject = new JSONObject(anError.getErrorBody().toString());
                                ToastUtils.makeToast(getApplicationContext(), jsonObject.getString("message"));
                            } catch (JSONException exception) {
                                exception.printStackTrace();
                            }
                        }else   if (anError.getErrorCode() == 401) {
                            myApp.logout(getActivity());
                        }
                        if (!CommonFunctions.isConnected(getActivity()))
                            myApp.noInternet(getActivity());
                    }
                });
    }
}
