package com.tiffintom.fragment;

import android.content.DialogInterface;
import android.os.Build;
import android.os.Bundle;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.Button;
import android.widget.EditText;

import androidx.annotation.NonNull;
import androidx.annotation.Nullable;
import androidx.appcompat.app.AlertDialog;

import com.google.android.material.bottomsheet.BottomSheetDialogFragment;
import com.google.gson.Gson;
import com.tiffintom.MyApp;
import com.tiffintom.R;
import com.tiffintom.common.CommonFunctions;
import com.tiffintom.common.Validators;
import com.tiffintom.interfaces.DialogDismissListener;
import com.tiffintom.models.Reservation;

/**
 * Created by Amrish on 20-08-2020.
 */
public class ChangeReservationDetailsFragment extends BottomSheetDialogFragment {
    private MyApp myApp = MyApp.getInstance();
    private AlertDialog progressDialog;
    private EditText etFirstName, etLastName, etEmail, etConfirmEmail, etPhone;
    private DialogDismissListener dialogDismissListener;
    private Button btnConfirm;
    private Reservation reservation;

    public static ChangeReservationDetailsFragment getInstance(Reservation reservation) {
        ChangeReservationDetailsFragment fragment = new ChangeReservationDetailsFragment();
        Bundle bundle = new Bundle();
        bundle.putString("reservation", new Gson().toJson(reservation));
        fragment.setArguments(bundle);
        return fragment;
    }

    public void setDialogDismissListener(DialogDismissListener dialogDismissListener) {
        this.dialogDismissListener = dialogDismissListener;
    }

    @Override
    public int getTheme() {
        if (Build.VERSION.SDK_INT > 21) {
            return R.style.BottomSheetDialogTheme;
        } else {
            return super.getTheme();
        }
    }

    @Nullable
    @Override
    public View onCreateView(@NonNull LayoutInflater inflater, @Nullable ViewGroup container, @Nullable Bundle savedInstanceState) {
        return inflater.inflate(R.layout.fragment_change_reservation_details, container, false);
    }

    @Override
    public void onViewCreated(@NonNull View view, @Nullable Bundle savedInstanceState) {
        super.onViewCreated(view, savedInstanceState);
        if (getArguments() != null) {
            reservation = new Gson().fromJson(getArguments().getString("reservation"), Reservation.class);
        }
        initViews(view);
        setListeners();
        updateViews();

    }


    private void initViews(View view) {
        progressDialog = CommonFunctions.customProgressDialog(getActivity(), "Loading...");
        etFirstName = view.findViewById(R.id.etFirstName);
        etPhone = view.findViewById(R.id.etPhone);
        etEmail = view.findViewById(R.id.etEmail);
        etConfirmEmail = view.findViewById(R.id.etConfirmEmail);
        btnConfirm = view.findViewById(R.id.btnConfirm);
    }

    private void updateViews() {
        if (reservation != null) {
            etPhone.setText(reservation.booking_phone);
            etEmail.setText(reservation.booking_email);
            etConfirmEmail.setText(reservation.booking_email);
            etFirstName.setText(reservation.customer_name);
        }
    }

    private void setListeners() {
        btnConfirm.setOnClickListener(view -> {
            if (isValid()) {
                if (dialogDismissListener != null)
                    dialogDismissListener.onDialogDismiss(reservation);
                dismiss();
            }
        });
    }

    private boolean isValid() {
        if (Validators.isNullOrEmpty(etFirstName.getText().toString())) {
            etFirstName.setError("Please enter name");
            etFirstName.requestFocus();
            return false;
        }
        if (!Validators.isEmail(etEmail.getText().toString())) {
            etEmail.setError("Please enter valid email address");
            etEmail.requestFocus();
            return false;
        }
        if (!etConfirmEmail.getText().toString().equals(etEmail.getText().toString())) {
            etConfirmEmail.setError("Entered email address is not matching.");
            etConfirmEmail.requestFocus();
            return false;
        }
        if (Validators.isNullOrEmpty(etPhone.getText().toString())) {
            etPhone.setError("Please enter mobile number");
            etPhone.requestFocus();
            return false;
        }
        reservation.customer_name = etFirstName.getText().toString();
        reservation.booking_phone = etPhone.getText().toString();
        reservation.booking_email = etEmail.getText().toString();
        return true;
    }


    @Override
    public void onDismiss(@NonNull DialogInterface dialog) {
        if (progressDialog != null) {
            progressDialog.dismiss();
        }
        super.onDismiss(dialog);
    }
}
