package com.tiffintom.fragment;

import android.content.DialogInterface;
import android.os.Build;
import android.os.Bundle;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;

import androidx.annotation.NonNull;
import androidx.annotation.Nullable;
import androidx.appcompat.app.AlertDialog;
import androidx.recyclerview.widget.LinearLayoutManager;
import androidx.recyclerview.widget.RecyclerView;

import com.google.android.material.bottomsheet.BottomSheetDialogFragment;
import com.google.gson.Gson;
import com.google.gson.reflect.TypeToken;
import com.tiffintom.MyApp;
import com.tiffintom.R;
import com.tiffintom.adapters.CheckoutEligibleAddressAdapter;
import com.tiffintom.common.CommonFunctions;
import com.tiffintom.interfaces.DialogDismissListener;
import com.tiffintom.models.Address;
import com.tiffintom.models.Header;

import java.lang.reflect.Type;
import java.util.ArrayList;
import java.util.List;

/**
 * Created by Amrish on 20-08-2020.
 */
public class CheckoutAddressesBottomSheetFragment extends BottomSheetDialogFragment {
    private MyApp myApp = MyApp.getInstance();
    private AlertDialog progressDialog;
    private RecyclerView rvAddress;
    private ArrayList<Object> feed = new ArrayList<>();
    private ArrayList<Address> deliverableAddress = new ArrayList<>();
    private ArrayList<Address> nonDeliverableAddress = new ArrayList<>();
    private CheckoutEligibleAddressAdapter addressAdapter;
    private DialogDismissListener dialogDismissListener;

    public static CheckoutAddressesBottomSheetFragment getInstance(ArrayList<Address> deliverableAddresses, ArrayList<Address> outOfDelivery) {
        CheckoutAddressesBottomSheetFragment fragment = new CheckoutAddressesBottomSheetFragment();
        Bundle bundle = new Bundle();
        bundle.putString("deliverable", new Gson().toJson(deliverableAddresses));
        bundle.putString("non_deliverable", new Gson().toJson(outOfDelivery));
        fragment.setArguments(bundle);
        return fragment;
    }

    public void setDialogDismissListener(DialogDismissListener dialogDismissListener) {
        this.dialogDismissListener = dialogDismissListener;
    }

    @Override
    public int getTheme() {
        if (Build.VERSION.SDK_INT > 21) {
            return R.style.BottomSheetDialogTheme;
        } else {
            return super.getTheme();
        }
    }

    @Nullable
    @Override
    public View onCreateView(@NonNull LayoutInflater inflater, @Nullable ViewGroup container, @Nullable Bundle savedInstanceState) {
        return inflater.inflate(R.layout.fragment_checkout_addresses, container, false);
    }

    @Override
    public void onViewCreated(@NonNull View view, @Nullable Bundle savedInstanceState) {
        super.onViewCreated(view, savedInstanceState);
        if (getArguments() != null) {
            Type type = new TypeToken<List<Address>>() {
            }.getType();
            deliverableAddress.addAll(new Gson().fromJson(getArguments().getString("deliverable"), type));
            nonDeliverableAddress.addAll(new Gson().fromJson(getArguments().getString("non_deliverable"), type));
        }
        initViews(view);
        updateViews();
        setListeners();

    }


    private void initViews(View view) {
        progressDialog = CommonFunctions.customProgressDialog(getActivity(), "Loading...");
        rvAddress = view.findViewById(R.id.rvAddress);

        addressAdapter = new CheckoutEligibleAddressAdapter(feed, (position, data) -> {
            if (dialogDismissListener != null) {
                myApp.getMyPreferences().saveCurrentAddress((Address) data);
                dialogDismissListener.onDialogDismiss(data);
                dismiss();
            }
        });
        rvAddress.setLayoutManager(new LinearLayoutManager(getActivity(), RecyclerView.VERTICAL, false));
        rvAddress.setAdapter(addressAdapter);
    }

    private void updateViews() {

        feed.clear();
        feed.addAll(deliverableAddress);
        if (nonDeliverableAddress.size() > 0) {
            feed.add(new Header());
            feed.addAll(nonDeliverableAddress);
        }
        addressAdapter.notifyDataSetChanged();
    }

    private void setListeners() {
    }

    private DialogDismissListener addAddressDialogDismissListener = o -> {
        if (dialogDismissListener != null)
            dialogDismissListener.onDialogDismiss(o);
        dismiss();
    };

    @Override
    public void onDismiss(@NonNull DialogInterface dialog) {
        if (progressDialog != null) {
            progressDialog.dismiss();
        }
        super.onDismiss(dialog);
    }
}
