package com.tiffintom.fragment;

import android.app.Dialog;
import android.graphics.Color;
import android.graphics.drawable.ColorDrawable;
import android.os.Bundle;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.TextView;

import androidx.annotation.NonNull;
import androidx.annotation.Nullable;
import androidx.fragment.app.DialogFragment;

import com.tiffintom.R;
import com.tiffintom.interfaces.DialogDismissListener;

/**
 * Created by Amrish on 11-11-2020.
 */
public class ConfirmationDialogFragment extends DialogFragment {

    @Override
    public void onStart() {
        super.onStart();
        getDialog().getWindow().setLayout(ViewGroup.LayoutParams.MATCH_PARENT, ViewGroup.LayoutParams.WRAP_CONTENT);
    }

    private String title, info;
    private boolean reject;
    private TextView btnConfirm, btnCancel;
    private TextView tvInfo, tvTitle;
    private DialogDismissListener dialogDismissListener;

    public static ConfirmationDialogFragment getInstance(String title, String info) {
        ConfirmationDialogFragment rejectReasonDialog = new ConfirmationDialogFragment();
        Bundle bundle = new Bundle();
        bundle.putString("title", title);
        bundle.putString("info", info);
        rejectReasonDialog.setArguments(bundle);
        return rejectReasonDialog;
    }

    @NonNull
    @Override
    public Dialog onCreateDialog(@Nullable Bundle savedInstanceState) {

        return super.onCreateDialog(savedInstanceState);
    }

    public void setDialogDismissListener(DialogDismissListener dialogDismissListener) {
        this.dialogDismissListener = dialogDismissListener;
    }

    @Nullable
    @Override
    public View onCreateView(@NonNull LayoutInflater inflater, @Nullable ViewGroup container, @Nullable Bundle savedInstanceState) {
        View view = inflater.inflate(R.layout.dialog_confirmation_fragment, container, false);
        getDialog().getWindow().setBackgroundDrawable(new ColorDrawable(Color.TRANSPARENT));
        return view;
    }

    @Override
    public void onViewCreated(@NonNull View view, @Nullable Bundle savedInstanceState) {
        super.onViewCreated(view, savedInstanceState);
        title = getArguments().getString("title");
        info = getArguments().getString("info");
        initViews(view);
        setListeners();
    }

    private void initViews(View view) {
        btnConfirm = view.findViewById(R.id.tvConfirm);
        btnCancel = view.findViewById(R.id.tvCancel);
        tvInfo = view.findViewById(R.id.tvInfo);
        tvTitle = view.findViewById(R.id.tvTitle);

        tvTitle.setText(title);
        tvInfo.setText(info);
    }

    private void setListeners() {
        btnConfirm.setOnClickListener(view -> {
            if (dialogDismissListener != null)
                dialogDismissListener.onDialogDismiss("confirm");
            dismiss();
        });
        btnCancel.setOnClickListener(view -> {
            if (dialogDismissListener != null)
                dialogDismissListener.onDialogDismiss("cancel");
            dismiss();
        });
    }


}
