package com.tiffintom.fragment;

import android.content.Context;
import android.content.Intent;
import android.graphics.Color;
import android.graphics.drawable.ColorDrawable;
import android.os.Bundle;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.Button;
import android.widget.EditText;
import android.widget.ImageView;
import android.widget.LinearLayout;
import android.widget.RadioButton;
import android.widget.RadioGroup;
import android.widget.TextView;

import androidx.annotation.NonNull;
import androidx.annotation.Nullable;
import androidx.appcompat.app.AlertDialog;
import androidx.recyclerview.widget.LinearLayoutManager;
import androidx.recyclerview.widget.RecyclerView;

import com.androidnetworking.AndroidNetworking;
import com.androidnetworking.error.ANError;
import com.androidnetworking.interfaces.JSONArrayRequestListener;
import com.androidnetworking.interfaces.JSONObjectRequestListener;
import com.google.gson.Gson;
import com.google.gson.reflect.TypeToken;
import com.stripe.android.ApiResultCallback;
import com.stripe.android.PaymentIntentResult;
import com.stripe.android.Stripe;
import com.stripe.android.model.ConfirmPaymentIntentParams;
import com.stripe.android.model.PaymentIntent;
import com.tiffintom.MyApp;
import com.tiffintom.R;
import com.tiffintom.activity.NoInternetActivity;
import com.tiffintom.activity.SplashActivity;
import com.tiffintom.adapters.TransactionHistoryAdapter;
import com.tiffintom.base.BaseFragment;
import com.tiffintom.common.CommonFunctions;
import com.tiffintom.interfaces.DialogDismissListener;
import com.tiffintom.interfaces.MoneyTextWatcher;
import com.tiffintom.models.PaymentMethod;
import com.tiffintom.models.RequestCreator;
import com.tiffintom.models.SavedCard;
import com.tiffintom.models.User;
import com.tiffintom.models.WalletHistory;
import com.tiffintom.network.ApiEndPoints;
import com.tiffintom.network.ApiUtils;
import com.tiffintom.utils.LogUtils;
import com.tiffintom.utils.ToastUtils;

import org.json.JSONArray;
import org.json.JSONException;
import org.json.JSONObject;

import java.lang.ref.WeakReference;
import java.lang.reflect.Type;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;

import static com.tiffintom.network.ApiEndPoints.ACCOUNT;
import static com.tiffintom.utils.Constants.CODE_NO_INTERNET;

/**
 * Created by Amrish on 31-08-2020.
 */
public class CreditsFragment extends BaseFragment {
    private Button btnAdd;
    private TextView tvBalance, tvCardNumber, tvRecentTransactionLabel;
    private RadioButton rb10, rb20, rb30, rbEdit, rbCustom;
    private RadioGroup rgAmount;
    private LinearLayout llCard;
    private User loggedInUser = myApp.getMyPreferences().getLoggedInUser();
    private PaymentMethod savedPaymentMethod = myApp.getMyPreferences().getDefaultPaymentMethod();
    private LinearLayout llLoading;
    private ArrayList<SavedCard> CardArraylist = new ArrayList<>();
    private ArrayList<SavedCard> savedCards = new ArrayList<>();
    private ArrayList<WalletHistory> transactionHistory = new ArrayList<>();
    private RecyclerView rvTransactions;
    private TransactionHistoryAdapter historyAdapter;
    private float loadAmount = 10;
    private ImageView ivMethod;

    public static CreditsFragment getInstance() {
        return new CreditsFragment();
    }

    @Nullable
    @Override
    public View onCreateView(@NonNull LayoutInflater inflater, @Nullable ViewGroup container, @Nullable Bundle savedInstanceState) {
        return inflater.inflate(R.layout.fragment_credits, container, false);
    }

    @Override
    public void onViewCreated(@NonNull View view, @Nullable Bundle savedInstanceState) {
        super.onViewCreated(view, savedInstanceState);
        updateViews();
        displayCard();
        setListeners();
        fetchTransactions();
    }

    @Override
    protected void initViews(View view) {
        super.initViews(view);
        tvBalance = view.findViewById(R.id.tvBalance);
        llCard = view.findViewById(R.id.cvCard);
        rb10 = view.findViewById(R.id.rb10);
        rb20 = view.findViewById(R.id.rb20);
        rb30 = view.findViewById(R.id.rb30);
        rbCustom = view.findViewById(R.id.rbCustom);
        rbEdit = view.findViewById(R.id.rbEdit);
        rgAmount = view.findViewById(R.id.rgAmount);
        tvCardNumber = view.findViewById(R.id.tvCardNumber);
        ivMethod = view.findViewById(R.id.ivMethod);
        tvRecentTransactionLabel = view.findViewById(R.id.tvRecentTransactionLabel);
        btnAdd = view.findViewById(R.id.btnAdd);
        llLoading = view.findViewById(R.id.llLoading);
        rvTransactions = view.findViewById(R.id.rvTransactionHistory);
        rvTransactions.setLayoutManager(new LinearLayoutManager(getActivity(), RecyclerView.VERTICAL, false));
        historyAdapter = new TransactionHistoryAdapter(getActivity(), transactionHistory, null);
        rvTransactions.setNestedScrollingEnabled(false);
        rvTransactions.setAdapter(historyAdapter);
    }

    private void updateViews() {
        tvBalance.setText("Balance: " + myApp.getCurrencySymbol() + MyApp.df.format(loggedInUser.wallet_amount));
        rb10.setText(myApp.getCurrencySymbol() + "10.00");
        rb20.setText(myApp.getCurrencySymbol() + "20.00");
        rb30.setText(myApp.getCurrencySymbol() + "30.00");

    }

    private void setListeners() {
        llCard.setOnClickListener(view -> {
            PaymentMethodsFragment paymentMethodsFragment = PaymentMethodsFragment.getInstance(true);
            paymentMethodsFragment.show(getChildFragmentManager(), "cards");
            paymentMethodsFragment.setDialogDismissListener(cardSelectLister);
        });
        btnAdd.setOnClickListener(view -> {
            if (isValid()) createPaymentIntent();
        });
        rgAmount.setOnCheckedChangeListener((radioGroup, i) -> {
            if (i == R.id.rb10) {
                loadAmount = 10;
            } else if (i == R.id.rb20) {
                loadAmount = 20;
            } else if (i == R.id.rb30) {
                loadAmount = 30;
            } else if (i == R.id.rbCustom) {
                loadAmount = customAmount;
            } else if (i == R.id.rbEdit) {
                showEditAmount();
            }
        });
    }

    float customAmount = 0;

    private void showEditAmount() {
        AlertDialog alertDialog = new AlertDialog.Builder(getActivity()).create();
        alertDialog.getWindow().setBackgroundDrawable(new ColorDrawable(Color.TRANSPARENT));
        LayoutInflater inflater = (LayoutInflater) getActivity().getSystemService(Context.LAYOUT_INFLATER_SERVICE);
        View dialogView = inflater.inflate(R.layout.dialog_edit_fund, null);

        TextView tvCancel = dialogView.findViewById(R.id.tvCancel);
        TextView tvConfirm = dialogView.findViewById(R.id.tvConfirm);
        EditText etAmount = dialogView.findViewById(R.id.etFundAmount);
        etAmount.addTextChangedListener(new MoneyTextWatcher(etAmount));
        if (customAmount > 0) {
            etAmount.setText(String.valueOf(customAmount));
            etAmount.setSelection(etAmount.getText().toString().length());
        }
        tvConfirm.setOnClickListener(view -> {
            if (!etAmount.getText().toString().isEmpty() && Float.parseFloat(etAmount.getText().toString().replace(",", "")) > 0) {
                customAmount = Float.parseFloat(etAmount.getText().toString().replace(",", ""));
                loadAmount = customAmount;
                rbCustom.setText(myApp.getCurrencySymbol() + MyApp.df.format(customAmount));
                rbCustom.setChecked(true);
                rbCustom.setVisibility(View.VISIBLE);
                alertDialog.dismiss();
            } else {
                ToastUtils.makeToast(getActivity(), "Please enter valid amount");
            }
        });
        tvCancel.setOnClickListener(view -> {
            if (loadAmount == 10) {
                rb10.setChecked(true);
            } else if (loadAmount == 20) {
                rb20.setChecked(true);
            } else if (loadAmount == 30) {
                rb30.setChecked(true);
            } else if (customAmount > 0) {
                rbCustom.setChecked(true);
            }
            alertDialog.dismiss();
        });
        alertDialog.setView(dialogView);
        alertDialog.setCancelable(false);
        alertDialog.show();
    }

    private boolean isValid() {
        if (savedPaymentMethod == null) {
            ToastUtils.makeToast(getActivity(), "Please select card first.");
            return false;
        }
        return true;
    }

    private void displayCard() {
        if (savedPaymentMethod != null && savedPaymentMethod.type.equals("stripe") && savedPaymentMethod.service_type.equals("normal")) {
            tvCardNumber.setText("**** " + savedPaymentMethod.card_number);
            llLoading.setVisibility(View.GONE);
            ivMethod.setImageResource(myApp.paymentMethodLogo(savedPaymentMethod));
        } else {
            fetchCards();
        }
    }

    private DialogDismissListener cardSelectLister = o -> {
        if (o instanceof PaymentMethod) {
            savedPaymentMethod = (PaymentMethod) o;
            displayCard();
        }
    };

    private void fetchCards() {
        if (getActivity() != null)
            getActivity().runOnUiThread(() -> llLoading.setVisibility(View.VISIBLE));

        AndroidNetworking.get(ApiEndPoints.NEW_CARDS).addQueryParameter("customer_id", String.valueOf(loggedInUser.id)).addQueryParameter("nopaginate", "1").build()
                .getAsJSONArray(new JSONArrayRequestListener() {
                    @Override
                    public void onResponse(JSONArray response) {
                        Type type = new TypeToken<List<SavedCard>>() {
                        }.getType();
                        savedCards.clear();

                        savedCards.addAll(new Gson().fromJson(response.toString(), type));
                        for (SavedCard savedCard : savedCards) {

                            if (savedCard.service_type.equalsIgnoreCase("normal")) {
                                CardArraylist.add(savedCard);
                            }
                        }
                        if (getActivity() != null) {
                            getActivity().runOnUiThread(() -> {
                                setPaymentMethod();
                                btnAdd.setVisibility(View.VISIBLE);
                            });
                        }
                        if (getActivity() != null)
                            getActivity().runOnUiThread(() -> llLoading.setVisibility(View.GONE));
                    }

                    @Override
                    public void onError(ANError anError) {
                        if (getActivity() != null)
                            getActivity().runOnUiThread(() -> llLoading.setVisibility(View.GONE));
                        anError.printStackTrace();
                        if (anError.getErrorCode() == 400) {
                            setPaymentMethod();
                            btnAdd.setVisibility(View.VISIBLE);
                        }
                        if (!CommonFunctions.isConnected(getActivity()))
                            myApp.noInternet(getActivity());
                    }
                });
    }

    private void setPaymentMethod() {
        if (CardArraylist.size() > 0) {
            SavedCard card = CardArraylist.get(0);
            savedPaymentMethod = new PaymentMethod();
            savedPaymentMethod.type = "stripe";
            savedPaymentMethod.title = "**** " + card.card_number;
            savedPaymentMethod.card_number = card.card_number;
            savedPaymentMethod.card_id = card.id;
            savedPaymentMethod.stripe_customer_id = card.stripe_customer_id;
            savedPaymentMethod.stripe_token_id = card.stripe_token_id;
            savedPaymentMethod.card_brand = card.card_brand;
            savedPaymentMethod.exp_month = card.exp_month;
            savedPaymentMethod.exp_year = card.exp_year;
            savedPaymentMethod.exp_year = card.exp_year;
            savedPaymentMethod.service_type = card.service_type;
            savedPaymentMethod.method_logo = myApp.paymentMethodLogo(savedPaymentMethod);
            displayCard();

        }

    }

    private void fetchTransactions() {
        AndroidNetworking.get(ApiEndPoints.NEW_WALLET_HISTORY)
                .addQueryParameter("customer_id", String.valueOf(loggedInUser.id))
                .addQueryParameter("nopaginate", "1")
                .build()
                .getAsJSONArray(new JSONArrayRequestListener() {
                    @Override
                    public void onResponse(JSONArray response) {
                          Type type = new TypeToken<List<WalletHistory>>() {
                            }.getType();
                            transactionHistory.clear();
                            transactionHistory.addAll(new Gson().fromJson(response.toString(), type));
                            if (getActivity() != null) {
                                getActivity().runOnUiThread(() -> {
                                    historyAdapter.notifyDataSetChanged();
                                });
                            }
                            fetchProfile();
                    }

                    @Override
                    public void onError(ANError anError) {
                        anError.printStackTrace();
                        LogUtils.e("WALLET HISTORY ERROR: ", anError.getErrorBody());

                        if (!CommonFunctions.isConnected(getActivity()))
                            myApp.noInternet(getActivity());
                    }
                });
    }

    private void fetchProfile() {
        AndroidNetworking.get(ApiEndPoints.NEW_ACCOUNT)
                .build()
                .getAsJSONObject(new JSONObjectRequestListener() {
                    @Override
                    public void onResponse(JSONObject response) {
                        User user1 = new Gson().fromJson(response.toString(), User.class);
                        myApp.getMyPreferences().saveLoggedInUser(user1);
                        tvBalance.setText("Balance: " + myApp.getCurrencySymbol() + MyApp.df.format(user1.wallet_amount));

                    }

                    @Override
                    public void onError(ANError anError) {
                        anError.printStackTrace();
                        if (!CommonFunctions.isConnected(getContext())) {
                            startActivityForResult(new Intent(getContext(), NoInternetActivity.class), CODE_NO_INTERNET);
                        }
                    }
                });
    }


    private HashMap<String, String> generatePaymentIntentParams() {
        HashMap<String, String> params = new HashMap<>();
        params.put("customer_id", String.valueOf(loggedInUser.id));
        params.put("amount", String.valueOf(loadAmount));
        params.put("stripe_token_id", savedPaymentMethod.stripe_token_id);
        params.put("stripe_customer_id", savedPaymentMethod.stripe_customer_id);
        params.put("service_type", "normal");
        LogUtils.e("Payment Params: ", new Gson().toJson(params));
        return params;
    }

    private void createPaymentIntent() {
        if (getActivity() != null) {
            getActivity().runOnUiThread(() -> progressDialog.show());
        }
        AndroidNetworking.post(ApiEndPoints.NEW_CARDS + "payment-intent").addBodyParameter(generatePaymentIntentParams())
                .build()
                .getAsJSONObject(new JSONObjectRequestListener() {
                    @Override
                    public void onResponse(JSONObject response) {
                        if (getActivity() != null) {
                            getActivity().runOnUiThread(() -> progressDialog.dismiss());
                        }
                        LogUtils.e(new Gson().toJson(response));
                        try {
                            String paymentSecret = response.getString("payment_intent_id");
                            performStripePayment(paymentSecret);
                        } catch (JSONException e) {
                            e.printStackTrace();


                        }

                    }

                    @Override
                    public void onError(ANError anError) {
                        anError.printStackTrace();
                        LogUtils.e("GENERATE PAYMENT INTENT ERROR");

                        if (getActivity() != null) {
                            getActivity().runOnUiThread(() -> progressDialog.dismiss());
                        }
                        if (!CommonFunctions.isConnected(getActivity()))
                            startActivityForResult(new Intent(getActivity(), NoInternetActivity.class), CODE_NO_INTERNET);
                    }
                });
    }

    private Stripe stripe;
    private String transactionId;

    private void performStripePayment(String paymentSecret) {
        if (getActivity() != null) {
            getActivity().runOnUiThread(() -> progressDialog.show());
        }
        ConfirmPaymentIntentParams confirmParams = ConfirmPaymentIntentParams
                .createWithPaymentMethodId(savedPaymentMethod.stripe_token_id, paymentSecret);
        stripe = new Stripe(getContext(), myApp.getStripePublishKey());
        stripe.confirmPayment(this, confirmParams);
    }

    @Override
    public void onActivityResult(int requestCode, int resultCode, @Nullable Intent data) {
        super.onActivityResult(requestCode, resultCode, data);
        if (data != null) {
            stripe.onPaymentResult(requestCode, data, new PaymentResultCallback(this));
        }
    }

    private static final class PaymentResultCallback
            implements ApiResultCallback<PaymentIntentResult> {

        @NonNull
        private final WeakReference<CreditsFragment> activityRef;

        PaymentResultCallback(CreditsFragment activity) {
            activityRef = new WeakReference<CreditsFragment>(activity);
            final CreditsFragment a = activityRef.get();
            if (activity.getActivity() != null && !activity.getActivity().isFinishing())
                a.progressDialog.show();
        }


        @Override
        public void onSuccess(@NonNull PaymentIntentResult result) {
            final CreditsFragment activity = activityRef.get();
            if (activity == null) {
                return;
            }
            PaymentIntent paymentIntent = result.getIntent();
            activity.managePaymentResponse(paymentIntent);
        }

        @Override
        public void onError(@NonNull Exception e) {
            final CreditsFragment activity = activityRef.get();
            if (activity == null) {
                return;
            }

            if (activity.getActivity() != null && !activity.getActivity().isFinishing())
                activity.progressDialog.dismiss();
            // Payment request failed – allow retrying using the same payment method
            ToastUtils.makeToast(activity.getActivity(), "Payment request failed");
        }
    }

    private void managePaymentResponse(PaymentIntent paymentIntent) {
        PaymentIntent.Status status = paymentIntent.getStatus();
        if (status == PaymentIntent.Status.Succeeded) {
            // Payment completed successfully
            ToastUtils.makeToast(getActivity(), "Payment successful");
            transactionId = paymentIntent.getId();
            addFunds();
        } else if (status == PaymentIntent.Status.RequiresPaymentMethod) {
            // Payment failed
            ToastUtils.makeToast(getActivity(), "Card authentication failed, Please retry");
        }
    }

    private void addFunds() {
        if (getActivity() != null) getActivity().runOnUiThread(() -> progressDialog.show());
        AndroidNetworking.post(ApiEndPoints.NEW_WALLET_HISTORY)
                .addBodyParameter("customer_id", String.valueOf(loggedInUser.id))
                .addBodyParameter("amount", String.valueOf(loadAmount))
                .addBodyParameter("purpose", "")
                .addBodyParameter("transaction_type", "")
                .addBodyParameter("transaction_details", "")
                .build()
                .getAsJSONObject(new JSONObjectRequestListener() {
                    @Override
                    public void onResponse(JSONObject response) {
                        if (getActivity() != null)
                            getActivity().runOnUiThread(() -> progressDialog.dismiss());
                        ToastUtils.makeToast(getActivity(), "Money added successfully");
                        fetchTransactions();
                    }

                    @Override
                    public void onError(ANError anError) {
                        anError.printStackTrace();
                        if (getActivity() != null)
                            getActivity().runOnUiThread(() -> progressDialog.dismiss());
                        LogUtils.e("ERROR IN ADD MONEY");

                        if (!CommonFunctions.isConnected(getActivity()))
                            myApp.noInternet(getActivity());
                    }
                });
    }
}
