package com.tiffintom.fragment;

import android.os.Build;
import android.os.Bundle;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.Button;
import android.widget.LinearLayout;
import android.widget.ProgressBar;
import android.widget.TextView;

import androidx.annotation.NonNull;
import androidx.annotation.Nullable;
import androidx.recyclerview.widget.GridLayoutManager;
import androidx.recyclerview.widget.LinearLayoutManager;
import androidx.recyclerview.widget.RecyclerView;

import com.airbnb.lottie.LottieAnimationView;
import com.androidnetworking.AndroidNetworking;
import com.androidnetworking.error.ANError;
import com.androidnetworking.interfaces.JSONArrayRequestListener;
import com.androidnetworking.interfaces.JSONObjectRequestListener;
import com.google.android.material.bottomsheet.BottomSheetDialogFragment;
import com.google.gson.Gson;
import com.google.gson.reflect.TypeToken;
import com.tiffintom.MyApp;
import com.tiffintom.R;
import com.tiffintom.adapters.CuisionitemAdapter;
import com.tiffintom.adapters.SelectedCuisionAdapter;
import com.tiffintom.interfaces.DialogDismissListener;
import com.tiffintom.interfaces.RecyclerViewItemClickListener;
import com.tiffintom.models.Cuisine;
import com.tiffintom.network.ApiEndPoints;
import com.tiffintom.utils.LogUtils;
import com.tiffintom.utils.ToastUtils;

import org.json.JSONArray;
import org.json.JSONException;
import org.json.JSONObject;

import java.lang.reflect.Type;
import java.util.ArrayList;
import java.util.List;

public class CuisineBottomSheetFragment extends BottomSheetDialogFragment {

    private RecyclerView rvCuisions, rvRemoveCuision;
    private CuisionitemAdapter cuisionitemAdapter;
    private SelectedCuisionAdapter selectedCuisionAdapter;
    private ArrayList<Cuisine> cuisineArrayList;
    private ArrayList<Cuisine> selectedCusionList;
    private RecyclerViewItemClickListener recyclerViewItemClickListener;
    private TextView tvCountSelectedCuision;
    public static int selectedCuisineCount = 0;
    private DialogDismissListener dialogDismissListener;
    private Button btnConfirm;
    private MyApp myApp = new MyApp();
    private ProgressBar progressBar;
    private LinearLayout ll_standardBottomSheet;
    private LottieAnimationView lodingView;

    public static CuisineBottomSheetFragment getInstance(ArrayList<Cuisine> selectedCuisines) {
        CuisineBottomSheetFragment cuisineBottomSheetFragment = new CuisineBottomSheetFragment();
        Bundle bundle = new Bundle();
        bundle.putString("selected_cusines", new Gson().toJson(selectedCuisines));
        cuisineBottomSheetFragment.setArguments(bundle);
        return cuisineBottomSheetFragment;
    }

    public void setDialogDismissListener(DialogDismissListener dialogDismissListener) {
        this.dialogDismissListener = dialogDismissListener;
    }

    @Override
    public int getTheme() {
        if (Build.VERSION.SDK_INT > 21) {
            return R.style.BottomSheetDialogTheme;
        } else {
            return super.getTheme();
        }
    }

    @Nullable
    @Override
    public View onCreateView(@NonNull LayoutInflater inflater, @Nullable ViewGroup container, @Nullable Bundle savedInstanceState) {
        selectedCuisineCount = 0;
        selectedCusionList = new ArrayList<>();
        cuisineArrayList = new ArrayList<>();
        if (getArguments() != null) {
            Type type = new TypeToken<List<Cuisine>>() {
            }.getType();
            if (getArguments().getString("selected_cusines") != null) {
                selectedCusionList.addAll(new Gson().fromJson(getArguments().getString("selected_cusines"), type));
            }
        }

        return inflater.inflate(R.layout.fragment_cuisine_type, container, false);
    }

    @Override
    public void onViewCreated(@NonNull View view, @Nullable Bundle savedInstanceState) {
        super.onViewCreated(view, savedInstanceState);

        initView(view);
        setListners();
        fetchCuisines();

    }

    private void fetchCuisines() {
        if (getActivity() != null)
            getActivity().runOnUiThread(() -> lodingView.setVisibility(View.VISIBLE));

        AndroidNetworking.get(ApiEndPoints.NEW_CUISINELIST)
                .addQueryParameter("nopaginate","1")
                .build()
                .getAsJSONArray(new JSONArrayRequestListener() {
                    @Override
                    public void onResponse(JSONArray response) {
                        cuisineArrayList.clear();
                                Type type = new TypeToken<List<Cuisine>>() {
                                }.getType();
                                cuisineArrayList.addAll(new Gson().fromJson(response.toString(), type));
                                for (Cuisine c : cuisineArrayList) {
                                    int index = selectedCusionList.indexOf(c);
                                    if (index != -1) {
                                        c.isSelected = selectedCusionList.get(index).isSelected;
                                    }
                                    if (c.isSelected) {
                                        selectedCuisineCount++;
                                    }
                                }
                                if (getActivity() != null) {
                                    getActivity().runOnUiThread(() -> {
                                        cuisionitemAdapter.notifyDataSetChanged();
                                    });
                                }
                        if (getActivity() != null)
                            getActivity().runOnUiThread(() -> {
                                lodingView.setVisibility(View.GONE);
                                ll_standardBottomSheet.setVisibility(View.VISIBLE);
                            });
                    }

                    @Override
                    public void onError(ANError anError) {
                        anError.printStackTrace();
                        LogUtils.e("::CUISINELIST ERROR::");
                        if (getActivity() != null)
                            getActivity().runOnUiThread(() -> lodingView.setVisibility(View.GONE));
                    }
                });
    }


    private void setListners() {
        cuisionitemAdapter = new CuisionitemAdapter(getActivity(), cuisineArrayList, (position, data) -> {
            Cuisine cuisine = (Cuisine) data;
            if (cuisine.isSelected) {
                selectedCusionList.add(cuisine);
                selectedCuisionAdapter.notifyDataSetChanged();
            } else {
                int cuisionIndex = selectedCusionList.indexOf(cuisine);
                if (cuisionIndex != -1) {
                    selectedCusionList.remove(cuisionIndex);
                    selectedCuisionAdapter.notifyDataSetChanged();
                }
            }
        });
        selectedCuisionAdapter = new SelectedCuisionAdapter(getActivity(), selectedCusionList, (position, data) -> {

            Cuisine cuisine = (Cuisine) data;
            if (cuisine.isSelected) {
                selectedCusionList.remove(cuisine);
                int index = cuisineArrayList.indexOf(cuisine);
                if (index != -1) {
                    cuisineArrayList.get(index).isSelected = false;
                }
                selectedCuisionAdapter.notifyDataSetChanged();
                cuisionitemAdapter.notifyDataSetChanged();
            }

        });

        rvCuisions.setLayoutManager(new GridLayoutManager(getActivity(), 3));
        rvCuisions.setAdapter(cuisionitemAdapter);

        rvRemoveCuision.setLayoutManager(new LinearLayoutManager(getActivity(), RecyclerView.HORIZONTAL, false));
        rvRemoveCuision.setAdapter(selectedCuisionAdapter);


        btnConfirm.setOnClickListener(view -> {
            if (selectedCusionList.size() > 0) {
                if (dialogDismissListener != null) {
                    dialogDismissListener.onDialogDismiss(selectedCusionList);
                    dismiss();
                }
            } else {
                ToastUtils.makeToast(getActivity(), "Please Select cuisines ");
            }

        });
    }

    private void initView(View view) {

        rvCuisions = view.findViewById(R.id.rvCuisine);
        rvRemoveCuision = view.findViewById(R.id.rvRemoveCusion);
        tvCountSelectedCuision = view.findViewById(R.id.tvSelectedCuisine);
        btnConfirm = view.findViewById(R.id.btnConfirm);
        progressBar = view.findViewById(R.id.pbLoading);
        ll_standardBottomSheet = view.findViewById(R.id.standardBottomSheet);
        lodingView = view.findViewById(R.id.lodingView);
    }

}
