package com.tiffintom.fragment;

import android.os.Bundle;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.TextView;

import androidx.annotation.NonNull;
import androidx.annotation.Nullable;
import androidx.recyclerview.widget.LinearLayoutManager;
import androidx.recyclerview.widget.RecyclerView;

import com.androidnetworking.error.ANError;
import com.androidnetworking.interfaces.JSONArrayRequestListener;
import com.google.gson.Gson;
import com.google.gson.reflect.TypeToken;
import com.tiffintom.MyApp;
import com.tiffintom.R;
import com.tiffintom.adapters.CheckoutEligibleAddressAdapter;
import com.tiffintom.base.BaseFragment;
import com.tiffintom.common.CommonFunctions;
import com.tiffintom.interfaces.DialogDismissListener;
import com.tiffintom.models.Address;
import com.tiffintom.models.ApiError;
import com.tiffintom.models.Header;
import com.tiffintom.models.User;
import com.tiffintom.network.ApiUtils;
import com.tiffintom.utils.LogUtils;
import com.tiffintom.utils.ToastUtils;

import org.json.JSONArray;
import org.json.JSONException;

import java.lang.reflect.Type;
import java.util.ArrayList;
import java.util.List;

public class DeliverableAddressesFragment extends BaseFragment {
    private RecyclerView rvAddress;
    private ArrayList<Address> addresses = new ArrayList<>();
    private CheckoutEligibleAddressAdapter addressAdapter;
    private DialogDismissListener dialogDismissListener;
    private TextView tvCancel, tvConfirm;
    private User loggedInUser = myApp.getMyPreferences().getLoggedInUser();
    private int res_id;
    private ArrayList<Object> feed = new ArrayList<>();
    private ArrayList<Address> nonDeliverableAddress = new ArrayList<>();
    private ArrayList<Address> DeliverableAddress = new ArrayList<>();

    public static DeliverableAddressesFragment getInstance() {
        return new DeliverableAddressesFragment();
    }

    public void setDialogDismissListener(DialogDismissListener dialogDismissListener) {
        this.dialogDismissListener = dialogDismissListener;
    }

    @Nullable
    @Override
    public View onCreateView(@NonNull LayoutInflater inflater, @Nullable ViewGroup container, @Nullable Bundle savedInstanceState) {
        new Thread(() -> {
            res_id = myApp.getAppDatabase().cartDao().getRestaurantId();
        }).start();
        return inflater.inflate(R.layout.fragment_addresslist, container, false);
    }

    @Override
    public void onViewCreated(@NonNull View view, @Nullable Bundle savedInstanceState) {
        super.onViewCreated(view, savedInstanceState);
        if (myApp.getMyPreferences().getLoggedInUser() != null) {
            fetchAddress();
        } else {
            ToastUtils.makeToast(getActivity(), "Please login first");
        }
        setListeners();
    }

    private void setListeners() {
        tvCancel.setOnClickListener(view -> {
            if (dialogDismissListener != null) dialogDismissListener.onDialogDismiss(null);
        });
        tvConfirm.setOnClickListener(view -> {
            Address defaultAddress = MyApp.getInstance().getMyPreferences().getCurrentAddress();
            if (defaultAddress != null) {
                for (Address ad : nonDeliverableAddress) {
                    if (ad.id == defaultAddress.id) {
                        defaultAddress = null;
                        break;
                    }
                }
            }
            if (dialogDismissListener != null)
                dialogDismissListener.onDialogDismiss(defaultAddress);
        });

    }

    private void fetchAddress() {
        if (getActivity() != null) {
            getActivity().runOnUiThread(() -> progressDialog.show());
        }
        ApiUtils.getCheckoutAddresses(loggedInUser.id, res_id,1)
                .getAsJSONArray(new JSONArrayRequestListener() {
                    @Override
                    public void onResponse(JSONArray response) {
                        if (getActivity() != null) {
                            getActivity().runOnUiThread(() -> progressDialog.dismiss());
                        }
                        Type type = new TypeToken<List<Address>>() {
                        }.getType();
                           addresses.clear();
                           DeliverableAddress.clear();
                            nonDeliverableAddress.clear();
                            addresses.addAll(new Gson().fromJson(response.toString(), type));
                            if (addresses!=null && addresses.size()!=0) {
                                for (Address address : addresses) {
                                    if (address.delivery_status.equalsIgnoreCase("out_of_delivery")) {
                                        nonDeliverableAddress.add(address);
                                    }else
                                    {
                                        DeliverableAddress.add(address);
                                    }
                                }
                                feed.clear();
                                feed.addAll(DeliverableAddress);
                                if (nonDeliverableAddress.size() > 0) {
                                    feed.add(new Header());
                                    feed.addAll(nonDeliverableAddress);
                                }
                            }

                        addressAdapter.notifyDataSetChanged();
                        if (addresses.size() == 0)
                            DeliverableChangeAddressDialog.viewPager.setCurrentItem(1);
                    }

                    @Override
                    public void onError(ANError anError) {
                        anError.printStackTrace();
                        if (anError.getErrorCode() == 400) {
                            DeliverableChangeAddressDialog.viewPager.setCurrentItem(1);
                        }
                        if (getActivity() != null) {
                            getActivity().runOnUiThread(() -> progressDialog.dismiss());
                        }
                        if (!CommonFunctions.isConnected(getActivity()))
                            myApp.noInternet(getActivity());
                    }
                });

    }

    @Override
    protected void initViews(View view) {
        super.initViews(view);
        rvAddress = view.findViewById(R.id.rvAddress);
        tvCancel = view.findViewById(R.id.tvCancel);
        tvConfirm = view.findViewById(R.id.tvConfirm);

        addressAdapter = new CheckoutEligibleAddressAdapter(feed, true, (position, data) -> {
            if (dialogDismissListener != null) {
                dialogDismissListener.onDialogDismiss(data);
            }
        });
        rvAddress.setLayoutManager(new LinearLayoutManager(getActivity(), RecyclerView.VERTICAL, false));
        rvAddress.setAdapter(addressAdapter);
    }
}
