package com.tiffintom.fragment;

import android.graphics.Color;
import android.graphics.drawable.ColorDrawable;
import android.os.Bundle;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.TextView;

import androidx.annotation.NonNull;
import androidx.annotation.Nullable;
import androidx.fragment.app.DialogFragment;
import androidx.fragment.app.FragmentPagerAdapter;
import androidx.viewpager.widget.ViewPager;

import com.google.android.material.appbar.AppBarLayout;
import com.google.android.material.tabs.TabLayout;
import com.google.gson.Gson;
import com.tiffintom.MyApp;
import com.tiffintom.R;
import com.tiffintom.adapters.ViewpagerAdapter;
import com.tiffintom.interfaces.DialogDismissListener;
import com.tiffintom.models.Address;
import com.tiffintom.models.User;

public class DeliverableChangeAddressDialog extends DialogFragment {
    private boolean hideList;
    private Address savedAddress;

    public static DeliverableChangeAddressDialog getInstance(boolean hideList) {
        DeliverableChangeAddressDialog changeAddressDialog = new DeliverableChangeAddressDialog();
        Bundle bundle = new Bundle();
        bundle.putBoolean("hideList", hideList);
        changeAddressDialog.setArguments(bundle);
        return changeAddressDialog;
    }

    private DialogDismissListener dialogDismissListener;
    private MyApp myApp = MyApp.getInstance();
    private User loggedInUser = myApp.getMyPreferences().getLoggedInUser();
    private String res_id;

    public static DeliverableChangeAddressDialog getInstance(boolean hideList, Address data) {
        DeliverableChangeAddressDialog changeAddressDialog = new DeliverableChangeAddressDialog();
        Bundle bundle = new Bundle();
        bundle.putBoolean("hideList", hideList);
        bundle.putString("saved_address", new Gson().toJson(data));
        changeAddressDialog.setArguments(bundle);
        return changeAddressDialog;
    }

    public static DeliverableChangeAddressDialog getInstance(String res_id) {
        DeliverableChangeAddressDialog changeAddressDialog = new DeliverableChangeAddressDialog();
        Bundle bundle = new Bundle();
        bundle.putString("res_id", res_id);
        changeAddressDialog.setArguments(bundle);
        return changeAddressDialog;
    }

    public void setDialogDismissListener(DialogDismissListener dialogDismissListener) {
        this.dialogDismissListener = dialogDismissListener;
    }

    public static ViewPager viewPager;

    @Nullable
    @Override
    public View onCreateView(@NonNull LayoutInflater inflater, @Nullable ViewGroup container, @Nullable Bundle savedInstanceState) {
        if (getArguments() != null) {
            hideList = getArguments().getBoolean("hideList");
            String savedAddressValue = getArguments().getString("saved_address");
            res_id = getArguments().getString("res_id");
            if (savedAddressValue != null)
                savedAddress = new Gson().fromJson(savedAddressValue, Address.class);
        }

        View dialogView = inflater.inflate(R.layout.dialog_checkout_changeaddress, container, false);

        getDialog().getWindow().setBackgroundDrawable(new ColorDrawable(Color.TRANSPARENT));
        TabLayout tabLayout = dialogView.findViewById(R.id.tabLayout);
        viewPager = dialogView.findViewById(R.id.viewPager);
        AppBarLayout appBarLayout = dialogView.findViewById(R.id.appbarLayout);
        TextView tvTitle = dialogView.findViewById(R.id.tvTitle);

        DeliverableAddressesFragment deliverableAddressesFragment = DeliverableAddressesFragment.getInstance();
        NewAddressFragment newAddressFragment = NewAddressFragment.getInstance(savedAddress, res_id);

        newAddressFragment.setDialogDismissListener(o -> {
            if (o != null && dialogDismissListener != null) {
                dialogDismissListener.onDialogDismiss(o);
            }
            dismiss();
        });
        deliverableAddressesFragment.setDialogDismissListener(o -> {
            if (o != null && dialogDismissListener != null) {
                dialogDismissListener.onDialogDismiss(o);
            }
            dismiss();
        });

        ViewpagerAdapter viewpagerAdapter = new ViewpagerAdapter(getChildFragmentManager(), FragmentPagerAdapter.BEHAVIOR_RESUME_ONLY_CURRENT_FRAGMENT);
        if (!hideList)
            viewpagerAdapter.addFragment(deliverableAddressesFragment, "Address book");
        viewpagerAdapter.addFragment(newAddressFragment, "New address");

        viewPager.setAdapter(viewpagerAdapter);
        tabLayout.setupWithViewPager(viewPager);

        if (hideList) {
            appBarLayout.setVisibility(View.GONE);
            tvTitle.setVisibility(View.VISIBLE);
        }
        return dialogView;
    }

}
