package com.tiffintom.fragment;

import static com.tiffintom.MyApp.isDineInOpen;
import static com.tiffintom.utils.Constants.CODE_NO_INTERNET;
import static com.tiffintom.utils.Constants.CODE_REFRESH;
import static com.tiffintom.utils.Constants.PERMISSION_CODE_STORAGE;

import android.Manifest;
import android.content.BroadcastReceiver;
import android.content.Context;
import android.content.Intent;
import android.content.IntentFilter;
import android.content.pm.PackageManager;
import android.graphics.Color;
import android.graphics.drawable.ColorDrawable;
import android.location.Location;
import android.net.Uri;
import android.os.Bundle;
import android.os.CountDownTimer;
import android.os.Handler;
import android.os.Looper;
import android.provider.Settings;
import android.text.Editable;
import android.text.TextWatcher;
import android.view.KeyEvent;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.EditText;
import android.widget.ImageView;
import android.widget.LinearLayout;
import android.widget.RelativeLayout;
import android.widget.TextView;

import androidx.annotation.NonNull;
import androidx.annotation.Nullable;
import androidx.appcompat.app.AlertDialog;
import androidx.coordinatorlayout.widget.CoordinatorLayout;
import androidx.core.app.ActivityCompat;
import androidx.localbroadcastmanager.content.LocalBroadcastManager;
import androidx.recyclerview.widget.LinearLayoutManager;
import androidx.recyclerview.widget.LinearSmoothScroller;
import androidx.recyclerview.widget.PagerSnapHelper;
import androidx.recyclerview.widget.RecyclerView;
import androidx.recyclerview.widget.SnapHelper;

import com.airbnb.lottie.LottieAnimationView;
import com.androidnetworking.AndroidNetworking;
import com.androidnetworking.common.ANRequest;
import com.androidnetworking.error.ANError;
import com.androidnetworking.interfaces.JSONArrayRequestListener;
import com.androidnetworking.interfaces.JSONObjectRequestListener;
import com.androidnetworking.interfaces.ParsedRequestListener;
import com.google.android.material.appbar.AppBarLayout;
import com.google.gson.Gson;
import com.google.gson.JsonSyntaxException;
import com.google.gson.reflect.TypeToken;
import com.tiffintom.BuildConfig;
import com.tiffintom.MyApp;
import com.tiffintom.R;
import com.tiffintom.activity.AuthActivity;
import com.tiffintom.activity.MainActivity;
import com.tiffintom.activity.NoInternetActivity;
import com.tiffintom.activity.TransportActivity;
import com.tiffintom.adapters.DineinChildrenAdapter;
import com.tiffintom.adapters.OffersAdapter;
import com.tiffintom.adapters.RestaurantDineInMenuAdapter;
import com.tiffintom.adapters.RestaurantMenuAdapter;
import com.tiffintom.adapters.RestaurantMenuCategoryAdapter;
import com.tiffintom.adapters.SnippetAdapter;
import com.tiffintom.base.BaseFragment;
import com.tiffintom.common.CommonFunctions;
import com.tiffintom.common.MyLocation;
import com.tiffintom.common.Validators;
import com.tiffintom.interfaces.DialogDismissListener;
import com.tiffintom.interfaces.DialogDismissListenerWithArray;
import com.tiffintom.interfaces.RecyclerViewItemClickListener;
import com.tiffintom.models.AllergyItem;
import com.tiffintom.models.BusinessRestaurant;
import com.tiffintom.models.CartSummary;
import com.tiffintom.models.Category;
import com.tiffintom.models.DineinCartItem;
import com.tiffintom.models.DineinModels.Children;
import com.tiffintom.models.DineinModels.CreateOrderAddonParmsItem;
import com.tiffintom.models.DineinModels.CreateOrderIngredientParamsItem;
import com.tiffintom.models.DineinModels.CreateOrderParamsItem;
import com.tiffintom.models.DineinModels.CreatedOrder;
import com.tiffintom.models.DineinModels.CustomerParams;
import com.tiffintom.models.DineinModels.DineinCategories;
import com.tiffintom.models.DineinModels.DineinMenus;
import com.tiffintom.models.DineinModels.DineinSiteSettings;
import com.tiffintom.models.DineinModels.OrderItem;
import com.tiffintom.models.DineinModels.OrderItemAddon;
import com.tiffintom.models.DineinModels.OrderItemIngredient;
import com.tiffintom.models.Header;
import com.tiffintom.models.Menu;
import com.tiffintom.models.OfferSimplify;
import com.tiffintom.models.OrderHistory;
import com.tiffintom.models.Postcode;
import com.tiffintom.models.Reservation;
import com.tiffintom.models.RestaurantMiniSnippet;
import com.tiffintom.models.UploadCartItems;
import com.tiffintom.models.User;
import com.tiffintom.network.ApiEndPoints;
import com.tiffintom.network.ApiUtils;
import com.tiffintom.notification.Config;
import com.tiffintom.utils.LogUtils;
import com.tiffintom.utils.ToastUtils;

import org.json.JSONArray;
import org.json.JSONException;
import org.json.JSONObject;

import java.lang.reflect.Type;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;

import ru.tinkoff.scrollingpagerindicator.ScrollingPagerIndicator;

/**
 * Created by Amrish on 28-10-2020.
 */
public class DineInHomeFragment extends BaseFragment {
    public static DineInHomeFragment getInstance() {
        return new DineInHomeFragment();
    }

    public static final int REQUEST_LOCATION = 1;
    private TextView tvLocation, tvChangeLocation;
    private EditText etSearch;
    private ImageView ivReservation;
    private LinearLayout llFilter;
    private LinearLayout llPostcode, llLocation, llDineIn;
    private EditText etPostcode;
    private ImageView ivCurrentLocation, ivLocationBack;
    private LottieAnimationView dineinAnimationView;
    private ImageView ivSearchPostcode;
    private RelativeLayout customToolbar;

    private RelativeLayout llMiniSnippet;
    private ArrayList<Object> snippetArrayList = new ArrayList<>();
    private RecyclerView rvBottomSnippet;
    private SnippetAdapter snippetAdapter;
    private ScrollingPagerIndicator snippetIndicator;

    private Postcode currentPostcode;
    private RecyclerView rvMenu, rvMenuCategory, rvMenuSubCategory;
    private RestaurantDineInMenuAdapter menuAdapter;
    private ArrayList<Object> filtered = new ArrayList<>();
    private ArrayList<Object> feed = new ArrayList<>();
    private ArrayList<Category> menucategoryArrayList = new ArrayList<>();
    private ArrayList<Category> categories = new ArrayList<>();
    private ArrayList<AllergyItem> allergies = new ArrayList<>();
    private BusinessRestaurant restaurant = myApp.getMyPreferences().getCurrentDineInRestaurantDetail();
    private RestaurantMenuCategoryAdapter categoryAdapter;
    private DineinChildrenAdapter childrenAdapter;
    RecyclerView.SmoothScroller categorySmoothScroller, childrenSmoothScroller, menuSmoothScroller;
    LinearLayoutManager menuLayoutManager, categoryLayoutManager, childrenLayoutManager;
    private User loggedInUser = myApp.getMyPreferences().getLoggedInUser();
    //private ArrayList<DineinSiteSettings> dineinSiteSettings = myApp.getMyPreferences().getDineinSiteSettings();
    private ArrayList<AllergyItem> mainAllergyItems = new ArrayList<>();
    private ArrayList<Menu> suggested = new ArrayList<>();

    private LinearLayout llLoading;
    private RecyclerView rvOffers;
    private LinearLayout llOffers;
    private ScrollingPagerIndicator offersIndicator;
    private OffersAdapter offersAdapter;
    private ArrayList<OfferSimplify> offers = new ArrayList<>();
    private AppBarLayout appBarLayout;
    private TextView tvRestaurantName;
    private ImageView ivShareTop, ivSearchTop, ivRestaurant;
    private RelativeLayout llCheckout;
    private ImageView ivBackTop;
    private TextView tvTotal;
    private String qr_code, table_number, table_Id;
    private int guest_count;
    private String message;
    private CoordinatorLayout llData;
    private LottieAnimationView postcodeAnimation;
    View rootView;

    private boolean showMenuImage = false;
    private ArrayList<DineinCategories> dineinCategories = new ArrayList<>();
    private ArrayList<Children> childrens = new ArrayList<>();
    private ArrayList<DineinMenus> dineinMenus = new ArrayList<>();
    private ArrayList<DineinMenus> dineinAllMenus = new ArrayList<>();
    private DialogDismissListener clearBasketLister;
    ArrayList<UploadCartItems> cartItemArrayList = new ArrayList<>();
    private BroadcastReceiver broadcastReceiver;

    private ArrayList<OrderItem> selectedCartItem = new ArrayList<>();
    ArrayList<OrderItemAddon> orderItemAddons = new ArrayList<>();
    ArrayList<OrderItemIngredient> orderItemIngredients = new ArrayList<>();

    private DineinSiteSettings productImageSettings;
    private CreatedOrder createdOrder;
    private DineinSiteSettings orderCompleteSetting;
    private CartSummary cartSummary;
    private ArrayList<DineinCartItem> dineinCartItems = new ArrayList<>();
    private boolean fromHistory = false;
    private int order_status_id;

    @Nullable
    @Override
    public View onCreateView(@NonNull LayoutInflater inflater, @Nullable ViewGroup container, @Nullable Bundle savedInstanceState) {
        if (getArguments() != null) {
            isDineInOpen = getArguments().getBoolean("is_dinein", false);
            qr_code = getArguments().getString("qr_code");
            table_number = getArguments().getString("table_number");
            table_Id = getArguments().getString("table_id");
            guest_count = getArguments().getInt("guests");
            fromHistory = getArguments().getBoolean("fromHistory");
        } else {
            isDineInOpen = false;
            qr_code = null;
        }
        rootView = inflater.inflate(R.layout.fragment_dinein_home, container, false);
        return rootView;
    }

    @Override
    public void onViewCreated(@NonNull View view, @Nullable Bundle savedInstanceState) {
        super.onViewCreated(view, savedInstanceState);
        if (loggedInUser == null) {
            RelativeLayout.LayoutParams params = (RelativeLayout.LayoutParams) llDineIn.getLayoutParams();
            params.addRule(RelativeLayout.ALIGN_PARENT_RIGHT);
            params.setMargins(0, 0, 0, 0);
            llDineIn.setLayoutParams(params);
        }
        updateViews();
        displayAddress();
        setListeners();
        fetchData();
        TransportActivity.isOrderPlaced = false;

        broadcastReceiver = new BroadcastReceiver() {
            public void onReceive(Context context, Intent intent) {

                boolean check_update = intent.getBooleanExtra("refresh", false);
                if (check_update) {
                    checkAndUpdateCheckoutLayout();
                }
            }
        };
    }

    @Override
    protected void initViews(View view) {
        super.initViews(view);
        appBarLayout = view.findViewById(R.id.filterAppbar);
        llData = view.findViewById(R.id.llData);
        postcodeAnimation = view.findViewById(R.id.animationPostcode);
        currentPostcode = myApp.getMyPreferences().getCurrentPostcode();
        tvUserName = view.findViewById(R.id.tvUserName);
        tvChangeLocation = view.findViewById(R.id.tvChangeLocation);
        llFilter = view.findViewById(R.id.llFilter);
        customToolbar = view.findViewById(R.id.customToolbar);
        etSearch = view.findViewById(R.id.etSearch);
        tvLocation = view.findViewById(R.id.tvLocation);
        ivReservation = view.findViewById(R.id.ivReservation);
        ivSearchPostcode = view.findViewById(R.id.ivSearch);
        llLocation = view.findViewById(R.id.llLocation);
        llDineIn = view.findViewById(R.id.llDineIn);
        llPostcode = view.findViewById(R.id.llPostcode);
        etPostcode = view.findViewById(R.id.etPostcode);
        ivCurrentLocation = view.findViewById(R.id.ivCurrentLocation);
        ivLocationBack = view.findViewById(R.id.ivBackArrow);
        ivBackTop = view.findViewById(R.id.ivBackArrowTop);
        rvBottomSnippet = view.findViewById(R.id.rvSnippetItems);
        snippetIndicator = view.findViewById(R.id.recyclerIndicator);

        llCheckout = view.findViewById(R.id.llCheckout);
        tvTotal = view.findViewById(R.id.tvTotal);

        dineinAnimationView = view.findViewById(R.id.dinein_animation_view);

        ivRestaurant = view.findViewById(R.id.ivRestaurant);
        tvRestaurantName = view.findViewById(R.id.tvRestaurant);
        ivSearchTop = view.findViewById(R.id.ivSearchTop);
        ivShareTop = view.findViewById(R.id.ivShare);

        llLoading = view.findViewById(R.id.llLoading);
        llMiniSnippet = view.findViewById(R.id.llMiniSnippet);
        tvTotal = view.findViewById(R.id.tvTotal);

        rvMenu = view.findViewById(R.id.rvMenu);
        rvMenuCategory = view.findViewById(R.id.rvMenuCategory);
        rvMenuSubCategory = view.findViewById(R.id.rvMenuSubCategory);

        rvOffers = view.findViewById(R.id.rvOffers);
        llOffers = view.findViewById(R.id.llOffers);
        offersIndicator = view.findViewById(R.id.offersIndicator);
        offersAdapter = new OffersAdapter(getActivity(), offers);

        orderCompleteSetting = myApp.findSetting("order_complete_mode");
        if (isDineInOpen)
            rvMenuSubCategory.setVisibility(View.VISIBLE);
        else
            rvMenuSubCategory.setVisibility(View.GONE);

        rvOffers.setLayoutManager(new LinearLayoutManager(getActivity(), RecyclerView.HORIZONTAL, false));
        new PagerSnapHelper().attachToRecyclerView(rvOffers);
        rvOffers.setAdapter(offersAdapter);
        offersIndicator.attachToRecyclerView(rvOffers);

        menuAdapter = new RestaurantDineInMenuAdapter(getActivity(), filtered, showMenuImage, (position, data) -> {
            if (restaurant.online_order.equalsIgnoreCase("no")) {
                showRestaurantIsClosed("Unfortunately, " + restaurant.restaurant_name + " is currently not accepting online orders", false);
            } else {
                if (data instanceof Header) {
                    if (((Header) data).addonId == 1) {
                        //Search clicked
                        startActivityForResult(new Intent(getActivity(), TransportActivity.class).putExtra("destination", "search_menu").putExtra("name", restaurant.restaurant_name).putExtra("hideToolbar", true), CODE_REFRESH);
                    }
                } else {
                    if (!isDineInOpen && (restaurant.restaurant_delivery.equalsIgnoreCase("no") && restaurant.restaurant_pickup.equalsIgnoreCase("no") && (restaurant.restaurant_status.equalsIgnoreCase("close")))) {
                        showRestaurantIsClosed("Unfortunately, " + restaurant.restaurant_name + " is currently closed", false);
                    } else {
                        if (isDineInOpen) {
                            clearBasketLister = o -> {
                                if (o instanceof String) {
                                    if (((String) o).equalsIgnoreCase("confirm")) {
                                        menuItemClick(position, data);
                                    }
                                }
                            };
                            checkForItemsInCart();
                        } else {
                            menuItemClick(position, data);
                        }
                    }
                }
            }
        });
        menuLayoutManager = new LinearLayoutManager(getActivity(), RecyclerView.VERTICAL, false);
        rvMenu.setLayoutManager(menuLayoutManager);
        rvMenu.setAdapter(menuAdapter);


        categoryAdapter = new RestaurantMenuCategoryAdapter(categories, true, this::scrollToCategory);
        categoryLayoutManager = new LinearLayoutManager(getActivity(), RecyclerView.HORIZONTAL, false);
        rvMenuCategory.setLayoutManager(categoryLayoutManager);
        rvMenuCategory.setAdapter(categoryAdapter);

        //sub category/children adapter
        childrenAdapter = new DineinChildrenAdapter(childrens, new RecyclerViewItemClickListener() {
            @Override
            public void onItemClick(int position, Object data) {
                Children children = (Children) data;
                fetchDineinMenuByCategory(String.valueOf(children.id));
                categoryAdapter.notifyDataSetChanged();

            }
        });
        childrenLayoutManager = new LinearLayoutManager(getActivity(), RecyclerView.HORIZONTAL, false);
        rvMenuSubCategory.setLayoutManager(childrenLayoutManager);
        rvMenuSubCategory.setAdapter(childrenAdapter);


        categorySmoothScroller = new LinearSmoothScroller(getActivity()) {
            @Override
            protected int getVerticalSnapPreference() {
                return LinearSmoothScroller.SNAP_TO_START;
            }
        };
        childrenSmoothScroller = new LinearSmoothScroller(getActivity()) {
            @Override
            protected int getVerticalSnapPreference() {
                return LinearSmoothScroller.SNAP_TO_START;
            }
        };
        menuSmoothScroller = new LinearSmoothScroller(getActivity()) {
            @Override
            protected int getVerticalSnapPreference() {
                return LinearSmoothScroller.SNAP_TO_START;
            }
        };
        rvBottomSnippet.setLayoutManager(new LinearLayoutManager(getActivity(), RecyclerView.HORIZONTAL, false));
        SnapHelper helper = new PagerSnapHelper();
        helper.attachToRecyclerView(rvBottomSnippet);
        snippetAdapter = new SnippetAdapter(snippetArrayList, (position, data) -> {
            manageLastOrderClick(data);
        });
        rvBottomSnippet.setAdapter(snippetAdapter);
        snippetIndicator.attachToRecyclerView(rvBottomSnippet);
    }

    private void setListeners() {
        message = "Checkout " + getString(R.string.app_name) + " App." +
                "iOS - https://apps.apple.com/in/app/id1437199436\n" +
                "Android - https://play.google.com/store/apps/details?id=" + BuildConfig.APPLICATION_ID;

        ivShareTop.setOnClickListener(view -> {
            Intent share = new Intent(Intent.ACTION_SEND);
            share.setType("text/plain");
            share.putExtra(Intent.EXTRA_TEXT, message);
            LogUtils.e("message::", message);
            startActivity(Intent.createChooser(share, "Share using"));
        });

        etSearch.setOnKeyListener((textView, i, keyEvent) -> {

            return false;
        });
        etSearch.addTextChangedListener(new TextWatcher() {
            @Override
            public void beforeTextChanged(CharSequence charSequence, int i, int i1, int i2) {

            }

            @Override
            public void onTextChanged(CharSequence charSequence, int i, int i1, int i2) {

            }

            @Override
            public void afterTextChanged(Editable editable) {
                if (editable.toString().length() == 0) {
                    filtered.clear();
                    filtered.addAll(feed);
                    menuAdapter.notifyDataSetChanged();
                } else {
                    searchMenu();
                }
            }
        });

        llDineIn.setOnClickListener(view -> {
            if (checkCameraPermission()) {
                openDineinScan();
            }
        });

        etPostcode.setOnKeyListener((textview, i, keyEvent) -> {
            if (keyEvent.getKeyCode() == KeyEvent.KEYCODE_BACK) {
                getActivity().onBackPressed();
            } else if (keyEvent.getKeyCode() == KeyEvent.KEYCODE_ENTER) {

                if (!Validators.isNullOrEmpty(etPostcode.getText().toString())) {
                    String postCode = etPostcode.getText().toString();
                    new Thread(() -> searchPostcode(postCode)).start();
                    if (getActivity() != null)
                        CommonFunctions.hideKeyboard(getActivity(), etPostcode);
                } else
                    ToastUtils.makeToast(getActivity(), "Please enter postcode");
            }
            return false;
        });

        ivCurrentLocation.setOnClickListener(view -> {
            if (checkLocationServicePermission()) {
                startLocationService();
            }
        });
        llLocation.setOnClickListener(view -> {
            llFilter.setVisibility(View.GONE);
            llLocation.setVisibility(View.GONE);
            llPostcode.setVisibility(View.VISIBLE);
            postcodeAnimation.setVisibility(View.VISIBLE);
            llMiniSnippet.setVisibility(View.GONE);
            llData.setVisibility(View.GONE);
            // showEnterLocation();
        });
        ivLocationBack.setOnClickListener(view -> {
            llFilter.setVisibility(View.VISIBLE);
            llLocation.setVisibility(View.VISIBLE);
            llPostcode.setVisibility(View.GONE);
            llData.setVisibility(View.VISIBLE);
            postcodeAnimation.setVisibility(View.GONE);
            if (snippetArrayList.size() > 0) {
                llMiniSnippet.setVisibility(View.VISIBLE);
            }
            if (getActivity() != null)
                CommonFunctions.hideKeyboard(getActivity(), etPostcode);
            //  hideEnterLocation();
        });
        ivSearchPostcode.setOnClickListener(view -> {
            if (Validators.isNullOrEmpty(etPostcode.getText().toString())) {
                ToastUtils.makeToast(getActivity(), "Please enter postcode");
            } else {
                String postCode = etPostcode.getText().toString();
                new Thread(() -> searchPostcode(postCode)).start();
            }
        });
        ivReservation.setOnClickListener(view -> {
            if (loggedInUser == null) {
                startActivity(new Intent(getActivity(), AuthActivity.class));
            } else {
                MakeReservationBottomSheetFragment makeReservationBottomSheetFragment = MakeReservationBottomSheetFragment.getInstance(restaurant.id);
                makeReservationBottomSheetFragment.show(getChildFragmentManager(), "make_reservation");
                makeReservationBottomSheetFragment.setDialogDismissListener(o -> {
                    if (o instanceof Reservation) {
                        Reservation reservation = (Reservation) o;
                        //open trackorder fragment
                        Intent intent = new Intent(getActivity(), TransportActivity.class);
                        intent.putExtra("destination", "trackOrder");
                        intent.putExtra("hideToolbar", true);
                        intent.putExtra("reservation_id", String.valueOf(reservation.id));
                        intent.putExtra("booking_id", reservation.booking_id);
                        getActivity().startActivity(intent);
                    }
                });
            }
        });
        ivSearchTop.setOnClickListener(view -> {
            startActivityForResult(new Intent(getActivity(), TransportActivity.class).putExtra("destination", "search_menu").putExtra("name", restaurant.restaurant_name).putExtra("hideToolbar", true).putExtra("is_dinein", isDineInOpen), CODE_REFRESH);
        });
        llCheckout.setOnClickListener(view -> {
            if (loggedInUser == null) {
                startActivity(new Intent(getActivity(), AuthActivity.class));
            } else {
                validateRestaurantOpeningAndCheckout();
            }
        });
        ivBackTop.setOnClickListener(view -> {
            getActivity().onBackPressed();
        });
    }

    private void checkForItemsInCart() {
        new Thread(() -> {
            int dbRestId = myApp.getAppDatabase().cartDao().getRestaurantId();
            if (dbRestId > 0) {
                if (getActivity() != null) {
                    getActivity().runOnUiThread(() -> {
                        showOtherRestaurantCartWarning();
                    });
                }
            } else {
                if (clearBasketLister != null) {
                    clearBasketLister.onDialogDismiss("confirm");
                }
            }
        }).start();
    }

    private void showOtherRestaurantCartWarning() {
        try {
            AlertDialog alertDialog = new AlertDialog.Builder(getActivity()).create();
            alertDialog.getWindow().setBackgroundDrawable(new ColorDrawable(Color.TRANSPARENT));
            LayoutInflater inflater = (LayoutInflater) getActivity().getSystemService(Context.LAYOUT_INFLATER_SERVICE);
            View dialogView = inflater.inflate(R.layout.dialog_remove_card, null);

            // TextView tvTitle = dialogView.findViewById(R.id.tvTitle);
            TextView tvMessage = dialogView.findViewById(R.id.tvMessage);
            TextView tvCancel = dialogView.findViewById(R.id.tvCancel);
            TextView tvConfirm = dialogView.findViewById(R.id.tvConfirm);
            /*tvTitle.setVisibility(View.VISIBLE);
            tvTitle.setText("Start new basket?");*/
            tvMessage.setText("Your basket already contains some items. Do you wish to clear the basket?");
            tvConfirm.setOnClickListener(view -> {
                try {
                    new Thread(() -> {
                        myApp.getMyPreferences().deleteOrderRestaurant();
                        myApp.getAppDatabase().cartDao().nukeTable();
                        CommonFunctions.deleteCartOnline("");
                        if (clearBasketLister != null) {
                            clearBasketLister.onDialogDismiss("confirm");
                        }
                    }).start();
                    MainActivity.tvCartItemsCountBadge.setText("0");
                    MainActivity.tvCartItemsCountBadge.setVisibility(View.GONE);
                    alertDialog.dismiss();
                } catch (Exception e) {
                    e.printStackTrace();
                }
            });
            tvCancel.setOnClickListener(view -> {
                alertDialog.dismiss();
            });
            alertDialog.setView(dialogView);
            alertDialog.setCancelable(false);
            alertDialog.show();
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    private void openDineinScan() {
       /* Intent i = new Intent(getActivity(), DineinScanActivity.class);
        startActivity(i);*/
    }

    public void hideDefaultUI() {
        ivRestaurant.setVisibility(View.GONE);
        tvUserName.setVisibility(View.GONE);
        tvRestaurantName.setVisibility(View.VISIBLE);
        ivSearchTop.setVisibility(View.GONE);
        ivShareTop.setVisibility(View.VISIBLE);
        llDineIn.setVisibility(View.GONE);
    }

    public void showDefaultUI() {
        ivRestaurant.setVisibility(View.VISIBLE);
        if (MyApp.getInstance().getMyPreferences().getLoggedInUser() != null) {
            tvUserName.setVisibility(View.VISIBLE);
        } else {
            tvUserName.setVisibility(View.GONE);
        }
        tvRestaurantName.setVisibility(View.GONE);
        ivSearchTop.setVisibility(View.GONE);
        ivShareTop.setVisibility(View.GONE);
        if (!isDineInOpen && restaurant.dine_in)
            llDineIn.setVisibility(View.VISIBLE);
        else llDineIn.setVisibility(View.GONE);
    }

    private void openCheckoutScreen() {
        startActivityForResult(new Intent(getActivity(), TransportActivity.class).putExtra("destination", "dinein_checkout").putExtra("is_dinein", true).putExtra("table_id", table_Id).putExtra("name", "Dine in Checkout").putExtra("table_number", table_number).putExtra("guests", guest_count).putExtra("qr_code", qr_code).putExtra("resId", restaurant.id).putExtra("created_order", new Gson().toJson(createdOrder)), CODE_REFRESH);
    }

    private void validateRestaurantOpeningAndCheckout() {
        // No need to check for Tiffintom
       /* if (!isDineInOpen && (restaurant.restaurant_delivery.equalsIgnoreCase("no") && restaurant.restaurant_pickup.equalsIgnoreCase("no") && (restaurant.restaurant_status.equalsIgnoreCase("close")))) {
            showRestaurantIsClosed("Unfortunately, " + restaurant.restaurant_name + " is currently closed", false);
        } else {*/
        openCheckoutScreen();
        // }
    }


    private void showRestaurantIsClosed(String message, boolean shouldGoBack) {
        RestaurantClosedDialogFragment restaurantClosedDialogFragment = RestaurantClosedDialogFragment.getInstance(message);
        restaurantClosedDialogFragment.show(getChildFragmentManager(), "restaurant_closed");
        restaurantClosedDialogFragment.setDialogDismissListener(o -> {
            if (shouldGoBack) {
                if (getActivity() != null)
                    getActivity().onBackPressed();
            }
        });
    }

    private void scrollToCategory(int position, Object data) {
        int selectedCatIndex = feed.indexOf(data);
        Category category = (Category) data;
        childrens.clear();
        if (category.children != null && category.children.size() > 0) {
            rvMenuSubCategory.setVisibility(View.VISIBLE);
            for (int i = 0; i < category.children.size(); i++) {
                Children children = new Children();
                children.name = category.children.get(i).title;
                children.id = category.children.get(i).id;
                childrens.add(children);
                feed.add(childrens);
            }
        } else {
            rvMenuSubCategory.setVisibility(View.GONE);
        }
        if (category.children != null && category.children.size() > 0) {
            if (DineinChildrenAdapter.selectedCategory == 0 || DineinChildrenAdapter.selectedCategory == category.children.get(0).id) {
                fetchDineinMenuByCategory(String.valueOf(category.children.get(0).id));
            }
        } else
            fetchDineinMenuByCategory(String.valueOf(category.category_id));


//        if (selectedCatIndex != -1) {
//            menuSmoothScroller.setTargetPosition(selectedCatIndex);
//        } else {
//            menuSmoothScroller.setTargetPosition(0);
//        }
//        appBarLayout.setExpanded(false, true);
//        menuLayoutManager.startSmoothScroll(menuSmoothScroller);
        childrenAdapter.notifyDataSetChanged();
        menuAdapter.notifyDataSetChanged();
    }

    private void updateViews() {
        ivReservation.setVisibility(View.VISIBLE);
        ivBackTop.setVisibility(View.VISIBLE);

        if (restaurant != null) {
            tvRestaurantName.setText(restaurant.restaurant_name);
            if (restaurant.dine_in && !isDineInOpen) {
                llDineIn.setVisibility(View.VISIBLE);
            } else {
                llDineIn.setVisibility(View.GONE);
            }
            if (restaurant.restaurant_booktable.equalsIgnoreCase("yes")) {
                ivReservation.setVisibility(View.VISIBLE);
            } else {
                ivReservation.setVisibility(View.GONE);
            }

        } else {
            llDineIn.setVisibility(View.GONE);
            ivReservation.setVisibility(View.GONE);
        }
        if (isDineInOpen && !Validators.isNullOrEmpty(qr_code)) {
            llDineIn.setVisibility(View.INVISIBLE);
            ivReservation.setVisibility(View.GONE);
        }

    }

    private void menuItemClick(int position, Object data) {
        if (data instanceof DineinMenus) {
            DineinMenus menu = (DineinMenus) data;
            getOrderItemFromProduct(menu, position);
        }
    }

    private void getOrderItemFromProduct(DineinMenus product, int position) {
        OrderItem orderItem = new OrderItem();
        orderItem.id = String.valueOf(product.id);
        orderItem.order_id = String.valueOf(product.id);
        orderItem.product_name = product.name;
        orderItem.product_id = String.valueOf(product.id);
        orderItem.product_short_name = product.short_name;
        orderItem.quantity = 1;
        orderItem.is_banquet = product.is_banquet;
        orderItem.price = product.price;
        orderItem.sub_total = (orderItem.quantity * orderItem.price);
        orderItem.total = orderItem.sub_total + orderItem.addons_price + orderItem.ingredients_price;
        orderItem.updater_id = String.valueOf(myApp.getMyPreferences().getDineinUser().id);
        openProductEdit(product, orderItem, position);

    }

    private void openProductEdit(DineinMenus menus, OrderItem orderItem, int position) {
        //first time product.qty=1;
        FragmentMenuAddonIngrediants addonIngrediants = FragmentMenuAddonIngrediants.getInstance(menus, orderItem, showMenuImage, restaurant.business.id);
        addonIngrediants.show(getChildFragmentManager(), "addons_ingredients");
        addonIngrediants.setDialogDismissListener(new DialogDismissListenerWithArray() {
            @Override
            public void onDialogDismiss(ArrayList<OrderItem> arrayList) {

                if (arrayList != null) {
                    // LogUtils.e("Array Data::" + new Gson().toJson(arrayList));

                    //add item to cart
                    setUpAddonIngredientInText(arrayList, orderItem, position);
                    getArrayData(arrayList);
                }
            }
        });
    }

    ArrayList<CreateOrderParamsItem> dineinOrderParamsItems = new ArrayList<>();
    CreateOrderParamsItem orderParamsItem = new CreateOrderParamsItem();

    private void getArrayData(ArrayList<OrderItem> dineinOrderItems) {
        if (dineinOrderItems.size() > 0) {
            dineinOrderParamsItems.clear();
            for (OrderItem oit : dineinOrderItems) {
                CreateOrderParamsItem orderItem = new CreateOrderParamsItem();
                orderItem.id = oit.id;
                orderItem.updater_id = oit.updater_id;
                //pending
                //    orderItem.product_id = oit.product_id;
                orderItem.product_name = oit.product_name;
                orderItem.addons_price = oit.addons_price;
                orderItem.sent_to_kitchen = oit.sent_to_kitchen;
                orderItem.instruction_price = oit.instruction_price;
                orderItem.ingredients_price = oit.ingredients_price;
                orderItem.special_instruction = oit.special_instruction;
                orderItem.price = oit.price;
                orderItem.quantity = oit.quantity;
                for (OrderItemAddon oia : oit.order_item_addons) {
                    CreateOrderAddonParmsItem addon = new CreateOrderAddonParmsItem();
                    addon.addon_id = oia.addon_id;
                    addon.updater_id = oia.updater_id;
                    addon.price = oia.price;
                    addon.quantity = oia.quantity;
                    addon.id = oia.id;
                    addon.total = oia.total;
                    orderItem.order_item_addons.add(addon);
                }
                for (OrderItemIngredient oi : oit.order_item_ingredients) {
                    CreateOrderIngredientParamsItem ingredient = new CreateOrderIngredientParamsItem();
                    ingredient.id = oi.id;
                    ingredient.product_ingredient_id = oi.product_ingredient_id;
                    ingredient.updater_id = oi.updater_id;
                    ingredient.quantity = oi.quantity;
                    ingredient.with = oi.with;
                    ingredient.without = oi.without;
                    ingredient.price = oi.price;
                    ingredient.total = oi.total;
                    orderItem.order_item_ingredients.add(ingredient);
                }
                dineinOrderParamsItems.add(orderItem);
            }

            for (CreateOrderParamsItem paramsItem : dineinOrderParamsItems) {
                orderParamsItem = paramsItem;
            }
        }

    }

    private void setUpAddonIngredientInText(ArrayList<OrderItem> orderItems, OrderItem item, int position) {
        StringBuilder selectedProductAddons = new StringBuilder();
        orderItemAddons.clear();
        orderItemIngredients.clear();
        float total_amount = 0;
        if ((orderItems.get(0).order_item_addons != null) && (orderItems.get(0).order_item_addons.size() > 0)) {
            orderItemAddons.addAll(orderItems.get(0).order_item_addons);
            item.order_item_addons.addAll(orderItems.get(0).order_item_addons);
        }
        if ((orderItems.get(0).order_item_ingredients != null) && (orderItems.get(0).order_item_ingredients.size() > 0)) {
            orderItemIngredients.addAll(orderItems.get(0).order_item_ingredients);
        }

        if (orderItemAddons.size() > 0) {
            selectedProductAddons.append("(");
            for (OrderItemAddon orderItemAddon : orderItemAddons) {
                if (selectedProductAddons.length() > 0) {
                    selectedProductAddons.append("(");
                }
                selectedProductAddons.append(orderItemAddon.addon_name).append(": ").append(myApp.getCurrencySymbol()).append(MyApp.df.format(orderItemAddon.price));
                if (selectedProductAddons.length() > 0) {
                    selectedProductAddons.append(")");
                }
                total_amount = total_amount + orderItemAddon.total;
            }
            selectedProductAddons.append(")");
        }

        if (orderItemIngredients.size() > 0) {
            if (orderItemAddons.size() > 0) {
                selectedProductAddons.append(",");
            }
            selectedProductAddons.append("(");
            for (OrderItemIngredient orderItemIngredient : orderItemIngredients) {
                if (selectedProductAddons.length() > 0) {
                    selectedProductAddons.append("(");
                }
                if (orderItemIngredient.with) {
                    selectedProductAddons.append("Extra ");
                    selectedProductAddons.append(orderItemIngredient.quantity).append(" ");
                }
                if (orderItemIngredient.without) {
                    selectedProductAddons.append("No ");
                }
                selectedProductAddons.append(orderItemIngredient.ingredient_name).append(": ").append(myApp.getCurrencySymbol()).append(MyApp.df.format(orderItemIngredient.total));
                if (selectedProductAddons.length() > 0) {
                    selectedProductAddons.append(")");
                }
                total_amount = total_amount + orderItemIngredient.total;
            }
            selectedProductAddons.append(")");
        }
        if (!(Validators.isNullOrEmpty(orderItems.get(0).special_instruction)) && (orderItems.get(0).instruction_price > 0)) {
            if (selectedProductAddons.length() > 0) {
                selectedProductAddons.append(",(");
            } else {
                selectedProductAddons.append("(");
            }
            selectedProductAddons.append(orderItems.get(0).special_instruction).append(": ").append(myApp.getCurrencySymbol()).append(MyApp.df.format(orderItems.get(0).instruction_price));
            if (selectedProductAddons.length() > 0) {
                selectedProductAddons.append(")");
            }
            total_amount = total_amount + orderItems.get(0).instruction_price;
        }

        LogUtils.e("total_amount" + String.valueOf(total_amount));
        item.addon_name = selectedProductAddons.toString();

        addItemToDineInCart(orderItems, total_amount, item, position, orderItemAddons, orderItemIngredients);
    }

    private void addItemToDineInCart(ArrayList<OrderItem> orderItems, float subItemTotal, OrderItem menuItem, int position, ArrayList<OrderItemAddon> orderAddons, ArrayList<OrderItemIngredient> orderIngredients) {
        new Thread(() -> {

            if (Validators.isNullOrEmpty(menuItem.addon_name)) {
                String variantName = null;
                float discount = 0;
                variantName = menuItem.product_name;
                if (orderItems != null && orderItems.size() > 0) {

                    variantName = (menuItem.product_name.equalsIgnoreCase(variantName)) ? "" : variantName;
                    //   discount = (menuItem.orginal_price * menuItem.getProductPercentage() / 100);
                    //pending product percenrtage
                }
                DineinCartItem dbDineInCartItem = myApp.getAppDatabase().dineinCartItemDao().getCartMenuItem(Integer.parseInt(menuItem.order_id), variantName);
                if (dbDineInCartItem == null) {
                    DineinCartItem cartItem = new DineinCartItem();
                    if (menuItem != null) {
                        OrderItem selectedVariant = menuItem;
                        cartItem.quantity = orderItems.get(0).quantity;
                        cartItem.Menu_price = (selectedVariant.price + subItemTotal) - discount;
                        cartItem.menu_id = Integer.parseInt(selectedVariant.product_id);
                        cartItem.id = Integer.parseInt(selectedVariant.order_id);//ye kaha se lau abi
                        cartItem.menu_name = menuItem.product_name;
                        cartItem.menu_size = (menuItem.product_name.equalsIgnoreCase(selectedVariant.product_short_name)) ? "" : selectedVariant.product_short_name;
                        cartItem.res_id = restaurant.id;
                        //  cartItem.menu_type = menuItem.menu_type;

                        cartItem.Addon_name = "";
                        cartItem.Addon_price = 0;
                        cartItem.Total = cartItem.quantity * (cartItem.Menu_price + cartItem.Addon_price);
                        cartItem.order_item_addons.addAll(orderAddons);
                        cartItem.order_item_ingredients.addAll(orderIngredients);
                        myApp.getAppDatabase().dineinCartItemDao().insertAll(cartItem);
                    }
                } else {
                    if (menuItem != null) {
                        OrderItem selectedVariant = menuItem;
                        dbDineInCartItem.quantity = orderItems.get(0).quantity;
                        dbDineInCartItem.Menu_price = +((selectedVariant.price + subItemTotal) - discount);
                        dbDineInCartItem.Total = dbDineInCartItem.quantity * (dbDineInCartItem.Menu_price + dbDineInCartItem.Addon_price);
                        myApp.getAppDatabase().dineinCartItemDao().update(dbDineInCartItem);
                    }

                }
            } else {
                //selected ???
                String variantName = null;
                float discount = 0;
                variantName = menuItem.addon_name;
                DineinCartItem dbDineInCartItem = myApp.getAppDatabase().dineinCartItemDao().getCartAddonMenuItem(Integer.parseInt(menuItem.order_id), variantName);
                if (dbDineInCartItem == null) {
                    DineinCartItem cartItem = new DineinCartItem();
                    if (menuItem != null) {
                        OrderItem selectedVariant = menuItem;
                        cartItem.quantity = orderItems.get(0).quantity;
                        cartItem.Menu_price = (selectedVariant.price + subItemTotal) - discount;
                        cartItem.menu_id = Integer.parseInt(selectedVariant.product_id);
                        cartItem.id = Integer.parseInt(selectedVariant.order_id);
                        cartItem.menu_name = menuItem.product_name;
                        cartItem.menu_size = (menuItem.product_name.equalsIgnoreCase(selectedVariant.product_short_name)) ? "" : selectedVariant.product_short_name;
                        cartItem.res_id = restaurant.id;
                        //  cartItem.menu_type = menuItem.menu_type;

                        cartItem.Addon_name = menuItem.addon_name;
                        cartItem.Addon_price = 0;
                        cartItem.Total = cartItem.quantity * (cartItem.Menu_price + cartItem.Addon_price);
                        cartItem.order_item_addons.addAll(orderAddons);
                        cartItem.order_item_ingredients.addAll(orderIngredients);
                        myApp.getAppDatabase().dineinCartItemDao().insertAll(cartItem);


                    }
                } else {
                    if (menuItem != null) {
                        OrderItem selectedVariant = menuItem;
                        dbDineInCartItem.quantity = orderItems.get(0).quantity;
                        dbDineInCartItem.Menu_price = +((selectedVariant.price + subItemTotal) - discount);
                        dbDineInCartItem.Total = dbDineInCartItem.quantity * (dbDineInCartItem.Menu_price + dbDineInCartItem.Addon_price);
                        myApp.getAppDatabase().dineinCartItemDao().update(dbDineInCartItem);
                    }

                }
            }


            if (getActivity() != null) {
                getActivity().runOnUiThread(() -> {
                    menuAdapter.notifyItemChanged(position);
                    checkAndUpdateCheckoutLayout();
                    if (createdOrder != null) {
//                TIME_TO_WAIT = 5000;
//                handler.removeCallbacks(runnable);
//                handler.postDelayed(runnable, TIME_TO_WAIT);
                        count = 1;
                        handler.removeCallbacks(runnable);
                        if (countDownTimer != null) {
                            countDownTimer.cancel();
                        }

                        startTimer();
                    }
                });
            }
        }).start();
    }

    private DialogDismissListener addItemCartDialogListener = new DialogDismissListener() {
        @Override
        public void onDialogDismiss(Object o) {
            new Thread(() -> {
                int resId = myApp.getAppDatabase().cartDao().getRestaurantId();
                if (!isDineInOpen) {
                    if (resId == restaurant.id) {
                        myApp.getMyPreferences().saveOrderRestaurantDetail(restaurant);
                    } else {
                        myApp.getMyPreferences().deleteOrderRestaurant();
                    }
                }
            }).start();
            if (getActivity() != null) {
                getActivity().runOnUiThread(() -> {
                    menuAdapter.notifyDataSetChanged();
                    checkAndUpdateCheckoutLayout();
                });
            }
        }
    };


    private void fetchData() {
        new Thread(this::fetchRestaurantDetails).start();
    }

    String business_id;

    private void fetchRestaurantDetails() {
        if (getActivity() != null)
            getActivity().runOnUiThread(() -> llLoading.setVisibility(View.VISIBLE));
        ANRequest anRequest;
        if (currentPostcode == null) {
            if (!isDineInOpen || Validators.isNullOrEmpty(qr_code)) {
                isDineInOpen = false;
                anRequest = ApiUtils.getRestaurantDetails(MyApp.RESTAURANT_ID, "");
            } else {
                anRequest = ApiUtils.getDineInRestaurant(qr_code, "");
            }
        } else {
            if (!isDineInOpen || Validators.isNullOrEmpty(qr_code)) {
                isDineInOpen = false;
                anRequest = ApiUtils.getRestaurantDetails(MyApp.RESTAURANT_ID, currentPostcode.post_code);
            } else {
                anRequest = ApiUtils.getDineInRestaurant(qr_code, currentPostcode.post_code);
            }
        }
        anRequest
                .getAsJSONObject(new JSONObjectRequestListener() {
                    @Override
                    public void onResponse(JSONObject response) {
                        if (getActivity() != null) {
                            getActivity().runOnUiThread(() -> {
                                llLoading.setVisibility(View.GONE);
                            });
                        }

                        try {
                            restaurant = new Gson().fromJson(response.toString(), BusinessRestaurant.class);
                            //  LogUtils.e("restaurant response::" + new Gson().toJson(response));
                            business_id = restaurant.business.id;
                        } catch (NumberFormatException e) {
                            e.printStackTrace();
                        }
                        if (isDineInOpen) {
                            ivBackTop.setVisibility(View.VISIBLE);
                        } else {
                            ivBackTop.setVisibility(View.GONE);
                        }

                        fetchDineInategoryAndMenus();


                    }

                    @Override
                    public void onError(ANError anError) {
                        anError.printStackTrace();

                        if (getActivity() != null) {
                            getActivity().runOnUiThread(() -> {
                                llLoading.setVisibility(View.GONE);
                            });
                        }
                        if (!CommonFunctions.isConnected(getActivity())) {
                            startActivityForResult(new Intent(getActivity(), NoInternetActivity.class), CODE_NO_INTERNET);
                        }
                    }
                });
    }


    private void fetchDineInategoryAndMenus() {
        if (getActivity() != null) {
            getActivity().runOnUiThread(() -> progressDialog.show());
        }
        ApiUtils.getDineInEposCategories("1", "0", "1", restaurant.business.id)
                .getAsJSONArray(new JSONArrayRequestListener() {
                    @Override
                    public void onResponse(JSONArray response) {
                        if (getActivity() != null) {
                            getActivity().runOnUiThread(() -> progressDialog.dismiss());
                        }
                        Type type = new TypeToken<List<DineinCategories>>() {
                        }.getType();
                        dineinCategories.clear();
                        dineinCategories = new ArrayList<>(new Gson().fromJson(response.toString(), type));
                        //  LogUtils.e("Dine in category responce::" + new Gson().toJson(dineinCategories));

                        for (int i = 0; i < dineinCategories.size(); i++) {
                            if (dineinCategories.get(i).children != null && dineinCategories.get(i).children.size() > 0) {
                                DineinChildrenAdapter.selectedCategory = dineinCategories.get(i).children.get(0).id;
                            }
                        }
                        fetchAllDineinMenu();
                        if (RestaurantMenuCategoryAdapter.selectedDineinCategory == 0 || RestaurantMenuCategoryAdapter.selectedDineinCategory == dineinCategories.get(0).id) {
                            RestaurantMenuCategoryAdapter.selectedDineinCategory = dineinCategories.get(0).id;
                            fetchDineinMenuByCategory(String.valueOf(dineinCategories.get(0).id));
                            if (getActivity() != null) {
                                getActivity().runOnUiThread(() -> {
                                    categoryAdapter.notifyDataSetChanged();
                                });
                            }
                        }
                        if (fromHistory) {
                            handler.removeCallbacks(runnable);
                            TIME_TO_WAIT = 60000;
                            handler.postDelayed(runnable, TIME_TO_WAIT);
                        }
                        updateViews();
                        setUpAdapters();
                    }

                    @Override
                    public void onError(ANError anError) {
                        anError.printStackTrace();
                        if (getActivity() != null) {
                            getActivity().runOnUiThread(() -> progressDialog.dismiss());
                        }
                    }
                });
    }

    private void fetchAllDineinMenu() {
        ApiUtils.getDineInEposMenus("", "1", "0", "1", restaurant.business.id)
                .getAsJSONArray(new JSONArrayRequestListener() {
                    @Override
                    public void onResponse(JSONArray response) {
                        if (getActivity() != null) {
                            getActivity().runOnUiThread(() -> progressDialog.dismiss());
                        }
                        Type type = new TypeToken<List<DineinMenus>>() {
                        }.getType();
                        dineinAllMenus.clear();
                        dineinAllMenus = new ArrayList<>(new Gson().fromJson(response.toString(), type));
                    }

                    @Override
                    public void onError(ANError anError) {
                        anError.printStackTrace();
                        if (getActivity() != null) {
                            getActivity().runOnUiThread(() -> progressDialog.dismiss());
                        }
                    }
                });
    }

    private void fetchDineinMenuByCategory(String id) {
//        if (getActivity() != null) {
//            getActivity().runOnUiThread(() -> progressDialog.show());
//        }
        ApiUtils.getDineInEposMenus(id, "1", "0", "1", restaurant.business.id)
                .getAsJSONArray(new JSONArrayRequestListener() {
                    @Override
                    public void onResponse(JSONArray response) {
                        if (getActivity() != null) {
                            getActivity().runOnUiThread(() -> progressDialog.dismiss());
                        }
                        Type type = new TypeToken<List<DineinMenus>>() {
                        }.getType();
                        dineinMenus.clear();
                        dineinMenus = new ArrayList<>(new Gson().fromJson(response.toString(), type));

                        setUpAdapters();

                    }

                    @Override
                    public void onError(ANError anError) {
                        anError.printStackTrace();
                        if (getActivity() != null) {
                            getActivity().runOnUiThread(() -> progressDialog.dismiss());
                        }
                    }
                });
    }


    private void searchMenu() {
        filtered.clear();
        ArrayList<Menu> searchedSuggested = new ArrayList<>();
        for (Menu menu : suggested)
            if (menu.menu_name.toLowerCase().contains(etSearch.getText().toString().toLowerCase())) {
                searchedSuggested.add(menu);
            }

        if (searchedSuggested.size() > 0) {
            filtered.add(new Category("Recommended", 0));
            filtered.addAll(searchedSuggested);
        }
        try {
            if (dineinAllMenus != null && dineinAllMenus.size() > 0) {
                ArrayList<DineinMenus> menuItems = new ArrayList<>();
                for (DineinMenus menuItem : dineinAllMenus) {
                    if (menuItem.name.toLowerCase().contains(etSearch.getText().toString().toLowerCase())) {
                        menuItems.add(menuItem);
                    }
                }
                if (menuItems.size() > 0) {
                    filtered.addAll(menuItems);
                }
            }

            if (getActivity() != null) {
                getActivity().runOnUiThread(() -> {
                    menuAdapter.notifyDataSetChanged();
                });
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    private void setUpAdapters() {
        if (getActivity() != null)
            getActivity().runOnUiThread(() -> llLoading.setVisibility(View.GONE));
        feed.clear();
        categories.clear();
        suggested.clear();
        menucategoryArrayList.clear();

//        feed.add(new Header("Menu"));
        //   Allergy allergy = new Allergy(allergies);
        //  feed.add(allergy);

        //    feed.add(new AllergyCaution("Please be advised that certain items from this restaurant contain the following allergens.", 0));
        // feed.add(new AllergyCaution("Please contact the restaurant directly if you require further information about allergens", R.drawable.ic_outline_info_24));
//
//        if (isDineInOpen) {
//            for (Menu menu : dinein_restaurant_menu)
//                if (menu.featured.equalsIgnoreCase("yes")) {
//                    menu.category_id = 0;
//                    suggested.add(menu);
//                }
//
//        }
        if (suggested.size() > 0) {
            feed.add(new Category("Recommended", 0));
            feed.addAll(suggested);
            Category c = new Category();
            c.category_name = "Recommended";
            c.category_id = 0;
            categories.add(c);
        }
        try {
            if (dineinCategories != null && dineinCategories.size() > 0) {
                for (int i = 0; i < dineinCategories.size(); i++) {
                    Category c = new Category();
                    c.category_name = dineinCategories.get(i).title;
                    c.category_id = dineinCategories.get(i).id;
                    c.children = dineinCategories.get(i).children;
                    categories.add(c);
                    //  feed.add(c);
                    //  feed.addAll(dineinMenus);
                    menucategoryArrayList.add(c);

                }
            }

            if (dineinMenus != null && dineinMenus.size() > 0) {
                Category c = new Category();
                c.category_id = dineinMenus.get(0).category_id;
                c.category_name = dineinMenus.get(0).category_name;
                feed.add(c);
                feed.addAll(dineinMenus);
            }

            if (getActivity() != null) {
                getActivity().runOnUiThread(() -> {
                    categoryAdapter.notifyDataSetChanged();
                    menuAdapter.notifyDataSetChanged();
                });
            }

        } catch (Exception e) {
            e.printStackTrace();
        }

//        for (DineinSiteSettings dineinSiteSettings : dineinSiteSettings) {
//            showMenuImage = dineinSiteSettings.key.equalsIgnoreCase("product_image_visible") && dineinSiteSettings.value.equalsIgnoreCase("yes");
//        }

        productImageSettings = myApp.findSetting("product_image_visible");
        showMenuImage = productImageSettings != null && productImageSettings.value.equalsIgnoreCase("yes");
        try {
            filtered.clear();
            filtered.addAll(feed);
            if (getActivity() != null) {
                getActivity().runOnUiThread(() -> {
                    menuAdapter.notifyDataSetChanged();
                });
            }
        } catch (Exception e) {
            e.printStackTrace();
        }

    }

    public boolean checkCameraPermission() {
        if (ActivityCompat.checkSelfPermission(getActivity(),
                Manifest.permission.CAMERA)
                != PackageManager.PERMISSION_GRANTED) {

            // Should we show an explanation?
            if (ActivityCompat.shouldShowRequestPermissionRationale(getActivity(),
                    Manifest.permission.CAMERA)) {

                new android.app.AlertDialog.Builder(getActivity())
                        .setTitle("Location permission required")
                        .setMessage("To scan qr code we require your camera permission, Please allow to continue.")
                        .setPositiveButton("ALLOW", (dialogInterface, i) -> {
                            requestPermissions(new String[]{Manifest.permission.CAMERA}, PERMISSION_CODE_STORAGE);
                        })
                        .setNegativeButton("DENY", (dialogInterface, i) -> dialogInterface.dismiss())
                        .create()
                        .show();


            } else {
                requestPermissions(new String[]{Manifest.permission.CAMERA}, PERMISSION_CODE_STORAGE);
            }
            return false;
        } else {
            return true;
        }
    }

    private Menu findMenuItemById(int id) {
        if (isDineInOpen) {
            if (id != -1) {
                Menu toBeFineMenu = new Menu();
                toBeFineMenu.id = id;

//                if (dinein_restaurant_menu != null && dinein_restaurant_menu.size() > 0) {
//                    int menuIndex = dinein_restaurant_menu.indexOf(toBeFineMenu);
//                    try {
//                        if (menuIndex != -1) {
//                            return dinein_restaurant_menu.get(menuIndex);
//                        } else {
//                            return null;
//                        }
//                    } catch (Exception e) {
//                        e.printStackTrace();
//                    }
//                } else {
//                    return null;
//                }
            } else {
                return null;
            }
        }
        return null;
    }

    private void searchPostcode(String postcode) {
        if (getActivity() != null) getActivity().runOnUiThread(progressDialog::show);
        AndroidNetworking.get(ApiEndPoints.search_postcode + postcode)
                .build()
                .getAsObject(Postcode.class, new ParsedRequestListener<Postcode>() {
                    @Override
                    public void onResponse(Postcode response) {
                        if (getActivity() != null)
                            CommonFunctions.hideKeyboard(getActivity(), etPostcode);
                        if (getActivity() != null)
                            getActivity().runOnUiThread(() -> {
                                progressDialog.dismiss();
                                currentPostcode = response;
                                etPostcode.setText("");
                                myApp.getMyPreferences().saveCurrentPostcode(currentPostcode);
                                displayAddress();
                            });
                    }

                    @Override
                    public void onError(ANError anError) {
                        if (getActivity() != null)
                            getActivity().runOnUiThread(() -> progressDialog.dismiss());
                        try {
                            JSONObject object = new JSONObject(anError.getErrorBody());
                            String toast = object.getString("message");
                            ToastUtils.makeToast(getActivity(), toast);
                        } catch (JSONException e) {
                            e.printStackTrace();
                        }
                        if (!CommonFunctions.isConnected(getActivity())) {
                            myApp.noInternet(getActivity());
                        }
                    }
                });
    }

    private void displayAddress() {
        llData.setVisibility(View.VISIBLE);
        if (snippetArrayList.size() > 0) {
            llMiniSnippet.setVisibility(View.VISIBLE);
        }
        llLocation.setVisibility(View.GONE);

    }

    private boolean checkLocationServicePermission() {
        if (ActivityCompat.checkSelfPermission(getActivity(),
                Manifest.permission.ACCESS_FINE_LOCATION)
                != PackageManager.PERMISSION_GRANTED) {

            // Should we show an explanation?
            if (ActivityCompat.shouldShowRequestPermissionRationale(getActivity(),
                    Manifest.permission.ACCESS_FINE_LOCATION)) {

                new android.app.AlertDialog.Builder(getActivity())
                        .setTitle("Location permission required")
                        .setMessage("To access your current postcode we require your location permission, Please allow to continue.")
                        .setPositiveButton("ALLOW", (dialogInterface, i) -> {
                            requestPermissions(new String[]{Manifest.permission.ACCESS_FINE_LOCATION}, REQUEST_LOCATION);
                        })
                        .setNegativeButton("DENY", (dialogInterface, i) -> dialogInterface.dismiss())
                        .create()
                        .show();


            } else {
                requestPermissions(new String[]{Manifest.permission.ACCESS_FINE_LOCATION}, REQUEST_LOCATION);
            }
            return false;
        } else {
            return true;
        }
    }

    private void startLocationService() {
        MyLocation myLocation = new MyLocation();
        progressDialog.show();
        myLocation.getLocation(getActivity(), new MyLocation.LocationResult() {
            @Override
            public void gotLocation(Location location) {
                android.location.Address address = CommonFunctions.getAddressByLocation(getActivity(), location.getLatitude(), location.getLongitude());
                if (getActivity() != null) {
                    getActivity().runOnUiThread(() -> {
                        progressDialog.dismiss();
                        if (address != null) {
                            String zipcode = address.getPostalCode();
                            if (!Validators.isNullOrEmpty(zipcode)) {
                                etPostcode.setText(zipcode);
                                searchPostcode(zipcode);
                            }
                        }
                    });
                }

            }
        });

    }

    private void updateMiniView() {
        snippetArrayList.clear();
        if (loggedInUser != null)
            new Thread(() -> {
                CartSummary cartSummary;
                dineinCartItems.clear();
                if (isDineInOpen) {
                    cartSummary = myApp.getAppDatabase().dineinCartItemDao().getCartSummary();
                    if (createdOrder != null)
                        dineinCartItems.addAll(myApp.getAppDatabase().dineinCartItemDao().getAll());
                } else {
                    cartSummary = myApp.getAppDatabase().cartDao().getCartSummary();
                }
                LogUtils.e("Cart Summary::", new Gson().toJson(cartSummary));
                if (cartSummary != null && cartSummary.total > 0) {
                    RestaurantMiniSnippet miniSnippet = new RestaurantMiniSnippet();
                    miniSnippet.items = String.valueOf(cartSummary.items);
                    miniSnippet.total = (cartSummary.total);
                    snippetArrayList.add(miniSnippet);
                    if (getActivity() != null) {
                        getActivity().runOnUiThread(() -> {
                            snippetAdapter.notifyDataSetChanged();
                            llMiniSnippet.setVisibility(View.VISIBLE);
                            if (!Validators.isNullOrEmpty(miniSnippet.items)) {
                                MainActivity.tvCartItemsCountBadge.setVisibility(View.VISIBLE);
                                MainActivity.tvCartItemsCountBadge.setText(miniSnippet.items);
                            } else {
                                MainActivity.tvCartItemsCountBadge.setVisibility(View.GONE);
                            }
                        });
                    }
                }
                fetchLastOrderDetails();
            }).start();
        else llMiniSnippet.setVisibility(View.GONE);
    }

    private void fetchLastDineinOrderDetails() {
        AndroidNetworking.get(ApiEndPoints.dinein_orders)
                .addQueryParameter("per_page", "1")
                .addQueryParameter("page", "1")
                .addQueryParameter("restaurant_id", (restaurant != null) ? String.valueOf(restaurant.id) : "")
                .addQueryParameter("customer_id", String.valueOf(loggedInUser.id))
                .build()
                .getAsJSONObject(new JSONObjectRequestListener() {
                    @Override
                    public void onResponse(JSONObject response) {
                        OrderHistory lastOrder = null;
                        try {
                            lastOrder = new Gson().fromJson(response.getJSONArray("data").get(0).toString(), OrderHistory.class);
                            if (lastOrder != null) {
                                if (lastOrder.status != null && !lastOrder.status.equalsIgnoreCase("failed") && lastOrder.reviews == null) {
                                    snippetArrayList.add(lastOrder);
                                }
                            }
                        } catch (JSONException e) {
                            e.printStackTrace();
                        }

                        if (getActivity() != null) {
                            getActivity().runOnUiThread(() -> {
                                snippetAdapter.notifyDataSetChanged();
                                if (snippetArrayList.size() > 0)
                                    llMiniSnippet.setVisibility(View.VISIBLE);
                                else llMiniSnippet.setVisibility(View.GONE);
                            });
                        }
                    }

                    @Override
                    public void onError(ANError anError) {
                        anError.printStackTrace();
                    }
                });
    }

    private void fetchLastOrderDetails() {
        AndroidNetworking.get(ApiEndPoints.orders)
                .addQueryParameter("per_page", "1")
                .addQueryParameter("page", "1")
                .addQueryParameter("restaurant_id", (restaurant != null) ? String.valueOf(restaurant.id) : "")
                .addQueryParameter("customer_id", String.valueOf(loggedInUser.id))
                .build()
                .getAsJSONObject(new JSONObjectRequestListener() {
                    @Override
                    public void onResponse(JSONObject response) {
                        OrderHistory lastOrder = null;
                        try {
                            lastOrder = new Gson().fromJson(response.getJSONArray("data").get(0).toString(), OrderHistory.class);
                            if (lastOrder != null) {
                                if (lastOrder.status != null && !lastOrder.status.equalsIgnoreCase("failed") && lastOrder.reviews == null) {
                                    snippetArrayList.add(lastOrder);
                                }
                            }
                        } catch (JSONException e) {
                            e.printStackTrace();
                        }
                        fetchLastDineinOrderDetails();
                        if (getActivity() != null) {
                            getActivity().runOnUiThread(() -> {
                                snippetAdapter.notifyDataSetChanged();
                                if (snippetArrayList.size() > 0)
                                    llMiniSnippet.setVisibility(View.VISIBLE);
                                else llMiniSnippet.setVisibility(View.GONE);
                            });
                        }
                    }

                    @Override
                    public void onError(ANError anError) {
                        anError.printStackTrace();
                        fetchLastDineinOrderDetails();
                    }
                });
    }


    private void manageLastOrderClick(Object data) {
        if (data instanceof OrderHistory) {
            OrderHistory orderDetail = (OrderHistory) data;
            Intent intent = new Intent(getActivity(), TransportActivity.class);
            if (Validators.isNullOrEmpty(orderDetail.order_type)) {
                intent.putExtra("is_dinein", true);
                intent.putExtra("destination", "orderView");
            } else {
                intent.putExtra("destination", "trackOrder");
            }
            intent.putExtra("hideToolbar", true);
            intent.putExtra("order_id", String.valueOf(orderDetail.id));
            if (orderDetail.status.equalsIgnoreCase("delivered")) {
                // rate order
                intent.putExtra("shouldRate", true);
            }
            startActivityForResult(intent, CODE_REFRESH);
        } else if (data instanceof String) {
            if (((String) data).equalsIgnoreCase("removed")) {
                new Thread(this::updateMiniView).start();
            }
        } else {
            startActivityForResult(new Intent(getActivity(), TransportActivity.class).putExtra("destination", "dinein_checkout").putExtra("is_dinein", true).putExtra("table_id", table_Id).putExtra("name", "Dine in Checkout").putExtra("table_number", table_number).putExtra("guests", guest_count).putExtra("qr_code", qr_code).putExtra("resId", restaurant.id).putExtra("business_id", business_id).putExtra("show_back", true).putExtra("created_order", new Gson().toJson(createdOrder)), CODE_REFRESH);
        }
    }

    Handler handler = new Handler();
    private static int TIME_TO_WAIT = 30000;

    Runnable runnable = new Runnable() {
        @Override
        public void run() {
            handler.postDelayed(runnable, TIME_TO_WAIT);
            fetchCreatedOrder();
        }
    };

    @Override
    public void onActivityResult(int requestCode, int resultCode, @Nullable Intent data) {
        super.onActivityResult(requestCode, resultCode, data);
        if (requestCode == CODE_REFRESH) {

            try {
                if (data != null) {
                    createdOrder = new Gson().fromJson(data.getStringExtra("created_order"), CreatedOrder.class);
                    if (createdOrder != null) {
                        handler.removeCallbacks(runnable);
                        TIME_TO_WAIT = 30000;
                        handler.postDelayed(runnable, TIME_TO_WAIT);
                    }
                }
            } catch (JsonSyntaxException e) {
                e.printStackTrace();
            }
//            checkAndUpdateCheckoutLayout();
        }
        if (requestCode == CODE_NO_INTERNET) {
            fetchData();
        }
    }

    int count = 1;
    CountDownTimer countDownTimer;

    private void startTimer() {

        new Handler(Looper.getMainLooper()).post(() -> countDownTimer = new CountDownTimer(20000, 1000) {
            @Override
            public void onTick(long millisUntilFinished) {
                // LogUtils.e("COUNT TIMER::" + count);
                count++;
            }

            @Override
            public void onFinish() {
                count = 1;
                handler.removeCallbacks(runnable);
                updateOrderItem();
            }
        }.start());

    }

    private Object genrateOrderCreateParams() {
        if (dineinCartItems != null && dineinCartItems.size() > 0) {
            HashMap<String, String> hashMap = new HashMap<>();

            ArrayList<CreateOrderParamsItem> orderItemArray = new ArrayList<>();
            for (DineinCartItem oit : dineinCartItems) {
                CreateOrderParamsItem orderItem = new CreateOrderParamsItem();
                orderItem.addons_price = oit.Addon_price;
                orderItem.instruction_price = oit.instruction_price;
                orderItem.ingredients_price = oit.ingredients_price;
                orderItem.price = oit.Menu_price;
                orderItem.product_id = String.valueOf(oit.menu_id);
                orderItem.product_name = oit.menu_name;
                orderItem.preparation_location_id = "";
                orderItem.quantity = oit.quantity;
                orderItem.sent_to_kitchen = oit.sent_to_kitchen;
                orderItem.updater_id = String.valueOf(myApp.getMyPreferences().getDineinUser().id);
                orderItem.order_spilt_id = "";
                orderItem.total = oit.Total;
                orderItem.special_instruction = "";
                orderItem.sub_total = oit.Total;
                orderItem.order_id = String.valueOf(createdOrder.id);
                for (OrderItem order : createdOrder.order_items) {
                    if (Integer.parseInt(order.product_id) == (oit.menu_id)) {
                        orderItem.id = order.id;//created order_item id
                    }
                }

                for (OrderItemAddon oia : oit.order_item_addons) {
                    CreateOrderAddonParmsItem addon = new CreateOrderAddonParmsItem();
                    addon.addon_id = oia.addon_id;
                    addon.updater_id = oia.updater_id;
                    addon.price = oia.price;
                    addon.quantity = oia.quantity;
                    addon.id = oia.id;
                    addon.total = oia.total;
                    orderItem.order_item_addons.add(addon);
                }
                for (OrderItemIngredient oi : oit.order_item_ingredients) {
                    CreateOrderIngredientParamsItem ingredient = new CreateOrderIngredientParamsItem();
                    ingredient.id = oi.id;
                    ingredient.product_ingredient_id = oi.product_ingredient_id;
                    ingredient.updater_id = oi.updater_id;
                    ingredient.quantity = oi.quantity;
                    ingredient.with = oi.with;
                    ingredient.without = oi.without;
                    ingredient.price = oi.price;
                    ingredient.total = oi.total;
                    orderItem.order_item_ingredients.add(ingredient);
                }
                orderItemArray.add(orderItem);
            }
            hashMap.put("order_items", new Gson().toJson(orderItemArray));

            CustomerParams customer = new CustomerParams();
            if (restaurant.epos_customer != null) {
                customer.id = restaurant.epos_customer.id;
                customer.mobile = restaurant.epos_customer.mobile;
                customer.name = restaurant.epos_customer.name;
                customer.email = restaurant.epos_customer.email;
                customer.street = !Validators.isNullOrEmpty(restaurant.epos_customer.street) ? restaurant.epos_customer.street : "";
                customer.postcode = !Validators.isNullOrEmpty(restaurant.epos_customer.postcode) ? restaurant.epos_customer.postcode : "";
                customer.house_no = !Validators.isNullOrEmpty(restaurant.epos_customer.house_no) ? restaurant.epos_customer.house_no : "";
                customer.city = !Validators.isNullOrEmpty(restaurant.epos_customer.city) ? restaurant.epos_customer.city : "";
                customer.distance = !Validators.isNullOrEmpty(restaurant.epos_customer.distance) ? restaurant.epos_customer.distance : "";
                customer.news_letter = !Validators.isNullOrEmpty(restaurant.epos_customer.news_letter) ? restaurant.epos_customer.news_letter : "";
            }

            hashMap.put("customer", new Gson().toJson(customer));

        }
        return null;
    }

    private void updateOrderItem() {
        AndroidNetworking.post(ApiEndPoints.dinein_create_order + createdOrder.id)
                .addBodyParameter(genrateOrderCreateParams())
                .addHeaders("business-id", myApp.getMyPreferences().getCurrentDineInRestaurantDetail().business.id)
                .build()
                .getAsJSONObject(new JSONObjectRequestListener() {
                    @Override
                    public void onResponse(JSONObject response) {
                        LogUtils.e("updated responce::" + new Gson().toJson(response));
                        fetchCreatedOrder();
                        TIME_TO_WAIT = 60000;
                        handler.postDelayed(runnable, TIME_TO_WAIT);
                    }

                    @Override
                    public void onError(ANError anError) {
                        LogUtils.e("update error" + anError.getErrorBody());
                    }
                });
    }

    private void fetchCreatedOrder() {
        String url = null;
        if (createdOrder == null) {
            url = ApiEndPoints.dinein_create_order;
        } else
            url = ApiEndPoints.dinein_create_order + createdOrder.id;
        AndroidNetworking.get(url)
                .addHeaders("business-id", restaurant.business.id)
                .build()
                .getAsJSONObject(new JSONObjectRequestListener() {
                    @Override
                    public void onResponse(JSONObject response) {
                        LogUtils.e("FETCH ORDER CREATED CALLED");
                        createdOrder = new Gson().fromJson(response.toString(), CreatedOrder.class);
                        //when to fetch api how to know
                        addOrupdateItemToDineInCart(createdOrder);
                    }

                    @Override
                    public void onError(ANError anError) {
                        LogUtils.e("order error::" + anError.getErrorBody());
                        if (!CommonFunctions.isConnected(getActivity())) {
                            startActivityForResult(new Intent(getActivity(), NoInternetActivity.class), CODE_NO_INTERNET);
                        }
                    }
                });

    }

    private void addOrupdateItemToDineInCart(CreatedOrder createdOrder) {
        ArrayList<DineinCartItem> cartItems = new ArrayList<>();
        ArrayList<OrderItemAddon> orderAddon = new ArrayList<>();
        ArrayList<OrderItemIngredient> orderIngredinet = new ArrayList<>();
        new Thread(() -> {
           // myApp.getAppDatabase().dineinCartItemDao().nukeTable();
            for (OrderItem historyCart : createdOrder.order_items) {
                DineinCartItem cartItem = new DineinCartItem();
                cartItem.quantity = historyCart.quantity;
                cartItem.Menu_price = historyCart.price;
                cartItem.menu_id = Integer.parseInt(historyCart.product_id);
                cartItem.id = Integer.parseInt(historyCart.id);
                cartItem.menu_name = historyCart.product_name;
                cartItem.menu_size = (historyCart.product_name.equalsIgnoreCase(historyCart.product_short_name)) ? "" : historyCart.product_short_name;
                cartItem.res_id = myApp.getMyPreferences().getCurrentRestaurantDetail().id;

                StringBuilder selectedProductAddons = new StringBuilder();
                float total_amount = 0;
                if ((historyCart.order_item_addons != null) && (historyCart.order_item_addons.size() > 0)) {
                    orderAddon.addAll(historyCart.order_item_addons);
                }
                if ((historyCart.order_item_ingredients != null) && (historyCart.order_item_ingredients.size() > 0)) {
                    orderIngredinet.addAll(historyCart.order_item_ingredients);
                }

                if (orderAddon.size() > 0) {
                    selectedProductAddons.append("(");
                    for (OrderItemAddon orderItemAddon : orderAddon) {
                        if (selectedProductAddons.length() > 0) {
                            selectedProductAddons.append("(");
                        }
                        selectedProductAddons.append(orderItemAddon.addon_name).append(": ").append(myApp.getCurrencySymbol()).append(MyApp.df.format(orderItemAddon.price));
                        if (selectedProductAddons.length() > 0) {
                            selectedProductAddons.append(")");
                        }
                        total_amount = total_amount + orderItemAddon.total;
                    }
                    selectedProductAddons.append(")");
                }

                if (orderIngredinet.size() > 0) {
                    if (orderAddon.size() > 0) {
                        selectedProductAddons.append(",");
                    }
                    selectedProductAddons.append("(");
                    for (OrderItemIngredient orderItemIngredient : orderIngredinet) {
                        if (selectedProductAddons.length() > 0) {
                            selectedProductAddons.append("(");
                        }
                        if (orderItemIngredient.with) {
                            selectedProductAddons.append("Extra ");
                            selectedProductAddons.append(orderItemIngredient.quantity).append(" ");
                        }
                        if (orderItemIngredient.without) {
                            selectedProductAddons.append("No ");
                        }
                        selectedProductAddons.append(orderItemIngredient.ingredient_name).append(": ").append(myApp.getCurrencySymbol()).append(MyApp.df.format(orderItemIngredient.total));
                        if (selectedProductAddons.length() > 0) {
                            selectedProductAddons.append(")");
                        }
                        total_amount = total_amount + orderItemIngredient.total;
                    }
                    selectedProductAddons.append(")");
                }
                if (!(Validators.isNullOrEmpty(historyCart.special_instruction)) && (historyCart.instruction_price > 0)) {
                    if (selectedProductAddons.length() > 0) {
                        selectedProductAddons.append(",(");
                    } else {
                        selectedProductAddons.append("(");
                    }
                    selectedProductAddons.append(historyCart.special_instruction).append(": ").append(myApp.getCurrencySymbol()).append(MyApp.df.format(historyCart.instruction_price));
                    if (selectedProductAddons.length() > 0) {
                        selectedProductAddons.append(")");
                    }
                    total_amount = total_amount + historyCart.instruction_price;
                }

                cartItem.Addon_name = selectedProductAddons.toString();
                cartItem.Addon_price = total_amount;
                cartItem.Total = cartItem.quantity * (cartItem.Menu_price + cartItem.Addon_price);
                cartItem.order_item_addons.addAll(historyCart.order_item_addons);
                cartItem.order_item_ingredients.addAll(historyCart.order_item_ingredients);
                cartItems.add(cartItem);
                DineinCartItem dbDineInCartItemExist = myApp.getAppDatabase().dineinCartItemDao().getDineinCartItemFromOrderItemId(Integer.parseInt(historyCart.id));
                if (dbDineInCartItemExist != null) {
                    myApp.getAppDatabase().dineinCartItemDao().updateAndReplace(cartItem);
                } else {
                    myApp.getAppDatabase().dineinCartItemDao().insertAll(cartItem);
                }


              /*  if (dbDineInCartItem == null || (dbDineInCartItem.quantity != historyCart.quantity && (!dbDineInCartItem.order_item_addons.equals(historyCart.order_item_addons)) && (!dbDineInCartItem.order_item_ingredients.equals(historyCart.order_item_ingredients))))
                    myApp.getAppDatabase().dineinCartItemDao().insertAll(cartItem);
                else {
                    dbDineInCartItem.quantity = historyCart.quantity;
                    dbDineInCartItem.Total = dbDineInCartItem.quantity * (dbDineInCartItem.Menu_price + dbDineInCartItem.Addon_price);
                    dbDineInCartItem.order_item_addons.clear();
                    dbDineInCartItem.order_item_ingredients.clear();
                    dbDineInCartItem.order_item_addons.addAll(historyCart.order_item_addons);
                    dbDineInCartItem.order_item_ingredients.addAll(historyCart.order_item_ingredients);
                    myApp.getAppDatabase().dineinCartItemDao().update(dbDineInCartItem);
                }
              */

                if (getActivity() != null) {
                    getActivity().runOnUiThread(() -> {
                        menuAdapter.notifyDataSetChanged();
                        checkAndUpdateCheckoutLayout();
                    });
                }
            }

            LogUtils.e("ALL DATA:::::::::::::::" + new Gson().toJson(myApp.getAppDatabase().dineinCartItemDao().getAll()));
        }).start();
    }

    @Override
    public void onRequestPermissionsResult(int requestCode, String[] permissions,
                                           int[] grantResults) {
        if (requestCode == PERMISSION_CODE_STORAGE) {
            if (grantResults.length > 0 && grantResults[0] == PackageManager.PERMISSION_GRANTED) {
                if (ActivityCompat.checkSelfPermission(getActivity(),
                        Manifest.permission.CAMERA)
                        == PackageManager.PERMISSION_GRANTED) {
                    openDineinScan();
                } else {
                    ToastUtils.makeSnackToast(getActivity(), "Permission denied.");
                }

            } else {
                if (!ActivityCompat.shouldShowRequestPermissionRationale(getActivity(),
                        Manifest.permission.CAMERA)) {
                    new android.app.AlertDialog.Builder(getActivity())
                            .setTitle("Camera permission required")
                            .setMessage("To scan we require camera permission, Please allow to continue.")
                            .setPositiveButton("ALLOW", (dialogInterface, i) -> {
                                Intent intent = new Intent();
                                intent.setAction(Settings.ACTION_APPLICATION_DETAILS_SETTINGS);
                                Uri uri = Uri.fromParts("package", getActivity().getPackageName(), null);
                                intent.setData(uri);
                                startActivity(intent);
                            })
                            .setNegativeButton("DENY", (dialogInterface, i) -> dialogInterface.dismiss())
                            .create()
                            .show();
                } else {
                    ToastUtils.makeSnackToast(getActivity(), "Permission denied.");
                }

            }
        }
        if (requestCode == REQUEST_LOCATION) {
            if (grantResults.length > 0 && grantResults[0] == PackageManager.PERMISSION_GRANTED) {
                if (ActivityCompat.checkSelfPermission(getActivity(),
                        Manifest.permission.ACCESS_FINE_LOCATION)
                        == PackageManager.PERMISSION_GRANTED) {
                    startLocationService();
                } else {
                    ToastUtils.makeSnackToast(getActivity(), "Permission denied.");
                }

            } else {
                if (!ActivityCompat.shouldShowRequestPermissionRationale(getActivity(),
                        Manifest.permission.ACCESS_FINE_LOCATION)) {
                    new android.app.AlertDialog.Builder(getActivity())
                            .setTitle("Location permission required")
                            .setMessage("To access your current postcode we require your location permission, Please allow to continue.")
                            .setPositiveButton("ALLOW", (dialogInterface, i) -> {
                                Intent intent = new Intent();
                                intent.setAction(Settings.ACTION_APPLICATION_DETAILS_SETTINGS);
                                Uri uri = Uri.fromParts("package", getActivity().getPackageName(), null);
                                intent.setData(uri);
                                startActivity(intent);
                            })
                            .setNegativeButton("DENY", (dialogInterface, i) -> dialogInterface.dismiss())
                            .create()
                            .show();
                } else {
                    ToastUtils.makeSnackToast(getActivity(), "Permission denied.");
                }

            }
        }

    }

    private void checkAndUpdateCheckoutLayout() {
        CommonFunctions.notifyCartCount(getContext());
        /*if (loggedInUser == null)
            updateCheckoutLayout();*/
        updateCheckoutLayout();
        // updateMiniView();
    }

    private void updateCheckoutLayout() {
        new Thread(() -> {
            if (isDineInOpen) {
                cartSummary = myApp.getAppDatabase().dineinCartItemDao().getCartSummary();
            } else {
                cartSummary = myApp.getAppDatabase().cartDao().getCartSummary();
            }
            if (getActivity() != null) {
                getActivity().runOnUiThread(() -> {
                    if (cartSummary != null && cartSummary.items > 0) {
                        llCheckout.setVisibility(View.VISIBLE);
                        tvTotal.setText("Total: " + myApp.getCurrencySymbol() + MyApp.df.format(cartSummary.total));
                    } else {
                        llCheckout.setVisibility(View.GONE);
                    }
                });

            }
        }).start();

    }

    @Override
    public void onResume() {
        if (restaurant != null)
            restaurant = restaurant;
        if (getArguments() != null) {
            isDineInOpen = getArguments().getBoolean("is_dinein");
        } else {
            isDineInOpen = false;
            qr_code = null;
        }
        if (getActivity() != null) {
            getActivity().runOnUiThread(() -> {
                if (menuAdapter != null)
                    menuAdapter.notifyDataSetChanged();
            });
        }
        CommonFunctions.hideKeyboard(getActivity(), rootView);

        if (getActivity() != null && broadcastReceiver != null)
            LocalBroadcastManager.getInstance(getActivity()).registerReceiver(broadcastReceiver, new IntentFilter(Config.PUSH_NOTIFICATION));

        super.onResume();

        checkAndUpdateCheckoutLayout();
    }


    @Override
    public void onStop() {
        if (getActivity() != null && broadcastReceiver != null)
            LocalBroadcastManager.getInstance(getActivity()).unregisterReceiver(broadcastReceiver);
        super.onStop();

        if (handler!=null&&runnable!=null) {
            handler.removeCallbacks(runnable);
        }

    }
}
