package com.tiffintom.fragment;

import android.graphics.ColorFilter;
import android.graphics.PorterDuff;
import android.graphics.PorterDuffColorFilter;
import android.os.Bundle;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.ImageView;
import android.widget.LinearLayout;
import android.widget.ProgressBar;
import android.widget.TextView;

import androidx.annotation.NonNull;
import androidx.annotation.Nullable;
import androidx.core.content.ContextCompat;
import androidx.fragment.app.FragmentPagerAdapter;
import androidx.viewpager.widget.ViewPager;

import com.airbnb.lottie.LottieAnimationView;
import com.airbnb.lottie.LottieProperty;
import com.airbnb.lottie.model.KeyPath;
import com.airbnb.lottie.value.LottieFrameInfo;
import com.airbnb.lottie.value.SimpleLottieValueCallback;
import com.google.android.material.tabs.TabLayout;
import com.tiffintom.R;
import com.tiffintom.adapters.ViewpagerAdapter;
import com.tiffintom.base.BaseFragment;

/**
 * Created by Aradhna on 17-02-2022.
 */
public class DineinHistoryTabsFragment extends BaseFragment {
    public static DineinHistoryTabsFragment getInstance() {
        DineinHistoryTabsFragment fragment = new DineinHistoryTabsFragment();
        Bundle bundle = new Bundle();
        fragment.setArguments(bundle);
        return fragment;
    }

    private ViewPager viewPager;
    private ViewpagerAdapter viewpagerAdapter;
    private TabLayout tabLayout;
    private ImageView ivBackArrow;
    private TextView tvScreenTitle;
    private LinearLayout llBackArrow;


    @Nullable
    @Override
    public View onCreateView(@NonNull LayoutInflater inflater, @Nullable ViewGroup container, @Nullable Bundle savedInstanceState) {
        return inflater.inflate(R.layout.fragment_tabs, container, false);
    }

    @Override
    public void onViewCreated(@NonNull View view, @Nullable Bundle savedInstanceState) {
        super.onViewCreated(view, savedInstanceState);
        setUpViewPager();
        llBackArrow.setOnClickListener(view1 -> {
            if (getActivity() != null) getActivity().onBackPressed();
        });
    }

    @Override
    protected void initViews(View view) {
        super.initViews(view);
        ProgressBar pbLoading = view.findViewById(R.id.pbLoading);
        LottieAnimationView lodingView = view.findViewById(R.id.lodingView);
        lodingView.addValueCallback(
                new KeyPath("**"),
                LottieProperty.COLOR_FILTER,
                new SimpleLottieValueCallback<ColorFilter>() {
                    @Override
                    public ColorFilter getValue(LottieFrameInfo<ColorFilter> frameInfo) {
                        return new PorterDuffColorFilter(ContextCompat.getColor(getActivity(), R.color.colorPrimary), PorterDuff.Mode.SRC_ATOP);
                    }
                }
        );
        tvUserName = view.findViewById(R.id.tvUserName);
        ivBackArrow = view.findViewById(R.id.ivBackArrow);
        llBackArrow = view.findViewById(R.id.llBackArrow);
        tvScreenTitle = view.findViewById(R.id.tvScreenTitle);

        viewPager = view.findViewById(R.id.viewPager);
        tabLayout = view.findViewById(R.id.tabLayout);


        tvScreenTitle.setVisibility(View.VISIBLE);
        tvScreenTitle.setText("Dine in History");
        pbLoading.setVisibility(View.GONE);
        lodingView.setVisibility(View.GONE);
        viewPager.setVisibility(View.VISIBLE);
        tabLayout.setVisibility(View.VISIBLE);
    }

    private void setUpViewPager() {

        viewpagerAdapter = new ViewpagerAdapter(getChildFragmentManager(), FragmentPagerAdapter.BEHAVIOR_RESUME_ONLY_CURRENT_FRAGMENT);
        viewpagerAdapter.addFragment(DineinOrderHistoryFragment.getInstance("active"), "Active");
        viewpagerAdapter.addFragment(DineinOrderHistoryFragment.getInstance("completed"), "Completed");
        viewPager.setAdapter(viewpagerAdapter);
        tabLayout.setupWithViewPager(viewPager);
        setTabIcons();
    }

    public String[] tabTitles = new String[]{"Active", "Completed"};
    public int[] tabIcons = new int[]{R.drawable.tab_icon_upcoming, R.drawable.tab_icon_previous};

    private void setTabIcons() {

        for (int i = 0; i < tabLayout.getTabCount(); i++) {
            TabLayout.Tab tab = tabLayout.getTabAt(i);
            View v = LayoutInflater.from(getActivity()).inflate(R.layout.item_tab_with_icon, null);
            TextView tv = v.findViewById(R.id.tvTabTitle);
            ImageView icon = v.findViewById(R.id.ivTabLogo);
            tv.setText(tabTitles[i]);
            icon.setImageResource(tabIcons[i]);
            tab.setCustomView(v);
            LinearLayout layout = ((LinearLayout) ((LinearLayout) tabLayout.getChildAt(0)).getChildAt(i));
            LinearLayout.LayoutParams layoutParams = (LinearLayout.LayoutParams) layout.getLayoutParams();
//            layoutParams.weight = 1f; // e.g. 0.5f
            layoutParams.width = LinearLayout.LayoutParams.WRAP_CONTENT;
            layout.setLayoutParams(layoutParams);
        }
    }
}
