package com.tiffintom.fragment;

import static com.tiffintom.utils.Constants.CODE_REFRESH;

import android.animation.Animator;
import android.content.Context;
import android.content.Intent;
import android.graphics.Color;
import android.graphics.drawable.ColorDrawable;
import android.os.Bundle;
import android.os.Handler;
import android.os.Looper;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.EditText;
import android.widget.LinearLayout;
import android.widget.TextView;

import androidx.annotation.NonNull;
import androidx.annotation.Nullable;
import androidx.appcompat.app.AlertDialog;
import androidx.recyclerview.widget.LinearLayoutManager;
import androidx.recyclerview.widget.RecyclerView;
import androidx.swiperefreshlayout.widget.SwipeRefreshLayout;

import com.airbnb.lottie.LottieAnimationView;
import com.androidnetworking.AndroidNetworking;
import com.androidnetworking.error.ANError;
import com.androidnetworking.interfaces.JSONArrayRequestListener;
import com.androidnetworking.interfaces.JSONObjectRequestListener;
import com.google.gson.Gson;
import com.google.gson.JsonSyntaxException;
import com.google.gson.reflect.TypeToken;
import com.iarcuschin.simpleratingbar.SimpleRatingBar;
import com.tiffintom.MyApp;
import com.tiffintom.R;
import com.tiffintom.activity.MainActivity;
import com.tiffintom.activity.TransportActivity;
import com.tiffintom.adapters.DineinOrderHistoryAdapter;
import com.tiffintom.base.BaseFragment;
import com.tiffintom.common.CommonFunctions;
import com.tiffintom.models.DineInOrderHistory;
import com.tiffintom.models.DineinCartItem;
import com.tiffintom.models.DineinModels.CreatedOrder;
import com.tiffintom.models.DineinModels.DineinPaymentMethods;
import com.tiffintom.models.DineinModels.DineinSiteSettings;
import com.tiffintom.models.DineinModels.EposCustomer;
import com.tiffintom.models.DineinModels.OrderItem;
import com.tiffintom.models.DineinModels.TablesStatuses;
import com.tiffintom.models.DineinModels.UsersList;
import com.tiffintom.models.User;
import com.tiffintom.network.ApiEndPoints;
import com.tiffintom.utils.LogUtils;
import com.tiffintom.utils.ToastUtils;

import org.json.JSONArray;
import org.json.JSONException;
import org.json.JSONObject;

import java.lang.reflect.Type;
import java.util.ArrayList;
import java.util.List;

/**
 * Created by Amrish on 07-09-2020.
 */
public class DineinOrderHistoryFragment extends BaseFragment {
    public static DineinOrderHistoryFragment getInstance() {
        return new DineinOrderHistoryFragment();
    }

    public static DineinOrderHistoryFragment getInstance(String status) {
        DineinOrderHistoryFragment fragment = new DineinOrderHistoryFragment();
        Bundle bundle = new Bundle();
        bundle.putString("status", status);
        fragment.setArguments(bundle);
        return fragment;
    }

    private RecyclerView rvOrders;
    private DineinOrderHistoryAdapter orderHistoryAdapter;
    private ArrayList<Object> feed = new ArrayList<>();
    private ArrayList<CreatedOrder> orders = new ArrayList<>();
    private EposCustomer eposCustomer;
    private SwipeRefreshLayout swipeRefreshLayout;
    private User loggedInUser = myApp.getMyPreferences().getLoggedInUser();
    private String status;
    private LinearLayout llNoData;
    private TextView tvMessage;
    private String business_id;

    ArrayList<DineinSiteSettings> siteSettings = new ArrayList<>();
    ArrayList<DineinPaymentMethods> paymentMethods = new ArrayList<>();
    ArrayList<TablesStatuses> tablesStatuses = new ArrayList<>();
    ArrayList<UsersList> userMinis = new ArrayList<>();

    @Nullable
    @Override
    public View onCreateView(@NonNull LayoutInflater inflater, @Nullable ViewGroup container, @Nullable Bundle savedInstanceState) {
        if (getArguments() != null) {
            status = getArguments().getString("status");
        }
        return inflater.inflate(R.layout.fragment_order_history, container, false);
    }

    @Override
    public void onViewCreated(@NonNull View view, @Nullable Bundle savedInstanceState) {
        super.onViewCreated(view, savedInstanceState);
        fetchData();
        fetchSiteSettings();
        setListener();
    }

    @Override
    protected void initViews(View view) {
        super.initViews(view);
        if (myApp.getMyPreferences().getCurrentRestaurantDetail() != null)
            business_id = myApp.getMyPreferences().getCurrentRestaurantDetail().business.id;
        llNoData = view.findViewById(R.id.llNoData);
        tvMessage = view.findViewById(R.id.tvMessage);
        swipeRefreshLayout = view.findViewById(R.id.swipeRefreshLayout);
        rvOrders = view.findViewById(R.id.rvOrders);

        orderHistoryAdapter = new DineinOrderHistoryAdapter(getActivity(), feed, this::viewReceipt, this::reOrder, this::openRatingDialog);
        rvOrders.setLayoutManager(new LinearLayoutManager(getActivity(), RecyclerView.VERTICAL, false));
        rvOrders.setAdapter(orderHistoryAdapter);
    }

    private void setListener() {
        swipeRefreshLayout.setOnRefreshListener(this::fetchData);
    }

    private void reOrder(int position, Object data) {
        if (data instanceof CreatedOrder) {
            CreatedOrder history = (CreatedOrder) data;
            new Thread(() -> {
                int dbRestId = myApp.getAppDatabase().cartDao().getRestaurantId();
                if (dbRestId > 0) {
                    if (getActivity() != null) {
                        getActivity().runOnUiThread(() -> {
                            showOtherRestaurantCartWarning(history);
                        });
                    }
                } else {
                    myApp.getAppDatabase().dineinCartItemDao().nukeTable();
                    addItemsInCart(history);
                }
            }).start();
        }

    }

    private void showOtherRestaurantCartWarning(CreatedOrder orderHistory) {
        AlertDialog alertDialog = new AlertDialog.Builder(getActivity()).create();
        alertDialog.getWindow().setBackgroundDrawable(new ColorDrawable(Color.TRANSPARENT));
        LayoutInflater inflater = (LayoutInflater) getActivity().getSystemService(Context.LAYOUT_INFLATER_SERVICE);
        View dialogView = inflater.inflate(R.layout.dialog_remove_card, null);

        TextView tvTitle = dialogView.findViewById(R.id.tvTitle);
        TextView tvMessage = dialogView.findViewById(R.id.tvMessage);
        TextView tvCancel = dialogView.findViewById(R.id.tvCancel);
        TextView tvConfirm = dialogView.findViewById(R.id.tvConfirm);
        tvTitle.setVisibility(View.VISIBLE);
        tvTitle.setText("Start new basket?");
        tvMessage.setText("Your basket already contains some items. Do you wish to clear the basket and add this order instead?");
        tvConfirm.setOnClickListener(view -> {
            new Thread(() -> {
                myApp.getAppDatabase().dineinCartItemDao().nukeTable();

                addItemsInCart(orderHistory);
            }).start();
            MainActivity.tvCartItemsCountBadge.setText("0");
            MainActivity.tvCartItemsCountBadge.setVisibility(View.GONE);
            alertDialog.dismiss();
        });
        tvCancel.setOnClickListener(view -> {
            alertDialog.dismiss();
        });
        alertDialog.setView(dialogView);
        alertDialog.setCancelable(false);
        alertDialog.show();
    }

    private void addItemsInCart(CreatedOrder orderHistory) {
        ArrayList<DineinCartItem> cartItems = new ArrayList<>();
        for (OrderItem historyCart : orderHistory.order_items) {
            DineinCartItem cartItem = new DineinCartItem();
            cartItem.quantity = historyCart.quantity;
            cartItem.Menu_price = historyCart.price;
            cartItem.menu_id = Integer.parseInt(historyCart.product_id);
            cartItem.menu_name = historyCart.product_name;
            cartItem.res_id = myApp.getMyPreferences().getCurrentRestaurantDetail().id;

            cartItem.Addon_name = orderHistory.subAddonName;
            cartItem.Addon_price = orderHistory.addon_total;
            cartItem.Total = cartItem.quantity * (cartItem.Menu_price + cartItem.Addon_price);

            cartItems.add(cartItem);
            new Thread(() -> {
                myApp.getAppDatabase().dineinCartItemDao().insertAll(cartItem);
            }).start();

        }

        if (cartItems.size() > 0) {
                Intent intent = new Intent(getActivity(), TransportActivity.class).putExtra("hideToolbar", true).putExtra("destination", "dinein_menu").putExtra("is_dinein", true).putExtra("nukeTable", true).putExtra("table_id", orderHistory.table_id).putExtra("guests", orderHistory.no_guest).putExtra("table_number", orderHistory.table_number).putExtra("qr_code", myApp.getMyPreferences().getQrCode()).putExtra("created_order", new Gson().toJson(orderHistory));
                startActivityForResult(intent, CODE_REFRESH);
        }
    }

    private void viewReceipt(int position, Object data) {

    }

    private void fetchData() {
        new Thread(this::fetchOrders).start();
    }

    private void fetchSiteSettings() {
        AndroidNetworking.get(ApiEndPoints.dinein_site_settings)
                .addQueryParameter("nopaginate", "1")
                .addHeaders("business-id", business_id)
                .build()
                .getAsJSONArray(new JSONArrayRequestListener() {
                    @Override
                    public void onResponse(JSONArray response) {
                        Type type = new TypeToken<List<DineinSiteSettings>>() {
                        }.getType();
                        siteSettings.addAll(new Gson().fromJson(response.toString(), type));
                        myApp.getMyPreferences().saveDineinSiteSettings(siteSettings);
                        myApp.dineinSiteSettings.clear();
                        myApp.dineinSiteSettings.addAll(siteSettings);
                        fetchPayments();
                    }

                    @Override
                    public void onError(ANError anError) {
                        anError.printStackTrace();
                        if (!CommonFunctions.isConnected(getActivity())) {
                            myApp.noInternet(getActivity());
                        }
                    }
                });

    }

    private void fetchPayments() {
        AndroidNetworking.get(ApiEndPoints.dinein_payment_methods)
                .addQueryParameter("nopaginate", "1")
                .addHeaders("business-id", business_id)
                .build()
                .getAsJSONArray(new JSONArrayRequestListener() {
                    @Override
                    public void onResponse(JSONArray response) {
                        Type type = new TypeToken<List<DineinPaymentMethods>>() {
                        }.getType();
                        paymentMethods.addAll(new Gson().fromJson(response.toString(), type));
                        myApp.getMyPreferences().saveDineinPaymentMethods(paymentMethods);
                        myApp.dineinPaymentMethods.clear();
                        myApp.dineinPaymentMethods.addAll(paymentMethods);
                        fetchTableStatuses();
                    }

                    @Override
                    public void onError(ANError anError) {
                        anError.printStackTrace();
                        if (!CommonFunctions.isConnected(getActivity())) {
                            myApp.noInternet(getActivity());
                        }
                    }
                });

    }

    private void fetchTableStatuses() {
        AndroidNetworking.get(ApiEndPoints.dinein_table_statuses)
                .addQueryParameter("nopaginate", "1")
                .addHeaders("business-id", business_id)
                .build()
                .getAsJSONArray(new JSONArrayRequestListener() {
                    @Override
                    public void onResponse(JSONArray response) {
                        Type type = new TypeToken<List<TablesStatuses>>() {
                        }.getType();
                        tablesStatuses.addAll(new Gson().fromJson(response.toString(), type));
                        myApp.getMyPreferences().saveDineinTablesStatus(tablesStatuses);
                        LogUtils.e("DINE IN TABLES");
                        fetchUserMini();
                    }

                    @Override
                    public void onError(ANError anError) {
                        anError.printStackTrace();
                        if (!CommonFunctions.isConnected(getActivity())) {
                            myApp.noInternet(getActivity());
                        }
                    }
                });

    }

    private void fetchUserMini() {
        AndroidNetworking.get(ApiEndPoints.dinein_user_mini)
                .addQueryParameter("nopaginate", "1")
                .addHeaders("business-id", business_id)
                .build()
                .getAsJSONArray(new JSONArrayRequestListener() {
                    @Override
                    public void onResponse(JSONArray response) {
                        Type type = new TypeToken<List<UsersList>>() {
                        }.getType();
                        userMinis.addAll(new Gson().fromJson(response.toString(), type));
                        myApp.getMyPreferences().saveDineinUser(userMinis.get(0));
                        LogUtils.e("USER MINI");
                    }

                    @Override
                    public void onError(ANError anError) {
                        anError.printStackTrace();
                        if (!CommonFunctions.isConnected(getActivity())) {
                            myApp.noInternet(getActivity());
                        }
                    }
                });
    }

    private void fetchOrders() {
        if (getActivity() != null)
            getActivity().runOnUiThread(() -> {
                swipeRefreshLayout.setRefreshing(true);
                llNoData.setVisibility(View.GONE);
            });
        AndroidNetworking.get(ApiEndPoints.dinein_orders_history)
                .addQueryParameter("user_id", String.valueOf(loggedInUser.id))
                .addQueryParameter("customer_id", (myApp.getMyPreferences().getCurrentDineInRestaurantDetail() != null) ? (myApp.getMyPreferences().getCurrentDineInRestaurantDetail().epos_customer.id) : "")
                .addHeaders("business-id", business_id)
                .build()
                .getAsJSONObject(new JSONObjectRequestListener() {
                    @Override
                    public void onResponse(JSONObject response) {
                        try {
                            if (getActivity() != null)
                                getActivity().runOnUiThread(() -> swipeRefreshLayout.setRefreshing(false));
                            Type type = new TypeToken<List<CreatedOrder>>() {
                            }.getType();
                            orders.clear();
                            eposCustomer = null;
                            try {
                                orders.addAll(new Gson().fromJson(response.getJSONObject("order_history").getJSONArray("data").toString(), type));
                                eposCustomer = new Gson().fromJson(response.getJSONObject("epos_customer").toString(), EposCustomer.class);
                            } catch (JSONException e) {
                                e.printStackTrace();
                            }
                            filterOrders();
                            if (getActivity() != null)
                                getActivity().runOnUiThread(() -> {
                                    if (feed.size() == 0) {
                                        llNoData.setVisibility(View.VISIBLE);
                                        tvMessage.setText("Seems like you don't have any dine in orders.");
                                    } else llNoData.setVisibility(View.GONE);
                                });
                        } catch (JsonSyntaxException e) {
                            e.printStackTrace();
                        }

                    }

                    @Override
                    public void onError(ANError anError) {
                        anError.printStackTrace();
                        if (getActivity() != null)
                            getActivity().runOnUiThread(() -> {
                                swipeRefreshLayout.setRefreshing(false);
                                llNoData.setVisibility(View.VISIBLE);
                                tvMessage.setText("Seems like you don't have any dine in orders yet.");
                            });
                        LogUtils.e("ORDER HISTORY ERROR");
                        if (!CommonFunctions.isConnected(getActivity())) {
                            myApp.noInternet(getActivity());
                        }
                    }
                });
    }

    private void filterOrders() {
        feed.clear();
        if (status == null) {
            feed.addAll(orders);
        } else {
            for (CreatedOrder orderHistory : orders) {
                if (status.equalsIgnoreCase("active") && (orderHistory.order_status_id == 1 || orderHistory.order_status_id == 2 || orderHistory.order_status_id == 7 || orderHistory.order_status_id == 12)) {
                    orderHistory.status = status;
                    feed.add(orderHistory);
                } else if (status.equalsIgnoreCase("completed") && (orderHistory.order_status_id == 4 || orderHistory.order_status_id == 5 || orderHistory.order_status_id == 6 || orderHistory.order_status_id == 10)) {
                    orderHistory.status = status;
                    feed.add(orderHistory);
                }
            }
        }
        if (getActivity() != null)
            getActivity().runOnUiThread(() -> {
                orderHistoryAdapter.notifyDataSetChanged();
                if (feed.size() == 0) llNoData.setVisibility(View.VISIBLE);
                else llNoData.setVisibility(View.GONE);
            });
    }


    private void openRatingDialog(int position, Object object) {
        DineInOrderHistory orderHistory = (DineInOrderHistory) object;
        AlertDialog alertDialog = new AlertDialog.Builder(getActivity()).create();
        alertDialog.getWindow().setBackgroundDrawable(new ColorDrawable(Color.TRANSPARENT));
        LayoutInflater inflater = (LayoutInflater) getActivity().getSystemService(Context.LAYOUT_INFLATER_SERVICE);
        View dialogView = inflater.inflate(R.layout.dialog_write_review, null);

        SimpleRatingBar ratingBar = dialogView.findViewById(R.id.rating);
        EditText editText = dialogView.findViewById(R.id.et_Reviewtext);
        TextView tv_Cancel = dialogView.findViewById(R.id.tvCancel);
        TextView tv_Confirm = dialogView.findViewById(R.id.tvConfirm);
        TextView tv_Info = dialogView.findViewById(R.id.tvInfo);
        tv_Info.setText(new StringBuilder("How was your meal from ").append(orderHistory.restaurant.restaurant_name).append("? \nLeave a review here."));

        tv_Confirm.setOnClickListener(view -> {

            alertDialog.dismiss();
            String review = editText.getText().toString();
            float ratings = ratingBar.getRating();
            sendrating(object, ratings, review);

        });
        tv_Cancel.setOnClickListener(view -> alertDialog.dismiss());


        alertDialog.setView(dialogView);
        alertDialog.setCancelable(false);
        alertDialog.show();

    }


    private void sendrating(Object object, float ratings, String review) {
        int rating = (int) ratings;
        progressDialog.show();
        AndroidNetworking.post(ApiEndPoints.reviews)
                .addBodyParameter("order_id", ((DineInOrderHistory) object).id + "")
                .addBodyParameter("restaurant_id", MyApp.RESTAURANT_ID)
                .addBodyParameter("customer_id", String.valueOf(loggedInUser.id))
                .addBodyParameter("order_type", "1")
                .addBodyParameter("rating", rating + "")
                .addBodyParameter("message", review)
                .addBodyParameter("responce", "")
                .build()
                .getAsJSONObject(new JSONObjectRequestListener() {
                    @Override
                    public void onResponse(JSONObject response) {

                        if (getActivity() != null)
                            getActivity().runOnUiThread(() -> {
                                progressDialog.hide();
                            });

                        showReviewDone();
                    }

                    @Override
                    public void onError(ANError anError) {
                        anError.printStackTrace();

                        if (getActivity() != null)
                            progressDialog.hide();
                        ToastUtils.makeToast(getActivity(), "Something went wrong");

                        if (!CommonFunctions.isConnected(getActivity()))
                            myApp.noInternet(getActivity());
                    }
                });

    }

    private void showReviewDone() {
        AlertDialog alertDialog = new AlertDialog.Builder(getActivity()).create();
        alertDialog.getWindow().setBackgroundDrawable(new ColorDrawable(Color.TRANSPARENT));
        LayoutInflater inflater = (LayoutInflater) getActivity().getSystemService(Context.LAYOUT_INFLATER_SERVICE);
        View dialogView = inflater.inflate(R.layout.dialog_order_placed, null);

        LottieAnimationView lottieAnimationView = dialogView.findViewById(R.id.animation_view);
        TextView tvMessage = dialogView.findViewById(R.id.tvMessage);
        lottieAnimationView.setAnimation(R.raw.review_star);
        tvMessage.setText("Review submitted successfully");
        lottieAnimationView.addAnimatorListener(new Animator.AnimatorListener() {
            @Override
            public void onAnimationStart(Animator animator) {

            }

            @Override
            public void onAnimationEnd(Animator animator) {
                new Handler(Looper.myLooper()).postDelayed(new Runnable() {
                    @Override
                    public void run() {
                        alertDialog.dismiss();
                        new Thread(() -> {
                            fetchData();
                        }).start();

                    }
                }, 10);

            }

            @Override
            public void onAnimationCancel(Animator animator) {

            }

            @Override
            public void onAnimationRepeat(Animator animator) {

            }
        });
        alertDialog.setView(dialogView);
        alertDialog.setCancelable(false);
        alertDialog.show();
    }
}
