package com.tiffintom.fragment;

import android.app.Dialog;
import android.content.DialogInterface;
import android.graphics.ColorFilter;
import android.graphics.PorterDuff;
import android.graphics.PorterDuffColorFilter;
import android.os.Build;
import android.os.Bundle;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.EditText;
import android.widget.FrameLayout;
import android.widget.LinearLayout;
import android.widget.TextView;

import androidx.annotation.NonNull;
import androidx.annotation.Nullable;
import androidx.appcompat.app.AlertDialog;
import androidx.core.content.ContextCompat;
import androidx.recyclerview.widget.LinearLayoutManager;
import androidx.recyclerview.widget.RecyclerView;

import com.airbnb.lottie.LottieAnimationView;
import com.airbnb.lottie.LottieProperty;
import com.airbnb.lottie.model.KeyPath;
import com.airbnb.lottie.value.LottieFrameInfo;
import com.airbnb.lottie.value.SimpleLottieValueCallback;
import com.androidnetworking.AndroidNetworking;
import com.androidnetworking.error.ANError;
import com.androidnetworking.interfaces.JSONArrayRequestListener;
import com.androidnetworking.interfaces.JSONObjectRequestListener;
import com.google.android.material.bottomsheet.BottomSheetBehavior;
import com.google.android.material.bottomsheet.BottomSheetDialog;
import com.google.android.material.bottomsheet.BottomSheetDialogFragment;
import com.google.gson.Gson;
import com.google.gson.reflect.TypeToken;
import com.tiffintom.MyApp;
import com.tiffintom.R;
import com.tiffintom.adapters.VoucherListAdapter;
import com.tiffintom.common.CommonFunctions;
import com.tiffintom.common.Validators;
import com.tiffintom.interfaces.DialogDismissListener;
import com.tiffintom.models.BusinessRestaurant;
import com.tiffintom.models.DineinModels.DineinVoucherList;
import com.tiffintom.models.RestaurantVoucher;
import com.tiffintom.network.ApiUtils;
import com.tiffintom.utils.LogUtils;
import com.tiffintom.utils.ToastUtils;

import org.json.JSONArray;
import org.json.JSONException;
import org.json.JSONObject;

import java.lang.reflect.Type;
import java.util.ArrayList;
import java.util.List;

/**
 * Created by Amrish on 18-09-2020.
 */
public class DineinVoucherListBottomsheetFragment extends BottomSheetDialogFragment {
    public static DineinVoucherListBottomsheetFragment getInstance(BusinessRestaurant restaurant) {
        Bundle bundle = new Bundle();
        bundle.putString("restaurant", new Gson().toJson(restaurant));
        DineinVoucherListBottomsheetFragment voucherListBottomsheetFragment = new DineinVoucherListBottomsheetFragment();
        voucherListBottomsheetFragment.setArguments(bundle);
        return voucherListBottomsheetFragment;
    }

    public static DineinVoucherListBottomsheetFragment getInstance(BusinessRestaurant restaurant, int order_type) {
        Bundle bundle = new Bundle();
        bundle.putString("restaurant", new Gson().toJson(restaurant));
        bundle.putInt("order_type", order_type);
        DineinVoucherListBottomsheetFragment voucherListBottomsheetFragment = new DineinVoucherListBottomsheetFragment();
        voucherListBottomsheetFragment.setArguments(bundle);
        return voucherListBottomsheetFragment;
    }

    private DialogDismissListener dialogDismissListener;
    private RecyclerView rvVoucherList;
    private VoucherListAdapter voucherListAdapter;
    private ArrayList<Object> voucherLists = new ArrayList<>();

    private EditText etPromocode;
    private TextView tvAvailableVouchers, tvApply;
    private LottieAnimationView lodingView;
    private LinearLayout llNoData;
    private TextView tvMessage;
    private BusinessRestaurant restaurant;
    private AlertDialog progressDialog;
    private int order_type;
    private String orderType;


    public void setDialogDismissListener(DialogDismissListener dialogDismissListener) {
        this.dialogDismissListener = dialogDismissListener;
    }

    @Override
    public int getTheme() {
        if (Build.VERSION.SDK_INT > 21) {
            return R.style.BottomSheetDialogTheme;
        } else {
            return super.getTheme();
        }
    }

    @NonNull
    @Override
    public Dialog onCreateDialog(Bundle savedInstanceState) {

        BottomSheetDialog dialog = (BottomSheetDialog) super.onCreateDialog(savedInstanceState);

        dialog.setOnShowListener((DialogInterface.OnShowListener) dialog1 -> {
            BottomSheetDialog d = (BottomSheetDialog) dialog1;
            FrameLayout bottomSheet = (FrameLayout) d.findViewById(com.google.android.material.R.id.design_bottom_sheet);
            if (bottomSheet != null)
                BottomSheetBehavior.from(bottomSheet).setState(BottomSheetBehavior.STATE_EXPANDED);
        });

        return dialog;
    }

    @Nullable
    @Override
    public View onCreateView(@NonNull LayoutInflater inflater, @Nullable ViewGroup container, @Nullable Bundle savedInstanceState) {
        return inflater.inflate(R.layout.fragment_voucher_list, container, false);

    }

    @Override
    public void onViewCreated(@NonNull View view, @Nullable Bundle savedInstanceState) {
        super.onViewCreated(view, savedInstanceState);
        if (getArguments() != null) {
            restaurant = new Gson().fromJson(getArguments().getString("restaurant"), BusinessRestaurant.class);
            order_type = getArguments().getInt("order_type");
        }
        if (order_type == 0) {
            orderType = "dine in";
        }
        initViews(view);
        setListeners();
        fetchVouhcerList();
    }

    private void fetchVouhcerList() {
        if (getActivity() != null) {
            getActivity().runOnUiThread(() -> {
                lodingView.setVisibility(View.VISIBLE);
                CommonFunctions.hideKeyboard(getActivity(), etPromocode);
            });
        }
        ApiUtils.getDineinVouchers(restaurant.business.id, restaurant.epos_customer != null ? restaurant.epos_customer.id : "", "0", orderType, CommonFunctions.getCurrentDate(), CommonFunctions.getCurrentDate(), "1")
                .getAsJSONArray(new JSONArrayRequestListener() {
                    @Override
                    public void onResponse(JSONArray response) {
                        if (getActivity() != null) {
                            getActivity().runOnUiThread(() -> {
                                lodingView.setVisibility(View.GONE);
                            });
                        }

                        Type type = new TypeToken<List<DineinVoucherList>>() {
                        }.getType();
                        LogUtils.e("Voucher responce::" + new Gson().toJson(response));
                        voucherLists.clear();
                        voucherLists.addAll(new Gson().fromJson(response.toString(), type));
                        voucherListAdapter.notifyDataSetChanged();
                    }

                    @Override
                    public void onError(ANError anError) {
                        LogUtils.e("GET DINEIN PROMO ANERROR" + anError.getErrorBody());
                        if (getActivity() != null) {
                            getActivity().runOnUiThread(() -> {
                                if (voucherLists.size() == 0) {
                                    llNoData.setVisibility(View.VISIBLE);
                                    tvAvailableVouchers.setVisibility(View.GONE);
                                    tvMessage.setText("Seems like you don't have any voucher yet.");
                                } else {
                                    llNoData.setVisibility(View.GONE);
                                    tvAvailableVouchers.setVisibility(View.VISIBLE);
                                }
                                lodingView.setVisibility(View.GONE);
                            });
                        }

                        try {
                            JSONObject jsonObject = new JSONObject(anError.getErrorBody());
                            ToastUtils.makeToast(getActivity(), jsonObject.getString("message"));
                        } catch (JSONException e) {
                            e.printStackTrace();
                        }


                    }
                });

    }

    private void setListeners() {
        tvApply.setOnClickListener(view -> {
            if (!Validators.isNullOrEmpty(etPromocode.getText().toString())) {
                applyPromocode();
            }
        });

    }

    private void initViews(View view) {
        progressDialog = CommonFunctions.customProgressDialog(getContext(), "");
        etPromocode = view.findViewById(R.id.etPromocode);
        tvApply = view.findViewById(R.id.tvApply);
        rvVoucherList = view.findViewById(R.id.rvVouchers);
        tvAvailableVouchers = view.findViewById(R.id.tvAvailableVouchers);
        lodingView = view.findViewById(R.id.lodingView);
        llNoData = view.findViewById(R.id.llNoData);
        tvMessage = view.findViewById(R.id.tvMessage);

        lodingView.addValueCallback(
                new KeyPath("**"),
                LottieProperty.COLOR_FILTER,
                new SimpleLottieValueCallback<ColorFilter>() {
                    @Override
                    public ColorFilter getValue(LottieFrameInfo<ColorFilter> frameInfo) {
                        return new PorterDuffColorFilter(ContextCompat.getColor(getActivity(), R.color.colorPrimary), PorterDuff.Mode.SRC_ATOP);
                    }
                }
        );

        voucherListAdapter = new VoucherListAdapter(voucherLists, (position, data) -> {
            if (data instanceof DineinVoucherList) {
                if (dialogDismissListener != null) {
                    dialogDismissListener.onDialogDismiss(data);
                    dismiss();
                }
            }
        });
        rvVoucherList.setAdapter(voucherListAdapter);
        rvVoucherList.setLayoutManager(new LinearLayoutManager(getActivity(), RecyclerView.VERTICAL, false));
    }

    private void applyPromocode() {
        if (getActivity() != null) {
            getActivity().runOnUiThread(() -> {
                lodingView.setVisibility(View.VISIBLE);
                CommonFunctions.hideKeyboard(getActivity(), etPromocode);
            });
        }
        ApiUtils.applyPromoCode(orderType, String.valueOf(restaurant.id), etPromocode.getText().toString())
                .getAsJSONObject(new JSONObjectRequestListener() {
                    @Override
                    public void onResponse(JSONObject response) {
                        RestaurantVoucher appliedVoucher = new Gson().fromJson(response.toString(), RestaurantVoucher.class);
                        if (dialogDismissListener != null) {
                            dialogDismissListener.onDialogDismiss(appliedVoucher);
                        }
                        dismiss();

                        if (getActivity() != null) {
                            getActivity().runOnUiThread(() -> {
                                lodingView.setVisibility(View.GONE);
                            });
                        }
                    }

                    @Override
                    public void onError(ANError anError) {
                        anError.printStackTrace();
                        LogUtils.e("APPLY PROMO ANERROR");
                        if (getActivity() != null) {
                            getActivity().runOnUiThread(() -> {
                                lodingView.setVisibility(View.GONE);
                            });
                        }

                    }
                });
    }

    @Override
    public void onDismiss(@NonNull DialogInterface dialog) {
        AndroidNetworking.cancel("recent_orders");
        super.onDismiss(dialog);
    }
}
