package com.tiffintom.fragment;

import android.Manifest;
import android.app.AlertDialog;
import android.content.Intent;
import android.content.pm.PackageManager;
import android.location.Address;
import android.location.Location;
import android.net.Uri;
import android.os.Bundle;
import android.provider.Settings;
import android.text.Editable;
import android.text.TextWatcher;
import android.view.KeyEvent;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.Button;
import android.widget.EditText;
import android.widget.ImageView;

import androidx.annotation.NonNull;
import androidx.annotation.Nullable;
import androidx.core.app.ActivityCompat;
import androidx.viewpager.widget.ViewPager;

import com.androidnetworking.AndroidNetworking;
import com.androidnetworking.error.ANError;
import com.androidnetworking.interfaces.ParsedRequestListener;
import com.google.gson.Gson;
import com.make.dots.dotsindicator.DotsIndicator;
import com.tiffintom.R;
import com.tiffintom.activity.MainActivity;
import com.tiffintom.activity.SplashActivity;
import com.tiffintom.adapters.EnterPostCodeViewPagerAdapter;
import com.tiffintom.base.BaseFragment;
import com.tiffintom.common.CommonFunctions;
import com.tiffintom.common.MyLocation;
import com.tiffintom.common.Validators;
import com.tiffintom.interfaces.DialogDismissListener;
import com.tiffintom.models.ApiError;
import com.tiffintom.models.Postcode;
import com.tiffintom.models.PostcodeViewPager;
import com.tiffintom.network.ApiEndPoints;
import com.tiffintom.utils.LogUtils;
import com.tiffintom.utils.ToastUtils;

import java.util.ArrayList;

import static com.facebook.FacebookSdk.getApplicationContext;
import static com.tiffintom.utils.Constants.PERMISSION_CODE_LOCATION;

import org.json.JSONException;
import org.json.JSONObject;

/**
 * Created by Amrish on 31-08-2020.
 */
public class EnterPostcodeFragment extends BaseFragment {
    private EditText etPostcode;
    private Button btnConfirm;
    private ImageView ivCurrentLocation, ivSearch;
    private ViewPager viewPager;
    private EnterPostCodeViewPagerAdapter viewPagerAdapter;
    private ArrayList<PostcodeViewPager> postcodearraylist = new ArrayList<>();
    private DotsIndicator dotsIndicator;

    public static EnterPostcodeFragment getInstance() {
        return new EnterPostcodeFragment();
    }

    @Nullable
    @Override
    public View onCreateView(@NonNull LayoutInflater inflater, @Nullable ViewGroup container, @Nullable Bundle savedInstanceState) {
        return inflater.inflate(R.layout.fragment_set_postcode, container, false);
    }

    @Override
    public void onViewCreated(@NonNull View view, @Nullable Bundle savedInstanceState) {
        super.onViewCreated(view, savedInstanceState);
        setListeners();
        setUpData();
    }

    private void setUpData() {
        PostcodeViewPager model = new PostcodeViewPager();
        model.image = R.drawable.info_icon_discover;
        model.title = "DISCOVER";
        model.subtitle = "Find your local restaurants";
        postcodearraylist.add(model);

        model = new PostcodeViewPager();
        model.image = R.drawable.info_icon_addmenu;
        model.title = "CHOOSE YOUR MENU";
        model.subtitle = "Select your favourite dishes";
        postcodearraylist.add(model);

        model = new PostcodeViewPager();
        model.image = R.drawable.info_icon_placeorder;
        model.title = "PLACE ORDER";
        model.subtitle = "Pay by card or cash";
        postcodearraylist.add(model);

        model = new PostcodeViewPager();
        model.image = R.drawable.info_icon_delivery;
        model.title = "GET DELIVERED";
        model.subtitle = "And enjoy";
        postcodearraylist.add(model);
        viewPagerAdapter.notifyDataSetChanged();
    }

    @Override
    protected void initViews(View view) {
        super.initViews(view);
        etPostcode = view.findViewById(R.id.etPostcode);
        btnConfirm = view.findViewById(R.id.btnConfirm);
        ivCurrentLocation = view.findViewById(R.id.ivCurrentLocation);
        viewPager = view.findViewById(R.id.viewPager);
        dotsIndicator = view.findViewById(R.id.dotsIndicator);
        ivSearch = view.findViewById(R.id.ivSearch);

        viewPagerAdapter = new EnterPostCodeViewPagerAdapter(getActivity(), postcodearraylist);
        viewPagerAdapter.registerDataSetObserver(dotsIndicator.getDataSetObserver());
        viewPager.setAdapter(viewPagerAdapter);
        dotsIndicator.setViewPager(viewPager);

    }

    private void setListeners() {
        etPostcode.addTextChangedListener(new TextWatcher() {
            @Override
            public void beforeTextChanged(CharSequence charSequence, int i, int i1, int i2) {

            }

            @Override
            public void onTextChanged(CharSequence charSequence, int i, int i1, int i2) {
                btnConfirm.setEnabled(isValid());
            }

            @Override
            public void afterTextChanged(Editable editable) {

            }
        });
        btnConfirm.setOnClickListener(view -> {
            if (Validators.isNullOrEmpty(etPostcode.getText().toString())) {
                ToastUtils.makeToast(getActivity(), "Please enter postcode");
            } else if (etPostcode.getText().toString().length() < 3) {
                ToastUtils.makeToast(getActivity(), "Please valid postcode");
            } else {
                String postcode = etPostcode.getText().toString();
                new Thread(() -> searchPostcode(postcode)).start();
            }
        });
        ivCurrentLocation.setOnClickListener(view -> {
            if (checkLocationPermission()) {
                startLocationService();
            }
        });

        ivSearch.setOnClickListener(view -> {
            if (Validators.isNullOrEmpty(etPostcode.getText().toString())) {
                ToastUtils.makeToast(getActivity(), "Please enter postcode");
            } else if (etPostcode.getText().toString().length() < 3) {
                ToastUtils.makeToast(getActivity(), "Please valid postcode");
            } else {
                String postcode = etPostcode.getText().toString();
                new Thread(() -> searchPostcode(postcode)).start();
            }
        });
        etPostcode.setOnKeyListener((textView, i, keyEvent) -> {
            //This is the filter
            if (keyEvent.getAction() != KeyEvent.ACTION_DOWN)
                return true;
            if (keyEvent.getKeyCode() == KeyEvent.KEYCODE_BACK) {
                getActivity().onBackPressed();
            } else if ((keyEvent.getKeyCode() == KeyEvent.KEYCODE_ENTER)) {
                if (Validators.isNullOrEmpty(etPostcode.getText().toString())) {
                    ToastUtils.makeToast(getActivity(), "Please enter postcode");
                }
                if (etPostcode.getText().toString().length() > 3) {
                    ToastUtils.makeToast(getActivity(), "Please valid postcode");
                } else {
                    String postcode = etPostcode.getText().toString();
                    new Thread(() -> searchPostcode(postcode)).start();
                }
            }
            return false;
        });
    }

    private boolean isValid() {
        if (etPostcode.getText().toString().length() >= 3) {
            return !Validators.isNullOrEmpty(etPostcode.getText().toString());
        }
        return false;
    }

    private void searchPostcode(String postcode) {
        if (getActivity() != null) getActivity().runOnUiThread(progressDialog::show);
        AndroidNetworking.get(ApiEndPoints.RMS_POSTCODE + postcode)
                .build()
                .getAsObject(Postcode.class, new ParsedRequestListener<Postcode>() {
                    @Override
                    public void onResponse(Postcode response) {
                        if (getActivity() != null)
                            CommonFunctions.hideKeyboard(getActivity(), etPostcode);
                        if (getActivity() != null)
                            getActivity().runOnUiThread(() -> progressDialog.dismiss());
                        myApp.getMyPreferences().saveCurrentPostcode(response);
                        startActivity(new Intent(getActivity(), SplashActivity.class).setFlags(Intent.FLAG_ACTIVITY_CLEAR_TASK | Intent.FLAG_ACTIVITY_NEW_TASK | Intent.FLAG_ACTIVITY_CLEAR_TOP));
                    }

                    @Override
                    public void onError(ANError anError) {
                        if (getActivity() != null)
                            CommonFunctions.hideKeyboard(getActivity(), etPostcode);
                        if (getActivity() != null)
                            getActivity().runOnUiThread(() -> progressDialog.dismiss());
                        if (anError.getErrorCode() == 400) {
                            try {
                                JSONObject jsonObject = new JSONObject(anError.getErrorBody().toString());
                                ToastUtils.makeToast(getApplicationContext(), jsonObject.getString("message"));
                            } catch (JSONException exception) {
                                exception.printStackTrace();
                            }
                        } else {
                            if (!CommonFunctions.isConnected(getActivity())) {
                                myApp.noInternet(getActivity());
                            }
                        }
                    }
                });
    }

    private DialogDismissListener addAddressDialogDismissListener = o -> {
        startActivity(new Intent(getActivity(), MainActivity.class));
        getActivity().finish();
    };

    public boolean checkLocationPermission() {
        if (ActivityCompat.checkSelfPermission(getActivity(),
                Manifest.permission.ACCESS_FINE_LOCATION)
                != PackageManager.PERMISSION_GRANTED) {

            // Should we show an explanation?
            if (ActivityCompat.shouldShowRequestPermissionRationale(getActivity(),
                    Manifest.permission.ACCESS_FINE_LOCATION)) {

                new AlertDialog.Builder(getActivity())
                        .setTitle("Location permission required")
                        .setMessage("To access your current postcode we require your location permission, Please allow to continue.")
                        .setPositiveButton("ALLOW", (dialogInterface, i) -> {
                            requestPermissions(new String[]{Manifest.permission.ACCESS_FINE_LOCATION}, PERMISSION_CODE_LOCATION);
                        })
                        .setNegativeButton("DENY", (dialogInterface, i) -> dialogInterface.dismiss())
                        .create()
                        .show();


            } else {
                requestPermissions(new String[]{Manifest.permission.ACCESS_FINE_LOCATION}, PERMISSION_CODE_LOCATION);
            }
            return false;
        } else {
            return true;
        }
    }


    @Override
    public void onRequestPermissionsResult(int requestCode, String permissions[], int[] grantResults) {
        if (requestCode == PERMISSION_CODE_LOCATION) {
            if (grantResults.length > 0 && grantResults[0] == PackageManager.PERMISSION_GRANTED) {
                if (ActivityCompat.checkSelfPermission(getActivity(),
                        Manifest.permission.ACCESS_FINE_LOCATION)
                        == PackageManager.PERMISSION_GRANTED) {
                    startLocationService();
                } else {
                    ToastUtils.makeToast(getActivity(), "Permission denied.");
                }

            } else {
                if (!ActivityCompat.shouldShowRequestPermissionRationale(getActivity(),
                        Manifest.permission.ACCESS_FINE_LOCATION)) {
                    new AlertDialog.Builder(getActivity())
                            .setTitle("Location permission required")
                            .setMessage("To access your current postcode we require your location permission, Please allow to continue.")
                            .setPositiveButton("ALLOW", (dialogInterface, i) -> {
                                Intent intent = new Intent();
                                intent.setAction(Settings.ACTION_APPLICATION_DETAILS_SETTINGS);
                                Uri uri = Uri.fromParts("package", getActivity().getPackageName(), null);
                                intent.setData(uri);
                                startActivity(intent);
                            })
                            .setNegativeButton("DENY", (dialogInterface, i) -> dialogInterface.dismiss())
                            .create()
                            .show();
                } else {
                    ToastUtils.makeToast(getActivity(), "Permission denied.");
                }

            }
        }
    }

    private void startLocationService() {
        MyLocation myLocation = new MyLocation();
        progressDialog.show();
        myLocation.getLocation(getActivity(), new MyLocation.LocationResult() {
            @Override
            public void gotLocation(Location location) {
                if (getActivity() != null) {
                    getActivity().runOnUiThread(() -> {
                        progressDialog.dismiss();
                    });
                }

                Address address = CommonFunctions.getAddressByLocation(getActivity(), location.getLatitude(), location.getLongitude());
                if (address != null) {
                    String zipcode = address.getPostalCode();
                    LogUtils.e("Location::", new Gson().toJson(location));
                    LogUtils.e("Zipcode::", zipcode);
                    if (!Validators.isNullOrEmpty(zipcode)) {
                        etPostcode.setText(zipcode);
                        btnConfirm.setEnabled(isValid());
                        btnConfirm.performClick();
                    }
                }
            }
        });
    }

}
