package com.tiffintom.fragment;

import android.content.Intent;
import android.os.Bundle;
import android.text.Editable;
import android.text.TextWatcher;
import android.util.Log;
import android.view.KeyEvent;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.EditText;

import androidx.annotation.NonNull;
import androidx.annotation.Nullable;
import androidx.recyclerview.widget.GridLayoutManager;
import androidx.recyclerview.widget.RecyclerView;

import com.google.gson.Gson;
import com.google.gson.reflect.TypeToken;
import com.tiffintom.R;
import com.tiffintom.activity.TransportActivity;
import com.tiffintom.adapters.ExploreCuisinesAdapter;
import com.tiffintom.base.BaseFragment;
import com.tiffintom.common.CommonFunctions;
import com.tiffintom.models.Cuisine;
import com.tiffintom.utils.LogUtils;

import java.lang.reflect.Type;
import java.util.ArrayList;
import java.util.List;

/**
 * Created by Amrish on 19-08-2020.
 */
public class ExploreCategoryFragment extends BaseFragment {
    public static ExploreCategoryFragment getInstance(ArrayList<Cuisine> cuisines) {
        ExploreCategoryFragment exploreCategoryFragment = new ExploreCategoryFragment();
        Bundle bundle = new Bundle();
        bundle.putString("cuisines", new Gson().toJson(cuisines));
        exploreCategoryFragment.setArguments(bundle);
        return exploreCategoryFragment;
    }

    private RecyclerView rvCategory;
    private ExploreCuisinesAdapter categoryAdapter;
    private ArrayList<Cuisine> cuisines = new ArrayList<>();
    private ArrayList<Cuisine> filteredCuisines = new ArrayList<>();
    private EditText etSearch;

    @Nullable
    @Override
    public View onCreateView(@NonNull LayoutInflater inflater, @Nullable ViewGroup container, @Nullable Bundle savedInstanceState) {
        if (getArguments() != null) {
            Type type = new TypeToken<List<Cuisine>>() {
            }.getType();
            cuisines.addAll(new Gson().fromJson(getArguments().getString("cuisines"), type));
            filteredCuisines.addAll(cuisines);
        }
        return inflater.inflate(R.layout.fragment_explore_category, container, false);
    }

    @Override
    public void onViewCreated(@NonNull View view, @Nullable Bundle savedInstanceState) {
        super.onViewCreated(view, savedInstanceState);
        setListeners();
    }

    @Override
    protected void initViews(View view) {
        super.initViews(view);
        rvCategory = view.findViewById(R.id.rvCategory);
        etSearch = view.findViewById(R.id.etSearch);
        categoryAdapter = new ExploreCuisinesAdapter(filteredCuisines, this::categoryClick);
        rvCategory.setLayoutManager(new GridLayoutManager(getActivity(), 2));
        rvCategory.setAdapter(categoryAdapter);
    }

    private void categoryClick(int position, Object data) {
        if (data instanceof Cuisine) {
            Cuisine cuisine = (Cuisine) data;
            startActivity(new Intent(getActivity(), TransportActivity.class).putExtra("isFiltered", true).putExtra("cuisine_id", cuisine.id).putExtra("cuisine_name", cuisine.cuisine_name).putExtra("name", cuisine.cuisine_name).putExtra("destination", "restaurants"));
        }
    }

    private void setListeners() {
        etSearch.setOnKeyListener((textView, i, keyEvent) -> {
            if (keyEvent.getKeyCode() == KeyEvent.KEYCODE_BACK) {
                getActivity().onBackPressed();
            } else if ((keyEvent.getKeyCode() == KeyEvent.KEYCODE_ENTER)) {
                filteredCuisines.clear();
                filteredCuisines.addAll(searchCuisine());
                categoryAdapter.notifyDataSetChanged();
            }
            if (getActivity() != null)
                CommonFunctions.hideKeyboard(getActivity(), etSearch);
            return true;
        });
        etSearch.addTextChangedListener(new TextWatcher() {
            @Override
            public void beforeTextChanged(CharSequence charSequence, int i, int i1, int i2) {

            }

            @Override
            public void onTextChanged(CharSequence charSequence, int i, int i1, int i2) {
            }

            @Override
            public void afterTextChanged(Editable editable) {
                filteredCuisines.clear();
                filteredCuisines.addAll(searchCuisine());
                categoryAdapter.notifyDataSetChanged();
            }
        });
    }

    private ArrayList<Cuisine> searchCuisine() {
        if (etSearch.getText().toString().isEmpty()) {
            return cuisines;
        }
        ArrayList<Cuisine> searchedRest = new ArrayList<>();
        for (Cuisine r : cuisines) {
            if (r.cuisine_name != null && r.cuisine_name.toLowerCase().contains(etSearch.getText().toString().toLowerCase())) {
                searchedRest.add(r);
            }
        }
        return searchedRest;

    }
}
