package com.tiffintom.fragment;

import android.content.Intent;
import android.os.Bundle;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.ImageView;
import android.widget.LinearLayout;
import android.widget.ProgressBar;
import android.widget.TextView;

import androidx.annotation.NonNull;
import androidx.annotation.Nullable;
import androidx.fragment.app.FragmentPagerAdapter;
import androidx.viewpager.widget.ViewPager;

import com.airbnb.lottie.LottieAnimationView;
import com.androidnetworking.AndroidNetworking;
import com.androidnetworking.error.ANError;
import com.androidnetworking.interfaces.JSONObjectRequestListener;
import com.google.android.material.tabs.TabLayout;
import com.google.gson.Gson;
import com.tiffintom.BuildConfig;
import com.tiffintom.MyApp;
import com.tiffintom.R;
import com.tiffintom.activity.NoInternetActivity;
import com.tiffintom.activity.SplashActivity;
import com.tiffintom.adapters.ViewpagerAdapter;
import com.tiffintom.base.BaseFragment;
import com.tiffintom.common.CommonFunctions;
import com.tiffintom.models.Address;
import com.tiffintom.models.Cuisine;
import com.tiffintom.models.HomeSearchParam;
import com.tiffintom.models.Postcode;
import com.tiffintom.models.Referral;
import com.tiffintom.models.Restaurant;
import com.tiffintom.models.RewardPoint;
import com.tiffintom.models.SearchResult;
import com.tiffintom.models.SiteSettings;
import com.tiffintom.models.User;
import com.tiffintom.network.ApiEndPoints;
import com.tiffintom.network.ApiUtils;
import com.tiffintom.utils.LogUtils;
import com.tiffintom.utils.ToastUtils;

import org.json.JSONException;
import org.json.JSONObject;

import java.util.ArrayList;

import static com.tiffintom.MyApp.searchResult;
import static com.tiffintom.utils.Constants.CODE_NO_INTERNET;

/**
 * Created by Amrish on 19-08-2020.
 */
public class ExploreFragment extends BaseFragment {
    public static ExploreFragment getInstance() {
        ExploreFragment fragment = new ExploreFragment();
        Bundle bundle = new Bundle();
        fragment.setArguments(bundle);
        return fragment;
    }

    private ViewPager viewPager;
    private TabLayout tabLayout;
    private ArrayList<Cuisine> cuisines = new ArrayList<>();
    public static ArrayList<Restaurant> restaurants = new ArrayList<>();
    private ProgressBar pbLoading;
    private User loggedInUser = myApp.getMyPreferences().getLoggedInUser();
    private Postcode currentPostcode;
    private Address defaultAddress;
    private LottieAnimationView lodingView;
    private TextView tvMessage;
    private LinearLayout llError;

    @Nullable
    @Override
    public View onCreateView(@NonNull LayoutInflater inflater, @Nullable ViewGroup container, @Nullable Bundle savedInstanceState) {
        return inflater.inflate(R.layout.fragment_tabs, container, false);
    }

    @Override
    public void onViewCreated(@NonNull View view, @Nullable Bundle savedInstanceState) {
        super.onViewCreated(view, savedInstanceState);
        if (searchResult != null)
            updateViews();
        else fetchData();
    }

    @Override
    protected void initViews(View view) {
        super.initViews(view);
        currentPostcode = myApp.getMyPreferences().getCurrentPostcode();
        defaultAddress = myApp.getMyPreferences().getCurrentAddress();
        tvUserName = view.findViewById(R.id.tvUserName);
        ImageView ivBackArrow = view.findViewById(R.id.ivBackArrow);
        LinearLayout llBackArrow = view.findViewById(R.id.llBackArrow);
        TextView tvScreenTitle = view.findViewById(R.id.tvScreenTitle);
        tvScreenTitle.setVisibility(View.VISIBLE);
        tvScreenTitle.setText("Explore");

        viewPager = view.findViewById(R.id.viewPager);
        tabLayout = view.findViewById(R.id.tabLayout);

        ivBackArrow.setVisibility(View.GONE);
        pbLoading = view.findViewById(R.id.pbLoading);
        lodingView = view.findViewById(R.id.lodingView);
        tvMessage = view.findViewById(R.id.tvMessage);
        llError = view.findViewById(R.id.llError);
    }

    private void setUpViewPager() {

        ViewpagerAdapter viewpagerAdapter = new ViewpagerAdapter(getChildFragmentManager(), FragmentPagerAdapter.BEHAVIOR_RESUME_ONLY_CURRENT_FRAGMENT);
        viewpagerAdapter.addFragment(ExploreCategoryFragment.getInstance(cuisines), "Categories");
        viewpagerAdapter.addFragment(ExploreNearbyRestaurantsFragment.getInstance(), "Near you");
        viewPager.setAdapter(viewpagerAdapter);
        tabLayout.setupWithViewPager(viewPager);
        tabLayout.setVisibility(View.VISIBLE);
        viewPager.setVisibility(View.VISIBLE);
    }

    private void fetchData() {
        new Thread(this::fetchHome).start();
    }

    private void fetchHome() {
      if (getActivity() != null)
            getActivity().runOnUiThread(() -> {
                lodingView.setVisibility(View.VISIBLE);
                llError.setVisibility(View.GONE);
            });
        ApiUtils.getHome(currentPostcode.post_code, (MyApp.getInstance().getMyPreferences().getLoggedInUser() == null)  ? "": String.valueOf(loggedInUser.id)).getAsJSONObject(new JSONObjectRequestListener() {
            @Override
            public void onResponse(JSONObject response) {
            MyApp.searchResult = new Gson().fromJson(response.toString(), SearchResult.class);
            fetchSiteSettings();
            updateViews();
            }

            @Override
            public void onError(ANError anError) {
                LogUtils.e("AN ERROR IN FETCH HOME");
                anError.printStackTrace();
                if (getActivity() != null)
                    getActivity().runOnUiThread(() -> {
                        llError.setVisibility(View.VISIBLE);
                        tvMessage.setText("There doesn't seem to be any restaurant available in your area.");
                        lodingView.setVisibility(View.GONE);
                    });

                if (!CommonFunctions.isConnected(getActivity()))
                    startActivityForResult(new Intent(getActivity(), NoInternetActivity.class), CODE_NO_INTERNET);
            }
        });
    }

    private void fetchSiteSettings() {
        AndroidNetworking.get(ApiEndPoints.NEW_SITE_SETTINGS)
                .build()
                .getAsJSONObject(new JSONObjectRequestListener() {
                    @Override
                    public void onResponse(JSONObject response) {
                        boolean updateAvailable = false;
                        MyApp.searchResult.siteSettings = new Gson().fromJson(response.toString(), SiteSettings.class);
                        myApp.getMyPreferences().saveSiteSettings(MyApp.searchResult.siteSettings );
                        AndroidNetworking.get(ApiEndPoints.NEW_REWARDS)
                                .build()
                                .getAsJSONObject(new JSONObjectRequestListener() {

                                    @Override
                                    public void onResponse(JSONObject response) {
                                        RewardPoint rewardPoint = new Gson().fromJson(response.toString(), RewardPoint.class);
                                        myApp.getMyPreferences().saveRewardPoint(rewardPoint);
                                    }

                                    @Override
                                    public void onError(ANError anError) {

                                    }
                                });
                        AndroidNetworking.get(ApiEndPoints.NEW_Referrals)
                                .build()
                                .getAsJSONObject(new JSONObjectRequestListener() {
                                    @Override
                                    public void onResponse(JSONObject response) {
                                        Referral referrals = new Gson().fromJson(response.toString(), Referral.class);
                                        myApp.getMyPreferences().saveReferral(referrals);
                                    }

                                    @Override
                                    public void onError(ANError anError) {
                                    }
                                });
                    }

                    @Override
                    public void onError(ANError anError) {

                    }
                });
    }
    private void updateViews() {
        if (searchResult != null) {
            MyApp.banne_path = searchResult.banner_path;
            myApp.getMyPreferences().saveSiteSettings(searchResult.siteSettings);
            cuisines.clear();
            cuisines.size();
            cuisines.addAll(searchResult.cuisines);
            restaurants.clear();
            restaurants.addAll(searchResult.restaurants);
            if (getActivity() != null) {
                getActivity().runOnUiThread(() -> {
                    llError.setVisibility(View.GONE);
                    lodingView.setVisibility(View.GONE);
                    setUpViewPager();
                });
            }
        }

    }

    @Override
    public void onActivityResult(int requestCode, int resultCode, @Nullable Intent data) {
        super.onActivityResult(requestCode, resultCode, data);
        if (requestCode == CODE_NO_INTERNET) {
            fetchData();
        }
    }
}
