package com.tiffintom.fragment;

import android.content.Intent;
import android.os.Bundle;
import android.view.LayoutInflater;
import android.view.MotionEvent;
import android.view.View;
import android.view.ViewGroup;

import androidx.annotation.NonNull;
import androidx.annotation.Nullable;
import androidx.appcompat.app.AlertDialog;
import androidx.core.widget.NestedScrollView;
import androidx.recyclerview.widget.LinearLayoutManager;
import androidx.recyclerview.widget.RecyclerView;

import com.google.android.gms.maps.CameraUpdateFactory;
import com.google.android.gms.maps.GoogleMap;
import com.google.android.gms.maps.MapView;
import com.google.android.gms.maps.SupportMapFragment;
import com.google.android.gms.maps.model.LatLng;
import com.google.android.gms.maps.model.MarkerOptions;
import com.tiffintom.R;
import com.tiffintom.activity.RestaurantDetailsActivity;
import com.tiffintom.adapters.ExploreNearByRestaurantAdapter;
import com.tiffintom.base.BaseFragment;
import com.tiffintom.models.Postcode;
import com.tiffintom.models.Restaurant;

import static com.tiffintom.utils.Constants.CODE_REFRESH;

/**
 * Created by Amrish on 19-08-2020.
 */
public class ExploreNearbyRestaurantsFragment extends BaseFragment {
    public static ExploreNearbyRestaurantsFragment getInstance() {
        ExploreNearbyRestaurantsFragment exploreCategoryFragment = new ExploreNearbyRestaurantsFragment();
        Bundle bundle = new Bundle();
        exploreCategoryFragment.setArguments(bundle);
        return exploreCategoryFragment;
    }

    private RecyclerView rvRestaurant;
    private ExploreNearByRestaurantAdapter restaurantAdapter;
    int res_id = 0;

    private MapView mapView;
    private SupportMapFragment mapFragment = null;
    private GoogleMap restuarantMap;
    private View mapImage;
    private NestedScrollView nestedScrollView;
    private Postcode currentPostcode = myApp.getMyPreferences().getCurrentPostcode();

    @Nullable
    @Override
    public View onCreateView(@NonNull LayoutInflater inflater, @Nullable ViewGroup container, @Nullable Bundle savedInstanceState) {
        new Thread(() -> res_id = myApp.getAppDatabase().cartDao().getRestaurantId()).start();
        return inflater.inflate(R.layout.fragment_explore_nearby, container, false);
    }

    @Override
    public void onViewCreated(@NonNull View view, @Nullable Bundle savedInstanceState) {
        super.onViewCreated(view, savedInstanceState);
    }

    @Override
    protected void initViews(View view) {
        super.initViews(view);
        rvRestaurant = view.findViewById(R.id.rvNearBy);
        mapImage = view.findViewById(R.id.mapImage);
        nestedScrollView = view.findViewById(R.id.nestedScrollView);
        restaurantAdapter = new ExploreNearByRestaurantAdapter(ExploreFragment.restaurants, this::restaurantClick);
        rvRestaurant.setLayoutManager(new LinearLayoutManager(getActivity(), RecyclerView.VERTICAL, false));
        rvRestaurant.setAdapter(restaurantAdapter);
        rvRestaurant.setNestedScrollingEnabled(false);
        fixMapScrolling();
        loadGoogleMap();
    }

    private void loadGoogleMap() {

        if (mapFragment == null) {
            mapFragment = SupportMapFragment.newInstance();
            mapFragment.getMapAsync(googleMap -> {
                if (getActivity() != null) {
                    restuarantMap = googleMap;
                    restuarantMap.clear();
                    for (Restaurant restaurant : ExploreFragment.restaurants) {
                        LatLng restaurantLocation = new LatLng(Double.parseDouble(restaurant.sourcelatitude), Double.parseDouble(restaurant.sourcelongitude));
                        googleMap.addMarker(new MarkerOptions()
                                .position(restaurantLocation)
                                .title(restaurant.restaurant_name));
                    }
                    float zoom = 10;
                    LatLng latLng = new LatLng(Double.parseDouble(currentPostcode.latitude), Double.parseDouble(currentPostcode.longitude));
                    googleMap.moveCamera(CameraUpdateFactory.newLatLngZoom(latLng, zoom));

                }
            });
            getChildFragmentManager()
                    .beginTransaction()
                    .add(R.id.mapView, mapFragment)
                    .commit();
        }

    }

    private void fixMapScrolling() {
        mapImage.setOnTouchListener((view, motionEvent) -> {
            if (motionEvent.getAction() == MotionEvent.ACTION_DOWN) {
                nestedScrollView.requestDisallowInterceptTouchEvent(true);
                return false;
            } else if (motionEvent.getAction() == MotionEvent.ACTION_UP) {
                nestedScrollView.requestDisallowInterceptTouchEvent(false);
                return true;
            } else if (motionEvent.getAction() == MotionEvent.ACTION_MOVE) {
                nestedScrollView.requestDisallowInterceptTouchEvent(true);
                return false;
            } else {
                return true;
            }
        });

    }

    private void restaurantClick(int position, Object data) {
        Restaurant clickedRestaurant = (Restaurant) data;
        Intent intent = new Intent(getActivity(), RestaurantDetailsActivity.class);
        intent.putExtra("resId", clickedRestaurant.id);
        intent.putExtra("res_name", clickedRestaurant.restaurant_name);
        startActivityForResult(intent, CODE_REFRESH);
    }

    private void showOtherRestaurantCartWarning(Restaurant clickedRestaurant) {
        AlertDialog.Builder alertDialog = new AlertDialog.Builder(getActivity());
        alertDialog.setTitle("Note");
        alertDialog.setMessage("You have added some items in cart from other restaurant, Do you want to go back and choose different Restaurant?");
        alertDialog.setPositiveButton("YES", (dialogInterface, i) -> {
            new Thread(() -> {
                myApp.getMyPreferences().deleteOrderRestaurant();
                myApp.getAppDatabase().cartDao().nukeTable();

                Intent intent = new Intent(getActivity(), RestaurantDetailsActivity.class);
                intent.putExtra("resId", clickedRestaurant.id);
                intent.putExtra("res_name", clickedRestaurant.restaurant_name);
                startActivityForResult(intent, CODE_REFRESH);

            }).start();
        });
        alertDialog.setNegativeButton("NO", (dialogInterface, i) -> {
            dialogInterface.cancel();
        });
        alertDialog.show();
    }

    @Override
    public void onResume() {
        if (mapFragment != null)
            mapFragment.onResume();
        new Thread(() -> res_id = myApp.getAppDatabase().cartDao().getRestaurantId()).start();
        super.onResume();
    }

    @Override
    public void onPause() {
        if (mapFragment != null) mapFragment.onPause();
        if (progressDialog != null) progressDialog.dismiss();
        super.onPause();
    }

    @Override
    public void onStop() {
        if (mapFragment != null) mapFragment.onStop();
        if (progressDialog != null) progressDialog.dismiss();
        super.onStop();
    }

    @Override
    public void onDestroy() {
        if (mapFragment != null)
            mapFragment.onDestroy();
        if (progressDialog != null) progressDialog.dismiss();
        super.onDestroy();
    }
}
