package com.tiffintom.fragment;

import android.content.Intent;
import android.os.Build;
import android.os.Bundle;
import android.text.Html;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.TextView;

import androidx.annotation.NonNull;
import androidx.annotation.Nullable;

import com.google.android.material.bottomsheet.BottomSheetDialogFragment;
import com.tiffintom.R;
import com.tiffintom.activity.DeepLinkProcessActivity;
import com.tiffintom.utils.LogUtils;

import me.saket.bettermovementmethod.BetterLinkMovementMethod;

public class FAQQuestionAnswerFragment extends BottomSheetDialogFragment {

    private TextView tvQuestion, tvAnswer;
    private String question, answer;


    public static FAQQuestionAnswerFragment getInstance(String question, String answer) {
        FAQQuestionAnswerFragment fragment = new FAQQuestionAnswerFragment();
        Bundle bundle = new Bundle();
        bundle.putString("question", question);
        bundle.putString("answer", answer);
        fragment.setArguments(bundle);
        return fragment;
    }

    @Override
    public int getTheme() {
        if (Build.VERSION.SDK_INT > 21) {
            return R.style.BottomSheetDialogTheme;
        } else {
            return super.getTheme();
        }
    }

    @Nullable
    @Override
    public View onCreateView(@NonNull LayoutInflater inflater, @Nullable ViewGroup container, @Nullable Bundle savedInstanceState) {

        if (getArguments() != null) {
            question = getArguments().getString("question");
            answer = getArguments().getString("answer");
        }

        return inflater.inflate(R.layout.fragment_question_answer, container, false);
    }

    @Override
    public void onViewCreated(@NonNull View view, @Nullable Bundle savedInstanceState) {
        super.onViewCreated(view, savedInstanceState);

        initViews(view);
    }

    private void initViews(View view) {

        tvQuestion = view.findViewById(R.id.tvQuestion);
        tvAnswer = view.findViewById(R.id.tvAnswer);

        if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.N) {
            tvAnswer.setText(Html.fromHtml(answer, Html.FROM_HTML_MODE_COMPACT));
        } else {
            tvAnswer.setText(Html.fromHtml(answer));
        }
        BetterLinkMovementMethod.linkifyHtml(tvAnswer)
                .setOnLinkClickListener(new BetterLinkMovementMethod.OnLinkClickListener() {
                    @Override
                    public boolean onClick(TextView textView, String url) {
                        startActivity(new Intent(getActivity(), DeepLinkProcessActivity.class).putExtra("url", url));
                        return true;
                    }
                })
                .setOnLinkLongClickListener(new BetterLinkMovementMethod.OnLinkLongClickListener() {
                    @Override
                    public boolean onLongClick(TextView textView, String url) {
                        LogUtils.e("LONG URL::", url);
                        return true;
                    }
                });
        tvQuestion.setText(question);
    }


}
