package com.tiffintom.fragment;

import android.content.Intent;
import android.os.Bundle;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.LinearLayout;

import androidx.annotation.NonNull;
import androidx.annotation.Nullable;
import androidx.appcompat.app.AlertDialog;
import androidx.recyclerview.widget.LinearLayoutManager;
import androidx.recyclerview.widget.RecyclerView;
import androidx.swiperefreshlayout.widget.SwipeRefreshLayout;

import com.androidnetworking.error.ANError;
import com.androidnetworking.interfaces.JSONArrayRequestListener;
import com.androidnetworking.interfaces.JSONObjectRequestListener;
import com.google.gson.Gson;
import com.google.gson.reflect.TypeToken;
import com.tiffintom.R;
import com.tiffintom.activity.RestaurantDetailsActivity;
import com.tiffintom.adapters.FavouriteRestaurantAdapter;
import com.tiffintom.base.BaseFragment;
import com.tiffintom.common.CommonFunctions;
import com.tiffintom.models.ApiError;
import com.tiffintom.models.FavouriteRestaurant;
import com.tiffintom.models.Postcode;
import com.tiffintom.models.Review;
import com.tiffintom.models.User;
import com.tiffintom.network.ApiUtils;
import com.tiffintom.utils.LogUtils;
import com.tiffintom.utils.ToastUtils;

import org.json.JSONArray;
import org.json.JSONException;
import org.json.JSONObject;

import java.lang.reflect.Type;
import java.util.ArrayList;
import java.util.List;

import static com.facebook.FacebookSdk.getApplicationContext;
import static com.tiffintom.utils.Constants.CODE_REFRESH;

/**
 * Created by Amrish on 19-08-2020.
 */
public class FavouriteRestaurantsFragment extends BaseFragment {
    public static FavouriteRestaurantsFragment getInstance() {
        FavouriteRestaurantsFragment exploreCategoryFragment = new FavouriteRestaurantsFragment();
        Bundle bundle = new Bundle();
        exploreCategoryFragment.setArguments(bundle);
        return exploreCategoryFragment;
    }

    private RecyclerView rvRestaurants;
    private FavouriteRestaurantAdapter favouriteRestaurantAdapter;
    private ArrayList<FavouriteRestaurant> favouriteRestaurants = new ArrayList<>();
    private SwipeRefreshLayout swipeRefreshLayout;
    private User loggedInUser = myApp.getMyPreferences().getLoggedInUser();
    private LinearLayout llNoData;
    private Postcode currentPostcode = myApp.getMyPreferences().getCurrentPostcode();

    @Nullable
    @Override
    public View onCreateView(@NonNull LayoutInflater inflater, @Nullable ViewGroup container, @Nullable Bundle savedInstanceState) {
        return inflater.inflate(R.layout.fragment_favourite_restaurants, container, false);
    }

    @Override
    public void onViewCreated(@NonNull View view, @Nullable Bundle savedInstanceState) {
        super.onViewCreated(view, savedInstanceState);
        setListeners();
        fetchData();
    }

    @Override
    protected void initViews(View view) {
        super.initViews(view);
        rvRestaurants = view.findViewById(R.id.rvRestaurants);
        swipeRefreshLayout = view.findViewById(R.id.swipeRefreshLayout);
        llNoData = view.findViewById(R.id.llNoData);

        favouriteRestaurantAdapter = new FavouriteRestaurantAdapter(getActivity(), favouriteRestaurants, this::favouriteClick, this::restaurantItemClick);
        rvRestaurants.setLayoutManager(new LinearLayoutManager(getActivity(), RecyclerView.VERTICAL, false));
        rvRestaurants.setAdapter(favouriteRestaurantAdapter);
    }

    private void setListeners() {
        swipeRefreshLayout.setOnRefreshListener(this::fetchData);
    }

    private void favouriteClick(int i, Object o) {
        if (getActivity() != null) {
            getActivity().runOnUiThread(() -> progressDialog.show());
        }
        FavouriteRestaurant r = (FavouriteRestaurant) o;
        ApiUtils.getFavouriteAction(String.valueOf(loggedInUser.id), String.valueOf(r.restaurant.id), "0")
                .getAsJSONObject(new JSONObjectRequestListener() {
                    @Override
                    public void onResponse(JSONObject response) {
                        if (getActivity() != null) {
                            getActivity().runOnUiThread(() -> progressDialog.dismiss());
                        }

                        favouriteRestaurants.remove(i);
                        favouriteRestaurantAdapter.notifyItemRemoved(i);
                        ToastUtils.makeSnackToast(getActivity(), r.restaurant.restaurant_name + " has been removed from favourites");

                        favouriteRestaurantAdapter.notifyDataSetChanged();
                        if (favouriteRestaurants.size() == 0) {
                            getActivity().runOnUiThread(() -> llNoData.setVisibility(View.VISIBLE));
                        } else {
                            getActivity().runOnUiThread(() -> llNoData.setVisibility(View.GONE));
                        }
                    }

                    @Override
                    public void onError(ANError anError) {
                        anError.printStackTrace();
                        LogUtils.e("ERROR WHILE FAVORITE :: ", anError.getErrorBody());
                        favouriteRestaurants.clear();
                        favouriteRestaurantAdapter.notifyDataSetChanged();
                        ApiError apiError = anError.getErrorAsObject(ApiError.class);
                        ToastUtils.makeSnackToast(getContext(), apiError.getMessage());
                        if (getActivity() != null) {
                            getActivity().runOnUiThread(() -> progressDialog.dismiss());
                            getActivity().runOnUiThread(() -> llNoData.setVisibility(View.VISIBLE));
                        }
                        if (!CommonFunctions.isConnected(getActivity()))
                            myApp.noInternet(getActivity());
                    }
                });
    }

    int res_id = 0;

    private void restaurantItemClick(int position, Object data) {
        FavouriteRestaurant clickedRestaurant = (FavouriteRestaurant) data;
        Intent intent = new Intent(getActivity(), RestaurantDetailsActivity.class);
        intent.putExtra("resId", clickedRestaurant.restaurant.id);
        intent.putExtra("res_name", clickedRestaurant.restaurant.restaurant_name);
        startActivityForResult(intent, CODE_REFRESH);
    }


    private void fetchData() {
        new Thread(this::fetchFavouriteRestaurants).start();
    }

    private void fetchFavouriteRestaurants() {
        if (getActivity() != null) {
            getActivity().runOnUiThread(() -> {
                swipeRefreshLayout.setRefreshing(true);
                llNoData.setVisibility(View.GONE);
            });
        }

        ApiUtils.getFavouritesList(String.valueOf(loggedInUser.id), "1", "1")
                .getAsJSONArray(new JSONArrayRequestListener() {
                    @Override
                    public void onResponse(JSONArray response) {
                        favouriteRestaurants.clear();
                        Type type = new TypeToken<List<FavouriteRestaurant>>() {
                        }.getType();
                        ArrayList<FavouriteRestaurant> reviews = new ArrayList<>(new Gson().fromJson(response.toString(), type));
                        favouriteRestaurants.addAll(reviews);
                        if (getActivity() != null) {
                            getActivity().runOnUiThread(() -> {
                                favouriteRestaurantAdapter.notifyDataSetChanged();
                            });
                        }
                        if (favouriteRestaurants.size() == 0) {
                            getActivity().runOnUiThread(() -> llNoData.setVisibility(View.VISIBLE));
                        } else {
                            getActivity().runOnUiThread(() -> llNoData.setVisibility(View.GONE));
                        }
                        if (getActivity() != null) {
                            getActivity().runOnUiThread(() -> swipeRefreshLayout.setRefreshing(false));
                        }

                    }
                    @Override
                    public void onError(ANError anError) {
                        LogUtils.e("ERROR WHILE FETCHING FAVOURITE LIST:::", anError.getErrorBody());
                        favouriteRestaurants.clear();
                        favouriteRestaurantAdapter.notifyDataSetChanged();
                        if (anError.getErrorCode() == 400) {
                            try {
                                JSONObject jsonObject = new JSONObject(anError.getErrorBody().toString());
                                ToastUtils.makeToast(getApplicationContext(), jsonObject.getString("message"));
                            } catch (JSONException exception) {
                                exception.printStackTrace();
                            }
                        }

                        if (getActivity() != null) {
                            getActivity().runOnUiThread(() -> {
                                swipeRefreshLayout.setRefreshing(false);
                                llNoData.setVisibility(View.VISIBLE);
                            });
                        }
                        if (!CommonFunctions.isConnected(getActivity()))
                            myApp.noInternet(getActivity());
                    }
                });
    }

    @Override
    public void onResume() {
        super.onResume();
        new Thread(() -> {
            res_id = myApp.getAppDatabase().cartDao().getRestaurantId();
            fetchFavouriteRestaurants();
        }).start();
    }
}