package com.tiffintom.fragment;

import android.app.Activity;
import android.os.Bundle;
import android.text.Editable;
import android.text.TextWatcher;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.EditText;
import android.widget.RelativeLayout;
import android.widget.TextView;

import androidx.annotation.NonNull;
import androidx.annotation.Nullable;
import androidx.appcompat.app.AlertDialog;

import com.androidnetworking.AndroidNetworking;
import com.androidnetworking.error.ANError;
import com.androidnetworking.interfaces.JSONObjectRequestListener;
import com.tiffintom.R;
import com.tiffintom.base.BaseFragment;
import com.tiffintom.common.CommonFunctions;
import com.tiffintom.common.Validators;
import com.tiffintom.models.User;
import com.tiffintom.network.ApiEndPoints;
import com.tiffintom.utils.ToastUtils;

import org.json.JSONObject;

public class FeedbackFragment extends BaseFragment {


    private TextView tvWordcounter, tvSend;
    private EditText etFeedbackSubject, etFeedback;
    private User loggedInUser = myApp.getMyPreferences().getLoggedInUser();
    protected AlertDialog progressDialog;
    private TextView tvSubject;
    private RelativeLayout rlTextview, rlEditText;
    private String subjectData;

    public static FeedbackFragment getInstance() {
        return new FeedbackFragment();
    }

    @Nullable
    @Override
    public View onCreateView(@NonNull LayoutInflater inflater, @Nullable ViewGroup container, @Nullable Bundle savedInstanceState) {
        return inflater.inflate(R.layout.fragment_feedback, container, false);
    }

    @Override
    public void onViewCreated(@NonNull View view, @Nullable Bundle savedInstanceState) {
        super.onViewCreated(view, savedInstanceState);
        setListner();
    }

    private void setListner() {
        rlTextview.setOnClickListener(view -> {
            FeedbackSubjectBottomSheetFragment feedbackSubjectBottomSheetFragment = FeedbackSubjectBottomSheetFragment.getInstance(subjectData);
            feedbackSubjectBottomSheetFragment.show(getChildFragmentManager(), "feedback subject");
            feedbackSubjectBottomSheetFragment.setDialogDismissListener(this::getSubject);
        });

        etFeedback.addTextChangedListener(new TextWatcher() {
            @Override
            public void beforeTextChanged(CharSequence charSequence, int i, int i1, int i2) {
            }

            @Override
            public void onTextChanged(CharSequence charSequence, int i, int i1, int i2) {
            }

            @Override
            public void afterTextChanged(Editable editable) {

                int length = etFeedback.length();
                tvWordcounter.setText(length + "/300");
                if (length > 300) {
                    tvWordcounter.setTextColor(getResources().getColor(R.color.red));
                } else {
                    tvWordcounter.setTextColor(getResources().getColor(R.color.black));
                }
            }
        });
        tvSend.setOnClickListener(view1 -> {
            if (isValid()) {
                new Thread(this::sendFeedback).start();
            }
        });
    }

    private boolean isValid() {
        etFeedbackSubject.setError(null);
        etFeedback.setError(null);
        if (Validators.isNullOrEmpty(tvSubject.getText().toString())) {
            ToastUtils.makeToast(getActivity(), "Please Select feedback subject");
            return false;
        }
        if (!Validators.isNullOrEmpty(subjectData) && subjectData.equalsIgnoreCase("other") && Validators.isNullOrEmpty(etFeedbackSubject.getText().toString())) {
            etFeedbackSubject.setError("Please enter Feedback Subject");
            etFeedbackSubject.requestFocus();
            return false;
        }
        if (Validators.isNullOrEmpty(etFeedback.getText().toString())) {
            etFeedback.setError("Please enter your Feedback");
            etFeedback.requestFocus();
            return false;
        }
        if (etFeedback.getText().toString().length() > 300) {
            etFeedback.setError("Character should be less then 300.");
            return false;
        }
        return true;
    }

    private void getSubject(Object o) {
        subjectData = (String) o;
        if (subjectData.equalsIgnoreCase("other")) {
            tvSubject.setText(subjectData);
            rlEditText.setVisibility(View.VISIBLE);
            etFeedbackSubject.setText("");
            etFeedbackSubject.requestFocus();
        } else {
            rlEditText.setVisibility(View.GONE);
            etFeedbackSubject.setText(subjectData);
            tvSubject.setText(subjectData);
        }
    }

    @Override
    protected void initViews(View view) {
        super.initViews(view);

        etFeedbackSubject = view.findViewById(R.id.etFeedbackSubject);
        etFeedback = view.findViewById(R.id.etFeedbackProblem);
        tvWordcounter = view.findViewById(R.id.tvWordCounter);
        tvSend = view.findViewById(R.id.tvSend);
        tvSubject = view.findViewById(R.id.tvFeedbackSubject);
        rlTextview = view.findViewById(R.id.rlTextSubject);
        rlEditText = view.findViewById(R.id.rlEditextSubject);
        progressDialog = CommonFunctions.customProgressDialog(getActivity(), "Loading...");

    }

    private void sendFeedback() {
        if (getActivity() != null)
            getActivity().runOnUiThread(() -> progressDialog.show());
        AndroidNetworking.post(ApiEndPoints.NEW_SEND_FEEDBACK)
                .addBodyParameter("customer_id", String.valueOf(loggedInUser.id))
                .addBodyParameter("subject", etFeedbackSubject.getText().toString())
                .addBodyParameter("message", etFeedback.getText().toString())
                .build()
                .getAsJSONObject(new JSONObjectRequestListener() {
                    @Override
                    public void onResponse(JSONObject response) {
                        if (getActivity() != null)
                            getActivity().runOnUiThread(() -> {
                                progressDialog.dismiss();
                            });

                             getActivity().setResult(Activity.RESULT_OK, CommonFunctions.setMessageResult("Feedback posted successfully"));
                            getActivity().onBackPressed();
                            ToastUtils.makeSnackToast(getActivity(), "Feedback posted successfully");

                    }

                    @Override
                    public void onError(ANError anError) {
                        if (getActivity() != null)
                            progressDialog.dismiss();
                        ToastUtils.makeToast(getActivity(), "Something went wrong");
                        if (!CommonFunctions.isConnected(getActivity()))
                            myApp.noInternet(getActivity());
                    }
                });
    }
}
