package com.tiffintom.fragment;

import android.graphics.ColorFilter;
import android.graphics.PorterDuff;
import android.graphics.PorterDuffColorFilter;
import android.os.AsyncTask;
import android.os.Build;
import android.os.Bundle;
import android.view.KeyEvent;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.Button;
import android.widget.EditText;
import android.widget.ImageView;
import android.widget.LinearLayout;
import android.widget.TextView;

import androidx.annotation.NonNull;
import androidx.annotation.Nullable;
import androidx.cardview.widget.CardView;
import androidx.core.content.ContextCompat;
import androidx.recyclerview.widget.GridLayoutManager;
import androidx.recyclerview.widget.LinearLayoutManager;
import androidx.recyclerview.widget.RecyclerView;

import com.airbnb.lottie.LottieAnimationView;
import com.airbnb.lottie.LottieProperty;
import com.airbnb.lottie.model.KeyPath;
import com.airbnb.lottie.value.LottieFrameInfo;
import com.airbnb.lottie.value.SimpleLottieValueCallback;
import com.androidnetworking.error.ANError;
import com.androidnetworking.interfaces.JSONArrayRequestListener;
import com.bumptech.glide.Glide;
import com.google.android.material.bottomsheet.BottomSheetDialogFragment;
import com.google.gson.Gson;
import com.google.gson.reflect.TypeToken;
import com.tiffintom.MyApp;
import com.tiffintom.R;
import com.tiffintom.adapters.DineInIngredientsAdapter;
import com.tiffintom.adapters.DineinAddonsAdapter;
import com.tiffintom.common.CommonFunctions;
import com.tiffintom.common.Validators;
import com.tiffintom.interfaces.DialogDismissListenerWithArray;
import com.tiffintom.interfaces.MoneyTextWatcher;
import com.tiffintom.models.DineinModels.Addon;
import com.tiffintom.models.DineinModels.DineinMenus;
import com.tiffintom.models.DineinModels.OrderItem;
import com.tiffintom.models.DineinModels.OrderItemAddon;
import com.tiffintom.models.DineinModels.OrderItemIngredient;
import com.tiffintom.models.DineinModels.Product;
import com.tiffintom.models.DineinModels.ProductAddons;
import com.tiffintom.models.DineinModels.ProductIngredients;
import com.tiffintom.network.ApiUtils;
import com.tiffintom.utils.LogUtils;
import com.tiffintom.utils.ToastUtils;

import org.json.JSONArray;

import java.io.File;
import java.lang.reflect.Type;
import java.util.ArrayList;
import java.util.List;
import java.util.concurrent.Callable;

public class FragmentMenuAddonIngrediants extends BottomSheetDialogFragment {

    private LinearLayout llLoadingView, llData, llLoading, llInstructions, llIngredientsView, llAddonsView, llIngredientsSelect, llAddonsSelect;
    private TextView tvMessage, tvIngredientsSelect, tvAddonsSelect;
    private View vIngredientsSelect, vAddonsSelect;

    private LottieAnimationView lottieAnimation;
    private ImageView ivProduct;
    private TextView tvProductName, tvPrice, tvQuantityModified;
    private ImageView ivMinusModify, ivPlusModify;

    private Button btnConfirm, btnCancel;
    private RecyclerView rvProductIngredients, rvProductAddons;
    private CardView cvAddSpecialInstruction;
    private EditText etSpecialInstruction, etSpecialInstructionPrice;
    private Button btnRemoveInstructions, btnConfirmInstructions;

    private ArrayList<ProductAddons> productAddons = new ArrayList<>();
    private ArrayList<ProductIngredients> productIngredients = new ArrayList<>();
    private DineinAddonsAdapter addonsAdapter;
    private DineInIngredientsAdapter ingredientsAdapter;
    private OrderItem orderItem, originalItem;
    private DineinMenus product;
    private String business_id, orderTypeId;
    private boolean showMenuImage = false;
    private int modifiedQuantity = 1;
    private MyApp myApp;

    private ArrayList<OrderItem> toBeSavedOrderItems = new ArrayList<>();
    private DialogDismissListenerWithArray dialogDismissListener;

    public void setDialogDismissListener(DialogDismissListenerWithArray dialogDismissListener) {
        this.dialogDismissListener = dialogDismissListener;
    }

    public static FragmentMenuAddonIngrediants getInstance(DineinMenus product, OrderItem orderItem, boolean showMenuImage, String business_id) {
        FragmentMenuAddonIngrediants addonIngrediants = new FragmentMenuAddonIngrediants();
        Bundle bundle = new Bundle();
        bundle.putString("product", new Gson().toJson(product));
        bundle.putString("orderItem", new Gson().toJson(orderItem));
        bundle.putString("business_id", business_id);
        bundle.putBoolean("showMenuImage", showMenuImage);
        addonIngrediants.setArguments(bundle);
        return addonIngrediants;
    }

    @Override
    public int getTheme() {
        if (Build.VERSION.SDK_INT > 21) {
            return R.style.BottomSheetDialogTheme;
        } else {
            return super.getTheme();
        }
    }

    @Nullable
    @Override
    public View onCreateView(@NonNull LayoutInflater inflater, @Nullable ViewGroup container, @Nullable Bundle savedInstanceState) {
        if (getArguments() != null) {
            product = new Gson().fromJson(getArguments().getString("product"), DineinMenus.class);
            orderItem = new Gson().fromJson(getArguments().getString("orderItem"), OrderItem.class);
            originalItem = new Gson().fromJson(getArguments().getString("orderItem"), OrderItem.class);
            business_id = getArguments().getString("business_id");
            showMenuImage = getArguments().getBoolean("showMenuImage");

        }
        return inflater.inflate(R.layout.fragment_menuaddon_ingrediant_bottomfragment, container, false);
    }

    @Override
    public void onViewCreated(@NonNull View view, @Nullable Bundle savedInstanceState) {
        super.onViewCreated(view, savedInstanceState);
        initViews(view);
        setListeners();
    }

    private void setListeners() {
        etSpecialInstructionPrice.addTextChangedListener(new MoneyTextWatcher(etSpecialInstructionPrice));

        llIngredientsSelect.setOnClickListener(v -> {
            try {
                changeColors(llIngredientsSelect);
                llAddonsView.setVisibility(View.GONE);
                if (productIngredients.size() == 0) {
                    showError("There are no ingredients available for\n selected product to modify");
                    llIngredientsView.setVisibility(View.GONE);
                } else {
                    llIngredientsView.setVisibility(View.VISIBLE);
                    llLoading.setVisibility(View.GONE);
                }
            } catch (Exception e) {
                e.printStackTrace();
            }
        });

        llAddonsSelect.setOnClickListener(v -> {
            try {
                changeColors(llAddonsSelect);
                llIngredientsView.setVisibility(View.GONE);
                if (productAddons.size() == 0) {
                    llAddonsView.setVisibility(View.GONE);
                    showError("There are no options available for\n selected product to apply");
                } else {
                    llLoading.setVisibility(View.GONE);
                    llAddonsView.setVisibility(View.VISIBLE);
                }
            } catch (Exception e) {
                e.printStackTrace();
            }
        });
        etSpecialInstruction.setOnKeyListener((textView, i, keyEvent) -> {
            try {
                if ((keyEvent.getKeyCode() == KeyEvent.KEYCODE_ENTER)) {
                    CommonFunctions.hideKeyboard(getActivity(), etSpecialInstruction);
                    btnConfirmInstructions.performClick();
                }
            } catch (Exception e) {
            }
            return false;
        });
        btnConfirmInstructions.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                try {
                    if (!Validators.isNullOrEmpty(etSpecialInstruction.getText().toString())) {
                        product.special_instruction = etSpecialInstruction.getText().toString();
                        if (Validators.isNullOrEmpty(etSpecialInstructionPrice.getText().toString())) {
                            product.instruction_price = 0;
                        } else {
                            product.instruction_price = Float.parseFloat(etSpecialInstruction.getText().toString());
                        }
                    }
                    updateViews();
                } catch (Exception e) {
                }
            }
        });

        btnRemoveInstructions.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                try {
                    CommonFunctions.hideKeyboard(getActivity(), etSpecialInstruction);
                    etSpecialInstruction.setText(null);
                    etSpecialInstructionPrice.setText("0.00");
                    llInstructions.setVisibility(View.GONE);
                    cvAddSpecialInstruction.setVisibility(View.VISIBLE);
                    product.special_instruction = null;
                    product.instruction_price = 0;
                } catch (Exception e) {
                }
            }
        });
        cvAddSpecialInstruction.setOnClickListener(v -> {
            try {
                llInstructions.setVisibility(View.VISIBLE);
                cvAddSpecialInstruction.setVisibility(View.GONE);
                etSpecialInstruction.requestFocus();
                etSpecialInstruction.setSelection(etSpecialInstruction.getText().toString().length());
                if (Validators.isNullOrEmpty(product.special_instruction)) {
                    btnConfirmInstructions.setVisibility(View.GONE);
                    btnRemoveInstructions.setVisibility(View.GONE);
                } else {
                    btnRemoveInstructions.setVisibility(View.VISIBLE);
                    btnConfirmInstructions.setVisibility(View.GONE);
                }
            } catch (Exception e) {

            }
        });
        etSpecialInstruction.setOnKeyListener((textView, i, keyEvent) -> {
            try {
                if ((keyEvent.getKeyCode() == KeyEvent.KEYCODE_ENTER)) {
                    CommonFunctions.hideKeyboard(getActivity(), etSpecialInstruction);
                    btnConfirmInstructions.performClick();
                }
            } catch (Exception e) {
            }
            return false;
        });
        ivPlusModify.setOnClickListener(v -> {
            try {
                modifiedQuantity++;
                tvQuantityModified.setText("" + modifiedQuantity);
            } catch (Exception e) {
            }
        });
        ivMinusModify.setOnClickListener(v -> {
            try {
                if (modifiedQuantity > 1) {
                    modifiedQuantity--;
                }
                tvQuantityModified.setText("" + modifiedQuantity);
            } catch (Exception e) {
            }
        });

        btnConfirm.setOnClickListener(v -> {
            try {
                if (validateItems()) {
                    new SaveOrderItemAsync(() -> {
                        if (dialogDismissListener != null)
                            dialogDismissListener.onDialogDismiss(toBeSavedOrderItems);//send saved order
                        dismiss();
                        return null;
                    }).executeOnExecutor(AsyncTask.THREAD_POOL_EXECUTOR);
                }
            } catch (Exception e) {
                e.printStackTrace();
            }
        });

        btnCancel.setOnClickListener(v -> {
            try {
                if (getActivity() != null) if (dialogDismissListener != null) {
                    dialogDismissListener.onDialogDismiss(null);
                }
            } catch (Exception e) {
            }
        });
    }

    private boolean validateItems() {
        // First validate addons
        boolean itemModified = true;
        ArrayList<OrderItem> savingOrderItems = new ArrayList<>();

        int position = -1;
        ArrayList<OrderItemAddon> allSelectedAddons = new ArrayList<>();
        for (ProductAddons productAddon : productAddons) {
            position++;
            if (productAddon.selectedQuantity == 0 && productAddon.min_quantity == 0) continue;
            if (productAddon.selectedQuantity >= productAddon.min_quantity && productAddon.selectedQuantity <= productAddon.max_quantity) {
                OrderItemAddon orderItemAddon = new OrderItemAddon();
                orderItemAddon.addon_id = productAddon.addon.id;
                orderItemAddon.addon_name = productAddon.addon.name;
                orderItemAddon.price = productAddon.addon.selected_price;
                orderItemAddon.updater_id = MyApp.getInstance().getMyPreferences().getDineinUser().id;
                orderItemAddon.quantity = 1;
                orderItemAddon.total = orderItemAddon.price * orderItemAddon.quantity;
                orderItemAddon.category = productAddon.category;
                allSelectedAddons.add(orderItemAddon);
                allSelectedAddons.addAll(productAddon.selectedOrderItemAddon);
            } else {

                addonsAdapter.expandedPosition = position;
                String message = "";
                if (productAddon.addon != null)
                    message = "Please select " + productAddon.addon.name;
                else message = "Please select addon";
                addonsAdapter.notifyDataSetChanged();
                LogUtils.e("Confirmation pending");
                ToastUtils.makeToast(getActivity(), message);
                return false;
            }

        }
        ArrayList<OrderItemIngredient> allSelectedIngredients = new ArrayList<>();
        for (ProductIngredients productIngredient : productIngredients) {
            if (productIngredient.quantity > 0) {
                OrderItemIngredient oitIngredient = new OrderItemIngredient();
                oitIngredient.product_ingredient_id = String.valueOf(productIngredient.id);
                oitIngredient.order_id = String.valueOf(orderItem.id);
                oitIngredient.updater_id = MyApp.getInstance().getMyPreferences().getDineinUser().id;
                oitIngredient.ingredient_id = String.valueOf(productIngredient.ingredient_id);
                oitIngredient.ingredient_name = productIngredient.ingredient_name;
                if (productIngredient.isAlreadyRemoved) {
                    oitIngredient.without = true;
                    oitIngredient.with = false;
                } else {
                    oitIngredient.without = false;
                    oitIngredient.with = true;
                }
                oitIngredient.quantity = productIngredient.quantity;
                oitIngredient.price = productIngredient.price;
                oitIngredient.total = oitIngredient.price * oitIngredient.quantity;
                allSelectedIngredients.add(oitIngredient);
            }

        }

        if (product != null) {
            orderItem.product_name = product.name;
            orderItem.product_id = String.valueOf(product.id);
            if (!orderItem.misc)
                orderItem.price = MyApp.calculatedProductPrice(orderTypeId, product);
        }
        orderItem.quantity = modifiedQuantity;
        if (!Validators.isNullOrEmpty(etSpecialInstructionPrice.getText().toString())) {
            orderItem.instruction_price = Float.parseFloat(etSpecialInstructionPrice.getText().toString());
        } else {
            orderItem.instruction_price = 0;
        }
        orderItem.special_instruction = etSpecialInstruction.getText().toString();

        orderItem.sub_total = (orderItem.quantity * orderItem.price);
        orderItem.order_item_addons = allSelectedAddons;
        orderItem.addons_price = getAddonPrice(orderItem);
        orderItem.order_item_ingredients = allSelectedIngredients;
        orderItem.ingredients_price = getIngredientPrice(orderItem);
        orderItem.total = orderItem.sub_total + (orderItem.addons_price * orderItem.quantity) + (orderItem.ingredients_price * orderItem.quantity) + (orderItem.instruction_price * orderItem.quantity);
        orderItem.updater_id = String.valueOf(MyApp.getInstance().getMyPreferences().getDineinUser().id);
        savingOrderItems.add(orderItem);

        toBeSavedOrderItems.clear();
        toBeSavedOrderItems.addAll(savingOrderItems);

        return true;
    }

    private float getAddonPrice(OrderItem orderItem) {
        float addonPrice = 0;
        for (OrderItemAddon orderItemAdd : orderItem.order_item_addons) {
            addonPrice = addonPrice + orderItemAdd.total;
        }
        return addonPrice;
    }

    private float getIngredientPrice(OrderItem orderItem) {
        float ingredientPrice = 0;
        for (OrderItemIngredient orderItemIngredient :
                orderItem.order_item_ingredients) {
            ingredientPrice = ingredientPrice + orderItemIngredient.total;
        }
        return ingredientPrice;
    }

    private void showError(String s) {
        llLoading.setVisibility(View.VISIBLE);
        tvMessage.setVisibility(View.VISIBLE);
        tvMessage.setText(s);
    }

    private class ProductDetailsAsync extends AsyncTask<String, String, String> {
        private ArrayList<ProductAddons> localAddons = new ArrayList<>();
        private ArrayList<ProductIngredients> localIngredients = new ArrayList<>();

        @Override
        protected String doInBackground(String... strings) {
            try {
                /*if (product.id > 0) {
                    orderItem = myApp.getAppDatabase().orderItemDao().view(orderItem._id);
                }*/

               /* try {
                    if (Validators.isNullOrEmpty(orderItem.product_id)) {
                        product = new DineinMenus();
                        product.name = orderItem.product_name;
                        product.misc = true;
                        product.delivery_price = orderItem.delivery_price;
                        product.waiting_price = orderItem.waiting_price;
                        product.web_price = orderItem.web_price;
                       // product.selected_price = orderItem.price;
                        product.takeaway_price = orderItem.takeaway_price;
                    } else {
                        product = myApp.getAppDatabase().dinein().view(orderItem.product_id);
                    }
                } catch (Exception e) {
                    e.printStackTrace();
                }*/

                if (product == null) return null;
                /*localAddons.addAll(myApp.getAppDatabase().productAddonDao().listByProduct(String.valueOf(product.id)));
                localIngredients.addAll(myApp.getAppDatabase().productIngredientDao().listByProduct(String.valueOf(product.id)));
*/
                for (ProductAddons productAddon : productAddons) {
                    int selectedAddonQuantity = 0;
                   // productAddon.addon = myApp.getAppDatabase().addonDao().view(productAddon.addon_id);
                   // productAddon.addon.children = new ArrayList<>();
                  //  productAddon.addon.children.addAll(myApp.getAppDatabase().addonDao().list(productAddon.addon.id));
                    if (orderItem._id > 0) {
                        OrderItemAddon dbAddon = myApp.getAppDatabase().orderItemAddonDao().find(orderItem._id, productAddon.addon_id);
                        if (dbAddon != null) {
                            productAddon.addon.selected = true;
                            //                    productAddon.selectedOrderItemAddon.add(dbAddon);
                        }
                    }

                    if (productAddon.addon.children != null && productAddon.addon.children.size() > 0) {
                        for (Addon addonL1 : productAddon.addon.children) {
                            if (orderItem._id > 0) {
                                OrderItemAddon dbAddonL1 = myApp.getAppDatabase().orderItemAddonDao().find(orderItem._id, addonL1.id);
                                if (dbAddonL1 != null) {
                                    selectedAddonQuantity++;
                                    addonL1.selected = true;
                                    productAddon.selectedOrderItemAddon.add(dbAddonL1);
                                }
                            }
                           // addonL1.children = new ArrayList<>();
                           // addonL1.children.addAll(myApp.getAppDatabase().addonDao().list(addonL1.id));
                            if (addonL1.children != null && addonL1.children.size() > 0) {
                                for (Addon addonL2 : addonL1.children) {
                                    if (orderItem._id > 0) {
                                        OrderItemAddon dbAddonL2 = myApp.getAppDatabase().orderItemAddonDao().find(orderItem._id, addonL2.id);
                                        if (dbAddonL2 != null) {
                                            selectedAddonQuantity++;
                                            addonL2.selected = true;
                                            productAddon.selectedOrderItemAddon.add(dbAddonL2);
                                        }
                                    }
                                   // addonL2.children = new ArrayList<>();
                                  //  addonL2.children.addAll(myApp.getAppDatabase().addonDao().list(addonL2.id));
                                }
                            }
                        }
                    }
                    productAddon.selectedQuantity = selectedAddonQuantity;
                    //LogUtils.e(productAddon);
                }
                for (ProductIngredients productIngredient : productIngredients) {
                    if (orderItem._id > 0) {
                        OrderItemIngredient dbOitIngredient = myApp.getAppDatabase().orderItemIngredientDao().find(orderItem._id, String.valueOf(productIngredient.id));
                        if (dbOitIngredient != null) {
                            productIngredient.isAlreadyRemoved = dbOitIngredient.without;
                            productIngredient.quantity = dbOitIngredient.quantity;
                        }
                    }
                }
            } catch (Exception e) {
                e.printStackTrace();
            }

            return null;
        }

        @Override
        protected void onPostExecute(String s) {
            super.onPostExecute(s);
            try {
                if (product != null) {
                    // TODO
                   /* if (myApp.productImageEnabled() && !Validators.isNullOrEmpty(product.image)) {
                        ivProduct.setVisibility(View.VISIBLE);
                        Glide.with(getActivity())
                                .load(new File(getActivity().getExternalFilesDir("epos2021").getAbsolutePath()) + "/" + product.id + ".png")
                                .into(ivProduct);
                    } else {
                        ivProduct.setVisibility(View.GONE);
                    }*/

                    if (Validators.isNullOrEmpty(orderItem.special_instruction)) {
                        cvAddSpecialInstruction.setVisibility(View.VISIBLE);
                    } else {
                        llInstructions.setVisibility(View.VISIBLE);
                        etSpecialInstruction.setText(orderItem.special_instruction);
                        etSpecialInstructionPrice.setText(MyApp.df.format(orderItem.instruction_price));
                        cvAddSpecialInstruction.setVisibility(View.GONE);
                    }

                   /* productAddons.clear();
                    productAddons.addAll(localAddons);*/
                    addonsAdapter.notifyDataSetChanged();

                 /*   productIngredients.clear();
                    productIngredients.addAll(localIngredients);*/
                    ingredientsAdapter.notifyDataSetChanged();

                    /*if (orderItem._id == 0) {
                        if (!product.auto_addon && !product.auto_modify) {
                            llIngredientsSelect.setVisibility(View.VISIBLE);
                            llAddonsSelect.setVisibility(View.GONE);
                            llIngredientsSelect.performClick();
                        } else if (product.auto_addon) {
                            llAddonsSelect.setVisibility(View.VISIBLE);
                            llAddonsSelect.performClick();
                            if (product.auto_modify) {
                                llIngredientsSelect.setVisibility(View.VISIBLE);
                            } else {
                                llIngredientsSelect.setVisibility(View.GONE);
                            }
                        } else if (product.auto_modify) {
                            llIngredientsSelect.setVisibility(View.VISIBLE);
                            llIngredientsSelect.performClick();
                            llAddonsSelect.setVisibility(View.GONE);
                        }
                    } else
                    {*/
                        if (productAddons.size() == 0 && productIngredients.size() == 0) {
                            llIngredientsSelect.setVisibility(View.VISIBLE);
                            llAddonsSelect.setVisibility(View.GONE);
                            llIngredientsSelect.performClick();
                        } else if (productAddons.size() > 0) {
                            llAddonsSelect.setVisibility(View.VISIBLE);
                            llAddonsSelect.performClick();
                            if (productIngredients.size() > 0) {
                                llIngredientsSelect.setVisibility(View.VISIBLE);
                            } else {
                                llIngredientsSelect.setVisibility(View.GONE);
                            }
                        } else if (productIngredients.size() > 0) {
                            llIngredientsSelect.setVisibility(View.VISIBLE);
                            llIngredientsSelect.performClick();
                            llAddonsSelect.setVisibility(View.GONE);
                        }
                   // }
                }
            } catch (Exception e) {
                e.printStackTrace();
            }
        }
    }

    private void updateViews() {
        if (orderItem != null) {
            tvProductName.setText(orderItem.product_name);
            tvPrice.setText(MyApp.getInstance().getCurrencySymbol() + MyApp.df.format(orderItem.price));
            etSpecialInstruction.setText(orderItem.special_instruction);
            etSpecialInstruction.setSelection(etSpecialInstruction.getText().toString().length());
            etSpecialInstructionPrice.setText(MyApp.df.format(orderItem.instruction_price));
            if (!Validators.isNullOrEmpty(etSpecialInstruction.getText().toString())) {
                llInstructions.setVisibility(View.VISIBLE);
                btnRemoveInstructions.setVisibility(View.VISIBLE);
                cvAddSpecialInstruction.setVisibility(View.GONE);
            } else {
                btnRemoveInstructions.setVisibility(View.GONE);
                cvAddSpecialInstruction.setVisibility(View.VISIBLE);
                llInstructions.setVisibility(View.GONE);
            }
            btnConfirmInstructions.setVisibility(View.GONE);
        }

        if (productAddons.size() > 0 && productIngredients.size() > 0) {
            llAddonsSelect.performClick();
            llAddonsSelect.setVisibility(View.VISIBLE);
            llIngredientsSelect.setVisibility(View.VISIBLE);
        } else {
            if (productAddons.size() == 0 && productIngredients.size() == 0) {
                llIngredientsSelect.setVisibility(View.VISIBLE);
                llAddonsSelect.setVisibility(View.GONE);
                llIngredientsSelect.performClick();
            } else if (productAddons.size() > 0) {
                llAddonsSelect.setVisibility(View.VISIBLE);
                llAddonsSelect.performClick();
                if (productIngredients.size() > 0) {
                    llIngredientsSelect.setVisibility(View.VISIBLE);
                } else {
                    llIngredientsSelect.setVisibility(View.GONE);
                }
            } else if (productIngredients.size() > 0) {
                llIngredientsSelect.setVisibility(View.VISIBLE);
                llIngredientsSelect.performClick();
                llAddonsSelect.setVisibility(View.GONE);
            }
        }
    }

    private void changeColors(View v) {
        vIngredientsSelect.setBackgroundResource(R.color.light_grey);
        tvIngredientsSelect.setAlpha(0.5f);

        vAddonsSelect.setBackgroundResource(R.color.light_grey);
        tvAddonsSelect.setAlpha(0.5f);

        if (v == llIngredientsSelect) {
            vIngredientsSelect.setBackgroundResource(R.color.colorPrimary);
            tvIngredientsSelect.setAlpha(1f);
        }
        if (v == llAddonsSelect) {
            vAddonsSelect.setBackgroundResource(R.color.colorPrimary);
            tvAddonsSelect.setAlpha(1f);
        }
    }

    private void initViews(View view) {
        myApp = MyApp.getInstance();
        llLoadingView = view.findViewById(R.id.llLoadingView);
        lottieAnimation = view.findViewById(R.id.lodingView);
        llData = view.findViewById(R.id.llData);
        llLoading = view.findViewById(R.id.llLoading);
        tvMessage = view.findViewById(R.id.tvMessage);

        llIngredientsSelect = view.findViewById(R.id.llIngredientsSelect);
        tvIngredientsSelect = view.findViewById(R.id.tvIngredientsSelect);
        vIngredientsSelect = view.findViewById(R.id.vIngredientsSelect);

        llAddonsSelect = view.findViewById(R.id.llAddonsSelect);
        tvAddonsSelect = view.findViewById(R.id.tvAddonsSelect);
        vAddonsSelect = view.findViewById(R.id.vAddonsSelect);

        tvProductName = view.findViewById(R.id.tvProductName);
        tvPrice = view.findViewById(R.id.tvPrice);
        ivProduct = view.findViewById(R.id.ivProduct);


        tvQuantityModified = view.findViewById(R.id.tvModifiedQuantity);
        ivPlusModify = view.findViewById(R.id.ivPlusModified);
        ivMinusModify = view.findViewById(R.id.ivMinusModified);

        etSpecialInstruction = view.findViewById(R.id.etSpecialInstruction);
        etSpecialInstructionPrice = view.findViewById(R.id.etSpecialInstructionPrice);
        cvAddSpecialInstruction = view.findViewById(R.id.cvAddSpecialInstruction);
        llInstructions = view.findViewById(R.id.llInstructions);
        btnRemoveInstructions = view.findViewById(R.id.btnRemoveInstruction);
        btnConfirmInstructions = view.findViewById(R.id.btnConfirmInstruction);

        llIngredientsView = view.findViewById(R.id.llIngredientsView);
        llAddonsView = view.findViewById(R.id.llAddonsView);

        rvProductIngredients = view.findViewById(R.id.rvProductIngredients);
        rvProductAddons = view.findViewById(R.id.rvProductAddons);

        btnCancel = view.findViewById(R.id.btnCancel);
        btnConfirm = view.findViewById(R.id.btnConfirm);


        lottieAnimation.addValueCallback(
                new KeyPath("**"),
                LottieProperty.COLOR_FILTER,
                new SimpleLottieValueCallback<ColorFilter>() {
                    @Override
                    public ColorFilter getValue(LottieFrameInfo<ColorFilter> frameInfo) {
                        return new PorterDuffColorFilter(ContextCompat.getColor(getActivity(), R.color.colorPrimary), PorterDuff.Mode.SRC_ATOP);
                    }
                }
        );

        addonsAdapter = new DineinAddonsAdapter(getActivity(), productAddons, orderTypeId, (position, data) -> {

        }, (position, data) -> {
            //Sub Addon Clicked
        });
        rvProductAddons.setLayoutManager(new LinearLayoutManager(getActivity(), RecyclerView.VERTICAL, false));
        rvProductAddons.setAdapter(addonsAdapter);

        ingredientsAdapter = new DineInIngredientsAdapter(productIngredients, (position, data) -> {
            ProductIngredients productIngredient = (ProductIngredients) data;

        }, (position, data) -> {
            ProductIngredients productIngredient = (ProductIngredients) data;

        });
        rvProductIngredients.setLayoutManager(new GridLayoutManager(getActivity(), 3));
        rvProductIngredients.setAdapter(ingredientsAdapter);


        if (showMenuImage) {
            ivProduct.setVisibility(View.VISIBLE);
        } else
            ivProduct.setVisibility(View.GONE);
        fetchAddons();

    }

    private void fetchAddons() {
        if (getActivity() != null)
            getActivity().runOnUiThread(() -> {
                llLoadingView.setVisibility(View.VISIBLE);
                llData.setVisibility(View.GONE);
            });
        ApiUtils.getDineInEposAddons(product.id, "0", "1", business_id)
                .getAsJSONArray(new JSONArrayRequestListener() {
                    @Override
                    public void onResponse(JSONArray response) {
                        if (getActivity() != null)
                            getActivity().runOnUiThread(() -> {
                                llLoadingView.setVisibility(View.GONE);
                                llData.setVisibility(View.VISIBLE);
                            });
                        productAddons.clear();
                        Type type = new TypeToken<List<ProductAddons>>() {
                        }.getType();
                        productAddons.addAll(new Gson().fromJson(response.toString(), type));

                        fetchIngredients();
                        addonsAdapter.notifyDataSetChanged();

                    }

                    @Override
                    public void onError(ANError anError) {
                        anError.printStackTrace();
                        fetchIngredients();
//                        if (getActivity() != null)
//                            getActivity().runOnUiThread(() -> {
//                                llLoadingView.setVisibility(View.GONE);
//                                llData.setVisibility(View.VISIBLE);
//                                llLoading.setVisibility(View.VISIBLE);
//                            });

                    }
                });
    }

    private void fetchIngredients() {
        if (getActivity() != null)
            getActivity().runOnUiThread(() -> {
                llLoadingView.setVisibility(View.VISIBLE);
                llData.setVisibility(View.GONE);
            });
        ApiUtils.getDineInEposIngredients(product.id, "0", "1", business_id)
                .getAsJSONArray(new JSONArrayRequestListener() {
                    @Override
                    public void onResponse(JSONArray response) {
                        if (getActivity() != null)
                            getActivity().runOnUiThread(() -> {
                                llLoadingView.setVisibility(View.GONE);
                                llData.setVisibility(View.VISIBLE);
                            });
                        Type type = new TypeToken<List<ProductIngredients>>() {
                        }.getType();
                        productIngredients.addAll(new Gson().fromJson(response.toString(), type));
                        ingredientsAdapter.notifyDataSetChanged();
                        updateViews();
                     //   new ProductDetailsAsync().executeOnExecutor(AsyncTask.THREAD_POOL_EXECUTOR);
                    }

                    @Override
                    public void onError(ANError anError) {
                        anError.printStackTrace();
                        updateViews();
                     //   new ProductDetailsAsync().executeOnExecutor(AsyncTask.THREAD_POOL_EXECUTOR);
                        if (getActivity() != null)
                            getActivity().runOnUiThread(() -> {
                                llLoadingView.setVisibility(View.GONE);
                                llData.setVisibility(View.VISIBLE);
                                if (productAddons.size() == 0 && productIngredients.size() == 0)
                                    llLoading.setVisibility(View.VISIBLE);
                            });

                    }
                });
    }

    private class SaveOrderItemAsync extends AsyncTask<OrderItem, String, String> {
        Callable<Void> nextThing;

        public SaveOrderItemAsync(Callable<Void> nextThing) {
            this.nextThing = nextThing;
        }

        @Override
        protected void onPreExecute() {
            super.onPreExecute();
        }

        @Override
        protected String doInBackground(OrderItem... orderItems) {
            ArrayList<OrderItem> orderItemDao = new ArrayList<>();
            for (OrderItem oit : toBeSavedOrderItems) {
                orderItemDao.addAll(myApp.getAppDatabase().orderItemDao().list(oit.order_id));

                LogUtils.e("orderitem data::" + new Gson().toJson(orderItemDao));
                if (orderItemDao.size() == 0) {
                    myApp.getAppDatabase().orderItemDao().insert(oit);
                } else {
                    if (oit.order_item_addons.size() == 0 && oit.order_item_ingredients.size() == 0) {
                        for (OrderItem dao : orderItemDao) {
                            dao.quantity++;
                            dao.total = dao.quantity * (dao.price + dao.addons_price + dao.instruction_price + dao.ingredients_price);
                            myApp.getAppDatabase().orderItemDao().update(dao);
                        }
                    } else {

                        ArrayList<OrderItem> selectedItem = new ArrayList<>();
                        ArrayList<OrderItemAddon> selectedAddon = new ArrayList<>();
                        try {
                            if (oit.order_item_addons.size() > 0) {
                                if (orderItemDao.get(0).order_item_addons.size() > 0) {
                                    for (OrderItemAddon orderItemAddon : orderItemDao.get(0).order_item_addons) {
                                        if (orderItemDao.get(0).order_item_addons.equals(oit.order_item_addons)) {
                                            selectedAddon.add(orderItemAddon);
                                        }

/*
                                        for (OrderItemAddon oitAddon : oit.order_item_addons) {
                                            //match addon_id
                                        }
*/
                                    }

                                    selectedItem.addAll(orderItemDao);
                                    selectedItem.get(0).order_item_addons.clear();
                                    selectedItem.get(0).order_item_addons.addAll(selectedAddon);
                                    boolean shouldUpdate = false;
                                    if (selectedAddon.size() > 0) {
                                        ArrayList<OrderItemAddon> orderItemAddons = new ArrayList<>();
                                        OrderItem daoItem = new OrderItem();
                                        for (OrderItem dao : selectedItem) {
                                            for (OrderItemAddon itemAddon : selectedAddon) {
                                                if (Validators.isNullOrEmpty(itemAddon.category)) {
                                                    itemAddon.quantity++;
                                                    shouldUpdate = true;
                                                } else {
                                                    shouldUpdate = false;
                                                }
                                                itemAddon.total = itemAddon.quantity * (oit.addons_price);
                                                orderItemAddons.add(itemAddon);
                                            }
                                            daoItem = dao;
                                            daoItem.order_item_addons.clear();
                                            daoItem.order_item_addons.addAll(orderItemAddons);
                                            if (shouldUpdate)
                                                MyApp.getInstance().getAppDatabase().orderItemDao().update(daoItem);
                                            else
                                                for (OrderItem dao1 : orderItemDao) {
                                                    dao1.order_item_addons.addAll(oit.order_item_addons);
                                                    myApp.getAppDatabase().orderItemDao().insert(dao1);
                                                }
                                        }

                                    } else {
                                        for (OrderItem dao : orderItemDao) {
                                            dao.order_item_addons.addAll(oit.order_item_addons);
                                            myApp.getAppDatabase().orderItemDao().insert(dao);
                                        }
                                    }
                                } else {
                                    for (OrderItem dao : orderItemDao) {
                                        dao.order_item_addons.addAll(oit.order_item_addons);
                                        myApp.getAppDatabase().orderItemDao().insert(dao);
                                    }
                                }
                            }

                        } catch (Exception e) {
                            e.printStackTrace();
                        }

                        try {
                            ArrayList<OrderItemIngredient> selectedIngredient = new ArrayList<>();
                            if (oit.order_item_ingredients.size() > 0) {
                                if (orderItemDao.get(0).order_item_ingredients.size() > 0) {
                                    if ((orderItemDao.get(0).order_item_ingredients).equals(oit.order_item_ingredients)) {
                                        // orderItemIngredient.selected_quantity = ingredient.quantity;
                                        selectedIngredient.addAll(orderItemDao.get(0).order_item_ingredients);
                                    }


                                    selectedItem.addAll(orderItemDao);
                                    selectedItem.get(0).order_item_ingredients.clear();
                                    selectedItem.get(0).order_item_ingredients.addAll(selectedIngredient);
                                    if (selectedIngredient.size() > 0) {
                                        ArrayList<OrderItemIngredient> orderItemIngredient = new ArrayList<>();
                                        OrderItem daoItem = new OrderItem();
                                        for (OrderItem dao : selectedItem) {
                                            for (OrderItemIngredient itemAddon : selectedIngredient) {
                                                itemAddon.total = itemAddon.quantity * (oit.ingredients_price);
                                                orderItemIngredient.add(itemAddon);
                                            }
                                            daoItem = dao;
                                            daoItem.order_item_ingredients.clear();
                                            daoItem.order_item_ingredients.addAll(orderItemIngredient);
                                            MyApp.getInstance().getAppDatabase().orderItemDao().update(daoItem);
                                        }

                                    } else {
                                        for (OrderItem dao : orderItemDao) {
                                            dao.order_item_ingredients.addAll(oit.order_item_ingredients);
                                            myApp.getAppDatabase().orderItemDao().insert(dao);
                                        }
                                    }
                                } else {
                                    for (OrderItem dao : orderItemDao) {
                                        dao.order_item_ingredients.addAll(oit.order_item_ingredients);
                                        myApp.getAppDatabase().orderItemDao().insert(dao);
                                    }
                                }
                            }
                        } catch (Exception e) {
                            e.printStackTrace();
                        }

                    }
//                    orderItemDao.addAll(myApp.getAppDatabase().orderItemDao().list(oit.order_id));
//                    if (orderItemDao.order_item_ingredients != null && orderItemDao.order_item_addons.size() > 0) {
//                        for (OrderItemAddon orderItemAddon : orderItemDao.order_item_addons) {
//                            if (orderItemAddon._id == oit.order_item_addons.)
//                        }
//                    }
                }

            }
            return null;
        }

        @Override
        protected void onPostExecute(String s) {
            super.onPostExecute(s);
            if (nextThing != null) {
                try {
                    nextThing.call();
                } catch (Exception e) {
                    e.printStackTrace();
                }
            }

        }
    }

}
