package com.tiffintom.fragment;

import android.os.Bundle;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;

import androidx.annotation.NonNull;
import androidx.annotation.Nullable;
import androidx.recyclerview.widget.LinearLayoutManager;
import androidx.recyclerview.widget.RecyclerView;
import androidx.swiperefreshlayout.widget.SwipeRefreshLayout;

import com.androidnetworking.AndroidNetworking;
import com.androidnetworking.error.ANError;
import com.androidnetworking.interfaces.JSONArrayRequestListener;
import com.androidnetworking.interfaces.JSONObjectRequestListener;
import com.google.gson.Gson;
import com.google.gson.reflect.TypeToken;
import com.tiffintom.R;
import com.tiffintom.adapters.FaqAdapter;
import com.tiffintom.base.BaseFragment;
import com.tiffintom.common.CommonFunctions;
import com.tiffintom.models.FAQ;
import com.tiffintom.network.ApiEndPoints;

import org.json.JSONArray;
import org.json.JSONException;
import org.json.JSONObject;

import java.lang.reflect.Type;
import java.util.ArrayList;
import java.util.List;

public class HelpFAQFragment extends BaseFragment {


    public static HelpFAQFragment getInstance() {
        return new HelpFAQFragment();
    }

    private ArrayList<Object> objects = new ArrayList<>();
    private FaqAdapter adapter;
    private RecyclerView rvHelp;
    private SwipeRefreshLayout swipeRefreshLayout;


    private String account_id;

    @Nullable
    @Override
    public View onCreateView(@NonNull LayoutInflater inflater, @Nullable ViewGroup container, @Nullable Bundle savedInstanceState) {


        if (getArguments() != null) {
            account_id = getArguments().getString("account_id");
        }

        return inflater.inflate(R.layout.fragment_help_main, container, false);

    }

    @Override
    public void onViewCreated(@NonNull View view, @Nullable Bundle savedInstanceState) {
        super.onViewCreated(view, savedInstanceState);

        getData();
        swipeRefreshLayout.setOnRefreshListener(this::getData);

    }


    private void getData() {
        new Thread(this::fetchCategories).start();

    }


    private void fetchCategories() {
        if (getActivity() != null)
            getActivity().runOnUiThread(() -> {
                swipeRefreshLayout.setRefreshing(true);

            });

        AndroidNetworking.get(ApiEndPoints.NEW_ACCOUNT_HELP)
                .addQueryParameter("customer_id", (myApp.getMyPreferences().getLoggedInUser() == null) ? "" : String.valueOf(myApp.getMyPreferences().getLoggedInUser().id))
                .build()
                .getAsJSONArray(new JSONArrayRequestListener() {
                    @Override
                    public void onResponse(JSONArray response) {

                        if (getActivity() != null)
                            getActivity().runOnUiThread(() -> {
                                swipeRefreshLayout.setRefreshing(false);
                            });

                           Type type = new TypeToken<List<FAQ>>() {
                            }.getType();
                            objects.clear();
                            try {
                                for (int i=0;i<response.length();i++)
                                {
                                    JSONObject jsonObject=response.getJSONObject(i);
                                    if (jsonObject.getString("id").equalsIgnoreCase(account_id))
                                    {
                                        objects.addAll(new Gson().fromJson(jsonObject.getJSONArray("account_answer").toString(), type));

                                    }
                                }
                                   } catch (JSONException e) {
                                e.printStackTrace();
                            }
                            adapter.notifyDataSetChanged();
                        }


                    @Override
                    public void onError(ANError anError) {
                        anError.printStackTrace();
                        if (getActivity() != null)
                            getActivity().runOnUiThread(() -> swipeRefreshLayout.setRefreshing(false));

                        if (!CommonFunctions.isConnected(getActivity()))
                            myApp.noInternet(getActivity());
                    }
                });

    }

    protected void initViews(View view) {
        super.initViews(view);
        rvHelp = view.findViewById(R.id.rvHelp);
        swipeRefreshLayout = view.findViewById(R.id.swipeRefreshLayout);

        rvHelp.setLayoutManager(new LinearLayoutManager(getActivity(), RecyclerView.VERTICAL, false));

        adapter = new FaqAdapter(objects, (position, data) -> {

            if (data instanceof FAQ) {
                FAQQuestionAnswerFragment faqQuestionAnswerFragment = FAQQuestionAnswerFragment.getInstance(((FAQ) data).question, ((FAQ) data).answer);
                faqQuestionAnswerFragment.show(getChildFragmentManager(), "question_answer");

            }
        });
        rvHelp.setAdapter(adapter);
    }

}
