package com.tiffintom.fragment;

import android.content.Intent;
import android.os.Bundle;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;

import androidx.annotation.NonNull;
import androidx.annotation.Nullable;
import androidx.recyclerview.widget.LinearLayoutManager;
import androidx.recyclerview.widget.RecyclerView;
import androidx.swiperefreshlayout.widget.SwipeRefreshLayout;

import com.androidnetworking.AndroidNetworking;
import com.androidnetworking.error.ANError;
import com.androidnetworking.interfaces.JSONArrayRequestListener;
import com.androidnetworking.interfaces.JSONObjectRequestListener;
import com.google.gson.Gson;
import com.google.gson.reflect.TypeToken;
import com.tiffintom.R;
import com.tiffintom.activity.AuthActivity;
import com.tiffintom.activity.SelectRestaurantActivity;
import com.tiffintom.activity.TransportActivity;
import com.tiffintom.adapters.FaqAdapter;
import com.tiffintom.base.BaseFragment;
import com.tiffintom.common.CommonFunctions;
import com.tiffintom.models.FAQCategory;
import com.tiffintom.models.Header;
import com.tiffintom.network.ApiEndPoints;
import com.tiffintom.utils.LogUtils;

import org.json.JSONArray;
import org.json.JSONException;
import org.json.JSONObject;

import java.lang.reflect.Type;
import java.util.ArrayList;
import java.util.List;

/**
 * Created by Amrish on 12-09-2020.
 */
public class HelpMainFragment extends BaseFragment {

    public static HelpMainFragment getInstance() {
        return new HelpMainFragment();
    }

    private ArrayList<Object> objects = new ArrayList<>();
    private FaqAdapter adapter;
    private RecyclerView rvHelp;
    private SwipeRefreshLayout swipeRefreshLayout;
    int chat_count, tiffintom_chat;

    @Nullable
    @Override
    public View onCreateView(@NonNull LayoutInflater inflater, @Nullable ViewGroup container, @Nullable Bundle savedInstanceState) {
        return inflater.inflate(R.layout.fragment_help_main, container, false);
    }

    @Override
    public void onViewCreated(@NonNull View view, @Nullable Bundle savedInstanceState) {
        super.onViewCreated(view, savedInstanceState);
        swipeRefreshLayout.setOnRefreshListener(this::fetchData);

    }

    @Override
    protected void initViews(View view) {
        super.initViews(view);
        rvHelp = view.findViewById(R.id.rvHelp);
        swipeRefreshLayout = view.findViewById(R.id.swipeRefreshLayout);

        rvHelp.setLayoutManager(new LinearLayoutManager(getActivity(), RecyclerView.VERTICAL, false));
        adapter = new FaqAdapter(objects, (position, data) -> {

            if (data instanceof FAQCategory) {
                startActivity(new Intent(getActivity(), TransportActivity.class).putExtra("destination", "faq").putExtra("name", "Help").putExtra("account_id", ((FAQCategory) data).id));
            } else {
                Header header = (Header) data;
                if (myApp.getMyPreferences().getLoggedInUser() == null) {
                    startActivity(new Intent(getActivity(), AuthActivity.class));
                } else {
                    if (header.addonId == 0) {
                        //chat
                        startActivity(new Intent(getActivity(), SelectRestaurantActivity.class));
                    } else {
                        // contact us
                        startActivity(new Intent(getActivity(), TransportActivity.class).putExtra("destination", "tiffintom_chat_window").putExtra("name", "Chat").putExtra("hideToolbar", true));
                    }
                }
            }

        });
        rvHelp.setAdapter(adapter);
    }

    private void fetchData() {
        new Thread(this::fetchMessageCount).start();
    }

    private void fetchCategories() {
        AndroidNetworking.get(ApiEndPoints.NEW_ACCOUNT_HELP)
                .addQueryParameter("customer_id", (myApp.getMyPreferences().getLoggedInUser() == null) ? "" : String.valueOf(myApp.getMyPreferences().getLoggedInUser().id))
                .build()
                .getAsJSONArray(new JSONArrayRequestListener() {
                    @Override
                    public void onResponse(JSONArray response) {
                        Type type = new TypeToken<List<FAQCategory>>() {
                        }.getType();
                        objects.clear();
                        objects.addAll(new Gson().fromJson(response.toString(), type));
                        objects.add(new Header("Chat", 0, chat_count));
                        objects.add(new Header("Contact Tiffintom", 1, tiffintom_chat));
                        adapter.notifyDataSetChanged();
                    }

                    @Override
                    public void onError(ANError anError) {
                        anError.printStackTrace();
                       /* if (getActivity() != null)
                            getActivity().runOnUiThread(() -> swipeRefreshLayout.setRefreshing(false));*/
                        if (!CommonFunctions.isConnected(getActivity()))
                            myApp.noInternet(getActivity());
                    }
                });
    }

    private void fetchMessageCount() {
        if (getActivity() != null)
            getActivity().runOnUiThread(() -> {
                swipeRefreshLayout.setRefreshing(true);
            });
        AndroidNetworking.get(ApiEndPoints.NEW_CHATALLCOUNT)
                .addQueryParameter("customer_id", (myApp.getMyPreferences().getLoggedInUser() == null) ? "" : String.valueOf(myApp.getMyPreferences().getLoggedInUser().id))
                .build()
                .getAsJSONObject(new JSONObjectRequestListener() {
                    @Override
                    public void onResponse(JSONObject response) {
                        if (getActivity() != null)
                            getActivity().runOnUiThread(() -> {
                                swipeRefreshLayout.setRefreshing(false);
                            });
                        try {
                            chat_count = Integer.parseInt(response.getString("order_help_count"));
                            tiffintom_chat = Integer.parseInt(response.getString("chat_count"));
                            fetchCategories();
                        } catch (JSONException exception) {
                            exception.printStackTrace();
                        }

                        adapter.notifyDataSetChanged();
                    }

                    @Override
                    public void onError(ANError anError) {
                        anError.printStackTrace();
                        if (getActivity() != null)
                            getActivity().runOnUiThread(() -> {
                                swipeRefreshLayout.setRefreshing(false);
                            });

                        if (!CommonFunctions.isConnected(getActivity()))
                            myApp.noInternet(getActivity());
                    }
                });

    }

    @Override
    public void onResume() {
        super.onResume();
        fetchData();
    }
}
