package com.tiffintom.fragment;

import android.Manifest;
import android.content.Context;
import android.content.Intent;
import android.content.pm.PackageManager;
import android.graphics.Color;
import android.graphics.drawable.ColorDrawable;
import android.location.Location;
import android.net.Uri;
import android.os.Bundle;
import android.provider.Settings;
import android.text.Editable;
import android.text.TextWatcher;
import android.util.DisplayMetrics;
import android.util.Log;
import android.view.KeyEvent;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.CheckBox;
import android.widget.EditText;
import android.widget.HorizontalScrollView;
import android.widget.ImageView;
import android.widget.LinearLayout;
import android.widget.RelativeLayout;
import android.widget.TextView;

import androidx.annotation.NonNull;
import androidx.annotation.Nullable;
import androidx.appcompat.app.AlertDialog;
import androidx.core.app.ActivityCompat;
import androidx.fragment.app.Fragment;
import androidx.recyclerview.widget.LinearLayoutManager;
import androidx.recyclerview.widget.PagerSnapHelper;
import androidx.recyclerview.widget.RecyclerView;
import androidx.recyclerview.widget.SnapHelper;
import androidx.swiperefreshlayout.widget.SwipeRefreshLayout;
import androidx.viewpager.widget.ViewPager;

import com.airbnb.lottie.LottieAnimationView;
import com.androidnetworking.AndroidNetworking;
import com.androidnetworking.error.ANError;
import com.androidnetworking.interfaces.JSONObjectRequestListener;
import com.androidnetworking.interfaces.ParsedRequestListener;
import com.google.android.material.appbar.AppBarLayout;
import com.google.android.material.floatingactionbutton.FloatingActionButton;
import com.google.gson.Gson;
import com.google.gson.reflect.TypeToken;
import com.make.dots.dotsindicator.DotsIndicator;
import com.tiffintom.MyApp;
import com.tiffintom.R;
import com.tiffintom.activity.DineInScanActivity;
import com.tiffintom.activity.MainActivity;
import com.tiffintom.activity.NoInternetActivity;
import com.tiffintom.activity.RestaurantDetailsActivity;
import com.tiffintom.activity.SignupActivity;
import com.tiffintom.activity.TransportActivity;
import com.tiffintom.adapters.BannerAdapter;
import com.tiffintom.adapters.BannerViewPagerAdapter;
import com.tiffintom.adapters.HomeCuisineAdapter;
import com.tiffintom.adapters.RestaurantAdapter;
import com.tiffintom.adapters.SnippetAdapter;
import com.tiffintom.base.BaseFragment;
import com.tiffintom.common.CommonFunctions;
import com.tiffintom.common.MyLocation;
import com.tiffintom.common.Validators;
import com.tiffintom.interfaces.DialogDismissListener;
import com.tiffintom.models.Address;
import com.tiffintom.models.ApiError;
import com.tiffintom.models.Banner;
import com.tiffintom.models.CartSummary;
import com.tiffintom.models.Cuisine;
import com.tiffintom.models.OrderHistory;
import com.tiffintom.models.Postcode;
import com.tiffintom.models.Referral;
import com.tiffintom.models.Restaurant;
import com.tiffintom.models.RestaurantMiniSnippet;
import com.tiffintom.models.Review;
import com.tiffintom.models.RewardPoint;
import com.tiffintom.models.SearchResult;
import com.tiffintom.models.SiteSettings;
import com.tiffintom.models.User;
import com.tiffintom.network.ApiEndPoints;
import com.tiffintom.network.ApiUtils;
import com.tiffintom.utils.Constants;
import com.tiffintom.utils.LogUtils;
import com.tiffintom.utils.ToastUtils;

import org.json.JSONException;
import org.json.JSONObject;

import java.lang.reflect.Type;
import java.util.ArrayList;
import java.util.Collections;
import java.util.List;

import ru.tinkoff.scrollingpagerindicator.ScrollingPagerIndicator;

import static android.app.Activity.RESULT_OK;
import static com.tiffintom.MyApp.searchResult;
import static com.tiffintom.network.ApiEndPoints.ACCOUNT;
import static com.tiffintom.network.ApiEndPoints.NEW_ORDERS;
import static com.tiffintom.utils.Constants.CODE_NO_INTERNET;
import static com.tiffintom.utils.Constants.CODE_REFRESH;
import static com.tiffintom.utils.Constants.FILTER_CLEAR;
import static com.tiffintom.utils.Constants.FILTER_DISTANCE_HIGH_LOW;
import static com.tiffintom.utils.Constants.FILTER_DISTANCE_LOW_HIGH;
import static com.tiffintom.utils.Constants.FILTER_MIN_ORDER_HIGH_LOW;
import static com.tiffintom.utils.Constants.FILTER_MIN_ORDER_LOW_HIGH;
import static com.tiffintom.utils.Constants.FILTER_NONE;
import static com.tiffintom.utils.Constants.FILTER_POPULAR;
import static com.tiffintom.utils.Constants.FILTER_RATING_HIGH_LOW;
import static com.tiffintom.utils.Constants.FILTER_RATING_LOW_HIGH;
import static com.tiffintom.utils.Constants.PERMISSION_CODE_LOCATION;

/**
 * Created by Amrish on 19-08-2020.
 */
public class HomeFragment extends BaseFragment {
    public static HomeFragment getInstance() {
        HomeFragment fragment = new HomeFragment();
        Bundle bundle = new Bundle();
        fragment.setArguments(bundle);
        return fragment;
    }

    private TextView tvLocation, tvChangeLocation;
    private EditText etSearch;
    private ImageView ivFilter;
    private HorizontalScrollView horizontalScrollView;
    private RecyclerView rvCuisine, rvBanner, rvRestaurant;
    private HomeCuisineAdapter homeCuisineAdapter;
    private BannerAdapter bannerAdapter;
    private RestaurantAdapter restaurantAdapter;
    private ArrayList<Cuisine> cuisines = new ArrayList<>();
    private ArrayList<Banner> banners = new ArrayList<>();
    private ArrayList<Restaurant> restaurants = new ArrayList<>();
    ArrayList<Restaurant> filteredRestaurants = new ArrayList<>();
    private SwipeRefreshLayout swipeRefreshLayout;
    private LinearLayout llLocation;
    private int currentSort = FILTER_NONE;
    private Postcode currentPostcode;
    private Address defaultAddress;
    private User loggedInUser = myApp.getMyPreferences().getLoggedInUser();
    private LinearLayout llData, llLoading, llFilter, llDineIn;
    private boolean isFiltered = false;
    private String cuisine_name;
    private Integer cuisine_id;
    private RelativeLayout customToolbar;
    private FloatingActionButton fabClose;
    private RelativeLayout llMiniSnippet;
    private ArrayList<Object> snippetArrayList = new ArrayList<>();
    private RecyclerView rvBottomSnippet;
    private SnippetAdapter snippetAdapter;
    private LottieAnimationView dineinAnimationView, loadingAnimation;
    private DotsIndicator bannerIndicator;
    private ViewPager vpBanners;
    private BannerViewPagerAdapter bannerViewPagerAdapter;
    private LinearLayout llPostcode;
    private EditText etPostcode;
    private ImageView ivCurrentLocation, ivLocationBack;
    public static final int REQUEST_LOCATION = 1;
    private TextView tvMessage;
    private ScrollingPagerIndicator snippetIndicator;
    private CheckBox rbAll;
    private ImageView ivSearchPostcode;
    private RelativeLayout rlCuisines;
    private AppBarLayout filterAppbar;
    private LinearLayout llBanners;
    public SiteSettings siteSettings;
    private LottieAnimationView animationPostcode;

    @Nullable
    @Override
    public View onCreateView(@NonNull LayoutInflater inflater, @Nullable ViewGroup container, @Nullable Bundle savedInstanceState) {
        if (getArguments() != null) {
            isFiltered = getArguments().getBoolean("isFiltered");
            cuisine_name = getArguments().getString("cuisine_name");
            cuisine_id = getArguments().getInt("cuisine_id");
        }
        return inflater.inflate(R.layout.fragment_home, container, false);
    }

    @Override
    public void onViewCreated(@NonNull View view, @Nullable Bundle savedInstanceState) {
        super.onViewCreated(view, savedInstanceState);

        if (loggedInUser == null) {
            RelativeLayout.LayoutParams params = (RelativeLayout.LayoutParams) llDineIn.getLayoutParams();
            params.addRule(RelativeLayout.ALIGN_PARENT_RIGHT);
            params.setMargins(0, 0, 0, 0);
            llDineIn.setLayoutParams(params);
        }
        if (myApp.getMyPreferences().getSiteSettings().dine_in.equalsIgnoreCase("1"))
        {
            llDineIn.setVisibility(View.VISIBLE);
        }else
        {
            llDineIn.setVisibility(View.GONE);
        }
        setListeners();
        if (!isFiltered)
            displayAddress();
        else {
            if (searchResult == null)
                fetchData();
            else updateViews();
        }
    }

    @Override
    protected void initViews(View view) {
        super.initViews(view);
        currentPostcode = myApp.getMyPreferences().getCurrentPostcode();
        defaultAddress = myApp.getMyPreferences().getCurrentAddress();
        tvUserName = view.findViewById(R.id.tvUserName);
        tvChangeLocation = view.findViewById(R.id.tvChangeLocation);
        etSearch = view.findViewById(R.id.etSearch);
        tvLocation = view.findViewById(R.id.tvLocation);
        ivFilter = view.findViewById(R.id.ivFilter);

        animationPostcode = view.findViewById(R.id.animationPostcode);
        llLocation = view.findViewById(R.id.llLocation);
        rvBottomSnippet = view.findViewById(R.id.rvSnippetItems);
        llMiniSnippet = view.findViewById(R.id.llMiniSnippet);
        llData = view.findViewById(R.id.llData);
        llLoading = view.findViewById(R.id.llLoading);
        llFilter = view.findViewById(R.id.llFilter);
        customToolbar = view.findViewById(R.id.customToolbar);

        llDineIn = view.findViewById(R.id.llDineIn);
        dineinAnimationView = view.findViewById(R.id.dinein_animation_view);

        tvMessage = view.findViewById(R.id.tvMessage);
        loadingAnimation = view.findViewById(R.id.animation_view);
        filterAppbar = view.findViewById(R.id.filterAppbar);
        rlCuisines = view.findViewById(R.id.rlCuisines);
        rvBanner = view.findViewById(R.id.rvBanner);
        rvCuisine = view.findViewById(R.id.rvCuisine);
        horizontalScrollView = view.findViewById(R.id.horizontal);
        rvRestaurant = view.findViewById(R.id.rvRestaurant);

        rbAll = view.findViewById(R.id.rbAll);

        swipeRefreshLayout = view.findViewById(R.id.swipeRefreshLayout);
        vpBanners = view.findViewById(R.id.vpBanners);
        bannerIndicator = view.findViewById(R.id.dotsBannerIndicator);
        llPostcode = view.findViewById(R.id.llPostcode);
        etPostcode = view.findViewById(R.id.etPostcode);
        ivCurrentLocation = view.findViewById(R.id.ivCurrentLocation);
        ivLocationBack = view.findViewById(R.id.ivBackArrow);
        ivSearchPostcode = view.findViewById(R.id.ivSearch);
        llBanners = view.findViewById(R.id.llBanners);

        snippetIndicator = view.findViewById(R.id.recyclerIndicator);
        homeCuisineAdapter = new HomeCuisineAdapter(getActivity(), cuisines, 0, this::onCategorySelect);
        LinearLayoutManager categoryLayout = new LinearLayoutManager(getActivity(), RecyclerView.HORIZONTAL, false);
        rvCuisine.setLayoutManager(categoryLayout);
        rvCuisine.setNestedScrollingEnabled(false);
        rvCuisine.setAdapter(homeCuisineAdapter);


        bannerViewPagerAdapter = new BannerViewPagerAdapter(getActivity(), banners, null, (position, data) -> onBannerSelect(position, data));
        vpBanners.setClipToPadding(false);
        DisplayMetrics displayMetrics = new DisplayMetrics();
        getActivity().getWindowManager().getDefaultDisplay().getMetrics(displayMetrics);
        int width = displayMetrics.widthPixels;
        int paddingToSet = width / 3;
        vpBanners.setPadding(0, 0, paddingToSet, 0);

        bannerViewPagerAdapter.registerDataSetObserver(bannerIndicator.getDataSetObserver());
        vpBanners.setAdapter(bannerViewPagerAdapter);
        bannerIndicator.setViewPager(vpBanners);

        restaurantAdapter = new RestaurantAdapter(filteredRestaurants, this::onFavouriteClick, this::onRestaurantClick);
        rvRestaurant.setLayoutManager(new LinearLayoutManager(getActivity(), RecyclerView.VERTICAL, false));
        rvRestaurant.setAdapter(restaurantAdapter);

        if (isFiltered) {
            llLocation.setVisibility(View.GONE);
            llPostcode.setVisibility(View.GONE);
            filterAppbar.setVisibility(View.GONE);
            rlCuisines.setVisibility(View.GONE);
            vpBanners.setVisibility(View.GONE);
            bannerIndicator.setVisibility(View.GONE);
            customToolbar.setVisibility(View.GONE);
        }
        rvBottomSnippet.setLayoutManager(new LinearLayoutManager(getActivity(), RecyclerView.HORIZONTAL, false));
        SnapHelper helper = new PagerSnapHelper();
        helper.attachToRecyclerView(rvBottomSnippet);
        snippetAdapter = new SnippetAdapter(snippetArrayList, (position, data) -> {
            manageLastOrderClick(data);
        });
        rvBottomSnippet.setAdapter(snippetAdapter);
        snippetIndicator.attachToRecyclerView(rvBottomSnippet);
    }

    private void setListeners() {


        swipeRefreshLayout.setOnRefreshListener(this::fetchData);
        ivFilter.setOnClickListener(view -> {
            HomeSortBottomDialogFragment homeSortBottomDialogFragment = HomeSortBottomDialogFragment.getInstance(currentSort);

            homeSortBottomDialogFragment.show(getChildFragmentManager(), "sort");
            homeSortBottomDialogFragment.setDialogDismissListener(sortDialogListener);
        });

        etSearch.setOnKeyListener((textView, i, keyEvent) -> {
            if (keyEvent.getKeyCode() == KeyEvent.KEYCODE_BACK) {
                getActivity().onBackPressed();
            } else if ((keyEvent.getKeyCode() == KeyEvent.KEYCODE_ENTER)) {
                filteredRestaurants.clear();
                filteredRestaurants.addAll(searchRestaurant());
                restaurantAdapter.notifyDataSetChanged();
                homeCuisineAdapter.selectedCuisine = -1;
                homeCuisineAdapter.notifyDataSetChanged();
                if (getActivity() != null)
                    CommonFunctions.hideKeyboard(getActivity(), etSearch);
            }
            return false;
        });
        etSearch.addTextChangedListener(new TextWatcher() {
            @Override
            public void beforeTextChanged(CharSequence charSequence, int i, int i1, int i2) {

            }

            @Override
            public void onTextChanged(CharSequence charSequence, int i, int i1, int i2) {

            }

            @Override
            public void afterTextChanged(Editable editable) {
                filteredRestaurants.clear();
                filteredRestaurants.addAll(searchRestaurant());
                restaurantAdapter.notifyDataSetChanged();
                homeCuisineAdapter.selectedCuisine = -1;
                homeCuisineAdapter.notifyDataSetChanged();
            }
        });

        llDineIn.setOnClickListener(view -> {
            if (checkLocationPermission()) {
                Intent i = new Intent(getActivity(), DineInScanActivity.class);
                startActivity(i);
            }
        });

        etPostcode.setOnKeyListener((textview, i, keyEvent) -> {
            if (keyEvent.getKeyCode() == KeyEvent.KEYCODE_BACK) {
                getActivity().onBackPressed();
            } else if (keyEvent.getKeyCode() == KeyEvent.KEYCODE_ENTER) {
                if (!Validators.isNullOrEmpty(etPostcode.getText().toString())) {
                    String postCode = etPostcode.getText().toString();
                    new Thread(() -> {
                        searchPostcode(postCode);
                    }).start();
                } else {
                    if (etPostcode.getText().toString().length() < 3) {
                        ToastUtils.makeToast(getActivity(), "Please valid postcode");
                    }
                    ToastUtils.makeToast(getActivity(), "Please enter postcode");
                }

            }
            return false;
        });

        ivCurrentLocation.setOnClickListener(view -> {
            if (checkLocationServicePermission()) {
                startLocationService();
            }
        });
        llLocation.setOnClickListener(view -> {
            llFilter.setVisibility(View.GONE);
            llLocation.setVisibility(View.GONE);
            llLoading.setVisibility(View.GONE);
            llPostcode.setVisibility(View.VISIBLE);
            rlCuisines.setVisibility(View.GONE);
            animationPostcode.setVisibility(View.VISIBLE);
            swipeRefreshLayout.setVisibility(View.GONE);
            // showEnterLocation();
        });
        ivLocationBack.setOnClickListener(view -> {
            llFilter.setVisibility(View.VISIBLE);
            llLocation.setVisibility(View.VISIBLE);
            llPostcode.setVisibility(View.GONE);
            animationPostcode.setVisibility(View.GONE);
            rlCuisines.setVisibility(View.VISIBLE);
            swipeRefreshLayout.setVisibility(View.VISIBLE);
            if (homeApiStatus == 0) {
                llLoading.setVisibility(View.VISIBLE);
            } else if (homeApiStatus == 2) {
                showErrorLayout();
            }
            if (getActivity() != null)
                CommonFunctions.hideKeyboard(getActivity(), etPostcode);
            //  hideEnterLocation();
        });
        rbAll.setOnCheckedChangeListener((compoundButton, b) -> {
            if (b) {
                if (homeCuisineAdapter != null) {
                    for (Cuisine c : cuisines)
                        c.isSelected = false;
                    homeCuisineAdapter.selectedCuisines.clear();
                    homeCuisineAdapter.notifyDataSetChanged();
                }
                filterByCuisine("All");
            } else {
                if (homeCuisineAdapter != null && homeCuisineAdapter.selectedCuisines.size() == 0) {
                    rbAll.setChecked(true);
                }
            }
        });
        rvCuisine.setOnTouchListener((view, motionEvent) -> true);

        ivSearchPostcode.setOnClickListener(view -> {
            if (Validators.isNullOrEmpty(etPostcode.getText().toString())) {
                ToastUtils.makeToast(getActivity(), "Please enter postcode");
            } else if (!Validators.isNullOrEmpty(etPostcode.getText().toString()) && etPostcode.getText().toString().length() < 3) {
                ToastUtils.makeToast(getActivity(), "Please valid postcode");
            } else {
                String postCode = etPostcode.getText().toString();
                new Thread(() -> {
                    searchPostcode(postCode);
                }).start();
            }
        });

    }

    private void showErrorLayout() {
        llLoading.setVisibility(View.VISIBLE);
        loadingAnimation.setAnimation(R.raw.no_records);
        loadingAnimation.playAnimation();
        tvMessage.setText("There doesn't seem to be any restaurants available in your area.");
        llData.setVisibility(View.GONE);
    }

    private boolean checkLocationServicePermission() {
        if (ActivityCompat.checkSelfPermission(getActivity(),
                Manifest.permission.ACCESS_FINE_LOCATION)
                != PackageManager.PERMISSION_GRANTED) {

            // Should we show an explanation?
            if (ActivityCompat.shouldShowRequestPermissionRationale(getActivity(),
                    Manifest.permission.ACCESS_FINE_LOCATION)) {

                new android.app.AlertDialog.Builder(getActivity())
                        .setTitle("Location permission required")
                        .setMessage("To access your current postcode we require your location permission, Please allow to continue.")
                        .setPositiveButton("ALLOW", (dialogInterface, i) -> {
                            requestPermissions(new String[]{Manifest.permission.ACCESS_FINE_LOCATION}, REQUEST_LOCATION);
                        })
                        .setNegativeButton("DENY", (dialogInterface, i) -> dialogInterface.dismiss())
                        .create()
                        .show();


            } else {
                requestPermissions(new String[]{Manifest.permission.ACCESS_FINE_LOCATION}, REQUEST_LOCATION);
            }
            return false;
        } else {
            return true;
        }
    }

    private void startLocationService() {
        MyLocation myLocation = new MyLocation();
        progressDialog.show();
        myLocation.getLocation(getActivity(), new MyLocation.LocationResult() {
            @Override
            public void gotLocation(Location location) {
                android.location.Address address = CommonFunctions.getAddressByLocation(getActivity(), location.getLatitude(), location.getLongitude());
                if (getActivity() != null) {
                    getActivity().runOnUiThread(() -> {
                        progressDialog.dismiss();
                        if (address != null) {
                            String zipcode = address.getPostalCode();
                            if (!Validators.isNullOrEmpty(zipcode) && zipcode.length() >= 3) {
                                etPostcode.setText(zipcode);
                                new Thread(() -> {
                                    searchPostcode(zipcode);
                                }).start();
                            }
                        }
                    });
                }

            }
        });

    }

    private void searchPostcode(String postcode) {
        if (getActivity() != null) getActivity().runOnUiThread(progressDialog::show);
        AndroidNetworking.get(ApiEndPoints.RMS_POSTCODE + postcode)
                .build()
                .getAsObject(Postcode.class, new ParsedRequestListener<Postcode>() {
                    @Override
                    public void onResponse(Postcode response) {

                        if (getActivity() != null)
                            CommonFunctions.hideKeyboard(getActivity(), etPostcode);
                        if (getActivity() != null)
                            getActivity().runOnUiThread(() -> {
                                progressDialog.dismiss();
                                currentPostcode = response;
                                etPostcode.setText("");
                                myApp.getMyPreferences().saveCurrentPostcode(currentPostcode);
                                CommonFunctions.hideKeyboard(getActivity(), etPostcode);
                                displayAddress();
                            });
                    }

                    @Override
                    public void onError(ANError anError) {
                        if (getActivity() != null)
                            getActivity().runOnUiThread(() -> progressDialog.dismiss());
                        if (anError.getErrorCode() == 400) {
                            ApiError apiError = anError.getErrorAsObject(ApiError.class);
                            ToastUtils.makeSnackToast(getActivity(), apiError.getMessage());
                        }
                        if (!CommonFunctions.isConnected(getActivity())) {
                            myApp.noInternet(getActivity());
                        }
                    }
                });
    }


    private void manageLastOrderClick(Object data) {
        if (data instanceof OrderHistory) {
            OrderHistory orderDetail = (OrderHistory) data;
            Intent intent = new Intent(getActivity(), TransportActivity.class);
            if (Validators.isNullOrEmpty(orderDetail.order_type)) {
                intent.putExtra("is_dinein", true);
                intent.putExtra("destination", "orderView");
            } else {
                intent.putExtra("destination", "trackOrder");
            }
            intent.putExtra("hideToolbar", true);

            intent.putExtra("order_id", String.valueOf(orderDetail.id));
            if (orderDetail.status.equalsIgnoreCase("delivered")) {
                // rate order
                intent.putExtra("shouldRate", true);
            }
            startActivityForResult(intent, CODE_REFRESH);
        } else if (data instanceof RestaurantMiniSnippet) {
            clearCartConfirmation((RestaurantMiniSnippet) data);
        } else if (data instanceof String) {
            if (((String) data).equalsIgnoreCase("removed")) {
                new Thread(() -> {
                    updateMiniView();
                }).start();
            }
        }
    }

    private void clearCartConfirmation(RestaurantMiniSnippet miniSnippet) {
        AlertDialog alertDialog = new AlertDialog.Builder(getActivity()).create();
        alertDialog.getWindow().setBackgroundDrawable(new ColorDrawable(Color.TRANSPARENT));
        LayoutInflater inflater = (LayoutInflater) getActivity().getSystemService(Context.LAYOUT_INFLATER_SERVICE);
        View dialogView = inflater.inflate(R.layout.dialog_remove_card, null);

        TextView tvMessage = dialogView.findViewById(R.id.tvMessage);
        TextView tvCancel = dialogView.findViewById(R.id.tvCancel);
        TextView tvConfirm = dialogView.findViewById(R.id.tvConfirm);
        tvMessage.setText("Are you sure you want to clear basket?");
        tvConfirm.setOnClickListener(view -> {
            new Thread(() -> {
                MyApp.getInstance().getMyPreferences().deleteOrderRestaurant();
                MyApp.getInstance().getAppDatabase().cartDao().nukeTable();
                if (MyApp.getInstance().getMyPreferences().getLoggedInUser() != null) {
                    CommonFunctions.deleteAllCart();
                }
                updateMiniView();
            }).start();
            MainActivity.tvCartItemsCountBadge.setText("0");
            MainActivity.tvCartItemsCountBadge.setVisibility(View.GONE);
            alertDialog.dismiss();
        });
        tvCancel.setOnClickListener(view -> {
            alertDialog.dismiss();
        });
        alertDialog.setView(dialogView);
        alertDialog.setCancelable(false);
        alertDialog.show();
    }

    private void displayAddress() {
        if (currentPostcode == null) {
            llFilter.setVisibility(View.GONE);
            llLocation.setVisibility(View.GONE);
            llPostcode.setVisibility(View.VISIBLE);
            etPostcode.requestFocus();
            ivLocationBack.setEnabled(false);
        } else {
            ivLocationBack.setEnabled(true);
            llLocation.setVisibility(View.VISIBLE);
            llPostcode.setVisibility(View.GONE);
            llFilter.setVisibility(View.VISIBLE);
            swipeRefreshLayout.setVisibility(View.VISIBLE);
            animationPostcode.setVisibility(View.GONE);
            rlCuisines.setVisibility(View.VISIBLE);
            StringBuilder addressString = new StringBuilder();
            if (!Validators.isNullOrEmpty(currentPostcode.street)) {
                addressString.append(currentPostcode.street).append(",");
            }
            if (!Validators.isNullOrEmpty(currentPostcode.post_town)) {
                addressString.append(currentPostcode.post_town).append(",");
            }
            tvLocation.setText(addressString.toString() + currentPostcode.post_code);
            tvLocation.setSelected(true);
            fetchData();
        }

    }

    public boolean checkLocationPermission() {
        if (ActivityCompat.checkSelfPermission(getActivity(),
                Manifest.permission.CAMERA)
                != PackageManager.PERMISSION_GRANTED) {

            // Should we show an explanation?
            if (ActivityCompat.shouldShowRequestPermissionRationale(getActivity(),
                    Manifest.permission.CAMERA)) {

                new android.app.AlertDialog.Builder(getActivity())
                        .setTitle("Location permission required")
                        .setMessage("To scan qr code we require your camera permission, Please allow to continue.")
                        .setPositiveButton("ALLOW", (dialogInterface, i) -> {
                            requestPermissions(new String[]{Manifest.permission.CAMERA}, PERMISSION_CODE_LOCATION);
                        })
                        .setNegativeButton("DENY", (dialogInterface, i) -> dialogInterface.dismiss())
                        .create()
                        .show();


            } else {
                requestPermissions(new String[]{Manifest.permission.CAMERA}, PERMISSION_CODE_LOCATION);
            }
            return false;
        } else {
            return true;
        }
    }

    private void onFavouriteClick(int position, Object data) {
        Restaurant r = (Restaurant) data;
        ApiUtils.getFavouriteAction(String.valueOf(loggedInUser.id), String.valueOf(r.id), (r.is_favourite == 1) ? "1" : "0")
                .getAsJSONObject(new JSONObjectRequestListener() {
                    @Override
                    public void onResponse(JSONObject response) {
                        if (r.is_favourite == 1)
                            ToastUtils.makeSnackToast(getActivity(), r.restaurant_name + " has been added to favourites");
                        else
                            ToastUtils.makeSnackToast(getActivity(), r.restaurant_name + " has been removed from favourites");

                    }

                    @Override
                    public void onError(ANError anError) {
                        anError.printStackTrace();
                        LogUtils.e("ERROR WHILE FAVORITE :: ", anError.getErrorBody());
                    }
                });
    }

    int res_id = 0;

    private void updateMiniView() {

        snippetArrayList.clear();
        new Thread(() -> {
            res_id = myApp.getAppDatabase().cartDao().getRestaurantId();
            if (res_id != 0) {

                Restaurant orderRestaurant = myApp.getMyPreferences().getOrderRestaurant();
                if (orderRestaurant != null) {
                    CartSummary cartSummary = myApp.getAppDatabase().cartDao().getCartSummary();
                    if (getActivity() != null) getActivity().runOnUiThread(() -> {
                        RestaurantMiniSnippet miniSnippet = new RestaurantMiniSnippet();
                        miniSnippet.res_id = res_id;
                        miniSnippet.restaurantName = orderRestaurant.restaurant_name;
                        miniSnippet.items = String.valueOf(cartSummary.items);
                        if (!Validators.isNullOrEmpty(orderRestaurant.logo_name)) {
                            miniSnippet.imageUrl = orderRestaurant.image_url;
                        }
                        snippetArrayList.add(miniSnippet);
                        if (getActivity() != null) {
                            getActivity().runOnUiThread(() -> {

                                snippetAdapter.notifyDataSetChanged();
                                llMiniSnippet.setVisibility(View.VISIBLE);
                            });
                        }
                        if (loggedInUser != null)
                            fetchLastOrderDetails();
                    });
                } else {
                    if (getActivity() != null) {
                        getActivity().runOnUiThread(() -> {
                            snippetAdapter.notifyDataSetChanged();

                        });
                    }
                    if (loggedInUser != null)
                        fetchLastOrderDetails();
                    else {
                        if (getActivity() != null)
                            getActivity().runOnUiThread(() -> llMiniSnippet.setVisibility(View.GONE));
                    }
                }
            } else {
                if (getActivity() != null) {
                    getActivity().runOnUiThread(() -> {
                        snippetAdapter.notifyDataSetChanged();
                    });
                }
                if (loggedInUser != null)
                    fetchLastOrderDetails();
                else if (getActivity() != null)
                    getActivity().runOnUiThread(() -> llMiniSnippet.setVisibility(View.GONE));
            }

        }).start();

    }

    private void onRestaurantClick(int position, Object data) {
        Restaurant clickedRestaurant = (Restaurant) data;
        Intent intent = new Intent(getActivity(), RestaurantDetailsActivity.class);
        intent.putExtra("resId", clickedRestaurant.id);
        intent.putExtra("res_name", clickedRestaurant.restaurant_name);
        startActivityForResult(intent, CODE_REFRESH);
    }

    private void showOtherRestaurantCartWarning(Restaurant clickedRestaurant) {
        AlertDialog.Builder alertDialog = new AlertDialog.Builder(getActivity(), R.style.MyDialog);
        alertDialog.setTitle("Note");
        alertDialog.setMessage("You have added some items in cart from other restaurant, Do you want to go back and choose different Restaurant?");
        alertDialog.setPositiveButton("YES", (dialogInterface, i) -> {
            new Thread(() -> {
                myApp.getMyPreferences().deleteOrderRestaurant();
                myApp.getAppDatabase().cartDao().nukeTable();

                Intent intent = new Intent(getActivity(), RestaurantDetailsActivity.class);
                intent.putExtra("resId", clickedRestaurant.id);
                intent.putExtra("res_name", clickedRestaurant.restaurant_name);
                startActivityForResult(intent, CODE_REFRESH);

            }).start();
        });
        alertDialog.setNegativeButton("NO", (dialogInterface, i) -> {
            dialogInterface.cancel();
        });
        alertDialog.show();
    }

    private void onBannerSelect(int position, Object data) {
        Banner banner = (Banner) data;

        if (!Validators.isNullOrEmpty(banner.restaurant_id)) {
            Intent intent = new Intent(getActivity(), RestaurantDetailsActivity.class);
            intent.putExtra("resId", Integer.parseInt(banner.restaurant_id));
            startActivityForResult(intent, CODE_REFRESH);
        } else if (!Validators.isNullOrEmpty(banner.banner_link)) {
            Intent browserIntent = new Intent(Intent.ACTION_VIEW, Uri.parse(banner.banner_link));
            startActivity(browserIntent);
        }
    }

    private void onCategorySelect(int position, Object data) {
        ArrayList<Cuisine> selectedCuisines = (ArrayList<Cuisine>) data;
        if (selectedCuisines.size() == 0) {
            rbAll.setChecked(true);
            filterByCuisine("All");
        } else {
            rbAll.setChecked(false);
            filterByCuisine(selectedCuisines);
        }
        if (rvCuisine.getChildAt(position) != null) {
            horizontalScrollView.smoothScrollTo((int) rvCuisine.getChildAt(position).getX() + (rbAll.getRight()), 0);
        } else horizontalScrollView.smoothScrollTo(0, 0);
    }


    private void fetchData() {
        new Thread(this::fetchHome).start();
    }

    private int homeApiStatus = 0; //0 not fetched, 1 fetched, 2 error;

    private void fetchHome() {
        homeApiStatus = 0;
        if (getActivity() != null)
            getActivity().runOnUiThread(() -> {
                if (!swipeRefreshLayout.isRefreshing()) {
                    llLoading.setVisibility(View.VISIBLE);
                    llData.setVisibility(View.GONE);
                    loadingAnimation.setAnimation(R.raw.loading_menu);
                    loadingAnimation.playAnimation();
                    tvMessage.setText("Searching for nearby restaurants");
                }
            });
        ApiUtils.getHome(currentPostcode.post_code, (MyApp.getInstance().getMyPreferences().getLoggedInUser() == null) ? "" : String.valueOf(loggedInUser.id)).getAsJSONObject(new JSONObjectRequestListener() {
            @Override
            public void onResponse(JSONObject response) {

                searchResult = new Gson().fromJson(response.toString(), SearchResult.class);
                fetchSiteSettings();

                updateViews();
            }

            @Override
            public void onError(ANError anError) {
                LogUtils.e("AN ERROR IN FETCH HOME");
                homeApiStatus = 2;
                anError.printStackTrace();
                if (getActivity() != null)
                    getActivity().runOnUiThread(() -> {
                        swipeRefreshLayout.setRefreshing(false);
                        llLoading.setVisibility(View.VISIBLE);
                        loadingAnimation.setAnimation(R.raw.no_records);
                        loadingAnimation.playAnimation();
                        tvMessage.setText("There doesn't seem to be any restaurants available in your area.");
                        llData.setVisibility(View.GONE);

                        if (!CommonFunctions.isConnected(getActivity()))
                            startActivityForResult(new Intent(getActivity(), NoInternetActivity.class), Constants.CODE_NO_INTERNET);
                    });
            }
        });
    }

    private void fetchSiteSettings() {
        AndroidNetworking.get(ApiEndPoints.NEW_SITE_SETTINGS)
                .build()
                .getAsJSONObject(new JSONObjectRequestListener() {
                    @Override
                    public void onResponse(JSONObject response) {
                        boolean updateAvailable = false;
                        searchResult.siteSettings = new Gson().fromJson(response.toString(), SiteSettings.class);
                        myApp.getMyPreferences().saveSiteSettings(searchResult.siteSettings);
                        AndroidNetworking.get(ApiEndPoints.NEW_REWARDS)
                                .build()
                                .getAsJSONObject(new JSONObjectRequestListener() {

                                    @Override
                                    public void onResponse(JSONObject response) {
                                        RewardPoint rewardPoint = new Gson().fromJson(response.toString(), RewardPoint.class);
                                        myApp.getMyPreferences().saveRewardPoint(rewardPoint);
                                    }

                                    @Override
                                    public void onError(ANError anError) {

                                    }
                                });
                        AndroidNetworking.get(ApiEndPoints.NEW_Referrals)
                                .build()
                                .getAsJSONObject(new JSONObjectRequestListener() {
                                    @Override
                                    public void onResponse(JSONObject response) {
                                        Referral referrals = new Gson().fromJson(response.toString(), Referral.class);
                                        myApp.getMyPreferences().saveReferral(referrals);
                                    }

                                    @Override
                                    public void onError(ANError anError) {
                                    }
                                });
                    }

                    @Override
                    public void onError(ANError anError) {

                    }
                });
    }

    private void updateViews() {
        if (loggedInUser != null) {
            User toBeUpdate = loggedInUser;
            toBeUpdate.first_user = searchResult.firstUser;
            myApp.getMyPreferences().saveLoggedInUser(toBeUpdate);
        }
        MyApp.banne_path = searchResult.banner_path;



        boolean notifyCuisine = false;
        if (cuisines.size() == 0) {
            notifyCuisine = true;
            cuisines.addAll(searchResult.cuisines);

        }
        banners.clear();
        banners.addAll(searchResult.banner_lists);

        //bannerAdapter.notifyDataSetChanged();
        bannerViewPagerAdapter.notifyDataSetChanged();

        restaurants.clear();
        restaurants.addAll(searchResult.restaurants);
        setUpFavouriteVariable(searchResult);
        filteredRestaurants.clear();
        filteredRestaurants.addAll(restaurants);
        if (isFiltered) {
            filterByCuisine(String.valueOf(cuisine_id));
        } else {
            restaurantAdapter.notifyDataSetChanged();
        }
        if (notifyCuisine) {
            homeCuisineAdapter.notifyDataSetChanged();
        }

        if (myApp.getMyPreferences().getSiteSettings() != null && myApp.getMyPreferences().getSiteSettings().banner_status.equalsIgnoreCase("1")) {
            llBanners.setVisibility(View.VISIBLE);
        } else {
            llBanners.setVisibility(View.GONE);
        }

        homeApiStatus = 1;
        if (getActivity() != null)
            getActivity().runOnUiThread(() -> {
                swipeRefreshLayout.setRefreshing(false);
                llLoading.setVisibility(View.GONE);
                llData.setVisibility(View.VISIBLE);
            });
    }

    private OrderHistory lastOrder;

    private void fetchLastOrderDetails() {
        AndroidNetworking.get(NEW_ORDERS)
                .addQueryParameter("per_page", "1")
                .addQueryParameter("page", "1")
                .addQueryParameter("customer_id", String.valueOf(loggedInUser.id))
                .build()
                .getAsJSONObject(new JSONObjectRequestListener() {
                    @Override
                    public void onResponse(JSONObject response) {

                        try {
                            String logoUrl = response.getJSONArray("data").getJSONObject(0).getJSONObject("restaurant").getString("image_url");
                            ArrayList<OrderHistory> lastOrder = null;
                            if (response.getJSONArray("data").length() > 0) {
                                Type type = new TypeToken<List<OrderHistory>>() {
                                }.getType();
                                lastOrder = new ArrayList<>(new Gson().fromJson(response.getJSONArray("data").toString(), type));
//                                    lastOrder = new Gson().fromJson(response.getJSONArray("data").get(0).toString(), OrderHistory.class);
                            }

                               /* OrderHistory lastDineinOrder = null;
                                if (response.getJSONObject("result").has("LastDineinorders")) {
                                    lastDineinOrder = new Gson().fromJson(response.getJSONObject("result").getJSONObject("LastDineinorders").toString(), OrderHistory.class);
                                }*/
                            for (OrderHistory orderHistory : lastOrder) {
                                if (orderHistory != null && orderHistory.restaurant != null) {
                                    if (!orderHistory.status.equalsIgnoreCase("failed")) {
                                        orderHistory.logoUrl = logoUrl;
                                        snippetArrayList.add(orderHistory);
                                    }
                                }
                            }
                                /*if (lastDineinOrder != null && lastDineinOrder.restaurant != null) {

                                    if (!lastDineinOrder.status.equalsIgnoreCase("failed") && lastDineinOrder.reviews.size() == 0) {
                                        lastDineinOrder.logoUrl = logoUrl + lastDineinOrder.restaurant.id + "/" + response.getJSONObject("result").getString("restaurant_logo_folder_name") + "/" + lastDineinOrder.restaurant.logo_name;

                                        snippetArrayList.add(lastDineinOrder);
                                    }
                                }*/
                            if (getActivity() != null) {
                                getActivity().runOnUiThread(() -> {
                                    snippetAdapter.notifyDataSetChanged();
                                    if (snippetArrayList.size() <= 1) {
                                        snippetIndicator.setVisibility(View.GONE);
                                    } else {
                                        snippetIndicator.setVisibility(View.VISIBLE);
                                    }
                                    if (snippetArrayList.size() > 0)
                                        llMiniSnippet.setVisibility(View.VISIBLE);
                                    else llMiniSnippet.setVisibility(View.GONE);
                                });
                            }
                        } catch (JSONException e) {
                            e.printStackTrace();
                        }
                    }

                    @Override
                    public void onError(ANError anError) {
                        anError.printStackTrace();
                        if (anError.getErrorCode() == 401) {
                            myApp.logout(getActivity());
                        }
                        LogUtils.e("ERROR IN ORDER VIEW");
                    }
                });
    }


    private void setUpFavouriteVariable(SearchResult searchResult) {
        if (!Validators.isNullOrEmpty(searchResult.FavouriteList)) {
            String[] favourites = searchResult.FavouriteList.split("\\s*,\\s*");
            MyApp.favouriteList.clear();
            Collections.addAll(MyApp.favouriteList, favourites);
            for (Restaurant restaurant : restaurants) {
                int restIndex = MyApp.favouriteList.indexOf(String.valueOf(restaurant.id));

                restaurant.is_favourite = restIndex != -1 ? 1 : 0;
            }
        }

    }

    private void filterByCuisine(String cuisine) {
        filteredRestaurants.clear();
        if (cuisine.equals("All")) {
            filteredRestaurants.addAll(restaurants);
        } else {
            for (Restaurant restaurant : restaurants) {
                if (restaurant.restaurant_cuisine.contains(cuisine)) {
                    filteredRestaurants.add(restaurant);
                }
            }
        }
        restaurantAdapter.notifyDataSetChanged();
    }

    private void filterByCuisine(ArrayList<Cuisine> selectedCuisines) {
        filteredRestaurants.clear();
        if (selectedCuisines.size() == 0) {
            filteredRestaurants.addAll(restaurants);
        } else {
            for (Restaurant restaurant : restaurants) {
                if (!Validators.isNullOrEmpty(restaurant.restaurant_cuisine)) {
                    String str = restaurant.restaurant_cuisine;
                    String[] string = str.replaceAll("\\[", "")
                            .replaceAll("]", "")
                            .split(",");
                    int[] restaurantCuisines = new int[string.length];  // declaring an array with the size of string
                    for (int i = 0; i < string.length; i++) {
                        restaurantCuisines[i] = Integer.valueOf(string[i]);
                    }
                    boolean canAdd = false;
                    for (int i = 0; i < restaurantCuisines.length; i++) {
                        for (Cuisine c : selectedCuisines) {
                            if (String.valueOf(restaurantCuisines[i]).equalsIgnoreCase(String.valueOf(c.id))) {
                                canAdd = true;
                            }
                        }
                    }
                    if (canAdd) {
                        filteredRestaurants.add(restaurant);
                    }

                }
            }
        }
        restaurantAdapter.notifyDataSetChanged();
    }

    private DialogDismissListener sortDialogListener = o -> {
        currentSort = (int) o;
        sortRestaurants((Integer) o);
    };

    private DialogDismissListener locationDialogDismissListener = new DialogDismissListener() {
        @Override
        public void onDialogDismiss(Object o) {
            if (o instanceof Address) {
                defaultAddress = (Address) o;
                myApp.getMyPreferences().saveCurrentAddress(defaultAddress);
                displayAddress();
            } else if (o instanceof Postcode) {
                currentPostcode = (Postcode) o;
                displayAddress();
            }
        }
    };

    private void sortRestaurants(int sortMethod) {
        if (sortMethod == FILTER_POPULAR) {
            filteredRestaurants.clear();
            filteredRestaurants.addAll(getPopularSort(restaurants));
        }
        if (sortMethod == FILTER_DISTANCE_HIGH_LOW) {
            filteredRestaurants.clear();
            filteredRestaurants.addAll(getDistanceSort(restaurants, 1));
        }
        if (sortMethod == FILTER_DISTANCE_LOW_HIGH) {
            filteredRestaurants.clear();
            filteredRestaurants.addAll(getDistanceSort(restaurants, 0));
        }
        if (sortMethod == FILTER_RATING_HIGH_LOW) {
            filteredRestaurants.clear();
            filteredRestaurants.addAll(getRatingSort(restaurants, 1));
        }
        if (sortMethod == FILTER_RATING_LOW_HIGH) {
            filteredRestaurants.clear();
            filteredRestaurants.addAll(getRatingSort(restaurants, 0));
        }
        if (sortMethod == FILTER_MIN_ORDER_HIGH_LOW) {
            filteredRestaurants.clear();
            filteredRestaurants.addAll(getMinOrderSort(restaurants, 1));
        }
        if (sortMethod == FILTER_MIN_ORDER_LOW_HIGH) {
            filteredRestaurants.clear();
            filteredRestaurants.addAll(getMinOrderSort(restaurants, 0));
        }
        if (sortMethod == FILTER_CLEAR) {
            filteredRestaurants.clear();
            filteredRestaurants.addAll(getDistanceSort(restaurants, 0));
        }
        restaurantAdapter.notifyDataSetChanged();


    }

    private ArrayList<Restaurant> getDistanceSort(ArrayList<Restaurant> list, int sort) {
        if (sort == 0) {
            //ascending
            Collections.sort(list, (lhs, rhs) -> Float.compare(lhs.distance, (rhs.distance)));
        } else {
            Collections.sort(list, (lhs, rhs) -> Float.compare(rhs.distance, (lhs.distance)));
        }

        return list;
    }

    private ArrayList<Restaurant> getRatingSort(ArrayList<Restaurant> list, int sort) {
        if (sort == 0) {
            //ascending
            Collections.sort(list, (lhs, rhs) -> Float.compare(lhs.average_rating, (rhs.average_rating)));
        } else {
            Collections.sort(list, (lhs, rhs) -> Float.compare(rhs.average_rating, (lhs.average_rating)));
        }

        return list;
    }

    private ArrayList<Restaurant> getMinOrderSort(ArrayList<Restaurant> list, int sort) {
        if (sort == 0) {
            //ascending
            Collections.sort(list, (lhs, rhs) -> Float.compare(lhs.minimum_order, (rhs.minimum_order)));
        } else {
            Collections.sort(list, (lhs, rhs) -> Float.compare(rhs.minimum_order, (lhs.minimum_order)));
        }

        return list;
    }

    private ArrayList<Restaurant> getPopularSort(ArrayList<Restaurant> list) {
        Collections.sort(list, (lhs, rhs) -> Integer.compare(rhs.no_order_count, (lhs.no_order_count)));
        ;
        return list;
    }

    private ArrayList<Restaurant> searchRestaurant() {
        if (etSearch.getText().toString().isEmpty()) {
            return restaurants;
        }
        ArrayList<Restaurant> searchedRest = new ArrayList<>();
        for (Restaurant r : restaurants) {
            if (r.restaurant_name != null && r.restaurant_name.toLowerCase().contains(etSearch.getText().toString().toLowerCase())) {
                searchedRest.add(r);
            }
        }
        return searchedRest;

    }

    @Override
    public void onActivityResult(int requestCode, int resultCode, @Nullable Intent data) {
        super.onActivityResult(requestCode, resultCode, data);
        if (requestCode == CODE_REFRESH) {
            if (resultCode == RESULT_OK) {
                fetchData();
            }
        } else if (requestCode == CODE_NO_INTERNET) {
            fetchData();
        }
    }

    @Override
    public void onRequestPermissionsResult(int requestCode, String[] permissions, int[] grantResults) {
        if (requestCode == PERMISSION_CODE_LOCATION) {
            if (grantResults.length > 0 && grantResults[0] == PackageManager.PERMISSION_GRANTED) {
                if (ActivityCompat.checkSelfPermission(getActivity(),
                        Manifest.permission.CAMERA)
                        == PackageManager.PERMISSION_GRANTED) {
                    Intent i = new Intent(getActivity(), DineInScanActivity.class);
                    startActivity(i);
                } else {
                    ToastUtils.makeToast(getActivity(), "Permission denied.");
                }

            } else {
                if (!ActivityCompat.shouldShowRequestPermissionRationale(getActivity(),
                        Manifest.permission.CAMERA)) {
                    new android.app.AlertDialog.Builder(getActivity())
                            .setTitle("Camera permission required")
                            .setMessage("To scan we require camera permission, Please allow to continue.")
                            .setPositiveButton("ALLOW", (dialogInterface, i) -> {
                                Intent intent = new Intent();
                                intent.setAction(Settings.ACTION_APPLICATION_DETAILS_SETTINGS);
                                Uri uri = Uri.fromParts("package", getActivity().getPackageName(), null);
                                intent.setData(uri);
                                startActivity(intent);
                            })
                            .setNegativeButton("DENY", (dialogInterface, i) -> dialogInterface.dismiss())
                            .create()
                            .show();
                } else {
                    ToastUtils.makeToast(getActivity(), "Permission denied.");
                }

            }
        }
        if (requestCode == REQUEST_LOCATION) {
            if (grantResults.length > 0 && grantResults[0] == PackageManager.PERMISSION_GRANTED) {
                if (ActivityCompat.checkSelfPermission(getActivity(),
                        Manifest.permission.ACCESS_FINE_LOCATION)
                        == PackageManager.PERMISSION_GRANTED) {
                    startLocationService();
                } else {
                    ToastUtils.makeToast(getActivity(), "Permission denied.");
                }

            } else {
                if (!ActivityCompat.shouldShowRequestPermissionRationale(getActivity(),
                        Manifest.permission.ACCESS_FINE_LOCATION)) {
                    new android.app.AlertDialog.Builder(getActivity())
                            .setTitle("Location permission required")
                            .setMessage("To access your current postcode we require your location permission, Please allow to continue.")
                            .setPositiveButton("ALLOW", (dialogInterface, i) -> {
                                Intent intent = new Intent();
                                intent.setAction(Settings.ACTION_APPLICATION_DETAILS_SETTINGS);
                                Uri uri = Uri.fromParts("package", getActivity().getPackageName(), null);
                                intent.setData(uri);
                                startActivity(intent);
                            })
                            .setNegativeButton("DENY", (dialogInterface, i) -> dialogInterface.dismiss())
                            .create()
                            .show();
                } else {
                    ToastUtils.makeToast(getActivity(), "Permission denied.");
                }

            }
        }

    }

    @Override
    public void onResume() {

        updateMiniView();
        dineinAnimationView.playAnimation();
        super.onResume();
    }
}
