package com.tiffintom.fragment;

import android.os.Build;
import android.os.Bundle;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.Button;
import android.widget.ImageView;
import android.widget.LinearLayout;
import android.widget.TextView;

import androidx.annotation.NonNull;
import androidx.annotation.Nullable;

import com.google.android.material.bottomsheet.BottomSheetDialogFragment;
import com.tiffintom.MyApp;
import com.tiffintom.R;
import com.tiffintom.interfaces.DialogDismissListener;
import com.tiffintom.models.User;

import static com.tiffintom.utils.Constants.FILTER_CLEAR;
import static com.tiffintom.utils.Constants.FILTER_DISTANCE_HIGH_LOW;
import static com.tiffintom.utils.Constants.FILTER_DISTANCE_HIGH_LOW_STRING;
import static com.tiffintom.utils.Constants.FILTER_DISTANCE_LOW_HIGH;
import static com.tiffintom.utils.Constants.FILTER_DISTANCE_LOW_HIGH_STRING;
import static com.tiffintom.utils.Constants.FILTER_MIN_ORDER_HIGH_LOW;
import static com.tiffintom.utils.Constants.FILTER_MIN_ORDER_HIGH_LOW_STRING;
import static com.tiffintom.utils.Constants.FILTER_MIN_ORDER_LOW_HIGH;
import static com.tiffintom.utils.Constants.FILTER_MIN_ORDER_LOW_HIGH_STRING;
import static com.tiffintom.utils.Constants.FILTER_NONE;
import static com.tiffintom.utils.Constants.FILTER_POPULAR;
import static com.tiffintom.utils.Constants.FILTER_POPULAR_STRING;
import static com.tiffintom.utils.Constants.FILTER_RATING_HIGH_LOW;
import static com.tiffintom.utils.Constants.FILTER_RATING_HIGH_LOW_STRING;
import static com.tiffintom.utils.Constants.FILTER_RATING_LOW_HIGH;
import static com.tiffintom.utils.Constants.FILTER_RATING_LOW_HIGH_STRING;

/**
 * Created by Amrish on 19-08-2020.
 */
public class HomeSortBottomDialogFragment extends BottomSheetDialogFragment {

    private User loggedInUser;
    private MyApp myApp = MyApp.getInstance();
    private DialogDismissListener dialogDismissListener;
    private LinearLayout llMostPopular, llDistance, llRating, llOrderValue;
    private TextView tvOrderValue, tvPopular, tvDistance, tvRating;
    private ImageView ivMostPopular, ivDistance, ivRating, ivOrderValue;
    private int currentFilter = FILTER_NONE;
    private Button btnConfirm, btnReset;

    public static HomeSortBottomDialogFragment getInstance(int currentFilter) {
        HomeSortBottomDialogFragment fragment = new HomeSortBottomDialogFragment();
        Bundle bundle = new Bundle();
        bundle.putInt("currentFilter", currentFilter);
        fragment.setArguments(bundle);
        return fragment;
    }

    @Override
    public int getTheme() {
        if (Build.VERSION.SDK_INT > 21) {
            return R.style.BottomSheetDialogTheme;
        } else {
            return super.getTheme();
        }
    }

    @Nullable
    @Override
    public View onCreateView(@NonNull LayoutInflater inflater, @Nullable ViewGroup container, @Nullable Bundle savedInstanceState) {
        if (getArguments() != null)
            currentFilter = getArguments().getInt("currentFilter", FILTER_NONE);
        return inflater.inflate(R.layout.fragment_sort_dialog, container, false);
    }

    @Override
    public void onViewCreated(@NonNull View view, @Nullable Bundle savedInstanceState) {
        super.onViewCreated(view, savedInstanceState);
        loggedInUser = myApp.getMyPreferences().getLoggedInUser();
        initViews(view);
        updateFilterText();
        setFilterUI(currentFilter);
        setListeners();
    }

    public void setDialogDismissListener(DialogDismissListener dialogDismissListener) {
        this.dialogDismissListener = dialogDismissListener;
    }

    protected void initViews(View view) {
        llDistance = view.findViewById(R.id.llDistance);
        llMostPopular = view.findViewById(R.id.llMostPopular);
        llOrderValue = view.findViewById(R.id.llOrderValue);
        llRating = view.findViewById(R.id.llRating);

        ivDistance = view.findViewById(R.id.ivDistance);
        ivMostPopular = view.findViewById(R.id.ivMostPopular);
        ivOrderValue = view.findViewById(R.id.ivOrderValue);
        ivRating = view.findViewById(R.id.ivRating);
        btnConfirm = view.findViewById(R.id.btnConfirm);
        btnReset = view.findViewById(R.id.btnReset);

        tvOrderValue = view.findViewById(R.id.tvOrderValue);
        tvDistance = view.findViewById(R.id.tvDistance);
        tvRating = view.findViewById(R.id.tvRating);
        tvPopular = view.findViewById(R.id.tvPopular);
    }

    private void setListeners() {
        llMostPopular.setOnClickListener(view -> {
            currentFilter = FILTER_POPULAR;
            setFilterUI(currentFilter);
        });
        llDistance.setOnClickListener(view -> {
//            if (currentFilter == FILTER_DISTANCE_LOW_HIGH) {
//                currentFilter = FILTER_DISTANCE_HIGH_LOW;
//            } else if (currentFilter == FILTER_DISTANCE_HIGH_LOW) {
//                currentFilter = FILTER_CLEAR;
//            } else {
//                currentFilter = FILTER_DISTANCE_LOW_HIGH;
//            }
            currentFilter = FILTER_DISTANCE_LOW_HIGH;
            setFilterUI(currentFilter);
        });
        llRating.setOnClickListener(view -> {
//            if (currentFilter == FILTER_RATING_HIGH_LOW) {
//                currentFilter = FILTER_RATING_LOW_HIGH;
//            } else if (currentFilter == FILTER_RATING_LOW_HIGH) {
//                currentFilter = FILTER_CLEAR;
//            } else {
//                currentFilter = FILTER_RATING_HIGH_LOW;
//            }
            currentFilter = FILTER_RATING_HIGH_LOW;
            setFilterUI(currentFilter);
        });
        llOrderValue.setOnClickListener(view -> {
//            if (currentFilter < FILTER_MIN_ORDER_LOW_HIGH) {
//                currentFilter = FILTER_MIN_ORDER_LOW_HIGH;
//            } else if (currentFilter == FILTER_MIN_ORDER_LOW_HIGH) {
//                currentFilter = FILTER_MIN_ORDER_HIGH_LOW;
//            } else {
//                currentFilter = FILTER_CLEAR;
//            }
            if (currentFilter == FILTER_MIN_ORDER_LOW_HIGH) {
                currentFilter = FILTER_MIN_ORDER_HIGH_LOW;
            } else {
                currentFilter = FILTER_MIN_ORDER_LOW_HIGH;
            }
            setFilterUI(currentFilter);
        });
        btnConfirm.setOnClickListener(view -> {
            if (dialogDismissListener != null) {
                dialogDismissListener.onDialogDismiss(currentFilter);
            }
            dismiss();
        });
        btnReset.setOnClickListener(view -> {
            currentFilter = FILTER_CLEAR;
            setFilterUI(currentFilter);
            if (dialogDismissListener != null) {
                dialogDismissListener.onDialogDismiss(currentFilter);
            }
            dismiss();
        });
    }

    private void setFilterUI(int filterCode) {
        if (filterCode == FILTER_POPULAR) {
            ivRating.setVisibility(View.INVISIBLE);
            ivDistance.setVisibility(View.INVISIBLE);
            ivOrderValue.setVisibility(View.INVISIBLE);
            ivMostPopular.setVisibility(View.VISIBLE);
        }
        if (filterCode == FILTER_DISTANCE_HIGH_LOW || filterCode == FILTER_DISTANCE_LOW_HIGH) {
            ivRating.setVisibility(View.INVISIBLE);
            ivOrderValue.setVisibility(View.INVISIBLE);
            ivMostPopular.setVisibility(View.INVISIBLE);
            ivDistance.setVisibility(View.VISIBLE);
        }
        if (filterCode == FILTER_MIN_ORDER_HIGH_LOW || filterCode == FILTER_MIN_ORDER_LOW_HIGH) {
            ivRating.setVisibility(View.INVISIBLE);
            ivMostPopular.setVisibility(View.INVISIBLE);
            ivDistance.setVisibility(View.INVISIBLE);
            ivOrderValue.setVisibility(View.VISIBLE);
        }
        if (filterCode == FILTER_RATING_HIGH_LOW || filterCode == FILTER_RATING_LOW_HIGH) {
            ivMostPopular.setVisibility(View.INVISIBLE);
            ivDistance.setVisibility(View.INVISIBLE);
            ivOrderValue.setVisibility(View.INVISIBLE);
            ivRating.setVisibility(View.VISIBLE);
        }
        if (filterCode == FILTER_CLEAR) {
            ivMostPopular.setVisibility(View.INVISIBLE);
            ivDistance.setVisibility(View.INVISIBLE);
            ivOrderValue.setVisibility(View.INVISIBLE);
            ivRating.setVisibility(View.INVISIBLE);
        }
        updateFilterText();
    }

    private void updateFilterText() {
        if (currentFilter == FILTER_MIN_ORDER_HIGH_LOW) {
            tvOrderValue.setText(FILTER_MIN_ORDER_HIGH_LOW_STRING);
        }
        if (currentFilter == FILTER_MIN_ORDER_LOW_HIGH) {
            tvOrderValue.setText(FILTER_MIN_ORDER_LOW_HIGH_STRING);
        }

        if (currentFilter == FILTER_RATING_LOW_HIGH) {
            tvRating.setText(FILTER_RATING_LOW_HIGH_STRING);
        }
        if (currentFilter == FILTER_RATING_LOW_HIGH) {
            tvRating.setText(FILTER_RATING_HIGH_LOW_STRING);
        }
        if (currentFilter == FILTER_DISTANCE_LOW_HIGH) {
            tvDistance.setText(FILTER_DISTANCE_LOW_HIGH_STRING);
        }
        if (currentFilter == FILTER_DISTANCE_HIGH_LOW) {
            tvDistance.setText(FILTER_DISTANCE_HIGH_LOW_STRING);
        }
        if (currentFilter == FILTER_POPULAR) {
            tvPopular.setText(FILTER_POPULAR_STRING);
        }
    }

}
