package com.tiffintom.fragment;

import android.Manifest;
import android.app.Dialog;
import android.content.DialogInterface;
import android.content.Intent;
import android.content.pm.PackageManager;
import android.location.Location;
import android.net.Uri;
import android.os.Build;
import android.os.Bundle;
import android.provider.Settings;
import android.view.KeyEvent;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.EditText;
import android.widget.FrameLayout;
import android.widget.ImageView;
import android.widget.ProgressBar;
import android.widget.TextView;

import androidx.annotation.NonNull;
import androidx.annotation.Nullable;
import androidx.appcompat.app.AlertDialog;
import androidx.core.app.ActivityCompat;
import androidx.recyclerview.widget.DividerItemDecoration;
import androidx.recyclerview.widget.LinearLayoutManager;
import androidx.recyclerview.widget.RecyclerView;

import com.airbnb.lottie.LottieAnimationView;
import com.androidnetworking.AndroidNetworking;
import com.androidnetworking.error.ANError;
import com.androidnetworking.interfaces.JSONObjectRequestListener;
import com.androidnetworking.interfaces.ParsedRequestListener;
import com.google.android.material.bottomsheet.BottomSheetBehavior;
import com.google.android.material.bottomsheet.BottomSheetDialog;
import com.google.android.material.bottomsheet.BottomSheetDialogFragment;
import com.google.gson.Gson;
import com.google.gson.reflect.TypeToken;
import com.tiffintom.MyApp;
import com.tiffintom.R;
import com.tiffintom.adapters.AddressAdapter;
import com.tiffintom.common.CommonFunctions;
import com.tiffintom.common.MyLocation;
import com.tiffintom.common.Validators;
import com.tiffintom.interfaces.DialogDismissListener;
import com.tiffintom.models.Address;
import com.tiffintom.models.ApiError;
import com.tiffintom.models.Postcode;
import com.tiffintom.network.ApiEndPoints;
import com.tiffintom.network.ApiUtils;
import com.tiffintom.utils.LogUtils;
import com.tiffintom.utils.ToastUtils;

import org.json.JSONException;
import org.json.JSONObject;

import java.lang.reflect.Type;
import java.util.ArrayList;
import java.util.List;

import static com.tiffintom.utils.Constants.PERMISSION_CODE_LOCATION;

/**
 * Created by Amrish on 20-08-2020.
 */
public class LocationSelectionBottomSheetFragment extends BottomSheetDialogFragment {
    private ImageView ivClose, ivSearch;
    private MyApp myApp = MyApp.getInstance();
    private AlertDialog progressDialog;
    private EditText etSearch;
    private RecyclerView rvAddress;
    private ProgressBar pbLoading;
    private ArrayList<Address> addresses = new ArrayList<>();
    private AddressAdapter addressAdapter;
    private DialogDismissListener dialogDismissListener;
    private TextView tvSavedAddress, tvCurrentLocation;
    private LottieAnimationView lodingView;

    public void setDialogDismissListener(DialogDismissListener dialogDismissListener) {
        this.dialogDismissListener = dialogDismissListener;
    }

    @Override
    public int getTheme() {
        if (Build.VERSION.SDK_INT > 21) {
            return R.style.BottomSheetDialogTheme;
        } else {
            return super.getTheme();
        }
    }

    @NonNull
    @Override
    public Dialog onCreateDialog(Bundle savedInstanceState) {

        BottomSheetDialog dialog = (BottomSheetDialog) super.onCreateDialog(savedInstanceState);

        dialog.setOnShowListener((DialogInterface.OnShowListener) dialog1 -> {
            BottomSheetDialog d = (BottomSheetDialog) dialog1;
            FrameLayout bottomSheet = (FrameLayout) d.findViewById(com.google.android.material.R.id.design_bottom_sheet);
            if (bottomSheet != null)
                BottomSheetBehavior.from(bottomSheet).setState(BottomSheetBehavior.STATE_EXPANDED);
        });

        return dialog;
    }

    @Nullable
    @Override
    public View onCreateView(@NonNull LayoutInflater inflater, @Nullable ViewGroup container, @Nullable Bundle savedInstanceState) {
        return inflater.inflate(R.layout.fragment_location_selection, container, false);
    }

    @Override
    public void onViewCreated(@NonNull View view, @Nullable Bundle savedInstanceState) {
        super.onViewCreated(view, savedInstanceState);
        initViews(view);
        setListeners();

        if (myApp.getMyPreferences().getLoggedInUser() != null) {
            new Thread(this::fetchAddress).start();
        }

    }


    private void initViews(View view) {
        progressDialog = CommonFunctions.customProgressDialog(getActivity(), "Loading...");
        ivClose = view.findViewById(R.id.ivClose);
        ivSearch = view.findViewById(R.id.ivSearch);
        etSearch = view.findViewById(R.id.etPincode);
        pbLoading = view.findViewById(R.id.pbLoading);
        rvAddress = view.findViewById(R.id.rvAddress);
        tvSavedAddress = view.findViewById(R.id.tvSavedAddress);
        tvCurrentLocation = view.findViewById(R.id.tvCurrentLocation);
        lodingView = view.findViewById(R.id.lodingView);

        addressAdapter = new AddressAdapter(getActivity(), addresses, (position, data) -> {
            if (dialogDismissListener != null) {
                myApp.getMyPreferences().saveCurrentAddress((Address) data);
                dialogDismissListener.onDialogDismiss(data);
                dismiss();
            }
        });
        rvAddress.setLayoutManager(new LinearLayoutManager(getActivity(), RecyclerView.VERTICAL, false));
        rvAddress.addItemDecoration(new DividerItemDecoration(getActivity(), DividerItemDecoration.VERTICAL));
        rvAddress.setAdapter(addressAdapter);
        etSearch.clearFocus();
    }

    private void setListeners() {
        ivClose.setOnClickListener(view -> dismiss());
        ivSearch.setOnClickListener(view -> {
            if (getActivity() != null)
                if (isValid()) {
                    String postCode = etSearch.getText().toString();
                    new Thread(() -> {
                        searchPostcode(postCode);
                    }).start();
                }
            ;
        });
        etSearch.setOnEditorActionListener((textView, i, keyEvent) -> {
            if (keyEvent != null && keyEvent.getKeyCode() == KeyEvent.KEYCODE_ENTER) {
                if (getActivity() != null) {
                    if (isValid()) {
                        String postCode = etSearch.getText().toString();
                        new Thread(() -> {
                            searchPostcode(postCode);
                        }).start();
                    }
                }
            }
            return false;
        });
        tvCurrentLocation.setOnClickListener(view -> {
            if (checkLocationPermission()) {
                startLocationService();
            }
        });
    }

    private boolean isValid() {
        if (Validators.isNullOrEmpty(etSearch.getText().toString())) {
            etSearch.setError("Please enter valid post code");
            etSearch.requestFocus();
            return false;
        }
        return true;
    }

    private void searchPostcode(String postcode) {
        if (getActivity() != null) getActivity().runOnUiThread(progressDialog::show);
        AndroidNetworking.get(ApiEndPoints.RMS_POSTCODE + postcode)
                .build()
                .getAsObject(Postcode.class, new ParsedRequestListener<Postcode>() {
                    @Override
                    public void onResponse(Postcode response) {
                        if (getActivity() != null)
                            CommonFunctions.hideKeyboard(getActivity(), etSearch);
                        if (getActivity() != null)
                            getActivity().runOnUiThread(() -> {
                                progressDialog.dismiss();
                            });
                        myApp.getMyPreferences().saveCurrentPostcode(response);
                        if (dialogDismissListener != null) {
                            dialogDismissListener.onDialogDismiss(postcode);
                        }
                        dismiss();
                    }

                    @Override
                    public void onError(ANError anError) {
                        if (getActivity() != null)
                            getActivity().runOnUiThread(() -> progressDialog.dismiss());
                        if (anError.getErrorCode() == 400) {
                            ApiError apiError = anError.getErrorAsObject(ApiError.class);
                            ToastUtils.makeSnackToast(getActivity(), apiError.getMessage());
                        }
                        if (!CommonFunctions.isConnected(getActivity())) {
                            myApp.noInternet(getActivity());
                        }
                    }
                });
    }

    private void fetchAddress() {
        if (getActivity() != null) {
            getActivity().runOnUiThread(() -> lodingView.setVisibility(View.VISIBLE));
        }
        ApiUtils.getAddresses(String.valueOf(myApp.getMyPreferences().getLoggedInUser().id))
                .getAsJSONObject(new JSONObjectRequestListener() {
                    @Override
                    public void onResponse(JSONObject response) {
                        String errorMessage = myApp.checkErrorMessage(response);
                        if (errorMessage == null) {

                            Type type = new TypeToken<List<Address>>() {
                            }.getType();
                            try {
                                addresses.clear();
                                addresses.addAll(new Gson().fromJson(response.getJSONObject("result").getJSONArray("addressBook").toString(), type));
                                addressAdapter.notifyDataSetChanged();
                                if (getActivity() != null) {
                                    getActivity().runOnUiThread(() -> tvSavedAddress.setVisibility(View.VISIBLE));
                                }
                            } catch (JSONException e) {
                                e.printStackTrace();
                                LogUtils.e("JSON EXCEPTION ", e.getLocalizedMessage());
                            }
                        } else {
                            LogUtils.e(errorMessage);
                        }
                        if (getActivity() != null) {
                            getActivity().runOnUiThread(() -> lodingView.setVisibility(View.GONE));
                        }
                    }

                    @Override
                    public void onError(ANError anError) {
                        anError.printStackTrace();
                        if (getActivity() != null) {
                            getActivity().runOnUiThread(() -> lodingView.setVisibility(View.GONE));
                        }
                    }
                });
    }

    private DialogDismissListener addAddressDialogDismissListener = o -> {
        if (dialogDismissListener != null)
            dialogDismissListener.onDialogDismiss(o);
        dismiss();
    };

    public boolean checkLocationPermission() {
        if (ActivityCompat.checkSelfPermission(getActivity(),
                Manifest.permission.ACCESS_FINE_LOCATION)
                != PackageManager.PERMISSION_GRANTED) {

            // Should we show an explanation?
            if (ActivityCompat.shouldShowRequestPermissionRationale(getActivity(),
                    Manifest.permission.ACCESS_FINE_LOCATION)) {

                new android.app.AlertDialog.Builder(getActivity())
                        .setTitle("Location permission required")
                        .setMessage("To access your current postcode we require your location permission, Please allow to continue.")
                        .setPositiveButton("ALLOW", (dialogInterface, i) -> {
                            requestPermissions(new String[]{Manifest.permission.ACCESS_FINE_LOCATION}, PERMISSION_CODE_LOCATION);
                        })
                        .setNegativeButton("DENY", (dialogInterface, i) -> dialogInterface.dismiss())
                        .create()
                        .show();


            } else {
                requestPermissions(new String[]{Manifest.permission.ACCESS_FINE_LOCATION}, PERMISSION_CODE_LOCATION);
            }
            return false;
        } else {
            return true;
        }
    }


    @Override
    public void onRequestPermissionsResult(int requestCode, String permissions[], int[] grantResults) {
        if (requestCode == PERMISSION_CODE_LOCATION) {
            if (grantResults.length > 0 && grantResults[0] == PackageManager.PERMISSION_GRANTED) {
                if (ActivityCompat.checkSelfPermission(getActivity(),
                        Manifest.permission.ACCESS_FINE_LOCATION)
                        == PackageManager.PERMISSION_GRANTED) {
                    startLocationService();
                } else {
                    ToastUtils.makeToast(getActivity(), "Permission denied.");
                }

            } else {
                if (!ActivityCompat.shouldShowRequestPermissionRationale(getActivity(),
                        Manifest.permission.ACCESS_FINE_LOCATION)) {
                    new android.app.AlertDialog.Builder(getActivity())
                            .setTitle("Location permission required")
                            .setMessage("To access your current postcode we require your location permission, Please allow to continue.")
                            .setPositiveButton("ALLOW", (dialogInterface, i) -> {
                                Intent intent = new Intent();
                                intent.setAction(Settings.ACTION_APPLICATION_DETAILS_SETTINGS);
                                Uri uri = Uri.fromParts("package", getActivity().getPackageName(), null);
                                intent.setData(uri);
                                startActivity(intent);
                            })
                            .setNegativeButton("DENY", (dialogInterface, i) -> dialogInterface.dismiss())
                            .create()
                            .show();
                } else {
                    ToastUtils.makeToast(getActivity(), "Permission denied.");
                }

            }
        }
    }

    MyLocation myLocation = new MyLocation();

    private void startLocationService() {
        progressDialog.show();
        myLocation.getLocation(getActivity(), new MyLocation.LocationResult() {
            @Override
            public void gotLocation(Location location) {
                if (getActivity() != null) {
                    getActivity().runOnUiThread(() -> {
                        progressDialog.dismiss();
                    });
                }

                android.location.Address address = CommonFunctions.getAddressByLocation(getActivity(), location.getLatitude(), location.getLongitude());
                if (address != null) {
                    String zipcode = address.getPostalCode();
                    if (!Validators.isNullOrEmpty(zipcode)) {
                        if (getActivity() != null) {
                            getActivity().runOnUiThread(() -> {
                                etSearch.setText(zipcode);
                            });
                        }
                        new Thread(() -> {
                            searchPostcode(zipcode);
                        }).start();

                    }
                }
            }
        });
    }

    @Override
    public void onDismiss(@NonNull DialogInterface dialog) {
        if (progressDialog != null) {
            progressDialog.dismiss();
        }
        super.onDismiss(dialog);
    }
}
