package com.tiffintom.fragment;

import static com.tiffintom.utils.Constants.CODE_REFRESH;

import android.app.DatePickerDialog;
import android.app.Dialog;
import android.app.TimePickerDialog;
import android.content.DialogInterface;
import android.content.Intent;
import android.os.Build;
import android.os.Bundle;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.Button;
import android.widget.EditText;
import android.widget.FrameLayout;
import android.widget.ImageView;
import android.widget.LinearLayout;
import android.widget.RadioGroup;
import android.widget.TextView;

import androidx.annotation.NonNull;
import androidx.annotation.Nullable;
import androidx.appcompat.app.AlertDialog;

import com.androidnetworking.AndroidNetworking;
import com.androidnetworking.error.ANError;
import com.androidnetworking.interfaces.JSONObjectRequestListener;
import com.google.android.material.bottomsheet.BottomSheetBehavior;
import com.google.android.material.bottomsheet.BottomSheetDialog;
import com.google.android.material.bottomsheet.BottomSheetDialogFragment;
import com.google.gson.Gson;
import com.tiffintom.MyApp;
import com.tiffintom.R;
import com.tiffintom.activity.AuthActivity;
import com.tiffintom.activity.TransportActivity;
import com.tiffintom.adapters.MenuVariantAdapter;
import com.tiffintom.common.CommonFunctions;
import com.tiffintom.common.Validators;
import com.tiffintom.interfaces.DialogDismissListener;
import com.tiffintom.models.OrderDetail;
import com.tiffintom.models.Reservation;
import com.tiffintom.models.TimeSlot;
import com.tiffintom.models.User;
import com.tiffintom.models.Variant;
import com.tiffintom.network.ApiEndPoints;
import com.tiffintom.network.ApiUtils;
import com.tiffintom.utils.LogUtils;
import com.tiffintom.utils.ToastUtils;

import org.json.JSONException;
import org.json.JSONObject;

import java.util.ArrayList;
import java.util.Calendar;

/**
 * Created by Amrish on 20-08-2020.
 */
public class MakeReservationBottomSheetFragment extends BottomSheetDialogFragment implements DialogDismissListener {
    private ImageView ivBasket, ivPlus, ivMinus, ivItem;
    private MyApp myApp = MyApp.getInstance();
    private AlertDialog progressDialog;
    private EditText etInstructions;
    private RadioGroup rgNumberOfPeople;
    private ArrayList<Variant> variants = new ArrayList<>();
    private DialogDismissListener dialogDismissListener;
    private TextView tvReservationDate, tvReservationTime, tvName, tvEmail, tvPhone, tvChange;
    private LinearLayout llDate, llTime;
    private MenuVariantAdapter addOnAdapter;
    private User loggedInCustomer;
    private DatePickerDialog datepicker;
    private TimePickerDialog timePickerDialog;
    private Button btnConfirm;
    private Reservation reservation;
    private int rest_id = 0;
    private TimeSlot selectedTimeSlot;
    private boolean moveNext = false;

    public static MakeReservationBottomSheetFragment getInstance(int rest_id) {
        MakeReservationBottomSheetFragment fragment = new MakeReservationBottomSheetFragment();
        Bundle bundle = new Bundle();
        bundle.putInt("rest_id", rest_id);
        fragment.setArguments(bundle);
        return fragment;
    }

    public void setDialogDismissListener(DialogDismissListener dialogDismissListener) {
        this.dialogDismissListener = dialogDismissListener;
    }

    @Override
    public int getTheme() {
        if (Build.VERSION.SDK_INT > 21) {
            return R.style.BottomSheetDialogTheme;
        } else {
            return super.getTheme();
        }
    }

    @NonNull
    @Override
    public Dialog onCreateDialog(Bundle savedInstanceState) {

        BottomSheetDialog dialog = (BottomSheetDialog) super.onCreateDialog(savedInstanceState);

        dialog.setOnShowListener((DialogInterface.OnShowListener) dialog1 -> {
            BottomSheetDialog d = (BottomSheetDialog) dialog1;
            FrameLayout bottomSheet = (FrameLayout) d.findViewById(com.google.android.material.R.id.design_bottom_sheet);
            if (bottomSheet != null)
                BottomSheetBehavior.from(bottomSheet).setState(BottomSheetBehavior.STATE_EXPANDED);
        });

        return dialog;
    }

    @Nullable
    @Override
    public View onCreateView(@NonNull LayoutInflater inflater, @Nullable ViewGroup container, @Nullable Bundle savedInstanceState) {
        if (getArguments() != null) {
            rest_id = getArguments().getInt("rest_id");
        }
        return inflater.inflate(R.layout.fragment_make_reservation, container, false);
    }

    @Override
    public void onViewCreated(@NonNull View view, @Nullable Bundle savedInstanceState) {
        super.onViewCreated(view, savedInstanceState);
        initViews(view);
        setListeners();
        updateViews();
    }


    private void initViews(View view) {
        reservation = new Reservation();
        if (myApp.getMyPreferences().getLoggedInUser() != null) {
            loggedInCustomer = myApp.getMyPreferences().getLoggedInUser();
        }
        progressDialog = CommonFunctions.customProgressDialog(getActivity(), "Loading...");
        progressDialog.getWindow().setDimAmount(0.5f);
        tvChange = view.findViewById(R.id.tvChange);
        tvEmail = view.findViewById(R.id.tvEmail);
        tvName = view.findViewById(R.id.tvName);
        tvPhone = view.findViewById(R.id.tvPhone);
        tvReservationDate = view.findViewById(R.id.tvReservationDate);
        tvReservationTime = view.findViewById(R.id.tvReservationTime);
        rgNumberOfPeople = view.findViewById(R.id.rgNumberOfPeople);
        etInstructions = view.findViewById(R.id.etInstructions);
        llDate = view.findViewById(R.id.cvReservationDate);
        llTime = view.findViewById(R.id.cvReservationTime);
        btnConfirm = view.findViewById(R.id.btnConfirm);


        final Calendar calendar = Calendar.getInstance();
        int day = calendar.get(Calendar.DAY_OF_MONTH);
        int month = calendar.get(Calendar.MONTH);
        int year = calendar.get(Calendar.YEAR);
        int hour = calendar.get(Calendar.HOUR_OF_DAY);
        int minute = calendar.get(Calendar.MINUTE);
        int second = calendar.get(Calendar.SECOND);
        datepicker = new DatePickerDialog(getActivity(),
                (view1, year1, monthOfYear, dayOfMonth) -> {
                    reservation.booking_date = String.format("%02d", dayOfMonth) + "-" + String.format("%02d", monthOfYear + 1) + "-" + year1;
                    tvReservationDate.setText(reservation.booking_date);
                    tvReservationTime.setText("");
                    selectedTimeSlot = null;
                    reservation.booking_time = null;

                    fetchTimeSlots();
                }, year, month, day);

        timePickerDialog = new TimePickerDialog(getActivity(), (timePicker, h, m) -> {
            reservation.booking_time = String.format("%02d", h) + ":" + String.format("%02d", m);
            tvReservationTime.setText(reservation.booking_time);
        }, hour, minute, false);
    }

    private void updateViews() {
        if (loggedInCustomer != null) {
            tvName.setText(loggedInCustomer.first_name + " " + loggedInCustomer.last_name);
            tvEmail.setText(loggedInCustomer.username);
            tvPhone.setText(loggedInCustomer.phone_number);
        }
    }

    private void setListeners() {

        llDate.setOnClickListener(view -> {
            Calendar calendar = Calendar.getInstance();
            calendar.add(Calendar.MONTH, +4);
            datepicker.getDatePicker().setMinDate(System.currentTimeMillis());
            datepicker.getDatePicker().setMaxDate(calendar.getTimeInMillis());
            datepicker.show();
        });
        llTime.setOnClickListener(view -> {
            if (!Validators.isNullOrEmpty(restaurantTimeSlots)) {
                ShowTimeSlotBottomSheetFragment timeSlotBottomSheetFragment = ShowTimeSlotBottomSheetFragment.getInstance(restaurantTimeSlots, true, selectedTimeSlot);
                timeSlotBottomSheetFragment.show(getChildFragmentManager(), "timeslots");
                timeSlotBottomSheetFragment.setDialogDismissListener(o -> {
                    selectedTimeSlot = (TimeSlot) o;
                    reservation.booking_time = selectedTimeSlot.time;
                    tvReservationTime.setText(selectedTimeSlot.name);
                });
            } else {
                moveNext = true;
//              llDate.performClick();
                showRestaurantIsClosed("Restaurant is not providing reservation at the moment",false);

            }
        });
        btnConfirm.setOnClickListener(view -> {
            if (isValid()) bookATable();
        });
        tvChange.setOnClickListener(view -> {
            reservation.booking_email = tvEmail.getText().toString();
            reservation.booking_phone = tvPhone.getText().toString();
            reservation.customer_name = tvName.getText().toString();
            ChangeReservationDetailsFragment changeReservationDetailsFragment = ChangeReservationDetailsFragment.getInstance(reservation);
            changeReservationDetailsFragment.show(getChildFragmentManager(), "change_details");
            changeReservationDetailsFragment.setDialogDismissListener(this);
        });
    }
    private void showRestaurantIsClosed(String message, boolean shouldGoBack) {
        RestaurantClosedDialogFragment restaurantClosedDialogFragment = RestaurantClosedDialogFragment.getInstance(message);
        restaurantClosedDialogFragment.show(getChildFragmentManager(), "restaurant_closed");
        restaurantClosedDialogFragment.setDialogDismissListener(o -> {
            if (shouldGoBack) {
                if (getActivity() != null)
                    getActivity().onBackPressed();
            }
        });
    }
    private boolean isValid() {
        if (Validators.isNullOrEmpty(reservation.booking_date)) {
            ToastUtils.makeToast(getActivity(), "Please select reservation date");
            return false;
        }
        if (Validators.isNullOrEmpty(reservation.booking_time)) {
            ToastUtils.makeToast(getActivity(), "Please select reservation time");
            return false;
        }

        if (Validators.isNullOrEmpty(tvName.getText().toString())) {
            ToastUtils.makeToast(getActivity(), "Please enter name");
            return false;
        }

        if (Validators.isNullOrEmpty(tvEmail.getText().toString())) {
            ToastUtils.makeToast(getActivity(), "Please enter email");
            return false;
        }

        if (Validators.isNullOrEmpty(tvPhone.getText().toString())) {
            ToastUtils.makeToast(getActivity(), "Please enter phone number");
            return false;
        }
        if (rgNumberOfPeople.getCheckedRadioButtonId() == R.id.rbOne) {
            reservation.guest_count = "1";
        }
        if (rgNumberOfPeople.getCheckedRadioButtonId() == R.id.rbTwo) {
            reservation.guest_count = "2";
        }
        if (rgNumberOfPeople.getCheckedRadioButtonId() == R.id.rbThree) {
            reservation.guest_count = "3";
        }
        if (rgNumberOfPeople.getCheckedRadioButtonId() == R.id.rbFour) {
            reservation.guest_count = "4";
        }
        if (rgNumberOfPeople.getCheckedRadioButtonId() == R.id.rbFive) {
            reservation.guest_count = "5";
        }
        if (rgNumberOfPeople.getCheckedRadioButtonId() == R.id.rbSixPlus) {
            reservation.guest_count = "6";
        }
        if (Validators.isNullOrEmpty(reservation.guest_count)) {
            ToastUtils.makeToast(getActivity(), "Please select number of guests");
            return false;
        }

        reservation.customer_name = tvName.getText().toString();
        reservation.booking_email = tvEmail.getText().toString();
        reservation.booking_phone = tvPhone.getText().toString();
        reservation.booking_instruction = etInstructions.getText().toString();
        reservation.restaurant_id = rest_id;
        reservation.customer_id = String.valueOf(loggedInCustomer.id);
        reservation.type = "Android";
        reservation.booking_id = "";
        if (myApp.getMyPreferences().getLoggedInUser().phone_verify.equalsIgnoreCase("false") && myApp.getMyPreferences().getSiteSettings().booking_verify.equalsIgnoreCase("1")) {
            PhoneOrEmailVerificationBottomsheet phoneOrEmailVerificationBottomsheet = PhoneOrEmailVerificationBottomsheet.getInstance(myApp.getMyPreferences().getLoggedInUser().phone_number, myApp.getMyPreferences().getLoggedInUser().id, myApp.getMyPreferences().getLoggedInUser().username, "phone");
            phoneOrEmailVerificationBottomsheet.show(getChildFragmentManager(), "verify phone");
            phoneOrEmailVerificationBottomsheet.setDialogDismissListener(o -> fetchProfile());
            return false;
        }
        return true;
    }

    private void bookATable() {
        getActivity().runOnUiThread(() -> progressDialog.show());
        ApiUtils.bookATable(reservation)
                .getAsJSONObject(new JSONObjectRequestListener() {
                    @Override
                    public void onResponse(JSONObject response) {
                        getActivity().runOnUiThread(() -> progressDialog.dismiss());
                        ToastUtils.makeToast(getActivity(), "Successfully table booked");
                        reservation = new Gson().fromJson(response.toString(), Reservation.class);
                        startActivity(new Intent(getActivity(), TransportActivity.class).putExtra("destination", "trackOrder").putExtra("reservation_id", reservation.id).putExtra("booking_id", reservation.booking_id).putExtra("hideToolbar", true));

                        if (dialogDismissListener != null)
                            dialogDismissListener.onDialogDismiss(null);
                        dismiss();
                    }

                    @Override
                    public void onError(ANError anError) {
                        anError.printStackTrace();
                        ToastUtils.makeToast(getActivity(), "Error during reserving table");
                        getActivity().runOnUiThread(() -> progressDialog.dismiss());

                        if (!CommonFunctions.isConnected(getActivity()))
                            myApp.noInternet(getActivity());
                    }
                });
    }

    private void fetchProfile() {
        AndroidNetworking.get(ApiEndPoints.NEW_ACCOUNT)
                .build()
                .getAsJSONObject(new JSONObjectRequestListener() {
                    @Override
                    public void onResponse(JSONObject response) {
                        User user1 = new Gson().fromJson(response.toString(), User.class);
                        myApp.getMyPreferences().saveLoggedInUser(user1);
                        myApp.profileChanged();
                        if (isValid()) bookATable();
                    }

                    @Override
                    public void onError(ANError anError) {
                        anError.printStackTrace();
                        if (getActivity() != null) {
                            getActivity().runOnUiThread(() -> progressDialog.dismiss());
                        }
                        if (!CommonFunctions.isConnected(getActivity()))
                            myApp.noInternet(getActivity());

                    }
                });
    }

    private String restaurantTimeSlots;

    private void fetchTimeSlots() {
        if (getActivity() != null) getActivity().runOnUiThread(() -> progressDialog.show());
        ApiUtils.getRestaurantTimeSlots(String.valueOf(rest_id), reservation.booking_date)
                .getAsJSONObject(new JSONObjectRequestListener() {
                    @Override
                    public void onResponse(JSONObject response) {
                        if (getActivity() != null)
                            getActivity().runOnUiThread(() -> progressDialog.dismiss());

                        try {
                            restaurantTimeSlots = response.getJSONObject("delivery").getString("today");
                            if (moveNext) {
                                llTime.performClick();
                                moveNext = false;
                            }
                        } catch (JSONException e) {
                            e.printStackTrace();
                            restaurantTimeSlots = null;
                            reservation.booking_date = null;
                            if (getActivity() != null)
                                getActivity().runOnUiThread(() -> tvReservationDate.setText(null));

                        }
                    }

                    @Override
                    public void onError(ANError anError) {
                        if (getActivity() != null)
                            getActivity().runOnUiThread(() -> progressDialog.dismiss());
                        restaurantTimeSlots = null;
                        reservation.booking_date = null;
                        tvReservationDate.setText(null);
                        anError.printStackTrace();
                        LogUtils.e("time slots error::" + anError.getLocalizedMessage());
                    }
                });
    }

    @Override
    public void onDialogDismiss(Object o) {
        if (o instanceof Reservation) {
            tvName.setText(((Reservation) o).customer_name);
            tvEmail.setText(((Reservation) o).booking_email);
            tvPhone.setText(((Reservation) o).booking_phone);
        }
    }

    @Override
    public void onDismiss(@NonNull DialogInterface dialog) {
        if (progressDialog != null) {
            progressDialog.dismiss();
        }
        super.onDismiss(dialog);
    }
}
