package com.tiffintom.fragment;

import android.animation.Animator;
import android.animation.AnimatorListenerAdapter;
import android.app.Dialog;
import android.content.DialogInterface;
import android.os.Build;
import android.os.Bundle;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.FrameLayout;
import android.widget.ImageView;
import android.widget.LinearLayout;
import android.widget.TextView;

import androidx.annotation.NonNull;
import androidx.annotation.Nullable;
import androidx.appcompat.app.AlertDialog;
import androidx.recyclerview.widget.LinearLayoutManager;
import androidx.recyclerview.widget.RecyclerView;

import com.androidnetworking.AndroidNetworking;
import com.androidnetworking.error.ANError;
import com.androidnetworking.interfaces.JSONObjectRequestListener;
import com.bumptech.glide.Glide;
import com.google.android.material.bottomsheet.BottomSheetBehavior;
import com.google.android.material.bottomsheet.BottomSheetDialog;
import com.google.android.material.bottomsheet.BottomSheetDialogFragment;
import com.google.gson.Gson;
import com.google.gson.reflect.TypeToken;
import com.tiffintom.MyApp;
import com.tiffintom.R;
import com.tiffintom.adapters.AddonsAdapter;
import com.tiffintom.adapters.MenuVariantAdapter;
import com.tiffintom.common.CommonFunctions;
import com.tiffintom.interfaces.DialogDismissListener;
import com.tiffintom.models.CartItem;
import com.tiffintom.models.CartSummary;
import com.tiffintom.models.DineinCartItem;
import com.tiffintom.models.Header;
import com.tiffintom.models.MainAddOn;
import com.tiffintom.models.Menu;
import com.tiffintom.models.MenuItem;
import com.tiffintom.models.SubAddOn;
import com.tiffintom.models.Variant;
import com.tiffintom.network.ApiEndPoints;
import com.tiffintom.utils.LogUtils;
import com.tiffintom.utils.ToastUtils;

import org.json.JSONException;
import org.json.JSONObject;

import java.lang.reflect.Type;
import java.util.ArrayList;
import java.util.List;

/**
 * Created by Amrish on 20-08-2020.
 */
public class MenuItemDetailBottomsheetFragment extends BottomSheetDialogFragment {
    private ImageView ivBasket, ivPlus, ivMinus, ivItem;
    private MyApp myApp = MyApp.getInstance();
    private AlertDialog progressDialog;
    private RecyclerView rvMenuItems;
    private Menu menuItem;
    private ArrayList<Variant> variants = new ArrayList<>();
    private DialogDismissListener dialogDismissListener;
    private TextView tvCartTotal, tvAddToBasket, tvMenuItemName, tvMenuItemDescription, tvAddOnTotal, tvQuantity;
    private MenuVariantAdapter variantAdapter;
    private CartItem cartItem;
    private DineinCartItem dineinCartItem;
    private Variant selectedVariant;
    private LinearLayout llAddToBasket;
    private LinearLayout cvMenuImage;
    private CartSummary cartSummary;
    private boolean isUpdate, alreadyInDb;
    private Menu productDetails;
    private ArrayList<Object> addonsFeed = new ArrayList<>();
    private ArrayList<MainAddOn> mainAddOns = new ArrayList<>();
    private AddonsAdapter addonAdapter;
    private RecyclerView rvAddons;
    private LinearLayout llData;
    private float addOnTotal;
    private boolean isDineIn;
    private String apiUrl;
    private boolean showImages;
    private LinearLayout llLoading;

    public static MenuItemDetailBottomsheetFragment getInstance(Menu menuItem) {
        MenuItemDetailBottomsheetFragment fragment = new MenuItemDetailBottomsheetFragment();
        Bundle bundle = new Bundle();
        bundle.putString("menuItem", new Gson().toJson(menuItem));
        fragment.setArguments(bundle);
        return fragment;
    }

    public static MenuItemDetailBottomsheetFragment getInstance(Menu menuItem, boolean isDineIn) {
        MenuItemDetailBottomsheetFragment fragment = new MenuItemDetailBottomsheetFragment();
        Bundle bundle = new Bundle();
        bundle.putString("menuItem", new Gson().toJson(menuItem));
        bundle.putBoolean("is_dinein", isDineIn);
        fragment.setArguments(bundle);
        return fragment;
    }

    public void setDialogDismissListener(DialogDismissListener dialogDismissListener) {
        this.dialogDismissListener = dialogDismissListener;
    }

    @Override
    public int getTheme() {
        if (Build.VERSION.SDK_INT > 21) {
            return R.style.BottomSheetDialogTheme;
        } else {
            return super.getTheme();
        }
    }

    @NonNull
    @Override
    public Dialog onCreateDialog(Bundle savedInstanceState) {

        BottomSheetDialog dialog = (BottomSheetDialog) super.onCreateDialog(savedInstanceState);

        dialog.setOnShowListener((DialogInterface.OnShowListener) dialog1 -> {
            BottomSheetDialog d = (BottomSheetDialog) dialog1;
            FrameLayout bottomSheet = (FrameLayout) d.findViewById(com.google.android.material.R.id.design_bottom_sheet);
            if (bottomSheet != null) {
                BottomSheetBehavior.from(bottomSheet).setState(BottomSheetBehavior.STATE_EXPANDED);
                BottomSheetBehavior.from(bottomSheet).setSkipCollapsed(true);
                BottomSheetBehavior.from(bottomSheet).setFitToContents(true);
                BottomSheetBehavior.from(bottomSheet).setPeekHeight(1000);
            }
        });

        return dialog;
    }

    @Nullable
    @Override
    public View onCreateView(@NonNull LayoutInflater inflater, @Nullable ViewGroup container, @Nullable Bundle savedInstanceState) {
        if (getArguments() != null) {
            menuItem = new Gson().fromJson(getArguments().getString("menuItem"), Menu.class);
            isDineIn = getArguments().getBoolean("is_dinein", false);
            if (isDineIn) {
                apiUrl = ApiEndPoints.NEW_DINEIN_PRODUCT_DETAILS;
            } else {
                apiUrl = ApiEndPoints.NEW_PRODUCT_DETAILS;
            }
        }
        return inflater.inflate(R.layout.fragment_menuitem_view, container, false);
    }

    @Override
    public void onViewCreated(@NonNull View view, @Nullable Bundle savedInstanceState) {
        super.onViewCreated(view, savedInstanceState);

        if (menuItem != null) {
            showImages = menuItem.showImage;
        }
        fetchCartSummary();
        initViews(view);
        fetchItemDetails();
        setListeners();
    }

    private void fetchCartSummary() {
        new Thread(() -> {
            if (isDineIn) {
                cartSummary = myApp.getAppDatabase().dineinCartItemDao().getCartSummary();
            } else {
                cartSummary = myApp.getAppDatabase().cartDao().getCartSummary();
            }
        }).start();
    }

    private void fetchItemDetails() {
        if (getActivity() != null) getActivity().runOnUiThread(() -> {
            llLoading.setVisibility(View.VISIBLE);
            llData.setVisibility(View.GONE);
        });
        AndroidNetworking.get(apiUrl + String.valueOf(menuItem.id))
                .setTag("product_detail")
                .build()
                .getAsJSONObject(new JSONObjectRequestListener() {
                    @Override
                    public void onResponse(JSONObject response) {
                        try {
                            LogUtils.e("Product_detail" + response.getJSONObject("variants").toString());
                        } catch (JSONException exception) {
                            exception.printStackTrace();
                        }
                        if (getActivity() != null) getActivity().runOnUiThread(() -> {
                            llLoading.postDelayed(() -> {
                                llLoading.animate()
                                        .alpha(0.0f)
                                        .setListener(new AnimatorListenerAdapter() {
                                            @Override
                                            public void onAnimationEnd(Animator animation) {
                                                super.onAnimationEnd(animation);
                                                llLoading.setVisibility(View.GONE);
                                                llData.setVisibility(View.VISIBLE);
                                            }
                                        });
                            }, 10);

                        });
                        productDetails = new Gson().fromJson(response.toString(), Menu.class);
                        menuItem = productDetails;
                        menuItem.variants = productDetails.variants;

                        updateViews();
                        manageAdapter();
                    }

                    @Override
                    public void onError(ANError anError) {
                        anError.printStackTrace();
                        if (getActivity() != null) getActivity().runOnUiThread(() ->
                        {
                            llLoading.setVisibility(View.GONE);
                            llData.setVisibility(View.GONE);
                        });
                        if (!CommonFunctions.isConnected(getActivity()))
                            myApp.noInternet(getActivity());
                    }
                });
    }


    private void initViews(View view) {
        progressDialog = CommonFunctions.customProgressDialog(getActivity(), "Loading...");
        ivBasket = view.findViewById(R.id.ivBasket);
        ivPlus = view.findViewById(R.id.ivPlus);
        ivMinus = view.findViewById(R.id.ivMinus);
        tvCartTotal = view.findViewById(R.id.tvCartTotal);
        tvAddToBasket = view.findViewById(R.id.tvAddToBasket);
        tvMenuItemName = view.findViewById(R.id.tvMenuTitle);
        tvMenuItemDescription = view.findViewById(R.id.tvMenuItemDescription);
        tvAddOnTotal = view.findViewById(R.id.tvAddOnTotal);
        tvQuantity = view.findViewById(R.id.tvQuantity);
        rvMenuItems = view.findViewById(R.id.rvMenuVariants);
        ivItem = view.findViewById(R.id.ivItem);
        llAddToBasket = view.findViewById(R.id.llAddToBasket);
        cvMenuImage = view.findViewById(R.id.cvMenuImage);
        rvAddons = view.findViewById(R.id.rvAddons);
        llData = view.findViewById(R.id.llData);
        llLoading = view.findViewById(R.id.llLoading);

        rvMenuItems.setLayoutManager(new LinearLayoutManager(getActivity(), RecyclerView.VERTICAL, false));
        rvAddons.setLayoutManager(new LinearLayoutManager(getActivity(), RecyclerView.VERTICAL, false));

        variantAdapter = new MenuVariantAdapter(getActivity(), variants, (position, data) -> {
            selectedVariant = (Variant) data;
            int variantIndex = variants.indexOf(selectedVariant);
            if (variantIndex != -1) selectedVariant = variants.get(variantIndex);
            if (selectedVariant.main_addons != null) {
                for (MainAddOn m : selectedVariant.main_addons) {
                    m.selected = false;
                    m.selectedSubAddons = 0;
                    m.selectedSubAddonsPrice = 0;
                    for (SubAddOn s : m.sub_addons) {
                        s.selected = false;
                    }
                }
            }
            addOnTotal = 0;

            manageAdapter();
            MenuVariantAdapter.selectedVariant = selectedVariant.id;
            if (isDineIn) {
                createDineInFirstItem();
                updateDineInCartItem();
            } else {
                createFirstItem();
                updateCartItem();
            }
        });
        rvMenuItems.setAdapter(variantAdapter);

        addonAdapter = new AddonsAdapter(getActivity(), addonsFeed, this::onAddonSelect);
        rvAddons.setAdapter(addonAdapter);

        rvMenuItems.setNestedScrollingEnabled(false);
        rvAddons.setNestedScrollingEnabled(false);
        llAddToBasket.setEnabled(false);
    }

    private Type addonsType = new TypeToken<List<MainAddOn>>() {
    }.getType();
    private Type varaintsType = new TypeToken<List<Variant>>() {
    }.getType();

    private void updateViews() {

        if (menuItem != null) {

            if (showImages) {
                cvMenuImage.setVisibility(View.VISIBLE);
            } else {
                cvMenuImage.setVisibility(View.GONE);
            }
            tvMenuItemName.setText(menuItem.menu_name);
            tvMenuItemDescription.setText(menuItem.menu_description);

            if (menuItem.image_url != null) {
                Glide.with(getActivity()).load(menuItem.image_url)
                        .placeholder(R.drawable.menu_item_placeholder)
                        .error(R.drawable.menu_item_placeholder)
                        .into(ivItem);
            } else {
                cvMenuImage.setVisibility(View.GONE);
            }

            variants.clear();
            if (isDineIn) {
                variants.addAll(menuItem.variants);
            } else {
                variants.addAll(menuItem.variants);
            }
            if (variants.size() > 0) {
                selectedVariant = variants.get(0);
                MenuVariantAdapter.selectedVariant = selectedVariant.id;
            }
            variantAdapter.notifyDataSetChanged();

            tvQuantity.setText("1");
            try {
                float discount = (selectedVariant.orginal_price * menuItem.getProductPercentage() / 100);
                tvAddOnTotal.setText(myApp.getCurrencySymbol() + MyApp.df.format(selectedVariant.orginal_price - discount));
                if (cartSummary == null) {
                    tvCartTotal.setText("Total: " + myApp.getCurrencySymbol() + MyApp.df.format(selectedVariant.orginal_price - discount));
                } else {
                    tvCartTotal.setText("Total: " + (myApp.getCurrencySymbol()) + MyApp.df.format(selectedVariant.orginal_price - discount + cartSummary.total));
                }
                ivMinus.setVisibility(View.INVISIBLE);
            } catch (NullPointerException e) {
                e.printStackTrace();
                new RuntimeException("If price not available Crash");
            }


        }

    }

    private void manageAdapter() {
           if (productDetails != null) {
                variants.clear();
                if (productDetails.variants != null)
                    variants.addAll(productDetails.variants);
                variantAdapter.notifyDataSetChanged();

                mainAddOns.clear();
                if (selectedVariant.main_addons != null)
                    mainAddOns.addAll(selectedVariant.main_addons);

                addonsFeed.clear();
                if (mainAddOns.size() > 0) {
                    StringBuilder mainAddonsName = new StringBuilder();
                    boolean enable = true;
                    ArrayList<SubAddOn> subAddOnArrayList = new ArrayList<>();
                    for (MainAddOn mainAddOn : selectedVariant.main_addons) {
                        int index = selectedVariant.main_addons.indexOf(mainAddOn);
                        mainAddonsName.append(mainAddOn.mainaddons_name);
                        if (index != selectedVariant.main_addons.size() - 1) {
                            mainAddonsName.append(", ");
                        }
                        if (mainAddOn.mainaddons_mini_count > mainAddOn.selectedSubAddons || mainAddOn.selectedSubAddons > mainAddOn.mainaddons_count) {
                            enable = false;
                        }
                        for (SubAddOn subAddOn : mainAddOn.sub_addons) {
                            if (subAddOn.selected) {
                                subAddOnArrayList.add(subAddOn);
                            }

                        }
                        if (mainAddOn.selected) {
                            if (subAddOnArrayList.size() == 0) {
                                enable = false;
                            }
                        }
                    }


                    llAddToBasket.setEnabled(enable);
//                addonsFeed.add(new Header(mainAddonsName.toString()));
                } else {
                    llAddToBasket.setEnabled(true);
                }
                addonsFeed.addAll(mainAddOns);

                addonAdapter.notifyDataSetChanged();
            }


    }

    private void expandIfSelected() {
        for (int i = 0; i < addonsFeed.size(); i++) {
            Object o = addonsFeed.get(i);
            if (o instanceof MainAddOn) {
                MainAddOn mainAddOn = (MainAddOn) o;
                if (mainAddOn.selected) {
                    Header header = new Header();
                    header.addonId = mainAddOn.id;
                    if (mainAddOn.mainaddons_mini_count > 0) {
                        header.title = mainAddOn.mainaddons_name + " Choose at least " + mainAddOn.mainaddons_mini_count + " add on(s)";
                    } else if (mainAddOn.mainaddons_mini_count == 0 && mainAddOn.mainaddons_count > 0) {
                        header.title = mainAddOn.mainaddons_name + " Choose up to " + mainAddOn.mainaddons_count + " add on(s)";
                    } else {
                        header.title = mainAddOn.mainaddons_name;
                    }
                    addonsFeed.add(header);
                    addonsFeed.addAll(mainAddOn.sub_addons);
                    addonAdapter.notifyDataSetChanged();
                }
            }
        }
    }

    private void findSelectedAddOnPrice() {
        if (cartItem == null && variants.size() > 0) {
            selectedVariant = variants.get(0);
            isUpdate = false;
            alreadyInDb = false;
        } else {
            isUpdate = true;
            alreadyInDb = true;
            for (Variant variant : productDetails.variants) {
                if (cartItem != null && variant.id == cartItem.id) {
                    selectedVariant = variant;
                }
            }
        }
        MenuVariantAdapter.selectedVariant = selectedVariant.id;

    }

    private void onAddonSelect(int position, Object data) {
        if (data instanceof MainAddOn) {
            MainAddOn mainAddOn = (MainAddOn) data;
            int destinationPosition = 0;
            if (mainAddOn.selected) {
                Header header = new Header();
                header.addonId = mainAddOn.id;
                if (mainAddOn.mainaddons_mini_count > 0) {
                    header.title = mainAddOn.mainaddons_name + " Choose at least " + mainAddOn.mainaddons_mini_count + " add on(s)";
                } else if (mainAddOn.mainaddons_mini_count == 0 && mainAddOn.mainaddons_count > 0) {
                    header.title = mainAddOn.mainaddons_name + " Choose up to " + mainAddOn.mainaddons_count + " add on(s)";
                } else {
                    header.title = mainAddOn.mainaddons_name;
                }

//                if (destinationPosition >= addonsFeed.size()) {
//                    addonsFeed.add(header);
//                    addonsFeed.addAll(mainAddOn.sub_addons);
//
//                } else {
//                    addonsFeed.add(destinationPosition, header);
//                    addonsFeed.addAll(destinationPosition + 1, mainAddOn.sub_addons);
//                }
                addonsFeed.add(header);
                addonsFeed.addAll(mainAddOn.sub_addons);
                addonAdapter.notifyDataSetChanged();
                validateButton();
            } else {
                Header header = new Header();
                header.addonId = mainAddOn.id;
                int headerIndex = addonsFeed.indexOf(header);
                if (headerIndex != -1) addonsFeed.remove(headerIndex);
                for (SubAddOn subAddOn : mainAddOn.sub_addons) {
                    subAddOn.selected = false;
                    int subAddonIndex = addonsFeed.indexOf(subAddOn);
                    if (subAddonIndex != -1) addonsFeed.remove(subAddonIndex);
                    LogUtils.e(subAddOn.subaddons_name, " is removed");
                }
//                addonsFeed.removeAll(mainAddOn.sub_addons);
                addOnTotal = addOnTotal - mainAddOn.selectedSubAddonsPrice;
                mainAddOn.selectedSubAddons = 0;
                mainAddOn.selectedSubAddonsPrice = 0;
                addonsFeed.set(position, mainAddOn);
                addonAdapter.notifyDataSetChanged();
                validateButton();
            }


        } else if (data instanceof SubAddOn) {
            SubAddOn subAddOn = (SubAddOn) data;
            MainAddOn mainAddOn = new MainAddOn();
            mainAddOn.id = subAddOn.mainaddons_id;
            int mainAddonIndex = mainAddOns.indexOf(mainAddOn);
            if (mainAddonIndex != -1) mainAddOn = mainAddOns.get(mainAddonIndex);
            else mainAddOn = null;
            if (mainAddOn != null) {
                int selected_subaddon = mainAddOn.selectedSubAddons;
                if (subAddOn.selected) {
                    // add subaddon
                    if (selected_subaddon >= mainAddOn.mainaddons_count) {
                        ToastUtils.makeToast(getContext(), "You can select only " + mainAddOn.mainaddons_count + " " + mainAddOn.mainaddons_name);
                        subAddOn.selected = !subAddOn.selected;
                        addonsFeed.set(position, subAddOn);
                    } else {
                        mainAddOn.selectedSubAddons++;
                        addOnTotal = addOnTotal + subAddOn.subaddons_price;
                        mainAddOn.selectedSubAddonsPrice = mainAddOn.selectedSubAddonsPrice + subAddOn.subaddons_price;
                        addonsFeed.set(mainAddonIndex, mainAddOn);

                    }
                } else {
                    // Remove subaddon
                    mainAddOn.selectedSubAddons--;
                    addOnTotal = addOnTotal - subAddOn.subaddons_price;
                    mainAddOn.selectedSubAddonsPrice = mainAddOn.selectedSubAddonsPrice - subAddOn.subaddons_price;
                    addonsFeed.set(mainAddonIndex, mainAddOn);

                    // update cart total;
//                    if (selected_subaddon <= mainAddOn.mainaddons_mini_count) {
//                        ToastUtils.makeToast(getContext(), "You have to select atleast " + mainAddOn.mainaddons_mini_count + " " + mainAddOn.mainaddons_name);
//                        subAddOn.selected = !subAddOn.selected;
//                        addonsFeed.set(position, subAddOn);
//                    } else {
//                        mainAddOn.selectedSubAddons--;
//                        addonsFeed.set(mainAddonIndex, mainAddOn);
//                        addOnTotal = addOnTotal - subAddOn.subaddons_price;
//                        // update cart total;
//                    }
                }
                validateButton();
            } else {
                subAddOn.selected = !subAddOn.selected;
                addonsFeed.set(position, subAddOn);
                ToastUtils.makeToast(getContext(), "Invalid addon");
            }
            addonAdapter.notifyDataSetChanged();
            LogUtils.e("Sub Addon Total: ", String.valueOf(addOnTotal));
        }
        if (isDineIn) {
            createDineInFirstItem();
            updateDineInCartItem();
        } else {
            createFirstItem();
            updateCartItem();
        }
    }

    private void validateButton() {
        boolean enable = true;
        ArrayList<SubAddOn> subAddOnsArraylist = new ArrayList<>();
        for (MainAddOn mainAddOn : selectedVariant.main_addons) {
            if (mainAddOn.mainaddons_mini_count > mainAddOn.selectedSubAddons || mainAddOn.selectedSubAddons > mainAddOn.mainaddons_count) {
                enable = false;
            }
            for (SubAddOn subAddOn : mainAddOn.sub_addons) {
                if (subAddOn.selected) {
                    subAddOnsArraylist.add(subAddOn);
                }
            }
            if (mainAddOn.selected) {
                if (subAddOnsArraylist.size() == 0) {
                    enable = false;
                }
            }
        }

        llAddToBasket.setEnabled(enable);

    }

    private void createFirstItem() {
        if (cartItem == null) {
            cartItem = new CartItem();
            cartItem.quantity = 1;
        }
    }

    private void createDineInFirstItem() {
        if (dineinCartItem == null) {
            dineinCartItem = new DineinCartItem();
            dineinCartItem.quantity = 1;
        }
    }


    private void setListeners() {
        ivMinus.setOnClickListener(view -> {
            new Thread(this::decrementCounter).start();
        });
        ivPlus.setOnClickListener(view -> {
            new Thread(this::incrementCounter).start();
        });
        llAddToBasket.setOnClickListener(view -> {

            if (isDineIn) {
                createDineInFirstItem();
                updateDineInCartItem();
            } else {
                createFirstItem();
                updateCartItem();
            }
            addToBasket();

        });
    }

    private void addToBasket() {
        String message = "";
        for (Object ob : addonsFeed) {
            if (ob instanceof MainAddOn) {
                MainAddOn mainAddOn = (MainAddOn) ob;
                if (mainAddOn.mainaddons_mini_count > mainAddOn.selectedSubAddons) {
                    message = "Please select at least " + mainAddOn.mainaddons_mini_count + " " + mainAddOn.mainaddons_name;
                    break;
                } else if (mainAddOn.mainaddons_count < mainAddOn.selectedSubAddons) {
                    message = "You can only select up to " + mainAddOn.mainaddons_count + " " + mainAddOn.mainaddons_name;
                    break;
                }
            }
        }
        if (!message.isEmpty()) {
            ToastUtils.makeToast(getActivity(), message);
        } else if (selectedVariant == null) {
            ToastUtils.makeToast(getActivity(), "Please select size first");
        } else if (isDineIn && dineinCartItem.quantity == 0) {
            ToastUtils.makeToast(getActivity(), "Please enter quantity");
        } else if (!isDineIn && cartItem.quantity == 0) {
            ToastUtils.makeToast(getActivity(), "Please enter quantity");
        } else {
            new Thread(() -> {
                insertUpdateDataBase();
                fetchCartSummary();
                if (getActivity() != null) {
                    getActivity().runOnUiThread(() -> {
                        alreadyInDb = true;
                        tvCartTotal.setText("Total: " + (myApp.getCurrencySymbol()) + MyApp.df.format(cartSummary.total));
                    });
                }
                if (dialogDismissListener != null) {
                    dialogDismissListener.onDialogDismiss(null);
                }
                dismiss();
            }).start();
        }
    }

    private void incrementCounter() {
        if (isDineIn) {
            createDineInFirstItem();
            dineinCartItem.quantity++;
            new Thread(this::updateDineInCartItem).start();
        } else {
            createFirstItem();
            cartItem.quantity++;
            new Thread(this::updateCartItem).start();
        }
        if (getActivity() != null) {
            getActivity().runOnUiThread(() -> {
                if (isDineIn) {
                    tvQuantity.setText(String.valueOf(dineinCartItem.quantity));
                    if (dineinCartItem.quantity >= 2) {
                        ivMinus.setVisibility(View.VISIBLE);
                    }
                } else {
                    tvQuantity.setText(String.valueOf(cartItem.quantity));
                    if (cartItem.quantity >= 2) {
                        ivMinus.setVisibility(View.VISIBLE);
                    }
                }

            });
        }
    }

    private void decrementCounter() {
        int quantity;
        if (isDineIn) {
            createDineInFirstItem();
            quantity = dineinCartItem.quantity;
        } else {
            createFirstItem();
            quantity = cartItem.quantity;
        }

        if (quantity <= 1) {
            if (alreadyInDb) {
                ToastUtils.makeToast(getActivity(), "Item removed from basket");
                if (isDineIn) {
                    myApp.getAppDatabase().dineinCartItemDao().delete(dineinCartItem);
                } else {
                    myApp.getAppDatabase().cartDao().delete(cartItem);
                }
                if (dialogDismissListener != null) {
                    dialogDismissListener.onDialogDismiss(null);
                }
                dismiss();
            } else {
                if (getActivity() != null) {
                    getActivity().runOnUiThread(() -> {
                        ivMinus.setVisibility(View.INVISIBLE);
                    });
                }
            }
        } else {
            if (isDineIn) {
                dineinCartItem.quantity--;
                new Thread(this::updateDineInCartItem).start();
            } else {
                cartItem.quantity--;
                new Thread(this::updateCartItem).start();
            }

            if (getActivity() != null) {
                getActivity().runOnUiThread(() -> {
                    if (isDineIn) {
                        tvQuantity.setText(String.valueOf(dineinCartItem.quantity));
                    } else {
                        tvQuantity.setText(String.valueOf(cartItem.quantity));
                    }
                    if (alreadyInDb) {
                        ivMinus.setVisibility(View.VISIBLE);
                    } else {
                        int q;
                        if (isDineIn) {
                            q = dineinCartItem.quantity;
                        } else {
                            q = cartItem.quantity;
                        }
                        if (q == 1) {
                            ivMinus.setVisibility(View.INVISIBLE);
                        } else {
                            ivMinus.setVisibility(View.VISIBLE);
                        }
                    }
                });
            }
        }
    }

    private void updateCartItem() {
        float discount = (selectedVariant.orginal_price * menuItem.getProductPercentage() / 100);
        if (mainAddOns.size() == 0) {
            cartItem.Menu_price = selectedVariant.orginal_price - discount;
            cartItem.menu_id = selectedVariant.menu_id;
            cartItem.id = selectedVariant.id;
            cartItem.menu_name = menuItem.menu_name;
            cartItem.menu_size = (menuItem.menu_name.equalsIgnoreCase(selectedVariant.sub_name)) ? "" : selectedVariant.sub_name;
            cartItem.res_id = menuItem.restaurant_id;
            cartItem.menu_type = menuItem.menu_type;

            cartItem.Addon_name = "";
            cartItem.Addon_price = 0;
            cartItem.Addon_price = addOnTotal;

            cartItem.Total = cartItem.quantity * (cartItem.Menu_price + cartItem.Addon_price);

        } else {
            ArrayList<MainAddOn> mainAddOnsSelected = new ArrayList<>();
            for (Variant variant : menuItem.variants) {
                for (MainAddOn mainAddOn : variant.main_addons)
                    if (mainAddOn.selected) {
                        mainAddOnsSelected.add(mainAddOn);
                    }
            }
            if (mainAddOnsSelected.size() > 0) {
                for (MainAddOn mainAddOn : mainAddOnsSelected)
                    for (SubAddOn subAddOn : mainAddOn.sub_addons) {
                        if (subAddOn.selected) {
                            cartItem.Menu_price = selectedVariant.orginal_price - discount;
                            cartItem.menu_id = selectedVariant.menu_id;
                            cartItem.id = selectedVariant.id;
                            cartItem.menu_name = menuItem.menu_name;
                            cartItem.menu_size = subAddOn.subaddons_name;
                            cartItem.res_id = menuItem.restaurant_id;
                            cartItem.menu_type = menuItem.menu_type;

                            cartItem.Addon_name = "";
                            cartItem.Addon_price = 0;
                            cartItem.Addon_price = addOnTotal;

                            cartItem.Total = cartItem.quantity * (cartItem.Menu_price + cartItem.Addon_price);
                        }
                    }
            } else {
                cartItem.Menu_price = selectedVariant.orginal_price - discount;
                cartItem.menu_id = selectedVariant.menu_id;
                cartItem.id = selectedVariant.id;
                cartItem.menu_name = menuItem.menu_name;
                cartItem.menu_size = selectedVariant.sub_name;
                cartItem.res_id = menuItem.restaurant_id;
                cartItem.menu_type = menuItem.menu_type;

                cartItem.Addon_name = "";
                cartItem.Addon_price = 0;
                cartItem.Addon_price = addOnTotal;

                cartItem.Total = cartItem.quantity * (cartItem.Menu_price + cartItem.Addon_price);
            }

            if (getActivity() != null) {
                getActivity().runOnUiThread(() -> {
                    tvAddOnTotal.setText(myApp.getCurrencySymbol() + MyApp.df.format(cartItem.Total));
                    tvCartTotal.setText("Total: " + myApp.getCurrencySymbol() + MyApp.df.format(cartItem.Total + cartSummary.total));
                });
            }
        }

    }

    private void updateDineInCartItem() {
        float discount = (selectedVariant.orginal_price * menuItem.getProductPercentage() / 100);
        dineinCartItem.Menu_price = selectedVariant.orginal_price - discount;
        dineinCartItem.menu_id = selectedVariant.menu_id;
        dineinCartItem.id = selectedVariant.id;
        dineinCartItem.menu_name = menuItem.menu_name;
        dineinCartItem.menu_size = (menuItem.menu_name.equalsIgnoreCase(selectedVariant.sub_name)) ? "" : selectedVariant.sub_name;
        dineinCartItem.res_id = menuItem.restaurant_id;
        dineinCartItem.menu_type = menuItem.menu_type;

        dineinCartItem.Addon_name = "";
        dineinCartItem.Addon_price = 0;
        dineinCartItem.Addon_price = addOnTotal;

        dineinCartItem.Total = dineinCartItem.quantity * (dineinCartItem.Menu_price + dineinCartItem.Addon_price);
        if (getActivity() != null) {
            getActivity().runOnUiThread(() -> {
                tvAddOnTotal.setText(myApp.getCurrencySymbol() + MyApp.df.format(dineinCartItem.Total));
                tvCartTotal.setText("Total: " + myApp.getCurrencySymbol() + MyApp.df.format(dineinCartItem.Total + cartSummary.total));
            });
        }

    }

    private void insertUpdateDataBase() {
        saveSelectedAddons();
        setAddOnSimpleName();
        if (isDineIn) {
            myApp.getAppDatabase().dineinCartItemDao().insertAll(dineinCartItem);
        } else {
//            myApp.getAppDatabase().cartDao().insertAll(cartItem);
//        }
            String variantName = null;
            float discount = (selectedVariant.orginal_price * menuItem.getProductPercentage() / 100);
            if (mainAddOns.size() == 0) {
                if (menuItem.variants != null && menuItem.variants.size() > 0) {
                    variantName = selectedVariant.sub_name;
                    variantName = (menuItem.menu_name.equalsIgnoreCase(variantName)) ? "" : variantName;
                    discount = (selectedVariant.orginal_price * menuItem.getProductPercentage() / 100);
                }
                CartItem dbCartItem = myApp.getAppDatabase().cartDao().getCartMenuItem(menuItem.id, variantName);
                if (dbCartItem == null) {
                    cartItem.Menu_price = selectedVariant.orginal_price - discount;
                    cartItem.menu_id = selectedVariant.menu_id;
                    cartItem.id = selectedVariant.id;
                    cartItem.menu_name = menuItem.menu_name;
                    cartItem.menu_size = (menuItem.menu_name.equalsIgnoreCase(selectedVariant.sub_name)) ? "" : selectedVariant.sub_name;
                    cartItem.res_id = menuItem.restaurant_id;
                    cartItem.menu_type = menuItem.menu_type;

                    cartItem.Addon_name = "";
                    cartItem.Addon_price = 0;
                    cartItem.Addon_price = addOnTotal;

                    cartItem.Total = cartItem.quantity * (cartItem.Menu_price + cartItem.Addon_price);
                    myApp.getAppDatabase().cartDao().insertAll(cartItem);
                } else {
                    if (menuItem.variants != null && menuItem.variants.size() > 0) {
                        for (Variant variant : menuItem.variants) {
                            if (variant.sub_name.equalsIgnoreCase(selectedVariant.sub_name)) {
                                dbCartItem.quantity = (dbCartItem.quantity + cartItem.quantity);
                                dbCartItem.Menu_price = +(variant.orginal_price - discount);
                                dbCartItem.Total = dbCartItem.quantity * (dbCartItem.Menu_price + dbCartItem.Addon_price);
                                myApp.getAppDatabase().cartDao().update(dbCartItem);
                            }
                        }
                    }
                }
            } else {
                ArrayList<MainAddOn> mainAddOnsSelected = new ArrayList<>();
                discount = (selectedVariant.orginal_price * menuItem.getProductPercentage() / 100);
                if (menuItem.variants != null && menuItem.variants.size() > 0) {

                    for (Variant variant : menuItem.variants) {
                        for (MainAddOn mainAddOn : variant.main_addons)
                            if (mainAddOn.selected) {
                                mainAddOnsSelected.add(mainAddOn);
                            }
                    }

                    if (mainAddOnsSelected.size() > 0) {
                        for (MainAddOn mainAddOn : mainAddOnsSelected)
                            for (SubAddOn subAddOn : mainAddOn.sub_addons) {
                                if (subAddOn.selected) {
                                    variantName = subAddOn.subaddons_name;
                                }
                            }
                    } else {
                        variantName = selectedVariant.sub_name;
                    }
                }
                CartItem dbCartItem = myApp.getAppDatabase().cartDao().getCartAddonMenuItem(menuItem.id, variantName);
                if (dbCartItem == null) {
                    myApp.getAppDatabase().cartDao().insertAll(cartItem);
                } else {
                    if (menuItem.variants != null && menuItem.variants.size() > 0) {

                        if (mainAddOnsSelected.size() > 0) {
                            for (MainAddOn mainAddOn : mainAddOnsSelected)
                                for (SubAddOn subAddOn : mainAddOn.sub_addons) {
                                    if (subAddOn.selected) {
                                        dbCartItem.quantity = (dbCartItem.quantity + cartItem.quantity);
                                        dbCartItem.Menu_price = +(selectedVariant.orginal_price - discount);
                                        dbCartItem.Total = dbCartItem.quantity * (dbCartItem.Menu_price + dbCartItem.Addon_price);
                                        myApp.getAppDatabase().cartDao().update(dbCartItem);
                                    }
                                }
                        } else {
                            dbCartItem.quantity = (dbCartItem.quantity + cartItem.quantity);
                            dbCartItem.Menu_price = +(selectedVariant.orginal_price - discount);
                            dbCartItem.Total = dbCartItem.quantity * (dbCartItem.Menu_price + dbCartItem.Addon_price);
                            myApp.getAppDatabase().cartDao().update(dbCartItem);
                        }

                    }
                }
            }

        }
    }

    private void setAddOnSimpleName() {

        StringBuilder builder = new StringBuilder();
        ArrayList<MainAddOn> mainAddOns;
        if (isDineIn) {
            mainAddOns = new Gson().fromJson(dineinCartItem.mainAddons, addonsType);
        } else {
            mainAddOns = new Gson().fromJson(cartItem.mainAddons, addonsType);
        }
        if (mainAddOns != null) {
            for (int i = 0; i < mainAddOns.size(); i++) {
                MainAddOn mainAddOn = mainAddOns.get(i);
                builder.append(mainAddOn.mainaddons_name);
                for (int j = 0; j < mainAddOn.sub_addons.size(); j++) {
                    builder.append("(" + mainAddOn.sub_addons.get(j).subaddons_name);
                    if (mainAddOn.sub_addons.get(j).subaddons_price == 0) {
                        builder.append("(" + myApp.getCurrencySymbol() + "0.00))");
                    } else {
                        builder.append("(").append(myApp.getCurrencySymbol()).append(mainAddOn.sub_addons.get(j).subaddons_price).append("))");
                    }

                    if (j != mainAddOn.sub_addons.size() - 1) {
                        builder.append(", ");
                    }
                }
                if (i != mainAddOns.size() - 1) {
                    builder.append(", ");
                }
            }
            if (isDineIn) dineinCartItem.Addon_name = builder.toString();
            else cartItem.Addon_name = builder.toString();
        }
    }

    private void saveSelectedAddons() {
        ArrayList<MainAddOn> toBeSavedMainAddons = new ArrayList<>();
        for (Object o : addonsFeed) {
            if (o instanceof MainAddOn) {
                MainAddOn mainAddOn = (MainAddOn) o;
                ArrayList<SubAddOn> toBoSavedSubAddons = new ArrayList<>();
                if (mainAddOn.selected) {
                    for (SubAddOn subAddOn : mainAddOn.sub_addons) {
                        if (subAddOn.selected) toBoSavedSubAddons.add(subAddOn);
                    }
                    mainAddOn.sub_addons.clear();
                    mainAddOn.sub_addons.addAll(toBoSavedSubAddons);
                    toBeSavedMainAddons.add(mainAddOn);
                }
            }
        }
        if (toBeSavedMainAddons.size() > 0) {
            if (isDineIn) dineinCartItem.mainAddons = new Gson().toJson(toBeSavedMainAddons);
            else
                cartItem.mainAddons = new Gson().toJson(toBeSavedMainAddons);
        }
    }


    @Override
    public void onDismiss(@NonNull DialogInterface dialog) {
        AndroidNetworking.cancel("product_detail");
        if (progressDialog != null) {
            progressDialog.dismiss();
        }
        super.onDismiss(dialog);
    }
}
