package com.tiffintom.fragment;

import android.app.Dialog;
import android.content.DialogInterface;
import android.os.Build;
import android.os.Bundle;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.Button;
import android.widget.FrameLayout;
import android.widget.TextView;

import androidx.annotation.NonNull;
import androidx.annotation.Nullable;
import androidx.recyclerview.widget.LinearLayoutManager;
import androidx.recyclerview.widget.RecyclerView;

import com.google.android.material.bottomsheet.BottomSheetBehavior;
import com.google.android.material.bottomsheet.BottomSheetDialog;
import com.google.android.material.bottomsheet.BottomSheetDialogFragment;
import com.google.gson.Gson;
import com.google.gson.reflect.TypeToken;
import com.tiffintom.R;
import com.tiffintom.adapters.MultipleProductOfferAdapter;
import com.tiffintom.interfaces.DialogDismissListener;
import com.tiffintom.models.MultiplePriceOffer;
import com.tiffintom.models.OfferProduct;

import java.lang.reflect.Type;
import java.util.ArrayList;
import java.util.List;

public class MultipleFreeItemsOfferBottomFragment extends BottomSheetDialogFragment {

    private ArrayList<OfferProduct> offerProducts = new ArrayList<>();
    private MultiplePriceOffer multiplePriceOffer;
    private RecyclerView rvOffers;
    private MultipleProductOfferAdapter multipleProductOfferAdapter;
    private Button btnConfirm;
    private DialogDismissListener dialogDismissListener;
    TextView tvSubtitle;

    public static MultipleFreeItemsOfferBottomFragment getInstance(MultiplePriceOffer multiplePriceOffer, ArrayList<OfferProduct> offerProducts) {
        MultipleFreeItemsOfferBottomFragment multipleFreeItemsOfferBottomFragment = new MultipleFreeItemsOfferBottomFragment();
        Bundle bundle = new Bundle();
        bundle.putString("offers", new Gson().toJson(offerProducts));
        bundle.putString("offer_details", new Gson().toJson(multiplePriceOffer));
        multipleFreeItemsOfferBottomFragment.setArguments(bundle);
        return multipleFreeItemsOfferBottomFragment;
    }

    public void setDialogDismissListener(DialogDismissListener dialogDismissListener) {
        this.dialogDismissListener = dialogDismissListener;
    }

    @Override
    public int getTheme() {
        if (Build.VERSION.SDK_INT > 21) {
            return R.style.BottomSheetDialogTheme;
        } else {
            return super.getTheme();
        }
    }

    @NonNull
    @Override
    public Dialog onCreateDialog(Bundle savedInstanceState) {

        BottomSheetDialog dialog = (BottomSheetDialog) super.onCreateDialog(savedInstanceState);

        dialog.setOnShowListener((DialogInterface.OnShowListener) dialog1 -> {
            BottomSheetDialog d = (BottomSheetDialog) dialog1;
            FrameLayout bottomSheet = (FrameLayout) d.findViewById(com.google.android.material.R.id.design_bottom_sheet);
            if (bottomSheet != null)
                BottomSheetBehavior.from(bottomSheet).setState(BottomSheetBehavior.STATE_EXPANDED);
        });

        return dialog;
    }

    @Nullable
    @Override
    public View onCreateView(@NonNull LayoutInflater inflater, @Nullable ViewGroup container, @Nullable Bundle savedInstanceState) {
        return inflater.inflate(R.layout.fragment_multiple_offers, container, false);
    }

    Type type = new TypeToken<List<OfferProduct>>() {
    }.getType();

    @Override
    public void onViewCreated(@NonNull View view, @Nullable Bundle savedInstanceState) {
        super.onViewCreated(view, savedInstanceState);
        if (getArguments() != null) {
            multiplePriceOffer = new Gson().fromJson(getArguments().getString("offer_details"), MultiplePriceOffer.class);
            offerProducts = new Gson().fromJson(getArguments().getString("offers"), type);
        }
        initViews(view);
        setListeners();
    }

    private void initViews(View view) {
        btnConfirm = view.findViewById(R.id.btnConfirm);
        rvOffers = view.findViewById(R.id.rvOffers);
        tvSubtitle = view.findViewById(R.id.tvSubtitle);
        tvSubtitle.setText("Please select " + multiplePriceOffer.offer_qty + " quantity");
        multipleProductOfferAdapter = new MultipleProductOfferAdapter(offerProducts, (position, data) -> {
            btnConfirm.setEnabled(multipleProductOfferAdapter.checkedQty == multipleProductOfferAdapter.maxQty);
        });
        multipleProductOfferAdapter.maxQty = multiplePriceOffer.offer_qty;
        int checkedQyt = 0;
        for (OfferProduct offerProduct : offerProducts) {
            checkedQyt = checkedQyt + offerProduct.quantity;
        }
        multipleProductOfferAdapter.checkedQty = checkedQyt;
        btnConfirm.setEnabled(multipleProductOfferAdapter.checkedQty == multipleProductOfferAdapter.maxQty);
        rvOffers.setLayoutManager(new LinearLayoutManager(getActivity(), RecyclerView.VERTICAL, false));
        rvOffers.setAdapter(multipleProductOfferAdapter);
    }

    private void setListeners() {
        btnConfirm.setOnClickListener(view -> {
            multiplePriceOffer.offerProducts = offerProducts;
            if (dialogDismissListener != null)
                dialogDismissListener.onDialogDismiss(multiplePriceOffer);
            dismiss();
        });
    }


}
