package com.tiffintom.fragment;

import android.os.Bundle;
import android.text.Editable;
import android.text.TextWatcher;
import android.view.KeyEvent;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.Button;
import android.widget.EditText;
import android.widget.RadioButton;
import android.widget.RadioGroup;
import android.widget.TextView;

import androidx.annotation.NonNull;
import androidx.annotation.Nullable;

import com.androidnetworking.AndroidNetworking;
import com.androidnetworking.common.ANRequest;
import com.androidnetworking.error.ANError;
import com.androidnetworking.interfaces.JSONArrayRequestListener;
import com.androidnetworking.interfaces.JSONObjectRequestListener;
import com.androidnetworking.interfaces.ParsedRequestListener;
import com.google.gson.Gson;
import com.google.gson.reflect.TypeToken;
import com.tiffintom.R;
import com.tiffintom.base.BaseFragment;
import com.tiffintom.common.CommonFunctions;
import com.tiffintom.common.Validators;
import com.tiffintom.interfaces.DialogDismissListener;
import com.tiffintom.models.Address;
import com.tiffintom.models.ApiError;
import com.tiffintom.models.Postcode;
import com.tiffintom.models.User;
import com.tiffintom.network.ApiEndPoints;
import com.tiffintom.network.ApiUtils;
import com.tiffintom.utils.LogUtils;
import com.tiffintom.utils.ToastUtils;

import org.json.JSONArray;
import org.json.JSONException;
import org.json.JSONObject;

import java.lang.reflect.Type;
import java.util.ArrayList;
import java.util.List;

public class NewAddressFragment extends BaseFragment {
    private EditText etPostcode, etHouseNumber;
    private TextView tvAddress, tvConfirm, tvCancel;
    private RadioGroup rgAddressTitle;
    private RadioButton rbHome, rbWork, rbOther;
    private Button btnSearch;
    private Postcode searchedPostcode, currentPostcode = myApp.getMyPreferences().getCurrentPostcode();
    private ArrayList<Address> addresses = new ArrayList<>();
    private DialogDismissListener dialogDismissListener;
    private User loggedInUser = myApp.getMyPreferences().getLoggedInUser();
    private Address savedAddress;
    private boolean moveNext = false;
    private String res_id;

    public static NewAddressFragment getInstance(Address savedAddress, String res_id) {
        NewAddressFragment newAddressFragment = new NewAddressFragment();
        Bundle bundle = new Bundle();
        bundle.putString("saved_address", new Gson().toJson(savedAddress));
        bundle.putString("res_id", res_id);
        newAddressFragment.setArguments(bundle);
        return newAddressFragment;
    }

    public void setDialogDismissListener(DialogDismissListener dialogDismissListener) {
        this.dialogDismissListener = dialogDismissListener;
    }

    @Nullable
    @Override
    public View onCreateView(@NonNull LayoutInflater inflater, @Nullable ViewGroup container, @Nullable Bundle savedInstanceState) {
        return inflater.inflate(R.layout.fragment_addnew_address, container, false);
    }

    @Override
    public void onViewCreated(@NonNull View view, @Nullable Bundle savedInstanceState) {
        super.onViewCreated(view, savedInstanceState);
        if (getArguments() != null) {
            if (currentPostcode != null)
                searchedPostcode = currentPostcode;
            res_id = getArguments().getString("res_id");
            String savedAddressValue = getArguments().getString("saved_address");
            if (savedAddressValue != null)
                savedAddress = new Gson().fromJson(savedAddressValue, Address.class);
        }
        updateUi();
        setListeners();
        updateAddress();
    }

    private void updateAddress() {

        if (savedAddress != null) {
            etPostcode.setText(savedAddress.zipcode);
            etHouseNumber.setText(savedAddress.flat_no);
            tvAddress.setText(savedAddress.address);
            etHouseNumber.setSelection(etHouseNumber.getText().length());
            if (savedAddress.title.equalsIgnoreCase("home")) {
                rbHome.setChecked(true);
            } else if (savedAddress.title.equalsIgnoreCase("work")) {
                rbWork.setChecked(true);
            } else {
                rbOther.setChecked(true);
            }
        }
    }

    @Override
    protected void initViews(View view) {
        super.initViews(view);
        progressDialog = CommonFunctions.customProgressDialog(getActivity(), "Loading...");
        progressDialog.getWindow().setDimAmount(0.5f);
        tvAddress = view.findViewById(R.id.etAddress);
        etHouseNumber = view.findViewById(R.id.etFlatNo);
        etPostcode = view.findViewById(R.id.etPostcode);
        rgAddressTitle = view.findViewById(R.id.rgAddressTitle);
        rbHome = view.findViewById(R.id.rbHome);
        rbWork = view.findViewById(R.id.rbWork);
        rbOther = view.findViewById(R.id.rbOther);
        btnSearch = view.findViewById(R.id.btnSearch);

        tvCancel = view.findViewById(R.id.tvCancel);
        tvConfirm = view.findViewById(R.id.tvConfirm);
    }

    private void setListeners() {

        btnSearch.setOnClickListener(view -> {
            etPostcode.setError(null);
            if (!Validators.isNullOrEmpty(etPostcode.getText().toString().replaceAll("\\s+", "").trim())) {
                String postCode = etPostcode.getText().toString().replaceAll("\\s+", "").trim();
                LogUtils.e(postCode);
                new Thread(() -> {
                    searchPostcode(postCode);
                }).start();
            } else {
                etPostcode.setError("Please enter postcode");
            }
        });
        etPostcode.setOnEditorActionListener((textView, i, keyEvent) -> {
            if (keyEvent != null && keyEvent.getKeyCode() == KeyEvent.KEYCODE_ENTER) {
                if (getActivity() != null) {
                    etPostcode.setError(null);
                    if (!Validators.isNullOrEmpty(etPostcode.getText().toString().replaceAll("\\s+", "").trim())) {
                        String postCode = etPostcode.getText().toString().replaceAll("\\s+", "").trim();
                        new Thread(() -> {
                            searchPostcode(postCode);
                        }).start();
                    } else {
                        etPostcode.setError("Please enter postcode");
                    }
                }
            }
            return false;
        });
        etPostcode.addTextChangedListener(new TextWatcher() {
            @Override
            public void beforeTextChanged(CharSequence charSequence, int i, int i1, int i2) {

            }

            @Override
            public void onTextChanged(CharSequence charSequence, int i, int i1, int i2) {

            }

            @Override
            public void afterTextChanged(Editable editable) {
                if (searchedPostcode != null && !searchedPostcode.post_code.equalsIgnoreCase(editable.toString().replaceAll("\\s+", "").trim())) {
                    searchedPostcode = null;
                }
            }
        });
        tvCancel.setOnClickListener(view -> {
            if (dialogDismissListener != null) dialogDismissListener.onDialogDismiss(null);
        });
        tvConfirm.setOnClickListener(view -> {
            if (isValid()) {
                if ((savedAddress != null && !savedAddress.zipcode.equalsIgnoreCase(etPostcode.getText().toString().replaceAll("\\s+", "").trim())) || (savedAddress == null && searchedPostcode == null)) {
                    moveNext = true;
                    String postCode = etPostcode.getText().toString().replaceAll("\\s+", "").trim();
                    new Thread(() -> {
                        searchPostcode(postCode);
                    }).start();
                } else {
                    new Thread(this::saveAddress).start();
                }

            }
        });
    }

    private void saveAddress() {
        if (getActivity() != null) getActivity().runOnUiThread(() -> progressDialog.show());
        Address toSaveAddress = new Address();

        if (savedAddress != null) {
            toSaveAddress.addressBookId = String.valueOf(savedAddress.id);
            toSaveAddress.zipcode = savedAddress.zipcode;
            toSaveAddress.latitude = savedAddress.latitude;
            toSaveAddress.longitude = savedAddress.longitude;
            if (res_id != null)
                toSaveAddress.resid = savedAddress.resid;
        } else {
            toSaveAddress.zipcode = searchedPostcode.post_code;
            toSaveAddress.latitude = searchedPostcode.latitude;
            toSaveAddress.longitude = searchedPostcode.longitude;
        }
        toSaveAddress.address = tvAddress.getText().toString();
        toSaveAddress.customer_id = String.valueOf(loggedInUser.id);
        toSaveAddress.flat_no = etHouseNumber.getText().toString();

        if (rgAddressTitle.getCheckedRadioButtonId() == R.id.rbHome) {
            toSaveAddress.title = "Home";
        }
        if (rgAddressTitle.getCheckedRadioButtonId() == R.id.rbWork) {
            toSaveAddress.title = "Work";
        }
        if (rgAddressTitle.getCheckedRadioButtonId() == R.id.rbOther) {
            toSaveAddress.title = "Other";
        }
        ANRequest anRequest;
        if (savedAddress == null) {
            anRequest = ApiUtils.saveAddress(toSaveAddress.address, toSaveAddress.customer_id, toSaveAddress.flat_no, toSaveAddress.title, toSaveAddress.zipcode, toSaveAddress.latitude, toSaveAddress.longitude, res_id);
        } else {
            anRequest = ApiUtils.updateAddress(toSaveAddress.addressBookId, toSaveAddress.address, String.valueOf(toSaveAddress.customer_id), toSaveAddress.flat_no, toSaveAddress.title, toSaveAddress.zipcode, toSaveAddress.latitude, toSaveAddress.longitude);
        }
        anRequest
                .getAsJSONObject(new JSONObjectRequestListener() {
                    @Override
                    public void onResponse(JSONObject response) {
                        if (getActivity() != null) {
                            CommonFunctions.hideKeyboard(getActivity(), etHouseNumber);
                            getActivity().runOnUiThread(() -> progressDialog.dismiss());
                        }
                            Address address = null;
                                address = new Gson().fromJson(response.toString(), Address.class);
                                if (!Validators.isNullOrEmpty(res_id)) {
                                    checkWithDeliverableAddresses(address);
                                } else {
                                    myApp.getMyPreferences().saveCurrentAddress(address);
                                    ToastUtils.makeSnackToast(getActivity(), "Address Added");
                                }

                            if (Validators.isNullOrEmpty(res_id) && dialogDismissListener != null)
                                dialogDismissListener.onDialogDismiss(address);
                    }

                    @Override
                    public void onError(ANError anError) {
                        anError.printStackTrace();
                        LogUtils.e("::SAVE ADDRESS ANERROR::");
                        if (anError.getErrorCode()==400) {
                            try {
                                addresses.clear();
                                JSONObject jsonObject = new JSONObject(anError.getErrorBody().toString());
                                ToastUtils.makeToast(getContext(),jsonObject.getString("message"));
                            } catch (JSONException exception) {
                                exception.printStackTrace();
                            }
                        }
                        if (getActivity() != null)
                            getActivity().runOnUiThread(() -> progressDialog.hide());

                        if (!CommonFunctions.isConnected(getActivity()))
                            myApp.noInternet(getActivity());
                    }
                });
    }

    private void checkWithDeliverableAddresses(Address address) {
        if (getActivity() != null) {
            getActivity().runOnUiThread(() -> progressDialog.show());
        }
        ArrayList<Address> nonDeliverableAddress = new ArrayList<>();
        ApiUtils.getCheckoutAddresses(loggedInUser.id, Integer.parseInt(res_id),1)
                .getAsJSONArray(new JSONArrayRequestListener() {
                    @Override
                    public void onResponse(JSONArray response) {
                        if (getActivity() != null) {
                            getActivity().runOnUiThread(() -> progressDialog.dismiss());
                        }
                         Type type = new TypeToken<List<Address>>() {
                        }.getType();
                        addresses.clear();
                        nonDeliverableAddress.clear();
                        addresses.addAll(new Gson().fromJson(response.toString(), type));

                        for (Address address:addresses)
                        {
                            if (address.delivery_status.equalsIgnoreCase("out_of_delivery"))
                            {
                                nonDeliverableAddress.add(address);

                            }
                        }
                        myApp.getMyPreferences().saveCurrentAddress(address);
                        if (nonDeliverableAddress.size() > 0) {
                            for (Address address1 : nonDeliverableAddress) {
                                if (address1.id == address.id) {
                                    if (dialogDismissListener != null) {
                                        dialogDismissListener.onDialogDismiss(null);
                                    }
                                    ToastUtils.makeSnackToast(getActivity(), "Sorry, entered address is not deliverable");
                                    myApp.getMyPreferences().saveCurrentAddress(null);
                                    return;
                                }
                            }
                        }
                        if (dialogDismissListener != null) {
                            dialogDismissListener.onDialogDismiss(address);
                        }
                        ToastUtils.makeSnackToast(getActivity(), "Address added");
                    }

                    @Override
                    public void onError(ANError anError) {
                        anError.printStackTrace();
                        if (getActivity() != null) {
                            getActivity().runOnUiThread(() -> progressDialog.dismiss());
                        }
                        if (!CommonFunctions.isConnected(getActivity()))
                            myApp.noInternet(getActivity());
                    }
                });

    }

    private boolean isValid() {
        etHouseNumber.setError(null);
        etPostcode.setError(null);
        tvAddress.setError(null);
        if (Validators.isNullOrEmpty(etPostcode.getText().toString().replaceAll("\\s+", "").trim())) {
            ToastUtils.makeToast(getActivity(), "Please enter postcode");
            return false;
        }
        if (Validators.isNullOrEmpty(etHouseNumber.getText().toString())) {
            etHouseNumber.setError("Please enter house number/name");
            etHouseNumber.requestFocus();
            return false;
        }


        return true;
    }

    private void searchPostcode(String postcode) {
        if (getActivity() != null) getActivity().runOnUiThread(progressDialog::show);
        AndroidNetworking.get(ApiEndPoints.RMS_POSTCODE + postcode)
                .build()
                .getAsObject(Postcode.class, new ParsedRequestListener<Postcode>() {
                    @Override
                    public void onResponse(Postcode response) {
                        if (getActivity() != null)
                            CommonFunctions.hideKeyboard(getActivity(), etPostcode);
                        if (getActivity() != null)
                            getActivity().runOnUiThread(() -> {
                                progressDialog.dismiss();
                                searchedPostcode = response;
                                myApp.getMyPreferences().saveCurrentPostcode(searchedPostcode);
                                if (savedAddress != null) {
                                    savedAddress.zipcode = searchedPostcode.post_code;
                                    savedAddress.latitude = searchedPostcode.latitude;
                                    savedAddress.longitude = searchedPostcode.longitude;
                                }
                                updateUi();
                            });

                    }

                    @Override
                    public void onError(ANError anError) {

                        anError.printStackTrace();
                        if (getActivity() != null)
                            getActivity().runOnUiThread(() -> progressDialog.dismiss());
                            try {
                                addresses.clear();
                                JSONObject jsonObject = new JSONObject(anError.getErrorBody().toString());
                                ToastUtils.makeToast(getContext(),"Invalid data");
                            } catch (JSONException exception) {
                                exception.printStackTrace();
                            }

                        if (!CommonFunctions.isConnected(getActivity())) {
                            myApp.noInternet(getActivity());
                        }
                    }
                });
    }

    private void updateUi() {
        if (searchedPostcode != null) {
            StringBuilder builder = new StringBuilder();
            builder.append(searchedPostcode.street);
            if (!Validators.isNullOrEmpty(searchedPostcode.dependent_locality)) {
                builder.append(", ");
                builder.append(searchedPostcode.dependent_locality);
            }
            if (!Validators.isNullOrEmpty(searchedPostcode.post_town)) {
                builder.append(", ");
                builder.append(searchedPostcode.post_town);
            }
            tvAddress.setText(builder.toString());
            etPostcode.setText(searchedPostcode.post_code);
            etHouseNumber.requestFocus();
            if (moveNext) new Thread(this::saveAddress).start();
        }
    }


}
