package com.tiffintom.fragment;

import android.animation.Animator;
import android.content.BroadcastReceiver;
import android.content.Context;
import android.content.Intent;
import android.content.IntentFilter;
import android.graphics.Color;
import android.graphics.drawable.ColorDrawable;
import android.os.Bundle;
import android.os.Handler;
import android.os.Looper;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.EditText;
import android.widget.LinearLayout;
import android.widget.TextView;

import androidx.annotation.NonNull;
import androidx.annotation.Nullable;
import androidx.appcompat.app.AlertDialog;
import androidx.localbroadcastmanager.content.LocalBroadcastManager;
import androidx.recyclerview.widget.LinearLayoutManager;
import androidx.recyclerview.widget.RecyclerView;
import androidx.swiperefreshlayout.widget.SwipeRefreshLayout;

import com.airbnb.lottie.LottieAnimationView;
import com.androidnetworking.AndroidNetworking;
import com.androidnetworking.error.ANError;
import com.androidnetworking.interfaces.JSONObjectRequestListener;
import com.google.gson.Gson;
import com.google.gson.reflect.TypeToken;
import com.iarcuschin.simpleratingbar.SimpleRatingBar;
import com.tiffintom.R;
import com.tiffintom.activity.RestaurantDetailsActivity;
import com.tiffintom.adapters.OrderHistoryAdapter;
import com.tiffintom.base.BaseFragment;
import com.tiffintom.common.CommonFunctions;
import com.tiffintom.interfaces.DialogDismissListener;
import com.tiffintom.models.CartItem;
import com.tiffintom.models.HistoryCart;
import com.tiffintom.models.OrderHistory;
import com.tiffintom.models.User;
import com.tiffintom.network.ApiEndPoints;
import com.tiffintom.notification.Config;
import com.tiffintom.utils.LogUtils;
import com.tiffintom.utils.ToastUtils;

import org.json.JSONException;
import org.json.JSONObject;

import java.lang.reflect.Type;
import java.util.ArrayList;
import java.util.List;

import static com.tiffintom.utils.Constants.CODE_REFRESH;

/**
 * Created by Amrish on 07-09-2020.
 */
public class OrderHistoryUpcomingFragment extends BaseFragment {
    public static OrderHistoryUpcomingFragment getInstance(String status) {
        OrderHistoryUpcomingFragment fragment = new OrderHistoryUpcomingFragment();
        Bundle bundle = new Bundle();
        bundle.putString("delivery_status", status);
        fragment.setArguments(bundle);
        return fragment;
    }

    private RecyclerView rvOrders;
    private OrderHistoryAdapter orderHistoryAdapter;
    private ArrayList<Object> feed = new ArrayList<>();
    private ArrayList<OrderHistory> orders = new ArrayList<>();
    private SwipeRefreshLayout swipeRefreshLayout;
    private User loggedInUser = myApp.getMyPreferences().getLoggedInUser();
    private String status;
    private LinearLayout llNoData;
    private TextView tvMessage;
    private BroadcastReceiver broadcastReceiver;

    @Nullable
    @Override
    public View onCreateView(@NonNull LayoutInflater inflater, @Nullable ViewGroup container, @Nullable Bundle savedInstanceState) {
        if (getArguments() != null) {
            status = getArguments().getString("delivery_status");
        }
        return inflater.inflate(R.layout.fragment_order_history, container, false);
    }

    @Override
    public void onViewCreated(@NonNull View view, @Nullable Bundle savedInstanceState) {
        super.onViewCreated(view, savedInstanceState);
        fetchData();
        setListener();
    }

    @Override
    protected void initViews(View view) {
        super.initViews(view);
        llNoData = view.findViewById(R.id.llNoData);
        tvMessage = view.findViewById(R.id.tvMessage);
        swipeRefreshLayout = view.findViewById(R.id.swipeRefreshLayout);
        rvOrders = view.findViewById(R.id.rvOrders);

        orderHistoryAdapter = new OrderHistoryAdapter(getActivity(), feed, this::viewReceipt, this::reOrder, this::openRatingDialog);
        rvOrders.setLayoutManager(new LinearLayoutManager(getActivity(), RecyclerView.VERTICAL, false));
        rvOrders.setAdapter(orderHistoryAdapter);
    }

    private void setListener() {
        swipeRefreshLayout.setOnRefreshListener(this::fetchData);
        broadcastReceiver = new BroadcastReceiver() {
            public void onReceive(Context context, Intent intent) {
                boolean check_update = intent.getBooleanExtra("refresh", false);
                if (check_update) {
                    fetchData();
                }
            }
        };
    }

    private void reOrder(int position, Object data) {
        if (data instanceof OrderHistory) {
            OrderHistory history = (OrderHistory) data;
            new Thread(() -> {
                int dbRestId = myApp.getAppDatabase().cartDao().getRestaurantId();
                if (dbRestId > 0) {
                    if (getActivity() != null) {
                        getActivity().runOnUiThread(() -> {
                            showOtherRestaurantCartWarning(history);
                        });
                    }
                } else {
                    addItemsInCart(history);
                }
            }).start();
        }

    }

    private void showOtherRestaurantCartWarning(OrderHistory orderHistory) {
        ConfirmationDialogFragment confirmationDialogFragment = ConfirmationDialogFragment.getInstance("Start new basket?", "Your basket already contains items from " + myApp.getMyPreferences().getOrderRestaurant().restaurant_name + ". Do you wish to clear the basket and add this item instead?");
        confirmationDialogFragment.show(getActivity().getSupportFragmentManager(), "confirm");
        confirmationDialogFragment.setDialogDismissListener(new DialogDismissListener() {
            @Override
            public void onDialogDismiss(Object o) {
                if (o instanceof String && ((String) o).equalsIgnoreCase("confirm")) {
                    new Thread(() -> {
                        myApp.getMyPreferences().deleteOrderRestaurant();
                        myApp.getAppDatabase().cartDao().nukeTable();
                        addItemsInCart(orderHistory);

                    }).start();
                }
            }
        });
    }

    private void addItemsInCart(OrderHistory orderHistory) {
        ArrayList<CartItem> cartItems = new ArrayList<>();
        for (HistoryCart historyCart : orderHistory.cart_view) {
            if (historyCart.total_price > 0 && !historyCart.subaddons_name.equalsIgnoreCase("offer")) {
                LogUtils.e(historyCart.menu_name);
                CartItem cartItem = new CartItem();
                cartItem.menu_id = historyCart.menu_id;
                cartItem.Addon_name = historyCart.subaddons_name;
                cartItem.menu_name = historyCart.menu_name;
                cartItem.res_id = historyCart.restaurant_id;

                cartItem.Menu_price = historyCart.menu_price;
                cartItem.quantity = historyCart.quantity;
                cartItem.Total = historyCart.total_price;

                if (historyCart.restaurant_menu != null)
                    cartItem.menu_type = historyCart.restaurant_menu.menu_type;
                cartItems.add(cartItem);
                new Thread(() -> {
                    myApp.getAppDatabase().cartDao().insertAll(cartItem);
                }).start();
            }
        }
        if (cartItems.size() > 0) {
            Intent intent = new Intent(getActivity(), RestaurantDetailsActivity.class);
            intent.putExtra("resId", orderHistory.restaurant.id);
            intent.putExtra("res_name", orderHistory.restaurant.restaurant_name);
            startActivityForResult(intent, CODE_REFRESH);
        }
    }

    private void viewReceipt(int position, Object data) {

    }

    private void fetchData() {
        new Thread(this::fetchOrders).start();
    }

    private void fetchOrders() {
        if (getActivity() != null)
            getActivity().runOnUiThread(() -> {
                swipeRefreshLayout.setRefreshing(true);
                llNoData.setVisibility(View.GONE);
            });
        AndroidNetworking.get(ApiEndPoints.NEW_ORDERS)
                .addQueryParameter("customer_id", String.valueOf(loggedInUser.id))
                .addQueryParameter("upcoming_status", "1")
                .addQueryParameter("per_page", "20")
                .addQueryParameter("page", "1")
                .build()
                .getAsJSONObject(new JSONObjectRequestListener() {
                    @Override
                    public void onResponse(JSONObject response) {
                        if (getActivity() != null)
                            getActivity().runOnUiThread(() -> swipeRefreshLayout.setRefreshing(false));
                        Type type = new TypeToken<List<OrderHistory>>() {
                        }.getType();
                        orders.clear();
                        try {
                            orders.addAll(new Gson().fromJson(response.getJSONArray("data").toString(), type));
                        } catch (JSONException exception) {
                            exception.printStackTrace();
                        }

                        filterOrders();
                        if(
                        getActivity() !=null)
                        getActivity().
                        runOnUiThread(() ->
                        {
                            if (feed.size() == 0) {
                                llNoData.setVisibility(View.VISIBLE);
                                tvMessage.setText(String.format("Seems like you don't have any %s orders.", status));
                            } else llNoData.setVisibility(View.GONE);
                        });
                }
                    @Override
                    public void onError(ANError anError) {
                        anError.printStackTrace();
                        if (getActivity() != null)
                            getActivity().runOnUiThread(() -> {
                                swipeRefreshLayout.setRefreshing(false);
                                llNoData.setVisibility(View.VISIBLE);
                                tvMessage.setText(String.format("Seems like you don't have any %s orders.", status));
                            });
                        LogUtils.e("ORDER HISTORY ERROR");
                        if (anError.getErrorCode() == 400) {
                            try {
                                JSONObject jsonObject = new JSONObject(anError.getErrorBody().toString());
//                                ToastUtils.makeToast(getContext(), jsonObject.getString("message"));
                            } catch (JSONException exception) {
                                exception.printStackTrace();
                            }
                        }else
                        {
                            if (!CommonFunctions.isConnected(getActivity()))
                                myApp.noInternet(getActivity());
                        }

                    }
                });
    }

    private void filterOrders() {
        feed.clear();
        for (OrderHistory orderHistory : orders) {
            if (status.equalsIgnoreCase("previous")) {
                if (orderHistory.status.equalsIgnoreCase("delivered") || orderHistory.status.equalsIgnoreCase("failed")) {
                    feed.add(orderHistory);
                }
            } else if (status.equalsIgnoreCase("upcoming")) {
                if (orderHistory.status.equalsIgnoreCase("accepted") || orderHistory.status.equalsIgnoreCase("pending") || orderHistory.status.equalsIgnoreCase("waiting") || orderHistory.status.equalsIgnoreCase("driver accepted") || orderHistory.status.equalsIgnoreCase("collected")) {
                    feed.add(orderHistory);
                }
            }
        }
        if (getActivity() != null)
            getActivity().runOnUiThread(() -> {
                orderHistoryAdapter.notifyDataSetChanged();
                if (feed.size() == 0) llNoData.setVisibility(View.VISIBLE);
                else llNoData.setVisibility(View.GONE);
            });
    }

    private void openRatingDialog(int position, Object object) {
        OrderHistory orderHistory = (OrderHistory) object;
        AlertDialog alertDialog = new AlertDialog.Builder(getActivity()).create();
        alertDialog.getWindow().setBackgroundDrawable(new ColorDrawable(Color.TRANSPARENT));
        LayoutInflater inflater = (LayoutInflater) getActivity().getSystemService(Context.LAYOUT_INFLATER_SERVICE);
        View dialogView = inflater.inflate(R.layout.dialog_write_review, null);

        SimpleRatingBar ratingBar = dialogView.findViewById(R.id.rating);
        EditText editText = dialogView.findViewById(R.id.et_Reviewtext);
        TextView tv_Cancel = dialogView.findViewById(R.id.tvCancel);
        TextView tv_Confirm = dialogView.findViewById(R.id.tvConfirm);
        TextView tv_Info = dialogView.findViewById(R.id.tvInfo);

        tv_Info.setText(new StringBuilder("How was your meal from ").append(orderHistory.restaurant.restaurant_name).append("? \nLeave a review here."));

        tv_Confirm.setOnClickListener(view -> {
            String review = editText.getText().toString();
            float ratings = ratingBar.getRating();
            if (ratings < 1) {
                ToastUtils.makeToast(getActivity(), "Please enter rating");
            } else {
                alertDialog.dismiss();
                sendrating(object, ratings, review);
            }

        });
        tv_Cancel.setOnClickListener(view -> alertDialog.dismiss());


        alertDialog.setView(dialogView);
        alertDialog.setCancelable(false);
        alertDialog.show();

    }


    private void sendrating(Object object, float ratings, String review) {
        int rating=(int) ratings;
        progressDialog.show();
        AndroidNetworking.post(ApiEndPoints.NEW_REVIEWS)
                .addBodyParameter("order_id", ((OrderHistory)object).id+"")
                .addBodyParameter("customer_id", String.valueOf(loggedInUser.id))
                .addBodyParameter("restaurant_id",((OrderHistory)object).restaurant_id+"")
                .addBodyParameter("responce", "")
                .addBodyParameter("rating", rating + "")
                .addBodyParameter("message", review)
                .addBodyParameter("order_type",((OrderHistory)object).order_type)
                .build()
                .getAsJSONObject(new JSONObjectRequestListener() {
                    @Override
                    public void onResponse(JSONObject response) {
                        if (getActivity() != null)
                            getActivity().runOnUiThread(() -> {
                                progressDialog.hide();
                            });
                        showReviewDone();
                    }

                    @Override
                    public void onError(ANError anError) {
                        anError.printStackTrace();
                        if (getActivity() != null)
                            progressDialog.hide();
                        ToastUtils.makeToast(getActivity(), "Something went wrong");

                        if (!CommonFunctions.isConnected(getActivity()))
                            myApp.noInternet(getActivity());
                    }
                });

    }

    private void showReviewDone() {
        AlertDialog alertDialog = new AlertDialog.Builder(getActivity()).create();
        alertDialog.getWindow().setBackgroundDrawable(new ColorDrawable(Color.TRANSPARENT));
        LayoutInflater inflater = (LayoutInflater) getActivity().getSystemService(Context.LAYOUT_INFLATER_SERVICE);
        View dialogView = inflater.inflate(R.layout.dialog_order_placed, null);

        LottieAnimationView lottieAnimationView = dialogView.findViewById(R.id.animation_view);
        TextView tvMessage = dialogView.findViewById(R.id.tvMessage);
        lottieAnimationView.setAnimation(R.raw.review_star);
        tvMessage.setText("Review submitted successfully");
        lottieAnimationView.addAnimatorListener(new Animator.AnimatorListener() {
            @Override
            public void onAnimationStart(Animator animator) {

            }

            @Override
            public void onAnimationEnd(Animator animator) {
                new Handler(Looper.myLooper()).postDelayed(new Runnable() {
                    @Override
                    public void run() {
                        alertDialog.dismiss();
                        new Thread(() -> {
                            fetchData();
                        }).start();

                    }
                }, 10);

            }

            @Override
            public void onAnimationCancel(Animator animator) {

            }

            @Override
            public void onAnimationRepeat(Animator animator) {

            }
        });
        alertDialog.setView(dialogView);
        alertDialog.setCancelable(false);
        alertDialog.show();
    }

    @Override
    public void onResume() {
        super.onResume();
        if (getActivity() != null && broadcastReceiver != null)
            LocalBroadcastManager.getInstance(getActivity()).registerReceiver(broadcastReceiver, new IntentFilter(Config.PUSH_NOTIFICATION));
    }

    @Override
    public void onStop() {
        if (getActivity() != null && broadcastReceiver != null)
            LocalBroadcastManager.getInstance(getActivity()).unregisterReceiver(broadcastReceiver);
        super.onStop();
    }
}
