package com.tiffintom.fragment;

import android.animation.Animator;
import android.content.BroadcastReceiver;
import android.content.Context;
import android.content.Intent;
import android.content.IntentFilter;
import android.graphics.Color;
import android.graphics.Paint;
import android.graphics.drawable.ColorDrawable;
import android.os.Bundle;
import android.os.Handler;
import android.os.Looper;
import android.view.KeyEvent;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.EditText;
import android.widget.ImageView;
import android.widget.LinearLayout;
import android.widget.TextView;

import androidx.annotation.NonNull;
import androidx.annotation.Nullable;
import androidx.appcompat.app.AlertDialog;
import androidx.core.widget.NestedScrollView;
import androidx.localbroadcastmanager.content.LocalBroadcastManager;
import androidx.recyclerview.widget.LinearLayoutManager;
import androidx.recyclerview.widget.RecyclerView;
import androidx.swiperefreshlayout.widget.SwipeRefreshLayout;

import com.airbnb.lottie.LottieAnimationView;
import com.androidnetworking.AndroidNetworking;
import com.androidnetworking.error.ANError;
import com.androidnetworking.interfaces.JSONObjectRequestListener;
import com.google.gson.Gson;
import com.iarcuschin.simpleratingbar.SimpleRatingBar;
import com.tiffintom.MyApp;
import com.tiffintom.R;
import com.tiffintom.activity.MainActivity;
import com.tiffintom.adapters.OrderHistoryCartAdapter;
import com.tiffintom.base.BaseFragment;
import com.tiffintom.common.CommonFunctions;
import com.tiffintom.common.Validators;
import com.tiffintom.models.HistoryCart;
import com.tiffintom.models.OrderDetail;
import com.tiffintom.models.OrderHistory;
import com.tiffintom.models.PaymentMethod;
import com.tiffintom.models.Review;
import com.tiffintom.models.User;
import com.tiffintom.network.ApiEndPoints;
import com.tiffintom.notification.Config;
import com.tiffintom.utils.LogUtils;
import com.tiffintom.utils.ToastUtils;

import org.json.JSONException;
import org.json.JSONObject;

import java.text.DecimalFormat;
import java.util.ArrayList;

import static com.tiffintom.network.ApiEndPoints.ACCOUNT;
import static com.tiffintom.network.ApiEndPoints.NEW_ORDERS;

/**
 * Created by Amrish on 07-09-2020.
 */
public class OrderReceiptFragment extends BaseFragment {
    private RecyclerView rvCartItems;
    private TextView tvTitle, tvSubTotal, tvServiceCharge, tvDeliveryFee, tvTipAmount, tvCharityAmount, tvDiscount, tvTotal, tvAddress, tvCardDetails, tvOrderId, tvScreenTitle, tvRewardDiscount;
    private ImageView ivMethod;
    private ArrayList<HistoryCart> carts = new ArrayList<>();
    private OrderHistoryCartAdapter cartAdapter;
    private DecimalFormat df = new DecimalFormat("#.##");
    private String orderId, restaurant_name;
    private NestedScrollView nestedScrollView;
    private SwipeRefreshLayout swipeRefreshLayout;
    private User loggedInUser = myApp.getMyPreferences().getLoggedInUser();
    private OrderDetail orderDetails;
    private ImageView ivBackArrow;
    private LinearLayout llBackArrow;
    private boolean isDineIn = false;
    private TextView tvWalletAmount, tvOrderType, tvOrderStatus, tvAddReview;
    private BroadcastReceiver broadcastReceiver;
    private boolean shouldRate;
    private boolean firstTime = true;
    private TextView tvInstructionTitle, tvInstructions, tvFailedReasonTitle, tvFailedReason;
    private LinearLayout cvInstructions, cvFailedReason;
    private boolean OrderPlace;

    public static OrderReceiptFragment getInstance() {
        OrderReceiptFragment fragment = new OrderReceiptFragment();
        Bundle bundle = new Bundle();
        fragment.setArguments(bundle);
        return fragment;
    }

    @Nullable
    @Override
    public View onCreateView(@NonNull LayoutInflater inflater, @Nullable ViewGroup container, @Nullable Bundle savedInstanceState) {
        if (getArguments() != null) {
            orderId = getArguments().getString("order_id");
            isDineIn = getArguments().getBoolean("is_dinein");
            shouldRate = getArguments().getBoolean("shouldRate");
            OrderPlace = getArguments().getBoolean("OrderPlace");
            restaurant_name = getArguments().getString("restaurant_name");
        }
        return inflater.inflate(R.layout.fragment_order_receipt, container, false);
    }

    @Override
    public void onViewCreated(@NonNull View view, @Nullable Bundle savedInstanceState) {
        super.onViewCreated(view, savedInstanceState);
        tvScreenTitle.setText("Receipt");
        tvScreenTitle.setVisibility(View.VISIBLE);
        view.setFocusableInTouchMode(true);
        view.requestFocus();
        view.setOnKeyListener(new View.OnKeyListener() {
            @Override
            public boolean onKey(View v, int keyCode, KeyEvent event) {
                if (keyCode == KeyEvent.KEYCODE_BACK && event.getAction() == KeyEvent.ACTION_UP) {
                    if (OrderPlace) {
                        startActivity(new Intent(getActivity(), MainActivity.class).setFlags(Intent.FLAG_ACTIVITY_CLEAR_TASK | Intent.FLAG_ACTIVITY_NEW_TASK | Intent.FLAG_ACTIVITY_CLEAR_TOP));
                    } else {
                        getActivity().onBackPressed();
                    }
                    return true;
                }
                return false;
            }
        });
        updateViews();
        fetchData();
        setListners();
        swipeRefreshLayout.setOnRefreshListener(this::fetchData);
        llBackArrow.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                if (OrderPlace) {
                    startActivity(new Intent(getActivity(), MainActivity.class).setFlags(Intent.FLAG_ACTIVITY_CLEAR_TASK | Intent.FLAG_ACTIVITY_NEW_TASK | Intent.FLAG_ACTIVITY_CLEAR_TOP));
                } else {
                    getActivity().onBackPressed();
                }
            }
        });
    }

    private void setListners() {


        tvAddReview.setOnClickListener(view -> openReviewDialog());
        broadcastReceiver = new BroadcastReceiver() {
            public void onReceive(Context context, Intent intent) {

                boolean check_update = intent.getBooleanExtra("refresh", false);
                if (check_update) {
                    fetchData();
                }
            }
        };
    }

    private void openReviewDialog() {
        AlertDialog alertDialog = new AlertDialog.Builder(getActivity()).create();
        alertDialog.getWindow().setBackgroundDrawable(new ColorDrawable(Color.TRANSPARENT));
        LayoutInflater inflater = (LayoutInflater) getActivity().getSystemService(Context.LAYOUT_INFLATER_SERVICE);
        View dialogView = inflater.inflate(R.layout.dialog_write_review, null);

        SimpleRatingBar ratingBar = dialogView.findViewById(R.id.rating);
        EditText editText = dialogView.findViewById(R.id.et_Reviewtext);
        TextView tv_Cancel = dialogView.findViewById(R.id.tvCancel);
        TextView tv_Confirm = dialogView.findViewById(R.id.tvConfirm);
        TextView tv_Info = dialogView.findViewById(R.id.tvInfo);

        if (restaurant_name != null) {
            tv_Info.setText(new StringBuilder("How was your meal from ").append(restaurant_name).append("? \nLeave a review here."));
        }

        tv_Confirm.setOnClickListener(view -> {
            String review = editText.getText().toString();
            float ratings = ratingBar.getRating();
            if (ratings < 1) {
                ToastUtils.makeToast(getActivity(), "Please enter rating");
            } else {
                sendrating(ratings, review);
                alertDialog.dismiss();
            }

        });
        tv_Cancel.setOnClickListener(view -> alertDialog.dismiss());
        alertDialog.setView(dialogView);
        alertDialog.setCancelable(false);
        alertDialog.show();

    }

    private void sendrating(float ratings, String review) {
        int rating = (int) ratings;
        progressDialog.show();
        AndroidNetworking.post(ApiEndPoints.NEW_REVIEWS)
                .addBodyParameter("order_id", orderId + "")
                .addBodyParameter("customer_id", String.valueOf(loggedInUser.id))
                .addBodyParameter("restaurant_id", String.valueOf(orderDetails.restaurant_id))
                .addBodyParameter("responce", "")
                .addBodyParameter("rating", rating + "")
                .addBodyParameter("message", review)
                .addBodyParameter("order_type", orderDetails.order_type)
                .build()
                .getAsJSONObject(new JSONObjectRequestListener() {
                    @Override
                    public void onResponse(JSONObject response) {
                        if (getActivity() != null)
                            getActivity().runOnUiThread(() -> {
                                progressDialog.hide();
                            });
                        showReviewDone();
                    }

                    @Override
                    public void onError(ANError anError) {
                        anError.printStackTrace();
                        if (getActivity() != null)
                            progressDialog.hide();
                        ToastUtils.makeToast(getActivity(), "Something went wrong");

                        if (!CommonFunctions.isConnected(getActivity()))
                            myApp.noInternet(getActivity());
                    }
                });


    }

    private void fetchData() {
        new Thread(this::fetchOrderDetails).start();
    }

    private void updateViews() {
        if (orderDetails != null) {
            PaymentMethod paymentMethod = new PaymentMethod();
            paymentMethod.type = orderDetails.payment_method;

            if (restaurant_name != null) {
                tvTitle.setText(String.format("Your receipt from %s on the %s", restaurant_name, CommonFunctions.formatTimestampZulu(orderDetails.created, "dd MMM yyyy 'at' hh:mm a")));
            }
            tvOrderId.setText("Order #" + orderDetails.order_number);

            if (!Validators.isNullOrEmpty(orderDetails.order_type)) {
                tvOrderType.setText(orderDetails.order_type.substring(0, 1).toUpperCase() + orderDetails.order_type.substring(1));
            } else {
                tvOrderType.setText("Dine in");
            }
            tvOrderStatus.setText(orderDetails.status.substring(0, 1).toUpperCase() + orderDetails.status.substring(1));

            carts.clear();
            if (isDineIn) {
                if (orderDetails.dinein_carts != null)
                    carts.addAll(orderDetails.dinein_carts);
            } else {
                if (orderDetails.cart_view != null)
                    carts.addAll(orderDetails.cart_view);
            }
            cartAdapter.notifyDataSetChanged();
            tvSubTotal.setText(String.format("Subtotal: %s%s", myApp.getCurrencySymbol(), MyApp.df.format(orderDetails.order_sub_total)));

            tvTotal.setText(String.format("Total: %s%s", myApp.getCurrencySymbol(), MyApp.df.format(orderDetails.order_grand_total + orderDetails.charity_amount)));

            if (orderDetails.offer_amount + orderDetails.voucher_amount > 0) {
                tvDiscount.setText(String.format("Discount: -%s%s", myApp.getCurrencySymbol(), MyApp.df.format(orderDetails.voucher_amount + orderDetails.offer_amount)));
                tvDiscount.setVisibility(View.VISIBLE);
            } else {
                tvDiscount.setVisibility(View.GONE);
            }
            if (orderDetails.reward_used.equalsIgnoreCase("yes") || orderDetails.reward_used.equalsIgnoreCase("y")) {
                tvRewardDiscount.setVisibility(View.VISIBLE);
                tvRewardDiscount.setText(String.format("Reward discount(%s%%): -%s%s", MyApp.df.format(orderDetails.reward_offer_percentage), myApp.getCurrencySymbol(), MyApp.df.format(orderDetails.reward_offer)));
            } else {
                tvRewardDiscount.setVisibility(View.GONE);
            }
            if (orderDetails.service_charge > 0) {
                tvServiceCharge.setText(String.format("Service charge: %s%s", myApp.getCurrencySymbol(), MyApp.df.format(orderDetails.service_charge)));
            }else
            {
                tvServiceCharge.setVisibility(View.GONE);
            }

            if (orderDetails.payment_method.equalsIgnoreCase("stripe")) {

                if (orderDetails.cardDetails != null && !Validators.isNullOrEmpty(orderDetails.cardDetails.card_brand)) {
                    paymentMethod.card_brand = orderDetails.cardDetails.card_brand;
                    tvCardDetails.setText(String.format("Your card **** %s was charged %s%s", orderDetails.cardDetails.card_number, myApp.getCurrencySymbol(), MyApp.df.format(orderDetails.order_grand_total + orderDetails.charity_amount - orderDetails.wallet_amount)));
                } else {
                    tvCardDetails.setText(String.format("Your card was charged %s%s", myApp.getCurrencySymbol(), MyApp.df.format(orderDetails.order_grand_total + orderDetails.charity_amount - orderDetails.wallet_amount)));
                }
            } else if (orderDetails.payment_method.equalsIgnoreCase("paypal")) {
                tvCardDetails.setText(String.format("Your paypal account was charged %s%s", myApp.getCurrencySymbol(), MyApp.df.format(orderDetails.order_grand_total + orderDetails.charity_amount - orderDetails.wallet_amount)));
            } else if (orderDetails.payment_method.equalsIgnoreCase("cod")) {
                tvCardDetails.setText(String.format("You Unpaid %s%s cash", myApp.getCurrencySymbol(), MyApp.df.format(orderDetails.order_grand_total + orderDetails.charity_amount - orderDetails.wallet_amount)));
            } else if (orderDetails.payment_method.equalsIgnoreCase("wallet") || orderDetails.payment_method.equalsIgnoreCase("credit")) {
                tvCardDetails.setText(String.format("%s%s paid via Tiffintom Credit", myApp.getCurrencySymbol(), MyApp.df.format(orderDetails.order_grand_total + orderDetails.charity_amount - orderDetails.wallet_amount)));
            }
            if (isDineIn) {
                tvAddress.setVisibility(View.GONE);
                tvDeliveryFee.setVisibility(View.GONE);

                if (orderDetails.gratuity > 0)
                    tvTipAmount.setText(String.format("Gratuity: %s%s", myApp.getCurrencySymbol(), MyApp.df.format(orderDetails.gratuity)));
                else
                    tvTipAmount.setVisibility(View.GONE);

            } else {
                if (orderDetails.delivery_charge > 0) {
                    tvDeliveryFee.setText(String.format("Delivery fee: %s%s", myApp.getCurrencySymbol(), MyApp.df.format(orderDetails.delivery_charge)));
                }else
                {
                    tvDeliveryFee.setVisibility(View.GONE);
                }
                if (!Validators.isNullOrEmpty(orderDetails.order_type) && orderDetails.order_type.equalsIgnoreCase("delivery")) {
                    if (orderDetails.status.equalsIgnoreCase("delivered")) {
                        tvAddress.setText(String.format("Delivered to %s ", orderDetails.address));
                    } else {
                        tvAddress.setText(String.format("Address: %s ", orderDetails.address));
                    }
                } else {
                    tvAddress.setVisibility(View.GONE);
                }


                if (orderDetails.driver_tip > 0)
                    tvTipAmount.setText(String.format("Tip Amount: %s%s", myApp.getCurrencySymbol(), MyApp.df.format(orderDetails.driver_tip)));
                else
                    tvTipAmount.setVisibility(View.GONE);

            }
            if (orderDetails.status.equalsIgnoreCase("delivered")) {
                if (orderDetails.reviews != null) {
                    Review review = orderDetails.reviews;
                    if (review.customer_id == MyApp.getInstance().getMyPreferences().getLoggedInUser().id) {
                        tvAddReview.setVisibility(View.GONE);
                    } else {
                        tvAddReview.setVisibility(View.VISIBLE);
                    }
                    tvAddReview.setVisibility(View.GONE);
                } else {
                    tvAddReview.setVisibility(View.VISIBLE);
                }
            } else {
                tvAddReview.setVisibility(View.GONE);
            }
            if (orderDetails.status.equalsIgnoreCase("failed") && !Validators.isNullOrEmpty(orderDetails.failed_reason)) {
                cvFailedReason.setVisibility(View.VISIBLE);
                tvFailedReasonTitle.setVisibility(View.VISIBLE);
                tvFailedReason.setText(orderDetails.failed_reason);
            } else {
                cvFailedReason.setVisibility(View.GONE);
                tvFailedReasonTitle.setVisibility(View.GONE);
            }
            if (orderDetails.charity_amount > 0)
                tvCharityAmount.setText(String.format("%s: %s%s", orderDetails.charity_message, myApp.getCurrencySymbol(), MyApp.df.format(orderDetails.charity_amount)));
            else {
                tvCharityAmount.setVisibility(View.GONE);
            }

            if (orderDetails.wallet_amount > 0)
                tvWalletAmount.setText(String.format("Wallet Amount: %s%s", myApp.getCurrencySymbol(), MyApp.df.format(orderDetails.wallet_amount)));
            else
                tvWalletAmount.setVisibility(View.GONE);

            ivMethod.setImageResource(myApp.paymentMethodLogo(paymentMethod));
            if (firstTime && shouldRate) {
                openReviewDialog();
            }
            if (Validators.isNullOrEmpty(orderDetails.order_description)) {
                tvInstructionTitle.setVisibility(View.GONE);
                cvInstructions.setVisibility(View.GONE);
            } else {
                tvInstructionTitle.setVisibility(View.VISIBLE);
                cvInstructions.setVisibility(View.VISIBLE);
                tvInstructions.setText((orderDetails.order_description));
            }
        }
    }

    @Override
    protected void initViews(View view) {
        super.initViews(view);
        tvUserName = view.findViewById(R.id.tvUserName);
        tvScreenTitle = view.findViewById(R.id.tvScreenTitle);
        ivBackArrow = view.findViewById(R.id.ivBackArrow);
        llBackArrow = view.findViewById(R.id.llBackArrow);
        rvCartItems = view.findViewById(R.id.rvCartItems);
        tvTitle = view.findViewById(R.id.tvTitle);
        tvOrderId = view.findViewById(R.id.tvOrderId);
        tvSubTotal = view.findViewById(R.id.tvSubTotal);
        tvServiceCharge = view.findViewById(R.id.tvServiceFee);
        tvDeliveryFee = view.findViewById(R.id.tvDeliveryFee);
        tvDiscount = view.findViewById(R.id.tvDiscount);
        tvTotal = view.findViewById(R.id.tvTotal);
        tvAddress = view.findViewById(R.id.tvAddress);
        tvCardDetails = view.findViewById(R.id.tvCardDetails);
        ivMethod = view.findViewById(R.id.ivMethod);
        swipeRefreshLayout = view.findViewById(R.id.swipeRefreshLayout);
        nestedScrollView = view.findViewById(R.id.nestedScrollView);

        tvTipAmount = view.findViewById(R.id.tvTipAmount);
        tvRewardDiscount = view.findViewById(R.id.tvRewardDiscount);
        tvCharityAmount = view.findViewById(R.id.tvCharityAmount);
        tvWalletAmount = view.findViewById(R.id.tvWalletAmount);
        tvOrderType = view.findViewById(R.id.tvOrderType);
        tvOrderStatus = view.findViewById(R.id.tvOrderStatus);
        tvAddReview = view.findViewById(R.id.tvAddReview);
        tvInstructionTitle = view.findViewById(R.id.tvInstructionsTitle);
        cvInstructions = view.findViewById(R.id.cvInstruction);
        tvInstructions = view.findViewById(R.id.tvInstructions);

        rvCartItems.setLayoutManager(new LinearLayoutManager(getActivity(), RecyclerView.VERTICAL, false));
        cartAdapter = new OrderHistoryCartAdapter(carts);
        rvCartItems.setNestedScrollingEnabled(false);
        rvCartItems.setAdapter(cartAdapter);


        tvFailedReason = view.findViewById(R.id.tvFailedReason);
        tvFailedReasonTitle = view.findViewById(R.id.tvFailedReasonTitle);
        cvFailedReason = view.findViewById(R.id.cvFailedReason);
        tvOrderId.setPaintFlags(Paint.UNDERLINE_TEXT_FLAG);
    }

    private void fetchOrderDetails() {
        if (getActivity() != null)
            getActivity().runOnUiThread(() -> {
                swipeRefreshLayout.setRefreshing(true);
                nestedScrollView.setVisibility(View.GONE);
            });
        AndroidNetworking.get(NEW_ORDERS + orderId).build()
                .getAsJSONObject(new JSONObjectRequestListener() {
                    @Override
                    public void onResponse(JSONObject response) {
                        if (getActivity() != null)
                            getActivity().runOnUiThread(() -> {
                                swipeRefreshLayout.setRefreshing(false);
                                nestedScrollView.setVisibility(View.VISIBLE);
                            });

                        orderDetails = new Gson().fromJson(response.toString(), OrderDetail.class);
                        updateViews();
                        firstTime = false;


                    }

                    @Override
                    public void onError(ANError anError) {
                        anError.printStackTrace();
                        LogUtils.e("ERROR IN ORDER VIEW");
                        if (getActivity() != null)
                            getActivity().runOnUiThread(() -> swipeRefreshLayout.setRefreshing(false));

                        if (!CommonFunctions.isConnected(getActivity()))
                            myApp.noInternet(getActivity());
                    }
                });
    }

    private void showReviewDone() {
        AlertDialog alertDialog = new AlertDialog.Builder(getActivity()).create();
        alertDialog.getWindow().setBackgroundDrawable(new ColorDrawable(Color.TRANSPARENT));
        LayoutInflater inflater = (LayoutInflater) getActivity().getSystemService(Context.LAYOUT_INFLATER_SERVICE);
        View dialogView = inflater.inflate(R.layout.dialog_order_placed, null);

        LottieAnimationView lottieAnimationView = dialogView.findViewById(R.id.animation_view);
        TextView tvMessage = dialogView.findViewById(R.id.tvMessage);
        lottieAnimationView.setAnimation(R.raw.review_star);
        tvMessage.setText("Review submitted successfully");
        lottieAnimationView.addAnimatorListener(new Animator.AnimatorListener() {
            @Override
            public void onAnimationStart(Animator animator) {

            }

            @Override
            public void onAnimationEnd(Animator animator) {
                new Handler(Looper.myLooper()).postDelayed(new Runnable() {
                    @Override
                    public void run() {
                        alertDialog.dismiss();
                        new Thread(() -> {
                            fetchData();
                        }).start();

                    }
                }, 10);

            }

            @Override
            public void onAnimationCancel(Animator animator) {

            }

            @Override
            public void onAnimationRepeat(Animator animator) {

            }
        });
        alertDialog.setView(dialogView);
        alertDialog.setCancelable(false);
        alertDialog.show();
    }

    @Override
    public void onResume() {
        super.onResume();
        if (getActivity() != null && broadcastReceiver != null)
            LocalBroadcastManager.getInstance(getActivity()).registerReceiver(broadcastReceiver, new IntentFilter(Config.PUSH_NOTIFICATION));
    }

    @Override
    public void onStop() {
        if (getActivity() != null && broadcastReceiver != null)
            LocalBroadcastManager.getInstance(getActivity()).unregisterReceiver(broadcastReceiver);
        super.onStop();
    }
}
