package com.tiffintom.fragment;

import android.content.DialogInterface;
import android.graphics.ColorFilter;
import android.graphics.PorterDuff;
import android.graphics.PorterDuffColorFilter;
import android.os.Build;
import android.os.Bundle;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.ProgressBar;

import androidx.annotation.NonNull;
import androidx.annotation.Nullable;
import androidx.core.content.ContextCompat;
import androidx.recyclerview.widget.LinearLayoutManager;
import androidx.recyclerview.widget.RecyclerView;

import com.airbnb.lottie.LottieAnimationView;
import com.airbnb.lottie.LottieProperty;
import com.airbnb.lottie.model.KeyPath;
import com.airbnb.lottie.value.LottieFrameInfo;
import com.airbnb.lottie.value.SimpleLottieValueCallback;
import com.androidnetworking.AndroidNetworking;
import com.androidnetworking.error.ANError;
import com.androidnetworking.interfaces.JSONArrayRequestListener;
import com.androidnetworking.interfaces.ParsedRequestListener;
import com.google.android.material.bottomsheet.BottomSheetDialogFragment;
import com.google.gson.Gson;
import com.google.gson.reflect.TypeToken;
import com.tiffintom.MyApp;
import com.tiffintom.R;
import com.tiffintom.adapters.PaymentMethodsAdapter;
import com.tiffintom.common.CommonFunctions;
import com.tiffintom.common.Validators;
import com.tiffintom.interfaces.DialogDismissListener;
import com.tiffintom.models.PaymentMethod;
import com.tiffintom.models.SavedCard;
import com.tiffintom.models.User;
import com.tiffintom.network.ApiEndPoints;
import com.tiffintom.utils.ToastUtils;

import org.json.JSONArray;

import java.lang.reflect.Type;
import java.util.ArrayList;
import java.util.List;

/**
 * Created by Amrish on 19-08-2020.
 */
public class PaymentMethodsBottomSheetFragment extends BottomSheetDialogFragment {

    private RecyclerView rvMethods;
    private PaymentMethodsAdapter paymentMethodsAdapter;
    private ArrayList<PaymentMethod> paymentMethods = new ArrayList<>();
    private ArrayList<SavedCard> savedCards = new ArrayList<>();
    private ProgressBar pbLoading;
    private User loggedInUser;
    private MyApp myApp = MyApp.getInstance();
    private DialogDismissListener dialogDismissListener;
    private ArrayList<PaymentMethod> restaurantPaymentMethods = new ArrayList<>();
    private PaymentMethod selectedPaymentMethod;
    private boolean dineIn;
    private String wallet_payment, dinein_wallet_payment;
    private LottieAnimationView lodingView;

    public static PaymentMethodsBottomSheetFragment getInstance(boolean onlyCards) {
        PaymentMethodsBottomSheetFragment fragment = new PaymentMethodsBottomSheetFragment();
        Bundle bundle = new Bundle();
        bundle.putBoolean("onlyCards", onlyCards);
        fragment.setArguments(bundle);
        return fragment;
    }

    public static PaymentMethodsBottomSheetFragment getInstance(boolean dineIn, ArrayList<PaymentMethod> restaurantPaymentMethods, String wallet_payment, String dinein_wallet_payment, PaymentMethod selectedPayementMethod) {
        PaymentMethodsBottomSheetFragment fragment = new PaymentMethodsBottomSheetFragment();
        Bundle bundle = new Bundle();
        bundle.putString("restaurant_payment_methods", new Gson().toJson(restaurantPaymentMethods));
        bundle.putString("selected_payment_method", new Gson().toJson(selectedPayementMethod));
        bundle.putString("wallet_payment", wallet_payment);
        bundle.putString("dinein_wallet_payment", dinein_wallet_payment);
        bundle.putBoolean("dineIn", dineIn);
        fragment.setArguments(bundle);
        return fragment;
    }

    private boolean onlyCards = false;

    @Override
    public int getTheme() {
        if (Build.VERSION.SDK_INT > 21) {
            return R.style.BottomSheetDialogTheme;
        } else {
            return super.getTheme();
        }
    }

    @Nullable
    @Override
    public View onCreateView(@NonNull LayoutInflater inflater, @Nullable ViewGroup container, @Nullable Bundle savedInstanceState) {
        if (getArguments() != null) {
            onlyCards = getArguments().getBoolean("onlyCards");
            dineIn = getArguments().getBoolean("dineIn");
            wallet_payment = getArguments().getString("wallet_payment");
            dinein_wallet_payment = getArguments().getString("dinein_wallet_payment");
            selectedPaymentMethod = new Gson().fromJson(getArguments().getString("selected_payment_method"), PaymentMethod.class);
            Type type = new TypeToken<List<PaymentMethod>>() {
            }.getType();
            if (getArguments().getString("restaurant_payment_methods") != null)
                restaurantPaymentMethods.addAll(new Gson().fromJson(getArguments().getString("restaurant_payment_methods"), type));
        }
        return inflater.inflate(R.layout.bottomsheet_fragment_payment_methods, container, false);
    }

    @Override
    public void onViewCreated(@NonNull View view, @Nullable Bundle savedInstanceState) {
        super.onViewCreated(view, savedInstanceState);
        loggedInUser = myApp.getMyPreferences().getLoggedInUser();
        initViews(view);
        if (onlyCards) {
            fetchCards();
        } else {
            fetchProfile();
        }
    }

    public void setDialogDismissListener(DialogDismissListener dialogDismissListener) {
        this.dialogDismissListener = dialogDismissListener;
    }

    protected void initViews(View view) {
        rvMethods = view.findViewById(R.id.rvPaymentMethods);
        pbLoading = view.findViewById(R.id.pbLoading);
        lodingView = view.findViewById(R.id.lodingView);
        lodingView.addValueCallback(
                new KeyPath("**"),
                LottieProperty.COLOR_FILTER,
                new SimpleLottieValueCallback<ColorFilter>() {
                    @Override
                    public ColorFilter getValue(LottieFrameInfo<ColorFilter> frameInfo) {
                        return new PorterDuffColorFilter(ContextCompat.getColor(getActivity(), R.color.colorPrimary), PorterDuff.Mode.SRC_ATOP);
                    }
                }
        );

        paymentMethodsAdapter = new PaymentMethodsAdapter(getActivity(), paymentMethods, onlyCards, this::methodClick);
        rvMethods.setLayoutManager(new LinearLayoutManager(getActivity(), RecyclerView.VERTICAL, false));
        rvMethods.setAdapter(paymentMethodsAdapter);
    }

    private void fetchProfile() {
        AndroidNetworking.get(ApiEndPoints.user_account)
                .build()
                .getAsObject(User.class, new ParsedRequestListener<User>() {
                    @Override
                    public void onResponse(User response) {
                        try {
                            myApp.getMyPreferences().saveLoggedInUser(response);
                            loggedInUser = myApp.getMyPreferences().getLoggedInUser();
                            myApp.profileChanged();
                            fetchCards();
                        } catch (Exception e) {
                            e.printStackTrace();
                        }

                    }

                    @Override
                    public void onError(ANError anError) {
                        anError.printStackTrace();
                        if (getActivity() != null) {
                            getActivity().runOnUiThread(() -> lodingView.setVisibility(View.GONE));
                        }
                        if (!CommonFunctions.isConnected(getActivity())) {
                            myApp.noInternet(getActivity());
                        }
                    }
                });
    }

    private void fetchCards() {

        if (getActivity() != null)
            getActivity().runOnUiThread(() -> lodingView.setVisibility(View.VISIBLE));

        AndroidNetworking.get(ApiEndPoints.cards)
                .addQueryParameter("customer_id", String.valueOf(loggedInUser.id))
                .addQueryParameter("nopaginate", "1")
                .build()
                .getAsJSONArray(new JSONArrayRequestListener() {
                    @Override
                    public void onResponse(JSONArray response) {
                        if (getActivity() != null)
                            getActivity().runOnUiThread(() -> lodingView.setVisibility(View.GONE));
                        Type type = new TypeToken<List<SavedCard>>() {
                        }.getType();
                        ArrayList<SavedCard> cards = new ArrayList<>();
                        cards.addAll(new Gson().fromJson(response.toString(), type));
                        savedCards.clear();
                        if (onlyCards) {
                            for (SavedCard card : cards) {
                                if (card.service_type.equalsIgnoreCase("normal")) {
                                    savedCards.add(card);
                                }
                            }

                        } else
                            savedCards.addAll(cards);
                        showMethods();
                    }

                    @Override
                    public void onError(ANError anError) {
                        if (getActivity() != null)
                            getActivity().runOnUiThread(() -> lodingView.setVisibility(View.GONE));
                        anError.printStackTrace();
                        showMethods();
                        if (!CommonFunctions.isConnected(getActivity())) {
                            myApp.noInternet(getActivity());
                        }
                    }
                });
    }

    private void showMethods() {
        paymentMethods.clear();
        PaymentMethod paymentMethod;
        if (restaurantPaymentMethods.size() > 0) {

            for (PaymentMethod restaurantPaymentMethod : restaurantPaymentMethods) {
                if (restaurantPaymentMethod.payment_method_name.equalsIgnoreCase("cod")) {
                    boolean canBeAdd = false;
                    if (dineIn && restaurantPaymentMethod.dinein_android_status.equalsIgnoreCase("y")) {
                        canBeAdd = true;
                    }
                    if (restaurantPaymentMethod.android_status.equalsIgnoreCase("y")) {
                        canBeAdd = true;
                    }
                    if (canBeAdd) {
                        paymentMethod = new PaymentMethod();
                        paymentMethod.type = "cod";
                        if (dineIn)
                            paymentMethod.title = "Pay at restaurant";
                        else
                            paymentMethod.title = "Cash on delivery";
                        paymentMethod.method_logo = myApp.paymentMethodLogo(paymentMethod);
                        paymentMethods.add(paymentMethod);
                    }
                } else if (restaurantPaymentMethod.payment_method_name.equalsIgnoreCase("stripe")) {
                    boolean canBeAdd = false;
                    if (dineIn && restaurantPaymentMethod.dinein_android_status.equalsIgnoreCase("y")) {
                        canBeAdd = true;
                    }
                    if (restaurantPaymentMethod.android_status.equalsIgnoreCase("y")) {
                        canBeAdd = true;
                    }
                    if (canBeAdd) {
                        for (SavedCard card : savedCards) {
                            paymentMethod = new PaymentMethod();
                            paymentMethod.type = "Stripe";
                            paymentMethod.title = "**** " + card.card_number;
                            paymentMethod.card_number = card.card_number;
                            paymentMethod.card_id = card.id;
                            paymentMethod.stripe_customer_id = card.stripe_customer_id;
                            paymentMethod.stripe_token_id = card.stripe_token_id;
                            paymentMethod.card_brand = card.card_brand;
                            paymentMethod.exp_month = card.exp_month;
                            paymentMethod.exp_year = card.exp_year;
                            paymentMethod.method_logo = myApp.paymentMethodLogo(paymentMethod);

                            paymentMethods.add(paymentMethod);
                        }

                    }
                } else if (restaurantPaymentMethod.payment_method_name.equalsIgnoreCase("paypal")) {
                    boolean canBeAdd = false;
                    if (restaurantPaymentMethod.android_status.equalsIgnoreCase("y")) {
                        canBeAdd = true;
                    }
                    if (dineIn && restaurantPaymentMethod.dinein_android_status.equalsIgnoreCase("y")) {
                        canBeAdd = false;
                    }
                    if (canBeAdd) {
                        paymentMethod = new PaymentMethod();
                        paymentMethod.type = "paypal";
                        paymentMethod.title = "PayPal";
                        paymentMethod.method_logo = myApp.paymentMethodLogo(paymentMethod);
                        paymentMethods.add(paymentMethod);
                    }
                } else if (restaurantPaymentMethod.payment_method_name.equalsIgnoreCase("wallet") || restaurantPaymentMethod.payment_method_name.equalsIgnoreCase("credit")) {
                    //TODO
                    /*if (MyApp.openedRestaurantDetails != null && MyApp.openedRestaurantDetails.site_setting.wallet_available.equalsIgnoreCase("yes")) {
                        boolean canBeAdd = false;
                        if (dineIn && restaurantPaymentMethod.dinein_android_status.equalsIgnoreCase("y")) {
                            canBeAdd = true;
                        }
                        if (restaurantPaymentMethod.android_status.equalsIgnoreCase("y")) {
                            canBeAdd = true;
                        }
*//*
                        if (dineIn) {
                            if (MyApp.openedRestaurantDetails.dinein_wallet_payment.equalsIgnoreCase("yes")) {
                                paymentMethod = new PaymentMethod();
                                paymentMethod.type = "Wallet";
                                paymentMethod.title = "Credit";
                                paymentMethod.subtitle = "Balance " + myApp.getCurrencySymbol() + MyApp.df.format(loggedInUser.wallet_amount);
                                paymentMethod.balance = loggedInUser.wallet_amount;
                                paymentMethod.method_logo = R.drawable.ic_tiffin;
                                paymentMethods.add(paymentMethod);
                            } else {
                                if (MyApp.openedRestaurantDetails.wallet_payment.equalsIgnoreCase("yes")) {
                                    paymentMethod = new PaymentMethod();
                                    paymentMethod.type = "Wallet";
                                    paymentMethod.title = "Credit";
                                    paymentMethod.subtitle = "Balance " + myApp.getCurrencySymbol() + MyApp.df.format(loggedInUser.wallet_amount);
                                    paymentMethod.balance = loggedInUser.wallet_amount;
                                    paymentMethod.method_logo = R.drawable.ic_tiffin;
                                    paymentMethods.add(paymentMethod);
                                }


                            }
                        }
*//*
                    }*/
                }

            }
            if (!Validators.isNullOrEmpty(wallet_payment)) {
                boolean canBeAdd = false;
                if (dineIn && dinein_wallet_payment.equalsIgnoreCase("yes")) {
                    canBeAdd = true;
                }
                if (!dineIn && wallet_payment.equalsIgnoreCase("yes")) {
                    canBeAdd = true;
                }
/*
                if (dineIn) {
                    if (MyApp.openedRestaurantDetails.dinein_wallet_payment.equalsIgnoreCase("yes")) {
                        paymentMethod = new PaymentMethod();
                        paymentMethod.type = "Wallet";
                        paymentMethod.title = "Credit";
                        paymentMethod.subtitle = "Balance " + myApp.getCurrencySymbol() + MyApp.df.format(loggedInUser.wallet_amount);
                        paymentMethod.balance = loggedInUser.wallet_amount;
                        paymentMethod.method_logo = R.drawable.ic_tiffin;
                        paymentMethods.add(paymentMethod);
                    } else {
                        if (MyApp.openedRestaurantDetails.wallet_payment.equalsIgnoreCase("yes")) {
                            paymentMethod = new PaymentMethod();
                            paymentMethod.type = "Wallet";
                            paymentMethod.title = "Credit";
                            paymentMethod.subtitle = "Balance " + myApp.getCurrencySymbol() + MyApp.df.format(loggedInUser.wallet_amount);
                            paymentMethod.balance = loggedInUser.wallet_amount;
                            paymentMethod.method_logo = R.drawable.ic_tiffin;
                            paymentMethods.add(paymentMethod);
                        }

                    }

                }
*/
            }
        } else {

            if (!onlyCards) {
                paymentMethod = new PaymentMethod();
                paymentMethod.type = "cod";
                if (dineIn)
                    paymentMethod.title = "Pay at restaurant";
                else
                    paymentMethod.title = "Cash on delivery";
                paymentMethod.method_logo = R.drawable.icon_cod;
                paymentMethods.add(paymentMethod);
            }
            for (SavedCard card : savedCards) {
                paymentMethod = new PaymentMethod();
                paymentMethod.type = "stripe";
                paymentMethod.title = "**** " + card.card_number;
                paymentMethod.card_number = card.card_number;
                paymentMethod.card_id = card.id;
                paymentMethod.stripe_customer_id = card.stripe_customer_id;
                paymentMethod.stripe_token_id = card.stripe_token_id;
                paymentMethod.card_brand = card.card_brand;
                paymentMethod.exp_month = card.exp_month;
                paymentMethod.exp_year = card.exp_year;
                paymentMethod.method_logo = myApp.paymentMethodLogo(paymentMethod);

                paymentMethods.add(paymentMethod);
            }
            if (!onlyCards && !dineIn) {
                paymentMethod = new PaymentMethod();
                paymentMethod.type = "paypal";
                paymentMethod.title = "PayPal";
                paymentMethod.method_logo = myApp.paymentMethodLogo(paymentMethod);
                paymentMethods.add(paymentMethod);
            }

            if (!onlyCards) {

/*
                if (dineIn) {
                    if (MyApp.openedRestaurantDetails.dinein_wallet_payment.equalsIgnoreCase("yes")) {
                        paymentMethod = new PaymentMethod();
                        paymentMethod.type = "Wallet";
                        paymentMethod.title = "Credit";
                        paymentMethod.subtitle = "Balance " + myApp.getCurrencySymbol() + MyApp.df.format(loggedInUser.wallet_amount);
                        paymentMethod.balance = loggedInUser.wallet_amount;
                        paymentMethod.method_logo = R.drawable.ic_tiffin;
                        paymentMethods.add(paymentMethod);
                    } else {
                        if (MyApp.openedRestaurantDetails.wallet_payment.equalsIgnoreCase("yes")) {
                            paymentMethod = new PaymentMethod();
                            paymentMethod.type = "Wallet";
                            paymentMethod.title = "Credit";
                            paymentMethod.subtitle = "Balance " + myApp.getCurrencySymbol() + MyApp.df.format(loggedInUser.wallet_amount);
                            paymentMethod.balance = loggedInUser.wallet_amount;
                            paymentMethod.method_logo = R.drawable.ic_tiffin;
                            paymentMethods.add(paymentMethod);
                        }

                    }

                }
*/


            }

        }

        paymentMethod = new PaymentMethod();
        paymentMethod.type = "add";
        paymentMethod.title = "Add payment type";
        paymentMethods.add(paymentMethod);

        paymentMethodsAdapter.notifyDataSetChanged();
    }

    private void methodClick(int position, Object data) {
        if (data instanceof PaymentMethod) {
            if (((PaymentMethod) data).type.equalsIgnoreCase("add")) {
                //TODO
                AddCardFragment addCardFragment = AddCardFragment.getInstance(true,null);
                addCardFragment.show(getChildFragmentManager(), "add_card");
                addCardFragment.setDialogDismissListener(addCardDialogListener);

            } else {
                if (((PaymentMethod) data).type.equalsIgnoreCase("wallet") || ((PaymentMethod) data).type.equalsIgnoreCase("credit")) {
                    if (((PaymentMethod) data).balance > 0) {

                        if (dialogDismissListener != null) {
                            dialogDismissListener.onDialogDismiss(data);
                            dismiss();
                        }
                    } else {
                        ToastUtils.makeToast(getActivity(), "Insufficient balance, Please select other payment method");
                    }
                } else {
                    if (dialogDismissListener != null) {
                        dialogDismissListener.onDialogDismiss(data);
                        dismiss();
                    }
                }
            }
        }
    }

    DialogDismissListener addCardDialogListener = new DialogDismissListener() {
        @Override
        public void onDialogDismiss(Object o) {
            fetchCards();
        }
    };

    @Override
    public void onDismiss(@NonNull DialogInterface dialog) {
        if (dialogDismissListener != null) {
            dialogDismissListener.onDialogDismiss(null);
            dismiss();
        }
    }
}
