package com.tiffintom.fragment;

import android.app.FragmentManager;
import android.graphics.Color;
import android.os.Build;
import android.os.Bundle;
import android.os.CountDownTimer;
import android.text.Editable;
import android.text.InputFilter;
import android.text.TextWatcher;
import android.util.Log;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.Button;
import android.widget.EditText;
import android.widget.LinearLayout;
import android.widget.TextView;

import androidx.annotation.NonNull;
import androidx.annotation.Nullable;
import androidx.appcompat.app.AlertDialog;

import com.androidnetworking.AndroidNetworking;
import com.androidnetworking.error.ANError;
import com.androidnetworking.interfaces.JSONObjectRequestListener;
import com.google.android.material.bottomsheet.BottomSheetDialogFragment;
import com.tiffintom.MyApp;
import com.tiffintom.R;
import com.tiffintom.common.CommonFunctions;
import com.tiffintom.common.Validators;
import com.tiffintom.interfaces.DialogDismissListener;
import com.tiffintom.models.ApiError;
import com.tiffintom.models.User;
import com.tiffintom.utils.LogUtils;
import com.tiffintom.utils.ToastUtils;

import org.json.JSONException;
import org.json.JSONObject;

import static com.tiffintom.network.ApiEndPoints.UPDATE_ACCOUNT;

public class PhoneOrEmailVerificationBottomsheet extends BottomSheetDialogFragment {

    private Button btnVerify;
    private EditText etType, etOtp;
    private DialogDismissListener dialogDismissListener;
    private String customerPhone, customerEmail, type;
    private int customerId;

    private Button btnSend;
    private TextView tvInfo,tvSkip,tvTimer;
    private LinearLayout llAfterOtpSend;
    private MyApp myApp = new MyApp();
    private TextView tvResedOtp, tvTitle, tvLabel;
    private AlertDialog progressDialog;
    private User loggedInUser = MyApp.getInstance().getMyPreferences().getLoggedInUser();

    public static PhoneOrEmailVerificationBottomsheet getInstance(String customerPhone, int customer_id, String email, String type) {
        PhoneOrEmailVerificationBottomsheet phoneVerificationBottomsheet = new PhoneOrEmailVerificationBottomsheet();
        Bundle bundle = new Bundle();
        bundle.putString("customer phone", customerPhone);
        bundle.putInt("customer booking_id", customer_id);
        bundle.putString("customer email", email);
        bundle.putString("type", type);
        phoneVerificationBottomsheet.setArguments(bundle);
        return phoneVerificationBottomsheet;
    }

    public void setDialogDismissListener(DialogDismissListener dialogDismissListener) {
        this.dialogDismissListener = dialogDismissListener;
    }

    @Override
    public int getTheme() {
        if (Build.VERSION.SDK_INT > 21) {
            return R.style.BottomSheetDialogTheme;
        } else {
            return super.getTheme();
        }
    }

    @Nullable
    @Override
    public View onCreateView(@NonNull LayoutInflater inflater, @Nullable ViewGroup container, @Nullable Bundle savedInstanceState) {
        if (getArguments() != null) {
            customerPhone = getArguments().getString("customer phone");
            customerId = getArguments().getInt("customer booking_id");
            customerEmail = getArguments().getString("customer email");
            type = getArguments().getString("type");
        }
        return inflater.inflate(R.layout.fragment_verify_phone, container, false);
    }

    @Override
    public void onViewCreated(@NonNull View view, @Nullable Bundle savedInstanceState) {
        super.onViewCreated(view, savedInstanceState);
        initViews(view);
        setListeners();
        updateViews();
    }

    private void updateViews() {
        if (!Validators.isNullOrEmpty(type)) {
            if (type.equalsIgnoreCase("email")) {
                etType.setText(customerEmail);
                tvTitle.setText("Verify Email");
                tvLabel.setText("Email");
                tvInfo.setText("A verification code will be sent to this email");
            } else {
                tvInfo.setText("A verification code will be sent to this phone number");
                etType.setText(customerPhone);
                tvTitle.setText("Verify Phone Number");
                tvLabel.setText("Phone number");
            }
        }
    }

    private void verifyOtp() {
        AndroidNetworking.post(UPDATE_ACCOUNT + customerId + (type.equalsIgnoreCase("phone") ? "/verify-otp" : "/email-verify-otp"))
                .addBodyParameter("otp", etOtp.getText().toString())
                .build()
                .getAsJSONObject(new JSONObjectRequestListener() {
                    @Override
                    public void onResponse(JSONObject response) {
                        CommonFunctions.hideKeyboard(getActivity(), btnVerify);
                        String message = type + " Verified";
                        ToastUtils.makeToast(getActivity(), message);
                        if (dialogDismissListener != null) {
                            dialogDismissListener.onDialogDismiss(response);
                        }
                        dismiss();
                    }

                    @Override
                    public void onError(ANError anError) {
                        anError.printStackTrace();
                        if (anError.getErrorCode() == 400) {
                            try {
                                JSONObject jsonObject = new JSONObject(anError.getErrorBody().toString());
                                ToastUtils.makeToast(getContext(), jsonObject.getString("message"));
                            } catch (JSONException exception) {
                                exception.printStackTrace();
                            }
                        }
                    }
                });

    }

    private void setListeners() {

        etType.addTextChangedListener(new TextWatcher() {
            @Override
            public void beforeTextChanged(CharSequence charSequence, int i, int i1, int i2) {

            }

            @Override
            public void onTextChanged(CharSequence charSequence, int i, int i1, int i2) {

                if (!customerPhone.equalsIgnoreCase(etType.getText().toString())) {
                    customerPhone = etType.getText().toString();
                  /*  btnSend.setEnabled(true);
                    btnSend.setBackgroundColor(getResources().getColor(R.color.black));*/
                }


            }

            @Override
            public void afterTextChanged(Editable editable) {

            }
        });
        btnVerify.setOnClickListener(view -> {
            if (isValid())
                verifyOtp();
        });
        tvResedOtp.setOnClickListener(view -> {
            resendOtp();
            etOtp.setText("");
        });
        btnSend.setOnClickListener(view -> {
            verifyCustomerPhoneOrEmail(type);
        });

    }

    private void resendOtp() {

        verifyCustomerPhoneOrEmail(type);
    }

    private boolean isValid() {
        etType.setError(null);
        etOtp.setError(null);
        if (!customerPhone.equalsIgnoreCase(etType.getText().toString()) && type.equalsIgnoreCase("phone")) {
            etType.setError(null);
            etType.requestFocus();
            customerPhone = etType.getText().toString();
            ToastUtils.makeToast(getContext(), "Mobile number changed,Please send code to verify");
            btnSend.setEnabled(true);
            btnSend.setBackgroundColor(getResources().getColor(R.color.black));
            return false;
        }
        if (Validators.isNullOrEmpty(etType.getText().toString())) {
            etType.setError("Please enter phone number");
            etType.requestFocus();
            return false;
        }
        if (Validators.isNullOrEmpty(etOtp.getText().toString())) {
            etOtp.setError("Please verification code");
            etOtp.requestFocus();
            return false;
        }
        return true;
    }

    private void initViews(View view) {
        progressDialog = CommonFunctions.customProgressDialog(getActivity(), "");
        etType = view.findViewById(R.id.etPhoneNumber);
        etOtp = view.findViewById(R.id.etOtp);
        btnVerify = view.findViewById(R.id.btnVerify);
        tvResedOtp = view.findViewById(R.id.tvResendOtp);
        tvTitle = view.findViewById(R.id.tvTitile);
        tvLabel = view.findViewById(R.id.tvLabel);
        llAfterOtpSend = view.findViewById(R.id.llAfterOtpSend);
        tvInfo = view.findViewById(R.id.tvInfo);
        tvSkip = view.findViewById(R.id.tvSkip);
        btnSend = view.findViewById(R.id.btnSend);
        tvTimer = view.findViewById(R.id.tvTimer);

        tvSkip.setVisibility(View.GONE);

        etType.setText(customerPhone);
        if (type.equalsIgnoreCase("phone")) {
            llAfterOtpSend.setVisibility(View.VISIBLE);
            int maxLength = 11;
            etType.setFilters(new InputFilter[]{new InputFilter.LengthFilter(maxLength)});
            verifyCustomerPhoneOrEmail(type);
        } else {
            etType.setEnabled(false);
        }


    }

    private void startTimer() {
        new CountDownTimer(30000, 1000) { //Set Timer for 5 seconds
            public void onTick(long millisUntilFinished) {
                int seconds = (int) (millisUntilFinished / 1000);
                int minutes = seconds / 60;
                seconds = seconds % 60;
                tvTimer.setText(String.format("%02d", minutes)
                        + ":" + String.format("%02d", seconds));

            }
            @Override
            public void onFinish() {
                btnSend.setEnabled(true);
                tvTimer.setText("00:00");
                btnSend.setBackgroundColor(Color.BLACK);
            }
        }.start();
    }

    private void verifyCustomerPhoneOrEmail(String type) {
        if (getActivity() != null) {
            getActivity().runOnUiThread(() -> progressDialog.show());
        }
        AndroidNetworking.get(UPDATE_ACCOUNT + "send-otp")
                .build()
                .getAsJSONObject(new JSONObjectRequestListener() {
                    @Override
                    public void onResponse(JSONObject response) {
                        if (getActivity() != null) {
                            getActivity().runOnUiThread(() -> {
                                llAfterOtpSend.setVisibility(View.VISIBLE);
                                btnSend.setEnabled(false);
                                btnSend.setBackgroundColor(getResources().getColor(R.color.disabled_color));
                                if (type.equalsIgnoreCase("phone")) {
                                    tvInfo.setText("A verification code has been sent to this phone number");
                                } else {
                                    tvInfo.setText("A verification code has been sent to this email");
                                }
                            });
                        }

                        String message = null;
                        try {
                            message = response.getString("message");

                            ToastUtils.makeToast(getActivity(), message);
                            if (type.equalsIgnoreCase("phone"))
                            {
                                startTimer();
                            }
                            if (getActivity() != null) {
                                getActivity().runOnUiThread(() -> progressDialog.dismiss());
                            }
                        } catch (JSONException exception) {
                            exception.printStackTrace();
                        }


                    }

                    @Override
                    public void onError(ANError anError) {
                        anError.printStackTrace();
                        if (getActivity() != null) {
                            getActivity().runOnUiThread(() -> progressDialog.dismiss());
                        }

                    }
                });

    }


}
