package com.tiffintom.fragment;

import static com.tiffintom.network.ApiEndPoints.UPDATE_ACCOUNT;

import android.content.Intent;
import android.graphics.Color;
import android.graphics.Paint;
import android.os.Bundle;
import android.os.CountDownTimer;
import android.text.Editable;
import android.text.InputFilter;
import android.text.TextWatcher;
import android.text.style.UnderlineSpan;
import android.view.Gravity;
import android.view.KeyEvent;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.Button;
import android.widget.EditText;
import android.widget.LinearLayout;
import android.widget.TextView;

import androidx.annotation.NonNull;
import androidx.annotation.Nullable;
import androidx.appcompat.app.AlertDialog;

import com.androidnetworking.AndroidNetworking;
import com.androidnetworking.error.ANError;
import com.androidnetworking.interfaces.JSONObjectRequestListener;
import com.androidnetworking.interfaces.ParsedRequestListener;
import com.google.gson.Gson;
import com.tiffintom.MyApp;
import com.tiffintom.R;
import com.tiffintom.activity.MainActivity;
import com.tiffintom.activity.SignupActivity;
import com.tiffintom.activity.SplashActivity;
import com.tiffintom.activity.TransportActivity;
import com.tiffintom.base.BaseFragment;
import com.tiffintom.common.CommonFunctions;
import com.tiffintom.common.Validators;
import com.tiffintom.interfaces.DialogDismissListener;
import com.tiffintom.models.User;
import com.tiffintom.network.ApiEndPoints;
import com.tiffintom.utils.LogUtils;
import com.tiffintom.utils.ToastUtils;

import org.json.JSONException;
import org.json.JSONObject;

public class PhoneOrEmailVerificationFragment extends BaseFragment {
    public static PhoneOrEmailVerificationFragment getInstance() {
        PhoneOrEmailVerificationFragment phoneOrEmailVerificationFragment = new PhoneOrEmailVerificationFragment();
        Bundle bundle = new Bundle();
        phoneOrEmailVerificationFragment.setArguments(bundle);
        return phoneOrEmailVerificationFragment;
    }

    private Button btnVerify;
    private EditText etType, etOtp;
    private View vBottomsheetIcon;
    private DialogDismissListener dialogDismissListener;
    private String customerPhone, customerEmail, type;
    private int customerId;
    private Button btnSend;
    private TextView tvInfo,tvTimer;
    private LinearLayout llAfterOtpSend;

    private TextView tvResedOtp, tvTitle, tvLabel,tvSkip;
    private AlertDialog progressDialog;
    private User loggedInUser = MyApp.getInstance().getMyPreferences().getLoggedInUser();

    public static PhoneOrEmailVerificationFragment getInstance(String customerPhone, int customer_id, String email, String type) {
        PhoneOrEmailVerificationFragment phoneVerificationBottomsheet = new PhoneOrEmailVerificationFragment();
        Bundle bundle = new Bundle();
        bundle.putString("customer_phone", customerPhone);
        bundle.putInt("customer_id", customer_id);
        bundle.putString("customer_email", email);
        bundle.putString("type", type);
        phoneVerificationBottomsheet.setArguments(bundle);
        return phoneVerificationBottomsheet;
    }

    public void setDialogDismissListener(DialogDismissListener dialogDismissListener) {
        this.dialogDismissListener = dialogDismissListener;
    }


    @Nullable
    @Override
    public View onCreateView(@NonNull LayoutInflater inflater, @Nullable ViewGroup container, @Nullable Bundle savedInstanceState) {
        if (getArguments() != null) {
            customerPhone = getArguments().getString("customer_phone");
            customerId = getArguments().getInt("customer_id");
            customerEmail = getArguments().getString("customer_email");
            type = getArguments().getString("type");
        }
        return inflater.inflate(R.layout.fragment_verify_phone, container, false);
    }

    @Override
    public void onViewCreated(@NonNull View view, @Nullable Bundle savedInstanceState) {
        super.onViewCreated(view, savedInstanceState);
        initViews(view);
        setListeners();
        updateViews();
    }

    private void updateViews() {
        if (!Validators.isNullOrEmpty(type)) {
            if (type.equalsIgnoreCase("email")) {
                etType.setText(customerEmail);
                tvTitle.setText("Verify Email");
                tvLabel.setText("Email");
                tvInfo.setText("A verification code will be sent to this email");
            } else {
                tvInfo.setText("A verification code will be sent to this phone number");
                etType.setText(customerPhone);
                tvTitle.setText("Verify Phone Number");
                tvLabel.setText("Phone number");
            }
        }
    }

    private void verifyOtp() {
        AndroidNetworking.post(UPDATE_ACCOUNT + customerId + (type.equalsIgnoreCase("phone") ? "/verify-otp" : "/email-verify-otp"))
                .addBodyParameter("otp", etOtp.getText().toString())
                .build()
                .getAsJSONObject(new JSONObjectRequestListener() {
                    @Override
                    public void onResponse(JSONObject response) {
                        String message = type + " Verified";
                        ToastUtils.makeToast(getActivity(), message);
                        fetchProfile();
                    }

                    @Override
                    public void onError(ANError anError) {
                        anError.printStackTrace();
                        if (anError.getErrorCode() == 400) {
                            try {
                                JSONObject jsonObject = new JSONObject(anError.getErrorBody().toString());
                                ToastUtils.makeToast(getContext(), jsonObject.getString("message"));
                            } catch (JSONException exception) {
                                exception.printStackTrace();
                            }
                        }
                    }
                });

    }

    private void setListeners() {
        tvSkip.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
               fetchProfile();
            }
        });
        etType.addTextChangedListener(new TextWatcher() {
            @Override
            public void beforeTextChanged(CharSequence charSequence, int i, int i1, int i2) {

            }

            @Override
            public void onTextChanged(CharSequence charSequence, int i, int i1, int i2) {

                if (!customerPhone.equalsIgnoreCase(etType.getText().toString())) {
                    customerPhone = etType.getText().toString();
//                    btnSend.setEnabled(true);
//                    btnSend.setBackgroundColor(getResources().getColor(R.color.black));
                }


            }

            @Override
            public void afterTextChanged(Editable editable) {

            }
        });
        btnVerify.setOnClickListener(view -> {
            if (isValid())
                verifyOtp();
        });
        tvResedOtp.setOnClickListener(view -> {
            resendOtp();
            etOtp.setText("");
        });
        btnSend.setOnClickListener(view -> {
            verifyCustomerPhoneOrEmail(type);
        });

    }

    private void resendOtp() {

        verifyCustomerPhoneOrEmail(type);
    }

    private boolean isValid() {
        etType.setError(null);
        etOtp.setError(null);
        if (!customerPhone.equalsIgnoreCase(etType.getText().toString()) && type.equalsIgnoreCase("phone")) {
            etType.setError(null);
            etType.requestFocus();
            customerPhone = etType.getText().toString();
            ToastUtils.makeToast(getContext(), "Mobile number changed,Please send code to verify");
            btnSend.setEnabled(true);
            btnSend.setBackgroundColor(getResources().getColor(R.color.black));
            return false;
        }
        if (Validators.isNullOrEmpty(etType.getText().toString())) {
            etType.setError("Please enter phone number");
            etType.requestFocus();
            return false;
        }
        if (Validators.isNullOrEmpty(etOtp.getText().toString())) {
            etOtp.setError("Please verification code");
            etOtp.requestFocus();
            return false;
        }
        return true;
    }

    @Override
    protected void initViews(View view) {
        super.initViews(view);
        progressDialog = CommonFunctions.customProgressDialog(getActivity(), "");
        etType = view.findViewById(R.id.etPhoneNumber);
        etOtp = view.findViewById(R.id.etOtp);
        btnVerify = view.findViewById(R.id.btnVerify);
        tvResedOtp = view.findViewById(R.id.tvResendOtp);
        tvTitle = view.findViewById(R.id.tvTitile);
        tvLabel = view.findViewById(R.id.tvLabel);
        llAfterOtpSend = view.findViewById(R.id.llAfterOtpSend);
        tvInfo = view.findViewById(R.id.tvInfo);
        tvSkip = view.findViewById(R.id.tvSkip);
        btnSend = view.findViewById(R.id.btnSend);
        vBottomsheetIcon = view.findViewById(R.id.vBottomsheetIcon);

        tvTimer = view.findViewById(R.id.tvTimer);
        vBottomsheetIcon.setVisibility(View.GONE);
        tvTitle.setGravity(Gravity.CENTER);

        tvSkip.setPaintFlags(Paint.UNDERLINE_TEXT_FLAG);
        etType.setText(customerPhone);
        if (type.equalsIgnoreCase("phone")) {
            llAfterOtpSend.setVisibility(View.VISIBLE);
            int maxLength = 11;
            etType.setFilters(new InputFilter[]{new InputFilter.LengthFilter(maxLength)});
            verifyCustomerPhoneOrEmail(type);
        } else {
            etType.setEnabled(false);
        }

        view.setFocusableInTouchMode(true);
        view.requestFocus();
        view.setOnKeyListener(new View.OnKeyListener() {
            @Override
            public boolean onKey(View v, int keyCode, KeyEvent event) {
                if( keyCode == KeyEvent.KEYCODE_BACK && event.getAction() == KeyEvent.ACTION_UP) {

                    return true;
                }
                return false;
            }
        });


    }
 private void startTimer() {
     new CountDownTimer(30000, 1000) { //Set Timer for 5 seconds
         public void onTick(long millisUntilFinished) {
             int seconds = (int) (millisUntilFinished / 1000);
             int minutes = seconds / 60;
             seconds = seconds % 60;
             tvTimer.setText(String.format("%02d", minutes)
                     + ":" + String.format("%02d", seconds));

         }

         @Override
         public void onFinish() {
             btnSend.setEnabled(true);
             tvTimer.setText("00:00");
             btnSend.setBackgroundColor(Color.BLACK);
         }
     }.start();
 }
    private void verifyCustomerPhoneOrEmail(String type) {
   /*     if (getActivity() != null) {
            getActivity().runOnUiThread(() -> progressDialog.show());
        }*/
        AndroidNetworking.get(UPDATE_ACCOUNT + "send-otp")
                .build()
                .getAsJSONObject(new JSONObjectRequestListener() {
                    @Override
                    public void onResponse(JSONObject response) {

                        if (getActivity() != null) {
                            getActivity().runOnUiThread(() -> {
                                llAfterOtpSend.setVisibility(View.VISIBLE);
                                btnSend.setEnabled(false);
                                btnSend.setBackgroundColor(getResources().getColor(R.color.disabled_color));
                                if (type.equalsIgnoreCase("phone")) {
                                    tvInfo.setText("A verification code has been sent to this phone number");
                                } else {
                                    tvInfo.setText("A verification code has been sent to this email");
                                }

                            try {  String message = null;

                                message = response.getString("message");
                                ToastUtils.makeToast(getActivity(), message);
                                startTimer();

                            } catch (JSONException exception) {
                                exception.printStackTrace();
                            }

                            });
                        }

                    }

                    @Override
                    public void onError(ANError anError) {
                        anError.printStackTrace();
                      /*  if (getActivity() != null) {
                            getActivity().runOnUiThread(() -> progressDialog.hide());
                        }*/

                    }
                });

    }
    private void fetchProfile() {
        if (getActivity() != null) {
            getActivity().runOnUiThread(() -> progressDialog.show());
        }
        AndroidNetworking.get(ApiEndPoints.NEW_ACCOUNT)
                .build()
                .getAsJSONObject(new JSONObjectRequestListener() {
                    @Override
                    public void onResponse(JSONObject response) {
                        if (getActivity() != null) {
                            getActivity().runOnUiThread(() -> progressDialog.dismiss());
                        }
                        User user1 = new Gson().fromJson(response.toString(), User.class);
                        myApp.getMyPreferences().saveLoggedInUser(user1);
                        loggedInUser = myApp.getMyPreferences().getLoggedInUser();
                        if (myApp.getMyPreferences().getCurrentPostcode() == null) {
                            Bundle bundle = new Bundle();
                            bundle.putString("destination", "postcode");
                            bundle.putBoolean("hideToolbar", true);
                            bundle.putBoolean("finish", true);
                            startActivity(new Intent(getActivity(), TransportActivity.class).putExtras(bundle).setFlags(Intent.FLAG_ACTIVITY_CLEAR_TASK | Intent.FLAG_ACTIVITY_NEW_TASK | Intent.FLAG_ACTIVITY_CLEAR_TOP));
                        } else {
                            startActivity(new Intent(getActivity(), MainActivity.class).setFlags(Intent.FLAG_ACTIVITY_CLEAR_TASK | Intent.FLAG_ACTIVITY_NEW_TASK | Intent.FLAG_ACTIVITY_CLEAR_TOP));
                            getActivity().finish();
                        }
                    }

                    @Override
                    public void onError(ANError anError) {
                        anError.printStackTrace();
                        if (getActivity() != null) {
                            getActivity().runOnUiThread(() -> progressDialog.dismiss());
                        }
                        if (!CommonFunctions.isConnected(getActivity()))
                            myApp.noInternet(getActivity());

                    }
                });
    }
}
