package com.tiffintom.fragment;

import android.Manifest;
import android.app.Activity;
import android.content.DialogInterface;
import android.content.Intent;
import android.content.pm.PackageManager;
import android.database.Cursor;
import android.graphics.Bitmap;
import android.graphics.BitmapFactory;
import android.net.Uri;
import android.os.Bundle;
import android.os.Environment;
import android.provider.MediaStore;
import android.provider.Settings;
import android.text.Editable;
import android.text.TextWatcher;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.ArrayAdapter;
import android.widget.Button;
import android.widget.EditText;
import android.widget.ImageView;
import android.widget.ProgressBar;
import android.widget.RelativeLayout;
import android.widget.TextView;

import androidx.annotation.NonNull;
import androidx.annotation.Nullable;
import androidx.appcompat.app.AlertDialog;
import androidx.core.app.ActivityCompat;
import androidx.core.content.ContextCompat;
import androidx.core.content.FileProvider;

import com.androidnetworking.AndroidNetworking;
import com.androidnetworking.error.ANError;
import com.androidnetworking.interfaces.JSONObjectRequestListener;
import com.bumptech.glide.Glide;
import com.google.gson.Gson;
import com.tiffintom.BuildConfig;
import com.tiffintom.R;
import com.tiffintom.activity.NoInternetActivity;
import com.tiffintom.base.BaseFragment;
import com.tiffintom.common.CommonFunctions;
import com.tiffintom.common.FileUtil;
import com.tiffintom.common.Validators;
import com.tiffintom.models.User;
import com.tiffintom.network.ApiEndPoints;
import com.tiffintom.network.ApiUtils;
import com.tiffintom.utils.Constants;
import com.tiffintom.utils.LogUtils;
import com.tiffintom.utils.ToastUtils;

import org.json.JSONException;
import org.json.JSONObject;

import java.io.File;
import java.io.IOException;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;

import droidninja.filepicker.FilePickerConst;

import static android.Manifest.permission.WRITE_EXTERNAL_STORAGE;
import static com.tiffintom.utils.Constants.CODE_CAMERA_PICKER;
import static com.tiffintom.utils.Constants.CODE_IMAGE_PICKER;
import static com.tiffintom.utils.Constants.PERMISSION_CODE_STORAGE;

/**
 * Created by Amrish on 26-08-2020.
 */
public class ProfileEditFragment extends BaseFragment {
    private EditText etFirstName, etLastName, etEmail, etPassword, etPhone;
    private Button btnUpdate;
    private TextView tvUserNameSub, tvProfileCompletion, tvChangePassword, tvVerifyPhone, tvVerifyEmail;
    private ImageView ivUser;
    private User loggedInUser;
    private User user;
    private ProgressBar imgPdLoading;
    private RelativeLayout rlEditUser;
    private String[] requiredPermissions = {Manifest.permission.READ_EXTERNAL_STORAGE, Manifest.permission.CAMERA, WRITE_EXTERNAL_STORAGE};

    public static ProfileEditFragment getInstance() {
        ProfileEditFragment fragment = new ProfileEditFragment();
        Bundle bundle = new Bundle();
        fragment.setArguments(bundle);
        return fragment;
    }

    @Nullable
    @Override
    public View onCreateView(@NonNull LayoutInflater inflater, @Nullable ViewGroup container, @Nullable Bundle savedInstanceState) {
        return inflater.inflate(R.layout.fragment_profile_edit, container, false);
    }

    @Override
    public void onViewCreated(@NonNull View view, @Nullable Bundle savedInstanceState) {
        super.onViewCreated(view, savedInstanceState);
        updateViews();
        setListeners();
    }

    @Override
    protected void initViews(View view) {
        super.initViews(view);
        loggedInUser = myApp.getMyPreferences().getLoggedInUser();
        etFirstName = view.findViewById(R.id.etFirstName);
        etLastName = view.findViewById(R.id.etLastName);
        etEmail = view.findViewById(R.id.etEmail);
        etPassword = view.findViewById(R.id.etPassword);
        btnUpdate = view.findViewById(R.id.btnUpdate);
        etPhone = view.findViewById(R.id.etPhone);
        imgPdLoading = view.findViewById(R.id.pbPercentage);
        tvChangePassword = view.findViewById(R.id.tvChangePassword);
        tvProfileCompletion = view.findViewById(R.id.tvProfileCompletion);
        tvUserName = view.findViewById(R.id.tvUserName);
        tvUserNameSub = view.findViewById(R.id.tvUserNameSub);
        ivUser = view.findViewById(R.id.ivUserImage);
        tvVerifyPhone = view.findViewById(R.id.tvVerifyPhone);
        tvVerifyEmail = view.findViewById(R.id.tvVerifyEmail);
        rlEditUser = view.findViewById(R.id.rlUSerImage);
        etEmail.setEnabled(false);
    }

    private void updateViews() {
        if (loggedInUser != null) {
            imgPdLoading.setProgress(CommonFunctions.getProfilePercentage(loggedInUser));
            tvProfileCompletion.setText("Profile is " + CommonFunctions.getProfilePercentage(loggedInUser) + "% completed");
            tvUserNameSub.setText("Hi, " + loggedInUser.first_name + " " + loggedInUser.last_name);
            etFirstName.setText(loggedInUser.first_name);
            etLastName.setText(loggedInUser.last_name);
            etEmail.setText(loggedInUser.username);
            etPhone.setText(loggedInUser.phone_number);

            etFirstName.setSelection(etFirstName.getText().length());
            etLastName.setSelection(etLastName.getText().length());
            etPhone.setSelection(etPhone.getText().length());

            btnUpdate.setEnabled(isFilledData());
            if (!Validators.isNullOrEmpty(loggedInUser.image_url)) {
                Glide.with(getActivity())
                        .load(loggedInUser.image_url)
                        .placeholder(R.drawable.user_profile_image_placeholder)
                        .error(R.drawable.user_profile_image_placeholder)
                        .into(ivUser);
            } else {
                ivUser.setImageResource(R.drawable.user_profile_image_placeholder);
            }
            if (!Validators.isNullOrEmpty(loggedInUser.social_id)) {
                tvChangePassword.setVisibility(View.GONE);
            } else {
                tvChangePassword.setVisibility(View.VISIBLE);
            }

            if (loggedInUser.email_verify.equalsIgnoreCase("true")) {
                tvVerifyEmail.setTextColor(ContextCompat.getColor(getActivity(), R.color.green_dark));
                tvVerifyEmail.setText("Verified");
            } else {
                tvVerifyEmail.setTextColor(ContextCompat.getColor(getActivity(), R.color.red));
                tvVerifyEmail.setText("Not Verified");

            }
            if (loggedInUser.phone_verify.equalsIgnoreCase("true")) {
                tvVerifyPhone.setTextColor(ContextCompat.getColor(getActivity(), R.color.green_dark));
                tvVerifyPhone.setText("Verified");
            } else {
                tvVerifyPhone.setTextColor(ContextCompat.getColor(getActivity(), R.color.red));
                tvVerifyPhone.setText("Not Verified");
            }
        }
    }

    String type = null;

    private void setListeners() {

        tvVerifyPhone.setOnClickListener(view -> {
            if (!loggedInUser.phone_verify.equalsIgnoreCase("true")) {
                type = "phone";
                openPhoneOrEmailVerificationBottomSheet(type);
            }
        });
        tvVerifyEmail.setOnClickListener(view -> {
            if (!loggedInUser.email_verify.equalsIgnoreCase("true")) {
                type = "email";
                openPhoneOrEmailVerificationBottomSheet(type);
            }
        });

        etFirstName.addTextChangedListener(new TextWatcher() {
            @Override
            public void beforeTextChanged(CharSequence charSequence, int i, int i1, int i2) {

            }

            @Override
            public void onTextChanged(CharSequence charSequence, int i, int i1, int i2) {
                btnUpdate.setEnabled(isFilledData());

            }

            @Override
            public void afterTextChanged(Editable editable) {

            }
        });

        etLastName.addTextChangedListener(new TextWatcher() {
            @Override
            public void beforeTextChanged(CharSequence charSequence, int i, int i1, int i2) {

            }

            @Override
            public void onTextChanged(CharSequence charSequence, int i, int i1, int i2) {
                btnUpdate.setEnabled(isFilledData());
            }


            @Override
            public void afterTextChanged(Editable editable) {

            }
        });

        etPhone.addTextChangedListener(new TextWatcher() {
            @Override
            public void beforeTextChanged(CharSequence charSequence, int i, int i1, int i2) {

            }

            @Override
            public void onTextChanged(CharSequence charSequence, int i, int i1, int i2) {
                btnUpdate.setEnabled(isFilledData());
            }

            @Override
            public void afterTextChanged(Editable editable) {

            }
        });
        btnUpdate.setOnClickListener(view -> {
            if (isValid()) updateProfile();
        });
        rlEditUser.setOnClickListener(view -> {
            if (Validators.isNullOrEmpty(loggedInUser.image_url)) {
                if (checkGallaryPermission()) {
                    pickImage();
                }
            } else {
                showEditRemoveProfile();
            }
        });
        tvChangePassword.setOnClickListener(view -> {
            ChangePasswordFragment changePasswordFragment = ChangePasswordFragment.getInstance(String.valueOf(loggedInUser.id), true);
            changePasswordFragment.show(getChildFragmentManager(), "change_password");
        });
    }

    private void showEditRemoveProfile() {
        AlertDialog.Builder builderSingle = new AlertDialog.Builder(getActivity());
        String[] animals = {"Edit profile picture", "Remove profile picture"};
        final ArrayAdapter<String> arrayAdapter = new ArrayAdapter<String>(getActivity(), android.R.layout.select_dialog_singlechoice);
        arrayAdapter.add("Remove profile picture");
        arrayAdapter.add("Edit profile picture");

        builderSingle.setItems(animals, new DialogInterface.OnClickListener() {
            @Override
            public void onClick(DialogInterface dialog, int which) {
                switch (which) {
                    case 0:

                        if (checkGallaryPermission()) {
                            pickImage();
                        }
                        break;

                    case 1:
                        removeProfileImage();
                        break;

                }
            }
        });


        builderSingle.show();

    }

    private void removeProfileImage() {
        AndroidNetworking.post(ApiEndPoints.UPDATE_ACCOUNT+loggedInUser.id+"/remove-image")
                .build()
                .getAsJSONObject(new JSONObjectRequestListener() {
                    @Override
                    public void onResponse(JSONObject response) {
                            ToastUtils.makeSnackToast(getActivity(), "Profile picture removed");
                            loggedInUser.image_url = null;
                            myApp.getMyPreferences().saveLoggedInUser(loggedInUser);
                            myApp.profileChanged();
                            updateViews();

                    }

                    @Override
                    public void onError(ANError anError) {
                        anError.printStackTrace();
                    }
                });
    }


    private void openPhoneOrEmailVerificationBottomSheet(String type) {
        PhoneOrEmailVerificationBottomsheet phoneOrEmailVerificationBottomsheet = PhoneOrEmailVerificationBottomsheet.getInstance(loggedInUser.phone_number, loggedInUser.id, loggedInUser.username, type);
        phoneOrEmailVerificationBottomsheet.show(getChildFragmentManager(), "verify phone");
        phoneOrEmailVerificationBottomsheet.setDialogDismissListener(o -> fetchProfile());
    }

    private boolean isFilledData() {
        return !Validators.isNullOrEmpty(etFirstName.getText().toString()) && (!Validators.isNullOrEmpty(etLastName.getText().toString())) && (!Validators.isNullOrEmpty(etPhone.getText().toString())) && (!Validators.isNullOrEmpty(etEmail.getText().toString()) && !(etFirstName.getText().toString().equals(loggedInUser.first_name) && etLastName.getText().toString().equals(loggedInUser.last_name) && etPhone.getText().toString().equals(loggedInUser.phone_number) && etEmail.getText().toString().equals(loggedInUser.username)) || (userImageFile != null));
    }

    private boolean checkGallaryPermission() {
        ArrayList<String> deniedPermissions = new ArrayList<>();
        ArrayList<String> rationalDeniedPermissions = new ArrayList<>();

        for (String permission : requiredPermissions) {
            if (ActivityCompat.checkSelfPermission(getActivity(), permission) == PackageManager.PERMISSION_DENIED) {
                deniedPermissions.add(permission);
                if (ActivityCompat.shouldShowRequestPermissionRationale(getActivity(), permission))
                    rationalDeniedPermissions.add(permission);

            }
        }
        if (rationalDeniedPermissions.size() > 0) {
            // Show why we need permissions in dialog and on dialog yes press request permissions;

            new android.app.AlertDialog.Builder(getActivity())
                    .setTitle("Storage permission required")
                    .setMessage("To access your photo we required storage permission, Please allow to continue.")
                    .setPositiveButton("ALLOW", (dialogInterface, i) -> {
                        String[] mStringArray = new String[deniedPermissions.size()];
                        mStringArray = deniedPermissions.toArray(mStringArray);

                        requestPermissions(mStringArray, PERMISSION_CODE_STORAGE);
                    })
                    .setNegativeButton("DENY", (dialogInterface, i) -> dialogInterface.dismiss())
                    .create()
                    .show();

            return false;
        } else {

            if (deniedPermissions.size() > 0) {
                String[] mStringArray = new String[deniedPermissions.size()];
                mStringArray = deniedPermissions.toArray(mStringArray);

                requestPermissions(mStringArray, PERMISSION_CODE_STORAGE);
                return false;
            } else {
                return true;
            }
        }

    }

    private void pickImage() {
        final CharSequence[] options = {"Take Photo", "Choose from Gallery", "Cancel"};
        AlertDialog.Builder builder = new AlertDialog.Builder(getActivity());
        builder.setItems(options, new DialogInterface.OnClickListener() {
            @Override
            public void onClick(DialogInterface dialog, int item) {
                if (options[item].equals("Take Photo")) {
                    dispatchTakePictureIntent();
                } else if (options[item].equals("Choose from Gallery")) {
                    Intent intent = new Intent(Intent.ACTION_PICK, android.provider.MediaStore.Images.Media.EXTERNAL_CONTENT_URI);
                    startActivityForResult(intent, CODE_IMAGE_PICKER);
                } else if (options[item].equals("Cancel")) {
                    dialog.dismiss();
                }
            }
        });
        builder.show();
    }

    private void dispatchTakePictureIntent() {
        Intent takePictureIntent = new Intent(MediaStore.ACTION_IMAGE_CAPTURE);
        // Ensure that there's a camera activity to handle the intent
        if (takePictureIntent.resolveActivity(getActivity().getPackageManager()) != null) {
            // Create the File where the photo should go
            File photoFile = null;
            try {
                photoFile = createImageFile();
            } catch (IOException ex) {
                // Error occurred while creating the File
            }
            // Continue only if the File was successfully created
            if (photoFile != null) {
                Uri photoURI = FileProvider.getUriForFile(getActivity(), BuildConfig.APPLICATION_ID + ".fileprovider",
                        photoFile);
                takePictureIntent.putExtra(MediaStore.EXTRA_OUTPUT, photoURI);
                startActivityForResult(takePictureIntent, CODE_CAMERA_PICKER);
            }
        }
    }

    String currentPhotoPath;

    private File createImageFile() throws IOException {
        // Create an image_url file name
        String timeStamp = new SimpleDateFormat("yyyyMMdd_HHmmss").format(new Date());
        String imageFileName = "JPEG_" + timeStamp + "_";
        File storageDir = getActivity().getExternalFilesDir(Environment.DIRECTORY_PICTURES);
        File image = File.createTempFile(
                imageFileName,  /* prefix */
                ".jpg",         /* suffix */
                storageDir      /* directory */
        );

        // Save a file: path for use with ACTION_VIEW intents
        currentPhotoPath = image.getAbsolutePath();
        return image;
    }

    private boolean isValid() {
        if (Validators.isNullOrEmpty(etFirstName.getText().toString())) {
            etFirstName.setError("Please enter first name");
            etFirstName.requestFocus();
            return false;
        }
        if (Validators.isNullOrEmpty(etLastName.getText().toString())) {
            etLastName.setError("Please enter last name");
            etLastName.requestFocus();
            return false;
        }
        if (!Validators.isEmail(etEmail.getText().toString())) {
            etEmail.setError("Please enter email address");
            etEmail.requestFocus();
            return false;
        }

        if (!Validators.isValidMobile(etPhone.getText().toString())) {
            etPhone.setError("Invalid mobile number");
            etPhone.requestFocus();
            return false;
        }

        return true;
    }

    private void updateProfile() {

        if (getActivity() != null) {
            getActivity().runOnUiThread(() -> progressDialog.show());
        }
        AndroidNetworking.upload(ApiEndPoints.UPDATE_ACCOUNT+loggedInUser.id)
                .addMultipartFile("image", userImageFile)
                .addMultipartParameter("id",String.valueOf(loggedInUser.id))
                .addMultipartParameter("first_name",etFirstName.getText().toString())
                .addMultipartParameter("last_name",etLastName.getText().toString())
                .addMultipartParameter("phone_number",etPhone.getText().toString())
                .addMultipartParameter("wallet_amount",String.valueOf(loggedInUser.wallet_amount))
                .addMultipartFile("cover_image",userImageFile)
                .addMultipartParameter("wallet_amount",String.valueOf(loggedInUser.wallet_amount))
                .addMultipartParameter("device_type",String.valueOf(loggedInUser.device_type))
                .addMultipartParameter("device_id",String.valueOf(loggedInUser.device_id))
                .addMultipartParameter("order_place",String.valueOf(loggedInUser.order_place))
                .addMultipartParameter("order_accept",String.valueOf(loggedInUser.order_accept))
                .addMultipartParameter("order_reject",String.valueOf(loggedInUser.order_reject))
                .addMultipartParameter("order_delivered",String.valueOf(loggedInUser.order_delivered))
                .addMultipartParameter("book_table",String.valueOf(loggedInUser.book_table))
                .addMultipartParameter("important_update",String.valueOf(loggedInUser.important_update))
                .addMultipartParameter("email_verify",String.valueOf(loggedInUser.email_verify))
                .addMultipartParameter("phone_verify",String.valueOf(!loggedInUser.phone_number.equalsIgnoreCase(etPhone.getText().toString())? "0" :loggedInUser.phone_verify))
                .build()
                .getAsJSONObject(new JSONObjectRequestListener() {
                    @Override
                    public void onResponse(JSONObject response) {
                        LogUtils.e("Response"+response.toString());
                        fetchProfile();

                        if (getActivity() != null) {
                            getActivity().runOnUiThread(() -> progressDialog.dismiss());
                        }
                    }

                    @Override
                    public void onError(ANError anError) {
                        anError.printStackTrace();
                        LogUtils.e("Error"+anError.getErrorDetail().toString());
                        if (getActivity() != null) {
                            getActivity().runOnUiThread(() -> progressDialog.dismiss());
                        }

                        if (!CommonFunctions.isConnected(getActivity()))
                            myApp.noInternet(getActivity());
                    }
                });

    }
    private void fetchProfile() {
        if (getActivity() != null) {
            getActivity().runOnUiThread(() ->  progressDialog.show());
        }
        User user = new User();
        user.customer_id = String.valueOf(myApp.getMyPreferences().getLoggedInUser().id);

      AndroidNetworking.get(ApiEndPoints.NEW_ACCOUNT).build()
                .getAsJSONObject(new JSONObjectRequestListener() {
                    @Override
                    public void onResponse(JSONObject response) {
                        if (getActivity() != null) {
                            getActivity().runOnUiThread(() -> progressDialog.dismiss());
                        }
                        User user1 = new Gson().fromJson(response.toString(), User.class);
                        myApp.getMyPreferences().saveLoggedInUser(user1);
                        loggedInUser = myApp.getMyPreferences().getLoggedInUser();
                        myApp.profileChanged();
                        updateViews();
                    }

                    @Override
                    public void onError(ANError anError) {
                        LogUtils.e(anError.getErrorBody());

                        if (getActivity() != null) {
                            getActivity().runOnUiThread(() -> progressDialog.dismiss());
                        }
                        if (!CommonFunctions.isConnected(getActivity()))
                            startActivityForResult(new Intent(getActivity(), NoInternetActivity.class), Constants.CODE_NO_INTERNET);
                    }
                });
    }

    Uri fileUri;
    File userImageFile;
    ArrayList<Uri> files;

    @Override
    public void onActivityResult(int requestCode, int resultCode, @Nullable Intent data) {
        super.onActivityResult(requestCode, resultCode, data);
        if (requestCode == PERMISSION_CODE_STORAGE) {
            if (resultCode == Activity.RESULT_OK && data != null) {
                ArrayList<Uri> dataList = data.getParcelableArrayListExtra(FilePickerConst.KEY_SELECTED_MEDIA);
                if (dataList != null && dataList.size() > 0) {
                    files = new ArrayList<>();
                    files.addAll(dataList);
                    fileUri = files.get(0);
                    ivUser.setImageURI(fileUri);
                    userImageFile = myApp.saveBitmapToFile(new File(getPath(fileUri)));
                    btnUpdate.setEnabled(isFilledData());
                }
            }
        }
        if (requestCode == CODE_IMAGE_PICKER && resultCode == Activity.RESULT_OK) {
            if (data == null) {
                //error
                return;
            }
            try {
                Uri uri = data.getData();
                userImageFile = FileUtil.from(getActivity(), uri);
                if (userImageFile != null && userImageFile.exists())
                    updateProfile();
            } catch (IOException e) {
                e.printStackTrace();
            }
        }

        if (requestCode == CODE_CAMERA_PICKER) {
            FileUtil.refreshGallery(getActivity(), currentPhotoPath);
            Bitmap photo = BitmapFactory.decodeFile(currentPhotoPath);
            userImageFile = FileUtil.saveScaledPhotoToFile(getActivity(), photo, currentPhotoPath);
            if (userImageFile != null && userImageFile.exists())
                updateProfile();
        }

    }

    private String getPath(Uri uri) {
        String[] projection = {MediaStore.Images.Media.DATA};
        Cursor cursor = getActivity().getContentResolver().query(uri, projection, null, null, null);
        if (cursor == null) return null;
        int column_index = cursor.getColumnIndexOrThrow(MediaStore.Images.Media.DATA);
        cursor.moveToFirst();
        String s = cursor.getString(column_index);
        cursor.close();
        return s;
    }

    @Override
    public void onRequestPermissionsResult(int requestCode, @NonNull String[] permissions, @NonNull int[] grantResults) {
        if (requestCode == PERMISSION_CODE_STORAGE) {
            ArrayList<String> rationalDeniedPermissions = new ArrayList<>();
            ArrayList<String> nonRationalDeniedPermission = new ArrayList<>();
            for (String permission : permissions) {
                if (ActivityCompat.checkSelfPermission(getActivity(), permission) == PackageManager.PERMISSION_DENIED) {
                    if (ActivityCompat.shouldShowRequestPermissionRationale(getActivity(), permission))
                        rationalDeniedPermissions.add(permission);
                    else {
                        nonRationalDeniedPermission.add(permission);
                    }
                }
            }

            if (nonRationalDeniedPermission.size() == 0 && rationalDeniedPermissions.size() == 0) {
                pickImage();
            } else {
                if (nonRationalDeniedPermission.size() > 0) {
                    new android.app.AlertDialog.Builder(getActivity())
                            .setTitle("Some permissions required")
                            .setMessage("To access your photo we require storage and camera permission, Please allow to continue.")
                            .setPositiveButton("ALLOW", (dialogInterface, i) -> {
                                Intent intent = new Intent();
                                intent.setAction(Settings.ACTION_APPLICATION_DETAILS_SETTINGS);
                                Uri uri = Uri.fromParts("package", getActivity().getPackageName(), null);
                                intent.setData(uri);
                                startActivity(intent);
                            })
                            .setNegativeButton("DENY", (dialogInterface, i) -> dialogInterface.dismiss())
                            .create()
                            .show();
                } else {
                    ToastUtils.makeToast(getActivity(), "Permission denied");
                }
            }

        }
    }
}
