package com.tiffintom.fragment;

import android.Manifest;
import android.app.Activity;
import android.content.DialogInterface;
import android.content.Intent;
import android.content.pm.PackageManager;
import android.database.Cursor;
import android.graphics.Bitmap;
import android.graphics.BitmapFactory;
import android.net.Uri;
import android.os.Bundle;
import android.os.Environment;
import android.provider.MediaStore;
import android.provider.Settings;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.ArrayAdapter;
import android.widget.ImageView;
import android.widget.LinearLayout;
import android.widget.ProgressBar;
import android.widget.TextView;

import androidx.annotation.NonNull;
import androidx.annotation.Nullable;
import androidx.appcompat.app.AlertDialog;
import androidx.core.app.ActivityCompat;
import androidx.core.content.FileProvider;
import androidx.recyclerview.widget.LinearLayoutManager;
import androidx.recyclerview.widget.RecyclerView;

import com.airbnb.lottie.LottieAnimationView;
import com.androidnetworking.AndroidNetworking;
import com.androidnetworking.error.ANError;
import com.androidnetworking.interfaces.JSONObjectRequestListener;
import com.bumptech.glide.Glide;
import com.google.gson.Gson;
import com.tiffintom.BuildConfig;
import com.tiffintom.R;
import com.tiffintom.activity.AddressesActivity;
import com.tiffintom.activity.NoInternetActivity;
import com.tiffintom.activity.ReservationHistoryActivity;
import com.tiffintom.activity.SplashActivity;
import com.tiffintom.activity.TransportActivity;
import com.tiffintom.adapters.ProfileMenuAdapter;
import com.tiffintom.base.BaseFragment;
import com.tiffintom.common.CommonFunctions;
import com.tiffintom.common.FileUtil;
import com.tiffintom.common.Validators;
import com.tiffintom.models.ProfileMenu;
import com.tiffintom.models.User;
import com.tiffintom.network.ApiEndPoints;
import com.tiffintom.network.ApiUtils;
import com.tiffintom.utils.Constants;
import com.tiffintom.utils.LogUtils;
import com.tiffintom.utils.ToastUtils;

import org.json.JSONObject;

import java.io.File;
import java.io.IOException;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;

import antonkozyriatskyi.circularprogressindicator.CircularProgressIndicator;
import droidninja.filepicker.FilePickerConst;

import static android.Manifest.permission.WRITE_EXTERNAL_STORAGE;
import static com.tiffintom.utils.Constants.CODE_CAMERA_PICKER;
import static com.tiffintom.utils.Constants.CODE_IMAGE_PICKER;
import static com.tiffintom.utils.Constants.CODE_NO_INTERNET;
import static com.tiffintom.utils.Constants.PERMISSION_CODE_STORAGE;


/**
 * Created by Amrish on 19-08-2020.
 */
public class ProfileFragment extends BaseFragment {
    public static ProfileFragment getInstance() {
        ProfileFragment fragment = new ProfileFragment();
        Bundle bundle = new Bundle();
        fragment.setArguments(bundle);
        return fragment;
    }

    private TextView tvUserProfilename, tvProfileCompletion, tvEditProfile, tvCurrency, tvPoints, tvCompleteProfile, tvScreenTitle;
    private ImageView ivUserImage;
    private User loggedInUser;
    private RecyclerView rvProfile;
    private ArrayList<ProfileMenu> profileMenus = new ArrayList<>();
    private ProfileMenuAdapter adapter;
    private LinearLayout llLoading, llEditProfile;
    private ProgressBar pbLoading, imgpdloding;
    private CircularProgressIndicator circularProgressIndicator;
    private User user;
    private boolean filePicker;
    private String[] requiredPermissions = {Manifest.permission.READ_EXTERNAL_STORAGE, Manifest.permission.CAMERA, WRITE_EXTERNAL_STORAGE};
    private LottieAnimationView lodingView;

    @Nullable
    @Override
    public View onCreateView(@NonNull LayoutInflater inflater, @Nullable ViewGroup container, @Nullable Bundle savedInstanceState) {
        return inflater.inflate(R.layout.fragment_profile, container, false);
    }

    @Override
    public void onViewCreated(@NonNull View view, @Nullable Bundle savedInstanceState) {
        super.onViewCreated(view, savedInstanceState);
        setListners();
        fetchProfile();

    }


    @Override
    protected void initViews(View view) {
        super.initViews(view);
        loggedInUser = myApp.getMyPreferences().getLoggedInUser();
        llLoading = view.findViewById(R.id.llLoading);
        pbLoading = view.findViewById(R.id.pbLoading);
        tvUserName = view.findViewById(R.id.tvUserName);
        tvProfileCompletion = view.findViewById(R.id.tvProfileCompletion);
        ivUserImage = view.findViewById(R.id.ivUserImage);
        imgpdloding = view.findViewById(R.id.pbTime);
        rvProfile = view.findViewById(R.id.rvProfile);
        tvEditProfile = view.findViewById(R.id.tvEditProfile);
        tvCurrency = view.findViewById(R.id.tvCurrency);
        tvPoints = view.findViewById(R.id.tvPoints);
        tvScreenTitle = view.findViewById(R.id.tvScreenTitle);
        tvUserProfilename = view.findViewById(R.id.tvUserProfilename);
        tvCompleteProfile = view.findViewById(R.id.tvCompleteProfile);
        llEditProfile = view.findViewById(R.id.ll_EditProfile);


    }

    private void setListners() {
        adapter = new ProfileMenuAdapter(getActivity(), profileMenus, (position, data) -> {
            manageProfileClick(data);
        });
        rvProfile.setLayoutManager(new LinearLayoutManager(getActivity(), RecyclerView.VERTICAL, false));
        rvProfile.setNestedScrollingEnabled(false);
        rvProfile.setAdapter(adapter);


        tvCompleteProfile.setOnClickListener(view1 -> startActivity(new Intent(getActivity(), TransportActivity.class).putExtra("destination", "profile_edit").putExtra("name", "Profile")));

        tvScreenTitle.setVisibility(View.VISIBLE);
        tvScreenTitle.setText("Profile");

        llEditProfile.setOnClickListener(view -> {
            if (Validators.isNullOrEmpty(loggedInUser.image_url)) {
                filePicker = true;
                if (checkGallaryPermission()) {
                    pickImage();
                }
            } else {
                showEditRemoveProfile();
            }

        });
    }

    private boolean checkGallaryPermission() {
        ArrayList<String> deniedPermissions = new ArrayList<>();
        ArrayList<String> rationalDeniedPermissions = new ArrayList<>();

        for (String permission : requiredPermissions) {
            if (ActivityCompat.checkSelfPermission(getActivity(), permission) == PackageManager.PERMISSION_DENIED) {
                deniedPermissions.add(permission);
                if (ActivityCompat.shouldShowRequestPermissionRationale(getActivity(), permission))
                    rationalDeniedPermissions.add(permission);

            }
        }
        if (rationalDeniedPermissions.size() > 0) {
            // Show why we need permissions in dialog and on dialog yes press request permissions;

            new android.app.AlertDialog.Builder(getActivity())
                    .setTitle("Storage permission required")
                    .setMessage("To access your photo we required storage permission, Please allow to continue.")
                    .setPositiveButton("ALLOW", (dialogInterface, i) -> {
                        String[] mStringArray = new String[deniedPermissions.size()];
                        mStringArray = deniedPermissions.toArray(mStringArray);

                        requestPermissions(mStringArray, PERMISSION_CODE_STORAGE);
                    })
                    .setNegativeButton("DENY", (dialogInterface, i) -> dialogInterface.dismiss())
                    .create()
                    .show();

            return false;
        } else {

            if (deniedPermissions.size() > 0) {
                String[] mStringArray = new String[deniedPermissions.size()];
                mStringArray = deniedPermissions.toArray(mStringArray);

                requestPermissions(mStringArray, PERMISSION_CODE_STORAGE);
                return false;
            } else {
                return true;
            }
        }

    }

    private void showEditRemoveProfile() {
        AlertDialog.Builder builderSingle = new AlertDialog.Builder(getActivity());
        String[] animals = {"Edit profile picture", "Remove profile picture"};
        final ArrayAdapter<String> arrayAdapter = new ArrayAdapter<String>(getActivity(), android.R.layout.select_dialog_singlechoice);
        arrayAdapter.add("Remove profile picture");
        arrayAdapter.add("Edit profile picture");

        builderSingle.setItems(animals, new DialogInterface.OnClickListener() {
            @Override
            public void onClick(DialogInterface dialog, int which) {
                switch (which) {
                    case 0:
                        filePicker = true;
                        if (checkGallaryPermission()) {
                            pickImage();
                        }
                        break;

                    case 1:
                        removeProfileImage();
                        break;

                }
            }
        });

        builderSingle.show();

    }

    private void removeProfileImage() {
        AndroidNetworking.post(ApiEndPoints.UPDATE_ACCOUNT+String.valueOf(loggedInUser.id)+"/remove-image")
                .build()
                .getAsJSONObject(new JSONObjectRequestListener() {
                    @Override
                    public void onResponse(JSONObject response) {
                           ToastUtils.makeSnackToast(getActivity(), "Profile picture removed");
                            loggedInUser.image_url = null;
                            myApp.getMyPreferences().saveLoggedInUser(loggedInUser);
                            myApp.profileChanged();
                            updateViews();

                    }

                    @Override
                    public void onError(ANError anError) {
                        anError.printStackTrace();
                        ToastUtils.makeToast(getContext(),anError.getErrorBody());
                    }
                });
    }

    private void updateProfile() {
        if (getActivity() != null) {
            getActivity().runOnUiThread(() -> progressDialog.show());
        }

        user = new User();
        user.first_name = loggedInUser.first_name;
        user.last_name = loggedInUser.last_name;
        user.phone = loggedInUser.phone_number;
        user.action = "MyAccount";
        user.page = "ProfileUpdate";
        user.customer_id = String.valueOf(loggedInUser.id);
        AndroidNetworking.upload(ApiEndPoints.UPDATE_ACCOUNT+loggedInUser.id)
                .addMultipartFile("image", userImageFile)
                .addMultipartParameter("id",String.valueOf(loggedInUser.id))
                .addMultipartParameter("first_name",String.valueOf(loggedInUser.first_name))
                .addMultipartParameter("last_name",String.valueOf(loggedInUser.last_name))
                .addMultipartParameter("phone_number",String.valueOf(loggedInUser.phone_number))
                .addMultipartParameter("wallet_amount",String.valueOf(loggedInUser.wallet_amount))
                .addMultipartFile("cover_image",userImageFile)
                .addMultipartParameter("wallet_amount",String.valueOf(loggedInUser.wallet_amount))
                .addMultipartParameter("device_type",String.valueOf(loggedInUser.device_type))
                .addMultipartParameter("device_id",String.valueOf(loggedInUser.device_id))
                .addMultipartParameter("order_place",String.valueOf(loggedInUser.order_place))
                .addMultipartParameter("order_accept",String.valueOf(loggedInUser.order_accept))
                .addMultipartParameter("order_reject",String.valueOf(loggedInUser.order_reject))
                .addMultipartParameter("order_delivered",String.valueOf(loggedInUser.order_delivered))
                .addMultipartParameter("book_table",String.valueOf(loggedInUser.book_table))
                .addMultipartParameter("important_update",String.valueOf(loggedInUser.important_update))
                .addMultipartParameter("email_verify",String.valueOf(loggedInUser.email_verify))
                .addMultipartParameter("phone_verify",String.valueOf(loggedInUser.phone_verify))
                .build()
                .getAsJSONObject(new JSONObjectRequestListener() {
                    @Override
                    public void onResponse(JSONObject response) {
                        filePicker = false;
                            if (!Validators.isNullOrEmpty(user.image_url)) {
                                ToastUtils.makeSnackToast(getActivity(), "Profile picture added");
                            } else {
                                ToastUtils.makeSnackToast(getActivity(), "Profile picture updated");
                            }
                            fetchProfile();

                        if (getActivity() != null) {
                            getActivity().runOnUiThread(() -> progressDialog.dismiss());
                        }
                    }

                    @Override
                    public void onError(ANError anError) {
                        anError.printStackTrace();
                        LogUtils.e(anError.getErrorBody());
                        if (getActivity() != null) {
                            getActivity().runOnUiThread(() -> progressDialog.dismiss());
                        }

                        if (!CommonFunctions.isConnected(getActivity()))
                            myApp.noInternet(getActivity());
                    }
                });

    }

    private void pickImage() {
        final CharSequence[] options = {"Take Photo", "Choose from Gallery", "Cancel"};
        AlertDialog.Builder builder = new AlertDialog.Builder(getActivity());
        builder.setItems(options, new DialogInterface.OnClickListener() {
            @Override
            public void onClick(DialogInterface dialog, int item) {
                if (options[item].equals("Take Photo")) {
                    dispatchTakePictureIntent();
                } else if (options[item].equals("Choose from Gallery")) {
                    Intent intent = new Intent(Intent.ACTION_PICK, android.provider.MediaStore.Images.Media.EXTERNAL_CONTENT_URI);
                    startActivityForResult(intent, CODE_IMAGE_PICKER);
                } else if (options[item].equals("Cancel")) {
                    dialog.dismiss();
                }
            }
        });
        builder.show();
    }

    private void dispatchTakePictureIntent() {
        Intent takePictureIntent = new Intent(MediaStore.ACTION_IMAGE_CAPTURE);
        // Ensure that there's a camera activity to handle the intent
        if (takePictureIntent.resolveActivity(getActivity().getPackageManager()) != null) {
            // Create the File where the photo should go
            File photoFile = null;
            try {
                photoFile = createImageFile();
            } catch (IOException ex) {
                // Error occurred while creating the File
            }
            // Continue only if the File was successfully created
            if (photoFile != null) {
                Uri photoURI = FileProvider.getUriForFile(getActivity(), BuildConfig.APPLICATION_ID + ".fileprovider",
                        photoFile);
                takePictureIntent.putExtra(MediaStore.EXTRA_OUTPUT, photoURI);
                startActivityForResult(takePictureIntent, CODE_CAMERA_PICKER);
            }
        }
    }

    String currentPhotoPath;

    private File createImageFile() throws IOException {
        // Create an image_url file name
        String timeStamp = new SimpleDateFormat("yyyyMMdd_HHmmss").format(new Date());
        String imageFileName = "JPEG_" + timeStamp + "_";
        File storageDir = getActivity().getExternalFilesDir(Environment.DIRECTORY_PICTURES);
        File image = File.createTempFile(
                imageFileName,  /* prefix */
                ".jpg",         /* suffix */
                storageDir      /* directory */
        );

        // Save a file: path for use with ACTION_VIEW intents
        currentPhotoPath = image.getAbsolutePath();
        return image;
    }


    private void manageProfileClick(Object data) {
        ProfileMenu profileMenu = (ProfileMenu) data;
        if (profileMenu.title.equalsIgnoreCase("reservation history")) {
            startActivity(new Intent(getActivity(), ReservationHistoryActivity.class));
        }
        if (profileMenu.title.equalsIgnoreCase("address book")) {
            startActivity(new Intent(getActivity(), AddressesActivity.class));
        }
        if (profileMenu.title.equalsIgnoreCase("my details")) {
            startActivity(new Intent(getActivity(), TransportActivity.class).putExtra("destination", "profile_edit").putExtra("name", "My Details"));
        }
        if (profileMenu.title.equalsIgnoreCase("wallet")) {
            startActivity(new Intent(getActivity(), TransportActivity.class).putExtra("destination", "wallet").putExtra("name", "Wallet"));
        }
        if (profileMenu.title.equalsIgnoreCase("rewards")) {
            startActivity(new Intent(getActivity(), TransportActivity.class).putExtra("destination", "rewards").putExtra("name", "Rewards"));
        }
        if (profileMenu.title.equalsIgnoreCase("tiffintom credit")) {
            startActivity(new Intent(getActivity(), TransportActivity.class).putExtra("destination", "credits").putExtra("name", "Tiffintom Credit"));
        }
        if (profileMenu.title.equalsIgnoreCase("my favourites")) {
            startActivity(new Intent(getActivity(), TransportActivity.class).putExtra("destination", "favourites").putExtra("name", "Your Favourites"));
        }
        if (profileMenu.title.equalsIgnoreCase("order history")) {
            startActivity(new Intent(getActivity(), TransportActivity.class).putExtra("destination", "orders").putExtra("name", "Order History").putExtra("hideToolbar", true));
        }
        if (profileMenu.title.equalsIgnoreCase("dine in history")) {
            startActivity(new Intent(getActivity(), TransportActivity.class).putExtra("destination", "dinein_history").putExtra("name", "Dine in History"));
        }
    }

    private void updateViews() {
        if (getActivity() != null) {
            getActivity().runOnUiThread(() -> llLoading.setVisibility(View.GONE));
        }
        if (loggedInUser != null) {
            imgpdloding.setProgress(CommonFunctions.getProfilePercentage(loggedInUser));
            tvUserProfilename.setText("Hi, " + loggedInUser.first_name + " " + loggedInUser.last_name);
            tvProfileCompletion.setText("Profile is " + CommonFunctions.getProfilePercentage(loggedInUser) + "% completed");
            tvCurrency.setText(myApp.getCurrencySymbol() + loggedInUser.wallet_amount);
            tvPoints.setText(loggedInUser.total_earn_point + "");

            if (CommonFunctions.getProfilePercentage(loggedInUser) == 100) {
                tvCompleteProfile.setVisibility(View.GONE);
            }

            if (!Validators.isNullOrEmpty(loggedInUser.image_url)) {
                if (getActivity() != null) {
                    Glide.with(getActivity())
                            .load(loggedInUser.image_url)
                            .placeholder(R.drawable.user_profile_image_placeholder)
                            .error(R.drawable.user_profile_image_placeholder)
                            .into(ivUserImage);
                }
            } else {
                ivUserImage.setImageResource(R.drawable.user_profile_image_placeholder);
            }
        }
        setUpMenu();
    }

    private void setUpMenu() {
        profileMenus.clear();
        profileMenus.add(new ProfileMenu("My Details", R.drawable.icon_profile));
        profileMenus.add(new ProfileMenu("My Favourites", R.drawable.icon_favourite));
        profileMenus.add(new ProfileMenu("Order History", R.drawable.icon_order_history));
        profileMenus.add(new ProfileMenu("Dine in History", R.drawable.icon_dinein_history));
        profileMenus.add(new ProfileMenu("Reservation History", R.drawable.icon_reservation));
        profileMenus.add(new ProfileMenu("Address Book", R.drawable.icon_address_book));
        if (myApp.getMyPreferences().getReferral() != null && myApp.getMyPreferences().getReferral().referral_option.equalsIgnoreCase("yes"))
            profileMenus.add(new ProfileMenu("Rewards", R.drawable.icon_rewards));
        profileMenus.add(new ProfileMenu("Wallet", R.drawable.icon_wallet));
        profileMenus.add(new ProfileMenu("Tiffintom Credit", R.drawable.tiffin_outline));

        adapter.notifyDataSetChanged();
    }

    private void fetchProfile() {
        if (getActivity() != null) {
            getActivity().runOnUiThread(() -> llLoading.setVisibility(View.VISIBLE));
        }
        AndroidNetworking.get(ApiEndPoints.NEW_ACCOUNT)
                .build()
                .getAsJSONObject(new JSONObjectRequestListener() {
                    @Override
                    public void onResponse(JSONObject response) {
                        if (getActivity() != null) {
                            getActivity().runOnUiThread(() -> llLoading.setVisibility(View.GONE));
                        }
                                User user1 = new Gson().fromJson(response.toString(), User.class);
                                myApp.getMyPreferences().saveLoggedInUser(user1);
                                loggedInUser = myApp.getMyPreferences().getLoggedInUser();
                                myApp.profileChanged();
                                updateViews();
                    }

                    @Override
                    public void onError(ANError anError) {
                        LogUtils.e(anError.getErrorBody());
                        if (anError.getErrorCode()==401)
                        {
                            myApp.logout(getActivity());
                        }
                        if (getActivity() != null) {
                            getActivity().runOnUiThread(() -> llLoading.setVisibility(View.GONE));
                        }
                        if (!CommonFunctions.isConnected(getActivity()))
                            startActivityForResult(new Intent(getActivity(), NoInternetActivity.class), Constants.CODE_NO_INTERNET);
                    }
                });
    }

    Uri fileUri;
    File userImageFile;

    ArrayList<Uri> files;

    @Override
    public void onActivityResult(int requestCode, int resultCode, @Nullable Intent data) {
        super.onActivityResult(requestCode, resultCode, data);
        if (requestCode == PERMISSION_CODE_STORAGE) {
            if (resultCode == Activity.RESULT_OK && data != null) {
                ArrayList<Uri> dataList = data.getParcelableArrayListExtra(FilePickerConst.KEY_SELECTED_MEDIA);
                if (dataList != null && dataList.size() > 0) {
                    files = new ArrayList<>();
                    files.addAll(dataList);
                    fileUri = files.get(0);
                    userImageFile = myApp.saveBitmapToFile(new File(getPath(fileUri)));
                    ivUserImage.setImageURI(fileUri);
                    updateProfile();
                }
            }
        }
        if (requestCode == CODE_IMAGE_PICKER && resultCode == Activity.RESULT_OK) {
            if (data == null) {
                //error
                return;
            }
            try {
                Uri uri = data.getData();
                userImageFile = FileUtil.from(getActivity(), uri);
                if (userImageFile != null && userImageFile.exists())
                    updateProfile();
            } catch (IOException e) {
                e.printStackTrace();
            }
        }

        if (requestCode == CODE_CAMERA_PICKER) {
            FileUtil.refreshGallery(getActivity(), currentPhotoPath);
            Bitmap photo = BitmapFactory.decodeFile(currentPhotoPath);
            userImageFile = FileUtil.saveScaledPhotoToFile(getActivity(), photo, currentPhotoPath);
            if (userImageFile != null && userImageFile.exists())
                updateProfile();
        }

        if (requestCode == CODE_NO_INTERNET) {
            fetchProfile();
        }

    }

    public String getPath(Uri uri) {
        String[] projection = {MediaStore.Images.Media.DATA};
        Cursor cursor = getActivity().getContentResolver().query(uri, projection, null, null, null);
        if (cursor == null) return null;
        int column_index = cursor.getColumnIndexOrThrow(MediaStore.Images.Media.DATA);
        cursor.moveToFirst();
        String s = cursor.getString(column_index);
        cursor.close();
        return s;
    }

    @Override
    public void onResume() {
        super.onResume();
        if (!filePicker)
            fetchProfile();
    }

    @Override
    public void onRequestPermissionsResult(int requestCode, @NonNull String[] permissions, @NonNull int[] grantResults) {
        if (requestCode == PERMISSION_CODE_STORAGE) {
            ArrayList<String> rationalDeniedPermissions = new ArrayList<>();
            ArrayList<String> nonRationalDeniedPermission = new ArrayList<>();
            for (String permission : permissions) {
                if (ActivityCompat.checkSelfPermission(getActivity(), permission) == PackageManager.PERMISSION_DENIED) {
                    if (ActivityCompat.shouldShowRequestPermissionRationale(getActivity(), permission))
                        rationalDeniedPermissions.add(permission);
                    else {
                        nonRationalDeniedPermission.add(permission);
                    }
                }
            }

            if (nonRationalDeniedPermission.size() == 0 && rationalDeniedPermissions.size() == 0) {
                pickImage();
            } else {
                if (nonRationalDeniedPermission.size() > 0) {
                    new android.app.AlertDialog.Builder(getActivity())
                            .setTitle("Some permissions required")
                            .setMessage("To access your photo we require storage and camera permission, Please allow to continue.")
                            .setPositiveButton("ALLOW", (dialogInterface, i) -> {
                                Intent intent = new Intent();
                                intent.setAction(Settings.ACTION_APPLICATION_DETAILS_SETTINGS);
                                Uri uri = Uri.fromParts("package", getActivity().getPackageName(), null);
                                intent.setData(uri);
                                startActivity(intent);
                            })
                            .setNegativeButton("DENY", (dialogInterface, i) -> dialogInterface.dismiss())
                            .create()
                            .show();
                } else {
                    ToastUtils.makeToast(getActivity(), "Permission denied");
                }
            }

        }
    }
}
