package com.tiffintom.fragment;

import android.app.Dialog;
import android.content.DialogInterface;
import android.content.Intent;
import android.os.Build;
import android.os.Bundle;
import android.text.Editable;
import android.text.TextWatcher;
import android.view.KeyEvent;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.EditText;
import android.widget.FrameLayout;
import android.widget.ProgressBar;
import android.widget.TextView;

import androidx.annotation.NonNull;
import androidx.annotation.Nullable;
import androidx.recyclerview.widget.LinearLayoutManager;
import androidx.recyclerview.widget.RecyclerView;

import com.airbnb.lottie.LottieAnimationView;
import com.androidnetworking.AndroidNetworking;
import com.androidnetworking.error.ANError;
import com.androidnetworking.interfaces.JSONObjectRequestListener;
import com.google.android.material.bottomsheet.BottomSheetBehavior;
import com.google.android.material.bottomsheet.BottomSheetDialog;
import com.google.android.material.bottomsheet.BottomSheetDialogFragment;
import com.google.gson.Gson;
import com.google.gson.reflect.TypeToken;
import com.tiffintom.MyApp;
import com.tiffintom.R;
import com.tiffintom.activity.TransportActivity;
import com.tiffintom.adapters.RecentOrdersAdapter;
import com.tiffintom.common.CommonFunctions;
import com.tiffintom.models.OrderHistory;
import com.tiffintom.models.User;
import com.tiffintom.network.ApiEndPoints;
import com.tiffintom.utils.LogUtils;
import com.tiffintom.utils.ToastUtils;

import org.json.JSONException;
import org.json.JSONObject;

import java.lang.reflect.Type;
import java.util.ArrayList;
import java.util.List;

/**
 * Created by Amrish on 18-09-2020.
 */
public class RecentOrdersBottomsheetFragment extends BottomSheetDialogFragment {
    public static RecentOrdersBottomsheetFragment getInstance() {
        return new RecentOrdersBottomsheetFragment();
    }

    private User loggedInUser = MyApp.getInstance().getMyPreferences().getLoggedInUser();
    private MyApp myApp = MyApp.getInstance();
    private RecyclerView rvOrders;
    private RecentOrdersAdapter orderHistoryAdapter;
    private ArrayList<OrderHistory> orders = new ArrayList<>();
    private ArrayList<OrderHistory> searchedOrders = new ArrayList<>();
    private EditText etSearch;
    private ProgressBar pbLoading;
    private TextView tvRecentOrders;
    private LottieAnimationView lodingView;

    @Override
    public int getTheme() {
        if (Build.VERSION.SDK_INT > 21) {
            return R.style.BottomSheetDialogTheme;
        } else {
            return super.getTheme();
        }
    }

    @NonNull
    @Override
    public Dialog onCreateDialog(Bundle savedInstanceState) {

        BottomSheetDialog dialog = (BottomSheetDialog) super.onCreateDialog(savedInstanceState);

        dialog.setOnShowListener((DialogInterface.OnShowListener) dialog1 -> {
            BottomSheetDialog d = (BottomSheetDialog) dialog1;
            FrameLayout bottomSheet = (FrameLayout) d.findViewById(com.google.android.material.R.id.design_bottom_sheet);
            if (bottomSheet != null)
                BottomSheetBehavior.from(bottomSheet).setState(BottomSheetBehavior.STATE_EXPANDED);
        });

        return dialog;
    }

    @Nullable
    @Override
    public View onCreateView(@NonNull LayoutInflater inflater, @Nullable ViewGroup container, @Nullable Bundle savedInstanceState) {
        return inflater.inflate(R.layout.fragment_recent_orders, container, false);

    }

    @Override
    public void onViewCreated(@NonNull View view, @Nullable Bundle savedInstanceState) {
        super.onViewCreated(view, savedInstanceState);
        initViews(view);
        fetchOrders();
        setListeners();
    }

    private void setListeners() {
        etSearch.setOnKeyListener((textView, i, keyEvent) -> {
            if (keyEvent.getKeyCode() == KeyEvent.KEYCODE_BACK) {
                getActivity().onBackPressed();
            } else if ((keyEvent.getKeyCode() == KeyEvent.KEYCODE_ENTER)) {
                searchedOrders.clear();
                searchedOrders.addAll(searchOrder());
                orderHistoryAdapter.notifyDataSetChanged();
                if (getActivity() != null)
                    CommonFunctions.hideKeyboard(getActivity(), etSearch);
            }
            return false;
        });
        etSearch.addTextChangedListener(new TextWatcher() {
            @Override
            public void beforeTextChanged(CharSequence charSequence, int i, int i1, int i2) {

            }

            @Override
            public void onTextChanged(CharSequence charSequence, int i, int i1, int i2) {

            }

            @Override
            public void afterTextChanged(Editable editable) {
                if (editable.toString().length() == 0) {
                    searchedOrders.clear();
                    searchedOrders.addAll(searchOrder());
                    orderHistoryAdapter.notifyDataSetChanged();
                }
            }
        });
    }

    private void initViews(View view) {
        etSearch = view.findViewById(R.id.etSearch);
        rvOrders = view.findViewById(R.id.rvRecentOrders);
        tvRecentOrders = view.findViewById(R.id.tvRecentOrders);
        pbLoading = view.findViewById(R.id.pbLoading);
        lodingView = view.findViewById(R.id.lodingView);
        orderHistoryAdapter = new RecentOrdersAdapter(searchedOrders, (position, data) -> {
            if (data instanceof OrderHistory) {
                startActivity(new Intent(getActivity(), TransportActivity.class).putExtra("destination", "restaurant_chat").putExtra("name", "Chat").putExtra("order_history", new Gson().toJson(data)).putExtra("hideToolbar", true));
                dismiss();
            }
        });
        rvOrders.setAdapter(orderHistoryAdapter);
        rvOrders.setLayoutManager(new LinearLayoutManager(getActivity(), RecyclerView.VERTICAL, false));
    }

    private void fetchOrders() {
        if (getActivity() != null)
            getActivity().runOnUiThread(() -> {
                lodingView.setVisibility(View.VISIBLE);
            });
        AndroidNetworking.get(ApiEndPoints.NEW_ORDERS)
                .addQueryParameter("customer_id", String.valueOf(loggedInUser.id))
                .build()
                .getAsJSONObject(new JSONObjectRequestListener() {
                    @Override
                    public void onResponse(JSONObject response) {
                        if (getActivity() != null)
                            getActivity().runOnUiThread(() -> {
                                lodingView.setVisibility(View.GONE);
                            });
                             Type type = new TypeToken<List<OrderHistory>>() {
                            }.getType();
                            searchedOrders.clear();
                            try {
                                searchedOrders.addAll(new Gson().fromJson(response.getJSONArray("data").toString(), type));
                                orders.clear();
                                orders.addAll(searchedOrders);
                            } catch (JSONException e) {
                                e.printStackTrace();
                                LogUtils.e(e.getLocalizedMessage());
                            }
                            orderHistoryAdapter.notifyDataSetChanged();


                        if (getActivity() != null) getActivity().runOnUiThread(() -> {
                            if (orders.size() == 0)
                                tvRecentOrders.setVisibility(View.GONE);
                            else tvRecentOrders.setVisibility(View.VISIBLE);
                        });
                    }

                    @Override
                    public void onError(ANError anError) {
                        anError.printStackTrace();
                        if (getActivity() != null)
                            getActivity().runOnUiThread(() -> {
                                lodingView.setVisibility(View.GONE);
                            });
                        LogUtils.e("ORDER LIST ERROR");

                        if (!CommonFunctions.isConnected(getActivity()))
                            myApp.noInternet(getActivity());
                    }
                });
    }

    private ArrayList<OrderHistory> searchOrder() {
        if (etSearch.getText().toString().isEmpty()) {
            return orders;
        }
        ArrayList<OrderHistory> searchedRest = new ArrayList<>();
        for (OrderHistory r : orders) {
            if (r.restaurant.restaurant_name != null && r.restaurant.restaurant_name.toLowerCase().contains(etSearch.getText().toString().toLowerCase())) {
                searchedRest.add(r);
            }
        }
        return searchedRest;

    }

    @Override
    public void onDismiss(@NonNull DialogInterface dialog) {
        AndroidNetworking.cancel("recent_orders");
        super.onDismiss(dialog);
    }
}
