package com.tiffintom.fragment;

import android.os.Bundle;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.LinearLayout;

import androidx.annotation.NonNull;
import androidx.annotation.Nullable;
import androidx.recyclerview.widget.LinearLayoutManager;
import androidx.recyclerview.widget.RecyclerView;
import androidx.swiperefreshlayout.widget.SwipeRefreshLayout;

import com.androidnetworking.AndroidNetworking;
import com.androidnetworking.error.ANError;
import com.androidnetworking.interfaces.JSONArrayRequestListener;
import com.androidnetworking.interfaces.JSONObjectRequestListener;
import com.google.gson.Gson;
import com.google.gson.reflect.TypeToken;
import com.tiffintom.R;
import com.tiffintom.adapters.FaqAdapter;
import com.tiffintom.adapters.ReferredAdapter;
import com.tiffintom.base.BaseFragment;
import com.tiffintom.models.ReferredList;
import com.tiffintom.models.User;
import com.tiffintom.network.ApiEndPoints;
import com.tiffintom.network.ApiUtils;
import com.tiffintom.utils.LogUtils;

import org.json.JSONArray;
import org.json.JSONException;
import org.json.JSONObject;

import java.lang.reflect.Type;
import java.util.ArrayList;
import java.util.List;

public class ReferralHistoryFragment extends BaseFragment {


    public static ReferralHistoryFragment getInstance() {
        return new ReferralHistoryFragment();
    }

    private RecyclerView rvReferHistory;
    private ArrayList<ReferredList> referredLists = new ArrayList<>();
    private FaqAdapter adapter;
    private RecyclerView rvHelp;
    private SwipeRefreshLayout swipeRefreshLayout;
    private User loggedInUser;
    private ReferredAdapter referredAdapter;
    private LinearLayout llNoData;

    @Nullable
    @Override
    public View onCreateView(@NonNull LayoutInflater inflater, @Nullable ViewGroup container, @Nullable Bundle savedInstanceState) {

        return inflater.inflate(R.layout.fragment_referral_history, container, false);

    }

    @Override
    public void onViewCreated(@NonNull View view, @Nullable Bundle savedInstanceState) {
        super.onViewCreated(view, savedInstanceState);

        getData();
        swipeRefreshLayout.setOnRefreshListener(this::getData);
        swipeRefreshLayout.setEnabled(false);
    }


    private void getData() {
        new Thread(this::fetchRefers).start();

    }

    private void fetchRefers() {
        if (getActivity() != null) {
            getActivity().runOnUiThread(() -> {
                progressDialog.show();
                llNoData.setVisibility(View.GONE);
            });
        }
        AndroidNetworking.get(ApiEndPoints.NEW_REFERRAL_HISTORY + "nopaginate=1").build()
                .getAsJSONArray(new JSONArrayRequestListener() {
                    @Override
                    public void onResponse(JSONArray response) {
                        if (getActivity() != null) {
                            getActivity().runOnUiThread(() -> {
                                progressDialog.dismiss();
                            });
                        }
                        Type type = new TypeToken<List<ReferredList>>() {
                        }.getType();
                        referredLists.clear();
                        referredLists.addAll(new Gson().fromJson(response.toString(), type));
                        referredAdapter.notifyDataSetChanged();


                    }

                    @Override
                    public void onError(ANError anError) {
                        if (getActivity() != null) {
                            if (getActivity() != null) {
                                getActivity().runOnUiThread(() -> {
                                    progressDialog.dismiss();
                                    llNoData.setVisibility(View.VISIBLE);
                                });
                            }
                        }
                    }
                });
    }


    protected void initViews(View view) {
        super.initViews(view);
        llNoData = view.findViewById(R.id.llNoData);
        loggedInUser = myApp.getMyPreferences().getLoggedInUser();
        rvHelp = view.findViewById(R.id.rvHelp);
        swipeRefreshLayout = view.findViewById(R.id.swipeRefreshLayout);
        rvReferHistory = view.findViewById(R.id.rvReferHistory);

        referredAdapter = new ReferredAdapter(getActivity(), referredLists);
        rvReferHistory.setLayoutManager(new LinearLayoutManager(getActivity(), RecyclerView.VERTICAL, false));
        rvReferHistory.setAdapter(referredAdapter);

    }

}
