package com.tiffintom.fragment;

import android.content.BroadcastReceiver;
import android.content.Context;
import android.content.Intent;
import android.content.IntentFilter;
import android.os.Bundle;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.LinearLayout;
import android.widget.ProgressBar;
import android.widget.TextView;

import androidx.annotation.NonNull;
import androidx.annotation.Nullable;
import androidx.localbroadcastmanager.content.LocalBroadcastManager;
import androidx.recyclerview.widget.LinearLayoutManager;
import androidx.recyclerview.widget.RecyclerView;

import com.airbnb.lottie.LottieAnimationView;
import com.androidnetworking.AndroidNetworking;
import com.androidnetworking.error.ANError;
import com.androidnetworking.interfaces.JSONObjectRequestListener;
import com.google.gson.Gson;
import com.google.gson.reflect.TypeToken;
import com.tiffintom.R;
import com.tiffintom.activity.AuthActivity;
import com.tiffintom.adapters.ReservationsAdapter;
import com.tiffintom.base.BaseFragment;
import com.tiffintom.common.CommonFunctions;
import com.tiffintom.interfaces.DialogDismissListener;
import com.tiffintom.models.Reservation;
import com.tiffintom.models.User;
import com.tiffintom.network.ApiEndPoints;
import com.tiffintom.network.ApiUtils;
import com.tiffintom.notification.Config;
import com.tiffintom.utils.ToastUtils;

import org.json.JSONException;
import org.json.JSONObject;

import java.lang.reflect.Type;
import java.util.ArrayList;
import java.util.List;

/**
 * Created by Amrish on 19-08-2020.
 */
public class ReservationsPreviousFragment extends BaseFragment {
    public static ReservationsPreviousFragment getInstance(String status) {
        ReservationsPreviousFragment exploreCategoryFragment = new ReservationsPreviousFragment();
        Bundle bundle = new Bundle();
        bundle.putString("delivery_status", status);
        exploreCategoryFragment.setArguments(bundle);
        return exploreCategoryFragment;
    }

    private RecyclerView rvReservations;
    private ReservationsAdapter reservationsAdapter;
    private ArrayList<Reservation> reservations = new ArrayList<>();
    private ArrayList<Reservation> main_reservations = new ArrayList<>();
    private String reservationStatus;
    private User loggedInUser;
    private TextView tvMessage;
    private ProgressBar pbLoading;
    private LinearLayout llNoData;
    private LottieAnimationView lodingView;
    private BroadcastReceiver broadcastReceiver;
    private Reservation selectedReservation;
    @Nullable
    @Override
    public View onCreateView(@NonNull LayoutInflater inflater, @Nullable ViewGroup container, @Nullable Bundle savedInstanceState) {
        return inflater.inflate(R.layout.fragment_reservations, container, false);
    }

    @Override
    public void onViewCreated(@NonNull View view, @Nullable Bundle savedInstanceState) {
        if (getArguments() != null) {
            reservationStatus = getArguments().getString("delivery_status");
        }
        super.onViewCreated(view, savedInstanceState);
        loggedInUser = myApp.getMyPreferences().getLoggedInUser();
        fetchReservations();
        broadcastReceiver = new BroadcastReceiver() {
            public void onReceive(Context context, Intent intent) {
                boolean check_update = intent.getBooleanExtra("refresh", false);
                if (check_update) {
                    fetchReservations();
                }
            }
        };
    }

    @Override
    protected void initViews(View view) {
        super.initViews(view);
        pbLoading = view.findViewById(R.id.pbLoading);
        lodingView = view.findViewById(R.id.lodingView);

        llNoData = view.findViewById(R.id.llNoData);
        tvMessage = view.findViewById(R.id.tvMessage);
        rvReservations = view.findViewById(R.id.rvReservations);
        reservationsAdapter = new ReservationsAdapter(getActivity(), reservations, this::reservationClick, this::showCancellationDialog);
        rvReservations.setLayoutManager(new LinearLayoutManager(getActivity(), RecyclerView.VERTICAL, false));
        rvReservations.setAdapter(reservationsAdapter);
    }

    private String bookingId, reason;
    private int position;

    private void showCancellationDialog(int i, Object o) {
        position = i;
        bookingId = ((Reservation) o).booking_id;
        selectedReservation=(Reservation)o;
        CancellationReasonFragment cancellationReasonFragment = CancellationReasonFragment.getInstance();
        cancellationReasonFragment.setDialogDismissListener(reservationCancelReasonDialogDismissListener);
        cancellationReasonFragment.show(getChildFragmentManager(), "cancel_reason");
    }

    private void cancelReservation() {
        progressDialog.show();
        AndroidNetworking.post(ApiEndPoints.NEW_BOOKING  + selectedReservation.id)
                .addBodyParameter("customer_id", String.valueOf(loggedInUser.id))
                .addBodyParameter("restaurant_id", String.valueOf(selectedReservation.restaurant_id))
                .addBodyParameter("booking_id", selectedReservation.booking_id)
                .addBodyParameter("customer_name", selectedReservation.customer_name)
                .addBodyParameter("guest_count", selectedReservation.guest_count)
                .addBodyParameter("booking_email", selectedReservation.booking_email)
                .addBodyParameter("booking_phone", selectedReservation.booking_phone)
                .addBodyParameter("booking_insturction", selectedReservation.booking_instruction)
                .addBodyParameter("booking_date", selectedReservation.booking_date)
                .addBodyParameter("booking_time", selectedReservation.booking_time)
                .addBodyParameter("type", "Android")
                .addBodyParameter("status", "Cancel")
                .addBodyParameter("reason", reason).build()
       .getAsJSONObject(new JSONObjectRequestListener() {
                    @Override
                    public void onResponse(JSONObject response) {
                        ToastUtils.makeToast(getActivity(), "Booking cancelled successfully");
                        try {
                            reservations.remove(position);
                            progressDialog.dismiss();
                            reservationsAdapter.notifyItemRemoved(position);
                            reservationsAdapter.notifyDataSetChanged();
                            reason = null;
                            bookingId = null;
                            position = -1;
                            if (reservations.size() == 0) {
                                getActivity().runOnUiThread(() -> llNoData.setVisibility(View.VISIBLE));
                            } else {
                                getActivity().runOnUiThread(() -> llNoData.setVisibility(View.GONE));
                            }
                        }catch (IndexOutOfBoundsException e)
                        {
                            e.printStackTrace();
                        }

                    }

                    @Override
                    public void onError(ANError anError) {
                        progressDialog.dismiss();
                        reservations.clear();
                        progressDialog.dismiss();
                        if (reservations.size() == 0) {
                            getActivity().runOnUiThread(() -> llNoData.setVisibility(View.VISIBLE));
                        } else {
                            getActivity().runOnUiThread(() -> llNoData.setVisibility(View.GONE));
                        }
                        anError.printStackTrace();
                        if (!CommonFunctions.isConnected(getActivity()))
                            myApp.noInternet(getActivity());
                    }
                });
    }

    private void fetchReservations() {
        if (getActivity() != null) {
            getActivity().runOnUiThread(() -> {
                lodingView.setVisibility(View.VISIBLE);
                llNoData.setVisibility(View.GONE);
            });
        }
        ApiUtils.getPreviousReservations(String.valueOf(loggedInUser.id))
                .getAsJSONObject(new JSONObjectRequestListener() {
                    @Override
                    public void onResponse(JSONObject response) {
                        Type type = new TypeToken<List<Reservation>>() {
                        }.getType();
                        main_reservations.clear();
                        try {
                            main_reservations.addAll(new Gson().fromJson(response.getJSONArray("data").toString(), type));
                        } catch (JSONException exception) {
                            exception.printStackTrace();
                        }
                        filterList();

                        if (getActivity() != null) {
                            getActivity().runOnUiThread(() -> {
                                if (reservations.size() == 0) {
                                    llNoData.setVisibility(View.VISIBLE);
                                    tvMessage.setText("Seems like you don't have any reservations yet.");
                                } else {
                                    llNoData.setVisibility(View.GONE);
                                }
                                lodingView.setVisibility(View.GONE);
                            });

                        }
                    }

                    @Override
                    public void onError(ANError anError) {
                        if (getActivity() != null) {
                            getActivity().runOnUiThread(() -> {
                                lodingView.setVisibility(View.GONE);
                                llNoData.setVisibility(View.VISIBLE);
                                tvMessage.setText("Seems like you don't have any reservations yet.");
                            });
                        }
                        anError.printStackTrace();
                        if (!CommonFunctions.isConnected(getActivity()))
                            myApp.noInternet(getActivity());
                    }
                });
    }

    private DialogDismissListener reservationCancelReasonDialogDismissListener = o -> {
        reason = (String) o;
        cancelReservation();
    };

    private void filterList() {
        reservations.clear();
        for (Reservation reservation : main_reservations) {
            if (reservationStatus.equalsIgnoreCase("upcoming")) {
                if (reservation.status.equalsIgnoreCase("pending")) {
                    reservations.add(reservation);
                }
            } else if (reservationStatus.equalsIgnoreCase("previous")) {
                if (!reservation.status.equalsIgnoreCase("pending")) {
                    reservations.add(reservation);
                }
            }
        }
        reservationsAdapter.notifyDataSetChanged();
        if (reservations.size() == 0) {
            llNoData.setVisibility(View.VISIBLE);
            tvMessage.setText("Seems like you don't have any reservations yet.");
        } else {
            llNoData.setVisibility(View.GONE);
        }
    }

    private void reservationClick(int position, Object data) {
        if (data instanceof Reservation) {
            if (loggedInUser == null) {
                startActivity(new Intent(getActivity(), AuthActivity.class));
            } else {
                MakeReservationBottomSheetFragment makeReservationBottomSheetFragment = MakeReservationBottomSheetFragment.getInstance((((Reservation) data).restaurant_id));
                makeReservationBottomSheetFragment.show(getChildFragmentManager(), "make_reservation");
                makeReservationBottomSheetFragment.setDialogDismissListener(reservationBookListener);
            }
        }
    }

    private DialogDismissListener reservationBookListener = o -> fetchReservations();

    @Override
    public void onResume() {
        super.onResume();
        if (getActivity() != null && broadcastReceiver != null)
            LocalBroadcastManager.getInstance(getActivity()).registerReceiver(broadcastReceiver, new IntentFilter(Config.PUSH_NOTIFICATION));
    }

    @Override
    public void onStop() {
        if (getActivity() != null && broadcastReceiver != null)
            LocalBroadcastManager.getInstance(getActivity()).unregisterReceiver(broadcastReceiver);
        super.onStop();
    }
}
