package com.tiffintom.fragment;

import android.graphics.Color;
import android.graphics.drawable.ColorDrawable;
import android.os.Bundle;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.TextView;

import androidx.annotation.NonNull;
import androidx.annotation.Nullable;
import androidx.fragment.app.DialogFragment;

import com.tiffintom.R;
import com.tiffintom.common.Validators;
import com.tiffintom.interfaces.DialogDismissListener;

public class RestaurantClosedDialogFragment extends DialogFragment {

    private TextView tvInfo, tvConfirm;
    private String closedInfoText;
    private DialogDismissListener dialogDismissListener;

    public static RestaurantClosedDialogFragment getInstance(String tvResCloedInfo) {
        RestaurantClosedDialogFragment closedDialogFragment = new RestaurantClosedDialogFragment();
        Bundle bundle = new Bundle();
        bundle.putString("message", tvResCloedInfo);
        closedDialogFragment.setArguments(bundle);
        return closedDialogFragment;
    }

    public void setDialogDismissListener(DialogDismissListener dialogDismissListener) {
        this.dialogDismissListener = dialogDismissListener;
    }

    @Nullable
    @Override
    public View onCreateView(@NonNull LayoutInflater inflater, @Nullable ViewGroup container, @Nullable Bundle savedInstanceState) {
        if (getArguments() != null) {
            closedInfoText = getArguments().getString("message");
        }

        View dialogView = inflater.inflate(R.layout.dialog_restaurant_closed, container, false);
        getDialog().getWindow().setBackgroundDrawable(new ColorDrawable(Color.TRANSPARENT));

        return dialogView;
    }

    @Override
    public void onViewCreated(@NonNull View view, @Nullable Bundle savedInstanceState) {
        super.onViewCreated(view, savedInstanceState);
        initView(view);
        setListners();
    }

    private void setListners() {
        if (!Validators.isNullOrEmpty(closedInfoText)) {
            tvInfo.setText(closedInfoText);
        }

        tvConfirm.setOnClickListener(view -> {
            dismiss();
            if (dialogDismissListener != null) dialogDismissListener.onDialogDismiss(null);
        });
    }

    private void initView(View view) {
        tvInfo = view.findViewById(R.id.tvInfo);
        tvConfirm = view.findViewById(R.id.tvOkey);

    }

}
