package com.tiffintom.fragment;

import android.app.Activity;
import android.content.Intent;
import android.net.Uri;
import android.os.Bundle;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.webkit.WebView;
import android.widget.ImageView;
import android.widget.LinearLayout;
import android.widget.TextView;

import androidx.annotation.NonNull;
import androidx.annotation.Nullable;
import androidx.core.content.ContextCompat;
import androidx.recyclerview.widget.RecyclerView;

import com.airbnb.lottie.LottieAnimationView;
import com.androidnetworking.error.ANError;
import com.androidnetworking.interfaces.JSONObjectRequestListener;
import com.bumptech.glide.Glide;
import com.google.android.gms.maps.CameraUpdateFactory;
import com.google.android.gms.maps.GoogleMap;
import com.google.android.gms.maps.MapsInitializer;
import com.google.android.gms.maps.OnMapReadyCallback;
import com.google.android.gms.maps.model.LatLng;
import com.google.android.gms.maps.model.MarkerOptions;
import com.google.gson.Gson;
import com.tiffintom.MyApp;
import com.tiffintom.R;
import com.tiffintom.activity.AuthActivity;
import com.tiffintom.activity.RestaurantDetailsActivity;
import com.tiffintom.adapters.ExploreNearByRestaurantAdapter;
import com.tiffintom.base.BaseFragment;
import com.tiffintom.common.AppMapView;
import com.tiffintom.common.CommonFunctions;
import com.tiffintom.common.Validators;
import com.tiffintom.models.Restaurant;
import com.tiffintom.models.RestaurantTiming;
import com.tiffintom.models.User;
import com.tiffintom.network.ApiUtils;
import com.tiffintom.utils.LogUtils;
import com.tiffintom.utils.ToastUtils;

import org.json.JSONObject;

import java.util.ArrayList;

import static com.tiffintom.common.CommonFunctions.df;

/**
 * Created by Amrish on 19-08-2020.
 */
public class RestaurantInfoFragment extends BaseFragment implements OnMapReadyCallback {
    public static RestaurantInfoFragment getInstance() {
        RestaurantInfoFragment fragment = new RestaurantInfoFragment();
        Bundle bundle = new Bundle();
        fragment.setArguments(bundle);
        return fragment;
    }

    private Restaurant restaurant = RestaurantDetailsActivity.openedRestaurant;
    private RecyclerView rvRestaurant;
    private ExploreNearByRestaurantAdapter restaurantAdapter;
    private final ArrayList<Restaurant> categories = new ArrayList<>();
    private TextView tvMinOrder, tvDistance, tvDeliveryFee, tvInfo, tvCall, tvAddress, tvMondayTime, tvTuesdayTime, tvWedTime, tvThursdayTime, tvFridayTime, tvSatTime, tvSundayTime, tvPickMondayTime, tvPickTuesdayTime, tvPickWedTime, tvPickThursdayTime, tvPickFridayTime, tvPickSatTime, tvPickSundayTime;
    private ImageView ivRestaurantLogo, ivFavouriteImage;
    private LottieAnimationView ivFavourite;
    private TextView tvMondayTimeLabel, tvTuesdayTimeLabel, tvWedTimeLabel, tvThursdayTimeLabel, tvFridayTimeLabel, tvSatTimeLabel, tvSundayTimeLabel, tvPickMondayTimeLabel, tvPickTuesdayTimeLabel, tvPickWedTimeLabel, tvPickThursdayTimeLabel, tvPickFridayTimeLabel, tvPickSatTimeLabel, tvPickSundayTimeLabel;
    private LinearLayout llCall, llMonday, llTuesday, llWed, llThursday, llFriday, llSaturday, llSunday, llPickupMonday, llPickupTuesday, llPickupWed, llPickupThursday, llPickupFriday, llPickupSunday, llPickupSaturday;
    private LinearLayout llMinOrder, llDistance, llDeliveryFee, llPickupTimes, llDeliveryTimes, llPickupTimesTab, llDeliveryTimesTab, llAboutus;
    private View vDelivery, vPickup;
    private LinearLayout cvTimes;
    private final User loggedInUser = myApp.getMyPreferences().getLoggedInUser();
    private WebView webView;
    private TextView tvRatingHeader, tvRatingInfo;
    private AppMapView mapView;

    @Nullable
    @Override
    public View onCreateView(@NonNull LayoutInflater inflater, @Nullable ViewGroup container, @Nullable Bundle savedInstanceState) {
        return inflater.inflate(R.layout.fragment_restaurant_info, container, false);
    }

    @Override
    public void onViewCreated(@NonNull View view, @Nullable Bundle savedInstanceState) {
        super.onViewCreated(view, savedInstanceState);
        try {
            mapView.onCreate(savedInstanceState);
            MapsInitializer.initialize(getActivity());
            updateViews();
            setListener();
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    @Override
    protected void initViews(View view) {
        super.initViews(view);
        tvMinOrder = view.findViewById(R.id.tvMinOrder);
        tvDistance = view.findViewById(R.id.tvDistance);
        tvDeliveryFee = view.findViewById(R.id.tvDeliveryFee);
        ivRestaurantLogo = view.findViewById(R.id.ivRestaurantLogo);
        ivFavouriteImage = view.findViewById(R.id.ivFavouriteImage);
        ivFavourite = view.findViewById(R.id.ivFavourite);
        tvAddress = view.findViewById(R.id.tvAddress);
        tvCall = view.findViewById(R.id.tvContact);
        tvInfo = view.findViewById(R.id.tvInfo);

        llAboutus = view.findViewById(R.id.llAboutus);
        llDistance = view.findViewById(R.id.llDistance);
        llDeliveryFee = view.findViewById(R.id.llDeliveryFee);
        llMinOrder = view.findViewById(R.id.llMinOrder);
        llDeliveryTimes = view.findViewById(R.id.llOpeningTimes);
        llPickupTimes = view.findViewById(R.id.llPickupTimes);
        llDeliveryTimesTab = view.findViewById(R.id.llOpeningTimesTab);
        llPickupTimesTab = view.findViewById(R.id.llPickupTimesTab);
        vDelivery = view.findViewById(R.id.vDelivery);
        llCall = view.findViewById(R.id.llCall);
        vPickup = view.findViewById(R.id.vPickup);
        cvTimes = view.findViewById(R.id.cvTimes);
        tvMondayTime = view.findViewById(R.id.tvMondayTime);
        tvTuesdayTime = view.findViewById(R.id.tvTuesdayTime);
        tvWedTime = view.findViewById(R.id.tvWednesdayTime);
        tvThursdayTime = view.findViewById(R.id.tvThursdayTime);
        tvFridayTime = view.findViewById(R.id.tvFridayTime);
        tvSatTime = view.findViewById(R.id.tvSaturdayTime);
        tvSundayTime = view.findViewById(R.id.tvSundayTime);

        tvPickMondayTime = view.findViewById(R.id.tvPickupMondayTime);
        tvPickTuesdayTime = view.findViewById(R.id.tvPickupTuesdayTime);
        tvPickWedTime = view.findViewById(R.id.tvPickupWednesdayTime);
        tvPickThursdayTime = view.findViewById(R.id.tvPickupThursdayTime);
        tvPickFridayTime = view.findViewById(R.id.tvPickupFridayTime);
        tvPickSatTime = view.findViewById(R.id.tvPickupSaturdayTime);
        tvPickSundayTime = view.findViewById(R.id.tvPickupSundayTime);

        tvMondayTimeLabel = view.findViewById(R.id.tvMondayTimeLabel);
        tvTuesdayTimeLabel = view.findViewById(R.id.tvTuesdayTimeLabel);
        tvWedTimeLabel = view.findViewById(R.id.tvWednesdayTimeLabel);
        tvThursdayTimeLabel = view.findViewById(R.id.tvThursdayTimeLabel);
        tvFridayTimeLabel = view.findViewById(R.id.tvFridayTimeLabel);
        tvSatTimeLabel = view.findViewById(R.id.tvSaturdayTimeLabel);
        tvSundayTimeLabel = view.findViewById(R.id.tvSundayTimeLabel);

        tvPickMondayTimeLabel = view.findViewById(R.id.tvPickupMondayTimeLabel);
        tvPickTuesdayTimeLabel = view.findViewById(R.id.tvPickupTuesdayTimeLabel);
        tvPickWedTimeLabel = view.findViewById(R.id.tvPickupWednesdayTimeLabel);
        tvPickThursdayTimeLabel = view.findViewById(R.id.tvPickupThursdayTimeLabel);
        tvPickFridayTimeLabel = view.findViewById(R.id.tvPickupFridayTimeLabel);
        tvPickSatTimeLabel = view.findViewById(R.id.tvPickupSaturdayTimeLabel);
        tvPickSundayTimeLabel = view.findViewById(R.id.tvPickupSundayTimeLabel);


        llMonday = view.findViewById(R.id.llMonday);
        llTuesday = view.findViewById(R.id.llTuesday);
        llWed = view.findViewById(R.id.llWed);
        llThursday = view.findViewById(R.id.llThursday);
        llFriday = view.findViewById(R.id.llFriday);
        llSaturday = view.findViewById(R.id.llSaturday);
        llSunday = view.findViewById(R.id.llSunday);

        llPickupMonday = view.findViewById(R.id.llPickupMonday);
        llPickupTuesday = view.findViewById(R.id.llPickupTuesday);
        llPickupWed = view.findViewById(R.id.llPickupWed);
        llPickupThursday = view.findViewById(R.id.llPickupThursday);
        llPickupFriday = view.findViewById(R.id.llPickupFriday);
        llPickupSaturday = view.findViewById(R.id.llPickupSaturday);
        llPickupSunday = view.findViewById(R.id.llPickupSunday);

        webView = view.findViewById(R.id.webView);
        tvRatingHeader = view.findViewById(R.id.tvRatingHeader);
        tvRatingInfo = view.findViewById(R.id.tvRatingInfo);

        mapView = view.findViewById(R.id.map);
        mapView.getMapAsync(this);
    }

    private void updateViews() {
        try {
            if (restaurant != null) {
                try {
                    llDeliveryFee.setVisibility(View.VISIBLE);
                    if (restaurant.deliveryCharges > 0) {
                        tvDeliveryFee.setText(CommonFunctions.StringAppender(myApp.getCurrencySymbol(), df.format(restaurant.deliveryCharges), " delivery fee"));
                    } else {
                        tvDeliveryFee.setText("Free Delivery");
                    }
                    if (restaurant.delivery_charge != null && restaurant.delivery_charge.equalsIgnoreCase("yes")) {
                        llDeliveryFee.setVisibility(View.VISIBLE);
                    } else {
                        llDeliveryFee.setVisibility(View.GONE);
                    }
                } catch (NumberFormatException e) {
                    llDeliveryFee.setVisibility(View.GONE);
                }
                llDeliveryFee.setVisibility(View.GONE);

                if (!Validators.isNullOrEmpty(restaurant.food_rating)) {
                    String html = restaurant.food_rating.replace("\r\n", "");
                    html = "<div style='text-align:center'>" + html + "</div>";
                    String mime = "text/html";
                    String encoding = "utf-8";
                    tvRatingHeader.setVisibility(View.VISIBLE);
                    tvRatingInfo.setVisibility(View.VISIBLE);
                    webView.getSettings().setJavaScriptEnabled(true);
                    webView.loadDataWithBaseURL(null, html, mime, encoding, null);
                    webView.setVisibility(View.VISIBLE);
                } else {
                    tvRatingHeader.setVisibility(View.GONE);
                    tvRatingInfo.setVisibility(View.GONE);
                    webView.setVisibility(View.GONE);
                }

                tvMinOrder.setText(CommonFunctions.StringAppender(restaurant.currency, String.valueOf(restaurant.minimum_order), " min cost"));
                tvDistance.setText(CommonFunctions.StringAppender(String.valueOf(restaurant.distance), " miles"));

                Glide.with(getActivity())
                        .load(restaurant.image_url)
                        .placeholder(R.drawable.restaurant_logo_placeholder)
                        .error(R.drawable.restaurant_logo_placeholder)
                        .into(ivRestaurantLogo);
                if (!Validators.isNullOrEmpty(restaurant.restaurant_about)) {
                    llAboutus.setVisibility(View.VISIBLE);
                    tvInfo.setText(restaurant.restaurant_about);
                } else {
                    llAboutus.setVisibility(View.GONE);
                }

                tvCall.setText(restaurant.contact_phone);
                tvAddress.setText(restaurant.contact_address);

                /*
                 * Time logic
                 * */
                vDelivery.setVisibility(View.INVISIBLE);
                llDeliveryTimes.setVisibility(View.GONE);

                vPickup.setVisibility(View.INVISIBLE);
                llPickupTimes.setVisibility(View.GONE);

                StringBuilder monday = new StringBuilder(), tuesday = new StringBuilder(), wednesday = new StringBuilder(), thursday = new StringBuilder(), friday = new StringBuilder(), saturday = new StringBuilder(), sunday = new StringBuilder();
                int closeCount = 0;
                if (restaurant.restaurant_delivery.equalsIgnoreCase("Yes")) {
                    if (!Validators.isNullOrEmpty(restaurant.monday_first_opentime)) {
                        if (!restaurant.monday_first_opentime.equalsIgnoreCase(restaurant.monday_first_closetime)) {
                            monday.append(restaurant.monday_first_opentime);
                            monday.append("-");
                            monday.append(restaurant.monday_first_closetime);
                        }

                        if (!Validators.isNullOrEmpty(restaurant.monday_second_opentime)) {
                            if (!restaurant.monday_second_opentime.equalsIgnoreCase(restaurant.monday_second_closetime)) {
                                if (!monday.toString().isEmpty()) {
                                    monday.append(", ");
                                }
                                monday.append(restaurant.monday_second_opentime);
                                monday.append("-");
                                monday.append(restaurant.monday_second_closetime);

                            }
                        }
                    }
                    if (monday.toString().isEmpty()) {
                        closeCount++;
                        tvMondayTime.setText("CLOSED");
                        llMonday.setVisibility(View.GONE);
                    } else {
                        closeCount--;
                        llMonday.setVisibility(View.VISIBLE);
                        if ((!Validators.isNullOrEmpty(restaurant.monday_status) && restaurant.monday_status.equalsIgnoreCase("close"))) {
                            tvMondayTime.setText("CLOSED");
                        } else
                            tvMondayTime.setText(monday.toString());
                    }

                    if (!Validators.isNullOrEmpty(restaurant.tuesday_first_opentime)) {
                        if (!restaurant.tuesday_first_opentime.equalsIgnoreCase(restaurant.tuesday_first_closetime)) {
                            tuesday.append(restaurant.tuesday_first_opentime);
                            tuesday.append("-");
                            tuesday.append(restaurant.tuesday_first_closetime);
                        }

                        if (!Validators.isNullOrEmpty(restaurant.tuesday_second_opentime)) {
                            if (!restaurant.tuesday_second_opentime.equalsIgnoreCase(restaurant.tuesday_second_closetime)) {
                                if (!tuesday.toString().isEmpty()) {
                                    tuesday.append(", ");
                                }
                                tuesday.append(restaurant.tuesday_second_opentime);
                                tuesday.append("-");
                                tuesday.append(restaurant.tuesday_second_closetime);

                            }
                        }
                    }
                    if (tuesday.toString().isEmpty()) {
                        closeCount++;
                        tvTuesdayTime.setText("CLOSED");
                        llTuesday.setVisibility(View.GONE);
                    } else {
                        closeCount--;
                        llTuesday.setVisibility(View.VISIBLE);
                        if ((!Validators.isNullOrEmpty(restaurant.tuesday_status) && restaurant.tuesday_status.equalsIgnoreCase("close"))) {
                            tvTuesdayTime.setText("CLOSED");
                        } else
                            tvTuesdayTime.setText(tuesday.toString());
                    }

                    if (!Validators.isNullOrEmpty(restaurant.wednesday_first_opentime)) {
                        if (!restaurant.wednesday_first_opentime.equalsIgnoreCase(restaurant.wednesday_first_closetime)) {
                            wednesday.append(restaurant.wednesday_first_opentime);
                            wednesday.append("-");
                            wednesday.append(restaurant.wednesday_first_closetime);
                        }

                        if (!Validators.isNullOrEmpty(restaurant.wednesday_second_opentime)) {
                            if (!restaurant.wednesday_second_opentime.equalsIgnoreCase(restaurant.wednesday_second_closetime)) {
                                if (!wednesday.toString().isEmpty()) {
                                    wednesday.append(", ");
                                }
                                wednesday.append(restaurant.wednesday_second_opentime);
                                wednesday.append("-");
                                wednesday.append(restaurant.wednesday_second_closetime);

                            }
                        }
                    }
                    if (wednesday.toString().isEmpty()) {
                        closeCount++;
                        tvWedTime.setText("CLOSED");
                        llWed.setVisibility(View.GONE);
                    } else {
                        closeCount--;
                        llWed.setVisibility(View.VISIBLE);
                        if ((!Validators.isNullOrEmpty(restaurant.wednesday_status) && restaurant.wednesday_status.equalsIgnoreCase("close"))) {
                            tvWedTime.setText("CLOSED");
                        } else
                            tvWedTime.setText(wednesday.toString());
                    }

                    if (!Validators.isNullOrEmpty(restaurant.thursday_first_opentime)) {
                        if (!restaurant.thursday_first_opentime.equalsIgnoreCase(restaurant.thursday_first_closetime)) {
                            thursday.append(restaurant.thursday_first_opentime);
                            thursday.append("-");
                            thursday.append(restaurant.thursday_first_closetime);
                        }

                        if (!Validators.isNullOrEmpty(restaurant.thursday_second_opentime)) {
                            if (!restaurant.thursday_second_opentime.equalsIgnoreCase(restaurant.thursday_second_closetime)) {
                                if (!thursday.toString().isEmpty()) {
                                    thursday.append(", ");
                                }
                                thursday.append(restaurant.thursday_second_opentime);
                                thursday.append("-");
                                thursday.append(restaurant.thursday_second_closetime);

                            }
                        }
                    }
                    if (thursday.toString().isEmpty()) {
                        closeCount++;
                        tvThursdayTime.setText("CLOSED");
                        llThursday.setVisibility(View.GONE);
                    } else {
                        closeCount--;
                        llThursday.setVisibility(View.VISIBLE);
                        if ((!Validators.isNullOrEmpty(restaurant.thursday_status) && restaurant.thursday_status.equalsIgnoreCase("close"))) {
                            tvThursdayTime.setText("CLOSED");
                        } else
                            tvThursdayTime.setText(thursday.toString());
                    }

                    if (!Validators.isNullOrEmpty(restaurant.friday_first_opentime)) {
                        if (!restaurant.friday_first_opentime.equalsIgnoreCase(restaurant.friday_first_closetime)) {
                            friday.append(restaurant.friday_first_opentime);
                            friday.append("-");
                            friday.append(restaurant.friday_first_closetime);
                        }

                        if (!Validators.isNullOrEmpty(restaurant.friday_second_opentime)) {
                            if (!restaurant.friday_second_opentime.equalsIgnoreCase(restaurant.friday_second_closetime)) {
                                if (!friday.toString().isEmpty()) {
                                    friday.append(", ");
                                }
                                friday.append(restaurant.friday_second_opentime);
                                friday.append("-");
                                friday.append(restaurant.friday_second_closetime);

                            }
                        }
                    }
                    if (friday.toString().isEmpty()) {
                        closeCount++;
                        tvFridayTime.setText("CLOSED");
                        llFriday.setVisibility(View.GONE);
                    } else {
                        closeCount--;
                        llFriday.setVisibility(View.VISIBLE);
                        if ((!Validators.isNullOrEmpty(restaurant.friday_status) && restaurant.friday_status.equalsIgnoreCase("close"))) {
                            tvFridayTime.setText("CLOSED");
                        } else
                            tvFridayTime.setText(friday.toString());
                    }

                    if (!Validators.isNullOrEmpty(restaurant.saturday_first_opentime)) {
                        if (!restaurant.saturday_first_opentime.equalsIgnoreCase(restaurant.saturday_first_closetime)) {
                            saturday.append(restaurant.saturday_first_opentime);
                            saturday.append("-");
                            saturday.append(restaurant.saturday_first_closetime);
                        }

                        if (!Validators.isNullOrEmpty(restaurant.saturday_second_opentime)) {
                            if (!restaurant.saturday_second_opentime.equalsIgnoreCase(restaurant.saturday_second_closetime)) {
                                if (!saturday.toString().isEmpty()) {
                                    saturday.append(", ");
                                }
                                saturday.append(restaurant.saturday_second_opentime);
                                saturday.append("-");
                                saturday.append(restaurant.saturday_second_closetime);

                            }
                        }
                    }
                    if (saturday.toString().isEmpty()) {
                        closeCount++;
                        tvSatTime.setText("CLOSED");
                        llSaturday.setVisibility(View.GONE);
                    } else {
                        closeCount--;
                        llSaturday.setVisibility(View.VISIBLE);
                        if ((!Validators.isNullOrEmpty(restaurant.saturday_status) && restaurant.saturday_status.equalsIgnoreCase("close"))) {
                            tvSatTime.setText("CLOSED");
                        } else
                            tvSatTime.setText(saturday.toString());
                    }

                    if (!Validators.isNullOrEmpty(restaurant.sunday_first_opentime)) {
                        if (!restaurant.sunday_first_opentime.equalsIgnoreCase(restaurant.sunday_first_closetime)) {
                            sunday.append(restaurant.sunday_first_opentime);
                            sunday.append("-");
                            sunday.append(restaurant.sunday_first_closetime);
                        }

                        if (!Validators.isNullOrEmpty(restaurant.sunday_second_opentime)) {
                            if (!restaurant.sunday_second_opentime.equalsIgnoreCase(restaurant.sunday_second_closetime)) {
                                if (!sunday.toString().isEmpty()) {
                                    sunday.append(", ");
                                }
                                sunday.append(restaurant.sunday_second_opentime);
                                sunday.append("-");
                                sunday.append(restaurant.sunday_second_closetime);

                            }
                        }
                    }
                    if (sunday.toString().isEmpty()) {
                        closeCount++;
                        tvSundayTime.setText("CLOSED");
                        llSunday.setVisibility(View.GONE);
                    } else {
                        closeCount--;
                        llSunday.setVisibility(View.VISIBLE);
                        if ((!Validators.isNullOrEmpty(restaurant.sunday_status) && restaurant.sunday_status.equalsIgnoreCase("close"))) {
                            tvSundayTime.setText("CLOSED");
                        } else
                            tvSundayTime.setText(sunday.toString());
                    }
                } else {
                    closeCount = 7;
                }


                if (closeCount >= 7) {
                    llDeliveryTimes.setVisibility(View.GONE);
                    llDeliveryTimesTab.setVisibility(View.GONE);
                } else {
                    llDeliveryTimes.setVisibility(View.VISIBLE);
                    llDeliveryTimesTab.setVisibility(View.VISIBLE);
                    vDelivery.setVisibility(View.VISIBLE);
                }
                int pickupCloseCount = 0;
                if (restaurant.restaurant_timing != null) {
                    RestaurantTiming restaurantTiming = restaurant.restaurant_timing;

                    StringBuilder pick_monday = new StringBuilder(), pick_tuesday = new StringBuilder(), pick_wednesday = new StringBuilder(), pick_thursday = new StringBuilder(), pick_friday = new StringBuilder(), pick_saturday = new StringBuilder(), pick_sunday = new StringBuilder();
                    if (!Validators.isNullOrEmpty(restaurantTiming.pick_monday_first_opentime)) {
                        if (!restaurantTiming.pick_monday_first_opentime.equalsIgnoreCase(restaurantTiming.pick_monday_first_closetime)) {
                            pick_monday.append(restaurantTiming.pick_monday_first_opentime);
                            pick_monday.append("-");
                            pick_monday.append(restaurantTiming.pick_monday_first_closetime);
                        }

                        if (!Validators.isNullOrEmpty(restaurantTiming.pick_monday_second_opentime)) {
                            if (!restaurantTiming.pick_monday_second_opentime.equalsIgnoreCase(restaurantTiming.pick_monday_second_closetime)) {
                                if (!pick_monday.toString().isEmpty()) {
                                    pick_monday.append(", ");
                                }
                                pick_monday.append(restaurantTiming.pick_monday_second_opentime);
                                pick_monday.append("-");
                                pick_monday.append(restaurantTiming.pick_monday_second_closetime);

                            }
                        }
                    }
                    if (pick_monday.toString().isEmpty()) {
                        pickupCloseCount++;
                        tvPickMondayTime.setText("CLOSED");
                        llPickupMonday.setVisibility(View.GONE);
                    } else {
                        pickupCloseCount--;
                        llPickupMonday.setVisibility(View.VISIBLE);
                        if ((!Validators.isNullOrEmpty(restaurantTiming.pick_monday_status) && restaurantTiming.pick_monday_status.equalsIgnoreCase("close"))) {
                            tvPickMondayTime.setText("CLOSED");
                        } else {
                            tvPickMondayTime.setText(pick_monday.toString());
                        }
                    }

                    if (!Validators.isNullOrEmpty(restaurantTiming.pick_tuesday_first_opentime)) {
                        if (!restaurantTiming.pick_tuesday_first_opentime.equalsIgnoreCase(restaurantTiming.pick_tuesday_first_closetime)) {
                            pick_tuesday.append(restaurantTiming.pick_tuesday_first_opentime);
                            pick_tuesday.append("-");
                            pick_tuesday.append(restaurantTiming.pick_tuesday_first_closetime);
                        }

                        if (!Validators.isNullOrEmpty(restaurantTiming.pick_tuesday_second_opentime)) {
                            if (!restaurantTiming.pick_tuesday_second_opentime.equalsIgnoreCase(restaurantTiming.pick_tuesday_second_closetime)) {
                                if (!pick_tuesday.toString().isEmpty()) {
                                    pick_tuesday.append(", ");
                                }
                                pick_tuesday.append(restaurantTiming.pick_tuesday_second_opentime);
                                pick_tuesday.append("-");
                                pick_tuesday.append(restaurantTiming.pick_tuesday_second_closetime);

                            }
                        }
                    }
                    if (pick_tuesday.toString().isEmpty()) {
                        pickupCloseCount++;
                        tvPickTuesdayTime.setText("CLOSED");
                        llPickupTuesday.setVisibility(View.GONE);
                    } else {
                        pickupCloseCount--;
                        llPickupTuesday.setVisibility(View.VISIBLE);
                        if ((!Validators.isNullOrEmpty(restaurantTiming.pick_tuesday_status) && restaurantTiming.pick_tuesday_status.equalsIgnoreCase("close"))) {
                            tvPickTuesdayTime.setText("CLOSED");
                        } else
                            tvPickTuesdayTime.setText(pick_tuesday.toString());
                    }

                    if (!Validators.isNullOrEmpty(restaurantTiming.pick_wednesday_first_opentime)) {
                        if (!restaurantTiming.pick_wednesday_first_opentime.equalsIgnoreCase(restaurantTiming.pick_wednesday_first_closetime)) {
                            pick_wednesday.append(restaurantTiming.pick_wednesday_first_opentime);
                            pick_wednesday.append("-");
                            pick_wednesday.append(restaurantTiming.pick_wednesday_first_closetime);
                        }

                        if (!Validators.isNullOrEmpty(restaurantTiming.pick_wednesday_second_opentime)) {
                            if (!restaurantTiming.pick_wednesday_second_opentime.equalsIgnoreCase(restaurantTiming.pick_wednesday_second_closetime)) {
                                if (!pick_wednesday.toString().isEmpty()) {
                                    pick_wednesday.append(", ");
                                }
                                pick_wednesday.append(restaurantTiming.pick_wednesday_second_opentime);
                                pick_wednesday.append("-");
                                pick_wednesday.append(restaurantTiming.pick_wednesday_second_closetime);

                            }
                        }
                    }
                    if (pick_wednesday.toString().isEmpty()) {
                        pickupCloseCount++;
                        tvPickWedTime.setText("CLOSED");
                        llPickupWed.setVisibility(View.GONE);
                    } else {
                        pickupCloseCount--;
                        llPickupWed.setVisibility(View.VISIBLE);
                        if ((!Validators.isNullOrEmpty(restaurantTiming.pick_wednesday_status) && restaurantTiming.pick_wednesday_status.equalsIgnoreCase("close"))) {
                            tvPickWedTime.setText("CLOSED");
                        } else {
                            tvPickWedTime.setText(pick_wednesday.toString());
                        }
                    }

                    if (!Validators.isNullOrEmpty(restaurantTiming.pick_thursday_first_opentime)) {
                        if (!restaurantTiming.pick_thursday_first_opentime.equalsIgnoreCase(restaurantTiming.pick_thursday_first_closetime)) {
                            pick_thursday.append(restaurantTiming.pick_thursday_first_opentime);
                            pick_thursday.append("-");
                            pick_thursday.append(restaurantTiming.pick_thursday_first_closetime);
                        }

                        if (!Validators.isNullOrEmpty(restaurantTiming.pick_thursday_second_opentime)) {
                            if (!restaurantTiming.pick_thursday_second_opentime.equalsIgnoreCase(restaurantTiming.pick_thursday_second_closetime)) {
                                if (!pick_thursday.toString().isEmpty()) {
                                    pick_thursday.append(", ");
                                }
                                pick_thursday.append(restaurantTiming.pick_thursday_second_opentime);
                                pick_thursday.append("-");
                                pick_thursday.append(restaurantTiming.pick_thursday_second_closetime);

                            }
                        }
                    }
                    if (pick_thursday.toString().isEmpty()) {
                        pickupCloseCount++;
                        tvPickThursdayTime.setText("CLOSED");
                        llPickupThursday.setVisibility(View.GONE);
                    } else {
                        pickupCloseCount--;
                        llPickupThursday.setVisibility(View.VISIBLE);
                        if ((!Validators.isNullOrEmpty(restaurantTiming.pick_thursday_status) && restaurantTiming.pick_thursday_status.equalsIgnoreCase("close"))) {
                            tvPickThursdayTime.setText("CLOSED");
                        } else {
                            tvPickThursdayTime.setText(pick_thursday.toString());
                        }
                    }

                    if (!Validators.isNullOrEmpty(restaurantTiming.pick_friday_first_opentime)) {
                        if (!restaurantTiming.pick_friday_first_opentime.equalsIgnoreCase(restaurantTiming.pick_friday_first_closetime)) {
                            pick_friday.append(restaurantTiming.pick_friday_first_opentime);
                            pick_friday.append("-");
                            pick_friday.append(restaurantTiming.pick_friday_first_closetime);
                        }

                        if (!Validators.isNullOrEmpty(restaurantTiming.pick_friday_second_opentime)) {
                            if (!restaurantTiming.pick_friday_second_opentime.equalsIgnoreCase(restaurantTiming.pick_friday_second_closetime)) {
                                if (!pick_friday.toString().isEmpty()) {
                                    pick_friday.append(", ");
                                }
                                pick_friday.append(restaurantTiming.pick_friday_second_opentime);
                                pick_friday.append("-");
                                pick_friday.append(restaurantTiming.pick_friday_second_closetime);

                            }
                        }
                    }
                    if (pick_friday.toString().isEmpty()) {
                        pickupCloseCount++;
                        tvPickFridayTime.setText("CLOSED");
                        llPickupFriday.setVisibility(View.GONE);
                    } else {
                        pickupCloseCount--;
                        llPickupFriday.setVisibility(View.VISIBLE);
                        if ((!Validators.isNullOrEmpty(restaurantTiming.pick_friday_status) && restaurantTiming.pick_friday_status.equalsIgnoreCase("close"))) {
                            tvPickFridayTime.setText("CLOSED");
                        } else {
                            tvPickFridayTime.setText(pick_friday.toString());
                        }
                    }

                    if (!Validators.isNullOrEmpty(restaurantTiming.pick_saturday_first_opentime)) {
                        if (!restaurantTiming.pick_saturday_first_opentime.equalsIgnoreCase(restaurantTiming.pick_saturday_first_closetime)) {
                            pick_saturday.append(restaurantTiming.pick_saturday_first_opentime);
                            pick_saturday.append("-");
                            pick_saturday.append(restaurantTiming.pick_saturday_first_closetime);
                        }

                        if (!Validators.isNullOrEmpty(restaurantTiming.pick_saturday_second_opentime)) {
                            if (!restaurantTiming.pick_saturday_second_opentime.equalsIgnoreCase(restaurantTiming.pick_saturday_second_closetime)) {
                                if (!pick_saturday.toString().isEmpty()) {
                                    pick_saturday.append(", ");
                                }
                                pick_saturday.append(restaurantTiming.pick_saturday_second_opentime);
                                pick_saturday.append("-");
                                pick_saturday.append(restaurantTiming.pick_saturday_second_closetime);

                            }
                        }
                    }
                    if (pick_saturday.toString().isEmpty()) {
                        pickupCloseCount++;
                        tvPickSatTime.setText("CLOSED");
                        llPickupSaturday.setVisibility(View.GONE);
                    } else {
                        pickupCloseCount--;
                        llPickupSaturday.setVisibility(View.VISIBLE);
                        if ((!Validators.isNullOrEmpty(restaurantTiming.pick_saturday_status) && restaurantTiming.pick_saturday_status.equalsIgnoreCase("close"))) {
                            tvPickSatTime.setText("CLOSED");
                        } else {
                            tvPickSatTime.setText(pick_saturday.toString());
                        }
                    }

                    if (!Validators.isNullOrEmpty(restaurantTiming.pick_sunday_first_opentime)) {
                        if (!restaurantTiming.pick_sunday_first_opentime.equalsIgnoreCase(restaurantTiming.pick_sunday_first_closetime)) {
                            pick_sunday.append(restaurantTiming.pick_sunday_first_opentime);
                            pick_sunday.append("-");
                            pick_sunday.append(restaurantTiming.pick_sunday_first_closetime);
                        }

                        if (!Validators.isNullOrEmpty(restaurantTiming.pick_sunday_second_opentime)) {
                            if (!restaurantTiming.pick_sunday_second_opentime.equalsIgnoreCase(restaurantTiming.pick_sunday_second_closetime)) {
                                if (!pick_sunday.toString().isEmpty()) {
                                    pick_sunday.append(", ");
                                }
                                pick_sunday.append(restaurantTiming.pick_sunday_second_opentime);
                                pick_sunday.append("-");
                                pick_sunday.append(restaurantTiming.pick_sunday_second_closetime);

                            }
                        }
                    }
                    if (pick_sunday.toString().isEmpty()) {
                        pickupCloseCount++;
                        tvPickSundayTime.setText("CLOSED");
                        llPickupSunday.setVisibility(View.GONE);
                    } else {
                        pickupCloseCount--;
                        llPickupSunday.setVisibility(View.VISIBLE);
                        if ((!Validators.isNullOrEmpty(restaurantTiming.pick_sunday_status) && restaurantTiming.pick_sunday_status.equalsIgnoreCase("close"))) {
                            tvPickSundayTime.setText("CLOSED");
                        } else {
                            tvPickSundayTime.setText(pick_sunday.toString());
                        }
                    }
                    if (pickupCloseCount >= 7) {
                        llPickupTimes.setVisibility(View.GONE);
                        llPickupTimesTab.setVisibility(View.GONE);
                        vPickup.setVisibility(View.INVISIBLE);
                    } else {
                        if (closeCount >= 7) {
                            llPickupTimes.setVisibility(View.VISIBLE);
                            llPickupTimesTab.setVisibility(View.VISIBLE);
                            vPickup.setVisibility(View.VISIBLE);
                        }
                    }

                } else {
                    llPickupTimes.setVisibility(View.GONE);
                    llPickupTimesTab.setVisibility(View.GONE);
                    pickupCloseCount = 7;
                }
                if (pickupCloseCount >= 7 && closeCount >= 7) {
                    cvTimes.setVisibility(View.GONE);
                }
                if (CommonFunctions.getTodayDay().equalsIgnoreCase("monday")) {
                    tvMondayTime.setTextColor(ContextCompat.getColor(getContext(), R.color.green_dark));
                    tvMondayTimeLabel.setTextColor(ContextCompat.getColor(getContext(), R.color.green_dark));

                    tvPickMondayTime.setTextColor(ContextCompat.getColor(getContext(), R.color.green_dark));
                    tvPickMondayTimeLabel.setTextColor(ContextCompat.getColor(getContext(), R.color.green_dark));
                } else if (CommonFunctions.getTodayDay().equalsIgnoreCase("tuesday")) {
                    tvTuesdayTime.setTextColor(ContextCompat.getColor(getContext(), R.color.green_dark));
                    tvTuesdayTimeLabel.setTextColor(ContextCompat.getColor(getContext(), R.color.green_dark));

                    tvPickTuesdayTime.setTextColor(ContextCompat.getColor(getContext(), R.color.green_dark));
                    tvPickTuesdayTimeLabel.setTextColor(ContextCompat.getColor(getContext(), R.color.green_dark));
                } else if (CommonFunctions.getTodayDay().equalsIgnoreCase("wednesday")) {
                    tvWedTime.setTextColor(ContextCompat.getColor(getContext(), R.color.green_dark));
                    tvWedTimeLabel.setTextColor(ContextCompat.getColor(getContext(), R.color.green_dark));

                    tvPickWedTime.setTextColor(ContextCompat.getColor(getContext(), R.color.green_dark));
                    tvPickWedTimeLabel.setTextColor(ContextCompat.getColor(getContext(), R.color.green_dark));
                } else if (CommonFunctions.getTodayDay().equalsIgnoreCase("thursday")) {
                    tvThursdayTime.setTextColor(ContextCompat.getColor(getContext(), R.color.green_dark));
                    tvThursdayTimeLabel.setTextColor(ContextCompat.getColor(getContext(), R.color.green_dark));

                    tvPickThursdayTime.setTextColor(ContextCompat.getColor(getContext(), R.color.green_dark));
                    tvPickThursdayTimeLabel.setTextColor(ContextCompat.getColor(getContext(), R.color.green_dark));
                } else if (CommonFunctions.getTodayDay().equalsIgnoreCase("friday")) {
                    tvFridayTime.setTextColor(ContextCompat.getColor(getContext(), R.color.green_dark));
                    tvFridayTimeLabel.setTextColor(ContextCompat.getColor(getContext(), R.color.green_dark));

                    tvPickFridayTime.setTextColor(ContextCompat.getColor(getContext(), R.color.green_dark));
                    tvPickFridayTimeLabel.setTextColor(ContextCompat.getColor(getContext(), R.color.green_dark));
                } else if (CommonFunctions.getTodayDay().equalsIgnoreCase("saturday")) {
                    tvSatTime.setTextColor(ContextCompat.getColor(getContext(), R.color.green_dark));
                    tvSatTimeLabel.setTextColor(ContextCompat.getColor(getContext(), R.color.green_dark));

                    tvPickSatTime.setTextColor(ContextCompat.getColor(getContext(), R.color.green_dark));
                    tvPickSatTimeLabel.setTextColor(ContextCompat.getColor(getContext(), R.color.green_dark));
                } else if (CommonFunctions.getTodayDay().equalsIgnoreCase("sunday")) {
                    tvSundayTime.setTextColor(ContextCompat.getColor(getContext(), R.color.green_dark));
                    tvSundayTimeLabel.setTextColor(ContextCompat.getColor(getContext(), R.color.green_dark));

                    tvPickSundayTime.setTextColor(ContextCompat.getColor(getContext(), R.color.green_dark));
                    tvPickSundayTimeLabel.setTextColor(ContextCompat.getColor(getContext(), R.color.green_dark));
                }
                if (tvMondayTime.getText().toString().equalsIgnoreCase("closed"))
                    tvMondayTime.setTextColor(ContextCompat.getColor(getContext(), R.color.colorRed));
                if (tvTuesdayTime.getText().toString().equalsIgnoreCase("closed"))
                    tvTuesdayTime.setTextColor(ContextCompat.getColor(getContext(), R.color.colorRed));
                if (tvWedTime.getText().toString().equalsIgnoreCase("closed"))
                    tvWedTime.setTextColor(ContextCompat.getColor(getContext(), R.color.colorRed));
                if (tvThursdayTime.getText().toString().equalsIgnoreCase("closed"))
                    tvThursdayTime.setTextColor(ContextCompat.getColor(getContext(), R.color.colorRed));
                if (tvFridayTime.getText().toString().equalsIgnoreCase("closed"))
                    tvFridayTime.setTextColor(ContextCompat.getColor(getContext(), R.color.colorRed));
                if (tvSatTime.getText().toString().equalsIgnoreCase("closed"))
                    tvSatTime.setTextColor(ContextCompat.getColor(getContext(), R.color.colorRed));
                if (tvSundayTime.getText().toString().equalsIgnoreCase("closed"))
                    tvSundayTime.setTextColor(ContextCompat.getColor(getContext(), R.color.colorRed));

                if (tvPickMondayTime.getText().toString().equalsIgnoreCase("closed"))
                    tvPickMondayTime.setTextColor(ContextCompat.getColor(getContext(), R.color.colorRed));
                if (tvPickTuesdayTime.getText().toString().equalsIgnoreCase("closed"))
                    tvPickTuesdayTime.setTextColor(ContextCompat.getColor(getContext(), R.color.colorRed));
                if (tvPickWedTime.getText().toString().equalsIgnoreCase("closed"))
                    tvPickWedTime.setTextColor(ContextCompat.getColor(getContext(), R.color.colorRed));
                if (tvPickThursdayTime.getText().toString().equalsIgnoreCase("closed"))
                    tvPickThursdayTime.setTextColor(ContextCompat.getColor(getContext(), R.color.colorRed));
                if (tvPickFridayTime.getText().toString().equalsIgnoreCase("closed"))
                    tvPickFridayTime.setTextColor(ContextCompat.getColor(getContext(), R.color.colorRed));
                if (tvPickSatTime.getText().toString().equalsIgnoreCase("closed"))
                    tvPickSatTime.setTextColor(ContextCompat.getColor(getContext(), R.color.colorRed));
                if (tvPickSundayTime.getText().toString().equalsIgnoreCase("closed"))
                    tvPickSundayTime.setTextColor(ContextCompat.getColor(getContext(), R.color.colorRed));
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    private void setListener() {
        llCall.setOnClickListener(v -> {
            try {
                if (restaurant != null && !Validators.isNullOrEmpty(restaurant.contact_phone)) {
                    String phone = restaurant.contact_phone;
                    Intent intent = new Intent(Intent.ACTION_DIAL, Uri.fromParts("tel", phone, null));
                    startActivity(intent);
                }
            } catch (Exception e) {
                e.printStackTrace();
            }
        });


        ivFavouriteImage.setOnClickListener(view -> {
            try {
                if (loggedInUser == null) {
                    startActivity(new Intent(getActivity(), AuthActivity.class));
                } else {
                    restaurant.is_favourite = !(restaurant.is_favourite == 1) ? 1 : 0;
                    if (restaurant.is_favourite == 1) {
                        MyApp.favouriteList.add(String.valueOf(restaurant.id));
                    } else {
                        MyApp.favouriteList.remove(String.valueOf(restaurant.id));
                    }
                    RestaurantDetailsActivity.openedRestaurant.is_favourite = restaurant.is_favourite;
                    if (restaurant.is_favourite == 1) {
                        ivFavourite.playAnimation();
                        ivFavouriteImage.setImageResource(R.drawable.ic_baseline_favorite_24);
                    } else {
                        ivFavourite.setProgress(0);
                        ivFavourite.cancelAnimation();
                        ivFavouriteImage.setImageResource(R.drawable.icon_favourite);
                    }
                    makeRestaurantFavourite();
                }
            } catch (Exception e) {
                e.printStackTrace();
            }
        });
        llDeliveryTimesTab.setOnClickListener(view -> {
            vDelivery.setVisibility(View.VISIBLE);
            llDeliveryTimes.setVisibility(View.VISIBLE);

            vPickup.setVisibility(View.INVISIBLE);
            llPickupTimes.setVisibility(View.GONE);
        });
        llPickupTimesTab.setOnClickListener(view -> {
            vPickup.setVisibility(View.VISIBLE);
            llPickupTimes.setVisibility(View.VISIBLE);

            vDelivery.setVisibility(View.INVISIBLE);
            llDeliveryTimes.setVisibility(View.GONE);
        });
    }

    private void makeRestaurantFavourite() {
        try {
            ApiUtils.getFavouriteAction(String.valueOf(loggedInUser.id), String.valueOf(restaurant.id), (restaurant.is_favourite == 1) ? "1" : "0")
                    .getAsJSONObject(new JSONObjectRequestListener() {
                        @Override
                        public void onResponse(JSONObject response) {
                            try {
                                String errorMessage = myApp.checkErrorMessage(response);
                                if (errorMessage == null) {
                                    LogUtils.e(new Gson().toJson(response));
                                    if (getActivity() != null)
                                        getActivity().setResult(Activity.RESULT_OK);
                                } else {
                                    restaurant.is_favourite = (restaurant.is_favourite == 1) ? 1 : 0;
                                    RestaurantDetailsActivity.openedRestaurant.is_favourite = restaurant.is_favourite;
                                    ToastUtils.makeToast(getActivity(), "Something went wrong!!");
                                }
                            } catch (Exception e) {
                                e.printStackTrace();
                            }
                        }

                        @Override
                        public void onError(ANError anError) {
                            try {
                                anError.printStackTrace();
                                LogUtils.e("ERROR WHILE FAVORITE :: ", anError.getErrorBody());
                                if (!CommonFunctions.isConnected(getActivity()))
                                    myApp.noInternet(getActivity());
                            } catch (Exception e) {
                                e.printStackTrace();
                            }
                        }
                    });
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    private void updateFavouriteUI() {
        try {
            restaurant = RestaurantDetailsActivity.openedRestaurant;
            if (MyApp.favouriteList != null) {
                int restIndex = MyApp.favouriteList.indexOf(String.valueOf(restaurant.id));
                restaurant.is_favourite = restIndex != -1 ? 1 : 0;
            }
            if (restaurant.is_favourite == 1) {
                ivFavourite.playAnimation();
                ivFavouriteImage.setImageResource(R.drawable.ic_baseline_favorite_24);
            } else {
                ivFavourite.setProgress(0);
                ivFavourite.cancelAnimation();
                ivFavouriteImage.setImageResource(R.drawable.icon_favourite);
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    @Override
    public void onMapReady(GoogleMap googleMap) {
        try {
            if (restaurant != null) {
                LatLng restaurantLocation = new LatLng(Double.parseDouble(restaurant.sourcelatitude), Double.parseDouble(restaurant.sourcelongitude));
                googleMap.addMarker(new MarkerOptions()
                        .position(restaurantLocation)
                        .title(restaurant.restaurant_name));
                float zoom = 15;
                googleMap.moveCamera(CameraUpdateFactory.newLatLngZoom(restaurantLocation, zoom));
            }
        } catch (NumberFormatException e) {
            e.printStackTrace();
        }
    }

    @Override
    public void onResume() {
        super.onResume();
        mapView.onResume();
        updateFavouriteUI();
    }

    @Override
    public void onDestroy() {
        super.onDestroy();
        mapView.onDestroy();
    }
}
