package com.tiffintom.fragment;

import android.app.Activity;
import android.content.Intent;
import android.os.Bundle;
import android.os.SystemClock;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.ImageView;
import android.widget.LinearLayout;
import android.widget.ProgressBar;
import android.widget.RelativeLayout;
import android.widget.TextView;

import androidx.annotation.NonNull;
import androidx.annotation.Nullable;
import androidx.cardview.widget.CardView;
import androidx.recyclerview.widget.LinearLayoutManager;
import androidx.recyclerview.widget.LinearSmoothScroller;
import androidx.recyclerview.widget.RecyclerView;

import com.airbnb.lottie.LottieAnimationView;
import com.androidnetworking.AndroidNetworking;
import com.androidnetworking.common.ANRequest;
import com.androidnetworking.error.ANError;
import com.androidnetworking.interfaces.JSONArrayRequestListener;
import com.androidnetworking.interfaces.JSONObjectRequestListener;
import com.bumptech.glide.Glide;
import com.google.android.material.appbar.AppBarLayout;
import com.google.android.material.appbar.CollapsingToolbarLayout;
import com.google.gson.Gson;
import com.google.gson.reflect.TypeToken;
import com.iarcuschin.simpleratingbar.SimpleRatingBar;
import com.tiffintom.BuildConfig;
import com.tiffintom.MyApp;
import com.tiffintom.R;
import com.tiffintom.activity.AuthActivity;
import com.tiffintom.activity.RestaurantDetailsActivity;
import com.tiffintom.activity.TransportActivity;
import com.tiffintom.adapters.RestaurantMenuAdapter;
import com.tiffintom.adapters.RestaurantMenuCategoryAdapter;
import com.tiffintom.base.BaseFragment;
import com.tiffintom.common.CommonFunctions;
import com.tiffintom.common.StickHeaderItemDecoration;
import com.tiffintom.common.Validators;
import com.tiffintom.interfaces.DialogDismissListener;
import com.tiffintom.models.Allergy;
import com.tiffintom.models.AllergyCaution;
import com.tiffintom.models.AllergyItem;
import com.tiffintom.models.CartItem;
import com.tiffintom.models.CartSummary;
import com.tiffintom.models.Category;
import com.tiffintom.models.Menu;
import com.tiffintom.models.DineinCartItem;
import com.tiffintom.models.Header;
import com.tiffintom.models.MenuItem;
import com.tiffintom.models.MenuItemNew;
import com.tiffintom.models.Postcode;
import com.tiffintom.models.Restaurant;
import com.tiffintom.models.Review;
import com.tiffintom.models.TimeSlots;
import com.tiffintom.models.UploadCartItems;
import com.tiffintom.models.User;
import com.tiffintom.models.Variant;
import com.tiffintom.network.ApiEndPoints;
import com.tiffintom.network.ApiUtils;
import com.tiffintom.utils.LogUtils;
import com.tiffintom.utils.ToastUtils;

import org.json.JSONArray;
import org.json.JSONException;
import org.json.JSONObject;

import java.lang.reflect.Type;
import java.util.ArrayList;
import java.util.List;

import static com.tiffintom.activity.RestaurantDetailsActivity.guest_count;
import static com.tiffintom.activity.RestaurantDetailsActivity.hideDefaultUI;
import static com.tiffintom.activity.RestaurantDetailsActivity.isDineIn;
import static com.tiffintom.activity.RestaurantDetailsActivity.openedRestaurant;
import static com.tiffintom.activity.RestaurantDetailsActivity.qrCode;
import static com.tiffintom.activity.RestaurantDetailsActivity.showDefaultUI;
import static com.tiffintom.activity.RestaurantDetailsActivity.tableNumber;
import static com.tiffintom.common.CommonFunctions.df;
import static com.tiffintom.utils.Constants.CODE_REFRESH;

/**
 * Created by Amrish on 19-08-2020.
 */
public class RestaurantMenuFragment extends BaseFragment {
    public static RestaurantMenuFragment getInstance() {
        RestaurantMenuFragment fragment = new RestaurantMenuFragment();
        Bundle bundle = new Bundle();
        fragment.setArguments(bundle);
        return fragment;
    }

    private ArrayList<Menu> restaurant_menu = new ArrayList<>();
    private ArrayList<Menu> dinein_restaurant_menu  = new ArrayList<>();
    private Postcode currentPostcode;
    private RecyclerView rvMenu, rvMenuCategory;
    private RestaurantMenuAdapter menuAdapter;
    private ArrayList<Object> feed = new ArrayList<>();
    private ArrayList<Category> categories = new ArrayList<>();
    private ArrayList<Category> menucategoryArrayList = new ArrayList<>();
    private ArrayList<AllergyItem> allergies = new ArrayList<>();

    private Restaurant restaurant = RestaurantDetailsActivity.openedRestaurant;
    private TextView tvRestaurant, tvDiscount, tvClosed, tvMinOrder, tvDistance, tvDeliveryFee, tvTotalReview, tvReservation;
    private SimpleRatingBar ratingBar;
    private CardView cvRestaurant;
    private ImageView ivRestaurantBanner, ivRestaurantLogo, ivFavouriteImage;
    private LottieAnimationView ivFavourite;
    private LinearLayout llMinOrder, llDistance, llDeliveryFee;
    private ProgressBar pbLoading;
    private RestaurantMenuCategoryAdapter categoryAdapter;
    RecyclerView.SmoothScroller categorySmoothScroller, menuSmoothScroller;
    LinearLayoutManager menuLayoutManager, categoryLayoutManager;
    private User loggedInUser = myApp.getMyPreferences().getLoggedInUser();
    private RelativeLayout llCheckout;
    private TextView tvTotal;
    private String message;
    private ArrayList<AllergyItem> mainAllergyItems = new ArrayList<>();
    private ArrayList<Menu> suggested = new ArrayList<>();
    private LottieAnimationView lodingView;
    private CollapsingToolbarLayout collapsingToolbar;
    private AppBarLayout appBarLayout;
    private ArrayList<UploadCartItems>cartItemArrayList=new ArrayList<>();
    @Nullable
    @Override
    public View onCreateView(@NonNull LayoutInflater inflater, @Nullable ViewGroup container, @Nullable Bundle savedInstanceState) {
        return inflater.inflate(R.layout.fragment_restaurant_menu, container, false);
    }

    @Override
    public void onViewCreated(@NonNull View view, @Nullable Bundle savedInstanceState) {
        super.onViewCreated(view, savedInstanceState);
        try {
            updateViews();
            setListeners();
            fetchData();
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    @Override
    protected void initViews(View view) {
        super.initViews(view);
        try {
            currentPostcode = myApp.getMyPreferences().getCurrentPostcode();
            pbLoading = view.findViewById(R.id.pbLoading);
            lodingView = view.findViewById(R.id.lodingView);

            llCheckout = view.findViewById(R.id.llMiniSnippet);
            tvTotal = view.findViewById(R.id.tvTotal);

            appBarLayout = view.findViewById(R.id.appbar_layout);
            collapsingToolbar = view.findViewById(R.id.collapsingToolbar);
            tvRestaurant = view.findViewById(R.id.tvRestaurant);
            tvClosed = view.findViewById(R.id.tvClosed);
            tvDiscount = view.findViewById(R.id.tvDiscount);
            tvMinOrder = view.findViewById(R.id.tvMinOrder);
            tvDistance = view.findViewById(R.id.tvDistance);
            tvDeliveryFee = view.findViewById(R.id.tvDeliveryFee);
            ratingBar = view.findViewById(R.id.rating);
            tvTotalReview = view.findViewById(R.id.tvTotalReviews);

            ivRestaurantBanner = view.findViewById(R.id.ivRestaurantBanner);
            ivRestaurantLogo = view.findViewById(R.id.ivRestaurantLogo);
            ivFavouriteImage = view.findViewById(R.id.ivFavouriteImage);
            ivFavourite = view.findViewById(R.id.ivFavourite);
            cvRestaurant = view.findViewById(R.id.cvRestaurant);

            llDistance = view.findViewById(R.id.llDistance);
            llDeliveryFee = view.findViewById(R.id.llDeliveryFee);
            llMinOrder = view.findViewById(R.id.llMinOrder);
            rvMenu = view.findViewById(R.id.rvMenu);
            rvMenuCategory = view.findViewById(R.id.rvMenuCategory);

            tvReservation = view.findViewById(R.id.tvReservation);

            menuAdapter = new RestaurantMenuAdapter(getActivity(), feed,  (int position, Object data) -> {
                try {
                    if (restaurant.online_order.equalsIgnoreCase("no")) {
                        showRestaurantIsClosed("Unfortunately, " + restaurant.restaurant_name + " is currently not accepting online orders", false);
                    } else {
                        if (data instanceof Header) {
                            if (((Header) data).addonId == 1) {
                                //Search clicked
                                startActivityForResult(new Intent(getActivity(), TransportActivity.class).putExtra("destination", "search_menu").putExtra("name", restaurant.restaurant_name).putExtra("hideToolbar", true), CODE_REFRESH);
                            }
                            if (((Header) data).addonId == 2) {
                                //Share clicked
                                message = "Checkout " + restaurant.restaurant_name + " restaurant on TiffinTom." +
                                        "iOS - https://apps.apple.com/in/app/id1437199436\n" +
                                        "Android - https://play.google.com/store/apps/details?id=" + BuildConfig.APPLICATION_ID;
                                Intent share = new Intent(Intent.ACTION_SEND);
                                share.setType("text/plain");
                                share.putExtra(Intent.EXTRA_TEXT, message);
                                startActivity(Intent.createChooser(share, "Share using"));
                            }

                        } else {
                            if (!isDineIn && restaurant.restaurant_delivery.equalsIgnoreCase("no") && restaurant.restaurant_pickup.equalsIgnoreCase("no") && restaurant.restaurant_status.equalsIgnoreCase("close")) {
                                showRestaurantIsClosed("Unfortunately, " + restaurant.restaurant_name + " is currently closed", false);
                            } else {
                                menuItemClick(position, data);
                            }
                        }
                    }
                } catch (Exception e) {
                    e.printStackTrace();
                }
            });
            menuLayoutManager = new LinearLayoutManager(getActivity(), RecyclerView.VERTICAL, false);
            rvMenu.setAdapter(menuAdapter);
            rvMenu.setLayoutManager(menuLayoutManager);
            rvMenu.addItemDecoration(new StickHeaderItemDecoration(menuAdapter));


            categoryAdapter = new RestaurantMenuCategoryAdapter(categories,false,this::scrollToCategory);
            categoryLayoutManager = new LinearLayoutManager(getActivity(), RecyclerView.HORIZONTAL, false);
            rvMenuCategory.setLayoutManager(categoryLayoutManager);
            rvMenuCategory.setAdapter(categoryAdapter);

            initlizeAlergies();
            setUpLessAllergies();
            updateCheckoutLayout();

            categorySmoothScroller = new LinearSmoothScroller(getActivity()) {
                @Override
                protected int getVerticalSnapPreference() {
                    return LinearSmoothScroller.SNAP_TO_START;
                }
            };
            menuSmoothScroller = new LinearSmoothScroller(getActivity()) {
                @Override
                protected int getVerticalSnapPreference() {
                    return LinearSmoothScroller.SNAP_TO_START;
                }
            };
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    private void handleMenuItemClick(int position, Object data) {
        try {
            if (openedRestaurant.online_order.equalsIgnoreCase("no")) {
                showRestaurantIsClosed("Unfortunately, " + restaurant.restaurant_name + " is currently not accepting online orders", false);
            } else {
                if (data instanceof Header) {
                    if (((Header) data).addonId == 1) {
                        //Search clicked
                        startActivityForResult(new Intent(getActivity(), TransportActivity.class).putExtra("destination", "search_menu").putExtra("name", restaurant.restaurant_name).putExtra("hideToolbar", true), CODE_REFRESH);
                    }
                    if (((Header) data).addonId == 2) {
                        //Share clicked
                        message = "Checkout " + restaurant.restaurant_name + " restaurant on TiffinTom." +
                                "iOS - https://apps.apple.com/in/app/id1437199436\n" +
                                "Android - https://play.google.com/store/apps/details?id=" + BuildConfig.APPLICATION_ID;

                        Intent share = new Intent(Intent.ACTION_SEND);
                        share.setType("text/plain");
                        share.putExtra(Intent.EXTRA_TEXT, message);
                        startActivity(Intent.createChooser(share, "Share using"));
                    }

                } else {
                    if (!isDineIn && isRestaurantClosed()) {
                        showRestaurantIsClosed("Unfortunately, " + restaurant.restaurant_name + " is currently closed", false);
                    }
                    else {
                        menuItemClick(position, data);
                    }
                }
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    private void initlizeAlergies() {
        try {
            allergies.clear();
            mainAllergyItems.clear();

            if (restaurant != null && restaurant.allergies != null) {
                if (restaurant.allergies.contains("Halal"))
                    mainAllergyItems.add(new AllergyItem("Halal", R.drawable.allergy_halal));

                if (restaurant.allergies.contains("Vegan"))
                    mainAllergyItems.add(new AllergyItem("Vegan", R.drawable.allergy_vegan));

                if (restaurant.allergies.contains("Vegetarian"))
                    mainAllergyItems.add(new AllergyItem("Vegetarian", R.drawable.allergy_vegetarian));

                if (restaurant.allergies.contains("Nuts"))
                    mainAllergyItems.add(new AllergyItem("Nuts", R.drawable.allergy_nuts));

                if (restaurant.allergies.contains("Gluten"))
                    mainAllergyItems.add(new AllergyItem("Gluten", R.drawable.allergy_gluten));

                if (restaurant.allergies.contains("Dairy"))
                    mainAllergyItems.add(new AllergyItem("Dairy", R.drawable.allergy_milk));

                if (restaurant.allergies.contains("Crustaceans"))
                    mainAllergyItems.add(new AllergyItem("Crustaceans", R.drawable.allergy_crab));

                if (restaurant.allergies.contains("Molluscs"))
                    mainAllergyItems.add(new AllergyItem("Molluscs", R.drawable.allergy_shell));

                if (restaurant.allergies.contains("Fish"))
                    mainAllergyItems.add(new AllergyItem("Fish", R.drawable.allergy_fish));

                if (restaurant.allergies.contains("Sulphur dioxide"))
                    mainAllergyItems.add(new AllergyItem("Sulphur dioxide", R.drawable.allergy_so2));

                if (restaurant.allergies.contains("Eggs"))
                    mainAllergyItems.add(new AllergyItem("Eggs", R.drawable.allergy_eggs));

                if (restaurant.allergies.contains("Sulphites"))
                    mainAllergyItems.add(new AllergyItem("Sulphite", R.drawable.allergy_sulphite));

            }
        } catch (Exception e) {
            e.printStackTrace();
        }

    }

    private void setUpAllAllergies() {
        allergies.clear();
        allergies.addAll(mainAllergyItems);

        if (allergies.size() > 8)
            allergies.add(new AllergyItem("Less", -1));
    }

    private void setUpLessAllergies() {
        try {
            allergies.clear();

            if (mainAllergyItems.size() > 7) {
                allergies = new ArrayList<>(mainAllergyItems.subList(0, 6));
                allergies.add(new AllergyItem("More", -1));
            } else {
                allergies.addAll(mainAllergyItems);
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    private void setListeners() {


        rvMenu.addOnScrollListener(new RecyclerView.OnScrollListener() {
            int scrolledPosition;
            @Override
            public void onScrolled(@NonNull RecyclerView recyclerView, int dx, int dy) {
                super.onScrolled(recyclerView, dx, dy);
                int visiblePosition = menuLayoutManager.findFirstCompletelyVisibleItemPosition();
                scrolledPosition = menuLayoutManager.findFirstVisibleItemPosition();
                if (feed.get(visiblePosition) instanceof Category) {
                    int catIndex = categories.indexOf(feed.get(visiblePosition));
                    if (catIndex != -1) {
                        RestaurantMenuCategoryAdapter.selectedCategory = (categories.get(catIndex)).category_id;
                        categoryAdapter.notifyDataSetChanged();
                        rvMenuCategory.scrollToPosition(catIndex);
                    }
                } else if (feed.get(visiblePosition) instanceof MenuItem) {
                    MenuItem m = (MenuItem) feed.get(visiblePosition);
                    int catId = m.category_id;
                    Category c = new Category("", m.category_id);
                    int catIndex = categories.indexOf(c);

                    if (catIndex != -1) {
                        RestaurantMenuCategoryAdapter.selectedCategory = (categories.get(catIndex)).category_id;
                        categoryAdapter.notifyDataSetChanged();
                        rvMenuCategory.scrollToPosition(catIndex);
                    }
                }
                if (scrolledPosition > 0) {
//                    appBarLayout.setExpanded(false, true);
                    hideDefaultUI();
                } else {
//                    appBarLayout.setExpanded(true, true);
                    showDefaultUI();
                }
            }

            @Override
            public void onScrollStateChanged(@NonNull RecyclerView recyclerView, int newState) {
                super.onScrollStateChanged(recyclerView, newState);
                if (scrolledPosition > 0) {
//                    appBarLayout.setExpanded(false, true);
                    hideDefaultUI();
                    RestaurantDetailsActivity.tabLayout.setVisibility(View.GONE);
                } else {
//                    appBarLayout.setExpanded(true, true);
                    showDefaultUI();
                    RestaurantDetailsActivity.tabLayout.setVisibility(View.VISIBLE);
                }
            }
        });

        tvReservation.setOnClickListener(view -> {
            try {
                if (loggedInUser == null) {
                    startActivity(new Intent(getActivity(), AuthActivity.class));
                } else {
                    MakeReservationBottomSheetFragment makeReservationBottomSheetFragment = MakeReservationBottomSheetFragment.getInstance(restaurant.id);
                    makeReservationBottomSheetFragment.show(getChildFragmentManager(), "make_reservation");
                }
            } catch (Exception e) {
                e.printStackTrace();
            }
        });
        ivFavouriteImage.setOnClickListener(view -> {
            try {
                if (loggedInUser == null) {
                    startActivity(new Intent(getActivity(), AuthActivity.class));
                } else {
                    restaurant.is_favourite = !(restaurant.is_favourite==1) ? 1 : 0;
                    if (restaurant.is_favourite==1) {
                        MyApp.favouriteList.add(String.valueOf(restaurant.id));
                    } else {
                        MyApp.favouriteList.remove(String.valueOf(restaurant.id));
                    }
                    RestaurantDetailsActivity.openedRestaurant.is_favourite = restaurant.is_favourite;
                    if (restaurant.is_favourite==1) {
                        ivFavourite.playAnimation();
                        ivFavouriteImage.setImageResource(R.drawable.ic_baseline_favorite_24);
                    } else {
                        ivFavourite.setProgress(0);
                        ivFavourite.cancelAnimation();
                        ivFavouriteImage.setImageResource(R.drawable.icon_favourite);
                    }
                    if (restaurant.is_favourite==1)
                        ToastUtils.makeSnackToast(getActivity(), restaurant.restaurant_name + " has been added to favourites");
                    else
                        ToastUtils.makeSnackToast(getActivity(), restaurant.restaurant_name + " has been removed from favourites");
                    makeRestaurantFavourite();
                }
            } catch (Exception e) {
                e.printStackTrace();
            }
        });
        long lastClickTime = 0;
        llCheckout.setOnClickListener(view -> {
            try {
                long currentTime = SystemClock.elapsedRealtime();
                if (currentTime - lastClickTime > 500) {
                    if (loggedInUser == null) {
                        startActivity(new Intent(getActivity(), AuthActivity.class));
                    } else {
                        if (!isDineIn)
                             validateRestaurantOpeningAndCheckout();
                        else openCheckoutScreen();
                    }
                }
            } catch (Exception e) {
                e.printStackTrace();
            }
        });
    }

    private void openCheckoutScreen() {
        try {
            if (isDineIn) {
                startActivityForResult(new Intent(getActivity(), TransportActivity.class).putExtra("destination", "dinein_checkout").putExtra("is_dinein", true).putExtra("name", "Dine in Checkout").putExtra("table_number", tableNumber).putExtra("guests", guest_count).putExtra("qr_code", qrCode).putExtra("resId", restaurant.id).putExtra("show_back", true), CODE_REFRESH);
            } else {
                new Thread(this::saveCartOnline).start();
                startActivityForResult(new Intent(getActivity(), TransportActivity.class).putExtra("destination", "cart").putExtra("name", "Checkout").putExtra("hideToolbar", true).putExtra("show_back", true), CODE_REFRESH);
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    private void validateRestaurantOpeningAndCheckout() {
        try {
            if (isRestaurantClosed()) {
                showRestaurantIsClosed("Unfortunately, " + restaurant.restaurant_name + " is currently closed", false);
            } else {
                openCheckoutScreen();
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
    }
    private void fetchTimeSlots() {
        try {
            ApiUtils.getRestaurantTimeSlots(String.valueOf(restaurant.id), CommonFunctions.getToday())
                    .getAsJSONObject(new JSONObjectRequestListener() {
                        @Override
                        public void onResponse(JSONObject response) {
                            try {
                                JSONObject JsonObject=new JSONObject(response.toString());
                                restaurant.delivery_time_slot = new Gson().fromJson(JsonObject.getJSONObject("delivery").toString(), TimeSlots.class);
                                restaurant.pickup_time_slot = new Gson().fromJson(JsonObject.getJSONObject("pickup").toString(), TimeSlots.class);

                            } catch (JSONException e) {
                                e.printStackTrace();
                            }
                        }

                        @Override
                        public void onError(ANError anError) {
                            anError.printStackTrace();
                        }
                    });
        } catch (Exception e) {
            e.printStackTrace();
        }
    }
    private boolean isRestaurantClosed() {
        boolean isClose;
        if (restaurant.status.equalsIgnoreCase("0") || (restaurant.restaurant_delivery.equalsIgnoreCase("no") && restaurant.restaurant_pickup.equalsIgnoreCase("no"))) {
            isClose = true;
        } else {
            if (!restaurant.restaurant_delivery.equalsIgnoreCase("no") && isSlotsAvailable(restaurant.delivery_time_slot)) {
                isClose = false;
            } else {
                isClose = true;
                isClose = restaurant.restaurant_pickup.equalsIgnoreCase("no") || !isSlotsAvailable(restaurant.pickup_time_slot);
            }
        }
        return isClose;
    }

    private boolean isSlotsAvailable(TimeSlots timeSlots) {
        return !(Validators.isNullOrEmpty(timeSlots.today) && Validators.isNullOrEmpty(timeSlots.tomorrow) && Validators.isNullOrEmpty(timeSlots.dayaftertomorrow));
    }

    private void showRestaurantIsClosed(String message, boolean shouldGoBack) {
        try {
            RestaurantClosedDialogFragment restaurantClosedDialogFragment = RestaurantClosedDialogFragment.getInstance(message);
            restaurantClosedDialogFragment.show(getChildFragmentManager(), "restaurant_closed");
            restaurantClosedDialogFragment.setDialogDismissListener(o -> {
                if (shouldGoBack) {
                    if (getActivity() != null)
                        getActivity().onBackPressed();
                }
            });
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    private void scrollToCategory(int position, Object data) {
        int selectedCatIndex = feed.indexOf(data);
        if (selectedCatIndex != -1) {
            menuSmoothScroller.setTargetPosition(selectedCatIndex);
        } else {
            menuSmoothScroller.setTargetPosition(0);
        }
        menuLayoutManager.startSmoothScroll(menuSmoothScroller);
    }

    private void updateViews() {
        try {
            if (restaurant != null) {
                tvRestaurant.setText(restaurant.restaurant_name);
                try {
                    llDeliveryFee.setVisibility(View.VISIBLE);
                    if (restaurant.deliveryCharges > 0) {
                        tvDeliveryFee.setText(CommonFunctions.StringAppender(myApp.getCurrencySymbol(), df.format(restaurant.deliveryCharges), " Del Fee"));
                    } else {
                        tvDeliveryFee.setText("Free Delivery");
                    }
                    if (restaurant.delivery_charge != null && restaurant.delivery_charge.equalsIgnoreCase("yes")) {
                        llDeliveryFee.setVisibility(View.VISIBLE);
                    } else {
                        llDeliveryFee.setVisibility(View.INVISIBLE);
                    }
                } catch (NumberFormatException e) {
                    llDeliveryFee.setVisibility(View.INVISIBLE);
                }
                tvMinOrder.setText(CommonFunctions.StringAppender(restaurant.currency, df.format(restaurant.minimum_order), " min cost"));
                tvDistance.setText(CommonFunctions.StringAppender(df.format(restaurant.distance), " miles"));
                if (restaurant.reviews != null)
                {
                    tvTotalReview.setText(CommonFunctions.StringAppender(String.valueOf(restaurant.reviews.size()), " Reviews"));
                }


                Glide.with(getActivity())
                        .load(restaurant.image_url)
                        .placeholder(R.drawable.restaurant_logo_placeholder)
                        .error(R.drawable.restaurant_logo_placeholder)
                        .into(ivRestaurantLogo);
                ratingBar.setRating(restaurant.average_rating);

                if (restaurant.promotions != null && restaurant.promotions.size() > 0) {
                    String bannerImage = restaurant.promotions.get(0).image_url;
                    Glide.with(getActivity())
                            .load(bannerImage)
                            .placeholder(R.drawable.restaurant_banner_placeholder)
                            .error(R.drawable.restaurant_banner_placeholder)
                            .into(ivRestaurantBanner);
                }

                if (restaurant.offer_texts != null && restaurant.offer_texts.size() > 0) {

                    StringBuilder offerText = new StringBuilder();
                    for (int i = 0; i < restaurant.offer_texts.size(); i++) {
                        offerText.append(restaurant.offer_texts.get(i).offer_name);
                        if (i != (restaurant.offer_texts.size() - 1)) {
                            offerText.append(", ");
                        }
                    }
                    tvDiscount.setText(offerText.toString());
                    tvDiscount.setVisibility(View.VISIBLE);
                } else {
                    tvDiscount.setVisibility(View.GONE);
                }
                tvDiscount.setSelected(true);
                tvDiscount.setSingleLine(true);

                if (isDineIn) {
                    tvReservation.setVisibility(View.GONE);
                } else {

                    if (restaurant.restaurant_booktable.equalsIgnoreCase("yes")) {
                        tvReservation.setVisibility(View.VISIBLE);
                    } else {
                        tvReservation.setVisibility(View.GONE);
                    }
                }
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    private void menuItemClick(int position, Object data) {
        try {
            if (data instanceof Menu) {
                if (!isDineIn && myApp.getMyPreferences().getOrderRestaurant() != null && myApp.getMyPreferences().getOrderRestaurant().id != restaurant.id) {
                    ConfirmationDialogFragment confirmationDialogFragment = ConfirmationDialogFragment.getInstance("Start new basket?", "Your basket already contains items from " + myApp.getMyPreferences().getOrderRestaurant().restaurant_name + ". Do you wish to clear the basket and add this item instead?");
                    confirmationDialogFragment.show(getChildFragmentManager(), "confirm");
                    confirmationDialogFragment.setDialogDismissListener(new DialogDismissListener() {
                        @Override
                        public void onDialogDismiss(Object o) {
                            if (o instanceof String && ((String) o).equalsIgnoreCase("confirm")) {
                                new Thread(() -> {
                                    myApp.getMyPreferences().deleteOrderRestaurant();
                                    myApp.getAppDatabase().cartDao().nukeTable();
                                    proceedToAddItemOrViewItem(position, data);
                                }).start();
                            }
                        }
                    });
                } else {
                    proceedToAddItemOrViewItem(position, data);
                }


            } else if (data instanceof String) {
                if (((String) data).equalsIgnoreCase("more")) {
                    setUpAllAllergies();
                } else {
                    setUpLessAllergies();
                }
                setUpAdapters();
            } else if (data instanceof Header) {
                if (((Header) data).addonId == 1) {
                    //Search clicked
                    startActivityForResult(new Intent(getActivity(), TransportActivity.class).putExtra("destination", "search_menu").putExtra("name", restaurant.restaurant_name).putExtra("hideToolbar", true), CODE_REFRESH);
                }
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    private void proceedToAddItemOrViewItem(int position, Object data) {
        try {
            if ((((Menu) data).price_option.equalsIgnoreCase("multiple") || ((Menu) data).menu_addon.equalsIgnoreCase("yes")) && ((Menu) data).variants.size() !=0) {
                MenuItemDetailBottomsheetFragment menuItemDetailBottomsheetFragment = MenuItemDetailBottomsheetFragment.getInstance((Menu) data, isDineIn);
                menuItemDetailBottomsheetFragment.show(getChildFragmentManager(), "addon");
                menuItemDetailBottomsheetFragment.setDialogDismissListener(addItemCartDialogListener);
            } else {
                if (isDineIn) {
                    addItemToDineInCart((Menu) data, position);
                } else {
                    addItemToCart((Menu) data, position);
                }
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    private void addItemToCart(Menu menuItem, int position) {
        try {
            new Thread(() -> {
                String variantName = null;
                float discount = 0;
                if (menuItem.variants != null && menuItem.variants.size() > 0) {
                    variantName = menuItem.variants.get(0).sub_name;
                    variantName = (menuItem.menu_name.equalsIgnoreCase(variantName)) ? "" : variantName;
                    discount = (menuItem.variants.get(0).orginal_price * menuItem.getProductPercentage() / 100);
                }
                CartItem dbCartItem = myApp.getAppDatabase().cartDao().getCartMenuItem(menuItem.id, variantName);
                if (dbCartItem == null) {
                    CartItem cartItem = new CartItem();
                    if (menuItem.variants != null && menuItem.variants.size() > 0) {
                        Variant selectedVariant = menuItem.variants.get(0);
                        cartItem.quantity = 1;
                        cartItem.Menu_price = selectedVariant.orginal_price - discount;
                        cartItem.menu_id = selectedVariant.menu_id;
                        cartItem.id = selectedVariant.id;
                        cartItem.menu_name = menuItem.menu_name;
                        cartItem.menu_size = (menuItem.menu_name.equalsIgnoreCase(selectedVariant.sub_name)) ? "" : selectedVariant.sub_name;
                        cartItem.res_id = menuItem.restaurant_id;
                        cartItem.menu_type = menuItem.menu_type;

                        cartItem.Addon_name = "";
                        cartItem.Addon_price = 0;
                        cartItem.Total = cartItem.quantity * (cartItem.Menu_price + cartItem.Addon_price);
                        myApp.getAppDatabase().cartDao().insertAll(cartItem);
                    }
                } else {
                    if (menuItem.variants != null && menuItem.variants.size() > 0) {
                        Variant selectedVariant = menuItem.variants.get(0);
                        dbCartItem.quantity++;
                        dbCartItem.Menu_price = +(selectedVariant.orginal_price - discount);
                        dbCartItem.Total = dbCartItem.quantity * (dbCartItem.Menu_price + dbCartItem.Addon_price);
                        myApp.getAppDatabase().cartDao().update(dbCartItem);
                    }

                }
                if (getActivity() != null) {
                    getActivity().runOnUiThread(() -> {
                        menuAdapter.notifyItemChanged(position);
                        updateCheckoutLayout();
                    });
                }
            }).start();
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    private void addItemToDineInCart(Menu menuItem, int position) {
        try {
            new Thread(() -> {
                String variantName = null;
                float discount = 0;
                if (menuItem.variants != null && menuItem.variants.size() > 0) {
                    variantName = menuItem.variants.get(0).sub_name;
                    variantName = (menuItem.menu_name.equalsIgnoreCase(variantName)) ? "" : variantName;
                    discount = (menuItem.variants.get(0).orginal_price * menuItem.getProductPercentage() / 100);
                }
                DineinCartItem dbDineInCartItem = myApp.getAppDatabase().dineinCartItemDao().getCartMenuItem(menuItem.id, variantName);
                if (dbDineInCartItem == null) {
                    DineinCartItem cartItem = new DineinCartItem();
                    if (menuItem.variants != null && menuItem.variants.size() > 0) {
                        Variant selectedVariant = menuItem.variants.get(0);
                        cartItem.quantity = 1;
                        cartItem.Menu_price = selectedVariant.orginal_price - discount;
                        cartItem.menu_id = selectedVariant.menu_id;
                        cartItem.id = selectedVariant.id;
                        cartItem.menu_name = menuItem.menu_name;
                        cartItem.menu_size = (menuItem.menu_name.equalsIgnoreCase(selectedVariant.sub_name)) ? "" : selectedVariant.sub_name;
                        cartItem.res_id = menuItem.restaurant_id;
                        cartItem.menu_type = menuItem.menu_type;

                        cartItem.Addon_name = "";
                        cartItem.Addon_price = 0;
                        cartItem.Total = cartItem.quantity * (cartItem.Menu_price + cartItem.Addon_price);
                        myApp.getAppDatabase().dineinCartItemDao().insertAll(cartItem);
                    }
                } else {
                    if (menuItem.variants != null && menuItem.variants.size() > 0) {
                        Variant selectedVariant = menuItem.variants.get(0);
                        dbDineInCartItem.quantity++;
                        dbDineInCartItem.Menu_price = +(selectedVariant.orginal_price - discount);
                        dbDineInCartItem.Total = dbDineInCartItem.quantity * (dbDineInCartItem.Menu_price + dbDineInCartItem.Addon_price);
                        myApp.getAppDatabase().dineinCartItemDao().update(dbDineInCartItem);
                    }

                }
                if (getActivity() != null) {
                    getActivity().runOnUiThread(() -> {
                        menuAdapter.notifyItemChanged(position);
                        updateCheckoutLayout();
                    });
                }
            }).start();
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    private DialogDismissListener addItemCartDialogListener = new DialogDismissListener() {
        @Override
        public void onDialogDismiss(Object o) {
            try {
                new Thread(() -> {
                    int resId = myApp.getAppDatabase().cartDao().getRestaurantId();
                    if (!isDineIn) {
                        if (resId == restaurant.id) {
                            myApp.getMyPreferences().saveOrderRestaurant(restaurant);
                        } else {
                            myApp.getMyPreferences().deleteOrderRestaurant();
                        }
                    }
                    updateCheckoutLayout();
                }).start();
                if (getActivity() != null) {
                    getActivity().runOnUiThread(() -> {
                        menuAdapter.notifyDataSetChanged();
                    });
                }
            } catch (Exception e) {
                e.printStackTrace();
            }
        }
    };

    private void updateCheckoutLayout() {
        try {
            if (!isDineIn && myApp.getMyPreferences().getOrderRestaurant() != null && myApp.getMyPreferences().getOrderRestaurant().id !=restaurant.id) {
                llCheckout.setVisibility(View.GONE);
            } else {
                new Thread(() -> {
                    CartSummary cartSummary;
                    if (isDineIn) {
                        cartSummary = myApp.getAppDatabase().dineinCartItemDao().getCartSummary();
                    } else {
                        cartSummary = myApp.getAppDatabase().cartDao().getCartSummary();
                    }
                    if (getActivity() != null) {
                        getActivity().runOnUiThread(() -> {
                            if (cartSummary != null && cartSummary.items > 0) {
                                llCheckout.setVisibility(View.VISIBLE);
                                tvTotal.setText("Total: " + myApp.getCurrencySymbol() + MyApp.df.format(cartSummary.total));
                                if (!isDineIn) {
                                    myApp.getMyPreferences().saveOrderRestaurant(restaurant);
                                }
                            } else {
                                llCheckout.setVisibility(View.GONE);
                            }
                        });

                    }
                    CommonFunctions.notifyCartCount(getContext());
                }).start();
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
    }


    private void makeRestaurantFavourite() {
        try {
            ApiUtils.getFavouriteAction(String.valueOf(loggedInUser.id), String.valueOf(restaurant.id), (restaurant.is_favourite==1) ? "1" : "0")
                    .getAsJSONObject(new JSONObjectRequestListener() {
                        @Override
                        public void onResponse(JSONObject response) {
                              if (getActivity() != null)
                                    getActivity().setResult(Activity.RESULT_OK);

                        }

                        @Override
                        public void onError(ANError anError) {
                            anError.printStackTrace();
                            LogUtils.e("ERROR WHILE FAVORITE :: ", anError.getErrorBody());
                            if (!CommonFunctions.isConnected(getActivity()))
                                myApp.noInternet(getActivity());
                        }
                    });
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    private void updateFavouriteUI() {
//        restaurant = RestaurantDetailsActivity.openedRestaurant;
        if (restaurant != null && restaurant.is_favourite==1) {
            ivFavourite.playAnimation();
            ivFavouriteImage.setImageResource(R.drawable.ic_baseline_favorite_24);
        } else {
            ivFavourite.setProgress(0);
            ivFavourite.cancelAnimation();
            ivFavouriteImage.setImageResource(R.drawable.icon_favourite);
        }
    }

    private void fetchData() {
        if (restaurant.menusDetails == null || restaurant.menusDetails.size() == 0)
            new Thread(this::fetchRestaurantDetails).start();
        else setUpAdapters();
    }

    private void fetchRestaurantDetails() {
        try {
            if (getActivity() != null)
                getActivity().runOnUiThread(() -> lodingView.setVisibility(View.VISIBLE));
            ANRequest anRequest;
            if (!isDineIn)
            {
                anRequest = ApiUtils.getRestaurantDetails(String.valueOf(restaurant.id), (MyApp.getInstance().getMyPreferences().getLoggedInUser() == null)  ? "": String.valueOf(loggedInUser.id),myApp.getMyPreferences().getCurrentPostcode().latitude,myApp.getMyPreferences().getCurrentPostcode().longitude);

            } else
            {
                anRequest = ApiUtils.getDineInRestaurant(qrCode, currentPostcode.post_code);
            }
            anRequest
                    .getAsJSONObject(new JSONObjectRequestListener() {
                        @Override
                        public void onResponse(JSONObject response) {
                            restaurant = new Gson().fromJson(String.valueOf(response), Restaurant.class);

                             if (isDineIn)
                                fetchDineInategoryAndMenus();
                            else fetchCategoryAndMenus();
                            fetchTimeSlots();
                            fetchReviews();


                        }

                        @Override
                        public void onError(ANError anError) {
                            anError.printStackTrace();
                            if (getActivity() != null) {
                                getActivity().runOnUiThread(() -> {
                                    lodingView.setVisibility(View.GONE);
                                });
                            }
                            if (!CommonFunctions.isConnected(getActivity()))
                                myApp.noInternet(getActivity());
                        }
                    });
        } catch (Exception e) {
            e.printStackTrace();
        }
    }
    private void fetchCategoryAndMenus() {
        try {
            ApiUtils.getMenuDetails("1", String.valueOf(restaurant.id), "1")
                    .getAsJSONArray(new JSONArrayRequestListener() {
                        @Override
                        public void onResponse(JSONArray response) {

                            Type type = new TypeToken<List<MenuItemNew>>() {
                            }.getType();
                            restaurant.new_restaurant_menu=new ArrayList<>();
                            ArrayList<MenuItemNew> menuItems = new ArrayList<>(new Gson().fromJson(response.toString(), type));

                            try {
                                restaurant.new_restaurant_menu.addAll(menuItems);
                                //  MyApp.menu_image_path = restaurant.restaurant_menus.get(0).image_url;
                                initlizeAlergies();
                                setUpLessAllergies();
    //                            menuAdapter.showMenuImage = restaurant.image_type.equalsIgnoreCase("yes");
                                restaurant_menu.clear();
                                for (MenuItemNew menuItem : restaurant.new_restaurant_menu) {
                                    for (Menu menu : menuItem.menu) {
                                        restaurant_menu.add(menu);
                                    }
                                }
                                updateViews();
                                setUpAdapters();
                            } catch (Exception e) {
                                e.printStackTrace();
                            }


                        }

                        @Override
                        public void onError(ANError anError) {
                            anError.printStackTrace();
                        }
                    });
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    private void fetchReviews() {
        try {
            restaurant.reviews.clear();
            AndroidNetworking.get(ApiEndPoints.RESTAURANTS_REVIEWS)
                    .addQueryParameter("nopaginate", "1")
                    .addQueryParameter("restaurant_id", String.valueOf(restaurant.id))
                    .build().getAsJSONArray(new JSONArrayRequestListener() {
                @Override
                public void onResponse(JSONArray response) {
                     Type type = new TypeToken<List<Review>>() {
                    }.getType();
                    ArrayList<Review> reviews = new ArrayList<>(new Gson().fromJson(response.toString(), type));
                    try {
                        restaurant.reviews.addAll(reviews);
                        updateViews();
                        setUpAdapters();

                        if (getActivity() != null) {
                            getActivity().runOnUiThread(() -> {
                                lodingView.setVisibility(View.GONE);
                            });
                        }
                    } catch (Exception e) {
                        e.printStackTrace();
                    }


                }

                @Override
                public void onError(ANError anError) {
                    anError.printStackTrace();
                }
            });
        } catch (Exception e) {
            e.printStackTrace();
        }
    }
    private void fetchDineInategoryAndMenus() {
        try {
            ApiUtils.get_Dine_in_MenuDetails("1", String.valueOf(restaurant.id), "1")
                    .getAsJSONArray(new JSONArrayRequestListener() {
                        @Override
                        public void onResponse(JSONArray response) {
                            Type type = new TypeToken<List<MenuItemNew>>() {
                            }.getType();
                            restaurant.new_dinein_restaurant_menus=new ArrayList<>();
                            ArrayList<MenuItemNew> menuItems = new ArrayList<>(new Gson().fromJson(response.toString(), type));

                            try {
                                restaurant.new_dinein_restaurant_menus.addAll(menuItems);
                                //  MyApp.menu_image_path = restaurant.restaurant_menus.get(0).image_url;
                                initlizeAlergies();
                                setUpLessAllergies();
    //                            menuAdapter.showMenuImage = restaurant.image_type.equalsIgnoreCase("yes");
                                dinein_restaurant_menu.clear();
                                for (MenuItemNew menuItem : restaurant.new_dinein_restaurant_menus) {
                                     for (Menu menu : menuItem.menu) {
                                        dinein_restaurant_menu.add(menu);
                                    }
                                }

                                updateViews();
                                setUpAdapters();
                            } catch (Exception e) {
                                e.printStackTrace();
                            }


                        }

                        @Override
                        public void onError(ANError anError) {
                            anError.printStackTrace();
                        }
                    });
        } catch (Exception e) {
            e.printStackTrace();
        }
    }
    private void setUpAdapters() {
        try {
            if (getActivity() != null)
                getActivity().runOnUiThread(() -> lodingView.setVisibility(View.GONE));
            feed.clear();
            categories.clear();
            suggested.clear();

            feed.add(new Header("Menu"));
            Allergy allergy = new Allergy(allergies);
            feed.add(allergy);

            //    feed.add(new AllergyCaution("Please be advised that certain items from this restaurant contain the following allergens.", 0));
            feed.add(new AllergyCaution("Please contact the restaurant directly if you require further information about allergens", R.drawable.ic_outline_info_24));

            if (isDineIn) {
                for (Menu menuItem : dinein_restaurant_menu) {
                    if (menuItem.featured.equalsIgnoreCase("yes")) {
                        menuItem.category_id = 0;
                        suggested.add(menuItem);
                    }
                }
            } else {
                for (Menu menu : restaurant_menu)
                    if (menu.featured.equalsIgnoreCase("yes")) {
                        menu.category_id = 0;
                        suggested.add(menu);
                    }
            }
            if (suggested.size() > 0) {
                feed.add(new Category("Recommended", 0));
                feed.addAll(suggested);
                Category c = new Category();
                c.category_name = "Recommended";
                c.category_id = 0;
                categories.add(c);
            }
            if (isDineIn) {
                if (restaurant.new_dinein_restaurant_menus != null && restaurant.new_dinein_restaurant_menus.size() > 0) {
                    for (int i = 0; i < restaurant.new_dinein_restaurant_menus.size(); i++) {
                        Category c = new Category();
                        c.category_name = restaurant.new_dinein_restaurant_menus.get(i).category_name;
                        c.category_id = restaurant.new_dinein_restaurant_menus.get(i).id;
                        categories.add(c);
                        feed.add(c);
                        feed.addAll(restaurant.new_dinein_restaurant_menus.get(i).menu);
                        c.menu = restaurant.new_dinein_restaurant_menus.get(i).menu;
                        menucategoryArrayList.add(c);

                    }

                    if (getActivity() != null) {
                        getActivity().runOnUiThread(() -> {
                            categoryAdapter.notifyDataSetChanged();
                            menuAdapter.notifyDataSetChanged();
                        });
                    }
                }
            } else {
                if (restaurant.new_restaurant_menu != null && restaurant.new_restaurant_menu.size() > 0) {
                    for (int i = 0; i < restaurant.new_restaurant_menu.size(); i++) {
                        Category c = new Category();
                        c.category_name = restaurant.new_restaurant_menu.get(i).category_name;
                        c.category_id = restaurant.new_restaurant_menu.get(i).id;
                        categories.add(c);
                        feed.add(c);
                        feed.addAll(restaurant.new_restaurant_menu.get(i).menu);
                        c.menu = restaurant.new_restaurant_menu.get(i).menu;
                        menucategoryArrayList.add(c);

                    }

                    if (getActivity() != null) {
                        getActivity().runOnUiThread(() -> {
                            categoryAdapter.notifyDataSetChanged();
                            menuAdapter.notifyDataSetChanged();
                        });
                    }
                }
           /* if (CategoryMenuDetail != null) {
                for (Category category :CategoryMenuDetail) {
                    categories.add(category);
                    feed.add(category);
                    feed.addAll(category.menu);

                    if (getActivity() != null) {
                        getActivity().runOnUiThread(() -> {
                            categoryAdapter.notifyDataSetChanged();
                            menuAdapter.notifyDataSetChanged();
                        });
                    }
                }
            }*/
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    private void saveCartOnline() {
        try {
            ArrayList<CartItem> cartItems = new ArrayList<>(myApp.getAppDatabase().cartDao().getAll());
            for (CartItem c : cartItems) {
                UploadCartItems uploadCartItems = new UploadCartItems();
                uploadCartItems.subaddons_name = c.Addon_name;//
                uploadCartItems.menu_price = c.Menu_price;//
                uploadCartItems.menu_id = c.menu_id;//
                uploadCartItems.menu_name = c.menu_name;//
                uploadCartItems.quantity = c.quantity;//
                uploadCartItems.restaurant_id = c.res_id;//
                uploadCartItems.total_price = c.Total;
                uploadCartItems.customer_id = loggedInUser.id;
                cartItemArrayList.add(uploadCartItems);
            }
            CommonFunctions.updateCartOnline(loggedInUser.id, new Gson().toJson(cartItemArrayList));
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    @Override
    public void onActivityResult(int requestCode, int resultCode, @Nullable Intent data) {
        super.onActivityResult(requestCode, resultCode, data);
        if (requestCode == CODE_REFRESH) {
            menuAdapter.notifyDataSetChanged();
            updateCheckoutLayout();
        }
    }

    @Override
    public void onResume() {
        super.onResume();
        updateFavouriteUI();
    }
}
