package com.tiffintom.fragment;

import android.os.Bundle;
import android.util.Log;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;

import androidx.annotation.NonNull;
import androidx.annotation.Nullable;
import androidx.recyclerview.widget.GridLayoutManager;
import androidx.recyclerview.widget.LinearLayoutManager;
import androidx.recyclerview.widget.RecyclerView;

import com.tiffintom.MyApp;
import com.tiffintom.R;
import com.tiffintom.activity.RestaurantDetailsActivity;
import com.tiffintom.adapters.ExploreNearByRestaurantAdapter;
import com.tiffintom.adapters.OffersAdapter;
import com.tiffintom.base.BaseFragment;
import com.tiffintom.common.CommonFunctions;
import com.tiffintom.models.DayOffer;
import com.tiffintom.models.DayPriceOffer;
import com.tiffintom.models.DayProductOffer;
import com.tiffintom.models.Menu;
import com.tiffintom.models.MenuItem;
import com.tiffintom.models.MenuItemNew;
import com.tiffintom.models.MultiplePriceOffer;
import com.tiffintom.models.Offer;
import com.tiffintom.models.OfferSimplify;
import com.tiffintom.models.PriceOffer;
import com.tiffintom.models.ProductOffer;
import com.tiffintom.models.Restaurant;
import com.tiffintom.models.User;

import java.text.DecimalFormat;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;

import static com.tiffintom.activity.RestaurantDetailsActivity.isDineIn;

/**
 * Created by Amrish on 19-08-2020.
 */
public class RestaurantOffersFragment extends BaseFragment {
    public static RestaurantOffersFragment getInstance() {
        RestaurantOffersFragment fragment = new RestaurantOffersFragment();
        Bundle bundle = new Bundle();
        fragment.setArguments(bundle);
        return fragment;
    }

    private RecyclerView rvRestaurant;
    private ExploreNearByRestaurantAdapter restaurantAdapter;
    private ArrayList<Restaurant> categories = new ArrayList<>();
    private Restaurant openedRestaurant = RestaurantDetailsActivity.openedRestaurant;
    private ArrayList<MenuItemNew> menuItems = openedRestaurant.new_restaurant_menu;
    private ArrayList<Menu> restaurant_menu = new ArrayList<>();
    private ArrayList<OfferSimplify> offers = new ArrayList<>();
    private RecyclerView rvOffers;
    private OffersAdapter adapter;
    private User loggedInUser = myApp.getMyPreferences().getLoggedInUser();

    @Nullable
    @Override
    public View onCreateView(@NonNull LayoutInflater inflater, @Nullable ViewGroup container, @Nullable Bundle savedInstanceState) {
        return inflater.inflate(R.layout.fragment_restaurant_offers, container, false);
    }

    @Override
    public void onViewCreated(@NonNull View view, @Nullable Bundle savedInstanceState) {
        if (isDineIn) {
            if (openedRestaurant != null && openedRestaurant.new_dinein_restaurant_menus != null) {
            for (MenuItemNew menuItem : openedRestaurant.new_dinein_restaurant_menus) {
                restaurant_menu.addAll(menuItem.menu);
            }
        }
        } else {
            if (openedRestaurant != null && openedRestaurant.new_restaurant_menu != null) {
                for (MenuItemNew menuItem : openedRestaurant.new_restaurant_menu) {
                    restaurant_menu.addAll(menuItem.menu);

                }
            }

        }
        super.onViewCreated(view, savedInstanceState);
    }

    @Override
    protected void initViews(View view) {
        super.initViews(view);
        try {
            rvOffers = view.findViewById(R.id.rvOffers);
            adapter = new OffersAdapter(getActivity(), offers, true);
            GridLayoutManager gridLayoutManager = new GridLayoutManager(getActivity(), 2);
            LinearLayoutManager linearLayoutManager = new LinearLayoutManager(getActivity(), RecyclerView.VERTICAL, false);
            rvOffers.setLayoutManager(gridLayoutManager);
            rvOffers.setAdapter(adapter);
            gridLayoutManager.setSpanSizeLookup(new GridLayoutManager.SpanSizeLookup() {
                @Override
                public int getSpanSize(int position) {
                    if ((position + 1) % 3 == 0) {
                        return 2;
                    } else {
                        return 1;
                    }
                }
            });
            setUpOffers();
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    private void restaurantClick(int position, Object data) {

    }

    private void setUpOffers() {
        try {
            offers.clear();
            SimpleDateFormat sdf = new SimpleDateFormat("EEEE");
            SimpleDateFormat dateFormat = new SimpleDateFormat("yyyy-MM-dd");
            Date d = new Date();
            String dayOfTheWeek = sdf.format(d);
            Date today = null;
            try {
                today = dateFormat.parse(dateFormat.format(new Date()));
            } catch (ParseException e) {
                e.printStackTrace();
            }

            if (openedRestaurant.reward_option.equalsIgnoreCase("yes")) {
                if (openedRestaurant.multiple_price_offers != null)
                    for (MultiplePriceOffer priceOffer : openedRestaurant.multiple_price_offers) {
                        try {

                        StringBuilder offerTitle = new StringBuilder();
                        StringBuilder offerSubTitle = new StringBuilder();
                        offerTitle.append("Get ").append(priceOffer.offer_qty).append(" selective product(s) free");
                        offerTitle.append(" on ").append(myApp.getCurrencySymbol()).append(MyApp.df.format(priceOffer.purchase_price));

                        offerSubTitle.append("This offer is valid only on ");
                        if (priceOffer.delivery_type.equalsIgnoreCase("1")) {
                            offerSubTitle.append(" delivery,");
                        }
                        if (priceOffer.pickup_type.equalsIgnoreCase("1")) {
                            offerSubTitle.append(" pickup,");
                        }
                        if (priceOffer.dinein_type) {
                            offerSubTitle.append(" dine-in,");
                        }

                        offerSubTitle.deleteCharAt(offerSubTitle.lastIndexOf(","));
                        offerSubTitle.append(" orders");
                        OfferSimplify offerSimplify = new OfferSimplify(offerTitle.toString(), offerSubTitle.toString());
                        offers.add(offerSimplify);

                        }catch(Exception e)
                        {
                            e.printStackTrace();
                        }
                    }
                // Set up day offers
                if (openedRestaurant.day_offers != null)
                    for (DayOffer dayOffer : openedRestaurant.day_offers) {

                        if (!dayOfTheWeek.equalsIgnoreCase(dayOffer.day)) {
                            break;
                        }
                        if (dayOffer.status == false) {
                            break;
                        }
                        StringBuilder offerTitle = new StringBuilder();
                        StringBuilder offerSubTitle = new StringBuilder();
                        if (dayOffer.day_offer_type.equalsIgnoreCase("1")) {
                            offerTitle.append(dayOffer.offer_amount).append("%");
                        } else {
                            offerTitle.append("Flat ").append(myApp.getCurrencySymbol()).append(dayOffer.offer_amount);
                        }
                        offerTitle.append(" OFF on ").append(myApp.getCurrencySymbol()).append(MyApp.df.format(dayOffer.purchase_price));

                        offerSubTitle.append("This offer is valid only on ");
                        if (dayOffer.delivery_type) {
                            offerSubTitle.append(" delivery,");
                        }
                        if (dayOffer.pickup_type) {
                            offerSubTitle.append(" pickup,");
                        }
                        if (dayOffer.dinein_type) {
                            offerSubTitle.append(" dine-in,");
                        }
                        offerSubTitle.deleteCharAt(offerSubTitle.lastIndexOf(","));
                        offerSubTitle.append(" orders");
                        OfferSimplify offerSimplify = new OfferSimplify(offerTitle.toString(), offerSubTitle.toString());
                        offers.add(offerSimplify);
                    }

                if (openedRestaurant.day_product_offers != null)
                    for (DayProductOffer offer : openedRestaurant.day_product_offers) {

                        if (!dayOfTheWeek.equalsIgnoreCase(offer.day_product)) {
                            break;
                        }
                        if (offer.status == false) {
                            break;
                        }
                        StringBuilder offerTitle = new StringBuilder();
                        StringBuilder offerSubTitle = new StringBuilder();
                        offerTitle.append(offer.offer_qty).append(" ").append(findMenuItemById(offer.offer_id).menu_name).append(" free on ").append(offer.product_qty).append(" ").append(findMenuItemById(offer.menu_id).menu_name);

                        offerSubTitle.append("This offer is valid on");
                        if (offer.delivery_type) {
                            offerSubTitle.append(" delivery,");
                        }
                        if (offer.pickup_type) {
                            offerSubTitle.append(" pickup,");
                        }
                        if (offer.dinein_type) {
                            offerSubTitle.append(" dine-in,");
                        }
                        offerSubTitle.deleteCharAt(offerSubTitle.lastIndexOf(","));
                        offerSubTitle.append(" orders");
                        OfferSimplify offerSimplify = new OfferSimplify(offerTitle.toString(), offerSubTitle.toString());
                        offers.add(offerSimplify);
                    }
                if (openedRestaurant.day_price_offers != null)
                    for (DayPriceOffer offer : openedRestaurant.day_price_offers) {

                        if (!dayOfTheWeek.equalsIgnoreCase(offer.day)) {
                            break;
                        }
                        if (offer.status == false) {
                            break;
                        }
                        StringBuilder offerTitle = new StringBuilder();
                        StringBuilder offerSubTitle = new StringBuilder();
                        offerTitle.append(offer.offer_qty).append(" ").append(findMenuItemById(offer.offer_id).menu_name).append(" free on ").append(myApp.getCurrencySymbol()).append(MyApp.df.format(offer.purchase_price)).append(" purchase");

                        offerSubTitle.append("This offer is valid on");
                        if (offer.delivery_type) {
                            offerSubTitle.append(" delivery,");
                        }
                        if (offer.pickup_type) {
                            offerSubTitle.append(" pickup,");
                        }
                        if (offer.dinein_type) {
                            offerSubTitle.append(" dine-in,");
                        }
                        offerSubTitle.deleteCharAt(offerSubTitle.lastIndexOf(","));
                        offerSubTitle.append(" orders");
                        OfferSimplify offerSimplify = new OfferSimplify(offerTitle.toString(), offerSubTitle.toString());
                        offers.add(offerSimplify);
                    }

                if (openedRestaurant.offers != null)
                    for (Offer offer : openedRestaurant.offers) {
                        Date startDate = null, endDate = null;
                        try {
                            startDate = dateFormat.parse(offer.offer_from);
                            endDate = dateFormat.parse(offer.offer_to);

                            if (today.after(endDate)) break;
                            if (today.before(startDate)) break;
                        } catch (ParseException e) {
                            e.printStackTrace();
                        }
                        StringBuilder offerTitle = new StringBuilder();
                        StringBuilder offerSubTitle = new StringBuilder();
                        if (offer.first_user.equalsIgnoreCase("y")) {
                            if (loggedInUser.first_user != null && loggedInUser.first_user.equalsIgnoreCase("yes")) {
                                offerTitle.append(new DecimalFormat("#.##").format(offer.free_percentage)).append("%").append(" OFF on ").append(myApp.getCurrencySymbol()).append(MyApp.df.format(offer.free_price));
                                offerSubTitle.append("This offer is valid for first time user and on");
                                if (offer.delivery_type) {
                                    offerSubTitle.append(" delivery,");
                                }
                                if (offer.pickup_type) {
                                    offerSubTitle.append(" pickup,");
                                }
                                if (offer.dinein_type) {
                                    offerSubTitle.append(" dine-in,");
                                }
                                offerSubTitle.deleteCharAt(offerSubTitle.lastIndexOf(","));
                                offerSubTitle.append(" orders,");
                                offerSubTitle.append(" Offer ends ").append(CommonFunctions.formatDate(offer.offer_to, "dd-MM-yyyy"));
                                OfferSimplify offerSimplify = new OfferSimplify(offerTitle.toString(), offerSubTitle.toString());
                                offers.add(offerSimplify);
                            }
                        }
                        if (offer.normal.equalsIgnoreCase("y")) {
                            offerTitle.append(new DecimalFormat("#.##").format(offer.normal_percentage)).append("%").append(" OFF on ").append(myApp.getCurrencySymbol()).append(MyApp.df.format(offer.normal_price));
                            offerSubTitle.append("This offer is valid on");
                            if (offer.delivery_type) {
                                offerSubTitle.append(" delivery,");
                            }
                            if (offer.pickup_type) {
                                offerSubTitle.append(" pickup,");
                            }
                            if (offer.dinein_type) {
                                offerSubTitle.append(" dine-in,");
                            }
                            offerSubTitle.deleteCharAt(offerSubTitle.lastIndexOf(","));
                            offerSubTitle.append(" order,");
                            offerSubTitle.append(" Offer ends ").append(CommonFunctions.formatDate(offer.offer_to, "dd-MM-yyyy"));
                            OfferSimplify offerSimplify = new OfferSimplify(offerTitle.toString(), offerSubTitle.toString());
                            offers.add(offerSimplify);
                        }


                    }

                if (openedRestaurant.product_offers != null)
                    for (ProductOffer offer : openedRestaurant.product_offers) {
                        Date startDate = null, endDate = null;
                        try {
                            startDate = dateFormat.parse(offer.offer_from);
                            endDate = dateFormat.parse(offer.offer_to);

                            if (today.after(endDate)) break;
                            if (today.before(startDate)) break;
                        } catch (ParseException e) {
                            e.printStackTrace();
                        }

                        if (offer.status == false) {
                            break;
                        }
                        StringBuilder offerTitle = new StringBuilder();
                        StringBuilder offerSubTitle = new StringBuilder();

                        if (findMenuItemById(offer.offer_id)!=null) {
                            offerTitle.append(offer.offer_qty).append(" ").append(findMenuItemById(offer.offer_id).menu_name).append(" free on ").append(offer.product_qty).append(" ").append(findMenuItemById(offer.menu_id).menu_name);
                        }

                        offerSubTitle.append("This offer is valid on");
                        if (offer.delivery_type) {
                            offerSubTitle.append(" delivery,");
                        }
                        if (offer.pickup_type) {
                            offerSubTitle.append(" pickup,");
                        }
                        if (offer.dinein_type) {
                            offerSubTitle.append(" dine-in,");
                        }
                        offerSubTitle.deleteCharAt(offerSubTitle.lastIndexOf(","));
                        offerSubTitle.append(" order,");
                        offerSubTitle.append(" Offer ends ").append(CommonFunctions.formatDate(offer.offer_to, "dd-MM-yyyy"));
                        OfferSimplify offerSimplify = new OfferSimplify(offerTitle.toString(), offerSubTitle.toString());
                        offers.add(offerSimplify);
                    }
                if (openedRestaurant.price_offers != null)
                    for (PriceOffer offer : openedRestaurant.price_offers) {
                        Date startDate = null, endDate = null;
                        try {
                            startDate = dateFormat.parse(offer.offer_from);
                            endDate = dateFormat.parse(offer.offer_to);

                            if (today.after(endDate)) break;
                            if (today.before(startDate)) break;
                        } catch (ParseException e) {
                            e.printStackTrace();
                        }
                        if (offer.status == false) {
                            break;
                        }
                        StringBuilder offerTitle = new StringBuilder();
                        StringBuilder offerSubTitle = new StringBuilder();
                        offerTitle.append(offer.offer_qty).append(" ").append(findMenuItemById(offer.offer_id).menu_name).append(" free on ").append(myApp.getCurrencySymbol()).append(MyApp.df.format(offer.purchase_price)).append(" purchase");

                        offerSubTitle.append("This offer is valid on");
                        if (offer.delivery_type) {
                            offerSubTitle.append(" delivery,");
                        }
                        if (offer.pickup_type) {
                            offerSubTitle.append(" pickup,");
                        }
                        if (offer.dinein_type) {
                            offerSubTitle.append(" dine-in,");
                        }
                        offerSubTitle.deleteCharAt(offerSubTitle.lastIndexOf(","));
                        offerSubTitle.append(" order,");
                        offerSubTitle.append(" Offer ends ").append(CommonFunctions.formatDate(offer.offer_to, "dd-MM-yyyy"));
                        OfferSimplify offerSimplify = new OfferSimplify(offerTitle.toString(), offerSubTitle.toString());
                        offers.add(offerSimplify);
                    }

            }
            if (myApp.getMyPreferences().getRewardPoint() != null && myApp.getMyPreferences().getRewardPoint().reward_option.equalsIgnoreCase("yes") && openedRestaurant.reward_option.equalsIgnoreCase("yes")) {
                offers.add(new OfferSimplify("Earn loyalty Points", "Earn loyalty points on your order"));
            }
            if (myApp.getMyPreferences().getReferral() != null && myApp.getMyPreferences().getReferral().referral_option.equalsIgnoreCase("yes")) {
                offers.add(new OfferSimplify("Recommend a friend", String.format("Receive %s reward points when you recommend a friend and they place an order with us", myApp.getMyPreferences().getReferral().invite_amount)));
            }
            adapter.notifyDataSetChanged();
        } catch (Exception e) {
            e.printStackTrace();
        }

    }

    private Menu findMenuItemById(int id) {
        if (id != -1) {
            Menu toBeFineMenu = new Menu();
            toBeFineMenu.id = id;
            if (restaurant_menu != null) {
                int menuIndex = restaurant_menu.indexOf(toBeFineMenu);
                if (menuIndex != -1) {
                    return restaurant_menu.get(menuIndex);
                } else {
                    return null;
                }
            } else {
                return null;
            }
        } else {
            return null;
        }
    }
}
