package com.tiffintom.fragment;

import android.os.Bundle;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.TextView;

import androidx.annotation.NonNull;
import androidx.annotation.Nullable;
import androidx.recyclerview.widget.LinearLayoutManager;
import androidx.recyclerview.widget.RecyclerView;

import com.iarcuschin.simpleratingbar.SimpleRatingBar;
import com.tiffintom.R;
import com.tiffintom.activity.RestaurantDetailsActivity;
import com.tiffintom.adapters.RestaurantReviewsAdapter;
import com.tiffintom.base.BaseFragment;
import com.tiffintom.models.Restaurant;
import com.tiffintom.models.Review;

import java.text.DecimalFormat;
import java.util.ArrayList;

/**
 * Created by Amrish on 19-08-2020.
 */
public class RestaurantReviewsFragment extends BaseFragment {
    public static RestaurantReviewsFragment getInstance() {
        RestaurantReviewsFragment fragment = new RestaurantReviewsFragment();
        Bundle bundle = new Bundle();
        fragment.setArguments(bundle);
        return fragment;
    }

    private RecyclerView rvReviews;
    private RestaurantReviewsAdapter reviewsAdapter;
    private ArrayList<Review> reviews = new ArrayList<>();
    private Restaurant restaurant = RestaurantDetailsActivity.openedRestaurant;
    private SimpleRatingBar ratingBar;
    private TextView tvAverageRating, tvRatingInfo;

    @Nullable
    @Override
    public View onCreateView(@NonNull LayoutInflater inflater, @Nullable ViewGroup container, @Nullable Bundle savedInstanceState) {
        return inflater.inflate(R.layout.fragment_reviews, container, false);
    }

    @Override
    public void onViewCreated(@NonNull View view, @Nullable Bundle savedInstanceState) {
        super.onViewCreated(view, savedInstanceState);
        updateViews();
    }

    @Override
    protected void initViews(View view) {
        super.initViews(view);
        try {
            rvReviews = view.findViewById(R.id.rvNearBy);
            tvAverageRating = view.findViewById(R.id.tvAverageRating);
            tvRatingInfo = view.findViewById(R.id.tvRatingInfo);
            ratingBar = view.findViewById(R.id.rating);
            reviewsAdapter = new RestaurantReviewsAdapter(getActivity(), reviews, this::reviewClick);
            rvReviews.setLayoutManager(new LinearLayoutManager(getActivity(), RecyclerView.VERTICAL, false));
            rvReviews.setAdapter(reviewsAdapter);
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    private void updateViews() {
        try {
            if (restaurant != null && restaurant.reviews != null) {
                reviews.clear();
                reviews.addAll(restaurant.reviews);
                reviewsAdapter.notifyDataSetChanged();
                DecimalFormat decimalFormat = new DecimalFormat("#.#");
                tvAverageRating.setText(decimalFormat.format(restaurant.average_rating) + "/5");
                ratingBar.setRating(restaurant.average_rating);
                if (restaurant.reviews != null) {
                    tvRatingInfo.setText(String.format("Based on %d reviews from customers", restaurant.reviews.size()));
                } else {
                    tvRatingInfo.setText("No rating yet");
                }
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    private void reviewClick(int position, Object data) {

    }
}
