package com.tiffintom.fragment;

import android.content.Intent;
import android.net.Uri;
import android.os.Bundle;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.LinearLayout;
import android.widget.ProgressBar;
import android.widget.TextView;

import androidx.annotation.NonNull;
import androidx.annotation.Nullable;

import com.facebook.CallbackManager;
import com.facebook.share.model.ShareLinkContent;
import com.facebook.share.widget.ShareDialog;
import com.google.android.material.floatingactionbutton.FloatingActionButton;
import com.tiffintom.BuildConfig;
import com.tiffintom.R;
import com.tiffintom.activity.TransportActivity;
import com.tiffintom.base.BaseFragment;
import com.tiffintom.common.Validators;
import com.tiffintom.models.User;
import com.tiffintom.utils.ToastUtils;

/**
 * Created by Amrish on 19-08-2020.
 */
public class RewardsFragment extends BaseFragment {
    public static RewardsFragment getInstance() {
        RewardsFragment exploreCategoryFragment = new RewardsFragment();
        Bundle bundle = new Bundle();
        exploreCategoryFragment.setArguments(bundle);
        return exploreCategoryFragment;
    }

    private TextView tvReferralCode, tvRewardPoints, tvWebsite, tvRewardPointsInfo;
    private User loggedInUser;
    private FloatingActionButton fabFacebook, fabWhatsapp, fabShare;
    public String message;
    private ShareDialog shareDialog;
    private CallbackManager callbackManager;
    private ProgressBar pbPoints;
    private LinearLayout llReferralHistory, llReferralWorks;
    @Nullable
    @Override
    public View onCreateView(@NonNull LayoutInflater inflater, @Nullable ViewGroup container, @Nullable Bundle savedInstanceState) {
        return inflater.inflate(R.layout.fragment_rewards, container, false);
    }

    @Override
    public void onViewCreated(@NonNull View view, @Nullable Bundle savedInstanceState) {
        super.onViewCreated(view, savedInstanceState);

        callbackManager = CallbackManager.Factory.create();
        shareDialog = new ShareDialog(getActivity());
        updateViews();
        setListeners();
    }

    @Override
    protected void initViews(View view) {
        super.initViews(view);
        loggedInUser = myApp.getMyPreferences().getLoggedInUser();
        tvReferralCode = view.findViewById(R.id.tvReferralCode);
        tvRewardPoints = view.findViewById(R.id.tvRewardPoints);
        tvWebsite = view.findViewById(R.id.tvWebsite);
        pbPoints = view.findViewById(R.id.pbPoints);
        tvRewardPointsInfo = view.findViewById(R.id.tvRewardPointsInfo);
        llReferralHistory = view.findViewById(R.id.llReferralHistory);
        llReferralWorks = view.findViewById(R.id.llHowReferralWorks);
        fabFacebook = view.findViewById(R.id.fabFacebook);
        fabWhatsapp = view.findViewById(R.id.fabWhatsapp);
        fabShare = view.findViewById(R.id.fabShare);

    }
    private void updateViews() {
        if (loggedInUser != null) {
            tvReferralCode.setText("Use code: " + loggedInUser.referral_code);
            tvRewardPoints.setText("You have " + loggedInUser.total_earn_point + " Points");
            if (!Validators.isNullOrEmpty(String.valueOf(loggedInUser.total_earn_point))) {
                if (loggedInUser.total_earn_point >= 100) {
                    pbPoints.setProgress(100);
                    tvRewardPointsInfo.setText("Get 10% off on loyalty Points redemption");
                } else {
                    pbPoints.setProgress(loggedInUser.total_earn_point);
                    tvRewardPointsInfo.setText("Earn " + (100 -loggedInUser.total_earn_point + " Points more to get 10% off"));
                }
            } else {
                pbPoints.setProgress(0);
            }
        }
    }

    private void setListeners() {

        message = "Signup with my referral code " + loggedInUser.referred_by + " on TiffinTom App.\n" +
                "iOS: https://apps.apple.com/in/app/tiffintom/id1437199436\n" +
                "Android: https://play.google.com/store/apps/details?id=" + BuildConfig.APPLICATION_ID;

        tvWebsite.setOnClickListener(view -> {
            startActivity(new Intent(Intent.ACTION_VIEW, Uri.parse(getString(R.string.complete_website))));
        });
        llReferralWorks.setOnClickListener(view -> {
            if (BuildConfig.DEBUG)
                startActivity(new Intent(getActivity(), TransportActivity.class).putExtra("destination", "webview").putExtra("url", "https://www.tiffintom.net/users/referralWorks"));
            else
                startActivity(new Intent(getActivity(), TransportActivity.class).putExtra("destination", "webview").putExtra("url", "https://www.tiffintom.com/users/referralWorks"));
        });
        fabFacebook.setOnClickListener(view -> {

            ShareToFaceBook();
        });
        fabWhatsapp.setOnClickListener(view -> {

            Intent whatsappIntent = new Intent(Intent.ACTION_SEND);
            whatsappIntent.setType("text/plain");
            whatsappIntent.setPackage("com.whatsapp");
            whatsappIntent.putExtra(Intent.EXTRA_TEXT, message);
            try {
                startActivity(whatsappIntent);
            } catch (android.content.ActivityNotFoundException ex) {
                ToastUtils.makeToast(getActivity(), "WhatsApp is not installed.");
            }
        });
        fabShare.setOnClickListener(view -> {
            Intent share = new Intent(Intent.ACTION_SEND);
            share.setType("text/plain");
            share.putExtra(Intent.EXTRA_TEXT, message);
            startActivity(Intent.createChooser(share, "Share using"));
        });
        llReferralHistory.setOnClickListener(view -> {
            startActivity(new Intent(getActivity(), TransportActivity.class).putExtra("destination", "referral_history").putExtra("name", "Referral History"));
        });
    }

    private void ShareToFaceBook() {
        if (ShareDialog.canShow(ShareLinkContent.class)) {
            ShareLinkContent linkContent = new ShareLinkContent.Builder()
                    .setContentUrl(Uri.parse("https://play.google.com/store/apps/details?id=" + BuildConfig.APPLICATION_ID))
                    .setQuote(message)
                    .build();
            shareDialog.show(linkContent);
        }
    }

    @Override
    public void onActivityResult(int requestCode, int resultCode, @Nullable Intent data) {
        super.onActivityResult(requestCode, resultCode, data);
        callbackManager.onActivityResult(requestCode, resultCode, data);
    }
}
