package com.tiffintom.fragment;

import android.app.Activity;
import android.content.Intent;
import android.os.Bundle;
import android.text.Editable;
import android.text.TextWatcher;
import android.view.KeyEvent;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.EditText;
import android.widget.ImageView;
import android.widget.LinearLayout;
import android.widget.RelativeLayout;
import android.widget.TextView;

import androidx.annotation.NonNull;
import androidx.annotation.Nullable;
import androidx.recyclerview.widget.LinearLayoutManager;
import androidx.recyclerview.widget.RecyclerView;

import com.androidnetworking.error.ANError;
import com.androidnetworking.interfaces.JSONArrayRequestListener;
import com.androidnetworking.interfaces.JSONObjectRequestListener;
import com.google.gson.Gson;
import com.google.gson.reflect.TypeToken;
import com.tiffintom.MyApp;
import com.tiffintom.R;
import com.tiffintom.activity.AuthActivity;
import com.tiffintom.activity.TransportActivity;
import com.tiffintom.adapters.RestaurantMenuAdapter;
import com.tiffintom.base.BaseFragment;
import com.tiffintom.common.CommonFunctions;
import com.tiffintom.common.Validators;
import com.tiffintom.interfaces.DialogDismissListener;
import com.tiffintom.models.CartItem;
import com.tiffintom.models.CartSummary;
import com.tiffintom.models.Category;
import com.tiffintom.models.DineinCartItem;
import com.tiffintom.models.Header;
import com.tiffintom.models.Menu;
import com.tiffintom.models.MenuItem;
import com.tiffintom.models.MenuItemNew;
import com.tiffintom.models.Restaurant;
import com.tiffintom.models.TimeSlots;
import com.tiffintom.models.UploadCartItems;
import com.tiffintom.models.User;
import com.tiffintom.models.Variant;
import com.tiffintom.network.ApiUtils;
import com.tiffintom.utils.LogUtils;

import java.lang.reflect.Type;
import java.util.ArrayList;
import java.util.List;

import static com.tiffintom.activity.RestaurantDetailsActivity.guest_count;
import static com.tiffintom.activity.RestaurantDetailsActivity.isDineIn;
import static com.tiffintom.activity.RestaurantDetailsActivity.openedRestaurant;
import static com.tiffintom.activity.RestaurantDetailsActivity.qrCode;
import static com.tiffintom.activity.RestaurantDetailsActivity.tableNumber;
import static com.tiffintom.utils.Constants.CODE_REFRESH;

import org.json.JSONArray;
import org.json.JSONException;
import org.json.JSONObject;

/**
 * Created by Amrish on 16-09-2020.
 */
public class SearchMenuFragment extends BaseFragment {
    public static SearchMenuFragment getInstance(String screenTitle) {
        SearchMenuFragment fragment = new SearchMenuFragment();
        Bundle bundle = new Bundle();
        bundle.putString("name", screenTitle);
        fragment.setArguments(bundle);
        return fragment;
    }

    private EditText etSearch;
    private RecyclerView rvMenu;
    private RestaurantMenuAdapter menuAdapter;
    private ArrayList<Object> feed = new ArrayList<>();
    private ArrayList<Object> filtered = new ArrayList<>();
    private ArrayList<Category> categories = new ArrayList<>();
    private ArrayList<Menu> suggested = new ArrayList<>();
    private Restaurant restaurant = MyApp.openedRestaurant;
    private ImageView ivBack;
    private LinearLayout llBackArrow;
    private User loggedInUser = myApp.getMyPreferences().getLoggedInUser();
    private RelativeLayout llCheckout;
    private TextView tvScreenTitle, tvCheckout, tvTotal;
    private String name;
    private ArrayList<Menu> restaurant_menu = new ArrayList<>();
    private ArrayList<Menu> dinein_restaurant_menu  = new ArrayList<>();
    private ArrayList<UploadCartItems> cartItemArrayList  = new ArrayList<>();
    @Nullable
    @Override
    public View onCreateView(@NonNull LayoutInflater inflater, @Nullable ViewGroup container, @Nullable Bundle savedInstanceState) {
        if (getArguments() != null) name = getArguments().getString("name");
        return inflater.inflate(R.layout.fragment_search_menu, container, false);
    }

    @Override
    public void onViewCreated(@NonNull View view, @Nullable Bundle savedInstanceState) {
        super.onViewCreated(view, savedInstanceState);

        if (!Validators.isNullOrEmpty(name)) {
            tvScreenTitle.setText(name);
        } else {
            tvScreenTitle.setVisibility(View.GONE);
        }
        setListeners();
        updateCheckoutLayout();
    }

    @Override
    protected void initViews(View view) {
        super.initViews(view);
        llCheckout = view.findViewById(R.id.llMiniSnippet);
        tvCheckout = view.findViewById(R.id.tvCheckout);
        tvTotal = view.findViewById(R.id.tvTotal);
        tvUserName = view.findViewById(R.id.tvUserName);
        etSearch = view.findViewById(R.id.etSearch);
        rvMenu = view.findViewById(R.id.rvMenu);
        tvScreenTitle = view.findViewById(R.id.tvScreenTitle);
        llBackArrow = view.findViewById(R.id.llBackArrow);
        menuAdapter = new RestaurantMenuAdapter(getActivity(), filtered,  (position, data) -> {
            if (isDineIn) {
                menuItemClick(position, data);
            } else {
                if (openedRestaurant.online_order.equalsIgnoreCase("no")) {
                    showRestaurantIsClosed("Unfortunately, " + restaurant.restaurant_name + " is currently not accepting online orders", false);
                } else {
                    if (isRestaurantClosed())
                        showRestaurantIsClosed("Unfortunately, " + restaurant.restaurant_name + " is currently closed", false);
                    else menuItemClick(position, data);
                }
            }
        });
        rvMenu.setLayoutManager(new LinearLayoutManager(getActivity(), RecyclerView.VERTICAL, false));
        rvMenu.setAdapter(menuAdapter);
        if (isDineIn)
            fetchDineInategoryAndMenus();
        else  fetchCategoryAndMenus();
        fetchTimeSlots();
    }

    private void fetchCategoryAndMenus() {
        ApiUtils.getMenuDetails("1", String.valueOf(restaurant.id), "1")
                .getAsJSONArray(new JSONArrayRequestListener() {
                    @Override
                    public void onResponse(JSONArray response) {

                        Type type = new TypeToken<List<MenuItemNew>>() {
                        }.getType();
                        restaurant.new_restaurant_menu=new ArrayList<>();
                        ArrayList<MenuItemNew> menuItems = new ArrayList<>(new Gson().fromJson(response.toString(), type));

                        try {
                            restaurant.new_restaurant_menu.addAll(menuItems);
                            restaurant_menu.clear();
                            for (MenuItemNew menuItem : restaurant.new_restaurant_menu) {
                                for (Menu menu : menuItem.menu) {
                                    restaurant_menu.add(menu);
                                }
                            }
                            setUpAdapters();
                        } catch (Exception e) {
                            e.printStackTrace();
                        }


                    }

                    @Override
                    public void onError(ANError anError) {
                        anError.printStackTrace();
                    }
                });
    }
    private void fetchDineInategoryAndMenus() {
        ApiUtils.get_Dine_in_MenuDetails("1", String.valueOf(restaurant.id), "1")
                .getAsJSONArray(new JSONArrayRequestListener() {
                    @Override
                    public void onResponse(JSONArray response) {
                        Type type = new TypeToken<List<MenuItemNew>>() {
                        }.getType();
                        restaurant.new_dinein_restaurant_menus=new ArrayList<>();
                        ArrayList<MenuItemNew> menuItems = new ArrayList<>(new Gson().fromJson(response.toString(), type));

                        try {
                            restaurant.new_dinein_restaurant_menus.addAll(menuItems);
//                            menuAdapter.showMenuImage = restaurant.image_type.equalsIgnoreCase("yes");
                            dinein_restaurant_menu.clear();
                            for (MenuItemNew menuItem : restaurant.new_dinein_restaurant_menus) {
                                for (Menu menu : menuItem.menu) {
                                    dinein_restaurant_menu.add(menu);
                                }
                            }
                            setUpAdapters();
                        } catch (Exception e) {
                            e.printStackTrace();
                        }


                    }

                    @Override
                    public void onError(ANError anError) {
                        anError.printStackTrace();
                    }
                });
    }
    private void fetchTimeSlots() {
        ApiUtils.getRestaurantTimeSlots(String.valueOf(restaurant.id), CommonFunctions.getToday())
                .getAsJSONObject(new JSONObjectRequestListener() {
                    @Override
                    public void onResponse(JSONObject response) {
                        try {
                            JSONObject JsonObject=new JSONObject(response.toString());
                            restaurant.delivery_time_slot = new Gson().fromJson(JsonObject.getJSONObject("delivery").toString(), TimeSlots.class);
                            restaurant.pickup_time_slot = new Gson().fromJson(JsonObject.getJSONObject("pickup").toString(), TimeSlots.class);
                            openedRestaurant = restaurant;
                        } catch (JSONException e) {
                            e.printStackTrace();
                        }
                    }

                    @Override
                    public void onError(ANError anError) {
                        anError.printStackTrace();
                    }
                });
    }
    private void setListeners() {
        etSearch.setOnKeyListener((textView, i, keyEvent) -> {
            if (keyEvent.getKeyCode() == KeyEvent.KEYCODE_BACK) {
                getActivity().onBackPressed();
            } else if ((keyEvent.getKeyCode() == KeyEvent.KEYCODE_ENTER)) {
                searchMenu();
                if (getActivity() != null)
                    CommonFunctions.hideKeyboard(getActivity(), etSearch);
            }
            return false;
        });

        etSearch.addTextChangedListener(new TextWatcher() {
            @Override
            public void beforeTextChanged(CharSequence charSequence, int i, int i1, int i2) {

            }

            @Override
            public void onTextChanged(CharSequence charSequence, int i, int i1, int i2) {

            }

            @Override
            public void afterTextChanged(Editable editable) {
                if (editable.toString().length() == 0) {
                    filtered.clear();
                    filtered.addAll(feed);
                    menuAdapter.notifyDataSetChanged();
                } else {
                    searchMenu();
                }
            }
        });
        llBackArrow.setOnClickListener(view1 -> {
            if (getActivity() != null) getActivity().onBackPressed();
        });
        llCheckout.setOnClickListener(view1 -> {
            if (loggedInUser == null) {
                startActivity(new Intent(getActivity(), AuthActivity.class));
            } else {
                if (!isDineIn)
                    validateRestaurantOpeningAndCheckout();
                else openCheckoutScreen();
            }
        });
    }


    private void showRestaurantIsClosed(String message, boolean shouldGoBack) {
        RestaurantClosedDialogFragment restaurantClosedDialogFragment = RestaurantClosedDialogFragment.getInstance(message);
        restaurantClosedDialogFragment.show(getChildFragmentManager(), "restaurant_closed");
        restaurantClosedDialogFragment.setDialogDismissListener(o -> {
            if (shouldGoBack) {
                if (getActivity() != null)
                    getActivity().onBackPressed();
            }
        });
    }

    private void searchMenu() {
        filtered.clear();
        ArrayList<Menu> searchedSuggested = new ArrayList<>();
        for (Menu menuItem : suggested) {
            if (menuItem.menu_name.toLowerCase().contains(etSearch.getText().toString().toLowerCase())) {
                searchedSuggested.add(menuItem);
            }
        }
        if (searchedSuggested.size() > 0) {
            filtered.add(new Category("Recommended", 0));
            filtered.addAll(searchedSuggested);
        }
        for (MenuItemNew c : restaurant.new_restaurant_menu) {
            ArrayList<Menu> menuItems = new ArrayList<>();
            for (Menu menuItem : c.menu) {
                if (menuItem.menu_name.toLowerCase().contains(etSearch.getText().toString().toLowerCase())) {
                    menuItems.add(menuItem);
                }
            }
            if (menuItems.size() > 0) {
//                filtered.add(c);
                filtered.addAll(menuItems);
            }
        }
        menuAdapter.notifyDataSetChanged();
    }

    private void setUpAdapters() {
        feed.clear();
        filtered.clear();
        categories.clear();
        suggested.clear();
        if (isDineIn) {
            for (Menu menuItem :dinein_restaurant_menu ) {
                if (menuItem.featured.equalsIgnoreCase("yes")) {
                    suggested.add(menuItem);
                }
            }
        } else {
            for (Menu menu : restaurant_menu) {
                if (menu.featured.equalsIgnoreCase("yes")) {
                    menu.category_id = 0;
                    suggested.add(menu);
                }
            }
        }
        if (suggested.size() > 0) {
            feed.add(new Category("Recommended", 0));
            feed.addAll(suggested);
        }


        if (restaurant.new_restaurant_menu != null && restaurant.new_restaurant_menu.size() > 0) {
            for (int i = 0; i < restaurant.new_restaurant_menu.size(); i++) {
                Category c = new Category();
                c.category_name = restaurant.new_restaurant_menu.get(i).category_name;
                c.category_id = restaurant.new_restaurant_menu.get(i).id;
                categories.add(c);
                feed.add(c);
                feed.addAll(restaurant.new_restaurant_menu.get(i).menu);
                c.menu = restaurant.new_restaurant_menu.get(i).menu;


            }
        }
        filtered.addAll(feed);

        if (getActivity() != null) {
            getActivity().runOnUiThread(() -> {
                menuAdapter.notifyDataSetChanged();
            });
        }
    }

    private void menuItemClick(int position, Object data) {
        if (data instanceof Menu) {
            if (!isDineIn && myApp.getMyPreferences().getOrderRestaurant() != null && myApp.getMyPreferences().getOrderRestaurant().id != restaurant.id) {
                ConfirmationDialogFragment confirmationDialogFragment = ConfirmationDialogFragment.getInstance("Start new basket?", "Your basket already contains items from " + myApp.getMyPreferences().getOrderRestaurant().restaurant_name + ". Do you wish to clear the basket and add this item instead?");
                confirmationDialogFragment.show(getActivity().getSupportFragmentManager(), "confirm");
                confirmationDialogFragment.setDialogDismissListener(new DialogDismissListener() {
                    @Override
                    public void onDialogDismiss(Object o) {
                        if (o instanceof String && ((String) o).equalsIgnoreCase("confirm")) {
                            new Thread(() -> {
                                myApp.getMyPreferences().deleteOrderRestaurant();
                                myApp.getAppDatabase().cartDao().nukeTable();
                                proceedToAddItemOrViewItem(position, data);
                            }).start();
                        }
                    }
                });
            } else {
                proceedToAddItemOrViewItem(position, data);
            }

        } else if (data instanceof Header) {
            if (((Header) data).addonId == 1) {
                //Search clicked
                startActivity(new Intent(getActivity(), TransportActivity.class).putExtra("destination", "search_menu"));
            }
        }
    }

    private void proceedToAddItemOrViewItem(int position, Object data) {
        if (((Menu) data).price_option.equalsIgnoreCase("multiple") || ((Menu) data).menu_addon.equalsIgnoreCase("yes")) {
            MenuItemDetailBottomsheetFragment menuItemDetailBottomsheetFragment = MenuItemDetailBottomsheetFragment.getInstance((Menu) data, isDineIn);
            menuItemDetailBottomsheetFragment.show(getChildFragmentManager(), "addon");
            menuItemDetailBottomsheetFragment.setDialogDismissListener(addItemCartDialogListener);
        } else {
            if (isDineIn) {
                addItemToDineInCart((Menu) data, position);
            } else {
                addItemToCart((Menu) data, position);
            }
        }
    }

    private void addItemToCart(Menu menuItem, int position) {
        new Thread(() -> {
            String variantName = null;
            float discount = 0;
            if (menuItem.variants != null && menuItem.variants.size() > 0) {
                variantName = menuItem.variants.get(0).sub_name;
                variantName = (menuItem.menu_name.equalsIgnoreCase(variantName)) ? "" : variantName;
                discount = (menuItem.variants.get(0).orginal_price * menuItem.getProductPercentage() / 100);
            }
            CartItem dbCartItem = myApp.getAppDatabase().cartDao().getCartMenuItem(menuItem.id, variantName);
            if (dbCartItem == null) {
                CartItem cartItem = new CartItem();
                if (menuItem.variants != null && menuItem.variants.size() > 0) {
                    Variant selectedVariant = menuItem.variants.get(0);
                    cartItem.quantity = 1;
                    cartItem.Menu_price = selectedVariant.orginal_price - discount;
                    cartItem.menu_id = selectedVariant.menu_id;
                    cartItem.id = selectedVariant.id;
                    cartItem.menu_name = menuItem.menu_name;
                    cartItem.menu_size = (menuItem.menu_name.equalsIgnoreCase(selectedVariant.sub_name)) ? "" : selectedVariant.sub_name;
                    cartItem.res_id = menuItem.restaurant_id;
                    cartItem.menu_type = menuItem.menu_type;

                    cartItem.Addon_name = "";
                    cartItem.Addon_price = 0;
                    cartItem.Total = cartItem.quantity * (cartItem.Menu_price + cartItem.Addon_price);
                    myApp.getAppDatabase().cartDao().insertAll(cartItem);
                }
            } else {
                if (menuItem.variants != null && menuItem.variants.size() > 0) {
                    Variant selectedVariant = menuItem.variants.get(0);
                    dbCartItem.quantity++;
                    dbCartItem.Menu_price = +(selectedVariant.orginal_price - discount);
                    dbCartItem.Total = dbCartItem.quantity * (dbCartItem.Menu_price + dbCartItem.Addon_price);
                    myApp.getAppDatabase().cartDao().update(dbCartItem);
                }
            }
            if (getActivity() != null) {
                getActivity().runOnUiThread(() -> {
                    menuAdapter.notifyItemChanged(position);
                    updateCheckoutLayout();
                });
            }
        }).start();
    }

    private void addItemToDineInCart(Menu menuItem, int position) {
        new Thread(() -> {
            String variantName = null;
            float discount = 0;
            if (menuItem.variants != null && menuItem.variants.size() > 0) {
                variantName = menuItem.variants.get(0).sub_name;
                variantName = (menuItem.menu_name.equalsIgnoreCase(variantName)) ? "" : variantName;
                discount = (menuItem.variants.get(0).orginal_price * menuItem.getProductPercentage() / 100);
            }
            DineinCartItem dbDineInCartItem = myApp.getAppDatabase().dineinCartItemDao().getCartMenuItem(menuItem.id, variantName);
            if (dbDineInCartItem == null) {
                DineinCartItem cartItem = new DineinCartItem();
                if (menuItem.variants != null && menuItem.variants.size() > 0) {
                    Variant selectedVariant = menuItem.variants.get(0);
                    cartItem.quantity = 1;
                    cartItem.Menu_price = selectedVariant.orginal_price - discount;
                    cartItem.menu_id = selectedVariant.menu_id;
                    cartItem.id = selectedVariant.id;
                    cartItem.menu_name = menuItem.menu_name;
                    cartItem.menu_size = (menuItem.menu_name.equalsIgnoreCase(selectedVariant.sub_name)) ? "" : selectedVariant.sub_name;
                    cartItem.res_id = menuItem.restaurant_id;
                    cartItem.menu_type = menuItem.menu_type;

                    cartItem.Addon_name = "";
                    cartItem.Addon_price = 0;
                    cartItem.Total = cartItem.quantity * (cartItem.Menu_price + cartItem.Addon_price);
                    myApp.getAppDatabase().dineinCartItemDao().insertAll(cartItem);
                }
            } else {
                if (menuItem.variants != null && menuItem.variants.size() > 0) {
                    Variant selectedVariant = menuItem.variants.get(0);
                    dbDineInCartItem.quantity++;
                    dbDineInCartItem.Menu_price = +(selectedVariant.orginal_price - discount);
                    dbDineInCartItem.Total = dbDineInCartItem.quantity * (dbDineInCartItem.Menu_price + dbDineInCartItem.Addon_price);
                    myApp.getAppDatabase().dineinCartItemDao().update(dbDineInCartItem);
                }

            }
            if (getActivity() != null) {
                getActivity().runOnUiThread(() -> {
                    menuAdapter.notifyItemChanged(position);
                    updateCheckoutLayout();
                });
            }
        }).start();
    }

    private DialogDismissListener addItemCartDialogListener = new DialogDismissListener() {
        @Override
        public void onDialogDismiss(Object o) {
            new Thread(() -> {
                int resId = myApp.getAppDatabase().cartDao().getRestaurantId();
                if (!isDineIn) {
                    if (resId == restaurant.id) {
                        myApp.getMyPreferences().saveOrderRestaurant(restaurant);
                    } else {
                        myApp.getMyPreferences().deleteOrderRestaurant();
                    }
                }
                updateCheckoutLayout();
            }).start();
            if (getActivity() != null) {
                getActivity().runOnUiThread(() -> {
                    menuAdapter.notifyDataSetChanged();
                });
            }
        }
    };

    private void updateCheckoutLayout() {
        getActivity().setResult(Activity.RESULT_OK);
        if (!isDineIn && myApp.getMyPreferences().getOrderRestaurant() != null && myApp.getMyPreferences().getOrderRestaurant().id != restaurant.id) {
            llCheckout.setVisibility(View.GONE);
        } else {
            new Thread(() -> {
                CartSummary cartSummary;
                if (isDineIn) {
                    cartSummary = myApp.getAppDatabase().dineinCartItemDao().getCartSummary();
                } else {
                    cartSummary = myApp.getAppDatabase().cartDao().getCartSummary();
                }
                if (getActivity() != null) {
                    getActivity().runOnUiThread(() -> {
                        if (cartSummary != null && cartSummary.items > 0) {
                            llCheckout.setVisibility(View.VISIBLE);
                            tvTotal.setText("Total: " + myApp.getCurrencySymbol() + MyApp.df.format(cartSummary.total));
                            if (!isDineIn) {
                                myApp.getMyPreferences().saveOrderRestaurant(restaurant);
                            }
                        } else {
                            llCheckout.setVisibility(View.GONE);
                        }
                    });

                }
                CommonFunctions.notifyCartCount(getContext());
            }).start();
        }

    }

    private void openCheckoutScreen() {
        if (isDineIn) {
            startActivityForResult(new Intent(getActivity(), TransportActivity.class).putExtra("destination", "dinein_checkout").putExtra("is_dinein", true).putExtra("name", "Dine in Checkout").putExtra("table_number", tableNumber).putExtra("guests", guest_count).putExtra("qr_code", qrCode).putExtra("resId", restaurant.id), CODE_REFRESH);
        } else {
            new Thread(this::saveCartOnline).start();
            startActivityForResult(new Intent(getActivity(), TransportActivity.class).putExtra("destination", "cart").putExtra("name", "Checkout").putExtra("hideToolbar", true), CODE_REFRESH);
        }
    }

    private void validateRestaurantOpeningAndCheckout() {
        if (isRestaurantClosed()) {
            showRestaurantIsClosed("Unfortunately, " + restaurant.restaurant_name + " is currently closed", false);
        } else {
            openCheckoutScreen();
        }
    }

    private boolean isRestaurantClosed() {
        boolean isClose;
        if (restaurant.status.equalsIgnoreCase("0") || (restaurant.restaurant_delivery.equalsIgnoreCase("no") && restaurant.restaurant_pickup.equalsIgnoreCase("no"))) {
            isClose = true;
        } else {
            if (!restaurant.restaurant_delivery.equalsIgnoreCase("no") && isSlotsAvailable(restaurant.delivery_time_slot)) {
                isClose = false;
            } else {
                isClose = true;
                isClose = restaurant.restaurant_pickup.equalsIgnoreCase("no") || !isSlotsAvailable(restaurant.pickup_time_slot);
            }
        }
        return isClose;
    }


    private boolean isSlotsAvailable(TimeSlots timeSlots) {
        return !(Validators.isNullOrEmpty(timeSlots.today) && Validators.isNullOrEmpty(timeSlots.tomorrow) && Validators.isNullOrEmpty(timeSlots.dayaftertomorrow));
    }

    private void saveCartOnline() {
        ArrayList<CartItem> cartItems = new ArrayList<>(myApp.getAppDatabase().cartDao().getAll());
        for (CartItem c : cartItems) {
            UploadCartItems uploadCartItems = new UploadCartItems();
            uploadCartItems.subaddons_name = c.Addon_name;//
            uploadCartItems.menu_price = c.Menu_price;//
            uploadCartItems.menu_id = c.menu_id;//
            uploadCartItems.menu_name = c.menu_name;//
            uploadCartItems.quantity = c.quantity;//
            uploadCartItems.restaurant_id = c.res_id;//
            uploadCartItems.total_price = c.Total;
            uploadCartItems.customer_id = loggedInUser.id;
            cartItemArrayList.add(uploadCartItems);
        }
        CommonFunctions.updateCartOnline(loggedInUser.id, new Gson().toJson(cartItemArrayList));
    }

    @Override
    public void onActivityResult(int requestCode, int resultCode, @Nullable Intent data) {
        super.onActivityResult(requestCode, resultCode, data);
        if (requestCode == CODE_REFRESH) {
            menuAdapter.notifyDataSetChanged();
            updateCheckoutLayout();
        }
    }

}
