package com.tiffintom.fragment;

import android.os.Build;
import android.os.Bundle;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;

import androidx.annotation.NonNull;
import androidx.annotation.Nullable;
import androidx.recyclerview.widget.LinearLayoutManager;
import androidx.recyclerview.widget.RecyclerView;

import com.google.android.material.bottomsheet.BottomSheetDialogFragment;
import com.tiffintom.R;
import com.tiffintom.adapters.SelectCountryAdapter;
import com.tiffintom.interfaces.DialogDismissListener;

import java.util.ArrayList;

public class SelectCountryBottomSheetFragment extends BottomSheetDialogFragment {

    private RecyclerView rvCountry;
    private DialogDismissListener dialogDismissListener;
    private SelectCountryAdapter countryAdapter;
    private ArrayList<String> countries = new ArrayList<>();
    private String selectedCountry;

    public static SelectCountryBottomSheetFragment getInstance(String selectedCountry) {
        SelectCountryBottomSheetFragment countryBottomSheetFragment = new SelectCountryBottomSheetFragment();
        Bundle bundle = new Bundle();
        bundle.putString("selectedCountry", selectedCountry);
        countryBottomSheetFragment.setArguments(bundle);
        return countryBottomSheetFragment;
    }

    public void setDialogDismissListener(DialogDismissListener dialogDismissListener) {
        this.dialogDismissListener = dialogDismissListener;
    }

    @Override
    public int getTheme() {
        if (Build.VERSION.SDK_INT > 21) {
            return R.style.BottomSheetDialogTheme;
        } else {
            return super.getTheme();
        }
    }

    @Nullable
    @Override
    public View onCreateView(@NonNull LayoutInflater inflater, @Nullable ViewGroup container, @Nullable Bundle savedInstanceState) {
        if (getArguments() != null) {
            selectedCountry = getArguments().getString("selectedCountry");
        }
        return inflater.inflate(R.layout.select_country_bottomsheet_fragment, container, false);
    }

    @Override
    public void onViewCreated(@NonNull View view, @Nullable Bundle savedInstanceState) {
        super.onViewCreated(view, savedInstanceState);
        initViews(view);
        setUpCountries();
    }

    private void setUpCountries() {
        countries.clear();
        countries.add("United Kingdom");
        countries.add("India");
        countryAdapter.notifyDataSetChanged();
    }

    private void initViews(View view) {

        rvCountry = view.findViewById(R.id.rvCountries);
        countryAdapter = new SelectCountryAdapter(getActivity(), countries, selectedCountry, (position, data) -> {
            if (dialogDismissListener != null) {
                dialogDismissListener.onDialogDismiss(data);
            }
            dismiss();
        });
        LinearLayoutManager categoryLayout = new LinearLayoutManager(getActivity(), RecyclerView.VERTICAL, false);
        rvCountry.setLayoutManager(categoryLayout);
        rvCountry.setNestedScrollingEnabled(false);
        rvCountry.setAdapter(countryAdapter);
    }

}
