package com.tiffintom.fragment;

import android.content.Context;
import android.content.Intent;
import android.graphics.Typeface;
import android.graphics.drawable.ColorDrawable;
import android.os.Bundle;
import android.view.Gravity;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.ImageView;
import android.widget.LinearLayout;
import android.widget.PopupWindow;
import android.widget.TextView;

import androidx.annotation.NonNull;
import androidx.annotation.Nullable;
import androidx.cardview.widget.CardView;
import androidx.recyclerview.widget.LinearLayoutManager;
import androidx.recyclerview.widget.RecyclerView;

import com.google.android.material.floatingactionbutton.FloatingActionButton;
import com.tiffintom.R;
import com.tiffintom.activity.MainActivity;
import com.tiffintom.activity.SplashActivity;
import com.tiffintom.activity.TransportActivity;
import com.tiffintom.adapters.CountryAdapter;
import com.tiffintom.base.BaseFragment;
import com.tiffintom.common.Validators;
import com.tiffintom.models.Country;
import com.tiffintom.utils.ToastUtils;

import java.util.ArrayList;

/**
 * Created by Amrish on 11-09-2020.
 */
public class SelectCountryFragment extends BaseFragment {
    private LinearLayout llUk, llIndia;
    private TextView tvIndia, tvUk;
    private String selectedCountry = "UK";
    private FloatingActionButton fabNext;
    private ImageView ivUk, ivIndia;
    private ArrayList<Country> countries = new ArrayList<>();
    private TextView tvSelectedCountry;
    private CardView cvCountry;
    private LinearLayout llNext;

    public static SelectCountryFragment getInstance() {
        return new SelectCountryFragment();
    }

    @Nullable
    @Override
    public View onCreateView(@NonNull LayoutInflater inflater, @Nullable ViewGroup container, @Nullable Bundle savedInstanceState) {
        return inflater.inflate(R.layout.fragment_set_country, container, false);
    }

    @Override
    public void onViewCreated(@NonNull View view, @Nullable Bundle savedInstanceState) {
        super.onViewCreated(view, savedInstanceState);
        selectedCountry = myApp.getMyPreferences().getSelectedCountry();
        setListeners();
        updateViews();
        setUpCountries();
    }

    private void updateViews() {
        if (myApp.getMyPreferences().getSelectedCountry() != null) {
            llNext.setEnabled(true);
            if (myApp.getMyPreferences().getSelectedCountry().equalsIgnoreCase("in")) {
                tvSelectedCountry.setText("India");
            } else if (myApp.getMyPreferences().getSelectedCountry().equalsIgnoreCase("uk")) {
                tvSelectedCountry.setText("United Kingdom");
            }
        } else {
            tvSelectedCountry.setHint("Select your country");
            llNext.setEnabled(false);
        }
    }

    @Override
    protected void initViews(View view) {
        super.initViews(view);
        llNext = view.findViewById(R.id.llNext);
        llIndia = view.findViewById(R.id.llIndia);
        llUk = view.findViewById(R.id.llUk);
        tvIndia = view.findViewById(R.id.tvIndia);
        tvUk = view.findViewById(R.id.tvUk);
        cvCountry = view.findViewById(R.id.cvCountry);
        tvSelectedCountry = view.findViewById(R.id.tvSelectedCountry);
        fabNext = view.findViewById(R.id.fabNext);
        ivUk = view.findViewById(R.id.ivUKMap);
        ivIndia = view.findViewById(R.id.ivIndiaMap);


    }

    private void setUpCountries() {
        countries.clear();
        countries.add(new Country("United Kingdom", "UK", 0, 0));
        countries.add(new Country("India", "IN", 0, 0));
    }

    private void setListeners() {
        cvCountry.setOnClickListener(view -> {
//            if (rvCountries.getVisibility() == View.VISIBLE) {
//                rvCountries.setVisibility(View.GONE);
//                fabNext.setVisibility(View.VISIBLE);
//            } else {
//                fabNext.setVisibility(View.GONE);
//                rvCountries.setVisibility(View.VISIBLE);
//            }
            openPopup();
        });
        llUk.setOnClickListener(view -> {
            llIndia.setScaleX(0.8f);
            llIndia.setScaleY(0.8f);
            tvIndia.setTypeface(tvIndia.getTypeface(), Typeface.NORMAL);

            llUk.setScaleX(1);
            llUk.setScaleY(1);
            tvUk.setTypeface(tvIndia.getTypeface(), Typeface.BOLD);
            selectedCountry = "UK";
            ivUk.setImageResource(R.drawable.country_uk_filled);
            ivIndia.setImageResource(R.drawable.country_india_outline);
        });
        llIndia.setOnClickListener(view -> {
            llUk.setScaleX(0.8f);
            llUk.setScaleY(0.8f);
            tvUk.setTypeface(tvIndia.getTypeface(), Typeface.NORMAL);

            llIndia.setScaleX(1);
            llIndia.setScaleY(1);
            tvIndia.setTypeface(tvIndia.getTypeface(), Typeface.BOLD);
            selectedCountry = "IN";
            ivUk.setImageResource(R.drawable.county_uk_outline);
            ivIndia.setImageResource(R.drawable.country_india_filled);
        });

        llNext.setOnClickListener(view -> {
            if (!Validators.isNullOrEmpty(selectedCountry)) {
                myApp.getMyPreferences().saveSelectedCountry(selectedCountry);
                startActivity(new Intent(getActivity(), SplashActivity.class).setFlags(Intent.FLAG_ACTIVITY_CLEAR_TASK | Intent.FLAG_ACTIVITY_NEW_TASK | Intent.FLAG_ACTIVITY_CLEAR_TOP));
            } else {
                ToastUtils.makeToast(getActivity(), "Please select country to continue");
            }
        });
    }

    private void openActivity() {
        startActivity(new Intent(getActivity(), MainActivity.class).setFlags(Intent.FLAG_ACTIVITY_CLEAR_TASK | Intent.FLAG_ACTIVITY_NEW_TASK | Intent.FLAG_ACTIVITY_CLEAR_TOP));

    }

    private void openActivity(Bundle bundle) {
        startActivity(new Intent(getActivity(), TransportActivity.class).putExtras(bundle).setFlags(Intent.FLAG_ACTIVITY_CLEAR_TASK | Intent.FLAG_ACTIVITY_NEW_TASK | Intent.FLAG_ACTIVITY_CLEAR_TOP));
    }

    PopupWindow changeSortPopUp;

    private void openPopup() {
        LinearLayout viewGroup = (LinearLayout) getActivity().findViewById(R.id.llPopup);
        LayoutInflater layoutInflater = (LayoutInflater) getActivity().getSystemService(Context.LAYOUT_INFLATER_SERVICE);
        View layout = layoutInflater.inflate(R.layout.popup_select_country, viewGroup);

        // Creating the PopupWindow
        changeSortPopUp = new PopupWindow(getActivity());
        changeSortPopUp.setContentView(layout);
        changeSortPopUp.setWidth(LinearLayout.LayoutParams.MATCH_PARENT);
        changeSortPopUp.setHeight(LinearLayout.LayoutParams.WRAP_CONTENT);
        changeSortPopUp.setFocusable(true);
        changeSortPopUp.setWindowLayoutMode(ViewGroup.LayoutParams.MATCH_PARENT, ViewGroup.LayoutParams.WRAP_CONTENT);
        // Some offset to align the popup a bit to the left, and a bit down, relative to button's position.
        int OFFSET_X = -20;
        int OFFSET_Y = 95;

        // Clear the default translucent background
        changeSortPopUp.setBackgroundDrawable(new ColorDrawable());
        RecyclerView rvCountries = layout.findViewById(R.id.rvCountries);
        TextView tvSelectCountry = layout.findViewById(R.id.tvCountryText);
        tvSelectCountry.setOnClickListener(view -> changeSortPopUp.dismiss());
        int location[] = new int[2];
        cvCountry.getLocationOnScreen(location);
        CountryAdapter countryAdapter = null;
        countryAdapter = new CountryAdapter(countries, selectedCountry, (view1, position, data) -> {
            rvCountries.setVisibility(View.GONE);
            selectedCountry = ((Country) data).smallName;
            tvSelectedCountry.setText(((Country) data).name);
            llNext.setEnabled(true);
            changeSortPopUp.dismiss();
        });
        rvCountries.setLayoutManager(new LinearLayoutManager(getActivity(), RecyclerView.VERTICAL, false));
        rvCountries.setAdapter(countryAdapter);
        // Displaying the popup at the specified location, + offsets.
//        changeSortPopUp.showAsDropDown(layout, 0, (int) cvCountry.getY() + 15);
        changeSortPopUp.showAtLocation(cvCountry, Gravity.NO_GRAVITY,
                location[0], (int) cvCountry.getY() + 16);
    }

}
