package com.tiffintom.fragment;

import android.app.Activity;
import android.content.BroadcastReceiver;
import android.content.Context;
import android.content.Intent;
import android.content.IntentFilter;
import android.graphics.Color;
import android.graphics.drawable.ColorDrawable;
import android.os.Bundle;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.LinearLayout;
import android.widget.TextView;

import androidx.annotation.NonNull;
import androidx.annotation.Nullable;
import androidx.appcompat.app.AlertDialog;
import androidx.localbroadcastmanager.content.LocalBroadcastManager;

import com.tiffintom.BuildConfig;
import com.tiffintom.R;
import com.tiffintom.activity.AuthActivity;
import com.tiffintom.activity.MainActivity;
import com.tiffintom.activity.TransportActivity;
import com.tiffintom.base.BaseFragment;
import com.tiffintom.common.Validators;
import com.tiffintom.utils.Constants;
import com.tiffintom.utils.LogUtils;
import com.tiffintom.utils.ToastUtils;

import static com.tiffintom.utils.Constants.CHAT_COUNT_UPDATE;

/**
 * Created by Amrish on 09-09-2020.
 */
public class SettingsFragment extends BaseFragment {
    public static SettingsFragment getInstance() {
        return new SettingsFragment();
    }

    private LinearLayout cvLogout, cvHelp, cvFeedback, cvAbout, cvBecomePartner, cvNotification;
    private TextView tvAppVersion;
    private TextView tvHelpCount;
    private BroadcastReceiver chatCountBroadCastReceiver;

    @Nullable
    @Override
    public View onCreateView(@NonNull LayoutInflater inflater, @Nullable ViewGroup container, @Nullable Bundle savedInstanceState) {
        return inflater.inflate(R.layout.fragment_setting, container, false);
    }

    @Override
    public void onViewCreated(@NonNull View view, @Nullable Bundle savedInstanceState) {
        super.onViewCreated(view, savedInstanceState);
        setListener();
        updateViews();
    }

    private void updateViews() {
        if (myApp.getMyPreferences().getLoggedInUser() == null) {
            cvLogout.setVisibility(View.GONE);
        }
        updateChatCount(myApp.chatCount);
    }

    @Override
    protected void initViews(View view) {
        super.initViews(view);
        tvUserName = view.findViewById(R.id.tvUserName);
        TextView tvScreenTitle = view.findViewById(R.id.tvScreenTitle);
        tvScreenTitle.setVisibility(View.VISIBLE);
        tvScreenTitle.setText("Settings");
        cvLogout = view.findViewById(R.id.cvLogout);
        cvHelp = view.findViewById(R.id.cvHelp);
        cvFeedback = view.findViewById(R.id.cvFeedback);
        cvAbout = view.findViewById(R.id.cvAbout);
        cvBecomePartner = view.findViewById(R.id.cvBecomePartner);
        cvNotification = view.findViewById(R.id.cvNotifications);
        tvAppVersion = view.findViewById(R.id.tvAppVersion);
        tvHelpCount = view.findViewById(R.id.tvHelpCount);


        tvAppVersion.setText(new StringBuilder().append("App Version ").append(BuildConfig.VERSION_NAME).toString());

        if (MainActivity.totalMessageCount > 0) {
            tvHelpCount.setVisibility(View.VISIBLE);
            tvHelpCount.setText(String.valueOf(MainActivity.totalMessageCount));
        } else {
            tvHelpCount.setVisibility(View.GONE);
        }
    }

    private void setListener() {
        cvLogout.setOnClickListener(view -> {
            askForLogout();
        });
        cvHelp.setOnClickListener(view -> {
            startActivity(new Intent(getActivity(), TransportActivity.class).putExtra("destination", "help_main").putExtra("name", "Help"));
        });
        cvFeedback.setOnClickListener(view -> {
            if (myApp.getMyPreferences().getLoggedInUser() == null) {
                startActivity(new Intent(getActivity(), AuthActivity.class));
            } else {
                getActivity().startActivityForResult(new Intent(getActivity(), TransportActivity.class).putExtra("destination", "feedback").putExtra("name", "Feedback"), Constants.CODE_MESSAGE);
            }
        });

        cvAbout.setOnClickListener(view -> startActivity(new Intent(getActivity(), TransportActivity.class).putExtra("destination", "about").putExtra("name", "About")));

        cvBecomePartner.setOnClickListener(view -> startActivity(new Intent(getActivity(), TransportActivity.class).putExtra("destination", "becomepartner").putExtra("name", "Become a partner")));

        cvNotification.setOnClickListener(view -> {
            if (myApp.getMyPreferences().getLoggedInUser() == null) {
                startActivity(new Intent(getActivity(), AuthActivity.class));
            } else {
                startActivity(new Intent(getActivity(), TransportActivity.class).putExtra("destination", "notification").putExtra("name", "Notifications"));
            }
        });
        chatCountBroadCastReceiver = new BroadcastReceiver() {
            @Override
            public void onReceive(Context context, Intent intent) {
                int count = intent.getIntExtra("count", 0);
                updateChatCount(count);
            }
        };
    }

    private void updateChatCount(int count) {
        if (count > 0) {
            tvHelpCount.setVisibility(View.VISIBLE);
            tvHelpCount.setText(String.valueOf(count));
        } else {
            tvHelpCount.setVisibility(View.GONE);
        }
    }

    private void askForLogout() {
        AlertDialog alertDialog = new AlertDialog.Builder(getActivity()).create();
        alertDialog.getWindow().setBackgroundDrawable(new ColorDrawable(Color.TRANSPARENT));
        LayoutInflater inflater = (LayoutInflater) getActivity().getSystemService(Context.LAYOUT_INFLATER_SERVICE);

        View dialogView = inflater.inflate(R.layout.dialog_logout, null);
        TextView tvCancle = dialogView.findViewById(R.id.tvCancel);
        TextView tvConfirm = dialogView.findViewById(R.id.tvConfirm);

        tvConfirm.setOnClickListener(view -> {
            doLogout();
            alertDialog.dismiss();

        });
        tvCancle.setOnClickListener(view -> {
            alertDialog.dismiss();
        });

        alertDialog.setView(dialogView);
        alertDialog.setCancelable(false);
        alertDialog.show();
    }

    private void doLogout() {
        LogUtils.e("DoLogout");
        myApp.logout(getActivity());
        ToastUtils.makeToast(getActivity(), "Logout successfully");
    }

    @Override
    public void onResume() {
        super.onResume();
        LocalBroadcastManager.getInstance(getActivity()).registerReceiver(chatCountBroadCastReceiver, new IntentFilter(CHAT_COUNT_UPDATE));
    }

    @Override
    public void onStop() {
        LocalBroadcastManager.getInstance(getActivity()).unregisterReceiver(chatCountBroadCastReceiver);
        super.onStop();
    }

    @Override
    public void onActivityResult(int requestCode, int resultCode, @Nullable Intent data) {
        super.onActivityResult(requestCode, resultCode, data);
        if (requestCode == Constants.CODE_MESSAGE && resultCode == Activity.RESULT_OK) {
            String message = data.getStringExtra("message");
            if (!Validators.isNullOrEmpty(message)) {
                LogUtils.e(message);
                ToastUtils.makeSnackToast(getActivity(), message);
            }
        }
    }
}
