package com.tiffintom.fragment;

import android.os.Build;
import android.os.Bundle;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.TextView;

import androidx.annotation.NonNull;
import androidx.annotation.Nullable;
import androidx.recyclerview.widget.LinearLayoutManager;
import androidx.recyclerview.widget.RecyclerView;

import com.google.android.material.bottomsheet.BottomSheetDialogFragment;
import com.google.gson.Gson;
import com.tiffintom.R;
import com.tiffintom.adapters.ShowTimeAdapter;
import com.tiffintom.common.CommonFunctions;
import com.tiffintom.common.Validators;
import com.tiffintom.interfaces.DialogDismissListener;
import com.tiffintom.models.TimeSlot;
import com.tiffintom.models.TimeSlots;
import com.tiffintom.utils.LogUtils;

import java.text.DateFormat;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Calendar;
import java.util.Date;
import java.util.Locale;

public class ShowTimeSlotBottomSheetFragment extends BottomSheetDialogFragment {


    private RecyclerView recyclerView;
    private ShowTimeAdapter adapter;
    private String slotTimes;
    private TimeSlots timeSlot;
    private ArrayList<TimeSlot> timeslots = new ArrayList<>();
    private DialogDismissListener dialogDismissListener;
    private TimeSlot selectedSlot;
    private ArrayList<String> times = new ArrayList<>();
    private boolean forReservation;
    private TextView tvScreenTitle;

    public static ShowTimeSlotBottomSheetFragment getInstance(TimeSlots timeSlots, TimeSlot selectedTimeSlot) {
        ShowTimeSlotBottomSheetFragment showTimeBottomSheetFragment = new ShowTimeSlotBottomSheetFragment();
        Bundle bundle = new Bundle();
        bundle.putString("slotTimes", new Gson().toJson(timeSlots));
        bundle.putString("selectedTimeSlot", new Gson().toJson(selectedTimeSlot));
        showTimeBottomSheetFragment.setArguments(bundle);
        return showTimeBottomSheetFragment;
    }

    public static ShowTimeSlotBottomSheetFragment getInstance(String times, boolean forReservation, TimeSlot selectedSlot) {
        ShowTimeSlotBottomSheetFragment showTimeBottomSheetFragment = new ShowTimeSlotBottomSheetFragment();
        Bundle bundle = new Bundle();
        bundle.putString("times", times);
        bundle.putBoolean("forReservation", forReservation);
        bundle.putString("selectedTimeSlot", new Gson().toJson(selectedSlot));
        showTimeBottomSheetFragment.setArguments(bundle);
        return showTimeBottomSheetFragment;
    }

    public void setDialogDismissListener(DialogDismissListener dialogDismissListener) {
        this.dialogDismissListener = dialogDismissListener;
    }

    @Override
    public int getTheme() {
        if (Build.VERSION.SDK_INT > 21) {
            return R.style.BottomSheetDialogTheme;
        } else {
            return super.getTheme();
        }
    }

    DateFormat timeSlotFormat = new SimpleDateFormat("hh:mm a", Locale.getDefault());
    DateFormat timeSlot24HFormat = new SimpleDateFormat("HH:mm", Locale.getDefault());

    @Nullable
    @Override
    public View onCreateView(@NonNull LayoutInflater inflater, @Nullable ViewGroup container, @Nullable Bundle savedInstanceState) {
        if (getArguments() != null) {
            slotTimes = getArguments().getString("slotTimes");
            forReservation = getArguments().getBoolean("forReservation");
            if (forReservation)
                times.addAll(Arrays.asList(getArguments().getString("times").split(",")));
            selectedSlot = new Gson().fromJson(getArguments().getString("selectedTimeSlot"), TimeSlot.class);
        }
        if (slotTimes != null) {
            timeSlot = new Gson().fromJson(slotTimes, TimeSlots.class);
            if (!Validators.isNullOrEmpty(timeSlot.today)) {
                String[] timeSlotsArray = timeSlot.today.split(",");
                for (String slot : timeSlotsArray) {
                    Date slotDateTime = CommonFunctions.convertStringDateToDate(CommonFunctions.formatMili(System.currentTimeMillis(), "dd-MM-yyyy") + " " + slot, "dd-MM-yyyy hh:mm a");
                    Date today = new Date();
                    if (slotDateTime.before(today)) {
                        TimeSlot s = new TimeSlot();
                        s.name = "ASAP";
                        s.time = "asap";
                        s.date = CommonFunctions.formatMili(System.currentTimeMillis(), "yyyy-MM-dd");
                        timeslots.add(s);
                        break;
                    }
                }

                for (String time : timeSlotsArray) {
                    Calendar calendar = Calendar.getInstance();
                    String today = CommonFunctions.formatMili(calendar.getTimeInMillis(), "yyyy-MM-dd");

                    TimeSlot slot = new TimeSlot();
                    slot.name = "Today: " + time;
                    try {
                        slot.name = "Today: " + timeSlot24HFormat.format(timeSlotFormat.parse(time));
                    } catch (ParseException e) {
                        e.printStackTrace();
                    }
                    slot.time = time;
                    slot.date = today;
                    slot.plusDay = 0;
                    timeslots.add(slot);

                }
            }
            if (!Validators.isNullOrEmpty(timeSlot.tomorrow)) {
                for (String time : timeSlot.tomorrow.split(",")) {
                    Calendar calendar = Calendar.getInstance();
                    calendar.add(Calendar.DAY_OF_MONTH, 1);
                    String tomorrowDate = CommonFunctions.formatMili(calendar.getTimeInMillis(), "yyyy-MM-dd");
                    TimeSlot slot = new TimeSlot();
                    slot.name = "Tomorrow: " + time;
                    try {
                        slot.name = "Tomorrow: " + timeSlot24HFormat.format(timeSlotFormat.parse(time));
                    } catch (ParseException e) {
                        e.printStackTrace();
                    }
                    slot.time = time;
                    slot.date = tomorrowDate;
                    slot.plusDay = 1;
                    timeslots.add(slot);
                }
            }
            if (!Validators.isNullOrEmpty(timeSlot.dayaftertomorrow)) {
                for (String time : timeSlot.dayaftertomorrow.split(",")) {
                    Calendar calendar = Calendar.getInstance();
                    calendar.add(Calendar.DAY_OF_MONTH, 2);
                    String dayAfterTomorrow = CommonFunctions.formatMili(calendar.getTimeInMillis(), "yyyy-MM-dd");
                    String dayAfterTomorrowSimple = CommonFunctions.formatMili(calendar.getTimeInMillis(), "dd/MM/yyyy");
                    // deliveryList.add(dayAfterTomorrow+": "+ time);

                    TimeSlot slot = new TimeSlot();
                    slot.name = dayAfterTomorrowSimple + ": " + time;
                    try {
                        slot.name = dayAfterTomorrowSimple + ": " + timeSlot24HFormat.format(timeSlotFormat.parse(time));
                    } catch (ParseException e) {
                        e.printStackTrace();
                    }
                    slot.time = time;
                    slot.date = dayAfterTomorrow;
                    slot.plusDay = 2;
                    timeslots.add(slot);
                }
            }
        }
        if (forReservation) {
            DateFormat timeResSlotFormat = new SimpleDateFormat("EEE hh:mm a", Locale.getDefault());
            DateFormat timeResSlot24HFormat = new SimpleDateFormat("HH:mm", Locale.getDefault());
            for (String time : times) {
                TimeSlot timeSlot = new TimeSlot();
                timeSlot.name = time;
                try {
                    timeSlot.name = timeResSlot24HFormat.format(timeResSlotFormat.parse(time));
                } catch (ParseException e) {
                    e.printStackTrace();
                }
                timeSlot.time = time;
                timeslots.add(timeSlot);
            }
        }


        return inflater.inflate(R.layout.fragment_show_time, container, false);
    }

    @Override
    public void onViewCreated(@NonNull View view, @Nullable Bundle savedInstanceState) {
        super.onViewCreated(view, savedInstanceState);
        initView(view);
        setListner();
        if (forReservation) {
            tvScreenTitle.setText("Select time");
        } else {
            tvScreenTitle.setText("Select date & time");
        }
    }

    private void setListner() {
        adapter = new ShowTimeAdapter(getActivity(), selectedSlot, timeslots, (position, data) -> {
            if (dialogDismissListener != null) {
                dialogDismissListener.onDialogDismiss(data);
            }
            dismiss();

        });
        recyclerView.setLayoutManager(new LinearLayoutManager(getActivity(), RecyclerView.VERTICAL, false));
        recyclerView.setAdapter(adapter);
        LogUtils.e(String.valueOf(adapter.getItemCount()));
    }

    private void initView(View view) {
        recyclerView = view.findViewById(R.id.rvTime);
        tvScreenTitle = view.findViewById(R.id.tvScreenTitle);
    }
}
