package com.tiffintom.fragment;

import android.graphics.Color;
import android.graphics.ColorFilter;
import android.graphics.PorterDuff;
import android.graphics.PorterDuffColorFilter;
import android.graphics.drawable.ColorDrawable;
import android.os.Bundle;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.LinearLayout;
import android.widget.TextView;

import androidx.annotation.NonNull;
import androidx.annotation.Nullable;
import androidx.core.content.ContextCompat;
import androidx.fragment.app.DialogFragment;
import androidx.recyclerview.widget.LinearLayoutManager;
import androidx.recyclerview.widget.RecyclerView;

import com.airbnb.lottie.LottieAnimationView;
import com.airbnb.lottie.LottieProperty;
import com.airbnb.lottie.model.KeyPath;
import com.airbnb.lottie.value.LottieFrameInfo;
import com.airbnb.lottie.value.SimpleLottieValueCallback;
import com.androidnetworking.AndroidNetworking;
import com.androidnetworking.error.ANError;
import com.androidnetworking.interfaces.JSONArrayRequestListener;
import com.google.gson.Gson;
import com.google.gson.JsonSyntaxException;
import com.google.gson.reflect.TypeToken;
import com.tiffintom.MyApp;
import com.tiffintom.R;
import com.tiffintom.adapters.TableSelectAdapater;
import com.tiffintom.interfaces.DialogDismissListener;
import com.tiffintom.interfaces.RecyclerViewItemClickListener;
import com.tiffintom.models.Tables;
import com.tiffintom.network.ApiEndPoints;
import com.tiffintom.utils.ToastUtils;

import org.json.JSONArray;
import org.json.JSONException;
import org.json.JSONObject;

import java.lang.reflect.Type;
import java.util.ArrayList;
import java.util.List;

public class TableSelectDialogFragment extends DialogFragment {

    private RecyclerView rvTables;
    private TextView tvCancel, tvConfirm;
    private LinearLayout llLoading, llNoData;
    private LottieAnimationView lodingView;
    private ArrayList<Tables> tables = new ArrayList<>();
    private TableSelectAdapater tableAdapter;
    private Tables selectedTable;

    private MyApp myApp = MyApp.getInstance();
    DialogDismissListener dialogDismissListener;

    public static TableSelectDialogFragment getInstance() {
        return new TableSelectDialogFragment();
    }

    public void setDialogDismissListener(DialogDismissListener dialogDismissListener) {
        this.dialogDismissListener = dialogDismissListener;
    }

    @Nullable
    @Override
    public View onCreateView(@NonNull LayoutInflater inflater, @Nullable ViewGroup container, @Nullable Bundle savedInstanceState) {
        View dialogView = inflater.inflate(R.layout.dialog_select_tables, container, false);

        getDialog().getWindow().setBackgroundDrawable(new ColorDrawable(Color.TRANSPARENT));

        return dialogView;
    }


    @Override
    public void onViewCreated(@NonNull View view, @Nullable Bundle savedInstanceState) {
        super.onViewCreated(view, savedInstanceState);
        initViews(view);
        setListeners();

    }

    private void fetchTables() {
        try {
            if (getActivity() != null) getActivity().runOnUiThread(() -> {
                llLoading.setVisibility(View.VISIBLE);
            });
            AndroidNetworking.get(ApiEndPoints.tables)
                    .addQueryParameter("nopaginate", "1")
                    .addHeaders("business-id", myApp.getMyPreferences().getCurrentDineInRestaurantDetail().business.id)
                    .build()
                    .getAsJSONArray(new JSONArrayRequestListener() {
                        @Override
                        public void onResponse(JSONArray response) {
                            try {
                                if (getActivity() != null) getActivity().runOnUiThread(() -> {
                                    llLoading.setVisibility(View.GONE);
                                });
                                tables.clear();
                                Type type = new TypeToken<List<Tables>>() {
                                }.getType();
                                ArrayList<Tables> tablesArrayList = new ArrayList<>();
                                tablesArrayList.addAll(new Gson().fromJson(response.toString(), type));
                                for (Tables table : tablesArrayList) {
                                    if (table.status.equalsIgnoreCase("Vacant")) {
                                        tables.add(table);
                                    }
                                }
                                if (getActivity() != null) {
                                    getActivity().runOnUiThread(() -> {
                                        if (tables.size() == 0) {
                                            llNoData.setVisibility(View.VISIBLE);
                                        } else {
                                            llNoData.setVisibility(View.GONE);
                                        }
                                        tableAdapter.notifyDataSetChanged();
                                    });
                                }
                            } catch (JsonSyntaxException e) {
                                e.printStackTrace();
                            }
                        }

                        @Override
                        public void onError(ANError anError) {
                            try {
                                if (getActivity() != null) getActivity().runOnUiThread(() -> {
                                    llLoading.setVisibility(View.GONE);
                                    if (tables.size() == 0) {
                                        llNoData.setVisibility(View.VISIBLE);
                                    } else {
                                        llNoData.setVisibility(View.GONE);
                                    }
                                });
                            } catch (Exception e) {
                                e.printStackTrace();
                            }
                            anError.printStackTrace();
                            try {
                                JSONObject object = new JSONObject(anError.getErrorBody());
                                String toast = object.getString("message");
                                ToastUtils.makeToast(getActivity(), toast);
                            } catch (JSONException e) {
                                e.printStackTrace();
                            }
                        }
                    });
        } catch (Exception e) {
            e.printStackTrace();
        }

    }

    private void setListeners() {
        tvCancel.setOnClickListener(view -> {
            try {
                dismiss();
                TableSelectAdapater.selectedTable = 0;
                tableAdapter.notifyDataSetChanged();
            } catch (Exception e) {
                e.printStackTrace();
            }
        });

        tvConfirm.setOnClickListener(view -> {
            try {
                if (tables.size() > 0) {
                    if (selectedTable != null) {
                        if (dialogDismissListener != null)
                            dialogDismissListener.onDialogDismiss(selectedTable);
                        dismiss();
                        TableSelectAdapater.selectedTable = 0;
                        tableAdapter.notifyDataSetChanged();
                    } else
                        ToastUtils.makeToast(getActivity(), "Please select a table");
                } else {
                    dismiss();
                }
            } catch (Exception e) {
                e.printStackTrace();
            }
        });
    }

    private void initViews(View view) {
        rvTables = view.findViewById(R.id.rvTables);
        tvCancel = view.findViewById(R.id.tvCancel);
        tvConfirm = view.findViewById(R.id.tvConfirm);
        llLoading = view.findViewById(R.id.llLoading);
        llNoData = view.findViewById(R.id.llNoData);
        lodingView = view.findViewById(R.id.loadingAnim);

        lodingView.addValueCallback(
                new KeyPath("**"),
                LottieProperty.COLOR_FILTER,
                new SimpleLottieValueCallback<ColorFilter>() {
                    @Override
                    public ColorFilter getValue(LottieFrameInfo<ColorFilter> frameInfo) {
                        return new PorterDuffColorFilter(ContextCompat.getColor(getActivity(), R.color.colorPrimary), PorterDuff.Mode.SRC_ATOP);
                    }
                }
        );

        tableAdapter = new TableSelectAdapater(tables, new RecyclerViewItemClickListener() {
            @Override
            public void onItemClick(int position, Object data) {
                try {
                    Tables tables = (Tables) data;
                    selectedTable = tables;
                    TableSelectAdapater.selectedTable = Integer.parseInt(tables.id);
                    tableAdapter.notifyDataSetChanged();
                } catch (NumberFormatException e) {
                    e.printStackTrace();
                }
            }
        });
        rvTables.setLayoutManager(new LinearLayoutManager(getActivity(), RecyclerView.VERTICAL, false));
        rvTables.setAdapter(tableAdapter);

        fetchTables();
    }
}
