package com.tiffintom.fragment;

import android.Manifest;
import android.app.Activity;
import android.content.BroadcastReceiver;
import android.content.DialogInterface;
import android.content.Intent;
import android.content.IntentFilter;
import android.content.pm.PackageManager;
import android.database.Cursor;
import android.graphics.Bitmap;
import android.graphics.BitmapFactory;
import android.graphics.Color;
import android.media.MediaRecorder;
import android.net.Uri;
import android.os.Build;
import android.os.Bundle;
import android.os.Environment;
import android.os.Handler;
import android.os.SystemClock;
import android.provider.MediaStore;
import android.provider.Settings;
import android.text.Editable;
import android.text.TextWatcher;
import android.view.LayoutInflater;
import android.view.MotionEvent;
import android.view.View;
import android.view.ViewGroup;
import android.view.animation.AlphaAnimation;
import android.view.animation.Animation;
import android.view.animation.LinearInterpolator;
import android.widget.Chronometer;
import android.widget.EditText;
import android.widget.ImageView;
import android.widget.LinearLayout;
import android.widget.TextView;

import androidx.annotation.NonNull;
import androidx.annotation.Nullable;
import androidx.appcompat.app.AlertDialog;
import androidx.core.app.ActivityCompat;
import androidx.core.content.ContextCompat;
import androidx.core.content.FileProvider;
import androidx.localbroadcastmanager.content.LocalBroadcastManager;
import androidx.recyclerview.widget.LinearLayoutManager;
import androidx.recyclerview.widget.RecyclerView;

import com.androidnetworking.AndroidNetworking;
import com.androidnetworking.common.ANRequest;
import com.androidnetworking.error.ANError;
import com.androidnetworking.interfaces.JSONArrayRequestListener;
import com.androidnetworking.interfaces.JSONObjectRequestListener;
import com.devlomi.record_view.OnBasketAnimationEnd;
import com.devlomi.record_view.OnRecordListener;
import com.devlomi.record_view.RecordButton;
import com.devlomi.record_view.RecordView;
import com.google.firebase.crashlytics.FirebaseCrashlytics;
import com.google.gson.Gson;
import com.google.gson.reflect.TypeToken;
import com.tiffintom.BuildConfig;
import com.tiffintom.R;
import com.tiffintom.adapters.ChatAdapter;
import com.tiffintom.base.BaseFragment;
import com.tiffintom.common.CommonFunctions;
import com.tiffintom.common.FileUtil;
import com.tiffintom.common.Validators;
import com.tiffintom.models.ChatMainQuestion;
import com.tiffintom.models.ChatMiniOrderSnippet;
import com.tiffintom.models.ChatOrder;
import com.tiffintom.models.ChatSubQuestion;
import com.tiffintom.models.Message;
import com.tiffintom.models.RequestCreator;
import com.tiffintom.models.User;
import com.tiffintom.network.ApiEndPoints;
import com.tiffintom.notification.Config;
import com.tiffintom.utils.LogUtils;
import com.tiffintom.utils.ToastUtils;

import org.json.JSONArray;
import org.json.JSONException;
import org.json.JSONObject;

import java.io.File;
import java.io.IOException;
import java.lang.reflect.Type;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Random;

import droidninja.filepicker.FilePickerConst;

import static android.Manifest.permission.RECORD_AUDIO;
import static android.Manifest.permission.WRITE_EXTERNAL_STORAGE;
import static com.facebook.FacebookSdk.getApplicationContext;
import static com.tiffintom.utils.Constants.CODE_CAMERA_PICKER;
import static com.tiffintom.utils.Constants.CODE_IMAGE_PICKER;
import static com.tiffintom.utils.Constants.PERMISSION_CODE_STORAGE;

/**
 * Created by Amrish on 12-09-2020.
 */
public class TiffintomChatWindowFragment extends BaseFragment {
    public static TiffintomChatWindowFragment getInstance() {
        return new TiffintomChatWindowFragment();
    }

    private RecyclerView rvMessages;
    private EditText etMessage;
    private ImageView ivSend, ivBackArrow;
    private User loggedInUser = myApp.getMyPreferences().getLoggedInUser();
    private ArrayList<Object> feed = new ArrayList<>();
    private ArrayList<Message> oldMessages = new ArrayList<>();
    private ArrayList<ChatOrder> miniOrders = new ArrayList<>();
    private ArrayList<ChatMainQuestion> chatMainQuestions = new ArrayList<>();
    private ArrayList<ChatMainQuestion> chatOrdersQuestions = new ArrayList<>();
    private ChatAdapter chatAdapter;
    private boolean messageSent = false;
    private LinearLayout llInput;
    private TextView tvScreenTitle;
    private LinearLayout llBackArrow;
    private BroadcastReceiver broadcastReceiver;
    public static String multiMediaPath;
    private ImageView ivRecord;
    private TextView tvRecordingTime;
    private ImageView ivRecordingStatus, ivCamera;
    private LinearLayout llRecordingView, llInputMain;
    private Chronometer chronometer;
    private RecordButton recordButton;
    private RecordView recordView;
    String AudioSavePathInDevice = null;
    File audioFile;
    MediaRecorder mediaRecorder;
    Random random;
    boolean SendMessageCalled = false;
    public static final int RequestPermissionCode = 1;
    private String[] requiredPermissions = {Manifest.permission.READ_EXTERNAL_STORAGE, Manifest.permission.CAMERA, WRITE_EXTERNAL_STORAGE};

    @Nullable
    @Override
    public View onCreateView(@NonNull LayoutInflater inflater, @Nullable ViewGroup container, @Nullable Bundle savedInstanceState) {
        return inflater.inflate(R.layout.fragment_tiffintom_chat_window, container, false);
    }

    @Override
    public void onViewCreated(@NonNull View view, @Nullable Bundle savedInstanceState) {
        super.onViewCreated(view, savedInstanceState);
        setListeners();
        getAllMessages(null, false);
    }

    @Override
    protected void initViews(View view) {
        super.initViews(view);
        rvMessages = view.findViewById(R.id.rvMessages);
        etMessage = view.findViewById(R.id.etMessage);
        ivSend = view.findViewById(R.id.ivSend);
        llInput = view.findViewById(R.id.llInput);
        ivBackArrow = view.findViewById(R.id.ivBackArrow);
        llBackArrow = view.findViewById(R.id.llBackArrow);
        tvScreenTitle = view.findViewById(R.id.tvScreenTitle);
        tvUserName = view.findViewById(R.id.tvUserName);
        tvScreenTitle.setVisibility(View.VISIBLE);
        tvScreenTitle.setText("Chat");
        recordButton = view.findViewById(R.id.record_button);
        recordView = view.findViewById(R.id.record_view);
        ((ImageView) recordView.findViewById(R.id.basket_img)).setColorFilter(Color.WHITE);
        ivCamera = view.findViewById(R.id.ivCamera);
        ivRecord = view.findViewById(R.id.ivRecord);
        llInputMain = view.findViewById(R.id.llInputMain);
        llRecordingView = view.findViewById(R.id.llRecordingView);
        tvRecordingTime = view.findViewById(R.id.tvRecordingTime);
        ivRecordingStatus = view.findViewById(R.id.ivRecordingStatus);
        chronometer = view.findViewById(R.id.chronometer);

        ivSend.setAlpha(0.4f);
        ivSend.setEnabled(false);
        chatAdapter = new ChatAdapter(feed, (position, data) -> {
            if (data instanceof ChatSubQuestion) {
                ChatSubQuestion subQuestion = (ChatSubQuestion) data;
                feed.remove(position);
                chatAdapter.notifyDataSetChanged();
                if (subQuestion.type.equalsIgnoreCase("sub")) {
                    // We will send question to backend and add static response item in adapter which we are sending as new Message();
                    sendMessage(null, subQuestion.question, new Message(subQuestion.sub_question.get(0).question, loggedInUser.id, 1, CommonFunctions.formatMili(System.currentTimeMillis(), "yyyy-MM-dd'T'hh:mm:ssZ")), null, null, "0");
                    rvMessages.scrollToPosition(feed.size() - 1);
                    toggleInputLayout(true);
                } else if (subQuestion.type.equalsIgnoreCase("order_help")) {
                    sendMessage(null, subQuestion.question, new Message(true, subQuestion.question, loggedInUser.id, 0, CommonFunctions.formatMili(System.currentTimeMillis(), "yyyy-MM-dd'T'hh:mm:ssZ")), null, null, "0");

                    toggleInputLayout(true);
                }
            } else if (data instanceof ChatOrder) {
                ChatOrder chatOrder = (ChatOrder) data;
                feed.remove(position);
                chatAdapter.notifyDataSetChanged();
                sendMessage(chatOrder.id, "Order #" + chatOrder.order_number + " from " + chatOrder.restaurant.restaurant_name, null, "TiffintomChatAfterOrderList", null, "0");
            }
        });
        rvMessages.setLayoutManager(new LinearLayoutManager(getContext(), RecyclerView.VERTICAL, false));
        rvMessages.setAdapter(chatAdapter);
    }

    private void setListeners() {
        blinkRecording();
        ivSend.setOnClickListener(view -> {
            if (!Validators.isNullOrEmpty(etMessage.getText().toString()) && etMessage.getText().toString().trim().length() > 0) {
                if (!messageSent) {
                    toggleInputLayout(false);
                    feed.add(new Message(etMessage.getText().toString(), loggedInUser.id, 0, CommonFunctions.formatMili(System.currentTimeMillis(), "yyyy-MM-dd'T'hh:mm:ssZ")));
                    feed.add(chatMainQuestions.get(0));
                    chatAdapter.notifyDataSetChanged();
                    messageSent = true;
                    CommonFunctions.hideKeyboard(getActivity(), etMessage);
                    rvMessages.scrollToPosition(feed.size() - 1);
                } else {
                    sendMessage(null, etMessage.getText().toString().trim(), null, null, null, "0");
                }
                etMessage.setText("");
            }
        });
        rvMessages.addOnLayoutChangeListener((view, left, top, right, bottom, oldLeft, oldTop, oldRight, oldBottom) -> {
            if (bottom < oldBottom) {
                rvMessages.postDelayed(new Runnable() {
                    @Override
                    public void run() {
                        rvMessages.scrollToPosition(
                                rvMessages.getAdapter().getItemCount() - 1);
                    }
                }, 100);
            }

        });
        ivCamera.setOnClickListener(view -> {
            if (checkGallaryPermission())
                pickImage();
        });
        recordButton.setRecordView(recordView);
        recordView.setOnRecordListener(new OnRecordListener() {
            @Override
            public void onStart() {
                startRecording();
                //Start Recording..
                llInputMain.setVisibility(View.INVISIBLE);
                LogUtils.e("RecordView:::", "onStart");
            }

            @Override
            public void onCancel() {
                //On Swipe To Cancel
                cancelRecording();
                LogUtils.e("RecordView:::", "onCancel");

            }

            @Override
            public void onFinish(long recordTime) {
                //Stop Recording..
                stopRecording();
                llInputMain.setVisibility(View.VISIBLE);
                LogUtils.e("RecordView:::", "onFinish");

                LogUtils.e("RecordTime:::", String.valueOf(recordTime));
            }

            @Override
            public void onLessThanSecond() {
                //When the record time is less than One Second
                llInputMain.setVisibility(View.VISIBLE);
                cancelRecording();
                LogUtils.e("RecordView:::", "onLessThanSecond");
            }
        });
        recordView.setOnBasketAnimationEndListener(new OnBasketAnimationEnd() {
            @Override
            public void onAnimationEnd() {
                LogUtils.e("RecordView", "Basket Animation Finished");
                llInputMain.setVisibility(View.VISIBLE);
            }
        });

        ivRecord.setOnClickListener(view -> {
        });
        ivRecord.setOnTouchListener((view, event) -> {
            switch (event.getAction()) {
                case MotionEvent.ACTION_DOWN:
                    startRecording();
                    startRecordingTimer();
                    //This is where my code for movement is initialized to get original location.
                    break;
                case MotionEvent.ACTION_UP:
                    stopRecordingTimer();
                    if (mediaRecorder != null) {
                        stopRecording();
                    }
                    return false;
                case MotionEvent.ACTION_MOVE:
                    //Code for movement here. This may include using a window manager to update the view
                    break;
            }
            return true;
        });
        llBackArrow.setOnClickListener(view -> {
            getActivity().onBackPressed();
        });
        etMessage.addTextChangedListener(new TextWatcher() {
            @Override
            public void beforeTextChanged(CharSequence charSequence, int i, int i1, int i2) {

            }

            @Override
            public void onTextChanged(CharSequence charSequence, int i, int i1, int i2) {
                if (etMessage.getText().toString().trim().length() == 0) {
                    ivSend.setAlpha(0.4f);
                    ivSend.setEnabled(false);
                } else {
                    ivSend.setAlpha(1f);
                    ivSend.setEnabled(true);
                }
            }

            @Override
            public void afterTextChanged(Editable editable) {

            }
        });

    }

    private boolean checkGallaryPermission() {
        ArrayList<String> deniedPermissions = new ArrayList<>();
        ArrayList<String> rationalDeniedPermissions = new ArrayList<>();

        for (String permission : requiredPermissions) {
            if (ActivityCompat.checkSelfPermission(getActivity(), permission) == PackageManager.PERMISSION_DENIED) {
                deniedPermissions.add(permission);
                if (ActivityCompat.shouldShowRequestPermissionRationale(getActivity(), permission))
                    rationalDeniedPermissions.add(permission);

            }
        }
        if (rationalDeniedPermissions.size() > 0) {
            // Show why we need permissions in dialog and on dialog yes press request permissions;

            new android.app.AlertDialog.Builder(getActivity())
                    .setTitle("Storage permission required")
                    .setMessage("To access your photo we required storage permission, Please allow to continue.")
                    .setPositiveButton("ALLOW", (dialogInterface, i) -> {
                        String[] mStringArray = new String[deniedPermissions.size()];
                        mStringArray = deniedPermissions.toArray(mStringArray);

                        requestPermissions(mStringArray, PERMISSION_CODE_STORAGE);
                    })
                    .setNegativeButton("DENY", (dialogInterface, i) -> dialogInterface.dismiss())
                    .create()
                    .show();

            return false;
        } else {
            if (deniedPermissions.size() > 0) {
                String[] mStringArray = new String[deniedPermissions.size()];
                mStringArray = deniedPermissions.toArray(mStringArray);

                requestPermissions(mStringArray, PERMISSION_CODE_STORAGE);
                return false;
            } else {
                return true;
            }
        }

    }

    private void toggleInputLayout(boolean show) {
        if (show) {
            llInput.setVisibility(View.VISIBLE);
            recordButton.setVisibility(View.VISIBLE);
        } else {
            llInput.setVisibility(View.GONE);
            recordButton.setVisibility(View.GONE);

        }
    }

    private void cancelRecording() {
        if (mediaRecorder != null) {
            try {
                mediaRecorder.stop();
                mediaRecorder.release();
                mediaRecorder = null;
                if (new File(AudioSavePathInDevice).exists()) {
                    new File(AudioSavePathInDevice).delete();
                }
                LogUtils.e("Recoding Cancelled");
            } catch (Exception e) {
                FirebaseCrashlytics.getInstance().log("On Cancel Recording::" + e.getMessage());
            }
        }
    }

    private void startRecordingTimer() {
        llInputMain.setVisibility(View.GONE);
        llRecordingView.setVisibility(View.VISIBLE);
        tvRecordingTime.setText("Recording ");
        chronometer.setBase(SystemClock.elapsedRealtime());
        chronometer.start();
    }

    private void stopRecordingTimer() {
        llInputMain.setVisibility(View.VISIBLE);
        llRecordingView.setVisibility(View.GONE);
        chronometer.stop();
    }

    private void blinkRecording() {
        Animation animation = new AlphaAnimation(1, 0); //to change visibility from visible to invisible
        animation.setDuration(500); //1 second duration for each animation cycle
        animation.setInterpolator(new LinearInterpolator());
        animation.setRepeatCount(Animation.INFINITE); //repeating indefinitely
        animation.setRepeatMode(Animation.REVERSE); //animation will start from end point once ended.
        ivRecordingStatus.startAnimation(animation); //to start animation
    }

    private void startRecording() {
        random = new Random();
        if (checkPermission()) {

            String root = Environment.getExternalStorageDirectory().getAbsolutePath();
            File directory = new File(root + "/TiffinTom/Chat/Audio/");
            String audioFileName = CommonFunctions.formatMili(System.currentTimeMillis(), "yyyy-MM-dd HH:mm:ss") + " Recording.mp3";
            if (!directory.exists()) directory.mkdirs();
            audioFile = new File(directory, audioFileName);

            mediaRecorder = new MediaRecorder();
            mediaRecorder.setAudioSource(MediaRecorder.AudioSource.MIC);
            mediaRecorder.setOutputFormat(MediaRecorder.OutputFormat.MPEG_4);
            mediaRecorder.setAudioEncoder(MediaRecorder.OutputFormat.AMR_NB);
            if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.O) {
                mediaRecorder.setOutputFile(audioFile);
            } else {
                mediaRecorder.setOutputFile(audioFile.getAbsolutePath());
            }
            try {
                mediaRecorder.prepare();
                mediaRecorder.start();
            } catch (IllegalStateException e) {
                // TODO Auto-generated catch block
                e.printStackTrace();
            } catch (IOException e) {
                // TODO Auto-generated catch block
                e.printStackTrace();
            }

        } else {
            requestPermission();
        }
    }

    private void stopRecording() {
        try {
            mediaRecorder.stop();
            mediaRecorder.release();
            sendMessage(null, null, null, null, audioFile, "2");
            mediaRecorder = null;
        } catch (Exception e) {
            FirebaseCrashlytics.getInstance().log("On Stop Recording::" + e.getMessage());
        }
    }

    boolean messagesFetched;
    boolean Progrssshow = true;

    private void getAllMessages(String lastMsgId, boolean autoRefresh) {
        if (Progrssshow) {
            progressDialog.show();
            Progrssshow = false;
        }
        for (Object o : feed) {
            if (o instanceof Message) {
                if (((Message) o).id == Integer.parseInt(lastMsgId)) {
                    feed.remove(lastMsgId);
                }
            }
        }
        HashMap<String, String> hashMap = new HashMap<>();
        hashMap.put("user_id", String.valueOf(loggedInUser.id));
        hashMap.put("nopaginate", "1");
        if (lastMsgId != null) {
            hashMap.put("last_id", lastMsgId);
        }
        AndroidNetworking.get(ApiEndPoints.NEW_CHAT)
                .addQueryParameter(hashMap)
                .build()
                .getAsJSONArray(new JSONArrayRequestListener() {
                    @Override
                    public void onResponse(JSONArray response) {

                        progressDialog.dismiss();
                        LogUtils.e("Fetching messages...");
                        messagesFetched = true;
                        if (SendMessageCalled == false) {
                            Type type = new TypeToken<List<Message>>() {
                            }.getType();
                            LogUtils.e("Found messages...");
                            oldMessages.clear();
                            oldMessages.addAll(new Gson().fromJson(response.toString(), type));
                            feed.addAll(oldMessages);
                            chatAdapter.notifyDataSetChanged();
                            rvMessages.scrollToPosition(feed.size() - 1);
                        } else {
                            SendMessageCalled = false;
                        }
                        if (!autoRefresh)
                            getQuestionMessages("TiffintomChatList");
                    }

                    @Override
                    public void onError(ANError anError) {
                        anError.printStackTrace();
                        progressDialog.dismiss();
                        if (!autoRefresh)
                            getQuestionMessages("TiffintomChatList");
                        if (!autoRefresh && !CommonFunctions.isConnected(getActivity()))
                            myApp.noInternet(getActivity());
                    }
                });
    }

    private void getMiniOrders() {
        HashMap<String, String> hashMap = new HashMap<>();
        hashMap.put("customer_id", String.valueOf(loggedInUser.id));
        hashMap.put("per_page", "3");
        hashMap.put("page", "1");

        AndroidNetworking.get(ApiEndPoints.NEW_ORDERS)
                .addQueryParameter(hashMap)
                .build()
                .getAsJSONObject(new JSONObjectRequestListener() {
                    @Override
                    public void onResponse(JSONObject response) {
                        Type type = new TypeToken<List<ChatOrder>>() {
                        }.getType();

                        try {
                            miniOrders.clear();
                            miniOrders.addAll(new Gson().fromJson(response.getJSONArray("data").toString(), type));
                            feed.add(new ChatMiniOrderSnippet("Hello, Please select your order for which you seek support", miniOrders));
                        } catch (JSONException e) {
                            e.printStackTrace();
                        }
                        chatAdapter.notifyDataSetChanged();
                        rvMessages.scrollToPosition(feed.size() - 1);
                    }

                    @Override
                    public void onError(ANError anError) {
                        anError.printStackTrace();
                        if (!CommonFunctions.isConnected(getActivity()))
                            myApp.noInternet(getActivity());
                    }
                });
    }

    private void getQuestionMessages(String page) {
        AndroidNetworking.post(ApiEndPoints.NEW_CHAT + "/chat-general-question")
                .addBodyParameter("id", String.valueOf(loggedInUser.id))
                .addBodyParameter("nopaginate", "1")
                .build()
                .getAsJSONArray(new JSONArrayRequestListener() {
                    @Override
                    public void onResponse(JSONArray response) {
                        Type type = new TypeToken<List<ChatMainQuestion>>() {
                        }.getType();
                        if (page.equals("TiffintomChatList")) {
                            chatMainQuestions.clear();
                            chatMainQuestions.addAll(new Gson().fromJson(response.toString(), type));
                            Message message = new Message(chatMainQuestions.get(0).question, loggedInUser.id, 1, CommonFunctions.formatMili(System.currentTimeMillis(), "yyyy-MM-dd'T'hh:mm:ssZ"));
                            feed.add(message);
                            chatAdapter.notifyDataSetChanged();
                            rvMessages.scrollToPosition(feed.size() - 1);
                            setAutoFetch();
                        } else if (page.equals("TiffintomChatAfterOrderList")) {
                            getOrderQuestionMessages();
                        }

                    }

                    @Override
                    public void onError(ANError anError) {
                        anError.printStackTrace();
                    }
                });
    }

    private void getOrderQuestionMessages() {
        AndroidNetworking.get(ApiEndPoints.NEW_CHAT + "/chat-order-question")
                .addQueryParameter("id", String.valueOf(loggedInUser.id))
                .addQueryParameter("nopaginate", "1")
                .build()
                .getAsJSONArray(new JSONArrayRequestListener() {
                    @Override
                    public void onResponse(JSONArray response) {
                        Type type = new TypeToken<List<ChatMainQuestion>>() {
                        }.getType();

                        chatOrdersQuestions.clear();
                        chatOrdersQuestions.addAll(new Gson().fromJson(response.toString(), type));

                        ChatMainQuestion chatMainQuestion = chatOrdersQuestions.get(0);
                        chatMainQuestion.forOrder = true;
                        feed.add(chatMainQuestion);
                        chatAdapter.notifyDataSetChanged();
                        rvMessages.scrollToPosition(feed.size() - 1);
                    }

                    @Override
                    public void onError(ANError anError) {
                        anError.printStackTrace();
                    }
                });
    }

    //0 msg,1 img 2 audio
    private void sendMessage(String orderId, String message, Message msg, String nextPageQuestions, File file, String type) {
        if (type.equals("2")) {
            if (getActivity() != null && progressDialog != null)
                getActivity().runOnUiThread(() -> progressDialog.show());
        }
        HashMap<String, String> params = new HashMap<>();

        if (type.equalsIgnoreCase("0"))
            params.put("message", message);
        else params.put("message", "");
        params.put("user_id", String.valueOf(loggedInUser.id));
        if (!Validators.isNullOrEmpty(orderId)) {
            params.put("order_id", orderId);
        }
        params.put("admin", "0");
        params.put("status", "1");
        params.put("is_attach", type);
        ANRequest.MultiPartBuilder androidNetworking = AndroidNetworking.upload(ApiEndPoints.NEW_CHAT)
                .addMultipartParameter(params);
        if (!type.equalsIgnoreCase("0")) {
            androidNetworking.addMultipartFile("attach", file);
        }
        androidNetworking
                .build()
                .getAsJSONObject(new JSONObjectRequestListener() {
                    @Override
                    public void onResponse(JSONObject response) {
                        Message m = new Gson().fromJson(response.toString(), Message.class);
                        feed.add(m);
                        if (msg != null) {
                            if (msg.order_help) {
                                getMiniOrders();
                            } else {
                                feed.add(msg);
                            }

                        }

                        SendMessageCalled = true;

                        chatAdapter.notifyDataSetChanged();
                        rvMessages.scrollToPosition(feed.size() - 1);

                        if (file != null && file.exists()) {
                            file.delete();
                        }
//
                        if (!Validators.isNullOrEmpty(nextPageQuestions)) {
                            getQuestionMessages(nextPageQuestions);
                        }
                        if (getActivity() != null)
                            getActivity().runOnUiThread(() -> progressDialog.dismiss());
                    }

                    @Override
                    public void onError(ANError anError) {
                        anError.printStackTrace();
                        if (getActivity() != null)
                            getActivity().runOnUiThread(() -> progressDialog.dismiss());
                        if (!CommonFunctions.isConnected(getActivity()))
                            myApp.noInternet(getActivity());

                    }
                });
    }

    private void pickImage() {
        final CharSequence[] options = {"Take Photo", "Choose from Gallery", "Cancel"};
        AlertDialog.Builder builder = new AlertDialog.Builder(getActivity());
        builder.setItems(options, new DialogInterface.OnClickListener() {
            @Override
            public void onClick(DialogInterface dialog, int item) {
                if (options[item].equals("Take Photo")) {
                    dispatchTakePictureIntent();
                } else if (options[item].equals("Choose from Gallery")) {
                    Intent intent = new Intent(Intent.ACTION_PICK, android.provider.MediaStore.Images.Media.EXTERNAL_CONTENT_URI);
                    startActivityForResult(intent, CODE_IMAGE_PICKER);
                } else if (options[item].equals("Cancel")) {
                    dialog.dismiss();
                }
            }
        });
        builder.show();
    }

    private void dispatchTakePictureIntent() {
        Intent takePictureIntent = new Intent(MediaStore.ACTION_IMAGE_CAPTURE);
        // Ensure that there's a camera activity to handle the intent
        if (takePictureIntent.resolveActivity(getActivity().getPackageManager()) != null) {
            // Create the File where the photo should go
            File photoFile = null;
            try {
                photoFile = createImageFile();
            } catch (IOException ex) {
                // Error occurred while creating the File
            }
            // Continue only if the File was successfully created
            if (photoFile != null) {
                Uri photoURI = FileProvider.getUriForFile(getActivity(), BuildConfig.APPLICATION_ID + ".fileprovider",
                        photoFile);
                takePictureIntent.putExtra(MediaStore.EXTRA_OUTPUT, photoURI);
                startActivityForResult(takePictureIntent, CODE_CAMERA_PICKER);
            }
        }
    }

    String currentPhotoPath;

    private File createImageFile() throws IOException {
        // Create an image_url file name
        String timeStamp = new SimpleDateFormat("yyyyMMdd_HHmmss").format(new Date());
        String imageFileName = "JPEG_" + timeStamp + "_";
        File storageDir = getActivity().getExternalFilesDir(Environment.DIRECTORY_PICTURES);
        File image = File.createTempFile(
                imageFileName,  /* prefix */
                ".jpg",         /* suffix */
                storageDir      /* directory */
        );

        // Save a file: path for use with ACTION_VIEW intents
        currentPhotoPath = image.getAbsolutePath();
        return image;
    }

    private void requestPermission() {
        ActivityCompat.requestPermissions(getActivity(), new
                String[]{WRITE_EXTERNAL_STORAGE, RECORD_AUDIO}, RequestPermissionCode);
    }

    @Override
    public void onRequestPermissionsResult(int requestCode,
                                           String permissions[], int[] grantResults) {
        if (requestCode == RequestPermissionCode) {
            if (grantResults.length > 0) {
                boolean StoragePermission = grantResults[0] ==
                        PackageManager.PERMISSION_GRANTED;
                boolean RecordPermission = grantResults[1] ==
                        PackageManager.PERMISSION_GRANTED;

                if (!StoragePermission || !RecordPermission) {
                    ToastUtils.makeToast(getActivity(), "Permissions denied");
                }
            }
        } else if (requestCode == PERMISSION_CODE_STORAGE) {
            ArrayList<String> rationalDeniedPermissions = new ArrayList<>();
            ArrayList<String> nonRationalDeniedPermission = new ArrayList<>();
            for (String permission : permissions) {
                if (ActivityCompat.checkSelfPermission(getActivity(), permission) == PackageManager.PERMISSION_DENIED) {
                    if (ActivityCompat.shouldShowRequestPermissionRationale(getActivity(), permission))
                        rationalDeniedPermissions.add(permission);
                    else {
                        nonRationalDeniedPermission.add(permission);
                    }
                }
            }

            if (nonRationalDeniedPermission.size() == 0 && rationalDeniedPermissions.size() == 0) {
                pickImage();
            } else {
                if (nonRationalDeniedPermission.size() > 0) {
                    new android.app.AlertDialog.Builder(getActivity())
                            .setTitle("Some permissions required")
                            .setMessage("To access your photo we require storage and camera permission, Please allow to continue.")
                            .setPositiveButton("ALLOW", (dialogInterface, i) -> {
                                Intent intent = new Intent();
                                intent.setAction(Settings.ACTION_APPLICATION_DETAILS_SETTINGS);
                                Uri uri = Uri.fromParts("package", getActivity().getPackageName(), null);
                                intent.setData(uri);
                                startActivity(intent);
                            })
                            .setNegativeButton("DENY", (dialogInterface, i) -> dialogInterface.dismiss())
                            .create()
                            .show();
                } else {
                    ToastUtils.makeToast(getActivity(), "Permission denied");
                }
            }
        }
    }

    public boolean checkPermission() {
        int result = ContextCompat.checkSelfPermission(getApplicationContext(),
                WRITE_EXTERNAL_STORAGE);
        int result1 = ContextCompat.checkSelfPermission(getApplicationContext(),
                RECORD_AUDIO);
        return result == PackageManager.PERMISSION_GRANTED &&
                result1 == PackageManager.PERMISSION_GRANTED;
    }

    Handler handler = new Handler();
    Runnable runnable = new Runnable() {
        @Override
        public void run() {
            String lastMsgId = null;
            for (Object o : feed) {
                if (o instanceof Message) {
                    if (((Message) o).id != 0) {
                        lastMsgId = String.valueOf(((Message) o).id);
                    }
                }
            }
            getAllMessages(lastMsgId, true);
            handler.postDelayed(runnable, 5 * 1000);
        }
    };

    private void setAutoFetch() {
        handler.postDelayed(runnable, 5 * 1000);
    }

    Uri fileUri;
    File userImageFile;

    @Override
    public void onActivityResult(int requestCode, int resultCode, @Nullable Intent data) {
        super.onActivityResult(requestCode, resultCode, data);
        if (requestCode == CODE_IMAGE_PICKER && resultCode == Activity.RESULT_OK) {
            if (data == null) {
                //error
                return;
            }
            try {
                Uri uri = data.getData();
                userImageFile = FileUtil.from(getActivity(), uri);
                if (userImageFile != null && userImageFile.exists())
                    sendMessage(null, null, null, null, userImageFile, "1");
            } catch (IOException e) {
                e.printStackTrace();
            }
        }

        if (requestCode == CODE_CAMERA_PICKER) {
            FileUtil.refreshGallery(getActivity(), currentPhotoPath);
            Bitmap photo = BitmapFactory.decodeFile(currentPhotoPath);
            userImageFile = FileUtil.saveScaledPhotoToFile(getActivity(), photo, currentPhotoPath);
            if (userImageFile != null && userImageFile.exists())
                sendMessage(null, null, null, null, userImageFile, "1");

        }
        if (requestCode == PERMISSION_CODE_STORAGE) {
            if (resultCode == Activity.RESULT_OK && data != null) {
                ArrayList<Uri> dataList = data.getParcelableArrayListExtra(FilePickerConst.KEY_SELECTED_MEDIA);
                if (dataList != null && dataList.size() > 0) {
                    ArrayList<Uri> files = new ArrayList<>(dataList);
                    fileUri = files.get(0);
                    userImageFile = myApp.saveBitmapToFile(new File(getPath(fileUri)));
                    if (userImageFile != null && userImageFile.exists())
                        sendMessage(null, null, null, null, userImageFile, "1");
                }
            }
        }
    }

    public String getPath(Uri uri) {
        String[] projection = {MediaStore.Images.Media.DATA};
        Cursor cursor = getActivity().getContentResolver().query(uri, projection, null, null, null);
        if (cursor == null) return null;
        int column_index = cursor.getColumnIndexOrThrow(MediaStore.Images.Media.DATA);
        cursor.moveToFirst();
        String s = cursor.getString(column_index);
        cursor.close();
        return s;
    }

    @Override
    public void onResume() {
        super.onResume();
        if (messagesFetched) setAutoFetch();
        if (getActivity() != null && broadcastReceiver != null)
            LocalBroadcastManager.getInstance(getActivity()).registerReceiver(broadcastReceiver, new IntentFilter(Config.PUSH_NOTIFICATION));
    }

    @Override
    public void onStop() {
        handler.removeCallbacks(runnable);
        if (mediaRecorder != null) mediaRecorder.stop();
        if (getActivity() != null && broadcastReceiver != null)
            LocalBroadcastManager.getInstance(getActivity()).unregisterReceiver(broadcastReceiver);
        super.onStop();
    }
}
