package com.tiffintom.fragment;

import android.app.Dialog;
import android.content.DialogInterface;
import android.os.Build;
import android.os.Bundle;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.EditText;
import android.widget.FrameLayout;
import android.widget.LinearLayout;
import android.widget.TextView;

import androidx.annotation.NonNull;
import androidx.annotation.Nullable;
import androidx.appcompat.app.AlertDialog;
import androidx.recyclerview.widget.LinearLayoutManager;
import androidx.recyclerview.widget.RecyclerView;

import com.androidnetworking.AndroidNetworking;
import com.androidnetworking.error.ANError;
import com.androidnetworking.interfaces.JSONObjectRequestListener;
import com.google.android.material.bottomsheet.BottomSheetBehavior;
import com.google.android.material.bottomsheet.BottomSheetDialog;
import com.google.android.material.bottomsheet.BottomSheetDialogFragment;
import com.google.gson.Gson;
import com.tiffintom.MyApp;
import com.tiffintom.R;
import com.tiffintom.adapters.VoucherListAdapter;
import com.tiffintom.common.CommonFunctions;
import com.tiffintom.common.Validators;
import com.tiffintom.interfaces.DialogDismissListener;
import com.tiffintom.models.ApiError;
import com.tiffintom.models.Restaurant;
import com.tiffintom.models.User;
import com.tiffintom.models.Voucher;
import com.tiffintom.network.ApiUtils;
import com.tiffintom.utils.LogUtils;
import com.tiffintom.utils.ToastUtils;

import org.json.JSONException;
import org.json.JSONObject;

import java.util.ArrayList;

/**
 * Created by Amrish on 18-09-2020.
 */
public class VoucherListBottomsheetFragment extends BottomSheetDialogFragment {
    public static VoucherListBottomsheetFragment getInstance(Restaurant restaurant,String OrderType,String deliveryDate) {
        Bundle bundle = new Bundle();
        bundle.putString("restaurant", new Gson().toJson(restaurant));
        bundle.putString("orderType",OrderType);
        bundle.putString("deliveryDate",deliveryDate);
        VoucherListBottomsheetFragment voucherListBottomsheetFragment = new VoucherListBottomsheetFragment();
        voucherListBottomsheetFragment.setArguments(bundle);
        return voucherListBottomsheetFragment;
    }

    private User loggedInUser = MyApp.getInstance().getMyPreferences().getLoggedInUser();
    private DialogDismissListener dialogDismissListener;
    private MyApp myApp = MyApp.getInstance();
    private RecyclerView rvVoucherList;
    private VoucherListAdapter voucherListAdapter;
    private EditText etPromocode;
    private TextView tvAvailableVouchers, tvApply, tvMessage;
    private Restaurant restaurant;
    private String orderType,deliveryDate;
    private AlertDialog progressDialog;
    private LinearLayout llNoData;
    private ArrayList<Object> feed = new ArrayList<>();

    public void setDialogDismissListener(DialogDismissListener dialogDismissListener) {
        this.dialogDismissListener = dialogDismissListener;
    }

    @Override
    public int getTheme() {
        if (Build.VERSION.SDK_INT > 21) {
            return R.style.BottomSheetDialogTheme;
        } else {
            return super.getTheme();
        }
    }

    @NonNull
    @Override
    public Dialog onCreateDialog(Bundle savedInstanceState) {

        BottomSheetDialog dialog = (BottomSheetDialog) super.onCreateDialog(savedInstanceState);

        dialog.setOnShowListener((DialogInterface.OnShowListener) dialog1 -> {
            BottomSheetDialog d = (BottomSheetDialog) dialog1;
            FrameLayout bottomSheet = (FrameLayout) d.findViewById(com.google.android.material.R.id.design_bottom_sheet);
            if (bottomSheet != null)
                BottomSheetBehavior.from(bottomSheet).setState(BottomSheetBehavior.STATE_EXPANDED);
        });

        return dialog;
    }

    @Nullable
    @Override
    public View onCreateView(@NonNull LayoutInflater inflater, @Nullable ViewGroup container, @Nullable Bundle savedInstanceState) {
        return inflater.inflate(R.layout.fragment_voucher_list, container, false);

    }

    @Override
    public void onViewCreated(@NonNull View view, @Nullable Bundle savedInstanceState) {
        super.onViewCreated(view, savedInstanceState);
        if (getArguments() != null) {
            restaurant = new Gson().fromJson(getArguments().getString("restaurant"), Restaurant.class);
            orderType=getArguments().getString("orderType");
            deliveryDate=getArguments().getString("deliveryDate");
        }
        initViews(view);
        setListeners();
    }

    private void setListeners() {
        tvApply.setOnClickListener(view -> {
            if (!Validators.isNullOrEmpty(etPromocode.getText().toString())) {
                applyPromocode();
            }
        });

    }

    private void initViews(View view) {
        progressDialog = CommonFunctions.customProgressDialog(getContext(), "");
        etPromocode = view.findViewById(R.id.etPromocode);
        llNoData = view.findViewById(R.id.llNoData);
        tvMessage = view.findViewById(R.id.tvMessage);
        tvApply = view.findViewById(R.id.tvApply);
        rvVoucherList = view.findViewById(R.id.rvVouchers);
        tvAvailableVouchers = view.findViewById(R.id.tvAvailableVouchers);
        if (restaurant != null) {
            if (restaurant.voucherLists != null && restaurant.voucherLists.size() > 0) {
                feed.addAll(restaurant.voucherLists);
                llNoData.setVisibility(View.GONE);
            }
            if (restaurant.voucherLists != null && restaurant.voucherLists.size() > 0) {
                feed.addAll(restaurant.voucherLists);
                llNoData.setVisibility(View.GONE);
            }
        }
        voucherListAdapter = new VoucherListAdapter(feed, (position, data) -> {
            if (data instanceof Voucher) {
                if (dialogDismissListener != null) {
                    dialogDismissListener.onDialogDismiss(data);
                    dismiss();
                }
            }
        });
        rvVoucherList.setAdapter(voucherListAdapter);
        rvVoucherList.setLayoutManager(new LinearLayoutManager(getActivity(), RecyclerView.VERTICAL, false));

        if (voucherListAdapter.getItemCount() == 0) {
            llNoData.setVisibility(View.VISIBLE);
            tvMessage.setText("There doesn't seem to be any promo codes available right now");
            tvAvailableVouchers.setVisibility(View.GONE);
        } else {
            llNoData.setVisibility(View.GONE);
            tvAvailableVouchers.setVisibility(View.VISIBLE);
        }
    }

    private void applyPromocode() {
        if (getActivity() != null) {
            getActivity().runOnUiThread(() -> {
                progressDialog.show();
                CommonFunctions.hideKeyboard(getActivity(), etPromocode);
            });
        }
        ApiUtils.applyPromoCode(String.valueOf(loggedInUser.id), String.valueOf(restaurant.id), etPromocode.getText().toString(),orderType,deliveryDate)
                .getAsJSONObject(new JSONObjectRequestListener() {
                    @Override
                    public void onResponse(JSONObject response) {
                               Voucher appliedVoucher = new Gson().fromJson(response.toString(), Voucher.class);
                                if (dialogDismissListener != null) {
                                    dialogDismissListener.onDialogDismiss(appliedVoucher);
                                }
                                dismiss();
                        if (getActivity() != null) {
                            getActivity().runOnUiThread(() -> {
                                progressDialog.dismiss();
                            });
                        }
                    }

                    @Override
                    public void onError(ANError anError) {
                        anError.printStackTrace();
                        LogUtils.e("APPLY PROMO ANERROR");
                        if (anError.getErrorCode() == 400) {
                            ApiError apiError = anError.getErrorAsObject(ApiError.class);
                            ToastUtils.makeSnackToast(getActivity(), apiError.getMessage());
                        }
                        if (getActivity() != null) {
                            getActivity().runOnUiThread(() -> {
                                progressDialog.dismiss();
                            });
                        }

                    }
                });
    }

    @Override
    public void onDismiss(@NonNull DialogInterface dialog) {
        AndroidNetworking.cancel("recent_orders");
        super.onDismiss(dialog);
    }
}
