package com.tiffintom.fragment;

import android.content.Context;
import android.graphics.Color;
import android.graphics.drawable.ColorDrawable;
import android.os.Bundle;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.ImageView;
import android.widget.LinearLayout;
import android.widget.ProgressBar;
import android.widget.RelativeLayout;
import android.widget.TextView;

import androidx.annotation.NonNull;
import androidx.annotation.Nullable;
import androidx.appcompat.app.AlertDialog;
import androidx.recyclerview.widget.LinearLayoutManager;
import androidx.recyclerview.widget.RecyclerView;
import androidx.viewpager.widget.ViewPager;

import com.airbnb.lottie.LottieAnimationView;
import com.androidnetworking.AndroidNetworking;
import com.androidnetworking.error.ANError;
import com.androidnetworking.interfaces.JSONArrayRequestListener;
import com.androidnetworking.interfaces.JSONObjectRequestListener;
import com.google.gson.Gson;
import com.google.gson.reflect.TypeToken;
import com.make.dots.dotsindicator.DotsIndicator;
import com.ocnyang.pagetransformerhelp.cardtransformer.AlphaAndScalePageTransformer;
import com.tiffintom.R;
import com.tiffintom.adapters.SaveCardsUIViewPagerAdapter;
import com.tiffintom.adapters.SavedCardsAdapter;
import com.tiffintom.adapters.SavedCardsUIAdapter;
import com.tiffintom.base.BaseFragment;
import com.tiffintom.common.CommonFunctions;
import com.tiffintom.interfaces.DialogDismissListener;
import com.tiffintom.interfaces.RecyclerViewItemClickListener;
import com.tiffintom.models.PaymentMethod;
import com.tiffintom.models.RequestCreator;
import com.tiffintom.models.Restaurant;
import com.tiffintom.models.SavedCard;
import com.tiffintom.models.User;
import com.tiffintom.network.ApiEndPoints;
import com.tiffintom.network.ApiUtils;
import com.tiffintom.utils.LogUtils;
import com.tiffintom.utils.ToastUtils;

import org.json.JSONArray;
import org.json.JSONException;
import org.json.JSONObject;

import java.lang.reflect.Type;
import java.util.ArrayList;
import java.util.List;

/**
 * Created by Amrish on 26-08-2020.
 */
public class WalletFragment extends BaseFragment {
    private LinearLayout llCod, llPaypal, llCards;
    private ImageView ivSelectCod, ivSelectPaypal, ivSelectCards;
    private RecyclerView rvCards, rvCardsHorizontal;
    private ProgressBar pbLoading;
    private ArrayList<SavedCard> savedCards = new ArrayList<>();
    private SavedCardsAdapter savedCardsAdapter;
    private SavedCardsUIAdapter savedCardsUIAdapter;
    private User loggedInUser;
    private ImageView ivAddCard, ivAddFirstCard;
    private TextView tvCardsLabel, tvSwipeRemoveCard;
    private ViewPager vpCardsHorizontal;
    private DotsIndicator cardIndicator;
    private SaveCardsUIViewPagerAdapter saveCardsUIViewPagerAdapter;
    private RelativeLayout rlCardLayout, rlFirstCardLayout;
    private LottieAnimationView lodingView;
    private TextView tvCashDefault, tvPaypalDefault, tvCardDefault;

    public static WalletFragment getInstance() {
        WalletFragment fragment = new WalletFragment();
        Bundle bundle = new Bundle();
        fragment.setArguments(bundle);
        return fragment;
    }

    @Nullable
    @Override
    public View onCreateView(@NonNull LayoutInflater inflater, @Nullable ViewGroup container, @Nullable Bundle savedInstanceState) {
        return inflater.inflate(R.layout.fragment_wallet, container, false);
    }

    @Override
    public void onViewCreated(@NonNull View view, @Nullable Bundle savedInstanceState) {
        super.onViewCreated(view, savedInstanceState);
        fetchCards();
        setListener();
        updateViews();
    }

    private void updateViews() {
        updateDefaultPaymentMethods();
    }

    private void updateDefaultPaymentMethods() {
        PaymentMethod defaultPaymentMethod = myApp.getMyPreferences().getDefaultPaymentMethod();
        if (defaultPaymentMethod != null) {
            if (defaultPaymentMethod.type.equalsIgnoreCase("cod")) {
                ivSelectCod.setImageResource(R.drawable.ic_baseline_done_24);
                ivSelectPaypal.setImageResource(R.drawable.dot_grey_checkbox);//dot_grey_circle
                ivSelectCards.setImageResource(R.drawable.dot_grey_checkbox);
                tvCashDefault.setVisibility(View.VISIBLE);
                tvCardDefault.setVisibility(View.GONE);
                tvPaypalDefault.setVisibility(View.GONE);
            }
            if (defaultPaymentMethod.type.equalsIgnoreCase("stripe")) {
                ivSelectCards.setImageResource(R.drawable.ic_baseline_done_24);
                ivSelectPaypal.setImageResource(R.drawable.dot_grey_checkbox);
                ivSelectCod.setImageResource(R.drawable.dot_grey_checkbox);
                tvCardDefault.setVisibility(View.VISIBLE);
                tvCashDefault.setVisibility(View.GONE);
                tvPaypalDefault.setVisibility(View.GONE);
            }
            if (defaultPaymentMethod.type.equalsIgnoreCase("paypal")) {
                ivSelectPaypal.setImageResource(R.drawable.ic_baseline_done_24);
                ivSelectCards.setImageResource(R.drawable.dot_grey_checkbox);
                ivSelectCod.setImageResource(R.drawable.dot_grey_checkbox);
                tvPaypalDefault.setVisibility(View.VISIBLE);
                tvCashDefault.setVisibility(View.GONE);
                tvCardDefault.setVisibility(View.GONE);
            }
        } else {
            ivSelectCod.setImageResource(R.drawable.ic_baseline_done_24);
            ivSelectPaypal.setImageResource(R.drawable.dot_grey_checkbox);
            ivSelectCards.setImageResource(R.drawable.dot_grey_checkbox);
            tvCardDefault.setVisibility(View.GONE);
            tvPaypalDefault.setVisibility(View.GONE);
            tvCashDefault.setVisibility(View.VISIBLE);
        }
    }

    @Override
    protected void initViews(View view) {
        super.initViews(view);
        loggedInUser = myApp.getMyPreferences().getLoggedInUser();
        llCards = view.findViewById(R.id.cvCard);
        llCod = view.findViewById(R.id.cvCod);
        llPaypal = view.findViewById(R.id.cvPaypal);
        rvCards = view.findViewById(R.id.rvCards);
        rvCardsHorizontal = view.findViewById(R.id.rvHorizontalCards);
        ivSelectCards = view.findViewById(R.id.ivSelectCard);
        ivSelectCod = view.findViewById(R.id.ivSelectCod);
        ivSelectPaypal = view.findViewById(R.id.ivSelectPaypal);
        pbLoading = view.findViewById(R.id.pbLoading);
        tvCardsLabel = view.findViewById(R.id.tvCardsLabel);
        ivAddCard = view.findViewById(R.id.ivAddCard);
        vpCardsHorizontal = view.findViewById(R.id.vpHorizontalCards);
        cardIndicator = view.findViewById(R.id.dotsIndicator);
        rlCardLayout = view.findViewById(R.id.rl_Card);
        tvSwipeRemoveCard = view.findViewById(R.id.tvSwipeRemove);
        rlFirstCardLayout = view.findViewById(R.id.rl_FirstCard);
        ivAddFirstCard = view.findViewById(R.id.ivAddFirstCard);
        tvCashDefault = view.findViewById(R.id.tvCashDefault);
        tvCardDefault = view.findViewById(R.id.tvCardDefault);
        tvPaypalDefault = view.findViewById(R.id.tvPaypalDefault);

        lodingView = view.findViewById(R.id.lodingView);
        rvCards.setLayoutManager(new LinearLayoutManager(getActivity(), RecyclerView.VERTICAL, false));
        savedCardsAdapter = new SavedCardsAdapter(getActivity(), savedCards, 0, (position, data) -> {
            updateDefaultPaymentMethods();
        }, (position, data) -> {
            if (data instanceof SavedCard)
                deleteConfirmation(((SavedCard) data).id);
        });
        rvCards.setAdapter(savedCardsAdapter);

////        rvCardsHorizontal.setLayoutManager(new LinearLayoutManager(getActivity(), RecyclerView.HORIZONTAL, false));
//        savedCardsUIAdapter = new SavedCardsUIAdapter(getActivity(), savedCards, rvCardsHorizontal, 0, new RecyclerViewItemClickListener() {
//            @Override
//            public void onItemClick(int position, Object data) {
//                if (data instanceof SavedCard)
//                    WalletFragment.this.deleteConfirmation(((SavedCard) data).booking_id);
//            }
//        });

        // rvCardsHorizontal.setAdapter(savedCardsUIAdapter);

        saveCardsUIViewPagerAdapter = new SaveCardsUIViewPagerAdapter(getActivity(), savedCards, 0, new RecyclerViewItemClickListener() {
            @Override
            public void onItemClick(int position, Object data) {
                if (data instanceof SavedCard)
                    WalletFragment.this.deleteConfirmation(((SavedCard) data).id);
            }
        });
        vpCardsHorizontal.setOffscreenPageLimit(3);
        vpCardsHorizontal.setPageMargin(8);
        saveCardsUIViewPagerAdapter.registerDataSetObserver(cardIndicator.getDataSetObserver());
        vpCardsHorizontal.setAdapter(saveCardsUIViewPagerAdapter);
        vpCardsHorizontal.setPageTransformer(true, new AlphaAndScalePageTransformer());
        cardIndicator.setViewPager(vpCardsHorizontal);
    }

    private void setListener() {
        ivAddCard.setOnClickListener(view -> {
            AddCardFragment addCardFragment = AddCardFragment.getInstance(true, null);
            addCardFragment.show(getChildFragmentManager(), "add_card");
            addCardFragment.setDialogDismissListener(addCardDialogDismissListener);
        });
        llCod.setOnClickListener(view -> {
            PaymentMethod paymentMethod = new PaymentMethod();
            paymentMethod.type = "cod";
            paymentMethod.title = "Cash on delivery";
            paymentMethod.method_logo = R.drawable.icon_cod;
            myApp.getMyPreferences().saveDefaultPaymentMethod(paymentMethod);
            updateDefaultPaymentMethods();
            savedCardsAdapter.notifyDataSetChanged();
        });
        llPaypal.setOnClickListener(view -> {
            PaymentMethod paymentMethod = new PaymentMethod();
            paymentMethod.type = "paypal";
            paymentMethod.title = "PayPal";
            paymentMethod.method_logo = R.drawable.icon_cod;
            myApp.getMyPreferences().saveDefaultPaymentMethod(paymentMethod);
            updateDefaultPaymentMethods();
            savedCardsAdapter.notifyDataSetChanged();
        });
        llCards.setOnClickListener(view -> {
            if (savedCards.size() == 0) {
                ToastUtils.makeToast(getContext(), "Please add your card");
            } else {
                if (myApp.getMyPreferences().getDefaultPaymentMethod() !=null)
                {
                    if (myApp.getMyPreferences().getDefaultPaymentMethod().type.equalsIgnoreCase("stripe"))
                    {
                        //Nothing to Show if any card selected
                    }else
                    {
                        ToastUtils.makeToast(getContext(), "Please select below card");
                    }
                }else
                {
                    ToastUtils.makeToast(getContext(), "Please select below card");
                }

            }
        });
        ivAddFirstCard.setOnClickListener(view -> {
            AddCardFragment addCardFragment = new AddCardFragment();
            addCardFragment.show(getChildFragmentManager(), "add_card");
            addCardFragment.setDialogDismissListener(addCardDialogDismissListener);
        });
    }

    private void fetchCards() {
        if (getActivity() != null)
            getActivity().runOnUiThread(() -> lodingView.setVisibility(View.VISIBLE));
        AndroidNetworking.get(ApiEndPoints.NEW_CARDS).addQueryParameter("customer_id", String.valueOf(loggedInUser.id)).addQueryParameter("nopaginate", "1").build()
                .getAsJSONArray(new JSONArrayRequestListener() {
                    @Override
                    public void onResponse(JSONArray response) {
                                Type type = new TypeToken<List<SavedCard>>() {
                                }.getType();
                                savedCards.clear();
                                savedCards.addAll(new Gson().fromJson(response.toString(), type));
                                if (getActivity() != null) {
                                    getActivity().runOnUiThread(() -> {
                                        tvCardsLabel.setText("Cards");
                                        saveCardsUIViewPagerAdapter.notifyDataSetChanged();
                                        rlCardLayout.setVisibility(View.VISIBLE);
                                        rlFirstCardLayout.setVisibility(View.GONE);
                                        tvSwipeRemoveCard.setVisibility(View.VISIBLE);
                                        updateDefaultPaymentMethods();
                                        savedCardsAdapter.notifyDataSetChanged();
                                    });
                                }

                        if (getActivity() != null)
                            getActivity().runOnUiThread(() -> lodingView.setVisibility(View.GONE));

                    }

                    @Override
                    public void onError(ANError anError) {
                        if (getActivity() != null)
                            getActivity().runOnUiThread(() -> lodingView.setVisibility(View.GONE));
                        anError.printStackTrace();
                        savedCards.clear();
                        tvSwipeRemoveCard.setVisibility(View.GONE);
                        saveCardsUIViewPagerAdapter.notifyDataSetChanged();
                        savedCardsAdapter.notifyDataSetChanged();
                        updateDefaultPaymentMethods();
                        if (!CommonFunctions.isConnected(getActivity()))
                            myApp.noInternet(getActivity());
                    }
                });
    }

    private void deleteConfirmation(int cardId) {
        AlertDialog alertDialog = new AlertDialog.Builder(getActivity()).create();
        alertDialog.getWindow().setBackgroundDrawable(new ColorDrawable(Color.TRANSPARENT));
        LayoutInflater inflater = (LayoutInflater) getActivity().getSystemService(Context.LAYOUT_INFLATER_SERVICE);
        View dialogView = inflater.inflate(R.layout.dialog_remove_card, null);

        TextView tvCancel = dialogView.findViewById(R.id.tvCancel);
        TextView tvConfirm = dialogView.findViewById(R.id.tvConfirm);
        tvConfirm.setOnClickListener(view -> {
            deleteCard(cardId);
            alertDialog.dismiss();
        });
        tvCancel.setOnClickListener(view -> {
            alertDialog.dismiss();
        });
        alertDialog.setView(dialogView);
        alertDialog.setCancelable(false);
        alertDialog.show();
    }

    private void deleteCard(int cardId) {
        if (getActivity() != null)
            getActivity().runOnUiThread(() -> progressDialog.show());
        AndroidNetworking.delete(ApiEndPoints.NEW_CARDS + String.valueOf(cardId)).build()
                .getAsJSONObject(new JSONObjectRequestListener() {
                    @Override
                    public void onResponse(JSONObject response) {
                        ToastUtils.makeSnackToast(getActivity(), "Successfully deleted card");
                        checkDefaultMethod(cardId);
                        fetchCards();

                        if (getActivity() != null)
                            getActivity().runOnUiThread(() -> progressDialog.dismiss());
                    }

                    @Override
                    public void onError(ANError anError) {
                        if (getActivity() != null)
                            getActivity().runOnUiThread(() -> progressDialog.dismiss());
                        anError.printStackTrace();
                        fetchCards();
                    }
                });
    }

    private void checkDefaultMethod(int cardId) {
        PaymentMethod defaultPaymentMethod = myApp.getMyPreferences().getDefaultPaymentMethod();
        if (defaultPaymentMethod != null && defaultPaymentMethod.type.equalsIgnoreCase("stripe") && defaultPaymentMethod.card_id == cardId) {
            myApp.getMyPreferences().saveDefaultPaymentMethod(null);
            updateDefaultPaymentMethods();
        }
    }

    private DialogDismissListener addCardDialogDismissListener = new DialogDismissListener() {
        @Override
        public void onDialogDismiss(Object o) {
            if (o instanceof String) {
                ToastUtils.makeSnackToast(getActivity(), String.valueOf(o));
            }
            fetchCards();
        }
    };
}
