package com.tiffintom.fragment.becompartner;

import android.content.Intent;
import android.os.Bundle;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.Button;
import android.widget.EditText;
import android.widget.LinearLayout;
import android.widget.TextView;

import androidx.annotation.NonNull;
import androidx.annotation.Nullable;

import com.androidnetworking.AndroidNetworking;
import com.androidnetworking.error.ANError;
import com.androidnetworking.interfaces.ParsedRequestListener;
import com.tiffintom.R;
import com.tiffintom.activity.TransportActivity;
import com.tiffintom.base.BaseFragment;
import com.tiffintom.common.CommonFunctions;
import com.tiffintom.common.Validators;
import com.tiffintom.models.ApiError;
import com.tiffintom.models.BecomePartner;
import com.tiffintom.models.Postcode;
import com.tiffintom.network.ApiEndPoints;
import com.tiffintom.utils.ToastUtils;

import static com.tiffintom.MyApp.becomePartner;

public class CompanyDetailFragment extends BaseFragment {

    private Button btnNext, btnBack, btnLookup;
    private EditText et_CompanyName, et_BuildingNo, et_Postcode;
    private Postcode searchPostcode;
    private LinearLayout ll_Address;
    private TextView tvAddress;


    public static CompanyDetailFragment getInstance() {
        return new CompanyDetailFragment();
    }

    @Nullable
    @Override
    public View onCreateView(@NonNull LayoutInflater inflater, @Nullable ViewGroup container, @Nullable Bundle savedInstanceState) {
        return inflater.inflate(R.layout.fragment_company_details, container, false);
    }

    @Override
    public void onViewCreated(@NonNull View view, @Nullable Bundle savedInstanceState) {
        super.onViewCreated(view, savedInstanceState);

        setListner();
        updateView();
    }

    private void setListner() {

        btnBack.setOnClickListener(view -> getActivity().onBackPressed());

        btnNext.setOnClickListener(view -> {
            if (isValid()) {
                new Thread(this::goNext).start();
            }
        });

        btnLookup.setOnClickListener(view -> {
            if (isValidDetails()) {
                if (Validators.isNullOrEmpty(et_Postcode.getText().toString())) {
                    ToastUtils.makeToast(getActivity(), "Please enter postcode");
                }else if (et_Postcode.getText().toString().length() < 3) {
                    ToastUtils.makeToast(getActivity(), "Please valid postcode");
                } else {
                    String postcode = et_Postcode.getText().toString().replaceAll("\\s+", "").trim();
                    new Thread(() -> searchPostcode(postcode)).start();
                }
            }
        });
    }

    private void goNext() {

        if (becomePartner == null) becomePartner = new BecomePartner();
        becomePartner.comapnyPostcode = searchPostcode;
        becomePartner.company_name = et_CompanyName.getText().toString();
        becomePartner.company_buldingno = et_BuildingNo.getText().toString();
        becomePartner.company_postcode = et_Postcode.getText().toString();

        startActivity(new Intent(getActivity(), TransportActivity.class).putExtra("destination", "menu&logo").putExtra("name", "Become a Partner"));

    }

    private void searchPostcode(String postcode) {
        searchPostcode = null;
        if (getActivity() != null) getActivity().runOnUiThread(progressDialog::show);
        AndroidNetworking.get(ApiEndPoints.RMS_POSTCODE + postcode)
                .build()
                .getAsObject(Postcode.class, new ParsedRequestListener<Postcode>() {
                    @Override
                    public void onResponse(Postcode response) {
                        searchPostcode = response;
                        if (getActivity() != null)
                            CommonFunctions.hideKeyboard(getActivity(), et_Postcode);
                        if (getActivity() != null)
                            getActivity().runOnUiThread(() -> {
                                progressDialog.dismiss();
                                ll_Address.setVisibility(View.VISIBLE);
                                tvAddress.setText(new StringBuilder(et_BuildingNo.getText().toString()).append(", ").append(searchPostcode.street));

                            });
                        myApp.getMyPreferences().saveCurrentPostcode(response);
                    }

                    @Override
                    public void onError(ANError anError) {
                        if (getActivity() != null)
                            getActivity().runOnUiThread(() -> {
                                progressDialog.dismiss();
                                ll_Address.setVisibility(View.GONE);
                            });
                        if (anError.getErrorCode() == 400) {
                            ApiError apiError = anError.getErrorAsObject(ApiError.class);
                            ToastUtils.makeSnackToast(getActivity(), apiError.getMessage());
                        }
                        if (!CommonFunctions.isConnected(getActivity())) {
                            myApp.noInternet(getActivity());
                        }
                    }
                });
    }

    private boolean isValidDetails() {

        et_CompanyName.setError(null);
        et_BuildingNo.setError(null);
        et_Postcode.setError(null);

        if (Validators.isNullOrEmpty(et_CompanyName.getText().toString())) {
            et_CompanyName.setError("Please enter registered company name");
            et_CompanyName.requestFocus();
            return false;
        }
        if (Validators.isNullOrEmpty(et_BuildingNo.getText().toString())) {
            et_BuildingNo.setError("Please enter registered building number");
            et_BuildingNo.requestFocus();
            return false;
        }
        if (Validators.isNullOrEmpty(et_Postcode.getText().toString())) {
            et_Postcode.setError("Please enter registered postcode");
            et_Postcode.requestFocus();
            return false;
        }
        return true;
    }

    private boolean isValid() {

        if (!isValidDetails()) return false;
        if (searchPostcode == null) {
            ToastUtils.makeToast(getActivity(), "Please lookup business address");
            return false;
        }
        return true;
    }


    @Override
    protected void initViews(View view) {
        super.initViews(view);
        btnNext = view.findViewById(R.id.btnNext);
        btnBack = view.findViewById(R.id.btnBack);
        et_CompanyName = view.findViewById(R.id.etCompanyName);
        et_BuildingNo = view.findViewById(R.id.etBuldingNumber);
        et_Postcode = view.findViewById(R.id.etRegisterdPostcode);
        btnLookup = view.findViewById(R.id.btnLookup);
        ll_Address = view.findViewById(R.id.llAddress);
        tvAddress = view.findViewById(R.id.tvAddress);
    }

    private void updateView() {

        if (becomePartner != null) {
            et_CompanyName.setText(becomePartner.company_name);
            et_BuildingNo.setText(becomePartner.company_buldingno);
            et_Postcode.setText(becomePartner.company_postcode);
        }
    }
}
