package com.tiffintom.fragment.becompartner;

import android.os.Build;
import android.os.Bundle;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.Button;
import android.widget.RadioButton;

import androidx.annotation.NonNull;
import androidx.annotation.Nullable;

import com.google.android.material.bottomsheet.BottomSheetDialogFragment;
import com.tiffintom.R;
import com.tiffintom.interfaces.DialogDismissListener;
import com.tiffintom.models.BecomePartner;
import com.tiffintom.utils.ToastUtils;

import static com.tiffintom.MyApp.becomePartner;

public class CompanyTypeBottomSheet extends BottomSheetDialogFragment {

    private RadioButton rbLimited, rbTrader, rbNotRegistered, rbOther, rbPartnerShip;
    private Button btnConfirm;
    private DialogDismissListener dialogDismissListener;

    public static CompanyTypeBottomSheet getInstance() {
        return new CompanyTypeBottomSheet();
    }

    public void setDialogDismissListener(DialogDismissListener dialogDismissListener) {
        this.dialogDismissListener = dialogDismissListener;
    }

    @Override
    public int getTheme() {
        if (Build.VERSION.SDK_INT > 21) {
            return R.style.BottomSheetDialogTheme;
        } else {
            return super.getTheme();
        }
    }

    @Nullable
    @Override
    public View onCreateView(@NonNull LayoutInflater inflater, @Nullable ViewGroup container, @Nullable Bundle savedInstanceState) {
        return inflater.inflate(R.layout.fragment_company_type, container, false);
    }

    @Override
    public void onViewCreated(@NonNull View view, @Nullable Bundle savedInstanceState) {
        super.onViewCreated(view, savedInstanceState);
        initViews(view);
        setListners();
    }

    private void setListners() {
        btnConfirm.setOnClickListener(view -> {
            if (rbLimited.isChecked() || rbTrader.isChecked() || rbPartnerShip.isChecked() || rbNotRegistered.isChecked() || rbOther.isChecked()) {

                if (becomePartner == null) becomePartner = new BecomePartner();
                if (rbLimited.isChecked())
                    becomePartner.company_type = rbLimited.getText().toString();

                else if (rbTrader.isChecked())
                    becomePartner.company_type = rbTrader.getText().toString();

                else if (rbPartnerShip.isChecked())
                    becomePartner.company_type = rbPartnerShip.getText().toString();

                else if (rbNotRegistered.isChecked())
                    becomePartner.company_type = rbNotRegistered.getText().toString();
                else
                    becomePartner.company_type = rbOther.getText().toString();

                if (dialogDismissListener != null) {
                    dialogDismissListener.onDialogDismiss(becomePartner.company_type);
                }
                dismiss();
            } else {
                ToastUtils.makeToast(getActivity(), "Please Select one option");
            }

        });


    }

    private void initViews(View view) {

        rbLimited = view.findViewById(R.id.rbLimited);
        rbTrader = view.findViewById(R.id.rbSoleTrader);
        rbPartnerShip = view.findViewById(R.id.rbPartnership);
        rbNotRegistered = view.findViewById(R.id.rbNotRegistered);
        rbOther = view.findViewById(R.id.rbOther);
        btnConfirm = view.findViewById(R.id.btnConfirm);

        if (becomePartner != null && becomePartner.company_type != null) {
            if (becomePartner.company_type.equals(rbLimited.getText().toString()))
                rbLimited.setChecked(true);
            else if (becomePartner.company_type.equals(rbTrader.getText().toString()))
                rbTrader.setChecked(true);
            else if (becomePartner.company_type.equals(rbPartnerShip.getText().toString()))
                rbPartnerShip.setChecked(true);
            else if (becomePartner.company_type.equals(rbNotRegistered.getText().toString()))
                rbNotRegistered.setChecked(true);
            else
                rbOther.setChecked(true);
        }

    }


}
