package com.tiffintom.fragment.becompartner;

import android.Manifest;
import android.app.Activity;
import android.content.DialogInterface;
import android.content.Intent;
import android.content.pm.PackageManager;
import android.database.Cursor;
import android.net.Uri;
import android.os.Bundle;
import android.provider.MediaStore;
import android.provider.Settings;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.ArrayAdapter;
import android.widget.Button;
import android.widget.ImageView;
import android.widget.LinearLayout;
import android.widget.TextView;

import androidx.annotation.NonNull;
import androidx.annotation.Nullable;
import androidx.appcompat.app.AlertDialog;
import androidx.core.app.ActivityCompat;

import com.tiffintom.R;
import com.tiffintom.activity.TransportActivity;
import com.tiffintom.base.BaseFragment;
import com.tiffintom.models.BecomePartner;
import com.tiffintom.utils.LogUtils;
import com.tiffintom.utils.ToastUtils;

import java.io.File;
import java.util.ArrayList;

import droidninja.filepicker.FilePickerBuilder;
import droidninja.filepicker.FilePickerConst;

import static com.tiffintom.MyApp.becomePartner;


public class MenuLogoFragment extends BaseFragment {

    private Button btnBack, btnNext;
    private TextView tvEditMenu, tvEditLogo, tvBrowseMenu, tvBrowseLogo;
    private ImageView ivMenu, ivLogo;
    private LinearLayout ll_Menu, ll_Logo;
    public static final int MENU_REQUEST_CODE = 1;
    public static final int LOGO_REQUEST_CODE = 2;

    private ArrayList<Uri> docPaths = new ArrayList<>();
    private ArrayList<Uri> photoPaths = new ArrayList<>();
    private String[] requiredPermissions = {Manifest.permission.READ_EXTERNAL_STORAGE, Manifest.permission.CAMERA};

    public static MenuLogoFragment getInstance() {
        return new MenuLogoFragment();
    }

    @Nullable
    @Override
    public View onCreateView(@NonNull LayoutInflater inflater, @Nullable ViewGroup container, @Nullable Bundle savedInstanceState) {
        return inflater.inflate(R.layout.fragment_menulogo, container, false);
    }

    @Override
    public void onViewCreated(@NonNull View view, @Nullable Bundle savedInstanceState) {
        super.onViewCreated(view, savedInstanceState);

        setListner();
        updateView();
    }

    private void updateView() {

        if (becomePartner != null) {
            if (becomePartner.menu_images != null && becomePartner.menu_images.size() > 0) {
                tvBrowseMenu.setVisibility(View.GONE);
                tvEditMenu.setVisibility(View.VISIBLE);
                ivMenu.setVisibility(View.VISIBLE);

            } else {
                tvBrowseMenu.setVisibility(View.VISIBLE);
                tvEditMenu.setVisibility(View.GONE);
                ivMenu.setVisibility(View.GONE);
            }
            if (becomePartner.logo != null) {
                tvBrowseLogo.setVisibility(View.GONE);
                tvEditLogo.setVisibility(View.VISIBLE);
                ivLogo.setVisibility(View.VISIBLE);
            } else {
                tvBrowseLogo.setVisibility(View.VISIBLE);
                tvEditLogo.setVisibility(View.GONE);
                ivLogo.setVisibility(View.GONE);
            }
        } else {
            tvBrowseMenu.setVisibility(View.VISIBLE);
            tvEditMenu.setVisibility(View.GONE);
            ivMenu.setVisibility(View.GONE);

            tvBrowseLogo.setVisibility(View.VISIBLE);
            tvEditLogo.setVisibility(View.GONE);
            ivLogo.setVisibility(View.GONE);
        }
    }

    private void setListner() {
        btnNext.setOnClickListener(view1 -> {
            isValid();
        });
        btnBack.setOnClickListener(view12 -> getActivity().onBackPressed());

        ll_Menu.setOnClickListener(view -> {
            if (checkStoragePermission()) {
                showSelectMenuType();
            }
        });

        ll_Logo.setOnClickListener(view -> {
            if (checkGalleryPermission()) {
                pickImage();
            }
        });
    }

    private boolean checkGalleryPermission() {
        ArrayList<String> deniedPermissions = new ArrayList<>();
        ArrayList<String> rationalDeniedPermissions = new ArrayList<>();
        for (String permission : requiredPermissions) {
            if (ActivityCompat.checkSelfPermission(getActivity(), permission) == PackageManager.PERMISSION_DENIED) {
                deniedPermissions.add(permission);
                if (ActivityCompat.shouldShowRequestPermissionRationale(getActivity(), permission))
                    rationalDeniedPermissions.add(permission);
            }
        }
        if (rationalDeniedPermissions.size() > 0) {
            // Show why we need permissions in dialog and on dialog yes press request permissions;

            new android.app.AlertDialog.Builder(getActivity())
                    .setTitle("Storage permission required")
                    .setMessage("To access logo image_url we required storage permission, Please allow to continue.")
                    .setPositiveButton("ALLOW", (dialogInterface, i) -> {
                        String[] mStringArray = new String[deniedPermissions.size()];
                        mStringArray = deniedPermissions.toArray(mStringArray);

                        requestPermissions(mStringArray, LOGO_REQUEST_CODE);
                    })
                    .setNegativeButton("DENY", (dialogInterface, i) -> dialogInterface.dismiss())
                    .create()
                    .show();

            return false;
        } else {

            if (deniedPermissions.size() > 0) {
                String[] mStringArray = new String[deniedPermissions.size()];
                mStringArray = deniedPermissions.toArray(mStringArray);

                requestPermissions(mStringArray, LOGO_REQUEST_CODE);
                return false;
            } else {
                return true;
            }
        }

    }

    private void pickImage() {
        FilePickerBuilder.getInstance()
                .setMaxCount(1)
                .enableImagePicker(true)
                .enableCameraSupport(true)
                .setActivityTitle("Select Logo")
                .pickPhoto(this, LOGO_REQUEST_CODE);
    }

    private void pickMenuFile() {
        FilePickerBuilder.getInstance()
                .enableImagePicker(true)
                .setMaxCount(10) //optional
                .setActivityTitle("Select Menu")
                .pickFile(this, MENU_REQUEST_CODE);
    }

    private void pickMenuImage() {
        FilePickerBuilder.getInstance()
                .setMaxCount(10)
                .enableImagePicker(true)
                .enableCameraSupport(true)
                .setActivityTitle("Select Menu")
                .pickPhoto(this, MENU_REQUEST_CODE);
    }

    private void showSelectMenuType() {
        AlertDialog.Builder builderSingle = new AlertDialog.Builder(getActivity());
        String[] animals = {"Select from documents", "Select from gallery"};
        final ArrayAdapter<String> arrayAdapter = new ArrayAdapter<String>(getActivity(), android.R.layout.select_dialog_singlechoice);
        arrayAdapter.add("Remove profile picture");
        arrayAdapter.add("Edit profile picture");

        builderSingle.setItems(animals, new DialogInterface.OnClickListener() {
            @Override
            public void onClick(DialogInterface dialog, int which) {
                switch (which) {
                    case 0:
                        pickMenuFile();
                        break;

                    case 1:
                        pickMenuImage();
                        break;

                }
            }
        });


        builderSingle.show();

    }

    private boolean checkStoragePermission() {
        ArrayList<String> deniedPermissions = new ArrayList<>();
        ArrayList<String> rationalDeniedPermissions = new ArrayList<>();

        for (String permission : requiredPermissions) {
            if (ActivityCompat.checkSelfPermission(getActivity(), permission) == PackageManager.PERMISSION_DENIED) {
                deniedPermissions.add(permission);
                if (ActivityCompat.shouldShowRequestPermissionRationale(getActivity(), permission))
                    rationalDeniedPermissions.add(permission);
            }
        }
        if (rationalDeniedPermissions.size() > 0) {
            // Show why we need permissions in dialog and on dialog yes press request permissions;

            new android.app.AlertDialog.Builder(getActivity())
                    .setTitle("Storage permission required")
                    .setMessage("To access your menu file we required storage permission, Please allow to continue.")
                    .setPositiveButton("ALLOW", (dialogInterface, i) -> {
                        String[] mStringArray = new String[deniedPermissions.size()];
                        mStringArray = deniedPermissions.toArray(mStringArray);

                        requestPermissions(mStringArray, MENU_REQUEST_CODE);
                    })
                    .setNegativeButton("DENY", (dialogInterface, i) -> dialogInterface.dismiss())
                    .create()
                    .show();

            return false;
        } else {

            if (deniedPermissions.size() > 0) {
                String[] mStringArray = new String[deniedPermissions.size()];
                mStringArray = deniedPermissions.toArray(mStringArray);

                requestPermissions(mStringArray, MENU_REQUEST_CODE);
                return false;
            } else {
                return true;
            }
        }

    }

    private void isValid() {

        startActivity(new Intent(getActivity(), TransportActivity.class).putExtra("destination", "summery").putExtra("name", "Become a Partner"));
    }

    @Override
    protected void initViews(View view) {
        super.initViews(view);
        btnNext = view.findViewById(R.id.btnNext);
        btnBack = view.findViewById(R.id.btnBack);
        ll_Menu = view.findViewById(R.id.ll_menu);
        ll_Logo = view.findViewById(R.id.ll_logo);
        tvEditMenu = view.findViewById(R.id.tvEditMenu);
        tvEditLogo = view.findViewById(R.id.tvEditLogo);
        ivMenu = view.findViewById(R.id.ivEditMenu);
        ivLogo = view.findViewById(R.id.ivEditLogo);
        tvBrowseMenu = view.findViewById(R.id.tvBrowseMenu);
        tvBrowseLogo = view.findViewById(R.id.tvBrowseLogo);

    }

    private Uri fileUri;

    @Override
    public void onActivityResult(int requestCode, int resultCode, @Nullable Intent data) {

        LogUtils.e(String.valueOf(requestCode));

        if (requestCode == MENU_REQUEST_CODE) {
            if (resultCode == Activity.RESULT_OK && data != null) {
                ArrayList<Uri> dataList;

                dataList = data.getParcelableArrayListExtra(FilePickerConst.KEY_SELECTED_DOCS);
                if (dataList == null || dataList.size() == 0) {
                    dataList = data.getParcelableArrayListExtra(FilePickerConst.KEY_SELECTED_MEDIA);
                }

                if (dataList != null && dataList.size() > 0) {
                    docPaths = new ArrayList<>();
                    docPaths.addAll(dataList);

                    if (becomePartner == null) becomePartner = new BecomePartner();
                    becomePartner.menu_images = new ArrayList<>();

                    for (Uri uri : docPaths) {
                        File f = new File(getPath(uri));
                        becomePartner.menu_images.add(f);
                        LogUtils.e("MenuData>>>", becomePartner.menu_images + "");
                    }
                }
                if (becomePartner != null && becomePartner.menu_images != null && becomePartner.menu_images.size() > 0) {
                    tvBrowseMenu.setVisibility(View.GONE);
                    tvEditMenu.setVisibility(View.VISIBLE);
                    ivMenu.setVisibility(View.VISIBLE);
                } else {
                    tvBrowseMenu.setVisibility(View.VISIBLE);
                    tvEditMenu.setVisibility(View.GONE);
                    ivMenu.setVisibility(View.GONE);
                }

            }

        }
        if (requestCode == LOGO_REQUEST_CODE) {
            if (resultCode == Activity.RESULT_OK && data != null) {
                ArrayList<Uri> dataList = data.getParcelableArrayListExtra(FilePickerConst.KEY_SELECTED_MEDIA);
                if (dataList != null && dataList.size() > 0) {
                    photoPaths = new ArrayList<>();
                    photoPaths.addAll(dataList);
                    tvBrowseLogo.setVisibility(View.GONE);
                    tvEditLogo.setVisibility(View.VISIBLE);
                    ivLogo.setVisibility(View.VISIBLE);
                    fileUri = photoPaths.get(0);
                    if (becomePartner == null) becomePartner = new BecomePartner();
                    {
                        becomePartner.logo = new File(getPath(fileUri));
                    }
                }

            }

        }
    }

    private String getPath(Uri uri) {
        String[] projection = {MediaStore.Images.Media.DATA};
        Cursor cursor = getActivity().getContentResolver().query(uri, projection, null, null, null);
        if (cursor == null) return null;
        int column_index = cursor.getColumnIndexOrThrow(MediaStore.Images.Media.DATA);
        cursor.moveToFirst();
        String s = cursor.getString(column_index);
        cursor.close();
        return s;
    }

    @Override
    public void onRequestPermissionsResult(int requestCode, @NonNull String[] permissions, @NonNull int[] grantResults) {
        super.onRequestPermissionsResult(requestCode, permissions, grantResults);
        if (requestCode == MENU_REQUEST_CODE) {
            ArrayList<String> rationalDeniedPermissions = new ArrayList<>();
            ArrayList<String> nonRationalDeniedPermission = new ArrayList<>();
            for (String permission : permissions) {
                if (ActivityCompat.checkSelfPermission(getActivity(), permission) == PackageManager.PERMISSION_DENIED) {
                    if (ActivityCompat.shouldShowRequestPermissionRationale(getActivity(), permission))
                        rationalDeniedPermissions.add(permission);
                    else {
                        nonRationalDeniedPermission.add(permission);
                    }
                }
            }
            if (nonRationalDeniedPermission.size() == 0 && rationalDeniedPermissions.size() == 0) {
                showSelectMenuType();
            } else {
                if (nonRationalDeniedPermission.size() > 0) {
                    new android.app.AlertDialog.Builder(getActivity())
                            .setTitle("Some permissions required")
                            .setMessage("To access your menu file we require storage and camera permission, Please allow to continue.")
                            .setPositiveButton("ALLOW", (dialogInterface, i) -> {
                                Intent intent = new Intent();
                                intent.setAction(Settings.ACTION_APPLICATION_DETAILS_SETTINGS);
                                Uri uri = Uri.fromParts("package", getActivity().getPackageName(), null);
                                intent.setData(uri);
                                startActivity(intent);
                            })
                            .setNegativeButton("DENY", (dialogInterface, i) -> dialogInterface.dismiss())
                            .create()
                            .show();
                } else {
                    ToastUtils.makeToast(getActivity(), "Permission denied");
                }
            }

        }

        if (requestCode == LOGO_REQUEST_CODE) {
            ArrayList<String> rationalDeniedPermissions = new ArrayList<>();
            ArrayList<String> nonRationalDeniedPermission = new ArrayList<>();
            for (String permission : permissions) {
                if (ActivityCompat.checkSelfPermission(getActivity(), permission) == PackageManager.PERMISSION_DENIED) {
                    if (ActivityCompat.shouldShowRequestPermissionRationale(getActivity(), permission))
                        rationalDeniedPermissions.add(permission);
                    else {
                        nonRationalDeniedPermission.add(permission);
                    }
                }
            }
            if (nonRationalDeniedPermission.size() == 0 && rationalDeniedPermissions.size() == 0) {
                pickImage();
            } else {
                if (nonRationalDeniedPermission.size() > 0) {
                    new android.app.AlertDialog.Builder(getActivity())
                            .setTitle("Some permissions required")
                            .setMessage("To access logo image_url we require storage and camera permission, Please allow to continue.")
                            .setPositiveButton("ALLOW", (dialogInterface, i) -> {
                                Intent intent = new Intent();
                                intent.setAction(Settings.ACTION_APPLICATION_DETAILS_SETTINGS);
                                Uri uri = Uri.fromParts("package", getActivity().getPackageName(), null);
                                intent.setData(uri);
                                startActivity(intent);
                            })
                            .setNegativeButton("DENY", (dialogInterface, i) -> dialogInterface.dismiss())
                            .create()
                            .show();
                } else {
                    ToastUtils.makeToast(getActivity(), "Permission denied");
                }
            }

        }


    }
}
