package com.tiffintom.fragment.becompartner;

import android.content.Intent;
import android.os.Bundle;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.Button;
import android.widget.EditText;
import android.widget.LinearLayout;
import android.widget.TextView;

import androidx.annotation.NonNull;
import androidx.annotation.Nullable;
import androidx.appcompat.widget.AppCompatSpinner;

import com.androidnetworking.AndroidNetworking;
import com.androidnetworking.error.ANError;
import com.androidnetworking.interfaces.ParsedRequestListener;
import com.google.gson.Gson;
import com.tiffintom.R;
import com.tiffintom.activity.TransportActivity;
import com.tiffintom.base.BaseFragment;
import com.tiffintom.common.CommonFunctions;
import com.tiffintom.common.Validators;
import com.tiffintom.fragment.CuisineBottomSheetFragment;
import com.tiffintom.models.ApiError;
import com.tiffintom.models.BecomePartner;
import com.tiffintom.models.Cuisine;
import com.tiffintom.models.Postcode;
import com.tiffintom.network.ApiEndPoints;
import com.tiffintom.utils.LogUtils;
import com.tiffintom.utils.ToastUtils;

import java.util.ArrayList;

import static com.facebook.FacebookSdk.getApplicationContext;
import static com.tiffintom.MyApp.becomePartner;

import org.json.JSONException;
import org.json.JSONObject;

public class PartnerDetailFragment extends BaseFragment {
    private Button btnNext, btnBack;
    private EditText etBussinessName, etBuildingNo, etPostcode;
    private EditText etFirstName, etSurname, etEmail, etPhoneNumber;
    private LinearLayout ll_Cuisine, ll_location, ll_delivery, ll_Company;
    private ArrayList<Cuisine> selectedCuisines = new ArrayList<>();
    private StringBuilder cusisineId = new StringBuilder();
    private StringBuilder cusisineName = new StringBuilder();
    private String strLocation;
    private TextView tvLocation, tvCuisines, tvDeliver, tvAddress, tvCompany;
    private AppCompatSpinner spinnerAddress;
    private EditText etBussinessPostcard;
    private Button btnLookup;
    private LinearLayout ll_Address;
    private Postcode searchPostcode;

    public static PartnerDetailFragment getInstance() {
        return new PartnerDetailFragment();
    }

    @Nullable
    @Override
    public View onCreateView(@NonNull LayoutInflater inflater, @Nullable ViewGroup container, @Nullable Bundle savedInstanceState) {
        return inflater.inflate(R.layout.fragment_becomepartner_detail, container, false);
    }

    @Override
    public void onViewCreated(@NonNull View view, @Nullable Bundle savedInstanceState) {
        super.onViewCreated(view, savedInstanceState);
        becomePartner = new BecomePartner();
        setListners();
        updateView();
    }

    private void setListners() {
        ll_Cuisine.setOnClickListener(view1 -> {
            CuisineBottomSheetFragment cuisineBottomSheetFragment = CuisineBottomSheetFragment.getInstance(selectedCuisines);
            cuisineBottomSheetFragment.show(getChildFragmentManager(), "cuisine types");
            cuisineBottomSheetFragment.setDialogDismissListener(this::getDataFromCuisine);
        });

        ll_location.setOnClickListener(view1 -> {
            NumberOfLocationBottomSheetFragment loactionBottomSheetFragment = NumberOfLocationBottomSheetFragment.getInstance();
            loactionBottomSheetFragment.show(getChildFragmentManager(), "location");
            loactionBottomSheetFragment.setDialogDismissListener(this::getNoOfLocation);
        });

        ll_delivery.setOnClickListener(view1 -> {
            ProvideDeliveryBottomSheet provideDeliveryBottomSheet = ProvideDeliveryBottomSheet.getInstance();
            provideDeliveryBottomSheet.show(getChildFragmentManager(), "deliver");
            provideDeliveryBottomSheet.setDialogDismissListener(this::getDeliver);

        });

        ll_Company.setOnClickListener(view -> {
            CompanyTypeBottomSheet companyTypeBottomSheet = CompanyTypeBottomSheet.getInstance();
            companyTypeBottomSheet.show(getChildFragmentManager(), "company");
            companyTypeBottomSheet.setDialogDismissListener(this::getCompanyType);
        });


        btnNext.setOnClickListener(view12 -> {

            if (isValid()) {
                new Thread(this::goNext).start();
            }
        });
        btnBack.setOnClickListener(view13 -> getActivity().onBackPressed());


        btnLookup.setOnClickListener(view -> {
            if (isValidBussiness()) {
                if (Validators.isNullOrEmpty(etPostcode.getText().toString())) {
                    ToastUtils.makeToast(getActivity(), "Please enter postcode");
                }else if (etPostcode.getText().toString().length() < 3) {
                    ToastUtils.makeToast(getActivity(), "Please valid postcode");
                } else {
                    String postcode = etPostcode.getText().toString().replaceAll("\\s+", "").trim();
                    new Thread(() -> searchPostcode(postcode)).start();
                }
            }
        });
    }


    private void searchPostcode(String postcode) {
        searchPostcode = null;
        if (getActivity() != null) getActivity().runOnUiThread(progressDialog::show);
        AndroidNetworking.get(ApiEndPoints.RMS_POSTCODE + postcode)
                .build()
                .getAsObject(Postcode.class, new ParsedRequestListener<Postcode>() {
                    @Override
                    public void onResponse(Postcode response) {
                        searchPostcode = response;
                        if (getActivity() != null)
                            CommonFunctions.hideKeyboard(getActivity(), etPostcode);
                        if (getActivity() != null)
                            getActivity().runOnUiThread(() -> {
                                progressDialog.dismiss();
                                searchPostcode = response;
                                ll_Address.setVisibility(View.VISIBLE);
                                tvAddress.setText(new StringBuilder(etBuildingNo.getText().toString()).append(", ").append(searchPostcode.street));

                            });
                        myApp.getMyPreferences().saveCurrentPostcode(response);
                    }

                    @Override
                    public void onError(ANError anError) {
                        if (getActivity() != null)
                            getActivity().runOnUiThread(() -> {
                                progressDialog.dismiss();
                                ll_Address.setVisibility(View.GONE);
                            });
                        if (anError.getErrorCode() == 400) {
                            try {
                                JSONObject jsonObject = new JSONObject(anError.getErrorBody().toString());
                                ToastUtils.makeToast(getApplicationContext(), jsonObject.getString("message"));
                                ll_Address.setVisibility(View.GONE);
                                tvAddress.setText("");
                            } catch (JSONException exception) {
                                exception.printStackTrace();
                            }
                        }
                        if (!CommonFunctions.isConnected(getActivity())) {
                            myApp.noInternet(getActivity());
                        }
                    }
                });
    }


    private boolean isValidBussiness() {

        etBussinessName.setError(null);
        etBuildingNo.setError(null);
        etPostcode.setError(null);

        if (Validators.isNullOrEmpty(etBussinessName.getText().toString())) {
            etBussinessName.setError("Please enter business name");
            etBussinessName.requestFocus();
            return false;
        }
        if (Validators.isNullOrEmpty(etBuildingNo.getText().toString())) {
            etBuildingNo.setError("Please enter building number");
            etBuildingNo.requestFocus();
            return false;
        }

        if (Validators.isNullOrEmpty(etPostcode.getText().toString())) {
            etPostcode.setError("Please enter postcode");
            etPostcode.requestFocus();
            return false;
        }
        return true;
    }

    private void getCompanyType(Object o) {
        LogUtils.e("COMAPNY::", o.toString());
        tvCompany.setText(becomePartner.company_type);
    }

    private void getDeliver(Object o) {
        LogUtils.e("DELIVER::", o.toString());
        tvDeliver.setText(becomePartner.deliver);
    }

    private void getNoOfLocation(Object o) {
        LogUtils.e("LOCATION::", o.toString());
        tvLocation.setText("Number of location " + "(" + becomePartner.no_of_location + ")");
    }

    private void getDataFromCuisine(Object o) {
        LogUtils.e("CUISINEDATA::", new Gson().toJson(o));
        selectedCuisines.clear();
        selectedCuisines.addAll((ArrayList<Cuisine>) o);
        cusisineName = new StringBuilder();
        cusisineId = new StringBuilder();
        for (int i = 0; i < selectedCuisines.size(); i++) {
            cusisineId.append(selectedCuisines.get(i).id);
            cusisineName.append(selectedCuisines.get(i).cuisine_name);
            if (i < selectedCuisines.size() - 1) {
                cusisineId.append(",");
                cusisineName.append(",");
            }
        }
        becomePartner.restaurant_cuisine = cusisineId.toString();
        tvCuisines.setText("Types of Cuisines (" + cusisineName.toString() + ")");
        LogUtils.e("CUISINEID::", cusisineId.toString());
    }

    private boolean isValid() {

        etFirstName.setError(null);
        etSurname.setError(null);
        etEmail.setError(null);
        etPhoneNumber.setError(null);

        if (!isValidBussiness()) return false;

        if (searchPostcode == null) {
            ToastUtils.makeToast(getActivity(), "Please lookup business address");
            return false;
        }

        if (Validators.isNullOrEmpty(etFirstName.getText().toString())) {
            etFirstName.setError("Please enter your first name");
            etFirstName.requestFocus();
            return false;
        }
        if (Validators.isNullOrEmpty(etSurname.getText().toString())) {
            etSurname.setError("Please enter your surname");
            etSurname.requestFocus();
            return false;
        }
        if (!Validators.isEmail(etEmail.getText().toString())) {
            etEmail.setError("Please enter valid email");
            etEmail.requestFocus();
            return false;
        }
        if (Validators.isNullOrEmpty(etPhoneNumber.getText().toString())) {
            etPhoneNumber.setError("Please enter your phone number");
            etPhoneNumber.requestFocus();
            return false;
        }
        if (!Validators.isValidMobile(etPhoneNumber.getText().toString())) {
            etPhoneNumber.setError("Invalid mobile number");
            etPhoneNumber.requestFocus();
            return false;
        }
        if (Validators.isNullOrEmpty(becomePartner.company_type)) {
            ToastUtils.makeToast(getActivity(), "Please Select company type ");
            return false;
        }

        if (Validators.isNullOrEmpty(becomePartner.restaurant_cuisine)) {
            ToastUtils.makeToast(getActivity(), "Please Select cuisines ");
            return false;
        }
        if (Validators.isNullOrEmpty(becomePartner.no_of_location)) {
            ToastUtils.makeToast(getActivity(), "Please Select numbers of location ");
            return false;
        }
        if (Validators.isNullOrEmpty(becomePartner.deliver)) {
            ToastUtils.makeToast(getActivity(), "Please Select delivers ");
            return false;
        }


        return true;

    }

    private void goNext() {

        if (becomePartner == null) becomePartner = new BecomePartner();

        becomePartner.restaurant_name = etBussinessName.getText().toString();
        becomePartner.building_no = etBuildingNo.getText().toString();
        becomePartner.postcode = searchPostcode;
        becomePartner.restaurant_postcode = etPostcode.getText().toString();
        becomePartner.first_name = etFirstName.getText().toString();
        becomePartner.surname = etSurname.getText().toString();
        becomePartner.email = etEmail.getText().toString();
        becomePartner.phone = etPhoneNumber.getText().toString();

        startActivity(new Intent(getActivity(), TransportActivity.class).putExtra("destination", "receivingorder").putExtra("name", "Become a Partner"));

    }


    @Override
    protected void initViews(View view) {
        super.initViews(view);
        btnBack = view.findViewById(R.id.btnBack);
        btnNext = view.findViewById(R.id.btnNext);
        ll_Cuisine = view.findViewById(R.id.llCuisine);
        ll_location = view.findViewById(R.id.llLocationCount);
        ll_delivery = view.findViewById(R.id.llDelivery);
        etBussinessName = view.findViewById(R.id.etBussinessName);
        etBuildingNo = view.findViewById(R.id.etBuldingNumber);
        etPostcode = view.findViewById(R.id.etPostcode);
        etFirstName = view.findViewById(R.id.etFirstName);
        etSurname = view.findViewById(R.id.etSurname);
        etEmail = view.findViewById(R.id.etEmail);
        etPhoneNumber = view.findViewById(R.id.etPhoneNumber);
        tvCuisines = view.findViewById(R.id.tvTypeOfCuisine);
        tvLocation = view.findViewById(R.id.tvNumberOfLocation);
        tvDeliver = view.findViewById(R.id.tvIsDeliver);
        btnLookup = view.findViewById(R.id.btnLookup);
        ll_Address = view.findViewById(R.id.llAddress);
        tvAddress = view.findViewById(R.id.tvAddress);
        ll_Company = view.findViewById(R.id.llCompany);
        tvCompany = view.findViewById(R.id.tvComapnyType);
    }

    private void updateView() {

        if (becomePartner != null) {

            etBussinessName.setText(becomePartner.restaurant_name);
            etBuildingNo.setText(becomePartner.building_no);
            etPostcode.setText(becomePartner.restaurant_postcode);
            etFirstName.setText(becomePartner.first_name);
            etSurname.setText(becomePartner.surname);
            etEmail.setText(becomePartner.email);
            etPhoneNumber.setText(becomePartner.phone);

        }
    }
}
