package com.tiffintom.fragment.becompartner;

import android.content.Intent;
import android.os.Bundle;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.Button;
import android.widget.CheckBox;

import androidx.annotation.NonNull;
import androidx.annotation.Nullable;

import com.androidnetworking.AndroidNetworking;
import com.androidnetworking.common.ANRequest;
import com.androidnetworking.error.ANError;
import com.androidnetworking.interfaces.JSONObjectRequestListener;
import com.google.gson.Gson;
import com.tiffintom.R;
import com.tiffintom.activity.TransportActivity;
import com.tiffintom.base.BaseFragment;
import com.tiffintom.common.CommonFunctions;
import com.tiffintom.network.ApiEndPoints;
import com.tiffintom.utils.LogUtils;
import com.tiffintom.utils.ToastUtils;

import org.json.JSONObject;

import java.io.File;
import java.util.ArrayList;

import static com.tiffintom.MyApp.becomePartner;

public class RestaurantAgreementFragment extends BaseFragment {

    private Button btnNext, btnBack;
    private CheckBox cbAgree;
    private ArrayList<ArrayList<File>> files;

    public static RestaurantAgreementFragment getInstance() {
        return new RestaurantAgreementFragment();
    }

    @Nullable
    @Override
    public View onCreateView(@NonNull LayoutInflater inflater, @Nullable ViewGroup container, @Nullable Bundle savedInstanceState) {
        return inflater.inflate(R.layout.fragment_restraunt_agreement, container, false);
    }

    @Override
    public void onViewCreated(@NonNull View view, @Nullable Bundle savedInstanceState) {
        super.onViewCreated(view, savedInstanceState);
        setListners();
    }

    private void setListners() {
        btnNext.setOnClickListener(view1 -> {
            if (cbAgree.isChecked()) {
                fetchData();
            } else {
                ToastUtils.makeToast(getActivity(), "Please confirm you have read and accepted our agreement");
            }
        });
        btnBack.setOnClickListener(view12 -> getActivity().onBackPressed());
        cbAgree.setOnCheckedChangeListener((compoundButton, b) -> {
            if (cbAgree.isChecked()) {
                btnNext.setClickable(true);
            }
        });
    }

    private void fetchData() {
        if (getActivity() != null) {
            getActivity().runOnUiThread(() -> progressDialog.show());
        }

        ANRequest.MultiPartBuilder anRequest = AndroidNetworking.upload(ApiEndPoints.NEW_BECOMEPARTNER)
                .addMultipartParameter("restaurant_name", becomePartner.restaurant_name)
                .addMultipartParameter("restaurant_address", becomePartner.building_no + ", " + becomePartner.postcode.street)
                .addMultipartParameter("restaurant_city", "")
                .addMultipartParameter("restaurant_postcode", becomePartner.restaurant_postcode)
                .addMultipartParameter("first_name", becomePartner.first_name)
                .addMultipartParameter("surname", becomePartner.surname)
                .addMultipartParameter("email", becomePartner.email)
                .addMultipartParameter("phone", becomePartner.phone)
                .addMultipartParameter("restaurant_cuisine", becomePartner.restaurant_cuisine)
                .addMultipartParameter("no_of_location", becomePartner.no_of_location)
                .addMultipartParameter("deliver", becomePartner.deliver)
                .addMultipartParameter("company_type", becomePartner.company_type)
                .addMultipartParameter("payment", becomePartner.payment)
                .addMultipartParameter("company_name", becomePartner.company_name)
                .addMultipartParameter("company_first_address", becomePartner.company_buldingno + ", " + becomePartner.comapnyPostcode.street)
                .addMultipartParameter("company_city", "")
                .addMultipartParameter("company_postcode", becomePartner.company_postcode);
        if (becomePartner.menu_images != null && becomePartner.menu_images.size() > 0) {
            files = new ArrayList<>();
            files.add(becomePartner.menu_images);
            anRequest.addMultipartFileList("menu_image[]", becomePartner.menu_images);
        } else {
            anRequest.addMultipartParameter("menu_image", "");
        }
        if (becomePartner.logo != null) {
            anRequest.addMultipartFile("logo", becomePartner.logo);
        } else {
            anRequest.addMultipartParameter("logo", "");
        }

        anRequest.build().getAsJSONObject(new JSONObjectRequestListener() {
            @Override
            public void onResponse(JSONObject response) {
                    startActivity(new Intent(getActivity(), TransportActivity.class).putExtra("destination", "thankyou").putExtra("name", "Become a Partner").setFlags(Intent.FLAG_ACTIVITY_CLEAR_TASK | Intent.FLAG_ACTIVITY_NEW_TASK | Intent.FLAG_ACTIVITY_CLEAR_TOP));
               if (getActivity() != null) {
                    getActivity().runOnUiThread(() -> progressDialog.dismiss());
                }
            }

            @Override
            public void onError(ANError anError) {
                anError.printStackTrace();
                if (getActivity() != null) {
                    getActivity().runOnUiThread(() -> progressDialog.dismiss());
                }
                if (!CommonFunctions.isConnected(getActivity()))
                    myApp.noInternet(getActivity());
            }
        });
    }

    @Override
    protected void initViews(View view) {
        super.initViews(view);

        btnBack = view.findViewById(R.id.btnBack);
        btnNext = view.findViewById(R.id.btnNext);
        cbAgree = view.findViewById(R.id.cbAgreed);
    }
}
