package com.tiffintom.indicator;

import android.content.Context;
import android.content.res.TypedArray;
import android.util.AttributeSet;
import android.view.LayoutInflater;
import android.widget.LinearLayout;

import androidx.annotation.Nullable;
import androidx.recyclerview.widget.GridLayoutManager;
import androidx.recyclerview.widget.RecyclerView;

import com.tiffintom.R;

import java.util.ArrayList;

/**
 * Created by Amrish on 29-10-2020.
 */
public class IndicatorView extends LinearLayout {
    public IndicatorView(Context context) {
        super(context);
    }

    private int activeItem = 2;
    private int totalItems = 3;
    private ArrayList<String> texts = new ArrayList<>();
    private IndicatorRecyclerAdapter adapter;
    private RecyclerView rvIndicator;
    private boolean showText;
    LinearLayout layout = null;

    public IndicatorView(Context context, @Nullable AttributeSet attrs) {
        super(context, attrs);
        initViews(context, attrs);


    }

    public IndicatorView(Context context, @Nullable AttributeSet attrs, int defStyleAttr) {
        super(context, attrs, defStyleAttr);

        initViews(context, attrs);
    }

    private void initViews(Context context, AttributeSet attrs) {
        TypedArray array = context.obtainStyledAttributes(attrs, R.styleable.IndicatorView, 0, 0);
        activeItem = array.getInt(R.styleable.IndicatorView_active_item, 2);
        showText = array.getBoolean(R.styleable.IndicatorView_show_texts, true);
        totalItems = array.getInt(R.styleable.IndicatorView_total_items, 3);
        if (activeItem > totalItems) activeItem = totalItems;
        LayoutInflater li = (LayoutInflater) context.getSystemService(Context.LAYOUT_INFLATER_SERVICE);
        layout = (LinearLayout) li.inflate(R.layout.indicator_view, this, true);
        rvIndicator = layout.findViewById(R.id.rvIndicators);
        adapter = new IndicatorRecyclerAdapter(texts);
        rvIndicator.setAdapter(adapter);
        int i = 1;
        while (i <= totalItems) {
            addItem();
            i++;
        }
        adapter.activePosition = activeItem;
        adapter.notifyDataSetChanged();

        array.recycle();
    }


    @Override
    public void invalidate() {
        super.invalidate();
        rvIndicator.setLayoutManager(new GridLayoutManager(getContext(), totalItems));
        adapter.activePosition = activeItem;
        adapter.showText = showText;
        adapter.notifyDataSetChanged();
    }

    public void setTexts(ArrayList<String> texts) {
        this.texts.clear();
        this.texts.addAll(texts);
        this.totalItems = this.texts.size();
        invalidate();
    }

    public void setCurrentItem(int currentItem) {
        this.activeItem = currentItem;
        invalidate();
    }

    public void setShowText(boolean showText) {
        this.showText = showText;
        invalidate();
    }

    private void addItem() {
        texts.add("Name " + (texts.size() + 1));
        rvIndicator.setLayoutManager(new GridLayoutManager(getContext(), texts.size()));
        adapter.notifyDataSetChanged();
    }

    private void removeItem() {
        if (adapter.activePosition == texts.size()) {
            adapter.activePosition--;
        }
        texts.remove(texts.size() - 1);
        rvIndicator.setLayoutManager(new GridLayoutManager(getContext(), texts.size()));
        adapter.notifyDataSetChanged();
    }

    private void selectNext() {
        if (adapter.activePosition < texts.size()) {
            adapter.activePosition++;
            adapter.notifyDataSetChanged();
        }
    }

    private void selectPrev() {
        if (adapter.activePosition > 0) {
            adapter.activePosition--;
            adapter.notifyDataSetChanged();
        }
    }
}
