package com.tiffintom.models;

import androidx.room.Dao;
import androidx.room.Delete;
import androidx.room.Insert;
import androidx.room.OnConflictStrategy;
import androidx.room.Query;
import androidx.room.Update;

import java.util.List;

/**
 * Created by Amrish on 24-08-2020.
 */
@Dao
public interface DineinCartItemDao {
    @Query("SELECT * FROM DineinCartItem")
    List<DineinCartItem> getAll();

    @Insert
    void insertAll(DineinCartItem... item);

    @Delete
    void delete(DineinCartItem item);

    @Query("SELECT SUM(quantity) as items, SUM(Total) as total FROM DineinCartItem")
    CartSummary getCartSummary();

    @Update
    void update(DineinCartItem item);

    @Update(onConflict = OnConflictStrategy.REPLACE)
    void updateAndReplace(DineinCartItem item);

    @Query("SELECT (SUM(quantity)) FROM dineincartitem WHERE menu_id=:menuId")
    int getMenuItemCartCounts(int menuId);

    @Query("SELECT res_id FROM dineincartitem GROUP BY res_id LIMIT 1")
    int getRestaurantId();

    @Query("DELETE FROM dineincartitem")
    void nukeTable();

    @Query("SELECT * FROM dineincartitem WHERE dineInCartItemId=:id")
    DineinCartItem getDineinCartItem(int id);

    @Query("SELECT * FROM dineincartitem WHERE id=:id")
    DineinCartItem getDineinCartItemFromOrderItemId(int id);

    @Query("SELECT * FROM dineincartitem WHERE menu_id=:menuId AND menu_size=:variantName AND (Addon_name is null or Addon_name= '') ")
    DineinCartItem getCartMenuItem(int menuId, String variantName);


    @Query("SELECT * FROM dineincartitem WHERE menu_id=:menuId AND Addon_name=:variantName")
    DineinCartItem getCartAddonMenuItem(int menuId, String variantName);

    @Query("SELECT * FROM dineincartitem WHERE menu_id=:menuId AND menu_name=:menuName")
    DineinCartItem getCartAddonMenuItemWithName(int menuId, String menuName);


}
