package com.tiffintom.network;

import com.androidnetworking.AndroidNetworking;
import com.androidnetworking.common.ANRequest;

import java.io.File;

/**
 * Created by Amrish on 19-08-2020.
 */
public class ApiUtils {
    public static ANRequest getLogin(Object o) {
        return AndroidNetworking.post(ApiEndPoints.NEW_CUSTOMER_LOGIN)
                .addApplicationJsonBody(o)
                .build();
    }

    public static ANRequest getSignup(Object o) {
        return AndroidNetworking.post(ApiEndPoints.NEW_CUSTOMER_SIGNUP)
                .addApplicationJsonBody(o)
                .build();
    }

    public static ANRequest getHome(String postcode,String customer_id) {
        return AndroidNetworking.post(ApiEndPoints.NEW_HOME_SEARCH)
                .addBodyParameter("postcode", postcode)
                .addBodyParameter("customer_id", customer_id)
                .build();
    }


    public static ANRequest getAddresses(String customer_id) {
        return AndroidNetworking.post(ApiEndPoints.ACCOUNT)
                .addBodyParameter("customer_id", customer_id)
                .addBodyParameter("page", "AddressBookList")
                .addBodyParameter("action", "MyAccount")
                .build();
    }

    public static ANRequest getRestaurantDetails(String restaurant_id,String customer_id,String latitute,String longitude) {
        return AndroidNetworking.get(ApiEndPoints.NEW_Restaurants_DETAILS+""+restaurant_id)
                .addQueryParameter("customer_id",customer_id)
                .addQueryParameter("latitude",latitute)
                .addQueryParameter("longitude",longitude)
                .build();
    }
    public static ANRequest getMenuDetails(String prefilled,String restaurant_id,String nopaginate) {
        return AndroidNetworking.get(ApiEndPoints.NEW_MENU_DETAILS)
                .addQueryParameter("prefilled", prefilled)
                .addQueryParameter("restaurant_id", restaurant_id)
                .addQueryParameter("nopaginate", nopaginate)
                .setTag("menu_details")
                .build();
    }

    public static ANRequest get_Dine_in_MenuDetails(String prefilled,String restaurant_id,String nopaginate) {
        return AndroidNetworking.get(ApiEndPoints.NEW_DINE_IN_MENU_DETAILS)
                .addQueryParameter("prefilled", prefilled)
                .addQueryParameter("restaurant_id", restaurant_id)
                .addQueryParameter("nopaginate", nopaginate)
                .setTag("menu_details")
                .build();
    }
    public static ANRequest getRestaurantTimeSlots(String restaurant_id, String today) {
        return AndroidNetworking.get(ApiEndPoints.NEW_Restaurants_TimeSlots)
                .addQueryParameter("restaurant_id", restaurant_id)
                .addQueryParameter("date", today)
                .setTag("Restaurant_TimeSlots")
                .build();
    }

    public static ANRequest getRestaurantDetails(String restaurant_id, String customer_id, String postcode) {
        return AndroidNetworking.get(ApiEndPoints.NEW_Restaurants_DETAILS+""+restaurant_id)
                .addQueryParameter("postcode", postcode)
                .addQueryParameter("customer_id", customer_id)
                .build();
    }

    public static ANRequest getRestaurantDetails(String restaurant_id, String postcode) {
        return AndroidNetworking.get(ApiEndPoints.business_restaurant_detail + restaurant_id)
                .addQueryParameter("postcode", postcode)
                .setTag("restaurant_details")
                .build();
    }


    public static ANRequest getDineInRestaurant(String qr_code, String postcode) {
        return AndroidNetworking.get(ApiEndPoints.NEW_DINE_IN_Restaurants_DETAILS)
                .addPathParameter("qr_code", qr_code)
                .build();
    }

    public static ANRequest getDineInRestaurantFromBusiness(String qr_code, String businessId) {
        return AndroidNetworking.get(ApiEndPoints.NEW_DINE_IN_Restaurants_DETAILS)
                .addPathParameter("qr_code", qr_code)
                .addHeaders("business-id", businessId)
                .build();
    }

    public static ANRequest getUpcomingReservations(String customer_id) {
        return AndroidNetworking.get(ApiEndPoints.NEW_BOOKING)
                .addQueryParameter("customer_id", customer_id)
                .addQueryParameter("per_page", "20")
                .addQueryParameter("page", "1")
                .addQueryParameter("status", "Pending")
                .build();
    }
    public static ANRequest getPreviousReservations(String customer_id) {
        return AndroidNetworking.get(ApiEndPoints.NEW_BOOKING)
                .addQueryParameter("customer_id", customer_id)
                .addQueryParameter("per_page", "20")
                .addQueryParameter("page", "1")
                .addQueryParameter("previous_status", "1")
                .build();
    }
    public static ANRequest getFavouriteAction(String customer_id, String restaurantId, String status) {
        return AndroidNetworking.post(ApiEndPoints.FAVOURITE_RESTAURANTS)
                .addBodyParameter("restaurant_id", restaurantId)
                .addBodyParameter("customer_id", customer_id)
                .addBodyParameter("is_favourite", status)
                .build();
    }

    public static ANRequest bookATable(Object o) {
        return AndroidNetworking.post(ApiEndPoints.NEW_BOOKING)
                .addApplicationJsonBody(o)
                .build();
    }

    public static ANRequest saveAddress(String address, String customer_id, String flat_no, String title, String zipcode, String lat, String longitude, String resid) {
      if (resid == null){
        return AndroidNetworking.post(ApiEndPoints.NEW_ADDRESSBOOK)
                .addBodyParameter("address", address)
                .addBodyParameter("user_id", customer_id)
                .addBodyParameter("flat_no", flat_no)
                .addBodyParameter("title", title)
                .addBodyParameter("zipcode", zipcode)
                .addBodyParameter("latitude", lat)
                .addBodyParameter("longitude", longitude)
                .build();}
      else {
          return AndroidNetworking.post(ApiEndPoints.NEW_ADDRESSBOOK)
                  .addBodyParameter("address", address)
                  .addBodyParameter("user_id", customer_id)
                  .addBodyParameter("flat_no", flat_no)
                  .addBodyParameter("title", title)
                  .addBodyParameter("zipcode", zipcode)
                  .addBodyParameter("latitude", lat)
                  .addBodyParameter("longitude", longitude)
                  .addBodyParameter("restaurant_id", resid)
                  .build();
      }
    }

    public static ANRequest updateAddress(String addressId, String address, String customer_id, String flat_no, String title, String zipcode, String lat, String longitude) {
        return AndroidNetworking.post(ApiEndPoints.NEW_ADDRESSBOOK+"/"+addressId)
                .addBodyParameter("address", address)
                .addBodyParameter("user_id", customer_id)
                .addBodyParameter("flat_no", flat_no)
                .addBodyParameter("title", title)
                .addBodyParameter("zipcode", zipcode)
                .addBodyParameter("latitude", lat)
                .addBodyParameter("longitude", longitude)
                .build();
    }

    public static ANRequest getAccount(String action, String page, String customer_id) {
        return AndroidNetworking.post(ApiEndPoints.ACCOUNT)
                .addBodyParameter("action", action)
                .addBodyParameter("page", page)
                .addBodyParameter("customer_id", customer_id)
                .build();
    }

    public static ANRequest removeSavedCard(String customer_id, String cardId) {
        return AndroidNetworking.post(ApiEndPoints.ACCOUNT)
                .addBodyParameter("action", "MyAccount")
                .addBodyParameter("page", "SavedCardDelete")
                .addBodyParameter("customer_id", customer_id)
                .addBodyParameter("stripeId", cardId)
                .build();
    }

    public static ANRequest getAccount(Object o, File image) {
        if (image == null) {

            return AndroidNetworking.post(ApiEndPoints.ACCOUNT)
                    .addApplicationJsonBody(o)
                    .build();
        } else {
            return AndroidNetworking.upload(ApiEndPoints.ACCOUNT)
                    .addMultipartFile("image", image)
                    .addMultipartParameter(o)
                    .build();
        }
    }
    public static ANRequest getFavouritesList(String customer_id, String nopaginate,String is_favourite) {
        return AndroidNetworking.get(ApiEndPoints.FAVOURITE_RESTAURANTS)
                .addQueryParameter("customer_id", customer_id)
                .addQueryParameter("nopaginate", nopaginate)
                .addQueryParameter("is_favourite", is_favourite)
                .build();
    }
    public static ANRequest applyPromoCode(String customer_id, String resId, String voucherCode,String orderType,String deliveryData) {
        return AndroidNetworking.post(ApiEndPoints.NEW_VOUCHERS+voucherCode)
                .addBodyParameter("restaurant_id", resId) //01-09-2020
                .addBodyParameter("customer_id", customer_id)
                .addBodyParameter("voucher_code", voucherCode)
                .addBodyParameter("order_type", orderType)
                .addBodyParameter("delivery_date", deliveryData)
                .addHeaders("device-type","Android")
                .build();
    }

    public static ANRequest getDineinVouchers(String business_id, String customer_id, String disabled, String order_type, String from_date, String to_date, String nopaginate) {
        return AndroidNetworking.get(ApiEndPoints.dinein_voucher)
                .addHeaders("business-id", business_id)
                .addQueryParameter("customer_id", customer_id)
                .addQueryParameter("disabled", disabled)
                .addQueryParameter("order_type", order_type)
                .addQueryParameter("from_date", from_date)
                .addQueryParameter("to_date", to_date)
                .addQueryParameter("nopaginate", "1")
                .build();
    }

    public static ANRequest applyPromoCode(String orderType, String resId, String voucherCode) {
        return AndroidNetworking.post(ApiEndPoints.apply_vouchers + voucherCode)
                .addBodyParameter("restaurant_id", resId)
                .addBodyParameter("order_type", orderType) //01-09-2020
                .addHeaders("device-type", "Android")
                .build();
    }


    public static ANRequest getCheckoutAddresses(int customerId, int restaurant_id,int nopaginate) {
        return AndroidNetworking.get(ApiEndPoints.NEW_ADDRESSBOOK)
                .addQueryParameter("customer_id", String.valueOf(customerId))
                .addQueryParameter("restaurant_id", String.valueOf(restaurant_id))
                .addQueryParameter("nopaginate", String.valueOf(nopaginate))
                .build();
    }

    public static ANRequest updateCartOnline(int customerId, String cartDetails) {
        return AndroidNetworking.post(ApiEndPoints.NEW_CARTUPDATEONLINE)
                .addBodyParameter("customer_id", String.valueOf(customerId))
                .addBodyParameter("carts", cartDetails)
                .build();
    }

    public static ANRequest deleteCartOnline() {
        return AndroidNetworking.delete(ApiEndPoints.cart_online_delete)
                .build();
    }

    public static ANRequest getDineInEposAddons(int product_id, String disabled, String nopaginate, String business_id) {
        return AndroidNetworking.get(ApiEndPoints.dinein_addons)
                .addQueryParameter("product_id", String.valueOf(product_id))
                .addQueryParameter("disabled", disabled)
                .addQueryParameter("nopaginate", nopaginate)
                .addHeaders("business-id", business_id)
                .setTag("restaurant_dinein_addons")
                .build();
    }

    public static ANRequest getDineInEposIngredients(int product_id, String disabled, String nopaginate, String business_id) {
        return AndroidNetworking.get(ApiEndPoints.dinein_ingredients)
                .addQueryParameter("product_id", String.valueOf(product_id))
                .addQueryParameter("disabled", disabled)
                .addQueryParameter("nopaginate", nopaginate)
                .addHeaders("business-id", business_id)
                .setTag("restaurant_dinein_ingredients")
                .build();
    }


    public static ANRequest getDineInEposMenus(String category_id, String prefilled, String disabled, String nopaginate, String business_id) {
        return AndroidNetworking.get(ApiEndPoints.dinein_menu)
                .addQueryParameter("disabled", disabled)
                .addQueryParameter("nopaginate", nopaginate)
                .addQueryParameter("category_id", category_id)
                .addHeaders("business-id", business_id)
                .setTag("restaurant_dinein_menu")
                .build();
    }


    public static ANRequest getDineInEposCategories(String prefilled, String disabled, String nopaginate, String business_id) {
        return AndroidNetworking.get(ApiEndPoints.dinein_categories)
                .addQueryParameter("prefilled", prefilled)
                .addQueryParameter("disabled", disabled)
                .addQueryParameter("nopaginate", nopaginate)
                .addHeaders("business-id", business_id)
                .setTag("restaurant_dinein_categories")
                .build();
    }

    public static ANRequest applyDineinPromoCode(String business_id, String orderType, String resId, String voucherCode) {
        return AndroidNetworking.post(ApiEndPoints.dinein_apply_vouchers + voucherCode)
                .addBodyParameter("restaurant_id", resId)
                .addBodyParameter("order_type", orderType)
                .addHeaders("business-id", business_id)
                .build();
    }
}
