package com.tiffintom.notification;

import static com.facebook.FacebookSdk.getApplicationContext;

import android.content.Context;
import android.content.Intent;

import androidx.annotation.NonNull;
import androidx.localbroadcastmanager.content.LocalBroadcastManager;

import com.google.firebase.messaging.FirebaseMessagingService;
import com.google.firebase.messaging.RemoteMessage;
import com.google.gson.Gson;
import com.tiffintom.MyApp;
import com.tiffintom.activity.MainActivity;
import com.tiffintom.common.Validators;
import com.tiffintom.utils.LogUtils;

import org.json.JSONException;
import org.json.JSONObject;

/**
 * Created by Aradhna on 11-12-2021.
 */
public class MyFirebaseMessagingService extends FirebaseMessagingService {

    private static final String TAG = MyFirebaseMessagingService.class.getSimpleName();
    private NotificationUtils notificationUtils;
    String message = "", title = "", type = "", order_status = "", order_id, booking_id;

    @Override
    public void onMessageReceived(RemoteMessage remoteMessage) {
        LogUtils.e("Notification received here!!");
        LogUtils.e("FCM: " + remoteMessage.getFrom());
        if (remoteMessage.getNotification() != null)
            LogUtils.e("FCM: " + remoteMessage.getNotification().getTitle());

        if (remoteMessage.getData().size() > 0) {
            try {
                JSONObject jsonObject = new JSONObject(remoteMessage.getData().toString());
                LogUtils.e("json reponce::" + new Gson().toJson(jsonObject));
                handleDataMessage(jsonObject);
                // Check if message contains a notification payload.
                if (remoteMessage.getNotification() != null) {
                    handleNotification(remoteMessage.getNotification().getBody());
                }
            } catch (JSONException e) {
                e.printStackTrace();
                LogUtils.e("json error::" + e.getLocalizedMessage());
            }
        }
        Intent pushNotification = new Intent(Config.PUSH_NOTIFICATION);
        pushNotification.putExtra("refresh", true);
        pushNotification.putExtra("orders", true);
        pushNotification.putExtra("reservations", true);
        pushNotification.putExtra("chat", true);
        pushNotification.putExtra("reviews", true);
        LocalBroadcastManager.getInstance(getApplicationContext()).sendBroadcast(pushNotification);
    }

    private void handleNotification(String message) {
        if (!NotificationUtils.isAppIsInBackground(getApplicationContext())) {
            // app is in foreground, broadcast the push message
            Intent pushNotification = new Intent(Config.PUSH_NOTIFICATION);
            pushNotification.putExtra("refresh", true);
            LocalBroadcastManager.getInstance(this).sendBroadcast(pushNotification);

            NotificationUtils notificationUtils = new NotificationUtils(getApplicationContext());
            notificationUtils.playNotificationSound();

        }
    }

    private void handleDataMessage(JSONObject jsonObject) {
        LogUtils.e("Notification Response");
        try {
            JSONObject data = jsonObject.getJSONObject("data");
            JSONObject payload = data.getJSONObject("payload");
            type = payload.getString("type");
            title = data.getString("title");
            message = data.getString("message");
            if (data.has("order_status"))
                order_status = data.getString("order_status");
            if (data.has("order_id"))
                order_id = data.getString("order_id");

            String timestamp = data.getString("timestamp");

            if (!NotificationUtils.isAppIsInBackground(getApplicationContext())) {
                // app is in foreground, broadcast the push message
                Intent pushNotification = new Intent(Config.PUSH_NOTIFICATION);
                pushNotification.putExtra("refresh", true);

                LocalBroadcastManager.getInstance(this).sendBroadcast(pushNotification);

                // play notification sound
                NotificationUtils notificationUtils = new NotificationUtils(getApplicationContext());
                notificationUtils.playNotificationSound();

                Intent resultIntent = new Intent(getApplicationContext(), MainActivity.class);
                if (type.equalsIgnoreCase("dinein")) {
                    resultIntent.putExtra("destination", "orderView").putExtra("order_id", order_id).putExtra("is_dinein", true);
                } else if (type.equalsIgnoreCase("order")) {
                    if (order_status.equalsIgnoreCase("failed") || order_status.equalsIgnoreCase("delivered")) {
                        resultIntent.putExtra("destination", "orderView").putExtra("order_id", order_id);
                    } else
                        resultIntent.putExtra("destination", "trackOrder").putExtra("order_id", order_id);
                } else if (type.equalsIgnoreCase("booking")) {
                    resultIntent.putExtra("destination", "reservation_history");//call reservation activity
                } else if (type.equalsIgnoreCase("restaurantchat")) {
                    resultIntent.putExtra("destination", "restaurant_chat").putExtra("order_id", order_id);//
                } else if (type.equalsIgnoreCase("tiffintomchat")) {
                    resultIntent.putExtra("destination", "tiffintom_chat_window").putExtra("order_id", order_id);//
                }
                showNotificationMessage(getApplicationContext(), type, title, message, timestamp, resultIntent);

            } else if (NotificationUtils.isAppIsInBackground(getApplicationContext())) {
                Intent resultIntent = new Intent(getApplicationContext(), MainActivity.class);
                if (type.equalsIgnoreCase("order")) {
                    if (order_status.equalsIgnoreCase("failed") || order_status.equalsIgnoreCase("delivered")) {
                        resultIntent.putExtra("destination", "orderView").putExtra("order_id", order_id);
                    } else
                        resultIntent.putExtra("destination", "trackOrder").putExtra("order_id", order_id);
                } else if (type.equalsIgnoreCase("booking")) {
                    resultIntent.putExtra("destination", "reservation_history");//call reservation activity
                } else if (type.equalsIgnoreCase("restaurantchat")) {
                    resultIntent.putExtra("destination", "restaurant_chat").putExtra("order_id", order_id);//
                } else if (type.equalsIgnoreCase("tiffintomchat")) {
                    resultIntent.putExtra("destination", "tiffintom_chat_window").putExtra("order_id", order_id);//
                }
                showNotificationMessage(getApplicationContext(), type, title, message, timestamp, resultIntent);

            } else {
                Intent resultIntent = new Intent(getApplicationContext(), MainActivity.class);
                if (type.equalsIgnoreCase("order")) {
                    if (order_status.equalsIgnoreCase("failed") || order_status.equalsIgnoreCase("delivered")) {
                        resultIntent.putExtra("destination", "orderView").putExtra("order_id", order_id);
                    } else
                        resultIntent.putExtra("destination", "trackOrder").putExtra("order_id", order_id);
                } else if (type.equalsIgnoreCase("booking")) {
                    resultIntent.putExtra("destination", "reservation_history");//call reservation activity
                } else if (type.equalsIgnoreCase("restaurantchat")) {
                    resultIntent.putExtra("destination", "restaurant_chat").putExtra("order_id", order_id);//
                } else if (type.equalsIgnoreCase("tiffintomchat")) {
                    resultIntent.putExtra("destination", "tiffintom_chat_window").putExtra("order_id", order_id);//
                }
                showNotificationMessage(getApplicationContext(), type, title, message, timestamp, resultIntent);
            }
        } catch (Exception e) {
            LogUtils.e(TAG, "Exception: " + e.getMessage());
        }
    }

    /**
     * Showing notification with text only
     */
    private void showNotificationMessage(Context context, String type, String title, String message, String timeStamp, Intent intent) {
        if (!Validators.isNullOrEmpty(String.valueOf(MyApp.getInstance().getMyPreferences().getLoggedInUser().id))) {
            notificationUtils = new NotificationUtils(context);
            intent.setFlags(Intent.FLAG_ACTIVITY_NEW_TASK | Intent.FLAG_ACTIVITY_CLEAR_TASK);
            notificationUtils.showNotificationMessage(title, message, timeStamp, intent);
        }

    }

    @Override
    public void onNewToken(@NonNull String s) {
        super.onNewToken(s);
        MyApp.getInstance().getMyPreferences().setUserFCMToken(s);
    }
}