package com.tiffintom.notification;

import android.app.Activity;
import android.app.Service;
import android.content.Context;
import android.content.Intent;
import android.net.ConnectivityManager;
import android.net.NetworkInfo;
import android.os.IBinder;
import android.util.Log;

import androidx.annotation.Nullable;
import androidx.localbroadcastmanager.content.LocalBroadcastManager;

import com.androidnetworking.AndroidNetworking;
import com.androidnetworking.error.ANError;
import com.androidnetworking.interfaces.JSONObjectRequestListener;
import com.google.gson.Gson;
import com.tiffintom.MyApp;
import com.tiffintom.activity.MainActivity;
import com.tiffintom.activity.ReservationHistoryActivity;
import com.tiffintom.activity.SplashActivity;
import com.tiffintom.activity.TransportActivity;
import com.tiffintom.models.User;
import com.tiffintom.utils.LogUtils;

import org.json.JSONArray;
import org.json.JSONException;
import org.json.JSONObject;

import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.Timer;
import java.util.TimerTask;

import static com.tiffintom.network.ApiEndPoints.ACCOUNT;

/**
 * Created by fabio on 30/01/2016.
 */
public class NotificationService extends Service {
    public int counter = 0;
    String message = "";
    String title = "", resId = "";
    String orderId = "", notificationId = "0";
    private NotificationUtils notificationUtils;
    public NotificationService notificationService = null;
    private User loggedInUser = MyApp.getInstance().getMyPreferences().getLoggedInUser();

    public NotificationService getInstance(Activity activity) {

        if (notificationService == null) {
            notificationService = new NotificationService(activity);
        }
        return notificationService;
    }

    public NotificationService(Context activity) {
        super();

        LogUtils.e("HERE_CONST", "here I am!");
    }

    public NotificationService() {
    }

    public NotificationService getInstance(MyApp myApplication) {
        if (notificationService == null) {
            notificationService = new NotificationService();
        }
        return notificationService;
    }

    @Override
    public int onStartCommand(Intent intent, int flags, int startId) {
        super.onStartCommand(intent, flags, startId);
        notificationUtils = new NotificationUtils(getApplicationContext());
        startTimer();
        return START_STICKY;
    }

    @Override
    public void onDestroy() {
        super.onDestroy();
        Log.i("EXIT", "ondestroy!");
        Intent broadcastIntent = new Intent(this, SensorRestarterBroadcastReceiver.class);
        sendBroadcast(broadcastIntent);
        stoptimertask();
    }

    private Timer timer;
    private TimerTask timerTask;
    long oldTime = 0;

    public void startTimer() {
        //set a new Timer
        timer = new Timer();

        //initialize the TimerTask's job
        if (isConnected(getApplicationContext()))
            initializeTimerTask();
        else LogUtils.e("No internet in notification");
        //schedule the timer, to wake up every 2 second
        if (timerTask != null) timer.schedule(timerTask, 2000, 2000); //
    }

    public static boolean isConnected(Context context) {
        ConnectivityManager cm = (ConnectivityManager) context.getSystemService(Context.CONNECTIVITY_SERVICE);
        NetworkInfo activeNetwork = cm.getActiveNetworkInfo();

        return activeNetwork != null && activeNetwork.isConnected();
    }

    /**
     * it sets the timer to print the counter every x seconds
     */
    public void initializeTimerTask() {
        timerTask = new TimerTask() {
            public void run() {
                try {
                    loggedInUser = MyApp.getInstance().getMyPreferences().getLoggedInUser();
                    if (loggedInUser != null && loggedInUser.id > 0) {
                        AndroidNetworking.post(ACCOUNT)
                                .addBodyParameter("customer_id", String.valueOf(loggedInUser.id))
                                .addBodyParameter("page", "notificationdata")
                                .addBodyParameter("action", "MyAccount")
                                .build()
                                .getAsJSONObject(new JSONObjectRequestListener() {
                                    @Override
                                    public void onResponse(JSONObject jsonObject) {
                                        try {
                                            if (jsonObject.getString("delivery_status").equalsIgnoreCase("OK")) {

                                                JSONObject jresult = jsonObject.getJSONObject("result");

                                                if (jresult.getInt("success") == 1) {
                                                    JSONArray jsonArray = jresult.getJSONArray("notification");

                                                    LogUtils.e("Notifcation: ", new Gson().toJson(jsonArray));
                                                    title = jsonArray.getJSONObject(0).getString("title");
                                                    message = jsonArray.getJSONObject(0).getString("content");
                                                    orderId = jsonArray.getJSONObject(0).getString("order_id");
                                                    resId = jsonArray.getJSONObject(0).getString("resid");
                                                    notificationId = String.valueOf(jsonArray.getJSONObject(0).get("id"));
                                                    String timestamp = null;
                                                    if (jsonArray.getJSONObject(0).has("created")) {
                                                        timestamp = jsonArray.getJSONObject(0).getString("created");
                                                    }
                                                    Intent intent = null;

                                                    try {
                                                        if (!NotificationUtils.isAppIsInBackground(getApplicationContext())) {
                                                            // app is in foreground, broadcast the push message
                                                            Intent pushNotification = new Intent(Config.PUSH_NOTIFICATION);
                                                            pushNotification.putExtra("refresh", true);
                                                            LocalBroadcastManager.getInstance(getApplicationContext()).sendBroadcast(pushNotification);

                                                            // play notification sound
                                                        }
                                                        intent = getCustomIntent(title, message, orderId, resId);
                                                        intent.putExtra("hideToolbar", true);
                                                        intent.putExtra("order_id", orderId);
                                                        intent.putExtra("resid", resId);
                                                        notificationUtils.showNotificationMessage(title, message, timestamp, intent);

                                                    } catch (Exception e) {
                                                        e.printStackTrace();
                                                    }

                                                    changeNotificationStatus(jsonArray.getJSONObject(0).getString("resid"), jsonArray.getJSONObject(0).getString("order_id"));


                                                }
                                            }
                                        } catch (JSONException e) {
                                            e.printStackTrace();
                                        }


                                    }

                                    @Override
                                    public void onError(ANError anError) {

                                    }
                                });

                    }
                } catch (Exception e) {
                    e.printStackTrace();
                    LogUtils.e("Order_response_error", e.getMessage());
                }

            }
        };
    }

    private Intent getCustomIntent(String title, String message, String orderId, String resId) {
        String lowerCaseTitle = title.toLowerCase();
        Intent intent = new Intent(getApplicationContext(), SplashActivity.class);
        if (lowerCaseTitle.contains("reservation")) {
            intent = new Intent(getApplicationContext(), ReservationHistoryActivity.class);
        } else if (lowerCaseTitle.contains("dine in")) {
            if (lowerCaseTitle.contains("placed") || lowerCaseTitle.contains("accepted") || lowerCaseTitle.contains("rejected") || lowerCaseTitle.contains("on its way") || lowerCaseTitle.contains("delayed") || lowerCaseTitle.contains("served")) {
                intent = new Intent(getApplicationContext(), TransportActivity.class);
                intent.putExtra("destination", "orderView");
                intent.putExtra("order_id", orderId);
                intent.putExtra("is_dinein", true);
            }
        } else if (lowerCaseTitle.contains("placed") || lowerCaseTitle.contains("accepted") || lowerCaseTitle.contains("declined") || lowerCaseTitle.contains("rejected") || lowerCaseTitle.contains("on its way") || lowerCaseTitle.contains("delayed") || lowerCaseTitle.contains("delivered")) {
            intent = new Intent(getApplicationContext(), TransportActivity.class);
            intent.putExtra("destination", "trackOrder");
            intent.putExtra("order_id", orderId);
        } else if (lowerCaseTitle.contains("review")) {
            intent = new Intent(getApplicationContext(), TransportActivity.class);
            intent.putExtra("destination", "orderView");
            intent.putExtra("order_id", orderId);
            intent.putExtra("shouldRate", true);
        } else if (message.toLowerCase().contains("left items in your cart")) {
            intent = new Intent(getApplicationContext(), MainActivity.class);
            intent.putExtra("destination", "cart");
        } else if (lowerCaseTitle.contains("tiffintom support")) {
            intent = new Intent(getApplicationContext(), TransportActivity.class);
            intent.putExtra("destination", "tiffintom_chat_window");
        } else if (lowerCaseTitle.contains("chat message")) {
            intent = new Intent(getApplicationContext(), TransportActivity.class);
            intent.putExtra("destination", "restaurant_chat");
            intent.putExtra("restaurant_id", resId);
        }

        return intent;
    }

    private void changeNotificationStatus(String resid, String order_id) {
        AndroidNetworking.post(ACCOUNT)
                .addBodyParameter("resid", resid)
                .addBodyParameter("customer_id", String.valueOf(loggedInUser.id))
                .addBodyParameter("order_id", order_id)
                .addBodyParameter("page", "notificationstatuschange")
                .addBodyParameter("action", "MyAccount")
                .build()
                .getAsJSONObject(new JSONObjectRequestListener() {
                    @Override
                    public void onResponse(JSONObject response) {
                        LogUtils.e("Notification changed Response: ", new Gson().toJson(response));
                    }

                    @Override
                    public void onError(ANError anError) {
                        anError.printStackTrace();
                        LogUtils.e("Notification delivery_status change anerror");
                    }
                });

    }

    /**
     * not needed
     */
    public void stoptimertask() {
        //stop the timer, if it's not already null
        if (timer != null) {
            timer.cancel();
            timer = null;
        }
    }

    @Nullable
    @Override
    public IBinder onBind(Intent intent) {
        return null;
    }

    public static long getTimeMilliSec(String timeStamp) {
        SimpleDateFormat format = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
        try {
            Date date = format.parse(timeStamp);
            return date.getTime();
        } catch (ParseException e) {
            e.printStackTrace();
        }
        return 0;
    }


}