package com.tiffintom.storageutils;

import android.content.Context;
import android.content.SharedPreferences;

import com.google.gson.Gson;
import com.google.gson.reflect.TypeToken;
import com.tiffintom.common.Validators;
import com.tiffintom.models.Address;
import com.tiffintom.models.BusinessRestaurant;
import com.tiffintom.models.ButtonStates;
import com.tiffintom.models.DineinModels.DineinPaymentMethods;
import com.tiffintom.models.DineinModels.DineinSiteSettings;
import com.tiffintom.models.DineinModels.TablesStatuses;
import com.tiffintom.models.DineinModels.UsersList;
import com.tiffintom.models.PaymentMethod;
import com.tiffintom.models.Postcode;
import com.tiffintom.models.Referral;
import com.tiffintom.models.Restaurant;
import com.tiffintom.models.RewardPoint;
import com.tiffintom.models.SiteSettings;
import com.tiffintom.models.User;
import com.tiffintom.utils.LogUtils;

import java.lang.reflect.Type;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * Created by Amrish on 08-08-2020.
 */
public class MyPreferences {
    private Context context;
    private String PREFERENCE_NAME = "Tiffintom";
    private String PREFERENCE_SETTINGS = "SiteSettings";
    private String PREFERENCE_DEVICE = "DeviceDetails";
    private String PREFERENCE_CART_RELATED = "Cart";
    private String PREFERENCE_REMEMBERDETAIL_RELATED = "RememberDetail";
    private String PREFERENCE_DINEIN_SETTINGS = "DineInSiteSettings";
    private String PREFERENCE_RESTAURANT_RELATED = "CurrentRestaurant";
    private String PREFERENCE_DINEINRESTAURANT_RELATED = "CurrentDineinRestaurant";
    private SharedPreferences userPreferences, siteSharedPreferences, cartPreferences, deviceSharedPref, RememberDetailSharedPref, dineInSiteSharedPreferences, dineinRestuarntPrefrence,qrPrefrence;
    private SharedPreferences.Editor editor;

    private String KEY_DATABASE_DETAILS = "database_connection_details";
    private String KEY_ACTIVE_TERMINAL = "active_terminal";
    private String KEY_LOGGED_IN_USER = "logged_in_user";
    private String KEY_LOGIN_SKIPPED = "login_skipped";
    private String KEY_RES_QRCODE = "res_qrcode";
    private String KEY_IS_LOGOUT = "is_login";
    private String KEY_CURRENT_POSTCODE = "current_postcode";
    private String KEY_CURRENT_ADDRESS = "current_address";
    private String KEY_DEVICE_ID = "device_id";
    private static final String KEY_REMEMBER_DETAILS = "remember_me";
    private static final String KEY_SITE_SETTINGS = "fetched_settings";
    private static final String KEY_REFERRAL = "fetched_referrals";
    private static final String KEY_REWARDS = "fetched_rewards";
    private static final String KEY_COUNTRY = "selected_country";  // values should be in IN, UK
    private static final String KEY_DEFAULT_PAYMENT_METHOD = "default_payment_method";
    private String KEY_FCM_ID = "fcm_id";
    private String KEY_RESTAURANT = "current_restaurant";
    private String KEY_RESTAURANT_BUTTON_STATES = "current_restaurant_button_states";
    private String KEY_RESTAURANT_ID = "id";
    private String KEY_RESTAURANT_NAME = "name";
    private static final String KEY_WELCOMED = "welcomed";
    private static final String KEY_REVIEWS = "fetched_review";
    private static final String KEY_DINEIN_SITE_SETTINGS = "fetched_settings";
    private static final String KEY_DINEIN_PAYMENT_METHOD = "dinein_payment_method";
    private static final String KEY_DINEIN_TABLES = "dine_in_tables";
    private static final String KEY_DINEIN_USER_MINI = "dinein_user_mini";
    private String PREFERENCE_QRCODE = "QrCode";

    public MyPreferences(Context context) {
        this.context = context;
        userPreferences = context.getSharedPreferences(PREFERENCE_NAME, Context.MODE_PRIVATE);
        siteSharedPreferences = context.getSharedPreferences(PREFERENCE_SETTINGS, Context.MODE_PRIVATE);
        deviceSharedPref = context.getSharedPreferences(PREFERENCE_DEVICE, Context.MODE_PRIVATE);
        cartPreferences = context.getSharedPreferences(PREFERENCE_CART_RELATED, Context.MODE_PRIVATE);
        RememberDetailSharedPref = context.getSharedPreferences(PREFERENCE_REMEMBERDETAIL_RELATED, Context.MODE_PRIVATE);
        cartPreferences = context.getSharedPreferences(PREFERENCE_RESTAURANT_RELATED, Context.MODE_PRIVATE);
        dineinRestuarntPrefrence = context.getSharedPreferences(PREFERENCE_DINEINRESTAURANT_RELATED, Context.MODE_PRIVATE);
        dineInSiteSharedPreferences = context.getSharedPreferences(PREFERENCE_DINEIN_SETTINGS, Context.MODE_PRIVATE);
        qrPrefrence = context.getSharedPreferences(PREFERENCE_QRCODE, Context.MODE_PRIVATE);
    }

    public void saveLoggedInUser(User user) {
        if (userPreferences != null && user != null) {
            SharedPreferences.Editor editor = userPreferences.edit();
            editor.putString(KEY_LOGGED_IN_USER, new Gson().toJson(user));
            editor.apply();
        }
    }

    public User getLoggedInUser() {
        if (userPreferences != null) {
            String savedValue = userPreferences.getString(KEY_LOGGED_IN_USER, null);
            if (Validators.isNullOrEmpty(savedValue)) return null;
            return new Gson().fromJson(savedValue, User.class);
        } else return null;
    }

    public void logout() {
        if (userPreferences != null) {
            userPreferences.edit().clear().apply();
        }
        if (cartPreferences != null) {
            cartPreferences.edit().clear().apply();
        }
    }

    public void login_skipped(boolean skipped) {
        if (userPreferences != null) {
            userPreferences.edit().putBoolean(KEY_LOGIN_SKIPPED, skipped).apply();
        }
    }

    public boolean isLoginSkipped() {
        if (userPreferences != null) {
            return userPreferences.getBoolean(KEY_LOGIN_SKIPPED, false);
        }
        return false;
    }

    public void saveCurrentPostcode(Postcode postcode) {
        if (userPreferences != null) {
            userPreferences.edit().putString(KEY_CURRENT_POSTCODE, new Gson().toJson(postcode)).apply();
        }
    }

    public Postcode getCurrentPostcode() {
        if (userPreferences != null) {
            String savedValue = userPreferences.getString(KEY_CURRENT_POSTCODE, null);
            if (Validators.isNullOrEmpty(savedValue)) return null;
            return new Gson().fromJson(savedValue, Postcode.class);
        } else return null;
    }

    public void setDeviceId(String deviceId) {
        if (deviceSharedPref != null) {
            deviceSharedPref.edit().putString(KEY_DEVICE_ID, deviceId).apply();
        }
    }

    public String getDeviceId() {
        if (deviceSharedPref != null) {
            return deviceSharedPref.getString(KEY_DEVICE_ID, null);
        }
        return null;
    }

    public void saveRememberDetails(User user) {
        if (RememberDetailSharedPref != null) {
            RememberDetailSharedPref.edit().putString(KEY_REMEMBER_DETAILS, new Gson().toJson(user)).apply();
        }
    }

    public User getRememberDetails() {
        if (RememberDetailSharedPref != null) {
            String savedValue = RememberDetailSharedPref.getString(KEY_REMEMBER_DETAILS, null);
            if (Validators.isNullOrEmpty(savedValue)) return null;
            return new Gson().fromJson(savedValue, User.class);
        }
        return null;
    }

    public void removeRememberDetails() {
        if (RememberDetailSharedPref != null) {
            RememberDetailSharedPref.edit().putString(KEY_REMEMBER_DETAILS, null).apply();
        }
    }

    public void saveSiteSettings(SiteSettings settings) {
        if (siteSharedPreferences != null) {
            siteSharedPreferences.edit().putString(KEY_SITE_SETTINGS, new Gson().toJson(settings)).apply();
        }
    }

    public SiteSettings getSiteSettings() {
        if (siteSharedPreferences != null) {
            String savedValue = siteSharedPreferences.getString(KEY_SITE_SETTINGS, null);
            if (Validators.isNullOrEmpty(savedValue)) return null;
            return new Gson().fromJson(savedValue, SiteSettings.class);
        }
        return null;
    }

    public void saveReferral(Referral settings) {
        if (siteSharedPreferences != null) {
            siteSharedPreferences.edit().putString(KEY_REFERRAL, new Gson().toJson(settings)).apply();
        }
    }

    public Referral getReferral() {
        if (siteSharedPreferences != null) {
            String savedValue = siteSharedPreferences.getString(KEY_REFERRAL, null);
            if (Validators.isNullOrEmpty(savedValue)) return null;
            return new Gson().fromJson(savedValue, Referral.class);
        }
        return null;
    }

    public void saveRewardPoint(RewardPoint settings) {
        if (siteSharedPreferences != null) {
            siteSharedPreferences.edit().putString(KEY_REWARDS, new Gson().toJson(settings)).apply();
        }
    }

    public RewardPoint getRewardPoint() {
        if (siteSharedPreferences != null) {
            String savedValue = siteSharedPreferences.getString(KEY_REWARDS, null);
            if (Validators.isNullOrEmpty(savedValue)) return null;
            return new Gson().fromJson(savedValue, RewardPoint.class);
        }
        return null;
    }

    public void saveSelectedCountry(String country) {
        if (siteSharedPreferences != null) {
            siteSharedPreferences.edit().putString(KEY_COUNTRY, country).apply();
        }
    }

    public String getSelectedCountry() {
        if (siteSharedPreferences != null) {
            String savedValue = siteSharedPreferences.getString(KEY_COUNTRY, null);
            if (Validators.isNullOrEmpty(savedValue)) return null;
            return savedValue;
        }
        return null;
    }

    public void saveDefaultPaymentMethod(PaymentMethod method) {
        if (userPreferences != null) {
            userPreferences.edit().putString(KEY_DEFAULT_PAYMENT_METHOD, new Gson().toJson(method)).apply();
        }
    }

    public PaymentMethod getDefaultPaymentMethod() {
        if (userPreferences != null) {
            String savedValue = userPreferences.getString(KEY_DEFAULT_PAYMENT_METHOD, null);
            if (Validators.isNullOrEmpty(savedValue)) return null;
            return new Gson().fromJson(savedValue, PaymentMethod.class);
        }
        return null;
    }

    public void saveCurrentAddress(Address address) {
        if (userPreferences != null) {
            userPreferences.edit().putString(KEY_CURRENT_ADDRESS, new Gson().toJson(address)).apply();
        }
    }

    public Address getCurrentAddress() {
        if (userPreferences != null) {
            String savedValue = userPreferences.getString(KEY_CURRENT_ADDRESS, null);
            if (Validators.isNullOrEmpty(savedValue)) return null;
            return new Gson().fromJson(savedValue, Address.class);
        } else return null;
    }

    public void saveOrderRestaurant(Restaurant restaurant) {
        if (cartPreferences != null) {
            cartPreferences.edit().putString(PREFERENCE_CART_RELATED, new Gson().toJson(restaurant)).apply();
        }
    }

    public Restaurant getOrderRestaurant() {
        if (cartPreferences != null) {
            String savedValue = cartPreferences.getString(PREFERENCE_CART_RELATED, null);
            if (Validators.isNullOrEmpty(savedValue)) return null;
            return new Gson().fromJson(savedValue, Restaurant.class);
        } else return null;
    }

    public void deleteOrderRestaurant() {
        if (cartPreferences != null)
            cartPreferences.edit().clear().apply();
    }

    public String getToken(String token) {
        return userPreferences.getString(token, null);
    }

    public Map<String, String> getHeaderTokens() {

        String xToken = getToken("x-token");
        String xRefreshToken = getToken("x-refresh-token");

        Map<String, String> map = new HashMap<>();
        if (xToken != null)
            map.put("x-token", xToken);
        if (xRefreshToken != null)
            map.put("x-refresh-token", xRefreshToken);

        return map;
    }

    public void saveHeaderTokens(String xToken, String rToken) {
        SharedPreferences.Editor editor = userPreferences.edit();
        editor.putString("x-token", xToken);
        editor.putString("x-refresh-token", rToken);
        editor.apply();
    }

    public void setUserFCMToken(String fcmToken) {
        if (deviceSharedPref != null) {
            deviceSharedPref.edit().putString(KEY_FCM_ID, fcmToken).apply();
        }
    }

    public String getUserFCMToken() {
        if (deviceSharedPref != null) {
            return deviceSharedPref.getString(KEY_FCM_ID, "");
        }
        return null;
    }

    public BusinessRestaurant getCurrentRestaurantDetail() {
        if (dineinRestuarntPrefrence != null) {
            String savedValue = dineinRestuarntPrefrence.getString(KEY_RESTAURANT, null);
            if (Validators.isNullOrEmpty(savedValue)) return null;
            return new Gson().fromJson(savedValue, BusinessRestaurant.class);
        } else return null;
    }

    public void saveOrderRestaurantDetail(BusinessRestaurant businessRestaurant) {
        if (cartPreferences != null) {
            cartPreferences.edit().putString(PREFERENCE_RESTAURANT_RELATED, new Gson().toJson(businessRestaurant)).apply();
        }
    }

    public BusinessRestaurant getOrderRestaurantDetail() {
        if (cartPreferences != null) {
            String savedValue = cartPreferences.getString(PREFERENCE_RESTAURANT_RELATED, null);
            if (Validators.isNullOrEmpty(savedValue)) return null;
            return new Gson().fromJson(savedValue, BusinessRestaurant.class);
        } else return null;
    }

    public void saveCurrentRestaurant(BusinessRestaurant restaurant) {
        if (cartPreferences != null) {
            cartPreferences.edit().putString(KEY_RESTAURANT, new Gson().toJson(restaurant)).apply();
            cartPreferences.edit().putString(KEY_RESTAURANT_ID, String.valueOf(restaurant.id)).apply();
            cartPreferences.edit().putString(KEY_RESTAURANT_NAME, restaurant.restaurant_name).apply();
        }
    }

    public void saveCurrentDineinRestaurant(BusinessRestaurant restaurant) {
        if (dineinRestuarntPrefrence != null) {
            dineinRestuarntPrefrence.edit().putString(KEY_RESTAURANT, new Gson().toJson(restaurant)).apply();
            dineinRestuarntPrefrence.edit().putString(KEY_RESTAURANT_ID, String.valueOf(restaurant.id)).apply();
            dineinRestuarntPrefrence.edit().putString(KEY_RESTAURANT_NAME, restaurant.restaurant_name).apply();
        }
    }

    public Restaurant getCurrentRestaurant() {
        if (cartPreferences != null) {
            String savedValue = cartPreferences.getString(KEY_RESTAURANT, null);
            if (Validators.isNullOrEmpty(savedValue)) return null;
            return new Gson().fromJson(savedValue, Restaurant.class);
        } else return null;
    }

    public BusinessRestaurant getCurrentDineInRestaurantDetail() {
        if (dineinRestuarntPrefrence != null) {
            String savedValue = dineinRestuarntPrefrence.getString(KEY_RESTAURANT, null);
            if (Validators.isNullOrEmpty(savedValue)) return null;
            return new Gson().fromJson(savedValue, BusinessRestaurant.class);
        } else return null;
    }

    public void saveButtonStates(ButtonStates buttonStates) {
        if (cartPreferences != null) {
            cartPreferences.edit().putString(KEY_RESTAURANT_BUTTON_STATES, new Gson().toJson(buttonStates)).apply();
        }
    }

    public ButtonStates getButtonStates() {
        if (cartPreferences != null) {
            String savedValue = cartPreferences.getString(KEY_RESTAURANT_BUTTON_STATES, null);
            if (Validators.isNullOrEmpty(savedValue)) return null;
            return new Gson().fromJson(savedValue, ButtonStates.class);
        } else return null;
    }


    public void saveDineinTablesStatus(ArrayList<TablesStatuses> settings) {
        if (dineinRestuarntPrefrence != null) {
            dineinRestuarntPrefrence.edit().putString(KEY_DINEIN_TABLES, new Gson().toJson(settings)).apply();
        }
    }

    public ArrayList<TablesStatuses> getDineinTablesStatus() {
        if (dineinRestuarntPrefrence != null) {
            String savedValue = dineinRestuarntPrefrence.getString(KEY_DINEIN_TABLES, null);
            if (Validators.isNullOrEmpty(savedValue)) return null;
            else {
                Type token = new TypeToken<List<TablesStatuses>>() {
                }.getType();
                return new Gson().fromJson(savedValue, token);
            }
        }
        return null;
    }

    public void saveDineinUser(UsersList settings) {
        if (dineinRestuarntPrefrence != null) {
            dineinRestuarntPrefrence.edit().putString(KEY_DINEIN_USER_MINI, new Gson().toJson(settings)).apply();
        }
    }

    public UsersList getDineinUser() {
        if (dineinRestuarntPrefrence != null) {
            String savedValue = dineinRestuarntPrefrence.getString(KEY_DINEIN_USER_MINI, null);
            if (Validators.isNullOrEmpty(savedValue)) return null;
            return new Gson().fromJson(savedValue, UsersList.class);

        }
        return null;
    }


    public void saveDineinSiteSettings(ArrayList<DineinSiteSettings> settings) {
        if (dineInSiteSharedPreferences != null) {
            dineInSiteSharedPreferences.edit().putString(KEY_DINEIN_SITE_SETTINGS, new Gson().toJson(settings)).apply();
        }
    }

    public void saveDineinPaymentMethods(ArrayList<DineinPaymentMethods> settings) {
        if (userPreferences != null) {
            userPreferences.edit().putString(KEY_DINEIN_PAYMENT_METHOD, new Gson().toJson(settings)).apply();
        }
    }

    public ArrayList<DineinPaymentMethods> getDineinPaymentMethods() {
        if (userPreferences != null) {
            String savedValue = userPreferences.getString(KEY_DINEIN_PAYMENT_METHOD, null);
            if (Validators.isNullOrEmpty(savedValue)) return null;
            else {
                Type token = new TypeToken<List<DineinPaymentMethods>>() {
                }.getType();
                return new Gson().fromJson(savedValue, token);
            }
        }
        return null;
    }

    public void saveQrCode(String qrcode) {
        if (qrPrefrence != null) {
            qrPrefrence.edit().putString(KEY_RES_QRCODE, qrcode).apply();
        }
    }

    public String getQrCode() {
        if (qrPrefrence != null) {
            return qrPrefrence.getString(KEY_RES_QRCODE, null);
        }
        return null;
    }

}
